/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_av.h
 * @ingroup core
 * The CNCAV library is a low level library for controlling the audio and video decoders on the device
 *
 *
The  CNCAV library is used to control audio and video playback on the chip.&nbsp; It is a lower level API that
manages an  AV device which is a combination of an audio and video handle and, optionally,  a hardware
demux handle.&nbsp; This  library should only be used to create a custom audio/video player.&nbsp; This document
gives a high level  overview of the most commonly used functions in the CNCAV library; for other  functions and
details on the API, please refer to the API documentation.</p>

@section cncav_fsm General  State Machine
The  overall state machine for the cncav module is shown in the diagram below.&nbsp; As with most of the CRISP
modules, the  general sequence for the state machine is init -&gt; open -&gt; start -&gt;  stop -&gt;
close.&nbsp; <strong>CNC_AV_Init()</strong> is typically done during the system startup  phase,
<strong>CNC_AV_Open()</strong>and <strong>CNC_AV_Close()</strong> are use when a media playback application
is active and <strong>CNC_AV_Start()</strong> and <strong>CNC_AV_Stop()</strong> are used for playback of
specific streams.</p>

@image html av_statemachine.jpg
@image rtf av_statemachine.jpg

The  AV module operates on a specified AV device, either <strong>CNC_AV_DEV0</strong> (for the primary
video layer) or <strong>CNC_AV_DEV1 </strong>for the secondary video layer; most  applications will use
<strong>CNC_AV_DEV0</strong> which is the standard audio/video device.&nbsp; The second video layer is
typically used  for analog TV or video conferencing applications.

@section cncav_setup Playback Setup
After  opening an a/v device and before starting playback, the device can be  configured based on the media
    being played.
  <strong>Stream Setup</strong><br
  The  primary function for a/v setup is <strong>CNC_AV_SetStreamType()</strong> which configures the audio
  and video format  for the elementary streams.&nbsp; The  function <strong>CNC_AV_SetInputMode()</strong> is
  used to identify whether the input is  from the memory or tuner (ts) interface.<br />
  Audio  / video synchronization is enabled and controlled using the function <strong>CNC_AV_AVSyncEnable</strong>
  that also allows a callback to be  registered when synchronization is achieved; the callback function can be
  disabled using <strong>CNC_AV_AVSyncDisableCallback()</strong>.&nbsp;  This can be used, for example, at the
  application layer to blank the  screen (and mute audio) until synchronization occurs.<br />
  Different  audio parameters can be adjusted using <strong>CNC_AV_SetAudioParams()</strong> to control mixing,
  equalization, surround  sound and rate conversion.</p>

<h3>Display Control </h3>
<p>The  location of the video on the screen can be controlled with the functions <strong>CNC_AV_SetOuputPosition()</strong>,
     <strong>CNC_AV_SetVideoWindow()</strong> and <strong>CNC_AV_SetVideoFullWindow()</strong>.&nbsp;  Aspect ratio
     can be adjusted using CNC_AV_SetAspectModeRatio().&nbsp; The alpha transparency of the video can  be
     controlled using the calls <strong>CNC_AV_SetVideoAlpha</strong> (specific value for alpha transparency) or
     <strong>CNC_AV_SetVideoFullAlpha()</strong> (for no transparency).&nbsp; The video output can be turned on and
     off using <strong>CNC_AV_DisableVideoOuput()</strong> and <strong>CNC_AV_EnableVideoOutput()</strong>.<br />
  Audio  output can be muted using <strong>CNC_AV_SetMute()</strong> and unmated using <strong>CNC_AV_ClearMute()</strong>.
  &nbsp;  Audio volume is controlled using the functions <strong>CNC_AV_SetVolume()</strong> (which takes an absolute
  volume level from  0 to 70) or <strong>CNC_AV_SetVolumeLevel()</strong> (which uses a non-linear scale from 0-30).&nbsp;
  The functions <strong>CNC_AV_SetVolumeUp()</strong> and <strong>CNC_AV_SetVolumeDown()</strong> can be used to step
  through the volume  levels.</p>

<h3>Writing  Elementary Stream Data</h3>
<p>Once  the AV device has been started, the application writes audio and video  elementary stream data into the
    input buffers of the device.&nbsp; Video data is written using <strong>CNC_AV_WriteVideoDataWithPTS()</strong>
    (for video data that contains timestamps)  and <strong>CNC_AV_WriteVideoData()</strong> (for video data without
    timestamps).&nbsp; Similarly, audio data is written using <strong>CNC_AV_WriteAudioDataWithPTS()</strong> and
    <strong>CNC_AV_WriteAudioData()</strong>.</p>

<h3>Trick  Mode</h3>
<p>During  playback, trick modes can be used to control the playback.&nbsp; <strong>CNC_AV_Pause()</strong> and
    <strong>CNC_AV_Resume()</strong>&nbsp;  are used to implement pause functionality in the application.&nbsp;
    For playback speed control, the AV  device should be stopped, and <strong>CNC_AV_SetTrickMode()</strong> should
    be called to enable trick mode  functions.&nbsp; <strong>CNC_AV_SetPlaySpeed()</strong> can be used to set the
    playback speed of the  media and <strong>CNC_AV_PlayOneFrame()</strong> can be used to play video frame by
    frame.&nbsp; More sophisticated trick  mode support is implemented in the higher level media processing libraries.</p>
 *
 */

#ifndef __CNC_AV_H__
#define __CNC_AV_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"
#include "cnc_demux.h"
//#include "cnc_xml.h"

#define SWITCH_CHANNEL_WITH_BLACK_SCREEN    TRUE
#define CHECK_VALID_AV_DEV(dev)  \
                do { \
                    if (dev >= CNC_AV_DEVNUM) \
                    { \
                        return RESULT_BAD_PARAMETER; \
                    } \
                } while (0)

/// Maximum Audio Volume Level Index
#define CNC_AV_MAX_VOLUME              255
/// Maximum Audio Volume Levels
#define CNC_AV_VOLUME_LEVEL_NUM        256

/// Unknownd Descriptor
#define CNC_AV_UNKNOWN_DESCRIPTOR                  0xFFFFFFFF

/// Value for audio stream type if no audio is in the stream
#define CNC_AV_STREAM_TYPE_NOAUDIO        -1
/// Value for video stream type if no video is in the stream
#define CNC_AV_STREAM_TYPE_NOVIDEO       -1

#ifdef ARCH_CSM1800H
#define CNC_AV_VIDEO_SURFACE_OPAQUE    0xFF
#else
#define CNC_AV_VIDEO_SURFACE_OPAQUE    0x3FF
#endif

#define CNC_AV_VIDEO_SURFACE_TRANSPARENT     0x00

/// Video Handle
typedef CNC_HANDLE HVIDEO;

typedef enum
{
      CNC_AV_AUD = 0,    ///< Audio Handle Selector
      CNC_AV_VID,        ///< Video Handle Selector
      CNC_AV_VIDCHANNEL, ///< Hardware Demux Video Channel Selector
      CNC_AV_AUDCHANNEL, ///< Hardware Demux Audio Channel Selector
      CNC_AV_NUM         ///< Number of Selectors
} cncAVComponent_t;

typedef enum
{
      CNC_AV_DEV0 = 0,  ///< Default AV device to use (audio 0, video 0)
      CNC_AV_DEV1,      ///< Usually used for analog video/audio input (audio 0, video 1)
      CNC_AV_DEV2,      ///< Special device for encoder - not valid for 1800L
      CNC_AV_DEVNUM     ///< Number of AV devices
} cncAVDevice_t;

typedef enum {
	CNC_AV_DEV_FULL,        ///< AV Device uses audio and video
	CNC_AV_DEV_AUDIO_ONLY,  ///< AV Device uses audio only
	CNC_AV_DEV_VIDEO_ONLY   ///< AV Device uses video only
} cncAVDeviceType_t;

typedef enum
{
      CNC_VIDEO_MODE_PANSCAN = 0, ///< Pan and Scan Video Mode
      CNC_VIDEO_MODE_LETTERBOX    ///< Letterbox Video Mode
} cncAVVideoMode_t;

typedef enum
{
      CNC_VIDEO_RATIO_4TO3 = 0,   ///< 4:3 Aspect Ratio
      CNC_VIDEO_RATIO_16TO9,      ///< 16:9 Aspect Ratio
      CNC_VIDEO_RATIO_AUTO        ///< Automatic Aspect Ratio
} cncAVVideoRatio_t;

typedef enum
{
      CNC_AUDIO_STEREO = 0,          ///< Audio Stereo PCM Output
      CNC_AUDIO_LEFT_MONO,   ///< Audio Left Mono PCM Ouptut
      CNC_AUDIO_RIGHT_MONO,      ///< Audio Right Mono PCM Output
      CNC_AUDIO_CHANNEL51,       ///< Audio 5.1 PCM Output
      CNC_AUDIO_INVALID          ///< Invalid Audio Output
} cncAVAudioPCMChannel_t;

typedef struct _cncAVVideoWindow_t
{
      CNC_SINT32 left;           ///< Left coordinate for video window
      CNC_SINT32 top;            ///< Top coordinate for video window (Y Offset)
      CNC_SINT32 width;          ///< Width coordinate of video window (right = left + width)
      CNC_SINT32 height;         ///< Height coordinate of video window (bottom = top + height)
} cncAVVideoWindow_t;

typedef struct _cncAVAudioOutputParam_t
{
      CNC_BOOL fdrc;             ///< DRC enabled - only for AC3
      CNC_BOOL fsurround;        ///< Surround Sound Enabled only for AC3
      CNC_BOOL fequalize;        ///< Equalization enabled
      CNC_BOOL fmixer;           ///< Mixer enabled
      cncAVAudioPCMChannel_t chmode; ///< Current PCM Output Mode
} cncAVAudioOutputParam_t;

typedef enum _cncAVAudioDecoderBufferStatus_t
{
    CNC_AV_AUDIO_DECODER_BUFFER_INVALID = -1,  ///<  Audio Decoder Buffer Status - Invalid
    CNC_AV_AUDIO_DECODER_BUFFER_EMPTY,         ///<  Audio Decoder Buffer Status - Empty
    CNC_AV_AUDIO_DECODER_BUFFER_NORMAL,        ///<  Audio Decoder Buffer Status - Normal
    CNC_AV_AUDIO_DECODER_BUFFER_FULL           ///<  Audio Decoder Buffer Status - Full
} cncAVAudioDecoderBufferStatus_t;

typedef enum _cncAVVideoDecoderBufferStatus_t
{
    CNC_AV_VIDEO_DECODER_BUFFER_INVALID = -1,  ///<  Video Decoder Buffer Status - Invalid
    CNC_AV_VIDEO_DECODER_BUFFER_EMPTY,         ///<  Video Decoder Buffer Status - Empty
    CNC_AV_VIDEO_DECODER_BUFFER_NORMAL,        ///<  Video Decoder Buffer Status - Normal
    CNC_AV_VIDEO_DECODER_BUFFER_FULL           ///<  Video Decoder Buffer Status - Full
} cncAVVideoDecoderBufferStatus_t;

typedef struct _cncAVDecoderBufferStatus_t
{
    cncAVAudioDecoderBufferStatus_t e_audBufStatus; ///< Audio Decoder Buffer Status
    cncAVVideoDecoderBufferStatus_t e_vidBufStatus; ///< Video Decoder Buffer Status
} cncAVDecoderBufferStatus_t;

typedef enum
{
      CNC_BASE_INPUT_TUNER,     ///< Tuner Input Mode
      CNC_BASE_INPUT_DMA,       ///< DMA (File, Memory) Input Mode
      CNC_BASE_INPUT_DIRECT     ///< Direct (VIB) Input Mode
} CNC_BASE_INPUT_MODE_e;

typedef struct _CNC_BASE_AV_Params_t
{
      cncAVDevice_t    Dev;           ///< AV Device Selection
      CSDEMUX_INPUT_MOD InputMode;   ///< 0--Tuner  ; 1-- DMA ;  2-- PFM
      CSVID_STREAM_TYPE	vtype;       ///< Video Stream Type
      CSAUD_STREAM_TYPE	atype;       ///< Audio Stream Type
      CSVID_DECODING_MOD vmode;      ///< Video Decoding Mode - Typically ANY
      CNC_UINT16 vpid;               ///< Video PID
      CNC_UINT16 apid;               ///< Audio PID
      CNC_UINT8   chid;              ///< channel ID, for multi-channel
} CNC_BASE_AV_Params_t;


typedef enum _cncAVVideoWindowFitType_t
{
    CNC_AV_WINDOW_FIT_TYPE_BEST_FIT,
    CNC_AV_WINDOW_FIT_TYPE_FULL_WINDOW,
    CNC_AV_WINDOW_FIT_TYPE_PILLAR_BOX,
    CNC_AV_WINDOW_FIT_TYPE_LETTER_BOX,
    CNC_AV_WINDOW_FIT_TYPE_FULL_WIDTH,
    CNC_AV_WINDOW_FIT_TYPE_FULL_HEIGHT,
    CNC_AV_WINDOW_FIT_TYPE_SPECICAL_MODE_0_FOR_AFD1100,
    CNC_AV_WINDOW_FIT_TYPE_NO_SCALE,                      ///< Display Video as it is without scaling
    CNC_AV_WINDOW_FIT_TYPE_MAX_NUM,
    CNC_AV_WINDOW_FIT_TYPE_DEFAULT = CNC_AV_WINDOW_FIT_TYPE_BEST_FIT
} cncAVVideoWindowFitType_t;


typedef struct _cncAVVideoWindowPARScale_t
{
    cncAVDevice_t deviceId;

    CNC_BOOL    videoWindowEnableWindowAreaHandle;
    CNC_SINT32  videoWindowPositionPosX;
    CNC_SINT32  videoWindowPositionPosY;
    CNC_SINT32  videoWindowPositionWidth;
    CNC_SINT32  videoWindowPositionHeight;

    CNC_BOOL    videoWindowEnableCutPositionHandle;
    CNC_SINT32  videoPictureSizeWidth;
    CNC_SINT32  videoPictureSizeHeight;
    CNC_SINT32  videoWindowCutPositionPosX;
    CNC_SINT32  videoWindowCutPositionPosY;
    CNC_SINT32  videoWindowCutPositionWidth;
    CNC_SINT32  videoWindowCutPositionHeight;

    CNC_BOOL    videoWindowEnablePARHandle;
    cncAVVideoWindowFitType_t videoWindowFitType;

} cncAVVideoWindowPARScaleObj_t;

typedef CNC_VOID (*cncAVDataEmptyCallback_t)(CNC_SINT32 type);

/**
 * Initialize the CNCAV Module
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Init(void);

/**
 * Initialize the hardware demux channels to use with the AV device
 * @param dualTuner If two tuners are used in the system, the hardware demux uses different channels.
 * This should be FALSE if there is only one tuner is in system.
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_InitDemuxChannels(CNC_BOOL dualTuner);

/**
 * Open the input audio/video device.
 * @param dev AV device
 * @param format
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Open(cncAVDevice_t dev, int format);

/**
 * Start the input audio/video device.
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Start(cncAVDevice_t dev);

/**
 * Stop the input audio/video device.
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Stop(cncAVDevice_t dev);

/**
 * INTERNAL - DO NOT USE - Stop the input audio/video device.
 * @param dev AV device
 * @param retainLastFrame Retain displaying the last frame
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_StopExt(cncAVDevice_t dev, CNC_SINT32  retainLastFrame);

/**
 * Close the input audio/video device.
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Close(cncAVDevice_t dev);

/**
 * Reinitialize the AV Module.  This is needed if another application has launched for AV operations.
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_ReInit(void);

/**
 * Default AV Sync handler.  Will enable video and audio when sync occurs
 * @param hdl CSVID handle for the video
 * @param sync 1 - sync has been achieved, 2 - display image is ready, -1 no AV sync achieved
 */
void CNC_AV_AVSyncDefaultHandler(CSVID_HANDLE hdl, CSVID_SYNC *sync);

/**
 * Clear AV Memory for further playback
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_ClearAVMemory(cncAVDevice_t dev);

/**
 * Restart video decoder for new stream
 * @param dev AV device
 * @param stream_type codec type for new video stream
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_RestartVideoDecoder(cncAVDevice_t dev,CSVID_STREAM_TYPE stream_type);

/**
 * Restart audio decoder for new stream
 * @param dev AV device
 * @param stream_type codec type for new audio stream
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_RestartAudioDecoder(cncAVDevice_t dev,CSAUD_STREAM_TYPE stream_type);

/**
 * Get audio decoder stc value
 * @param dev AV device
 * @param stc point of the variable that would be set as stc
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Get_AudioStc(cncAVDevice_t dev,CNC_UINT64 *stc);

/**
 * Write data to the audio buffer (if there is no audio PTS)
 * @param dev AV device
 * @param src pointer to the input audio data
 * @param size size of the input audio data
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_WriteAudioData(cncAVDevice_t dev, CNC_UINT8 *src, CNC_SINT32 size);

/**
 * Write data to the audio buffer with an audio PTS value
 * @param dev AV device
 * @param src pointer to the input audio data
 * @param size size of the input audio data
 * @param pts timestamp of the audio
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_WriteAudioDataWithPTS(cncAVDevice_t dev, CNC_UINT8 *src, CNC_SINT32 size, CNC_UINT64 pts);

/**
 * Write data to the video buffer (if there is no video PTS)
 * @param dev AV device
 * @param src pointer to the input video data
 * @param size size of the input video data
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_WriteVideoData(cncAVDevice_t dev, CNC_UINT8 *src, CNC_SINT32 size);

/**
 * Write data to the video buffer with an video PTS value
 * @param dev AV device
 * @param src pointer to the input video data
 * @param size size of the input video data
 * @param pts timestamp of the video
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_WriteVideoDataWithPTS(cncAVDevice_t dev, CNC_UINT8 *src, CNC_SINT32 size, CNC_UINT64 pts);

/**
 * Pause the AV device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Pause(cncAVDevice_t dev);

/**
 * Resume the AV device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Resume(cncAVDevice_t dev);

/**
 * Freeze the AV device (leaving last frame image)
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Freeze(cncAVDevice_t dev);

/**
 * Set the playspeed of the AV device
 * @param dev AV device
 * @param speed set the play speed to 1X, 2X, 4X, 8X, 16X, 32X
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetPlaySpeed(cncAVDevice_t dev, CSVID_PLAYSPEED speed);

/**
 * Identify if the device is playing from the tuner or the memory
 * @param dev AV device
 * @param flag CNC_BOOL_TRUE if from memory (files, streams), CNC_BOOL_FALSE if from tuner
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_PlayFromMemory(cncAVDevice_t dev, CNC_BOOL flag);
/**
 * Enable or disable trick mode support for the AV device
 * @param dev AV device
 * @param trick CNC_BOOL_FALSE to disable, CNC_BOOL_TRUE to enable
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetTrickMode(cncAVDevice_t dev, CNC_BOOL trick);

/**
 * Disable the video output; no video will display for this device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_DisableVideoOutput(cncAVDevice_t dev);

/**
 * Enable the video output; video layer will start displaying for this device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_EnableVideoOutput(cncAVDevice_t dev);

/**
 * Enable analog audio output for the device (for instance for RCA analog ouput or 3.5mm output jack
 * @param dev AV device
 * @param spdifType either PCM or compressed
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_EnableAnalogAudio(cncAVDevice_t dev, CSAUD_SPDIF_TYPE spdifType);

/**
 * Disable analog audio output for the device (for instance for RCA analog ouput or 3.5mm output jack
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_DisableAnalogAudio(cncAVDevice_t dev);

/**
 * Set the stream type for the audio and video for the device
 * @param dev AV device
 * @param audio Audio stream type
 * @param video Video stream type
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetStreamType(cncAVDevice_t dev, CSAUD_STREAM_TYPE audio, CSVID_STREAM_TYPE video);

/**
 * Set the input mode (blocking or non-blocking) for the audio and video in the device
 * @param dev AV device
 * @param aMode Audio blocking or non-blocking
 * @param vMode Video blocking or non-blocking
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetInputMode(cncAVDevice_t dev, CSAUD_INPUT_MODE aMode, CSVID_INPUT vMode);

/**
 * Get the video decoding mode for the device; All, I Frame, IP Frame, SP Frame
 * @param dev AV device
 * @return video decoding mode of the device
 */
CSVID_DECODING_MOD CNC_AV_GetDecodeMode(HVIDEO dev);

/**
 * Set the video decoding mode for the device; All, I Frame, IP Frame, SP Frame
 * @param dev AV device
 * @param mode Video decoding mode
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetDecodeMode(cncAVDevice_t dev, CSVID_DECODING_MOD mode);

/**
 * Set the sequence header (width, height, frame rate) for the AV device
 * @param dev AV device
 * @param hdr video sequence header structure
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetSequenceHeader(cncAVDevice_t dev, CSVID_SequenceHeader hdr);

/**
 * Get the video sequence header (width, height frame rate) for the AV device
 * @param dev AV device
 * @param hdr Video Sequence Header
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetSequenceHeader(cncAVDevice_t dev, CSVID_SequenceHeader *hdr);

/**
 * Stop the hardware demuxing channels for the AV device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_StopDemuxChannels(cncAVDevice_t dev);

/**
 * Set the channel source for the hardware demuxer of the device: tuner, DMA, direct
 * @param dev AV device
 * @param inputmode whether the input source is tuner, DMA or direct
 * @param Chl_ID id for the hardware demuxer channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetChannelSource(cncAVDevice_t dev, cncDemuxInputMode_t inputmode, CNC_UINT8 Chl_ID);

/**
 * Recreate the hardware demuxer channels of the device; should be called before restarting AV devices using the hardware demuxer
 * @param dev AV device
 * @param SwitchMode whether the input source is tuner, DMA or direct
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_RecreateDemuxChannels(cncAVDevice_t dev, CNC_UINT8 SwitchMode);

/**
 * Set the video alpha transparency
 * @param dev AV device
 * @param alpha transparency value between 0 and 0xFF
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetVideoAlpha(cncAVDevice_t dev, CNC_UINT32 alpha);

/**
 * Set the video alpha transparency to no transparency (full display)
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetVideoFullAlpha(cncAVDevice_t dev);

/**
 * Mute the audio on the AV device; save the mute state
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetMute(cncAVDevice_t dev);

/**
 * Unmute the audio on the AV device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_ClearMute(cncAVDevice_t dev);

/**
 * Get the mute status of the AV device
 * @param dev AV device
 * @return audio PTS value
 */
CNC_BOOL CNC_AV_GetMuteStatus(cncAVDevice_t dev);

/** 
 * Set Volume without save
 * @param devIndex 
 * @param u8_volumeValue 
 * @return 
 */
Result_Type_e CNC_AV_SetVolume(cncAVDevice_t devIndex,CNC_UINT8 u8_volumeValue);

/**
 * Setup the system volume
 * @param devIndex AV device
 * @param u8_volumeValue volume value
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetGlobalVolume(cncAVDevice_t devIndex,CNC_UINT8 u8_volumeValue);

/**
 * Get the system volume
 * @param devIndex AV device
 * @return system volume value
 */
CNC_UINT32 CNC_AV_GetGlobalVolume(cncAVDevice_t devIndex);

/**
 * Reset the system volume
 * @param devIndex AV device
 * @return NULL
 */
void CNC_AV_ResetGlobalVolume(cncAVDevice_t devIndex);


/**
 * Set the output audio parameters of the device
 * @param dev AV device
 * @param AudioParam structure defining audio output values
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetAudioParams(cncAVDevice_t dev, cncAVAudioOutputParam_t *AudioParam);

/**
 * Set the sample rate for HDMI output
 * @param dev AV device
 * @param CSAUD_SAMPLE_RATE target sample rate
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetSampleRate(cncAVDevice_t dev, CSAUD_SAMPLE_RATE rate);

/**
 * Get the output audio parameters of the device
 * @param dev AV device
 * @param AudioParam structure in which to return the audio output values
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetAudioParams(cncAVDevice_t dev, cncAVAudioOutputParam_t *AudioParam);

/**
 * Utility function for playing a movie with a single I-Frame (for video layer background pictures)
 * @param dev AV device
 * @param File File name for the IFrame movie
 * @param sType format of the IFrame file
 * @param XPos x position on the display
 * @param YPos y position on the display
 * @param Width width to display
 * @param Height height to display
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_IFramePlay(cncAVDevice_t dev, const char *File, CSVID_STREAM_TYPE sType, CNC_UINT16 XPos, CNC_UINT16 YPos, CNC_UINT16 Width, CNC_UINT16 Height);

/**
 * Stop the I-Frame playback for the device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_IFrameStop(cncAVDevice_t dev);

/**
 * Set the video playback location for the video for the device
 * @param dev AV device
 * @param vidwindow rectangle on the display in which to display
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetVideoWindow(cncAVDevice_t dev, const cncAVVideoWindow_t *vidwindow);


/**
 * Get the current video playback location for the video for the device
 * @param dev AV device
 * @param vidwindow rectangle on the display where video is displayed
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetVideoWindow(cncAVDevice_t dev, cncAVVideoWindow_t *vidwindow);

/**
 * Set the video playback location to full screen (based on output display settings)
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetVideoFullWindow(cncAVDevice_t dev);

/**
 * Set the video output mode based on the video stream size
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
void CNC_AV_SetVideoWindowMode(cncAVDevice_t dev);

/**
 * Change the audio pid for AV device using hardware demux channels
 * @param AudioPID New PID to use for the audio
 * @param AudioType Type of audio contained in the new PID
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_ChangeAudioPid( CNC_UINT16 AudioPID, CSAUD_STREAM_TYPE AudioType );

/**
 * Get the current PTS value; if has video, return video PTS; if audio only, return audio PTS
 * @param dev AV device
 * @return current PTS value
 */
unsigned int CNC_AV_GetPTS(cncAVDevice_t dev);

/**
 * Enable AV Synchronization for the device and provide a callback for AV sync status events
 * @param dev AV device
 * @param call_back_function callback function for AV synchronization events
 * @param timeout timeout for callback if no AV sync event occurs
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_AVSyncEnable(cncAVDevice_t dev, void(* call_back_function)(CSVID_HANDLE, CSVID_SYNC *), CNC_UINT32 timeout);

/**
 * Disable the AV sync callback function; should be called after synchronization is achieved
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_AVSyncDisableCallback(cncAVDevice_t dev);

/**
 * Get the audio PTS value
 * @param dev AV device
 * @param pts audio PTS value (45 Khz Counter)
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetAudioPTS(cncAVDevice_t dev, CNC_UINT32 *pts);

/**
 * Get the video PTS value
 * @param dev AV device
 * @param pts video PTS value (45 Khz Counter)
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetVideoPTS(cncAVDevice_t dev, CNC_UINT32 *pts);

/**
 * Set the audio mode for the device (Stereo, Left, Right, Mono)
 * @param dev AV device
 * @param mode Mode for the audio PCM Channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetAudioMode(cncAVDevice_t dev, CSAUD_PCM_CHANNEL mode);

/**
 * Set the video PID for the hardware demuxer
 * @param dev AV device
 * @param pid PID for the video
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetChannelVideoPID(cncAVDevice_t dev, CNC_UINT16 pid);

/**
 * Set the audio PID for the hardware demuxer
 * @param dev AV device
 * @param pid PID for the audio
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetChannelAudioPID(cncAVDevice_t dev, CNC_UINT16 pid);

/**
 * Enable or disable the device for using the hardware demuxer
 * @param dev AV device
 * @param flag CNC_BOOL_TRUE to use the hardware demuxer, CNC_BOOL_FALSE to bypass the hardware demuxer
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_UseDemuxChannels(cncAVDevice_t dev, CNC_BOOL flag);

/**
 * Set the transport format for the hardware demuxer
 * @param handle Hardware demux handle
 * @param type Select from MPEG2TS, TS only, ...
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetDemuxSourceType(int handle, CSDEMUX_STREAM_TYPE type);

/**
 * Get the demux handle for the device
 * @param handle selection of the demux channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
CNC_DemuxSourceHandle CNC_AV_GetDemuxSource(int handle);

/**
 * check whether the current device buffer is emptied
 * @param dev AV device
 * @param type 0 for audio, 1 for video
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
CNC_BOOL CNC_AV_BufferIsFinished(cncAVDevice_t dev, int type);

/**
 * Stop the audio channel in the hardware demuxer
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_StopAudioChannel(cncAVDevice_t dev);

/**
 * Start the audio channel in the hardware demuxer
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_StartAudioChannel(cncAVDevice_t dev);

/**
 * Disable AV synchronization for the device
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_AVSyncDisable(cncAVDevice_t dev);

/**
 * Get the current audio mode (stereo, left, others)
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
CSAUD_PCM_CHANNEL CNC_AV_GetAudioMode(cncAVDevice_t dev);

/**
 * Set the PCM rate for audio in the device
 * @param dev AV device
 * @param rate the rate to set for the device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetPCMRate(cncAVDevice_t dev, CSAUD_SAMPLE_RATE rate);

/**
 * Set the PCM capability for the device
 * @param dev AV device
 * @param pcm_capability structure defining the PCM capabilities
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetPCMCapability(cncAVDevice_t dev, CSAUD_PCM_Capability_s pcm_capability);

/**
 * Get the status of the audio and video buffers for the device
 * @param dev AV device
 * @param decBufStatus structure to hold the current audio and video bufferStatus
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetBufferStatus(cncAVDevice_t dev, cncAVDecoderBufferStatus_t *decBufStatus);

/**
 * Get the size of the audio and video buffers for the device
 * @param dev AV device
 * @param audioBufSize size of the audio buffer
 * @param videoBufSize size of the video buffer
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_GetBufferSize(cncAVDevice_t dev, CNC_UINT32 *audioBufSize, CNC_UINT32 *videoBufSize);

/**
 * Set the AV Sync version; newer version uses different AZ sync mechanism
 * @param dev AV device
 * @param version 0 for old version, 1 for new version
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetAVSyncVer(cncAVDevice_t dev,unsigned int version);

/**
 * Set a flag preparing the AV device for a change in the frame zie
 * @param dev AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetFrameSizeChange(cncAVDevice_t dev);

/**
 * Get the AV device handle from the video handle (used in AV sync callback functions)
 * @param hDev AV component handle
 * @return AV Device for the handle
 */
cncAVDevice_t CNC_AV_GetDevByHandle(CSVID_HANDLE hDev);

/**
 * Get the internal handle of the audio, video or demux channels for the AV device
 * @param dev AV device
 * @param Componet The component of the AV device
 * @return specified handle
 */
CNC_HANDLE CNC_AV_GetAVHandle(cncAVDevice_t dev, cncAVComponent_t Componet);

/**
 * Set the input source for the audio input - DDR or ADC - not for 6611L/1800L
 * @param dev AV device
 * @param source Audio Input Source
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetAudioInputSource(cncAVDevice_t dev, CSAUD_INPUT_SOURCE source);

/**
 * INTERNAL - DO NOT USE: Base AV Playback Start
 * @param AV_Params parameters for the AV device
 * @param SyncCallback function callback for AV Sync status
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_BASE_AVPlay(CNC_BASE_AV_Params_t *AV_Params , void(* SyncCallback)(CSVID_HANDLE, CSVID_SYNC *));

/**
 * INTERNAL - DO NOT USE: Base AV Playback Stop
 * @param AV_Params parameters for the AV device
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_BASE_AVStop(CNC_BASE_AV_Params_t *AV_Params);

/**
 * Change equalizer type
 * @param dev AV device
 * @param type Equalise Type
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetEQType(cncAVDevice_t dev, DRIVER_EQUALIZER_TYPE type);

/**
* Enable deinterlace
* @param dev AV device
* @return if successful returns RESULT_OK, error returns: RESULT_FAIL
*/
Result_Type_e CNC_AV_EnableDint(cncAVDevice_t dev);

/**
* Disable deinterlace
* @param dev AV device
* @return if successful returns RESULT_OK, error returns: RESULT_FAIL
*/
Result_Type_e CNC_AV_DisableDint(cncAVDevice_t dev);

/**
* Set the PAR Scaaling. This API can implement three functions.
* 1. Set the video window area size. Enable bool_videoWindowEnableWindowAreaHandle to open it.
* 2. Cut the video picturn. Enable bool_videoWindowEnablePicCutPositionHandle to open it.
* 3. Setup the video fit mode.Enable bool_videoWindowEnablePARHandle to open it.
* @param  videoPARObj a structure include some flag and parameters. The flag for enable or disable some functions.
* @return if successful returns 0, error returns: -1
*/

Result_Type_e CNC_AV_Set_VideoWindowAttribute(cncAVVideoWindowPARScaleObj_t *videoPARObj);

/**
* get the screen width, height and AspRatio depend on the current output mode.
* @param mode current output mode.
* @param[out] width display width
* @param[out] height display height
* @param[out] screenAspRatio display aspect ratio
* @return if successful returns RESULT_OK, error returns: RESULT_FAIL
*/
Result_Type_e CNC_AV_GetScreenSizeFromMode(CSTVOUT_MODE mode, CNC_SINT32 *width, CNC_SINT32 *height, CNC_FLOAT32 *screenAspRatio);

/**
 * Get stream attribute for the current AV device
 * @param dev AV device
 * @param width stream width
 * @param height stream height
 * @param frameRate stream frame rate
 */
void CNC_AV_GetStreamAttribute(cncAVDevice_t dev,CNC_SINT32 *width,CNC_SINT32 *height,CNC_SINT32 *frameRate);

/**
* Register AV PFM Buffer Empty Callback Functions.
* @param dev AV device
* @param vidEmptyThreshold - Video Buffer Empty Threshold
* @param audEmptyThreshold - Audio Buffer Empty Threshold
* @param avDataEmptyCbf - Call Back function to notify audio &/ Video data Empty
* @return if successful returns RESULT_OK, error returns: RESULT_FAIL
*/
Result_Type_e CNC_AV_RegisterBufEmptyCbf(cncAVDevice_t dev, CNC_UINT32 vidEmptyThreshold, CNC_UINT32 audEmptyThreshold, cncAVDataEmptyCallback_t avDataEmptyCbf);

/**
* UnRegister AV PFM Buffer Empty Callback Functions.
* @param dev AV device
*/
Result_Type_e CNC_AV_UnRegisterBufEmptyCbf(cncAVDevice_t dev);



/************************************************************************************************************************/
/*            Follow APIs will be deleted in the future.                                                                */
/************************************************************************************************************************/

/**
 * Notice: This API will be deleted in future. Pls use the new api CNC_AV_Set_VideoWindowAttribute() to implememnt the related functions.
 * Set the output position of video and possible cropped area
 * @param dev AV device
 * @param recsrc Source Window
 * @param recdst Destination Window
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetOutputPosition(cncAVDevice_t dev, const cncAVVideoWindow_t* recsrc, cncAVVideoWindow_t *recdst);

/**
 * Notice: This API will be deleted in future. Pls use the new api CNC_AV_Set_VideoWindowAttribute() to implememnt the related functions.
 * INTERNAL - DO NOT USE - Set the output position of video and possible cropped area
 * @param dev AV device
 * @param recsrc Source Window dimensions
 * @param recdst Destination Window dimensions
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetOutputPositionExt(cncAVDevice_t dev, const cncAVVideoWindow_t* recsrc, cncAVVideoWindow_t *recdst);


/**
 * Notice: This API will be deleted in future. Pls use the new api CNC_AV_Set_VideoWindowAttribute() to implememnt the related functions.
 * Set the aspect ratio of the video for the AV device
 * @param dev AV device
 * @param mode pan and scan or letterbox
 * @param ratio 4:3, 16:9 or auto detect
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetAspectModeRatio(cncAVDevice_t dev, cncAVVideoMode_t mode, cncAVVideoRatio_t ratio);

/**
 * Notice: This API will be deleted in future. Pls use api CNC_Setting_GetScreenVideoSize() to get the size.
 * Get width and height of the video window for the current AV device
 * @param width Video Window Width
 * @param height Video Window Height
 */
void CNC_AV_GetCurWindowSize(int* width, int* height);

/**
 * Notice: This API is used to set the endian flag for current stream
 * @param dev AV device
 * @param endian 1:big endian; 0:little endian
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetAUDEndian(cncAVDevice_t dev, CNC_UINT8 endian);

/**
 * Set the SPDIF Audio Format
 * @param format SPDIF Format
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetSPDIFAudioFormat(CSAUD_SPDIF_TYPE format);

/**
 * Get the SPDIF Audio Format
 * @return SPDIF Format
 */
CSAUD_SPDIF_TYPE CNC_AV_GetSPDIFAudioFormat(void) ;

/**
 * Set the HDMI Audio Format
 * @param format HDMI Format
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetHDMIAudioFormat(CSAUD_HDMI_MODE format);

/**
 * Get the HDMI Audio Format
 * @return HDMI Format
 */
CSAUD_HDMI_MODE CNC_AV_GetHDMIAudioFormat(void) ;

/**
 * Notice: This API is used to set NO audio flag for current stream
 * @param dev AV device
 * @param NOAudio 0: has audio, 1:no audio
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */

Result_Type_e CNC_AV_SetNoAudio(cncAVDevice_t dev, CNC_UINT8 NOAudio);


/**
 * Notice: the frame numbers include the video header numbers.
 * @param dev AV device
 * @return the left video frame numbers in the cpb buffer.
 */

CNC_UINT32 CNC_AV_GetVideoDirLeftNum(cncAVDevice_t dev);

/**
 * Notice: the frame numbers include the video header numbers.
 * @param dev AV device
 * @return the totle video frame numbers in the cpb buffer.
 */
CNC_UINT32 CNC_AV_GetVideoDirTotleNum(cncAVDevice_t dev);

/**
 * Notice: The error frames are skipped based on the tuner request.
 * @param dev AV device
 * @return void
 */
CNC_VOID CNC_AV_SetErrorSkipMode(cncAVDevice_t dev);

/**
 * Notice: This API is used to set the step size(no of Frames) of Video
 * @param dev AV device
 * @param noOfFrames 
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_Step(cncAVDevice_t dev, CNC_UINT16 noOfFrames);

/**
 * Notice: This API is used for the accurately seek.
 * @param dev AV device
 * @param pts 
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_AV_SetStartPts(cncAVDevice_t dev,CNC_SINT64 pts);

#ifdef __cplusplus
}
#endif

#endif
