/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_demux.h
 * @ingroup core
 * The CNCDEMUX library is a low level library to control the on-chip hardware demux engine
 * The  CNCDEMUX library is used to access the hardware demux resources on the  chip.&nbsp; For general media
 * playback,  software demuxing is used and, therefore, this library is not typically  used.  The library
 * is primarily  setup for supporting broadcast protocol stacks (DVB, ISDB, etc.) where  transport streams can
 * have more complex operations.
 */

#ifndef  __CNC_DEMUX_H__
#define  __CNC_DEMUX_H__

#ifdef __cplusplus
extern "C" {
#endif
#include "cnc_common_typedef.h"
#include "cnc_util.h"

#ifdef ARCH_CSM1800H
#define  CNC_DEMUX_MAX_NUMBER_FILTERS      (64)
#define  CNC_DEMUX_MAX_NUMBER_SLOTS      (64)
#else
#define  CNC_DEMUX_MAX_NUMBER_FILTERS      (32)
#define  CNC_DEMUX_MAX_NUMBER_SLOTS      (32)
#endif

#define  CNC_DEMUX_MAX_NUMBER_PES_FILTERS      (17)
#define  CNC_DEMUX_MAX_NUMBER_FILTER_DEPTH (8)
#define  DemuxChannelValidityCheck      0x44090
#define  MaxPesSlot 2

/**
 * Filter Mode
 */
typedef enum _cncDemuxFilterMode_t
{
	CNC_DEMUX_FILTER_MODE_DEFAULT = 0X0, //Repeat Version
	CNC_DEMUX_FILTER_MODE_ONE_SHOT = 0X01, //!< CNC_DEMUX_FILTER_MODE_ONE_SHOT
	CNC_DEMUX_FILTER_MODE_NO_VERSION = 0X02//!< CNC_DEMUX_FILTER_MODE_NO_VERSION
} cncDemuxFilterMode_t;

/**
 * Channel type in demux
 */
typedef enum _cncDemuxChannelType_t
{
    CNC_DEMUX_DATA_PSI_CHANNEL = 0,//!< CNC_DEMUX_DATA_PSI_CHANNEL
    CNC_DEMUX_DATA_PES_CHANNEL,    //!< CNC_DEMUX_DATA_PES_CHANNEL
    CNC_DEMUX_VIDEO_CHANNEL,       //!< CNC_DEMUX_VIDEO_CHANNEL
    CNC_DEMUX_AUDIO_CHANNEL,       //!< CNC_DEMUX_AUDIO_CHANNEL
    CNC_DEMUX_PCR_CHANNEL          //!< CNC_DEMUX_PCR_CHANNEL
} cncDemuxChannelType_t;

/**
 * Input mode; tuner, memory (DDR) or direct
 */
typedef enum _cncDemuxInputMode_t
{
	CNC_DEMUX_INPUT_MODE_TUNER = 0,//!< CNC_DEMUX_INPUT_MODE_TUNER
	CNC_DEMUX_INPUT_MODE_DMA,      //!< CNC_DEMUX_INPUT_MODE_DMA
	CNC_DEMUX_INPUT_MODE_DIRECT    //!< CNC_DEMUX_INPUT_MODE_DIRECT
} cncDemuxInputMode_t;

/**
 * Type of filter (PSI, PES, PCR)
 */
typedef enum _cncDemuxFilterType_t
{
    CNC_DEMUX_DATA_PSI_FILTER = 0,
    CNC_DEMUX_DATA_PES_FILTER,
    CNC_DEMUX_PCR_FILTER
} cncDemuxFilterType_t;

/**
 * status of demux state
 */
typedef enum _cncDemuxStates_t
{
	CNC_DEMUX_STATE_NORMAL = 0,  //!< CNC_DEMUX_STATE_NORMAL
	CNC_DEMUX_STATE_FAILURE = -1,//!< CNC_DEMUX_STATE_FAILURE
} cncDemuxStates_t;

/**
 * Type of the demux slot (video, audio, section, PES)
 */
typedef enum _cncDemuxSlotType_t
{
        CNC_DEMUX_SLOT_VIDEO  = 0,//!< CNC_DEMUX_SLOT_VIDEO
        CNC_DEMUX_SLOT_AUDIO,     //!< CNC_DEMUX_SLOT_AUDIO
        CNC_DEMUX_SLOT_SECTION,   //!< CNC_DEMUX_SLOT_SECTION
        CNC_DEMUX_SLOT_PES_DATA   //!< CNC_DEMUX_SLOT_PES_DATA
} cncDemuxSlotType_t;

/**
 * buffer mode (1K or 4k mode)
 */
typedef enum _cncDemuxBufferMode_t
{
	CNC_DEMUX_BUFFER_MODE_1K = 0,//!< CNC_DEMUX_BUFFER_MODE_1K
	CNC_DEMUX_BUFFER_MODE_4K     //!< CNC_DEMUX_BUFFER_MODE_4K
} cncDemuxBufferMode_t;

/**
 * Handle for demux
 */
typedef CSDEMUX_HANDLE     		CNC_DemuxSourceHandle;

/**
 * Callback structure for demux callbacks
 * @param Channel_Handle current channel handle
 * @param Filter_Handle current filter handle
 * @param userparam user-defined data
 * @param timedout whether from timeout
 * @param data data
 * @param size data size
 */
typedef void (*cncDemuxCallBack_t)(
                                                      CNC_UINT32		       Channel_Handle,
                                                      CNC_UINT32			Filter_Handle,
                                                      void* userparam,
                                                      CNC_BOOL timedout,
                                                      CNC_UINT8 const* data,
                                                      CNC_UINT32 size);

/**
 * Demux filter structure
 */
typedef struct _cncDemuxFilter_t
{
    CSDEMUX_HANDLE Slot_Handle;
    CNC_UINT32 Channel_Handle;
    CSDEMUX_FILTER_ID Filter_id;
    CSDEMUX_HANDLE Filter_Handle;
    cncDemuxFilterMode_t Mode;
    cncOSSemaphore_t* DmxSemHandle;
    cncDemuxCallBack_t callback;
    CNC_UINT8 Nomatch[12];
    CNC_UINT8 SWFilterMatch[12];
    CNC_UINT8 SWFilterMask[12];
    CNC_UINT32 DestinationSize0;
    CNC_UINT8 * Destination0_p;
    CNC_UINT32 BufferSize;
    CNC_UINT32 Timeout;
    CNC_UINT32 Timer;
    CNC_UINT16 Pid;
    CNC_UINT8 Inuse;
    CNC_UINT8 Running;
    cncDemuxFilterType_t FilterType;
    void* userparam;
}cncDemuxFilter_t;

/**
 * Demux channel structure
 */
typedef  struct _cncDemuxChannelHandle_t
{
	CNC_UINT16 					Channel_id;
	CNC_UINT16 					Channel_Pid;
	CSDEMUX_HANDLE Slot_Handle;
	CSDEMUX_HANDLE Filter_Handle;
	CSDEMUX_HANDLE Video_Handle;
	CSDEMUX_HANDLE Audio_Handle;
	cncDemuxChannelType_t Channel_type;
	CNC_UINT32 	 ChannelCheckHandle;
} cncDemuxChannelHandle_t;

typedef  cncDemuxFilter_t cncDemuxFilterHandle_t;

typedef void *cncDemuxFilterPESHandle_t;
typedef void *cncDemuxDescriptorHandle_t;

/**
 * Filter parameters
 */
typedef struct _cncDemuxFilterParams_t
{
	CNC_UINT8 *FilterValues_p;
	CNC_UINT8 *FilterMasks_p;
	CNC_UINT8 *ModePattern_p;			/* SectionFilter only : not for the other 2 */
//	CSXPORT_FilterRepeatMode_t RepeatMode;
} cncDemuxFilterParams_t;

/**
 * PES Callback format
 * @param userparam
 * @param handle
 * @param timedout
 * @param data
 * @param size
 */
typedef void (*cncDemuxPESCallBack_t)(void* userparam,
                                cncDemuxFilterPESHandle_t  handle,
                                CNC_BOOL timedout,
                                CNC_UINT8 const* data,
                                CNC_UINT32 size);

/**
 * PES Data
 */
typedef struct _cncDemuxPESInfo_t
{
	CSDEMUX_HANDLE		 Slot_Handle;
	CSDEMUX_HANDLE		 Filter_Handle;
	cncDemuxSlotType_t              Slot_Type;
	CSDEMUX_FILTER_ID	 Filter_ID;
	CSDEMUX_PIDFT_ID	 Slot_ID;
	CSDEMUX_CHL_ID		 Channel_ID;
	cncDemuxFilterParams_t FilterSet;
	cncDemuxFilterMode_t	 Mode;
	cncOSSemaphore_t*	 DmxSemHandle;
	cncDemuxPESCallBack_t 	 callback;
	cncDemuxBufferMode_t buff_mode;
	CNC_UINT8					 Nomatch[8];
	CNC_UINT8					 SWFilterMatch[8];
	CNC_UINT8					 SWFilterMask[8];
	CNC_UINT32 				 DestinationSize0;
	CNC_UINT8 *				 Destination0_p;
	CNC_UINT32 				 BufferSize;
	CNC_UINT32                              Timeout;
	CNC_UINT32				 StartTime;
	CNC_UINT16				 Pid;
	CNC_UINT8                   Inuse;
	CNC_UINT8                   Running;
	CNC_BOOL             IfGetData_inCurrentCircle;
	CNC_BOOL            crc_check_enable;
} cncDemuxPESInfo_t;

/**
 * DES value
 */
typedef struct _cncDemuxDESValue_t
{
	unsigned int key_len;
	unsigned int key[6];
} cncDemuxDESValue_t;

/**
 * DES Parameters
 */
typedef struct _cncDemuxDESParams_t
{
	CSDEMUX_HANDLE Slot_handle[64];
	int Pid[64];
	CNC_BOOL isused[64];
} cncDemuxDESParams_t;

/**
 * Demux Source Information
 */
 typedef struct _cncDemuxSource_t
{
	CNC_UINT32 source_num;
	CNC_DemuxSourceHandle source_handle[10];
} cncDemuxSource_t;


/**
 * Initialize the CNC_Demux module
 * @param dualTuner if true, initialize the demux for both tuner inputs, if false, initialize only 1 tuner input
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_Init(CNC_BOOL dualTuner);

/**
 * Finalize the CNC_Demux module
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_Term(void);

/**
 * Open a demux channel
 * @param id id for the demux channel to open
 * @param inputMode associate the demux channel to a tuner or DMA input
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_OpenChannel(CSDEMUX_CHL_ID id, CSDEMUX_INPUT_MOD inputMode);

/**
 * Set the tuner input to either serial or parallel mode
 * @param id id for the tuner demux channel to open
 * @param tunerMode whether the tuner is parallel or serial
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetTunerInputMode(CSDEMUX_CHL_ID id, CSDEMUX_TUNER_MOD tunerMode);

/**
 * Set the demux channel switch mode
 * @param id id for the demux channel to open
 * @param switchMode demux channel for the switching
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetSwitchMode(CSDEMUX_CHL_ID id, CSDEMUX_SWITCH_MOD switchMode);

/**
 * Enable the demux channel
 * @param id id for the  demux channel to open
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_ChannelEnable(CSDEMUX_CHL_ID id);

/**
 * Create a flush channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_CreateFlushChannel(void);

/**
 * Get the source handles for each of the demux channels
 * @param demuxsource structure for returning the source handles
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_GetSourceHandle(cncDemuxSource_t* demuxsource);

/**
 * Write data in the DMA source buffer for the demux channel
 * @param handle handle to the source (read from CNC_Demux_GetSourceHandle)
 * @param buf buffer to the data to write
 * @param len size (in bytes) of the data to be written
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_ChannelDMAWrite(CNC_DemuxSourceHandle handle, const CNC_UINT8 *const buf, CNC_UINT32 len);

/**
 * Create a channel in the demuxer of the given type (for channels with video and audio only)
 * @param channelhandle handle to demux channels
 * @param channeltype type of channel to create
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_CreateChannel(cncDemuxChannelHandle_t*channelhandle ,cncDemuxChannelType_t channeltype);

/**
 * Create a channel in the demuxer of the given type (for channels with extended PES, PSI and PCR channels)
 * @param channelhandle handle to demux channels
 * @param channeltype type of channel to create
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_CreateChannel_Ext(cncDemuxChannelHandle_t* channelhandle, cncDemuxChannelType_t channeltype);

/**
 * Setup the audio and video sources for the demux channels
 * @param vchannelhandle handle to video channel
 * @param achannelhandle handle to the audio channel
 * @param inputmod identify where the input source is tuner or DMA
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetAVChannelSource(cncDemuxChannelHandle_t* vchannelhandle,cncDemuxChannelHandle_t* achannelhandle, cncDemuxInputMode_t inputmod);

/**
 * Start the processing of the demux channel
 * @param channelhandle handle to demux channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_StartChannel(cncDemuxChannelHandle_t* channelhandle);

/**
 * Stop the demuxing on the channel
 * @param channelhandle handle to demux channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_StopChannel(cncDemuxChannelHandle_t* channelhandle);

/**
 * Delete the channel
 * @param channelhandle handle to demux channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_DeleteChannel(cncDemuxChannelHandle_t* channelhandle);

/**
 * Add a filter to the demux channel
 * @param channelhandle handle to demux channel
 * @param filterhandle handle to the filter (created with CNC_Demux_AllocateFilter)
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_ChannelAddFilter(cncDemuxChannelHandle_t* channelhandle,cncDemuxFilterHandle_t* filterhandle);

/**
 * Delete a filter in the demux channel
 * @param channelhandle handle to demux channel
 * @param filterhandle handle to the filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_ChannelDeleteFilter(cncDemuxChannelHandle_t* channelhandle,cncDemuxFilterHandle_t* filterhandle);

/**
 * Allocate a new filter structure
 * @param filterhandle pointer to the allocated filter
 * @param filtertype type of filter to allocate
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_AllocateFilter(cncDemuxFilterHandle_t** filterhandle, cncDemuxFilterType_t filtertype);

/**
 * Deallocate a filter structure
 * @param filterhandle pointer to a previously allocated filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_DeallocateFilter(cncDemuxFilterHandle_t* filterhandle);

/**
 * Set the parameters for the filter
 * @param filterhandle pointer to the filter
 * @param bufsize buffer size
 * @param match match array for the filter
 * @param mask mask for the filter
 * @param Nomatch don't match array for the filter
 * @param mode filter mode
 * @param timeout timeout for the filter
 * @param callback callback function
 * @param userparam data to pass into the callback function
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetFilterParams(cncDemuxFilterHandle_t*     filterhandle,
                                                                        CNC_UINT32                bufsize,
                                                                        CNC_UINT8 const         match[8],
                                                                        CNC_UINT8 const         mask[8],
                                                                        CNC_UINT8 const 		Nomatch[8],
                                                                        cncDemuxFilterMode_t    mode,
                                                                        CNC_UINT32                timeout,
                                                                        cncDemuxCallBack_t      callback,
                                                                        void*                userparam);

/**
 * Enable the filter
 * @param filterhandle pointer to the filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_EnableFilter(cncDemuxFilterHandle_t* filterhandle);

/**
 * Disable the filter
 * @param filterhandle pointer to the filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_DisableFilter(cncDemuxFilterHandle_t* filterhandle);

/**
 * Enable the filter
 * @param filterhandle pointer to the filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_EnableFilter_Ext(cncDemuxFilterHandle_t* filterhandle);

/**
 * Flush the filter buffer
 * @param filterhandle pointer to the filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_FlushFilterBuffer(cncDemuxFilterHandle_t* filterhandle);

/**
 * Set the pid for the channel
 * @param channelhandle pointer to the filter
 * @param Pid pid to set for the channel
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetChannelPid(cncDemuxChannelHandle_t* channelhandle,CNC_UINT16 Pid);

/**
 * Allocate a descrambler structure
 * @return handle to the descramble structure
 */
cncDemuxDescriptorHandle_t CNC_Demux_AllocateDescrambler(void);

/**
 * Delete the descrambler structure
 * @param hDesc handle to the descrambler
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_DeleteDescrambler(cncDemuxDescriptorHandle_t hDesc);

/**
 * Set the pid for the descrambler
 * @param hDesc handle to the descrambler
 * @param Pid pid to descramble
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetDescPid(cncDemuxDescriptorHandle_t hDesc, CNC_UINT16 Pid);

/**
 * Set the odd key for the descrambler
 * @param hDesc handle to the descrambler
 * @param key odd key for the descrambler
 * @param keylen length of the key
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetDescOddKey(cncDemuxDescriptorHandle_t hDesc, CNC_UINT8 *key, CNC_SINT32 keylen);

/**
 * Set the even key for the descrambler
 * @param hDesc handle to the descrambler
 * @param key even key for the descrambler
 * @param keylen length of the key
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetDescEvenKey(cncDemuxDescriptorHandle_t hDesc, CNC_UINT8 *key, CNC_SINT32 keylen);

/**
 * Enable the descrambler
 * @param hDesc handle to the descrambler
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_EnableDescambler(cncDemuxDescriptorHandle_t hDesc);

/**
 * Disable the descrambler
 * @param hDesc handle to the descrambler
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_DisableDescambler(cncDemuxDescriptorHandle_t hDesc);

/**
 * Create a PES Channel (combines functions of other calls)
 * @param channelhandle handle to the channel
 * @param filterhandle PES filter created
 * @param pid pid for the filter
 * @param callback callback function for the filter
 * @param userparam user data to pass into the callback function
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_CreatePESChannel(cncDemuxChannelHandle_t* channelhandle, cncDemuxFilterHandle_t**  filterhandle,CNC_UINT16 pid, cncDemuxCallBack_t callback ,void* userparam);

/**
 * Delete a PES Channel (combines functions of other calls)
 * @param channelhandle handle to the channel
 * @param filterhandle PES filter
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_DeletePESChannel(cncDemuxChannelHandle_t channelhandle, cncDemuxFilterHandle_t*  filterhandle);

Result_Type_e CNC_Demux_Filter_ReadData(CSDEMUX_HANDLE handle, unsigned char *buf, unsigned int *size);
Result_Type_e CNC_Demux_PESFilterSetup(unsigned short Pid,void (* call_back_pes_function)(CSDEMUX_HANDLE hFilter, CSDEMUX_SECEVENT *event),CSDEMUX_HANDLE *hPidFilter,CSDEMUX_HANDLE *hTableIdFilter);
Result_Type_e CNC_Demux_PESFilterFree(CSDEMUX_HANDLE hPidFilter,CSDEMUX_HANDLE hTableIdFilter);

/**
 * INTERNAL FUNCTION: DO NOT USE
 * @param channelhandle handle to demux channels
 * @param SwitchMod choose between types of switch
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_ReCreateAVChannelPDFT(cncDemuxChannelHandle_t* channelhandle, CNC_UINT8 SwitchMod);

/**
 * Set a callback into video demux filter, for monitor Scramble Status.
 * @param callback is the notify callback
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_Demux_SetScrambleStatusNotify(demux_av_scramble_cb callback);

#ifdef __cplusplus
}
#endif

#endif

