/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_filemgr.h
 * @ingroup core
 * The CNCFILEMGR library is used to access the disks, partitions and files mounted in the system
 *
 * The  CNCFILEMGR library is used to control access to files stored on storage devices  in the system
    (typically USB disk drives).&nbsp;  The file manager contains two API interfaces <strong>CNC_DISK_xxx</strong>
    and <strong>CNC_FILE_xxx</strong>.&nbsp;  The <strong>CNC_DISK</strong> interface is used for interacting with
    disks mounted through the USB interface; the <strong>CNC_FILE</strong> interface is for interacting with the list
    of files stored on folders on those disks.

  The  CNCFILEMGR module is initialized with the call <strong>CNC_DISK_Init()</strong> (to initialize the disk storage)
  and <strong>CNC_FILE_Init()</strong> (to initialize the file manager).&nbsp; Both of these calls should be made
  during system initialization.

  <strong>Disk  Functions</strong><br />
  The  function <strong>CNC_DISK_GetCount()</strong> will return the number of disks mounted in
  the system.&nbsp; <strong>CNC_DISK_GetInfo()</strong> will return the number of each disk, the
  number of partitions on the disk, and the name of the disk.&nbsp; The function <strong>CNC_DISK_GetUSBStatus()</strong>
  can be used to identify whether a USB disk  is available.&nbsp; <br />
  The  function <strong>CNC_DISK_GetPartitionPath()</strong> will return the file system path name for
  the partition in the system.&nbsp; The  application can register a callback that is called when the
  USB is plugged in or unplugged using the function <strong>CNC_Disk_InstallCallback()</strong>and
  <strong>CNC_DISK_UninstallCallback()</strong>.

  <strong>File Functions</strong><br />
  The  CNC_FILE module is used to manage lists of files from different folders on the
  disk and to perform operations on specific files.<br />
  For  a given file path name, the following functions can be used to get information
  about the path: <strong>CNC_FILE_GetType()</strong> returns the type of the file, <strong>CNC_FILE_GetSize()</strong>
  returns the size in bytes of the file, and <strong>CNC_FILE_Exists()</strong> verifies the path points
  to a valid  file.&nbsp; <strong>CNC_FILE_IsReadOnly()</strong> can be used to check the access rights to
  the file and <strong>CNC_FILE_IsDirectory()</strong> can be used to determine if the path points  to a folder.<br />
  At  a given path, the function CNC_FILE_GetListByType() is used to retrieve a data
  structure to the list of files available in the current path.

  The  module allows for operations on the files as well using the functions <strong>CNC_FILE_Delete()</strong>,
 *
 */
#ifndef _CNC_FILEMGR_H_
#define _CNC_FILEMGR_H_
#include "cnc_common_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif

/// Maximum path size for the USB Path
#define CNC_FILE_MAX_USB_PATH                100
/// Maximum number of partitions per disk
#define CNC_DISK_MAX_PARTITION 10
/// Max number of Mounted Disk Devices
#define CNC_DISK_MAX_DEV_NUMBERS 4
/// Max number of partitions per disk
#define CNC_DISK_MAX_PARTITION_NUMBERS 10
/// Max pathname length supported
#define CNC_FILE_MAX_PATH_LEN 4096
#define CNC_FILE_MAX_METADATA_SIZE 2048
/* Minimum Recording Content Duration */
#define CNC_FILE_MIN_RECORDING_DURATION 10000
typedef enum _cncFileType_t {
    LIST = 0,    ///< File is a list (folder)
    PHOTO,       ///< File has a photo extension
    MUSIC,       ///< File has a music extension
    VIDEO,       ///< File has a video extension
    TORRENT,     ///< File has a bit torrent extension
    SPECIAL,     ///< File has a special extension
    DISK,        ///< File is a disk
    UNKNOWN,     ///< Unrecognized file type
    ALL,         ///< All extensions
    ALL_SUPPORT, ///< All supported extensions
    PLAY_LIST,
    OTHER,       ///< Other
} cncFileType_t;

typedef enum _cncDiskFormat_t {
    FORMAT_START=0,
    FAT32,
    FORMAT_END,
    
} cncDiskFormat_t;

typedef struct _cncFileListInfo_t {
    int occupied;            ///< file list is locked
    CNC_SINT32 number;       ///< Number of files in the list
    CNC_SINT64 size;         ///< Size of the file (in bytes)
    char name[CNC_FILE_MAX_PATH_LEN]; ///< Name of the current list location
    char path[CNC_FILE_MAX_PATH_LEN]; ///< Full pathname of the current location
    cncFileType_t type;          ///< File type used to generate the list
} cncFileListInfo_t;

typedef enum {
    CNC_USB_REMOVE = 0,      ///< USB has been removed
    CNC_USB_INSERT,          ///< USB has been inserted
    CNC_USB_UNKNOWN          ///< Status of the USB is unknown
} cncDiskUSBStatus_t;

typedef struct _cncDiskUSBFile_t {
    CNC_SINT32 usb_num;      ///< Number (disk number) of the USB
    char sdname[50];         ///< Device name of the USB
    char usbname[50];        ///< File system pathname of the USB
} cncDiskUSBFile_t;

typedef struct _cncDiskDeviceInfo_t {
    int dev_type;                                    ///< 0:storge device 1: xd_rom
    int dev_partition_numbers;                       ///< Number of Partitions on the device
    char file_system[CNC_DISK_MAX_PARTITION_NUMBERS][20];     ///< ntfs,fat32...
    char dev_sys_name[20];                           ///< like sda,sdb,sr0...
    char dev_mount_path[CNC_DISK_MAX_PARTITION_NUMBERS][50];  ///< exp: /home/usb0
    char dev_sys_path[CNC_DISK_MAX_PARTITION_NUMBERS][50];    ///< exp: /dev/sd0
    struct _cncDiskDeviceInfo_t *next_device;
} cncDiskDeviceInfo_t;

typedef struct _cncDiskPartitionInfo_t {
    char name[50];                 ///< Partition Name
    CNC_SINT32 number;             ///< Partition Number
    char dev_path[CNC_DISK_MAX_PARTITION][50];//device path
    char path[CNC_DISK_MAX_PARTITION][50];  ///< Partition Pathname
} cncDiskPartitionInfo_t;

typedef struct _cncDiskInfo_t {
    CNC_SINT32 disk_num;                  ///< Disk Number
    cncDiskPartitionInfo_t disk[CNC_DISK_MAX_PARTITION]; ///< Disk Partitions Information
} cncDiskInfo_t;

typedef struct _cncDiskMemStatus_t
{
    CNC_SINT32       availSpaceInPercent;
    CNC_DWORD        availSpaceInBytes;
    CNC_BOOL         isDiskFull;
} cncDiskMemStatus_t;

/// Callback function type for receiving DISK plug/unplug events
typedef void (*cncDiskCallback_t)(const char * disk_path);

/**
 * Initialize the CNC_DISK module
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_DISK_Init(void);

/**
 * Get the status of the USB Disks
 * @return if USB disk is available then USB_INSERT, else USB_REMOVE
 */
cncDiskUSBStatus_t CNC_DISK_GetUSBStatus(void);

/**
 * Install a callback function when disks are inserted or removed
 * @return none
 */
void CNC_DISK_InstallCallback(cncDiskCallback_t callback);

/**
 * Uninstall the disk callback function
 * @return none
 */
void CNC_DISK_UninstallCallback(void);

/**
 * Get the number of disks mounted in the system
 * @return number of disks mounted
 */
int CNC_DISK_GetCount(void);

/**
 * Get information about the disk including partitions
 * @return structure containing the number of disks and partition information for each partition
 */
cncDiskInfo_t * CNC_DISK_GetInfo(void);
/**
 * Format partition partition_path: exp: "/dev/sda1" 
 Now only support FAT32
 */

Result_Type_e CNC_DISK_Format(char *partition_path,cncDiskFormat_t format);


/**
 * Get the pathname for the partition
 * @param path string pointer into which to store the path name
 * @param diskIndex index of the disk
 * @param partitionIndex index of the partition
 * @return none
 */
void CNC_DISK_GetPartitionPath(char* path, CNC_UINT8 diskIndex, CNC_UINT8 partitionIndex);

/**
 * Utility routine for printing information about the disk
 * @param info structure returned from CNC_DISK_GetInfo
 * @return none
 */
void CNC_DISK_PrintInfo(const cncDiskInfo_t *info);

/**
 * Utility routine for fetching specified disk Memory status
 * @param diskMemStatus structure will return with available Memory Space details
 * @return 0 - success, any other value treated as error
 */
CNC_SINT32 CNC_DISK_GetMemoryStatus(CNC_CHAR *path, cncDiskMemStatus_t *diskMemStatus);

/**
 * Retrieve the list of files based on the input type
 * @param path pathname to the folder
 * @param type type of file to use in the filter
 * @param listcount pointer to an integer with the number of files returned
 * @return array of filelist_info structures for each of the files returned
 */
cncFileListInfo_t * CNC_FILE_GetListByType(char *path, cncFileType_t type, CNC_SINT32 *listcount);

/**
 * Step into a new folder
 * @param pathname pathname to the folder
 * @param file_num number of files in the folder
 * @param list_num number of subfolders in the folder
 * @param type file type of file to use in the filter
 * @return array of filelist_info structures for each of the files returned
 */
cncFileListInfo_t * CNC_FILE_FolderIn(char *pathname, CNC_SINT32 *file_num, CNC_SINT32 *list_num, cncFileType_t type);

/**
 * Step into a parent folder
 * @param pathname pathname to the folder
 * @param file_num number of files in the folder
 * @param list_num number of subfolders in the folder
 * @param type file type of file to use in the filter
 * @return array of filelist_info structures for each of the files returned
 */
cncFileListInfo_t * CNC_FILE_FolderOut(char *pathname, CNC_SINT32 *file_num, CNC_SINT32 *list_num, cncFileType_t type);

#if 0
/**
 * Sort two files in ASCII order
 * @param pathname pathname to the folder
 * @param file_num number of files in the folder
 * @param list_num number of subfolders in the folder
 * @return array of filelist_info structures for each of the files returned
 */
int CNC_FILE_SortByASCII(const void *filelist_1, const void *filelist_2);
#endif

/**
 * Get the type of the file based on the file name
 * @param file_name name of the file
 * @return type of the file (defined in file_type enumeration)
 */
cncFileType_t CNC_FILE_GetFileType(char* file_name);

/**
 * Get the size of the file
 * @param file_name name of the file
 * @return size (in bytes) of the file
 */
CNC_SINT64 CNC_FILE_GetSize(const char * file_name);

/**
 * Check if the file exists
 * @param path pathname for the folder
 * @param name name of the file
 * @return CNC_BOOL_TRUE if file exists, CNC_BOOL_FALSE if file doesn't exist
 */
CNC_BOOL CNC_FILE_Exists(const char *path, char *name);

/**
 * Check if the file is read only
 * @param path pathname to the file
 * @return CNC_BOOL_TRUE if file is read only, otherwise CNC_BOOL_FALSE
 */
CNC_BOOL CNC_FILE_IsReadOnly(const char* path);

/**
 * Check if the file is a folder (directory)
 * @param fullpath pathname to the file
 * @return CNC_BOOL_TRUE if file is a directory, otherwise CNC_BOOL_FALSE
 */
CNC_BOOL CNC_FILE_IsDirectory(const char *fullpath);

/**
 * Delete the file
 * @param fullpath full pathname to the file
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_FILE_Delete(const char *fullpath);

/**
 * Copy the file into paste memory
 * @param path pathname to the folder containing the file
 * @param name name of the file
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_FILE_Copy(const char *path, const char * name);

/**
 * Paste the file (from paste memory) to the new path
 * @param path pathname to the folder in which to paste the file
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_FILE_Paste(const char *path);

/**
 * Paste the file (from paste memory) to the new path and remove from previous path
 * @param path pathname to the folder in which to paste the file
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_FILE_CutPaste(const char *path);

/**
 * Utility string compare function
 * @param p1 first string
 * @param p2 second string
 * @return index where the strings differ, 0 if they match
 */
int CNC_str_cmp(const char *p1, const char *p2);

#ifdef __cplusplus
}
#endif

#endif

