/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_http.h
 * The CNCHTTP library provides access to HTTP communication
 * @ingroup http
 */

#ifndef _CNC_HTTPCLIENT_API_H_
#define _CNC_HTTPCLIENT_API_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_common_error.h"

#ifdef ARCH_CSM1900
    #include <curl/curl.h>
#endif


/* =========================== Macro definitions ========================== */
#define CNC_HTTP_URL_MAX                    1024
#define CNC_FILE_PATH_MAX                   1024
#define CNC_HTTP_HEADER_MAX                  256

/* =========================== Type definitions =========================== */
typedef enum _cncHttpClientState_t
{
    CNC_HTTP_CLIENT_STATE_NONE = 0,
    CNC_HTTP_CLIENT_STATE_READY,
    CNC_HTTP_CLIENT_STATE_CONNECTING,
    CNC_HTTP_CLIENT_STATE_CONNECTED,
    CNC_HTTP_CLIENT_STATE_HEADERS_PARSED,
    CNC_HTTP_CLIENT_STATE_DOWNLOAD_IN_PROGRESS,
    CNC_HTTP_CLIENT_STATE_DOWNLOAD_PAUSE,       // TODO: Need to implement
    CNC_HTTP_CLIENT_STATE_DOWNLOAD_RESUME,      // TODO: Need to implement 
    CNC_HTTP_CLIENT_STATE_DOWNLOAD_COMPLETED,
    CNC_HTTP_CLIENT_STATE_CLOSING,
    CNC_HTTP_CLIENT_STATE_CLOSED,

    // Errors 
    CNC_HTTP_CLIENT_STATE_ERROR = -900
} cncHttpClientState_t;

typedef enum _cncHttpClientStatusCode_t
{
    CNC_HTTP_CLIENT_STATUS_NONE               = 0,

    CNC_HTTP_CLIENT_STATUS_CONTINUE           = 100, /**< OK to continue with request */
    CNC_HTTP_CLIENT_STATUS_SWITCH_PROTOCOLS   = 101, /**< server has switched protocols */

    CNC_HTTP_CLIENT_STATUS_OK                 = 200, /**< request completed */
    CNC_HTTP_CLIENT_STATUS_CREATED            = 201, /**< object created, reason new URI */
    CNC_HTTP_CLIENT_STATUS_ACCEPTED           = 202, /**< async completion */
    CNC_HTTP_CLIENT_STATUS_PARTIAL            = 203, /**< partial completion */
    CNC_HTTP_CLIENT_STATUS_NO_CONTENT         = 204, /**< no information to return */ 
    CNC_HTTP_CLIENT_STATUS_RESET_CONTENT      = 205, /**< request completed, but clear form */
    CNC_HTTP_CLIENT_STATUS_PARTIAL_CONTENT    = 206, /**< partial GET furfilled */

    CNC_HTTP_CLIENT_STATUS_AMBIGUOUS          = 300, /**< server couldn't decide what to return */
    CNC_HTTP_CLIENT_STATUS_MOVED              = 301, /**< object permanently moved */
    CNC_HTTP_CLIENT_STATUS_REDIRECT           = 302, /**< object temporarily moved */
    CNC_HTTP_CLIENT_STATUS_REDIRECT_METHOD    = 303, /**< redirection with new access method */
    CNC_HTTP_CLIENT_STATUS_NOT_MODIFIED       = 304, /**< if-modified-since was not modified */ 
    CNC_HTTP_CLIENT_STATUS_USE_PROXY          = 305, /**< redirection to proxy, location header specifies proxy to use */
    CNC_HTTP_CLIENT_STATUS_TEMPORARY_REDIRECT = 307, /**< temporary Redirect */

    CNC_HTTP_CLIENT_STATUS_BAD_REQUEST        = 400, /**< request with invalid syntax */
    CNC_HTTP_CLIENT_STATUS_DENIED             = 401, /**< unauthorized access */
    CNC_HTTP_CLIENT_STATUS_PAYMENT_REQ        = 402, /**< payment required */
    CNC_HTTP_CLIENT_STATUS_FORBIDDEN          = 403, /**< request forbidden */
    CNC_HTTP_CLIENT_STATUS_NOT_FOUND          = 404, /**< object not found */
    CNC_HTTP_CLIENT_STATUS_BAD_METHOD         = 405, /**< method is not allowed */
    CNC_HTTP_CLIENT_STATUS_NONE_ACCEPTABLE    = 406, /**< no response acceptable to client found */
    CNC_HTTP_CLIENT_STATUS_PROXY_AUTH_REQ     = 407, /**< proxy authentication required */
    CNC_HTTP_CLIENT_STATUS_REQUEST_TIMEOUT    = 408, /**< server timed out waiting for request */
    CNC_HTTP_CLIENT_STATUS_CONFLICT           = 409, /**< user should resubmit with more info */
    CNC_HTTP_CLIENT_STATUS_GONE               = 410, /**< the resource is no longer available */
    CNC_HTTP_CLIENT_STATUS_LENGTH_REQUIRED    = 411, /**< the server refused to accept request without Content-length */
    CNC_HTTP_CLIENT_STATUS_PRECOND_FAILED     = 412, /**< precondition given in request failed */
    CNC_HTTP_CLIENT_STATUS_REQUEST_TOO_LARGE  = 413, /**< request entity was too large */
    CNC_HTTP_CLIENT_STATUS_URI_TOO_LONG       = 414, /**< request URI too long */
    CNC_HTTP_CLIENT_STATUS_UNSUPPORTED_MEDIA  = 415, /**< unsupported media type */
    CNC_HTTP_CLIENT_STATUS_REQUEST_RANGE_NS   = 416, /**< Requested Range Not Satisfiable */

    CNC_HTTP_CLIENT_STATUS_SERVER_ERROR       = 500, /**< internal server error */
    CNC_HTTP_CLIENT_STATUS_NOT_SUPPORTED      = 501, /**< method not supported */
    CNC_HTTP_CLIENT_STATUS_BAD_GATEWAY        = 502, /**< error response received from gateway */
    CNC_HTTP_CLIENT_STATUS_SERVICE_UNAVAIL    = 503, /**< unable to handle request temporarily overloaded */
    CNC_HTTP_CLIENT_STATUS_GATEWAY_TIMEOUT    = 504, /**< Gateway timeout */
    CNC_HTTP_CLIENT_STATUS_VERSION_NOT_SUP    = 505  /**< HTTP version not support */
} cncHttpClientStatusCode_t;

typedef enum _cncHttpClientMethod_t
{
    CNC_HTTP_CLIENT_METHOD_GET = 0, /**< HTTP Methods */
    CNC_HTTP_CLIENT_METHOD_POST,
    CNC_HTTP_CLIENT_METHOD_HEAD,
    CNC_HTTP_CLIENT_METHOD_OPTIONS,
    CNC_HTTP_CLIENT_METHOD_CONNECT,
    CNC_HTTP_CLIENT_METHOD_PUT,
    CNC_HTTP_CLIENT_METHOD_DELETE,
    CNC_HTTP_CLIENT_METHOD_TRACE,
    CNC_HTTP_CLIENT_METHOD_CUSTOM  /**< Need to pass custom method which need to be sent */
}cncHttpClientMethod_t;

typedef enum _cncHttpClientOption_t
{
    CNC_HTTP_CLIENT_OPT_NONE = 0,
    CNC_HTTP_CLIENT_OPT_URL,

    CNC_HTTP_CLIENT_OPT_METHOD,

    CNC_HTTP_CLIENT_OPT_HEADER,
    CNC_HTTP_CLIENT_OPT_COUNTRY_CODE,

    CNC_HTTP_CLIENT_OPT_COOKIE,
}cncHttpClientOption_t;

typedef enum _cncHttpClientInfo_t
{
    CNC_HTTP_CLIENT_INFO_NONE = 0,
    CNC_HTTP_CLIENT_INFO_URL,
    CNC_HTTP_CLIENT_INFO_HEADER,
    CNC_HTTP_CLIENT_INFO_CONTENT_LENGTH,
    CNC_HTTP_CLIENT_INFO_CONTENT_TYPE,
    CNC_HTTP_CLIENT_INFO_STATUS_CODE,
    CNC_HTTP_CLIENT_INFO_PROTOCOL,
} cncHttpClientInfo_t;

typedef enum _cncHttpClientType_t
{
    CNC_HTTP_CLIENT_MTHTTP = 0,
    CNC_HTTP_CLIENT_CURL,  /**< libcurl based implementation */
    CNC_HTTP_CLIENT_MAX,
} cncHttpClientType_t;

/**
 * \struct  cncHttpsClientConfig_t
 * \brief   Structure which Contains HTTPS (SSL/TLS) Client Configuration information
 */
typedef struct _cncHttpsClientConfig_t
{
    CNC_BOOL                            verifyPeer;                     /**< Verify Peer if set to true */
    CNC_BOOL                            verifyHost;                     /**< Verify Server Cert if set to true */
    CNC_CHAR                            a_certPath[CNC_FILE_PATH_MAX];  /**< Certificate file location */
    CNC_CHAR                            a_caPath[CNC_FILE_PATH_MAX];    /**< CA File location */
} cncHttpsClientConfig_t;

/**
 * \struct  cncHttpClientConfig_t
 * \brief   A structure which Contains HTTP Client Configuration information
 */
typedef struct _cncHttpClientConfig_t
{
    cncHttpClientType_t                 clientType;
    cncHttpsClientConfig_t              httpsConfig;
    CNC_SINT32                          sockBufSize;
    CNC_SINT32                          connectTimeOut;  /**< in secs */
    CNC_SINT32                          nwTimeOut;
    CNC_SINT8                           maxReDirectCnt;
    CNC_SINT8                           retryCnt;
    CNC_SINT8                           pipelineReqCnt;
    CNC_BOOL                            isHttpKeepAliveReq;
    CNC_BOOL                            isTcpKeepAliveReq;
    CNC_BOOL                            isItBlockMode;
}cncHttpClientConfig_t;

/**
 * \struct  cncHttpClientResourceInfo_t
 * \brief   
 */
typedef struct _cncHttpClientResourceInfo_t
{
    CNC_CHAR                    a_contentType[CNC_HTTP_HEADER_MAX];
    CNC_CHAR                   *p_buffer;                            /**< buffer */
    CNC_SINT32                  buffSize;
    CNC_SINT64                  contentLength;
    CNC_SINT64                  bytesConsumedFromStart;              /**< write done Lenth */
    cncHttpClientStatusCode_t   httpStatusCode;                      /**< HTTP Status Code */
    cncHttpClientState_t        httpClientState;
} cncHttpClientResourceInfo_t;

typedef CNC_VOID (*F_cncHttpClientDownloadCbf_t)(CNC_VOID *p_v_args, cncHttpClientResourceInfo_t *p_resourceInfo); 

#ifdef ARCH_CSM1900
    typedef CURLcode (*cncHttpSSL_CTX_CB_t)(CURL *curl, void *ssl_ctx, void *userptr); 
#endif

/**
 * \struct  cncHttpClientDownloadParam_t
 * \brief   A structure which Contains HTTP Client Configuration information
 */
typedef struct _cncHttpClientDownloadParam_t
{
    CNC_CHAR                         a_url[CNC_HTTP_URL_MAX];
    CNC_CHAR                         a_filePath[CNC_FILE_PATH_MAX];  /**< file location where to write data if NULL it will dump to buffer */
    CNC_CHAR                         a_userAgent[CNC_HTTP_HEADER_MAX];
    CNC_CHAR                         a_contentType[CNC_HTTP_HEADER_MAX];
    CNC_CHAR                         a_cookie[CNC_HTTP_HEADER_MAX];
    CNC_CHAR                         a_soap[CNC_HTTP_HEADER_MAX];
    CNC_UINT32                       totalTimeout;                   /**< Transfer timeout to connect, send, recv data */
    CNC_UINT32                       progressEventInterval;          /**< Download Progress Event Interval */
    CNC_SINT32                       startOffset;                    /**< Range Start Offset */
    CNC_SINT32                       endOffset;                      /**< Range End Offset */
    cncHttpClientMethod_t            method;
    CNC_VOID                        *p_userData;                     /**< application handle */
    CNC_CHAR                        *p_buffer;                       /**< POST Data */
    CNC_SINT32                       buffSize;                       /**< POST Data size */
    F_cncHttpClientDownloadCbf_t     p_f_httpClientCbf;
#ifdef ARCH_CSM1900
    cncHttpSSL_CTX_CB_t              p_f_httpSSL_CTXCb;
#endif    
    CNC_SINT64                       maxBandwidth;                   /**< Maximum bandwidth to be used by curl in Bytes per second */
} cncHttpClientDownloadParam_t;

typedef enum _cncHttpClientProtocol_t
{
    CNC_HTTP_CLIENT_PROTO_UNKNOWN = 0,
    CNC_HTTP_CLIENT_PROTO_HTTP,
    CNC_HTTP_CLIENT_PROTO_ICY
} cncHttpClientProtocol_t;


/* ======================== Function declarations ========================= */
/**
 * Initialize HTTP Client
 * @return E_cncErrCode_t        - returns error code
 */
E_cncErrCode_t CNC_HttpClient_Init(void);

/**
 * DeInitialize HTTP Client
 * @return E_cncErrCode_t        - returns error code
 */
E_cncErrCode_t CNC_HttpClient_DeInit(void);

/**
 * Create an HTTP client with a provided options like user agent.
 * @param  pp_httpClientHandle   [out] handle for the http client
 * @param  pconfigParam          [in]  which consists of user/upper layer specified http Client config params.
 * @return E_cncErrCode_t        [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_Create(CNC_VOID **pp_httpClientHandle, cncHttpClientConfig_t *p_configParam);

/**
 * Destroy an HTTP client.
 * @param p_httpClientHandle    [in]  handle for the http client
 * @return E_cncErrCode_t       [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_Destroy(CNC_VOID *p_httpClientHandle);

/**
 * Set Options like Adding Header/URL/Country Code/ to HTTP client.
 * @param  p_httpClientHandle    [in]  handle for the http client
 * @param  option                [in]  Http Option
 * @param  variable args         [in]  pass arguments based on option
 *         CNC_HTTP_CLIENT_OPT_URL             - one argument type char * url
 *         CNC_HTTP_CLIENT_OPT_METHOD          - one argument type cncHttpClientMethod_t 
 *         CNC_HTTP_CLIENT_OPT_HEADER          - two arguments of char * Header name, Value
 *         CNC_HTTP_CLIENT_OPT_COUNTRY_CODE    - one argument  char * of country code   
 * @return E_cncErrCode_t   [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_SetOption(CNC_VOID *p_httpClientHandle, cncHttpClientOption_t option, ...);

/**
 * Get Info like Header/URL/Country Code/ to HTTP client.
 * @param  p_httpClientHandle    [in]  handle for the http client
 * @param  info                  [in]  http info type
 * @param  variable args         [out]  pass arguments based on option
 *         CNC_HTTP_CLIENT_INFO_URL            - one argument type char * url
 *         CNC_HTTP_CLIENT_INFO_HEADER         - three arguments of type 
 *                                                 Header Name - char * 
 *                                                 Header Value - char * 
 *                                                 Header length - int *
 *         CNC_HTTP_CLIENT_INFO_CONTENT_LENGTH - one argument type long long
 *         CNC_HTTP_CLIENT_INFO_CONTENT_TYPE   - one argument type char *
 *         CNC_HTTP_CLIENT_INFO_STATUS_CODE    - one argument type int * 
 *         CNC_HTTP_CLIENT_INFO_PROTOCOL       - one argument type cncHttpClientProtocol_t* 
 * @return E_cncErrCode_t   [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_GetInfo(CNC_VOID *p_httpClientHandle, cncHttpClientInfo_t info, ...);

/**
 * Send request to Server.
 * @param  p_httpClientHandle    [in]  handle for the http client
 * @param  customReq             [in]  custom request which will override the default request 
 * @return E_cncErrCode_t        [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_SendRequest(CNC_VOID *p_httpClientHandle, CNC_CHAR *p_customReq);

/**
 * Recv Response from Server(Only Headers).
 * @param  p_httpClientHandle    [in]  handle for the http client
 * @return E_cncErrCode_t        [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_RecvResponse(CNC_VOID *p_httpClientHandle);

/**
 * Send request to Server.
 * @param  p_httpClientHandle    [in]  handle for the http client
 * @param  p_dataBuf             [in/out] Buffer to hold the data
 * @param  p_reqSize             [in/out] Req Bytes/Read Bytes
 * @return E_cncErrCode_t        [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_ReadData(CNC_VOID *p_httpClientHandle, CNC_UINT8 *p_dataBuf, CNC_SINT32 *p_reqSize);

/**
 * Write Data to Server.
 * @param  p_httpClientHandle [in]  handle for the http client
 * @param  p_dataBuf          [in] Buffer to hold the data
 * @param  p_buffSize         [in] Buffer Bytes
 * @return E_cncErrCode_t     [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_WriteData(CNC_VOID *p_httpClientHandle, CNC_UINT8 *p_dataBuf, CNC_SINT32 *p_buffSize);

/**
 * Shutdown an HTTP client.
 * @param p_httpClientHandle    [in]  handle for the http client
 * @return E_cncErrCode_t       [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_Shutdown(CNC_VOID *p_httpClientHandle);

/**
 * Download any requested URL to File or Buffer. 
 * This is a blocking Call, it will not return until it downloads Completes.
 * Data Can be downloaded to File or buffer
 * - File:
 *        User need to mention the a_filePath in p_downloadParam structure
 * - Buffer:
 *        If User doesnt specify file path content will be downloaded to p_buffer of size buffSize. 
 *        If it fails to allocate memory for the content it will return Error. 
 *        p_buffer will be freed on CNC_HttpClient_DestroyDownloadData. 
 *
 * Need to call CNC_HttpClient_DestroyDownloadData API for destroying instance
 * @param pp_httpClientHandle           [in/out]  handle for the http client
 * @param p_downloadParam               [in]      
 * @param p_configParam                 [in]
 * @param pp_resourceInfo               [in/out] 
 * @return E_cncErrCode_t               [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_DownloadFullStream(CNC_VOID **pp_httpClientHandle, cncHttpClientDownloadParam_t *p_downloadParam, 
                                                 cncHttpClientConfig_t *p_configParam, cncHttpClientResourceInfo_t **pp_resourceInfo);

/**
 * Destroy Download Data
 * @param cncHttpClientDwDataInfo_t [in]
 * @return E_cncErrCode_t           [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_DestroyDownloadData(CNC_VOID *p_httpClientHandle);

/**
 * Download any requested URL, it is asyncronous Download.
 * Data Can be downloaded to File or buffer
 * - File:
 *        User need to mention the a_filePath in p_downloadParam structure
 * - Buffer:
 *        If User doesnt specify file path, received data will be called through callback if p_f_httpClientCbf is set
 *        User should make a copy p_buffer of buffSize content
 *
 * Need to call CNC_HttpClient_DownloadStop API for destroying instance
 * @param p_httpClientHandle                  [in]  handle for the http client
 * @param p_downloadParam                     [in]
 * @param p_configParam                       [in]
 * @return E_cncErrCode_t                     [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_DownloadStart(CNC_VOID **pp_httpClientHandle, cncHttpClientDownloadParam_t *p_downloadParam, 
                                            cncHttpClientConfig_t *p_configParam);

/**
 * Stopping the Http Download.
 * @param  p_httpClientHandle           [in]  handle for the http client
 * @return E_cncErrCode_t               [out] returns error code
 */
E_cncErrCode_t CNC_HttpClient_DownloadStop(CNC_VOID *p_httpClientHandle);

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_HTTPCLIENT_API_H_ */

