#ifndef __include_mtexpires_h__
#define __include_mtexpires_h__

/*
 * BS:
 *----
 * Expires is a entity header field used in HTTP that indicates a date/time
 * after which the response is old.
 *
 * The format is an absolute date and time as defined by HTTP-date;
 * it MUST be in RFC 1123 date format:
 *
 * Expires = "Expires" ":" HTTP-date
 *
 * An example of its use is
 *
 * Expires: Thu, 01 Dec 1994 16:00:00 GMT
 *
 * Note: if a response includes a Cache-Control field with the max-
 * age directive, that directive overrides the Expires field.
 *
 */
#include <time.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

/*
 * BS:
 * Class to find the expiry time of a cookie.
 */
class MtExpires
{
    private:
        time_t expires; /*BS: Time in seconds to expire*/

    private:
        int comp(MtExpires b);

    public:
        /*
         * BS:
         * Passing = Expires
         * Nothing = Doesn't expire
         */
        enum Expires
        {
            Passing = -1, Nothing = -2
        };

        /*
         * BS: Assigning 0 to the time to expire. Most probably must expire after the
         * session is closed
         */
        MtExpires(void) :
            expires(0)
        {
        }
        ;
        /*
         * BS:
         * Must expire after time t
         */
        MtExpires(time_t t) :
            expires(t)
        {
        }
        ;
        /*
         * BS:
         * Passing or Nothing
         */
        MtExpires(Expires e) :
            expires(e)
        {
        }
        ;
        /*
         * BS:
         * Overloading =
         * Assign t to expires and return the caller
         */
        MtExpires operator =(time_t t)
        {
            expires = t;
            return *this;
        }
        ;
        MtExpires operator =(Expires e)
        {
            expires = e;
            return *this;
        }
        ;

        /*
         * BS:
         * Operator Overloading - Refer .c file
         */
        MtExpires operator =(std::string &s);

        /*
         * BS:
         * Returns the expiry time
         */
        operator time_t()
        {
            return expires;
        }
        ;

        /*
         * BS:
         * Returns true or false
         */
        bool operator ==(Expires e)
        {
            return comp(MtExpires(e)) == 0;
        }
        ;
        bool operator !=(Expires e)
        {
            return comp(MtExpires(e)) != 0;
        }
        ;
        bool operator <(Expires e)
        {
            return comp(MtExpires(e)) < 0;
        }
        ;
        bool operator ==(MtExpires e)
        {
            return comp(e) == 0;
        }
        ;
        bool operator <(MtExpires e)
        {
            return comp(e) < 0;
        }
        ;
        bool operator <(time_t t)
        {
            return comp(MtExpires(t)) < 0;
        }
        ;
};

#endif
