#ifndef __include_mthttp_h__
#define __include_mthttp_h__

#include <string.h>
#include "mtdebug.h"
#include "cnc_http.h"

#ifdef __cplusplus
#include "mterror.h"
#include "mtcookie.h"

#ifdef CRISP_ENABLE_HTTPS_SUPPORT
#include <openssl/ssl.h>
#ifndef OPENSSL_SYS_WIN32
#include <openssl/engine.h>
#endif
#endif

/*
 * BS:
 *----
 * These four classes are the one that the user has to define to perform a basic
 * http implementation.
 *
 */
class MtHttpService;
class MtHttpStream;
class MtHttpRequest;
class MtHttpResponse;

/* BS :
 *-----
 * Class for defining ports
 * Overloads the operator "=" to assign the port value and return the object
 *
 * Functions used :
 *----------------
 * MtHttpPort :
 *      Assigns the port number to the member 'port'
 * MtHttpMethod :
 *      Assigns the method used to the member 'member'
 * MtHttpContentLength :
 *      Assigns the length of the entity body after encoding to the ContentLength
 *      variable
 */

class MtHttpPort
{
private:
    int port;
public:
    /* BS: defining default ports for http and https */
    enum Ports
    {
        HTTP = 80, HTTPS = 443
    };
    MtHttpPort (Ports a = HTTP) :
        port (a)
    {
    }
    ;/* BS: Constructor which assigns
     * default value to the HTTP port
     */
    MtHttpPort operator = (Ports a)
    {
        port = a;
        return *this;
    }
    ;
    MtHttpPort operator = (int a)
    {
        port = a;
        return *this;
    }
    ;
    operator int ()
    {
        return port;
    }
    ;
    bool operator == (Ports a)
    {
        return port == a;
    }
    ;
};

/* BS:
 * Class for defining methods such as GET, POST, HEAD
 * Overloads the operator "=" to assign the method and return the object
 */
class MtHttpMethod
{
private:
    int method;
public:
    enum Methods
    {
        OPTIONS, HEAD, GET, POST, PUT, DELETE, TRACE, CONNECT /* BS: Added by BS*/
    };
    MtHttpMethod (Methods a = GET) :
        method (a)
    {
    }
    ;
    /*
     * BS:
     * Constructor which assigns
     * default method GET
     * Might be wrong as there is
     * no default method. Also other
     * methods are not defined in the
     * enum
     */
    MtHttpMethod operator = (Methods a)
    {
        method = a;
        return *this;
    }
    ;
    operator int ()
    {
        return method;
    }
    ;
    bool operator == (Methods a)
    {
        return method == a;
    }
    ;
};

typedef unsigned int MtHttpStatusCode;

/*
 * BS:
 * Class for defining the content length which indicates the size of the entity
 * body. Also indicates the type of encoding used.
 *
 */
class MtHttpContentLength
{
private:
    int contentLength;
public:
    enum ContentLength
    {
        Chunked = -2, Unknown = -1
    };
    /*
     * BS:
     * Constructor which returns 0 if
     * 0 is passed
     */
    MtHttpContentLength (int a = 0) :
        contentLength (a)
    {
    }
    ;
    /*
     * BS:
     * Operator overloading of = to return the contentlength and += to append.
     */
    MtHttpContentLength operator = (ContentLength a)
    {
        contentLength = a;
        return *this;
    }
    ;
    MtHttpContentLength operator = (int a)
    {
        contentLength = a;
        return *this;
    }
    ;
    MtHttpContentLength operator += (int a)
    {
        contentLength += a;
        return *this;
    }
    ;
    operator int ()
    {
        return contentLength;
    }
    ;
    /* BS:
     * Compares an integer with ContentLength.
     * Returns true if equal else false
     */
    bool operator == (ContentLength a)
    {
        return contentLength == a;
    }
    ;

	long long s64_contentLength;
};

/*
 * BS:
 *----
 * This class provides the option of providing the client whether to block the
 * cookie or not.
 *
 * Dependency:
 *------------
 * None
 */
class MtHttpOption
{
private:
    int option;
public:
    enum Option
    {
        NonBlocking = 0, Cookie
    };
    MtHttpOption (void) :
        option (0)
    {
    }
    ;
    MtHttpOption operator = (MtHttpOption o)
    {
        this->option = o.option;
        return *this;
    }
    ;
    MtHttpOption operator += (Option o)
    {
        this->option |= (1 << o);
        return *this;
    }
    ;
    MtHttpOption operator -= (Option o)
    {
        this->option &= ~(1 << o);
        return *this;
    }
    ;
    bool operator == (Option o)
    {
        return option & (1 << o);
    }
    ;
    bool operator != (Option o)
    {
        return !(option & (1 << o));
    }
    ;
};

/*
 * BS:
 *----
 * The main service which handles the process of getting the cookie information.
 * This class also contains a function which handles the option class and
 * enables or disables cookies based on the requirement.
 *
 * Dependency:
 *------------
 * MtHttpOption
 * MtCookie
 */
class MtHttpService
{
    friend class MtHttpStream;

private:
    MtHttpOption option;

    MtCookie cookie;

public:
    MtHttpService (void);
    virtual ~MtHttpService (void);

    void setOption (MtHttpOption::Option o, bool enable = true);

    int setCookie (MtURL &url, std::string &setcookie)
    {
        return this->cookie.setCookie (url, setcookie);
    }
    ;
    std::string getCookie (MtURL &url)
    {
        return this->cookie.getCookie (url);
    }
    ;

};

/*
 * BS:
 *----
 * Defines the request header format of the HTTP Client. Contains fields for url,
 * method, header, body and the length.
 *
 * Dependency:
 *------------
 * MtURL
 * MtHttpMethod
 * MtHttpStream
 */
class MtHttpRequest
{
    friend class MtHttpStream;

private:
    MtHttpStream *httpStream;

    MtURL url;
    MtURL proxyServer;
    MtHttpMethod method;
    std::string header;
    char *body;
    int length;
    std::string countryCode;

public:
    MtHttpRequest (MtHttpStream *httpStream);
    virtual ~MtHttpRequest (void);

    void clear (void);
    void setMethod (MtHttpMethod method);
    void setURL (const std::string &url);
    void setProxyServer (const std::string &proxyServer);
    void setBody (const char *post, int len);
    void addHeader (const std::string &name, const std::string &value);
    std::string getUrl (void);

    void setCountryCode(const std::string &countryCode);
    std::string getCountryCode(void);
    std::string getHostName (void);

private:
    void shutdown (void);
    std::string getScheme (void);
    std::string getPort (void);
    std::string toString (void);
};

/*
 * BS:
 *----
 * Defines the response field formats. This will interpret the response which the
 * client receives from the server and present it to the client in a usable form
 * which the client can use it for future actions.
 *
 * Dependency:
 *------------
 * MtHttpStream
 * MtHttpStatusCode
 * MtHttpContentLength
 *
 */
class MtHttpResponse
{
    friend class MtHttpStream;

private:
    MtHttpStream *httpStream;

    std::string header;
    std::string version;
    MtHttpStatusCode statusCode;
    std::string reasonPhrase;
    MtHttpContentLength contentLength;
    std::string allowedMethods;
    bool keepAlive;

public:

    cncHttpClientProtocol_t Protocol;
    MtHttpResponse (MtHttpStream *httpStream);
    virtual ~MtHttpResponse (void);

    std::string getVersion (void);
    MtHttpStatusCode getStatusCode (void);
    std::string getReasonPhrase (void);

    MtHttpContentLength getContentLength (void);
    std::string getHeader (const std::string &name, int index = 0);

    int readBody (char *part, int size);
    MtHttpContentLength getRealContentLength (void);
    cncHttpClientProtocol_t getProtocolName(void);

private:
    void shutdown (void);
    void addHeader (int c);
    bool isEndOfHeader (void);
    int parseStatusline (void);
    void parseHeader (void);
};


/*
 * BS:
 *----
 * The class which defines the stream. Consists of operations related to the
 * socket descriptor.
 *
 * Dependency:
 *------------
 * HttpRequest
 * HttpResponse
 * HttpService
 */
class MtHttpStream
{
    friend class MtHttpRequest;
    friend class MtHttpResponse;

private:
    MtHttpService *httpService;
    MtHttpRequest httpRequest;
    MtHttpResponse httpResponse;

    int sd; /* BS: Socket Descriptor */
    std::string hostName;
    bool keepAlive;
    struct addrinfo *addrinfo;
    MtHttpPort port;
    int redirectCount;

    struct
    {
        bool flag;
        /*
         * BS:
         * Contains information pertaining to SSL such as version number, cipher
         * settings, session specific data etc
         */
#ifdef CRISP_ENABLE_HTTPS_SUPPORT
        SSL_CTX *sslCtx;
        SSL *ssl;
        SSL_SESSION *session;
#endif
    } secure;

    struct
    {
        char buff[256];
        int ptr;
        int len;
    } buffer;

    int realContentLength;
    int sendIndex;

    /*
     * BS:
     *----
     * Defines the properties of a chunk used in chunk transfer encoding.
     */
    struct
    {
        std::string buff;
        int size;
    } chunk;

    MtError error;
    int sysErrNum;  // For backing up errno
    int retryCount; // Current retry count for connection

    /*
     * BS:
     *----
     * Defines values for the state of the request and response which can be
     * interpreted by the client for further action.
     */
    enum InternalState
    {
        Standby = 0,
        Start,
        Resolv,
        Socket,
        Connect,
        WaitConnected,
        SecureConnect,
        SendRequestHeader,
        SendRequestBody,
        RecvStatusline,
        RecvHeader,
        Redirect,
        RecvBody,
        RecvUnknownLength,
        RecvChunkSize,
        RecvChunkData,
        RecvChunkTrailer,
        Complete,
        __MaxInternalState,
    } internalState;

    static int
            (MtHttpStream::*connectingProcedure[MtHttpStream::__MaxInternalState])( void );

            MtHttpOption option;
            bool pipeLining;
            bool pipeLiningRequest;
        public :
            enum State
            {
                Ready = 0,
                Connecting,
                Loading,
                Completed,
                __MaxState,
            };

        private :
            int connectingProcess( void );
            int standbyProcedure( void );
            int startProcedure( void );
            int resolvProcedure( void );
            int socketProcedure( void );
            int connectProcedure( void );
            int waitConnectProcedure( void );
            int secureConnectProcedure( void );
            int sendRequestHeaderProcedure( void );
            int sendRequestBodyProcedure( void );
            int recvStatuslineProcedure( void );
            int recvHeaderProcedure( void );

            int redirectingProcedure( void );

            int loadingProcess( char *part, int size );
            int recvBodyProcedure( char *part, int size );
            int recvUnknownLengthProcedure( char *part, int size );
            int recvChunkSizeProcedure( char *part, int size );
            int recvChunkDataProcedure( char *part, int size );
            int recvChunkTrailerProcedure( char *part, int size );

            int completeProcedure( void );

            int writeSocket( const char *buff, int len );
            int readSocket( char *buff, int len );
            int readBuffer( char *buff, int len );
            int getcBuffer( void );
            int read( char *buff, int len );

        public :
            MtHttpStream( MtHttpService *httpService );
            virtual ~MtHttpStream( void );

            void setOption( MtHttpOption::Option o, bool enable = true );

            MtHttpRequest *getRequest( void );
            int sendRequest( void );
            int sendPipeLinedRequest(void);
            int getPipeLinedResponse(void);
            MtHttpStream::State getState( void );
            MtHttpResponse *getResponse( void );
            void shutdown( void );
            void abort( void );
            MtError setCookie( char *buff, int len );
            MtError getCookie( char *buff, int len );

        private :
            void socketClose( void );
            MtError setError( MtError error );
            MtError peekError( void );
        };

#endif

    /**********************************************************

     C Language Application Programming Interface

     **********************************************************/

#ifdef __cplusplus
extern "C"
{
#endif

    typedef char MtHttpServiceClass;
    typedef short MtHttpStreamClass;
    typedef int MtHttpRequestClass;
    typedef long MtHttpResponseClass;


#define MtHttpOption_NonBlocking     0
#define MtHttpOption_Cookie          1

#define MtHttpMethod_GET              0
#define MtHttpMethod_POST             1
#define MtHttpMethod_HEAD             2

#define MtHttpContentLength_Chunked   -2
#define MtHttpContentLength_Unknown   -1

#define MtHttpStream_Ready        0
#define MtHttpStream_Connecting   1
#define MtHttpStream_Loading      2
#define MtHttpStream_Completed    3

#define MtError_Ok                    0
#define MtError_Error                 -1000
#define MtError_InvalidState          (MtError_Error-1)
#define MtError_UnknownScheme         (MtError_Error-2)
#define MtError_CantConnected         (MtError_Error-3)
#define MtError_Handshake             (MtError_Error-4)
#define MtError_Disconnected          (MtError_Error-5)
#define MtError_ConnectionClosed      (MtError_Error-6)
#define MtError_NoResource            (MtError_Error-7)
#define MtError_LoadingCompleted      (MtError_Error-8)
#define MtError_RedirectCountExceeded (MtError_Error-9)



    /*---------------------------------------------------------

     MtHttpService

     ---------------------------------------------------------*/

    extern MtHttpServiceClass *MtHttpService_New (void);
    extern void MtHttpService_Delete (MtHttpServiceClass *instance);
    extern void MtHttpService_SetOption (MtHttpServiceClass *instance,
            int option, int enable);

    /*---------------------------------------------------------

     MtHttpStream

     ---------------------------------------------------------*/

    extern MtHttpStreamClass
            *MtHttpStream_New (MtHttpServiceClass *httpService);
    extern void MtHttpStream_Delete (MtHttpStreamClass *instance);
    extern void MtHttpStream_SetOption (MtHttpStreamClass *instance,
            int option, int enable);
    extern MtHttpRequestClass *MtHttpStream_GetRequest (
            MtHttpStreamClass *instance);
    extern int MtHttpStream_SendRequest (MtHttpStreamClass *instance);
    extern int MtHttpStream_GetState (MtHttpStreamClass *instance);
    extern MtHttpResponseClass *MtHttpStream_GetResponse (
            MtHttpStreamClass *instance);
    extern int MtHttpStream_SendPipeLinedRequest(MtHttpStreamClass *instance);

    extern int MtHttpStream_GetPipeLinedResponse(MtHttpStreamClass *instance);
    
    extern int MtHttpStream_Shutdown (MtHttpStreamClass *instance);
    extern int MtHttpStream_Abort(MtHttpStreamClass *instance);

    extern int MtHttpStream_GetCookie(MtHttpStreamClass *instance, char *p_ch_outBuf, int s32_outBufLenAvailable);
    extern int MtHttpStream_SetCookie(MtHttpStreamClass *instance, char *p_ch_outBuf, int s32_outBufLenAvailable);
    /*---------------------------------------------------------

     MtHttpRequest

     ---------------------------------------------------------*/

    extern int MtHttpRequest_SetMethod (MtHttpRequestClass *instance,
            int method);
    extern int MtHttpRequest_SetURL (MtHttpRequestClass *instance,
            const char *url);
    extern int MtHttpRequest_SetProxyServer(MtHttpRequestClass *instance,
            const char *proxyServer);
    extern int MtHttpRequest_SetBody (MtHttpRequestClass *instance,
            const char *post, int len);
    extern int MtHttpRequest_AddHeader (MtHttpRequestClass *instance,
            const char *name, const char *value);
    extern int MtHttpRequest_SetCountryCode (MtHttpRequestClass *instance,
            const char *countryCode);

    int MtHttpRequest_Clear(MtHttpRequestClass *instance);

    /*---------------------------------------------------------

     MtHttpResponse

     ---------------------------------------------------------*/

    extern cncHttpClientProtocol_t MtHttpResponse_GetProtocolName(MtHttpResponseClass *instance);
    extern const char
            *MtHttpResponse_GetVersion (MtHttpResponseClass *instance);
    extern int MtHttpResponse_GetStatusCode (MtHttpResponseClass *instance);
    extern const char *MtHttpResponse_GetReasonPhrase (
            MtHttpResponseClass *instance);
    extern int MtHttpResponse_GetContentLength (MtHttpResponseClass *instance);
    extern long long MtHttpResponse_GetContentLengthExt (MtHttpResponseClass *instance);
    extern const char *MtHttpResponse_GetHeader (MtHttpResponseClass *instance,
            const char *name, int index);
    extern int MtHttpResponse_GetUrl(MtHttpRequestClass *instance, char *p_ch_outBuf, int s32_outBufLenAvailable);
	extern int MtHttpResponse_GetHeaderExt(MtHttpResponseClass *instance, const char *name, int index, char *p_ch_outBuf, int s32_outBufLenAvailable);
    extern int MtHttpRequest_GetHostName(MtHttpRequestClass *instance, char *p_ch_outBuf, int s32_outBufLenAvailable);
    extern int MtHttpResponse_ReadBody (MtHttpResponseClass *instance,
            char *part, int size);
    extern int MtHttpResponse_GetRealContentLength (
            MtHttpResponseClass *instance);

#ifdef __cplusplus
}
#endif

#endif
