/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
*/

/**
 * @file cnc_input.h
 * @ingroup core
 * The CNCINPUT module is use to manage the input devices
 */
#ifndef __CNC_INPUT_H__
#define __CNC_INPUT_H__

#include "cnc_input_keys.h"

#include "cnc_common_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    CNC_INPUT_DEVICE_TYPE_NONE = 0x00,
    CNC_INPUT_DEVICE_TYPE_LINUX = 0x01, // Also adds IR remote support
    CNC_INPUT_DEVICE_TYPE_ANYMOTE = 0x02,
    CNC_INPUT_DEVICE_TYPE_CEC = 0x04,
    CNC_INPUT_DEVICE_TYPE_XBMC_EVENTSERVER = 0x08,
    CNC_INPUT_DEVICE_TYPE_ALL = 0xFF
} cncInputDeviceType_t;

typedef enum {
    CNC_INPUT_KEY_UNKNOWN,
    CNC_INPUT_KEY_DOWN,
    CNC_INPUT_KEY_UP,
    CNC_INPUT_KEY_REPEAT
} cncInputKeyEvent_t;

typedef enum {
    CNC_INPUT_MOD_NONE = 0x0000,
    CNC_INPUT_MOD_SHIFT = 0x0001,
    CNC_INPUT_MOD_CTRL = 0x0002,
    CNC_INPUT_MOD_ALT = 0x0004,
    CNC_INPUT_MOD_GUI = 0x0010,
    CNC_INPUT_MOD_NUM = 0x0020,
    CNC_INPUT_MOD_CAPS = 0x0040,
    CNC_INPUT_MOD_ALL = 0xFFFF,
} cncInputMod_t;

typedef struct {
    cncInputKeyEvent_t event; // Type of event
    cncInputKeyCode_t keycode;
    cncInputMod_t modifiers;

    int device;   // Needed?
} cncInputKey_t;

typedef struct {
    int x;  /// [0..width]
    int y;  /// [0..height]
    int xdelta;  /// Delta since last callback
    int ydelta;  /// Delta since last callback

    int device; // Needed? to diferenciate several pointers
} cncInputMouse_t;


// The comment of each enum indicates wich is the message that is
// passed in the callback
typedef enum {
    CNC_INPUT_EVENT_NEW_DEVICE, // the device name ("/dev/input/input2")
    CNC_INPUT_EVENT_IP_PAIRED, // No param
    CNC_INPUT_EVENT_IP_PAIRED_CANCEL, // No param
    CNC_INPUT_EVENT_IP_CONNECTED, // The ip remote device name
    CNC_INPUT_EVENT_IP_DISCONNECTED, // No param
    CNC_INPUT_EVENT_IP_SECRET, // The secret to show the user.
                               // Will cancel the pairing if it
                               // gets a CNC_BOOL_FALSE when notified
                               // the event
    CNC_INPUT_EVENT_IP_UPGRADE_IMAGE, // Filename
    CNC_INPUT_EVENT_TEXT, // Some devices does not send keystrokes, but a string
                          // e.g google-tv-remote text input
} cncInputEventType_t;

typedef enum {
    CNC_INPUT_EVENT_TO_INPUT_UPGRADE_MSG,
} cncInputEventToInput_t;

typedef struct {
    cncInputEventType_t event;
    const char *description;
} cncInputEvent_t;

typedef enum {
    CNC_INPUT_TYPE_KEY = 0x01,
    CNC_INPUT_TYPE_MOUSE = 0x02,
    CNC_INPUT_TYPE_EVENT = 0x04,
} cncInputType_t;

typedef struct {
    cncInputType_t type;
    union {
        cncInputKey_t key;
        cncInputMouse_t mouse;
        cncInputEvent_t event;
    };
} cncInputData_t;

typedef enum {
     CNC_INPUT_PARAM_SCREEN_X, /// Width of the screen, used to clamp/scale mouse positions
     CNC_INPUT_PARAM_SCREEN_Y, /// Height of the screen, used to clamp/scale mouse positions
     CNC_INPUT_PARAM_ANYMOTE_NAME, /// Name used for the device in anymote
     CNC_INPUT_PARAM_CEC_NAME, /// Name used for the device in CEC
} cncInputParam_t;

typedef CNC_BOOL (*cncInputCallback_t)(const cncInputData_t *data);


/** Starts the input system
 * @param enabledMask Enabled input systems
 */
CNC_BOOL CNC_Input_Init(cncInputDeviceType_t enabledMask);

/** Register a key callback function
 * This function can be called several times with different callback
 * functions. Each function only will be called once for each key
 * event. If the functions is called twice with the same handler
 * it will update the mask for this handler.
 * @param handler callback function
 * @param typeMask which events subscribe for
 */
void CNC_Input_RegisterHandler(cncInputCallback_t handler,
                               cncInputType_t typeMask);

/** Remove a key callback function
 * Remove a key callback function from the callback list. If it is not
 * in the list it does nothing.
 * @param handler Handler to be removed
 */
void CNC_Input_RemoveHandler(cncInputCallback_t handler);

/**
 * Sends to the input modules an even, e.g., upgrade msgs to anymote client
 * @param event event to send
 * @param eventText text to send
 */
void CNC_Input_NotifyToInput(cncInputEventToInput_t event,
                             const char *eventText);

/** 
 * Configure an integer parameter of an input module
 * @param module module to configure, it could be CNC_INPUT_DEVICE_TYPE_NONE if
 *               configuring a generic parameter, as CNC_INPUT_PARAM_SCREEN_X
 * @param param Parameter to set
 * @param value Integer value to set
 */
void CNC_Input_ParamInt(cncInputDeviceType_t module, cncInputParam_t param,
                        int value);

/** 
 * Configure a string parameter of an input module
 * @param module module to configure, it could be CNC_INPUT_DEVICE_TYPE_NONE if
 *               configuring a generic parameter
 * @param param Parameter to set
 * @param value String value to set
 */
void CNC_Input_ParamStr(cncInputDeviceType_t module, cncInputParam_t param,
                        const char *value);

/** 
 * Get the ascii equivalent of the key event. It uses the US keymapping.
 * @param keyEvent Event to transform
 * @return The ascii equivalent of the keyevent, 0 if it cant be transformed
 */
char CNC_Input_GetCharFromEvent(const cncInputKey_t *keyEvent);

#ifdef __cplusplus
}
#endif

#endif  // __CNC_INPUT_H__
