/** 
 * @file key_extract.h
 * @brief This file contains headers, structures and function declaration  used for key extraction process
 * @author Cavium Networks Inc.,
 * @version 0.1
 * @date 2012-02-08
 */

#ifndef __KEY_EXTRACT_H__
#define __KEY_EXTRACT_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"
#include "cssec.h"

/* for blobName field of Blob_t */
#define ELLIPTIC    (0)
#define DTCP_KEY    (1)
#define HDCP_KEY    (2)
#define MARLIN_KEY  (3)
#define HDCP_TX_KEY (4)
#define PKCS12_KEY  (5)

typedef struct _Blob_t {

	unsigned int blobName; 
	unsigned int length;
	unsigned int offset;
} Blob_t;

typedef struct {
    unsigned char *buf;
    unsigned int  buf_len;
} buf_arg_t;

#define BLOB_FILE	            "/proc/blob"
#define BLOB_SIZE		        0x00100000 /* 4 Kbyte*/
#define CONTENT_KEY_SIZE        16

typedef enum _KEY_EXTRACT_ERROR_CODE
{
    KEY_EXTRACT_SUCCESS                         = 0,  ///< Key Extraction and Decryption Completed
    KEY_EXTRACT_ERROR                           = -1, ///< Error on Key Extraction or on Decryption process
    KEY_EXTRACT_CRYPTO_INIT_FAILED              = -2, ///< Initialization of Crypto operation failed
    KEY_EXTRACT_CRYPTO_UNINIT_FAILED            = -3, ///< Termination of Crypto operation failed
    KEY_EXTRACT_INVALID_CHANNEL_NO              = -4, ///< Invalid Crypto Channel used for Crypto operation
    KEY_EXTRACT_CRYPTO_CHANNEL_OPEN_FAILED      = -5, ///< Opening Crypto Channel Failed
    KEY_EXTRACT_CRYPTO_CHANNEL_CLOSE_FAILED     = -6, ///< Closing Crypto Channel Failed
    KEY_EXTRACT_SETTING_CRYPTO_MODE_FAILED      = -7, ///< Setting Crypto Mode Failed
    KEY_EXTRACT_SETTING_CONTENT_KEY_FAILED      = -8, ///< Setting Content Key for Cryto operation failed
    KEY_EXTRACT_SETTING_HOST_KEY_FAILED         = -9, ///< Setting Host Key for Crypto process failed
    KEY_EXTRACT_BLOB_READ_FAILED                = -10,///< Reading of Blob image from memory failed
    KEY_EXTRACT_BLOB_COUNT_EXCEED               = -11 ///< List of Blob image exceeded Maximum blob image count
} KEY_EXTRACT_ERROR_CODE;

/** 
 * \fn KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_dtcp_dtla_constant(unsigned char *dtla, int *size, unsigned int crypto_chl)
 * \brief To extract and decrypt the DTCP DTLA constant (key) from Blob memory area which is placed by Secure Boot process. 
 * \param[in] dtla        - Pointer to the DTLA constant place where the extracted and decrypted content need to be placed for the use of DTCP/DLNA application
 * \param[in] size        - Size of the DTLA constant read from Blob area
 * \param[in] crypto_chl  - Crypto Channel to be used to perform decryption
 *
 * \returns KEY_EXTRACT_ERROR_CODE. Upon successfull extraction and decrytion of key returns KEY_EXTRACT_SUCCESS(0), otherwise returns negative value. 
 */
KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_dtcp_dtla_constant(unsigned char *dtla, int *size, unsigned int crypto_chl);

/** 
 * \fn KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_hdcp_key(unsigned char *hdcp_key, int *size, unsigned int crypto_chl)
 * \brief  To extract and decrypt the HDCP key from Blob memory area which is placed by Secure Boot process.
 * \param[in] hdcp_key    - Pointer to the HDCP key place where the extracted and decrypted key need to be placed for the use of HDCP application

 * \param[in] size        - Size of the HDCP key read from Blob area
 * \param[in] crypto_chl  - Crypto Channel to be used to perform decryption
 *
 * \returns KEY_EXTRACT_ERROR_CODE.  Upon successfull extraction and decrytion of key returns KEY_EXTRACT_SUCCESS(0), otherwise returns negative value.
 */
KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_hdcp_key(unsigned char *hdcp_key, int *size, unsigned int crypto_chl);

KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_hdcp_tx_key(unsigned char *hdcp_key, int *size, unsigned int crypto_chl);

KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_marlin_key(unsigned char *marlin_key, int *size, unsigned int crypto_chl);

KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_pkcs12_key(unsigned char *pkcs12_key, int *size, unsigned int crypto_chl);

KEY_EXTRACT_ERROR_CODE KEY_EXTRACT_dtla_key(unsigned char *dtcp_key, int *size, unsigned int crypto_chl);

int KEY_EXTRACT_CountSecureBlobs(void);

#ifdef __cplusplus
}
#endif

#endif //~ __KEY_EXTRACT_H__
