/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_mediaformat.h
 * @ingroup common
 * Data structures for information and configuration of media pipeline control
 *
 * Defines the following:
 * @li Data structures for media format info
 * @li Data structures for playback status info
 */

#ifndef _CNC_MEDIAFORMAT_H_
#define _CNC_MEDIAFORMAT_H_

#ifdef __cplusplus
extern "C" {
#endif

/* =============================== System Includes ================================= */
// None

/* =============================== Includes ================================= */
#include "cnc_av.h"
#include "cnc_common_typedef.h"
#include "cnc_common_error.h"
#include "cnc_subtitle.h"

/* ================================ Macros =================================== */
#define CNC_MEDIA_VID_MAX_STREAM_NUM                   5  //!< the maxium number of video streams in a file
#define CNC_MEDIA_AUD_MAX_STREAM_NUM                   15 //!< the maxium number of audio streams in a file
#define CNC_MEDIA_SUB_MAX_STREAM_NUM                   16 //!< the maxium number of subtitle streams in a file
#define CNC_MEDIA_DATA_MAX_STREAM_NUM                  16 //!< The maxium number of data streams in a file 
#define CNC_MEDIA_MAX_LANG_CODE_SIZE                   (4) 
#define CNC_MEDIA_MAX_ES                               10  

/* =============================== typedef Info ============================== */
typedef enum _cncMediaClass_t
{
    CNC_MEDIA_CLASS_NONE = 0,    /** None */
    CNC_MEDIA_CLASS_IMAGE,       /** Image Class */
    CNC_MEDIA_CLASS_AUDIO,       /** Audio Class */
    CNC_MEDIA_CLASS_VIDEO,       /** Video only or both Audio/Video */
} cncMediaClass_t;

/**
 * @enum    cncMediaFormat_t
 * @brief   Definition of various media formats
 *          This will be used to identify the format of stream (say file) in general and also the individual elementary streams
 */
typedef enum _cncMediaFormat_t
{
    CNC_MEDIA_FORMAT_NONE = 0,

    /** Audio */
    CNC_MEDIA_FORMAT_AUDIO_ATRAC3,           /**< ATRAC3 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_ATRAC3_PLUS,      /**< ATRAC3 Plus Audio format */
    CNC_MEDIA_FORMAT_AUDIO_G711,             /**< G711 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_G722,             /**< G722 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_G723,             /**< G723 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_G726,             /**< G726 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_G729,             /**< G729 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_AMR,              /**< AMR Audio format */
    CNC_MEDIA_FORMAT_AUDIO_AC3,              /**< AC3 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_FLAC,             /**< FLAC Audio format */
    CNC_MEDIA_FORMAT_AUDIO_DD_PLUS,          /**< DD Plus Audio format */
    CNC_MEDIA_FORMAT_AUDIO_MLP,              /**< MLP Audio format */
    CNC_MEDIA_FORMAT_AUDIO_DTS,              /**< DTS Audio format */
    CNC_MEDIA_FORMAT_AUDIO_DTS_HD,           /**< DTS Hd Audio format */
    CNC_MEDIA_FORMAT_AUDIO_MP1,              /**< MP1 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_MP2,              /**< MP2 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_MP3,              /**< MP3 Audio format */
    CNC_MEDIA_FORMAT_AUDIO_MPEG2_AAC,        /**< MPEG2 AAC Audio format */
    CNC_MEDIA_FORMAT_AUDIO_AAC_LATM,         /**< MPEG4 HE AAC Audio format */
    CNC_MEDIA_FORMAT_AUDIO_WMA,              /**< WMA Audio format */
    CNC_MEDIA_FORMAT_AUDIO_WMAPRO,           /**< WMA-Pro Audio format */
    CNC_MEDIA_FORMAT_AUDIO_WMA_LOSSLESS,     /**< WMA Lossless Audio format */
    CNC_MEDIA_FORMAT_AUDIO_OGG_VORBIS,       /**< OGG Vorbis Audio format */
    CNC_MEDIA_FORMAT_AUDIO_RA,               /**< Real Audio format */
    CNC_MEDIA_FORMAT_AUDIO_REAL_COOK,        /**< Real Cook Audio format */
    CNC_MEDIA_FORMAT_AUDIO_REAL_SIPR,        /**< Real SIPR Audio format */
    CNC_MEDIA_FORMAT_AUDIO_APE,              /**< APE Audio format */
    CNC_MEDIA_FORMAT_AUDIO_LPCM,             /**< PCM Audio format */
    CNC_MEDIA_FORMAT_AUDIO_AIFF,             /**< AIFF Audio format */
    CNC_MEDIA_FORMAT_AUDIO_AU,               /**< AU Audio format */
    CNC_MEDIA_FORMAT_AUDIO_WAV,              /**< WAV Audio format */
    CNC_MEDIA_FORMAT_AUDIO_ALAW,             /**< ALAW Audio format */
    CNC_MEDIA_FORMAT_AUDIO_MULAW,            /**< MULAW Audio format */
    CNC_MEDIA_FORMAT_AUDIO_TRUEHD,           /**< TRUE HD Audio format */

    /** Video */
    CNC_MEDIA_FORMAT_VIDEO_MPEG1_PART2,      /**< MPEG1 Part-2 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_MPEG2_PART2,      /**< MPEG2 Part-2 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_MPEG4_PART2,      /**< MPEG4 Part-2 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_H261,             /**< H261 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_H263,             /**< H263 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_H264,             /**< H264 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_WMV,              /**< WMV Video Format */
    CNC_MEDIA_FORMAT_VIDEO_VC1,              /**< VC1 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_AVS,              /**< AVS Video Format */
    CNC_MEDIA_FORMAT_VIDEO_RV,               /**< RV Video Format */
    CNC_MEDIA_FORMAT_VIDEO_MJPEG,            /**< MJPEG Video Format */
    CNC_MEDIA_FORMAT_VIDEO_VP8,              /**< VP8 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_THO,              /**< Theora Video Format */
    CNC_MEDIA_FORMAT_VIDEO_VP9,              /**< VP9 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_MSMPEG4V1,        /**< Microsoft MPEG-4 Version-1 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_MSMPEG4V2,        /**< Microsoft MPEG-4 Version-2 Video Format */
    CNC_MEDIA_FORMAT_VIDEO_MSMPEG4V3,        /**< Microsoft MPEG-4 Version-3 Video Format */

    /** Container */
    CNC_MEDIA_FORMAT_CONTAINER_3GP,          /**< 3GP Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_3G2,          /**< 3G2 Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_ASF,          /**< ASF Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_AVI,          /**< AVI Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_FLV,          /**< FLV Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_MPEG_PS,      /**< MPEG PS Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_MPEG_TS,      /**< MPEG TS Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_MP4,          /**< MP4 Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_DMF,          /**< DMF Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_MKV,          /**< MKV Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_MXF,          /**< MXF Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_MOV,          /**< MOV Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_OGG,          /**< OGG Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_RM,           /**< RM Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_VOB,          /**< VOB Container Format */
    CNC_MEDIA_FORMAT_CONTAINER_WEBM,         /**< WEBM Container Format */

    /** Image */
    CNC_MEDIA_FORMAT_IMAGE_JPG,             /**< JPEG Image Format */
    CNC_MEDIA_FORMAT_IMAGE_PNG,             /**< PNG Image Format */
    CNC_MEDIA_FORMAT_IMAGE_BMP,             /**< BMP Image Format */
    CNC_MEDIA_FORMAT_IMAGE_GIF              /**< GIF Image Format */
} cncMediaFormat_t;

/**
 * @struct
 * @enum    cncAudioSampleFormat_t
 * @brief   Definition of different Audio PCM Sample Formats
 */
typedef enum _cncAudioSampleFormat_t
{
    CNC_AUDIO_SAMPLE_FMT_NONE = -1,         /**< Invalid Sample Format */
    CNC_AUDIO_SAMPLE_FMT_U8,                /**< Un Signed 8 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_S8,                /**< Signed 8 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_U16,               /**< Un Signed 16 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_S16,               /**< Signed 16 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_U24,               /**< Un Signed 24 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_S24,               /**< Signed 24 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_U32,               /**< un Signed 32 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_S32,               /**< Signed 32 bit Sample Format */
    CNC_AUDIO_SAMPLE_FMT_FLT,               /**< Float Sample Format */
    CNC_AUDIO_SAMPLE_FMT_DBL,               /**< Double Sample Format */
    CNC_AUDIO_SAMPLE_FMT_NB                 /**< Number of sample formats. DO NOT USE if linking dynamically */
} cncAudioSampleFormat_t;

/* TODO: Move Audio MetaData structure to proper place */
/**
 * @struct cncMediaMetaData_t
 * @brief  Media Meta Data Info
 * @note
 */
typedef struct _cncMediaMetaData_t
{
    CNC_CHAR                        albumArtUrl[512];  /**< Album Art URL - Provided by DMC/DMS/external players */
    CNC_CHAR                        title[128];        /**< Stream Title */
    CNC_CHAR                        author[128];       /**< Stream Author */
    CNC_CHAR                        copyright[128];    /**< Stream Copyright Info */
    CNC_CHAR                        album[128];        /**< Stream Album */
    CNC_CHAR                        genre[32];         /**< Stream genre */
    CNC_CHAR                        artist[128];       /**< Stream Artist */
    CNC_SINT32                      year;              /**< ID3 year, 0 if none */
    CNC_SINT32                      track;             /**< track number, 0 if none */
} cncMediaMetaData_t;

/**
 * @struct cncAudioFormatInfo_t
 * @brief  Audio Format Information like Format Type, BitRate, Sample Rate, NoOfChannels etc 
 * @see cncMediaFormat_t 
 * @see cncAudioSampleFormat_t 
 */
typedef struct _cncAudioFormatInfo_t
{
    cncMediaFormat_t                format;            /**< Format Type Like MP3, aac, ac3 ...*/
    CNC_SINT32                      bitRate;           /**< stream bitrate */
    CNC_SINT32                      sampleRate;        /**< Sampling Rate */
    CNC_SINT32                      numOfChannels;     /**< No Of Channels */
    CNC_SINT32                      bigEndian;         /**< Endian Type */
    cncAudioSampleFormat_t          audSampleFormat;   /**< Sample Format */
    struct
    {
        cncMediaFormat_t            albumArtImageType; /**< Album Art Image Type like JPEG/PNG/BMP.. */
        CNC_CHAR                   *albumArtBuf;       /**< Album Art Image buffer */
        CNC_SINT32                  albumArtSize;      /**< Album Art Image Size */
    } albumArtInfo;
} cncAudioFormatInfo_t;

/**
 * @struct cncVideoFormatInfo_t
 * @brief  Contains Video Format Information, Like, Format type, h264, mpeg4 ..., BitRate, Video Frame Width, Height..etc
 * @see    cncMediaFormat_t
 */
typedef struct _cncVideoFormatInfo_t
{
    cncMediaFormat_t                format;         /**< Format Type h264, mpeg4 ... */
    CNC_SINT32                      bitRate;        /**< Stream bitrate */
    CNC_FLOAT32                     frameRate;      /**< Frame Rate */
    CNC_SINT32                      width;          /**< Frame Width */
    CNC_SINT32                      height;         /**< Frame Height */
} cncVideoFormatInfo_t;

/**
 * @struct  cncImageFormatInfo_t
 * @brief   Contains Image Format Information, Like, type JPEG/PNG.., Image Width & Height
 * @see     cncMediaFormat_t 
 */
typedef struct _cncImageFormatInfo_t
{
    cncMediaFormat_t                format;         /**< Format Type, Like JPEG/PNG/BMP ..  */
    CNC_SINT32                      width;          /**< Image Width */
    CNC_SINT32                      height;         /**< Image Height */
} cncImageFormatInfo_t;

/**
 * @struct  cncMediaFormatDetail_t
 * @brief   Contains Media Fomrat Details, Audio/Video/Image details
 * @see     cncAudioFormatInfo_t
 * @see     cncVideoFormatInfo_t 
 * @see     cncImageFormatInfo_t
 */
typedef struct _cncMediaFormatDetail_t
{
    cncAudioFormatInfo_t            audioInfo;            /**< Audio Format Info */
    cncVideoFormatInfo_t            videoInfo;            /**< Video Format Info */
    cncImageFormatInfo_t            imageInfo;            /**< Image Format Info */
    CNC_SINT32                      rawPacketSize;        /**< Container packet size (Ex: Transport Stream - 188/192/204) */
    CNC_BOOL                        isTimeStampedPacket;  /**< If 'TRUE' each packet has a valid time stamp (Ex: Time Stamped Transport Stream) */
} cncMediaFormatDetail_t;

/**
 * @struct   cncMediaFormatInfo_t
 * @brief    Contains Media Format Information, Like, Media Class, Format Type, Duration, bitrate Etc
 * @see      cncMediaClass_t
 * @see      cncMediaFormat_t
 * @see      cncMediaFormatDetail_t
 * @see      cncMediaMetaData_t
 */
typedef struct _cncMediaFormatInfo_t
{
    cncMediaClass_t                 mediaClass;     /**< Media Class */
    cncMediaFormat_t                format;         /**< Format Type, Like jpeg, png, mp3, aac, mpeg4, h264, ts, mp4.. etc*/
    cncMediaFormatDetail_t          formatDetail;   /**< Audio/Video/Image Format detailed information */
    cncMediaMetaData_t              metaData;       /**< MetaData Info like, title, Genre, Album ..etc */
    CNC_SINT32                      durationInMsec; /**< Stream Duration in Milli Sec */
    CNC_SINT32                      bitRate;        /**< Stream Bitrate */
    CNC_SINT64                      size;           /**< Stream Size */
} cncMediaFormatInfo_t;

/**
 * @enum    cncPlaybackState_t
 * @brief   Defines different states of Media Playback
 */
typedef enum _cncPlaybackState_t
{
    CNC_PLAYBACK_STATUS_IDLE = 0,            /**< Playback is stopped or not started yet */
    CNC_PLAYBACK_STATUS_STARTING_PLAYBACK,   /**< Media Player accepted the start request and will start processing request. Or in simple terms LOADING */
    CNC_PLAYBACK_STATUS_PREPARED,            /**< Completed the probing process of given URI */
    CNC_PLAYBACK_STATUS_PLAYBACK_STARTED,    /**< START request successfully processed. AV/Image will be getting rendered if prebuffering is off */
    CNC_PLAYBACK_STATUS_PLAYING,             /**< Media (AV/Image) is getting played (rendered). Check cncTrickPlayMode_t for current trick mode (Play/Pause/FF/FR/SF/SR etc) */
    CNC_PLAYBACK_STATUS_PLAYBACK_SUSPENDED,  /**< Playback suspended and buffers are flushed */
    CNC_PLAYBACK_STATUS_STOPPING,            /**< STOP requested accepted. Starting to process STOP request. */
    CNC_PLAYBACK_STATUS_NO_DATA,             /**< Not able to receive data (Ex: network timeout) */
    CNC_PLAYBACK_STATUS_END_OF_STREAM,       /**< Playback reached end of Stream. Not applicable for LIVE */
    CNC_PLAYBACK_STATUS_START_OF_STREAM,     /**< Playback reached start of stream. Not applicable for LIVE */
    CNC_PLAYBACK_STATUS_PRE_BUFFERING,       /**< Buffering till threshold. Playback (rendering) will be on hold till buffering reaches 100%. Or in simple terms BUFFERING */
    CNC_PLAYBACK_STATUS_LOW_SIGNAL,          /**< Signal strength not good. Ex: Packet loss in UDP */
    CNC_PLAYBACK_STATUS_SIGNAL_RECOVERED     /**< Signal strength became good from bad */
} cncPlaybackState_t;

/**
 * @enum   cncTrickPlayMode_t
 * @brief  Trick play Mode, Like, PAUSE/FastForward/FastReverse..etc
 */
typedef enum _cncTrickPlayMode_t
{
    CNC_PLAYBACK_TRICK_MODE_NONE = 0,             /**< Invalid Trick Mode */
    CNC_PLAYBACK_TRICK_MODE_PLAY_NORMAL,          /**< Play Normal, Like from PAUSE/FF/FR.. etc */
    CNC_PLAYBACK_TRICK_MODE_PAUSE,                /**< Pause */
    CNC_PLAYBACK_TRICK_MODE_UNPAUSE,              /**< UnPause previous PAUSE Trick Mode */
    CNC_PLAYBACK_TRICK_MODE_FAST_FORWARD,         /**< Fast Forward */
    CNC_PLAYBACK_TRICK_MODE_FAST_REVERSE,         /**< Fast Reverse */
    CNC_PLAYBACK_TRICK_MODE_SLOW_FORWARD,         /**< Slow Forward */
    CNC_PLAYBACK_TRICK_MODE_SLOW_REVERSE,         /**< Slow Reverse */
    CNC_PLAYBACK_TRICK_MODE_FRAME_FORWARD,        /**< Frame Forward */
    CNC_PLAYBACK_TRICK_MODE_FRAME_REVERSE,        /**< Frame Reverse  */
    CNC_PLAYBACK_TRICK_MODE_JUMP                  /**< JUMP */
} cncTrickPlayMode_t;

/**
 * @enum   cncTrickJumpMode_t
 * @brief  Trick Jump Mode, Like Byte Based Jump, Time Based Jump, Catch Live
 */
typedef enum _cncTrickJumpMode_t
{
    CNC_TRICK_JUMP_MODE_NONE = 0,     /**< Nop */
    CNC_TRICK_JUMP_MODE_BYTE,         /**< Jump to specified offset based on Byte Position */
    CNC_TRICK_JUMP_MODE_TIME,         /**< Jump to specified offset based on Time Position */
    CNC_TRICK_JUMP_MODE_CATCH_LIVE    /**< Catch live from TimeShift Playback */
} cncTrickJumpMode_t;

/**
 * @enum  cncTrickFramesMode_t
 * @brief Mode of data given to CNC-Decoder
 */
typedef enum _cncTrickFramesMode_t
{
    CNC_TRICK_FRAME_MODE_NONE = 0,               /**< Media Player will decide the mode suitable for requested stream */
    CNC_TRICK_FRAME_MODE_ONLY_INTRA,             /**< Only I-Frames */
    CNC_TRICK_FRAME_MODE_IBP_NORMAL,             /**< IBP Frames of original stream. Media Player will do decoding at requested speed (Ex: 2x) */
    CNC_TRICK_FRAME_MODE_IBP_NORMAL_WITH_AUDIO,  /**< IBP Frames of original stream with Audio. Media Player will do decoding at requested speed (Ex: 1.5x) */
    CNC_TRICK_FRAME_MODE_IBP_SCALED,             /**< IBP Frames of separate stream for requested trickmode (Ex: 'test.5x.ts', 'test.-5x.ts' etc*/
    CNC_TRICK_FRAME_MODE_SEEK_AND_PLAY           /**< Continuous "Seek and Play". This can be done using Time Seek or Byte Seek internally by media player. */
}cncTrickFramesMode_t;

/**
 * @struct cncTrickInfo_t
 * @brief Contains Trick information, Like, Trick Mode, Spped, Jump Mode, Offset..etc
 * @see cncTrickPlayMode_t
 * @see cncTrickJumpMode_t
 * @see cncTrickFramesMode_t
 */
typedef struct _cncTrickInfo_t
{
    cncTrickPlayMode_t              trickMode;                   /**< Trick Mode */
    CNC_FLOAT32                     trickSpeed;                  /**< Trick Speed */
    cncTrickJumpMode_t              jumpMode;                    /**< Jump Mode */
    CNC_SINT64                      jumpOffSet;                  /**< Bytes or MilliSeconds based on jump mode */
    cncTrickFramesMode_t            frameMode;                   /**< Display Frame Mode Like I-Frame, IBP Frame...etc */
    CNC_BOOL                        b_retainModeAfterSeek;       /**< Flag to Preserve Previous trick mode after current trick operation Play -> PAUSE -> JUMP => should be in PAUSE */
} cncTrickInfo_t;

/**
 * @struct  cncPlaybackStatus_t
 * @brief   Contains Media Playback Status Information, Like, Current Playback Time, playing in TimeShift Mode, Trick Info..etc
 * @see     cncPlaybackState_t
 * @see     cncTrickInfo_t
 * @see     E_cncErrCode_t
 */
typedef struct _cncPlaybackStatus_t
{
    cncPlaybackState_t              playbackStatus;              /**< Playback Status */
    CNC_SINT32                      curPlayTimeInMilliSec;       /**< Current Playback time in Milli Secs */
    CNC_SINT32                      timeToLiveInMilliSec;        /**< Lag Time from current playback position wrt Live Data */
    CNC_BOOL                        playingInTimeShiftMode;      /**< Flag to specify playback is in TimeShift Mode */
    CNC_SINT32                      preBufferingStatus;          /**< Prebuffering Status */
    CNC_SINT32                      decVidBufOccupancyPercent;   /**< Percentage of Video Buffer Occupancy, Can be used to fetch High Resolution video for HLS if BW is too good */
    CNC_SINT32                      decAudBufOccupancyPercent;   /**< percentage of Audio Buffer Occupancy, Can be used to fetch High Resolution video for HLS if BW is too good */
    CNC_SINT32                      vidBufOccupancyinBytes;      /**< Video Buffer Occupancy in Bytes, Can be used to fetch High Resolution video for HLS if BW is too good */
    CNC_SINT32                      audBufOccupancyInBytes;      /**< Audio Buffer Occupancy in Bytes, Can be used to fetch High Resolution video for HLS if BW is too good */
    cncTrickInfo_t                  trickInfo;                   /**< Trick info */
    E_cncErrCode_t                  errorStatus;                 /**< Error Status */
} cncPlaybackStatus_t;

/*
 * Sub Segment Index Info (ssix) Information
 */
typedef struct _cncSubSegmentIndexInfo_t
{
   CNC_UINT8                              subSegLevel;
   CNC_UINT32                             subsegRangeSize;
   CNC_UINT64                             subSegOffset;
} cncSubSegmentIndexInfo_t;

/*
 * Segment Index Info (sidx) Information
 */
typedef struct _cncSegmentIndexInfo_t
{
   CNC_UINT32                             segNumber;
   CNC_UINT64                             segOffset;
   CNC_UINT32                             segSize;
   CNC_UINT32                             segDuration;
   CNC_UINT32                             subSegCount;
   cncSubSegmentIndexInfo_t              *p_subSegments;
} cncSegmentIndexInfo_t;

/*
 * Segment Index Table Info 
 */
typedef struct _cncSegmentIndexTable_t
{
   CNC_UINT32                             segCount;
   cncSegmentIndexInfo_t                 *p_segment;
   struct _cncSegmentIndexTable_t        *p_nextSegmentIndex;
} cncSegmentIndexTable_t;

/*
 * Structure for storing segment index and subsegment index media information
 */
typedef struct _cncSegmentIndexTableList_t
{
   CNC_UINT32                             segIndexCount;
   cncSegmentIndexTable_t                *p_segmentIndex;
} cncSegmentIndexTableList_t; 

/**
 *      \brief dataType Specifies data type
 */
typedef enum _cncMediaDataType_t
{
    CNC_MEDIA_DATA_TYPE_GENERIC = 0,                          /*!< To generic (Private) Data */
    CNC_MEDIA_DATA_TYPE_AUD_ES_FRAME_HEADER,                  /*!< To Audio ES Raw Data header */
    CNC_MEDIA_DATA_TYPE_AUD_ES_FRAME,                         /*!< To Audio ES Raw Data*/
    CNC_MEDIA_DATA_TYPE_AUD_CONTAINER,                        /*!< To Audio ES Packetized Data */
    CNC_MEDIA_DATA_TYPE_VID_ES_FRAME_HEADER,                  /*!< To Video ES Raw Data header */
    CNC_MEDIA_DATA_TYPE_VID_ES_FRAME,                         /*!< To Video ES Raw Data */
    CNC_MEDIA_DATA_TYPE_VID_CONTAINER,                        /*!< To Video ES Packetized Data */
    CNC_MEDIA_DATA_TYPE_CONTAINER,                            /*!< To complete Container Data */
    CNC_MEDIA_DATA_TYPE_SUBTITLE,                             /*!< To complete subtitle Data */
    CNC_MEDIA_DATA_TYPE_IMAGE                                 /*!< To complete Thumbnail Data */
} cncMediaDataType_t;

/**
 * \enum cncMediaPtsType_t
 * \brief  PTS type
 */
typedef enum _cncMediaPtsType_t
{
    CNC_MEDIA_PTS_TYPE_UNKNOWN = 0,                   /*!< To Unknown Pts type */
    CNC_MEDIA_PTS_TYPE_REL,                           /*!< To Relative Pts type */
    CNC_MEDIA_PTS_TYPE_ABS                            /*!< To Absolute Pts type */
} cncMediaPtsType_t;

/**
 * \struct dataPacketInfoTag
 * \brief  Data info
 */
typedef struct _cncDataPktInfo_t
{
    cncMediaDataType_t                    dataType;
    cncMediaPtsType_t                     ptsType;
    CNC_UINT8                            *p_dataBuf;
    CNC_BOOL                              isItKeyFrame;
    CNC_SINT32                            dataBufLength;
    CNC_SINT64                            dataPts;
    CNC_SFLOAT64                          chunkTime;
    CNC_SINT32                            trackId;
    CNC_SINT32                            bitrateIndex;
} cncMediaDataPkt_t;

typedef struct _cncMediaAudioTrackInfo_t
{
    cncMediaFormat_t                      format;                             //!< media type 
    CNC_UINT8                             isItBigEndian;
    CNC_SINT32                            sampleRate;
    CNC_SINT32                            noOfChannels;
    cncAudioSampleFormat_t                sampleFormat;
    CNC_UINT32                            bitRate;
    CNC_SFLOAT64                          fps;                                //!< frame rate of current audio
} cncMediaAudioTrackInfo_t;

typedef struct _cncMediaVideoTrackInfo_t
{
    cncMediaFormat_t                      format;                             //!< media type 
    CNC_SINT64                            firstPts;                           //!< first pts of video
    CNC_SFLOAT64                          fps;                                //!< frame rate of video
    CNC_UINT32                            bitRate;                            //!< es bitrate of video
    CNC_SINT8                             frameRate;
    CNC_SINT32                            width;                              //!< width of frame
    CNC_SINT32                            height;                             //!< height of frame
    CNC_SINT32                            frameType;                          //!< 0 - Fixed Frame Rate, 1 - Variable Frame Rate */
    S_cncRational_t                       sampleAspectRatio;
    CNC_SINT32                            totalKeyFrameCnt;                   //!< Total Key-frames present in the stream */
} cncMediaVideoTrackInfo_t;

typedef struct _cncMediaSubtitleTrackInfo_t
{
    cncSubtitleType_t                     type;
    CNC_BOOL                              isClosedCaption;
    cncSubtitleInputMethodType_t          inputType;
} cncMediaSubtitleTrackInfo_t;

typedef struct _cncMediaContainerTrackInfo_t
{
    CNC_UINT32                            bitRate;                                 //!< container bit rate
} cncMediaContainerTrackInfo_t;

typedef union _cncMediaTrackDetail_t
{
    cncMediaAudioTrackInfo_t              audioInfo;
    cncMediaVideoTrackInfo_t              videoInfo;
    cncMediaSubtitleTrackInfo_t           subtitleInfo;
    cncMediaContainerTrackInfo_t          containerInfo;
} cncMediaTrackDetails_t;

typedef struct _cncMediaTrackInfo_t
{    
    CNC_BOOL                              isSelected;
    cncMediaMetaData_t                    metaData;
    CNC_CHAR                              lang[CNC_MEDIA_MAX_LANG_CODE_SIZE];
    CNC_VOID                             *p_v_handle;                             //!< Track Handle 
    cncMediaDataType_t                    dataType;
    cncMediaTrackDetails_t                detail; 
} cncMediaTrackInfo_t;

typedef struct _cncMediaTrackList_t
{        
    CNC_BOOL                              isVideoPresent;
    CNC_BOOL                              isAudioPresent;
    CNC_BOOL                              isSubtitlePresent;
    CNC_SINT8                             numOfTracks;
    CNC_UINT8                             isSeekSupported;                         //!< TODO: Need to remove it
    CNC_ULONG32                           msDuration;                              //!< duration in ms of this file
    CNC_UINT32                            bitRate;                                 //!< container bit rate
    CNC_VOID                             *p_v_handle;                              //!< Handle for control settings
    cncMediaTrackInfo_t                   a_tracks[CNC_MEDIA_MAX_ES];
} cncMediaTrackList_t;

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_MEDIAFORMAT_H_ */
