/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_mediasource.h
 * @ingroup media
 * Data structures used to define source information about media content
 */

#ifndef _CNC_MEDIA_SOURCE_H_
#define _CNC_MEDIA_SOURCE_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ======================== System Includes =============================== */
// None


/* ========================= User Includes ================================ */
#include "cnc_common_typedef.h"
#include "cnc_mediaformat.h"

/* =========================== Macro definitions ========================== */
#define CNC_MAX_URL_LENGTH                (1024+256) /**< Maximum URL Length */
#define CNC_MAX_USER_AGENT_LENGTH         256        /**< Maximum UserAgent Length */
#define CNC_MAX_PROXY_SERVER_PATH_SIZE    32         /**< Maximum Proxy Server URL Length */

/* ============================ Type Definitions ========================== */

/**
 * \enum _cncMediaSourceConfigMode_t
 * \brief  Media Source config mode for set/get.
 */
typedef enum _cncMediaSourceConfigMode_t
{
    CNC_MEDIA_SOURCE_CONFIG_MODE_GET = 0,  /**< This may be for future use, currently its not supported.*/
    CNC_MEDIA_SOURCE_CONFIG_MODE_SET       /**< To set config mode on buffer Tuner.*/
} cncMediaSourceConfigMode_t;

/**
 * \enum _cncMediaSourceConfigType_t
 * \brief  Media Source config type for bitrate/resolution/DRM Key/ DRM IV/ playback Status.
 */
typedef enum _cncMediaSourceConfigType_t
{
    CNC_MEDIA_SOURCE_CONFIG_TYPE_BITRATE_CHANGE = 0x01,  /**< Notify buffer tuner for bitrate change.*/
    CNC_MEDIA_SOURCE_CONFIG_TYPE_RESOLUTION_CHANGE,      /**< Notify buffer tuner for resolution change.*/
    CNC_MEDIA_SOURCE_CONFIG_TYPE_DRM_KEY,                /**< Notify buffer tuner for DRM Key change change.*/ 
    CNC_MEDIA_SOURCE_CONFIG_TYPE_DRM_IV,                 /**< Notify buffer tuner for DRM IV change change.*/  
    CNC_MEDIA_SOURCE_CONFIG_TYPE_PLAYBACK_STATUS         /**< Notify buffer tuner for Playback status for EOS, NODATA.*/ 
} cncMediaSourceConfigType_t;

/**
 * \struct _cncMediaSourceConfigCtlInfo_t
 * \brief  Media Source Config Control info.
 */
typedef struct _cncMediaSourceConfigCtlInfo_t
{
    cncMediaSourceConfigMode_t           configMode;
    cncMediaSourceConfigType_t           configType;
    cncPlaybackStatus_t                  playbackStatus; /**< Contains Playback Status Information */
} cncMediaSourceConfigCtlInfo_t;

/**
 * \enum cncMediaSourceWhence_t
 * \brief Start position for seek offset
 */
typedef enum _cncMediaSourceWhence_t
{
    CNC_MEDIA_SOURCE_WHENCE_SEEK_SET  = SEEK_SET,  /**< Same behaviour as SEEK_SET defined in stdio.h */
    CNC_MEDIA_SOURCE_WHENCE_SEEK_CUR  = SEEK_CUR,  /**< Same behaviour as SEEK_CUR defined in stdio.h */
    CNC_MEDIA_SOURCE_WHENCE_SEEK_END  = SEEK_END,  /**< Same behaviour as SEEK_END defined in stdio.h */
    CNC_MEDIA_SOURCE_WHENCE_SEEK_SIZE = 0x10000,   /**< Return the filesize without seeking anywhere. Return <0 if not supported */
} cncMediaSourceWhence_t;

/**
 * @brief         Function pointer definition for push buffer tuner data
 * @param [in]    p_v_resourceHandle 
 * @param [in]    p_mediaPacketInfo
 */
typedef E_cncErrCode_t (*cncMediaSourcePushData_t)(CNC_VOID *p_v_resourceHandle, cncMediaDataPkt_t *p_mediaPacketInfo);

/**
 * @brief         Function pointer definition for pull buffer tuner data
 * @param [in]    p_v_trackHandle - Same handle in trackList.a_tracks[trackId].p_v_handle.
 * @param [in]    p_mediaPacketInfo
 */
typedef CNC_SINT32 (*cncMediaSourcePullDataCbf_t)(CNC_VOID *p_v_trackHandle, cncMediaDataPkt_t *p_mediaPacketInfo, CNC_SINT32 reqSize);

/**
 * @brief         Function pointer definition for buffer data byte seek
 * @param [in]    p_v_trackHandle - Same handle in trackList.a_tracks[trackId].p_v_handle.
 * @param [in]    offset
 * @param [in]    whence
 */
typedef CNC_SINT64 (*cncMediaSourceByteSeekCbf_t)(CNC_VOID *p_v_trackHandle, CNC_SINT64 offset, cncMediaSourceWhence_t whence);

/**
 * @brief         Function pointer definition for buffer data time seek.
 * @param [in]    p_v_resourceHandle 
 * @param [in]    trackId
 * @param [in]    offset
 * @param [in]    flags
 */
typedef CNC_SINT64 (*cncMediaSourceTimeSeekCbf_t)(CNC_VOID *p_v_resourceHandle, CNC_SINT32 trackId, CNC_SINT64 offset, CNC_SINT32 flags);

/**
 * @brief         Function pointer definition for buffer tuner control setting
 * @param [in]    p_v_handle 
 * @param [in]    p_mediaSourceConfigCtlInfo
 */
typedef E_cncErrCode_t (*cncMediaSourceCtlSettings_t)(CNC_VOID *p_v_handle, cncMediaSourceConfigCtlInfo_t *p_mediaSourceConfigCtlInfo);

/**
 * @enum    cncMediaSourceAccessType_t
 * @brief   Specifies media source access type Like PULL/PUSH
 * @note   
 */
typedef enum _cncMediaSourceAccessType_t
{
    CNC_MEDIA_SOURCE_ACCESS_TYPE_UNKNOWN = 0,
    CNC_MEDIA_SOURCE_ACCESS_TYPE_PULL_CONTAINER,
    CNC_MEDIA_SOURCE_ACCESS_TYPE_PULL_ES,
    CNC_MEDIA_SOURCE_ACCESS_TYPE_PUSH_CONTAINER,
    CNC_MEDIA_SOURCE_ACCESS_TYPE_PUSH_ES,
    CNC_MEDIA_SOURCE_ACCESS_TYPE_PULL_DATA,
    CNC_MEDIA_SOURCE_ACCESS_TYPE_PUSH_DATA
} cncMediaSourceAccessType_t;


/**
 * @enum    cncStreamSourceType_t
 * @brief   Specifies Input Data Source Type Like FILE/HTTP/HLS
 * @note   
 */
typedef enum _cncStreamSourceType_t 
{
    CNC_STREAM_SOURCE_TYPE_UNKNOWN = 0,                    /**< Unknown stream source type */
    CNC_STREAM_SOURCE_TYPE_FILE,                           /**< FILE stream source type */
    CNC_STREAM_SOURCE_TYPE_HTTP,                           /**< HTTP stream source type */
    CNC_STREAM_SOURCE_TYPE_HLS,                            /**< HLS stream source type */
    CNC_STREAM_SOURCE_TYPE_RTSP,                           /**< RTSP stream source type */
    CNC_STREAM_SOURCE_TYPE_RTP,                            /**< RTP stream source type */
    CNC_STREAM_SOURCE_TYPE_MMS,                            /**< MMS stream source type */
    CNC_STREAM_SOURCE_TYPE_SMOOTH_STREAMING,               /**< Smooth Streaming stream source type */
    CNC_STREAM_SOURCE_TYPE_ADOBE_PRIMETIME,                /**< Adobe PrimeTime stream source type */
    CNC_STREAM_SOURCE_TYPE_DASH,                           /**< DASH stream source type */
    CNC_STREAM_SOURCE_TYPE_BUFFER                          /**< Buffer source type */
} cncStreamSourceType_t;

/**
 * @enum   cncHttpTransferEncodingType_t  
 * @brief  Specifies http server data transfer encoding type
 * @note   
 */
typedef enum _cncHttpTransferEncodingType_t
{
    CNC_HTTP_TRANSFER_ENCODING_TYPE_UNKNOWN = 0,           /**< Unknown transfer encoding type */
    CNC_HTTP_TRANSFER_ENCODING_TYPE_NORMAL,                /**< Server sends static data, content length known */
    CNC_HTTP_TRANSFER_ENCODING_TYPE_CHUNKED                /**< Server sends dynamically generated data, content length - unknown*/
} cncHttpTransferEncodingType_t;

/**
 * @enum   cncStreamingMode_t  
 * @brief  Specifies Streaming Mode, Like LIVE or VOD
 * @note   
 */
typedef enum _cncStreamingMode_t
{
    CNC_STREAMING_MODE_UNKNOWN = 0,        /**< Unknown(LIVE/VOD) streaming mode */
    CNC_STREAMING_MODE_LIVE,               /**< Live Streaming Mode */
    CNC_STREAMING_MODE_VOD                 /**< VOD Streaming Mode */
} cncStreamingMode_t;

/**
 * @enum   cncMediaServerCategory_t  
 * @brief  Specifies Server category like DLNA, Youtube, Apache, Shoutcast..etc
 * @note   
 */
typedef enum _cncMediaServerCategory_t
{
    CNC_MEDIA_SERVER_CATEGORY_UNKNOWN = 0,    /**< Dont know anything about Server. Need to find or worst case assume it as Generic Category */
    CNC_MEDIA_SERVER_CATEGORY_GENERIC,        /**< Simple HTTP/RTSP/HLS Server. Dont know much about the server's additional features */
    CNC_MEDIA_SERVER_CATEGORY_DLNA,           /**< Server which adheres to DLNA Standard */
    CNC_MEDIA_SERVER_CATEGORY_IPSP,           /**< Server which adheres to IPSP Standard */
    CNC_MEDIA_SERVER_CATEGORY_C2M,            /**< Content-to-Media Server */
    CNC_MEDIA_SERVER_CATEGORY_YOUTUBE,        /**< Youtube Server */
    CNC_MEDIA_SERVER_CATEGORY_APACHE,         /**< Apache Server */
    CNC_MEDIA_SERVER_CATEGORY_SHOUTCAST       /**< ICY / Standard */
} cncMediaServerCategory_t;

/**
 * @enum   cncStreamQualityAdaptType_t
 * @brief  Defines different types of adapting stream quality to current download rate (current network bandwidth)
 * @note   This is used only for moving from lower to higher bandwidths based on download rate.
 *         Moving Higher to lower bandwidth is done using only approximation (bandwidth nearest to download rate).
 */
typedef enum _cncStreamQualityAdaptType_t
{
    CNC_STREAM_QUALITY_ADAPT_TYPE_NONE = 0,      /**< cncmediactl will choose the type internally */
    CNC_STREAM_QUALITY_ADAPT_TYPE_INCREMENTAL,   /**< Switching from lower to higher bitrate is done using one step at a time regardless of current download rate. Ex: Low → average → high → highest */
    CNC_STREAM_QUALITY_ADAPT_TYPE_NEAREST        /**< Switch to the nearest bitrate w.r.t to download rate */
} cncStreamQualityAdaptType_t;


/**
 * @enum   cncHlsPlayerType_t
 * @brief  Specifies which player(Native/Adobe Primetime) to choose while playing HLS stream.
 * @note
 */
typedef enum _cncHlsPlayerType_t
{
    CNC_HLS_PLAYER_NONE = 0,      /**< hls stream will be played using Native player by default */
    CNC_HLS_PLAYER_ADOBE_PRIMTIME, /**< hls stream will be played using Adobe Primetime player */
    CNC_HLS_PLAYER_NATIVE        /**< hls stream will be played using Native player */
} cncHlsPlayerType_t;

/**
 * @struct  cncUnknownSourceInfo_t
 * @brief   Holds information of unknown source
 * @note   
 */
typedef struct _cncUnknownSourceInfo_t
{
    CNC_CHAR               url[CNC_MAX_URL_LENGTH];                      /**< Unknown Source URL */
    CNC_CHAR               userAgent[CNC_MAX_USER_AGENT_LENGTH];         /**< User Agent, Which can be set to unknown server */
} cncUnknownSourceInfo_t;

/**
 * @struct  cncFileSourceInfo_t
 * @brief   Holds information of file source
 * @note   
 */
typedef struct _cncFileSourceInfo_t
{
    CNC_CHAR               url[CNC_MAX_URL_LENGTH];                       /**< File Source URL */
} cncFileSourceInfo_t;

/**
 * @struct  cncHttpSourceInfo_t
 * @brief   Holds information of http source
 * @see     cncHttpTransferEncodingType_t
 */
typedef struct _cncHttpSourceInfo_t
{
    CNC_CHAR                           url[CNC_MAX_URL_LENGTH];              /**< HTTP Source URL */
    CNC_CHAR                           userAgent[CNC_MAX_USER_AGENT_LENGTH]; /**< UserAgent details (Optional) */
    cncHttpTransferEncodingType_t      httpTransferEncoding;                 /**< Chunked/Normal transfer Encoding type */
} cncHttpSourceInfo_t;

/**
 * @struct  cncHlsSourceInfo_t
 * @brief   Holds information of hls source
 * @note   
 */
typedef struct _cncHlsSourceInfo_t
{
    CNC_CHAR               url[CNC_MAX_URL_LENGTH];                        /**< HLS Source URL */
    CNC_CHAR               userAgent[CNC_MAX_USER_AGENT_LENGTH];           /**< UserAgent details (Optional) */
    CNC_CHAR               proxyServer[CNC_MAX_PROXY_SERVER_PATH_SIZE];    /**< Proxy Server details (Optional) like proxy server IP and port no to set proxy */
    cncHlsPlayerType_t     hlsPlayerType;                                  /**< Specifies which player(Native/Adobe Primetime) to choose while playing HLS stream. */
} cncHlsSourceInfo_t;


/**
 * @struct  cncSsSourceInfo_t
 * @brief   Holds information of Smooth Streaming Source
 * @note   
 */
typedef struct _cncSsSourceInfo_t
{
    CNC_CHAR               url[CNC_MAX_URL_LENGTH];  /**< SmoothStream Source URL */
} cncSsSourceInfo_t;

/**
 * @struct  cncDashSourceInfo_t
 * @brief   Holds information of DASH Source
 * @note   
 */
typedef struct _cncDashSourceInfo_t
{
    CNC_CHAR               url[CNC_MAX_URL_LENGTH];   /**< DASH Source URL */
} cncDashSourceInfo_t;


/**
 * @struct  cncPrimeTimeSourceInfo_t
 * @brief   Holds information of primeTime Source
 * @note   
 */
typedef struct _cncPrimeTimeSourceInfo_t
{
    CNC_CHAR               url[CNC_MAX_URL_LENGTH];   /**< DASH Source URL */
} cncPrimeTimeSourceInfo_t;

/**
 * @struct  cncBufferSourceInfo_t
 * @brief   Holds information of Buffer Source pull, push callbacks and track related information.
 * @note   This is used push pull of A/V/S data to buffer tuner. 
 *         App will populate track information, pullDataCbf, byteSeekCbf and timeSeekCbf function pointers.
 *         cncmedia will populate pushData and ctlSettings function pointers.
 *         Currently it supports only push ES data method.
 *         Some more features like bitrate change notification, EOS notification, trickplay support like Seek, pause, FF/FR and pull container need to be added.
 */
typedef struct _cncBufferSourceInfo_t
{
    cncMediaSourceAccessType_t     accessType;     /**< Pull/push/container/ES type*/
    cncMediaTrackList_t            trackList;      /**< to maintain tracklist for A/V/S*/
    cncMediaSourcePushData_t       pushData;       /**< fn to push the buffer tuner data */
    cncMediaSourcePullDataCbf_t    pullDataCbf;    /**< fn to pull the buffer tuner data */
    cncMediaSourceByteSeekCbf_t    byteSeekCbf;    /**< fn to byte seek in the buffer tuner data */
    cncMediaSourceTimeSeekCbf_t    timeSeekCbf;    /**< fn to time seek in the buffer tuner data */
    cncMediaSourceCtlSettings_t    ctlSettings;    /**< control settings for buffer tuner */
} cncBufferSourceInfo_t;

/**
 * @struct  cncSourceInfo_t 
 * @brief   Holds Input Data Source information like FILE, HTTP, HLS, SmoothStreaming, DASH, Buffer
 * @see     cncUnknownSourceInfo_t
 * @see     cncFileSourceInfo_t
 * @see     cncHttpSourceInfo_t
 * @see     cncHlsSourceInfo_t
 * @see     cncSsSourceInfo_t
 * @see     cncDashSourceInfo_t
 * @see     cncBufferSourceInfo_t
 */
typedef union _cncSourceInfo_t
{
    cncUnknownSourceInfo_t   unknownSourceInfo;    /**< Unknown source information like url and userAgent */
    cncFileSourceInfo_t      fileSourceInfo;       /**< FILE source information like url */
    cncHttpSourceInfo_t      httpSourceInfo;       /**< HTTP source information like url, userAgent and httpTransferEncoding */
    cncHlsSourceInfo_t       hlsSourceInfo;        /**< HLS source information like url, userAgent and PoxyServer */
    cncSsSourceInfo_t        ssSourceInfo;         /**< Smooth Streaming source information like url */
    cncDashSourceInfo_t      dashSourceInfo;       /**< DASH source information like url */
    cncPrimeTimeSourceInfo_t primeTimeSourceInfo;  /**< PrimeTime source information like url */
    cncBufferSourceInfo_t    bufferSourceInfo;     /**< buffer source information like url and fp */
} cncSourceInfo_t;

/**
 * @struct  cncStreamBandwidthConfig_t
 * @brief   Defines configuration settings for adaptive streaming
 * @note    minVideoResolutionAtStart has more priority w.r.t minBandwidthAtStart
 * @see     cncStreamQualityAdaptType_t
 */
typedef struct _cncStreamBandwidthConfig_t
{
    CNC_SINT32                    minBandwidthAtStart;       /**< Minimum bandwidth (in bps) of the stream to be selected at start of playback (seek).  0 -> Least, -1 -> Max, Non-zero +ve ->  Minimum Bandwidth */
    CNC_SINT32                    minVideoResolutionAtStart; /**< Minimum video resolution (in pixels) of the stream to be selected at start of playback (seek). 0 -> least, -1 -> Max, Non-zero +ve -> Minimum Video Resolution */
    CNC_SINT32                    bandwidthOverHead;         /**< Bandwidth overhead in % to reach to switch to next higher content quality. 0 -> No overhead */
    CNC_SINT32                    manifestDownloadTimeout;   /**< Manifest download timeout in milliseconds. 0 -> Will be choosen by cncmediactl internally */
    cncStreamQualityAdaptType_t   adaptType;                 /**< Bandwidth adapt type */
} cncStreamBandwidthConfig_t;

/**
 * @struct cncMediaSourceInfo_t
 * @brief  Holds media source information like input data source, server category, mode of transmission..etc
 * @see    cncStreamSourceType_t
 * @see    cncMediaServerCategory_t
 * @see    cncSourceInfo_t
 * @see    cncStreamingMode_t
 * @see    cncStreamBandwidthConfig_t 
 */
typedef struct _cncMediaSourceInfo_t
{
    cncStreamSourceType_t        streamSource;                  /**< Stream source information like FILE/HTTP/HLS/RTSP/RTP/MMS/SS/DASH */
    cncMediaServerCategory_t     serverCategory;                /**< Server Category information like DLNA/IPSP/C2M/YOUTUBE/APACHE/SHOUTCAST */
    cncSourceInfo_t              sourceInfo;                    /**< Specific source information for each stream source like for HLS source info is url, UserAgent and ProxyServer */
    CNC_SINT32                   timeSeekSupported;             /**< Flag indicates Time Seek is supported by source */
    CNC_SINT32                   byteSeekSupported;             /**< Flag indicates Byte Seek is supported by source */
    CNC_BOOL                     selectAllAudioTracks;          /**< If true initiates all Audio Tracks either for recording/playback  */
    CNC_BOOL                     selectAllSubtitleTracks;       /**< If true initiates all Subtitle Tracks either for recording/playback */ 
    CNC_UINT32                   maxNetworkBandWidth;           /**< Max Network Bandwidth of the source */
    cncStreamingMode_t           streamingMode;                 /**< Streaming mode type LIVE/VOD */
    CNC_UINT8                    loopFlag;                      /**< Flag indicates loop is supported by source */
    CNC_SINT32                   preBufferingTimeInSecs;        /**< Prebuffering time in seconds while playing */
    CNC_SINT32                   preBufferingTimeAtStartInSecs; /**< Prebuffering time in seconds at start of playback */
    CNC_SINT64                   preBufferingInBytes;           /**< Total no of bytes to prebuffer */
    CNC_SINT32                   httpStallingSupportStatus;     /**< Flag indicates http stalling support by source */ 
    CNC_SINT32                   isMediaTranscoded;             /**< Flag indicates media transcoded or not */
    cncStreamBandwidthConfig_t   bandwidthConfig;               /**< Bandwidth configuration like start with start bandwidth value, start video resolution value, adoptive type incremental/nearest */
} cncMediaSourceInfo_t;
/* ======================== Function declarations ========================= */

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_MEDIA_SOURCE_H_ */
