/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Defines the APIs for PVR Database
 * @file cnc_pvr_db.h
 * @ingroup media
 * @note  This file should be used only for end applications of the media pipeline
 */

#ifndef _CNC_PVR_DB_H_
#define _CNC_PVR_DB_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ======================== System Includes =============================== */
// None

/* ========================= User Includes ================================ */
#include "cnc_common_typedef.h"

/* =========================== Macro definitions ========================== */
// None

/* ============================ Type Definitions ========================== */

/**
 * @enum    cncFileMgrRecListSortType_t
 * @brief   Recording list sort type
 * @note
 */
typedef enum _cncFileMgrRecListSortType_t
{
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_NONE = 0,                   /**< No sorting for recorded content list */
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_REC_DATE_ASCND,             /**< Sort list by recording date in ascending order */
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_REC_DATE_DSCND,             /**< Sort list by recording date in descending order */
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_PROG_NAME_ASCND,            /**< Sort list by program name in ascending order */
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_PROG_NAME_DSCND,            /**< Sort list by program name in descending order */
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_DURATION_ASCND,             /**< Sort list by duration in ascending order */
    CNC_FILE_MGR_REC_LIST_SORT_TYPE_DURATION_DSCND              /**< Sort list by duration in descending order */
} cncFileMgrRecListSortType_t;

/**
 * @enum    cncFileMgrRecContentEditType_t
 * @brief   Recorded Content Edit Type
 * @note
 */
typedef enum _cncFileMgrRecContentEditType_t
{
    CNC_FILE_MGR_REC_CONTENT_EDIT_TYPE_CHANNEL_NAME = 0,        /**< Edit the channel name of recorded content info */
    CNC_FILE_MGR_REC_CONTENT_EDIT_TYPE_PROG_NAME,               /**< Edit the program name of recorded content info */
    CNC_FILE_MGR_REC_CONTENT_EDIT_TYPE_PROG_DESC                /**< Edit the program description of recorded content info */
} cncFileMgrRecContentEditType_t;

/**
 * @enum    cncFileMgrThumbnailType_t
 * @brief   Contains the information about the type of thumbnail present in recorded content
 * @note
 */
typedef enum _cncFileMgrThumbnailType_t
{
    CNC_FILE_MGR_THUMBNAIL_TYPE_NONE = 0,                       /**< No thumbnail is present */
    CNC_FILE_MGR_THUMBNAIL_TYPE_JPEG,                           /**< Thumbnail type is JPEG */
    CNC_FILE_MGR_THUMBNAIL_TYPE_BMP                             /**< Thumbnail type is BMP */
} cncFileMgrThumbnailType_t;

/**
 * @enum    cncFileMgrThumbnailInfo_t
 * @brief   Contains information about the thumbnail present in the recorded content
 * @note
 */
typedef struct _cncFileMgrThumbnailInfo_t
{
    cncFileMgrThumbnailType_t   type;                           /**< Thumbnail Type (= 0 if not present) */
    CNC_BOOL                    isEncrypted;                    /**< TRUE if Thumbnail is encrypted, else it is FALSE */
    CNC_SINT32                  width;                          /**< Thumbnail Width */
    CNC_SINT32                  height;                         /**< Thumbnail Height */
    CNC_SINT32                  timeOffset;                     /**< Time offset in milliseconds at which thumbnail was generated or needs to be generted */
    CNC_SINT32                  size;                           /**< Size of thumbnail */
} cncFileMgrThumbnailInfo_t;

/**
 * @enum    cncFileMgrRecContentHeader_t
 * @brief   Defines the header of recording content info file.
 * @note    Used internally in crisp. Application should not use this.
 */
typedef struct _cncFileMgrRecContentHeader_t
{
    CNC_CHAR        cavId[8];           /**< File signature/magic number to identify if the recorded content is recorded using crisp SDK */
    CNC_SINT32      versionNum;         /**< Version number to indentify the SDK version used for recording */
    CNC_SINT32      playbackInfoLen;    /**< Length of the recorded content playback information */
} cncFileMgrRecContentHeader_t;

/**
 * @enum    cncFileMgrRecContentInfo_t
 * @brief   Contains the information of recorded content (current used only for OTT recording)
 * @note
 */
typedef struct _cncFileMgrRecContentInfo_t
{
    CNC_SINT32                           parentalRating;        /**< Parental rating of the recorded program in years */
    CNC_SINT32                           contentDuration;       /**< Recorded content duration in milli seconds */
    CNC_SINT32                           resumePlaybackTime;    /**< Previous stop playback time in milli seconds */
    CNC_CHAR                            *p_channelName;         /**< Channel name */
    CNC_CHAR                            *p_progTitle;           /**< Program name (title) */
    CNC_CHAR                            *p_progDesc;            /**< Program description (summary) */
    CNC_CHAR                            *p_contentPath;         /**< Path where the particular content in the linked list entry is recorded */
    CNC_CHAR                            *p_lastPlaybackDate;    /**< Last playback date in “YYYY-MM-DD HH:MM:SS” format. If “”, then it is newly recorded content (not played ever) */
    CNC_CHAR                            *p_recordingDate;       /**< Date of recording in "YYYY-mm-DD HH:MM:SS" format */
    CNC_CHAR                            *p_thumbnailPath;       /**< Path where the thumbnail of particular content in the linked list entry is present */
    cncFileMgrThumbnailInfo_t           *p_thumbnailInfo;       /**< Thumbnail Information */
} cncFileMgrRecContentInfo_t;

/* ======================== Function declarations ========================= */
/**
 * @brief           Gets the Recorded content list for the given folder path
 * @param [in/out]  pp_v_handle
 *                      Instance reference of content database which needs to be used
 *                      Content data base is generated only when *pp_v_handle is NULL
 *                      If *pp_v_handle is not NULL, already generated data base will be used for getting content list
 * @param [out]     ppp_recContentTable
 *                      Reference to list of entries requested
 * @param [in]      p_dirPath
 *                      Folder which needs to be scanned recursively for creating content data base and getting recorded content list
 * @param [in]      startIndex
 *                      Start index from where the content list ppp_recContentTable needs to be provided
 * @param [in/out]  p_reqNumEntries
 *                      No.of entries to be read from the starting index. This will be updated back with the no.of entries available for reading
 * @param [in]      sortType
 *                      Type of sorting to be done on content list
 * @param [out]     p_totalNumEntries
 *                      Total number of entries in the content database
 * @see             cncFileMgrRecContentInfo_t
 * @see             cncFileMgrRecListSortType_t
 * @return          0 On Success
 */
CNC_SINT32 CNC_FileMgr_GetRecContentList(CNC_VOID                     **pp_v_handle,
                                         cncFileMgrRecContentInfo_t  ***ppp_recContentTable,
                                         const CNC_CHAR                *p_dirPath,
                                         CNC_SINT32                     startIndex,
                                         CNC_SINT32                    *p_reqNumEntries,
                                         CNC_SINT32                    *p_totalNumEntries,
                                         cncFileMgrRecListSortType_t    sortType);

/**
 * @brief           Frees up the recorder content list information pointed by pp_v_handle
 * @param [in/out]  pp_v_handle
 *                      Reference to the list which needs to be freed
 * @return          0 On Success
 */
CNC_SINT32 CNC_FileMgr_FreeRecContentList(CNC_VOID **pp_v_handle);

/**
 * @brief           Deletes the recorded content exists in the given path
 * @param [in]      p_contentPath
 *                      Recorded content Path
 * @return          0 On Success
 */
CNC_SINT32 CNC_FileMgr_DeleteRecContent(CNC_CHAR *p_contentPath);

/**
 * @brief           Updates recorded content information field type with data provided by user
 * @param [in]      p_contentPath
 *                      Recorded content Path
 * @param [in]      p_data
 *                      New Data from user
 * @param [in]      editType
 *                      Field which is to be edited and filled with new data (p_data)
 * @see             cncFileMgrRecContentEditType_t
 * @return          0 On Success
 */
CNC_SINT32 CNC_FileMgr_EditRecContentInfo(CNC_CHAR *p_contentPath, CNC_CHAR *p_data, cncFileMgrRecContentEditType_t editType);

/**
 * @brief           Gets the Thumbnail present in the Recorded content list from the given folder path
 * @param [in]      p_contentPath
 *                      Recorded content path from where the thumbnail data to be read
 * @param [in/out]  p_thumbnailInfo
 *                      Thumbnail Info which contains the information about the thumbnail present in given content path
 * @param [out]     p_thumbnailData
 *                      Thumbnail Data read from the given content path
 * @see             cncFileMgrThumbnailInfo_t
 * @note            User has to allocate memory as given below before calling this API:
 *                              p_thumbnailData = malloc(p_thumbnailInfo->size);
 * @return          0 On Success
 */
CNC_SINT32 CNC_FileMgr_ReadThumbnail(CNC_CHAR *p_contentPath, cncFileMgrThumbnailInfo_t *p_thumbnailInfo, CNC_UINT8 *p_thumbnailData);

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_PVR_DB_H_ */

