/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
/**
 * @file cnc_system.h
 * @ingroup core
 * The CNCSYSTEM module is use to manage the system and device resources including
 * the display output, graphics layers, peripherals, etc.
 *
 * The  CNCSYSTEM model provides a set of functions to manage and control the hardware  devices.
 * It consists of a number of functional groups including:</p>
 *   @li CNC_DEV  for initializing the system devices and core resources of the chip.</li>
 *   @li CNC_Setting  for managing default settings in EEPROM</li>
 *   @li CNC_Input for defining key mappings and handling remote events</li>
 *   @li CNC_Peripheral for working with peripherals like I2C, GPIO, etc.</li>
 *
 * A type startup sequence in an application would initialize the system module and devices, for example:
 * @verbatim
   CNC_System_Init();
   CNC_DEV_Init();
   CNC_DEV_EnableFrontPanel (CNC_BOOL_FALSE);
   CNC_DEV_SetGraphicsConfig (OSD_LAYER_0, defaultGraphicsConfig);
   CNC_DEV_EnableGraphics (OSD_LAYER_0);
   CNC_DEV_OpenHDOutput (settingData.TVOutMode);
   CNC_DEV_OpenSDOutput (settingData.SDTVOutMode);
   CNC_DEV_EnableSDOutput();
 * @endverbatim
 *
 */

#ifndef __CNC_SYSTEM_H__
#define __CNC_SYSTEM_H__

#include <stdarg.h>

#include "csapi.h"
#include "cnc_common_typedef.h"
#include "cnc_av.h"
#include "cnc_util.h"

#ifdef __cplusplus
extern "C" {
#endif

#define LOADER_EEPROM_ADDR_PID  (512)
#define LOADER_EEPROM_ADDR_TABLEID (LOADER_EEPROM_ADDR_PID+2)
#define LOADER_EEPROM_ADDR_FRE (LOADER_EEPROM_ADDR_PID+4)
#define LOADER_EEPROM_ADDR_TYPE (LOADER_EEPROM_ADDR_PID+6)
#define LOADER_EEPROM_ADDR_MOD (LOADER_EEPROM_ADDR_PID+8)
#define LOADER_EEPROM_ADDR_SYM (LOADER_EEPROM_ADDR_PID+10)
#define LOADER_EEPROM_ADDR_KERNEL_OLDVERSION (LOADER_EEPROM_ADDR_PID+12)
#define LOADER_EEPROM_ADDR_KERNEL_NEWVERSION (LOADER_EEPROM_ADDR_PID+14)
#define LOADER_EEPROM_ADDR_ROOTFS_OLDVERSION (LOADER_EEPROM_ADDR_PID+16)
#define LOADER_EEPROM_ADDR_ROOTFS_NEWVERSION (LOADER_EEPROM_ADDR_PID+18)
#define LOADER_EEPROM_ADDR_CLEARUSRDB (LOADER_EEPROM_ADDR_PID+20)
#define LOADER_FLAG (0X47)

#define LOADER_EEPROM_ADDR_FLAG (1)

#define GPIO_MIN_PORT    (0)
#define GPIO_MAX_PORT    (15)

#define CNC_POWER_TYPE_INDEX(index)    (1<<index)

#define CHECK_I2C_PARAMS_VALID(p) do { \
    if (NULL == p) { \
        return RESULT_BAD_PARAMETER; \
    } \
} while(0)

#define CNC_I2C_DEBUG printf

#define TVOUT_MODE_USE_EDID         TVOUT_MODE_MAX
#define TVOUT_MODE_NOT_SET          0xFFFFFFFF

typedef CNC_SINT32 (*CNC_TraceCallbackFunction_t)  (const CNC_CHAR    *moduleName,
                                                    const CNC_SINT32   logType,
                                                    const CNC_CHAR    *funcName,
                                                    const CNC_CHAR    *fileName,
                                                    const CNC_SINT32   lineNum,
                                                    const CNC_UINT32   stringID,
                                                    const CNC_CHAR    *fmt,
                                                    va_list            args) __attribute__((format(printf, 7, 0)));

/**
 * Register a callback function for capturing trace outputs from cnclibrary (removes console printing).
 * @param devicePrint user callback function
 */
void CNC_TRACE_RegisterCallback(CNC_TraceCallbackFunction_t  devicePrint);

/**
 *  Implements printing of trace messages to syslog.
 *
 *  This should registered via CNC_TRACE_RegisterCallback, not called directly.
 *  This implementation sends the module id at appropriate priority for the message,
 *  time in microseconds, and for debug logs includes function and line number.
 */
extern CNC_TraceCallbackFunction_t g_CNC_TRACE_Syslog;

typedef void (*CNC_DEV_CallbackFunction_t)  (CNC_SINT32    event,
                                             CNC_SINT32    *data);

typedef enum {
    CNC_DEV_EVENT_NONE,
    CNC_DEV_EVENT_RESOLUTION_CHANGED,               ///< data in the callback will contain the new resolution
    CNC_DEV_EVENT_HDMI_PLUG_STATUS_CHANGED,         ///< data in the callback will contain the plug status
    CNC_DEV_EVENT_HDCP_STATUS_CHANGED,              ///< data in the callback will contain the downstream info
    CNC_DEV_EVENT_HDCP_SUPPORTED_CHANGED,           ///< data in the callback will indicate HDCP supported, true or false
    CNC_DEV_EVENT_CEC_STANDBY,                      ///< data in the callback will contain Cec status
    CNC_DEV_EVENT_CEC_DEVICE_ACTIVE ,               ///< data in the callback will contain the downstream info
    CNC_DEV_EVENT_CEC_PLAY,                         ///< data in the callback will contain the downstream info
} CNC_DEV_EVENTS_e;

typedef enum
{
    CNC_SMARTCARD_STATUS_IN,
    CNC_SMARTCARD_STATUS_OUT,
    CNC_SMARTCARD_STATUS_ERROR
} cncSmartCardStatus_t;

typedef enum
{
    CNC_SMARTCARD_STANDARD_ISO7816,
    CNC_SMARTCARD_STANDARD_NDS,
    CNC_SMARTCARD_STANDARD_EMV96,
    CNC_SMARTCARD_STANDARD_EMV2000,
    CNC_SMARTCARD_STANDARD_ECHOCHAR_T,
    CNC_SMARTCARD_STANDARD_UNDEFINE
} cncSmartCardStandard_t;

typedef enum _cncPowerModule_t
{
    CNC_POWER_USB        =   CNC_POWER_TYPE_INDEX(0),  // Power Selector USB
    CNC_POWER_SDIO       =   CNC_POWER_TYPE_INDEX(1),  // Power Selector SDIO Driver
    CNC_POWER_XPORT      =   CNC_POWER_TYPE_INDEX(2),  // Power Selector for Hardware Demux Engine
    CNC_POWER_VIB        =   CNC_POWER_TYPE_INDEX(3),  // Power Selector Video Input
    CNC_POWER_VIDEO      =   CNC_POWER_TYPE_INDEX(4),  // Power Selector Video Decoder
    CNC_POWER_AUDDAC     =   CNC_POWER_TYPE_INDEX(5),  // Power Selector Audio Decoder
    CNC_POWER_AUDCLK     =   CNC_POWER_TYPE_INDEX(6),  // Power Selector Audio Clock
    CNC_POWER_AUDIODACIF =   CNC_POWER_TYPE_INDEX(7),  // Power Selector Audio DAC
    CNC_POWER_AUDIOPLL   =   CNC_POWER_TYPE_INDEX(8),  // Power Selector Audio PLL
    CNC_POWER_CRYPTO     =   CNC_POWER_TYPE_INDEX(9),  // Power Selector Security Processor
    CNC_POWER_GRAPHICS   =   CNC_POWER_TYPE_INDEX(10),  // Power Selector Graphics Engine
    CNC_POWER_GRAPHICSOUT=   CNC_POWER_TYPE_INDEX(11),  // Power Selector Graphics Display Feeder
    CNC_POWER_CVBS       =   CNC_POWER_TYPE_INDEX(12),  // Power Selector TV0 (HDMI) Output
    CNC_POWER_YUV        =   CNC_POWER_TYPE_INDEX(13),  // Power Selector TV1 (Analog) Output
    CNC_POWER_HDMI       =   CNC_POWER_TYPE_INDEX(14),  // Power Selector HDMI Output
    CNC_POWER_JPEG       =   CNC_POWER_TYPE_INDEX(15),  // Power Selector JPEG
    CNC_POWER_AHB2       =   CNC_POWER_TYPE_INDEX(16),  // Power Selector AHB2
    CNC_POWER_CEC        =   CNC_POWER_TYPE_INDEX(17),  // Power Selector CEC
    CNC_POWER_IR         =   CNC_POWER_TYPE_INDEX(18),  // Power Selector IR
    CNC_POWER_VIDEOCLK   =   CNC_POWER_TYPE_INDEX(19),  // Power Selector ?Video Output Clock?
    CNC_POWER_VIBINPUT   =   CNC_POWER_TYPE_INDEX(20),  // Power Selector VPB Input
    CNC_POWER_ALL        =   0xffffffff,                // Power ALL
    CNC_POWER_MAX        =   21,
} cncPowerModule_t;

typedef enum _cncPowerMode_t
{
    CNC_POWER_MODE_SLEEP = 0,
    CNC_POWER_MODE_WAKE
} cncPowerMode_t;

typedef enum
{
    CNC_DEV_STATUS_RUNNING, ///< the device is in a running state
    CNC_DEV_STATUS_STANDBY, ///< the device is in standby state
    CNC_DEV_STATUS_UNKNOWN
} CNC_DEV_STATUS_e;

/**
 * This is the main structure for the devices used in the system
 */
typedef struct _cncDeviceContainer_t
{
    CSFP_HANDLE fp_handle;
    CSOSD_HANDLE graphic0_handle;
    CSOSD_HANDLE graphic1_handle;
    CSTVOUT_HANDLE tvout0_handle;
    CSTVOUT_HANDLE tvout1_handle;
    CSTVOUT_MODE tvoutmode;
    CSTVOUT_MODE sdtvoutmode;
    CSOSD_Config graphicsconfig;
    CNC_DEV_CallbackFunction_t callback_function;
} cncDeviceContainer_t;

typedef enum
{
    CNC_CGMS_NONE  = 0,
    CNC_CGMS_COPY_NO,
    CNC_CGMS_COPY_ONCE,
    CNC_CGMS_COPY_FREE,
    CNC_CGMS_COPY_NOMORE,
    CNC_CGMS_4_3_NORMAL,
    CNC_CGMS_16_9_NORMAL,
    CNC_CGMS_4_3_LETTERBOX,
    CNC_CGMS_APS_OFF,
    CNC_CGMS_APS_SPLIT_OFF,
    CNC_CGMS_APS_SPLIT_2LINE,
    CNC_CGMS_APS_SPLIT_4LINE,
    CNC_CGMS_ANALOG_SOURCE
} cncDeviceCGMS_t;

typedef enum
{
    CNC_WSS_NONE  = 0,
    CNC_WSS_COPY_NO,
    CNC_WSS_COPY_YES,
    CNC_WSS_COPYRIGHT_FREE,
    CNC_WSS_4_3_NORMAL,
    CNC_WSS_14_9_LETTERBOX_CENTRE,
    CNC_WSS_14_9_LETTERBOX_TOP,
    CNC_WSS_16_9_LETTERBOX_CENTRE,
    CNC_WSS_16_9_LETTERBOX_TOP,
    CNC_WSS_G16_9_LETTERBOX_CENTRE,
    CNC_WSS_14_9_NORMAL,
    CNC_WSS_16_9_NORMAL,
    CNC_WSS_COPYRIGHT_ASSERTED,
} cncDeviceWSS_t;


typedef enum
{
    CNC_LANG_CHINESE = 0,
    CNC_LANG_ENGLISH,
    CNC_LANG_SPANISH,
    CNC_LANG_PORTUGUESE,
    CNC_LANG_MAX_NUM,
    CNC_LANG_UNKNOWN
} cncSettingLanguage_t;

typedef struct _cncSettings_t
{
    CNC_SINT16 TimeOffset; // units: minute, eg: Beijing = 8*60; -2*60 and so on
    CNC_UINT8 TimeMode; // 0 network mode, 1 broadcast mode(TDT), 2 systerm mode,

    // program setting
    CNC_UINT8 Volume; // 0 - 255

    // display setting
    //CNC_UINT8 OSDAlpha; // 0 -255
    CNC_UINT32 Brightness; //brightness
    CNC_UINT32 Contrast; // Contrast
    CNC_UINT32 Saturation; //Saturation
    CNC_UINT8 TVOutScale; //  2 auto; 0  4:3;   1 16:9;	 defout:0
    CSTVOUT_MODE TVOutMode;
    CSTVOUT_MODE SDTVOutMode;

    // Output setting
    cncAVVideoWindowFitType_t TVAspectMode;

    CNC_UINT8 TV_system; //0 NTSC,1 PAL

    // Upgrade setting
    CNC_UINT8 firmware_upgrade;//0via usb,1 viaintenet

    // Network Setting
	/*
 	 * Start Obsolete, need to be removed
	 */
    CNC_UINT8 network_aut_get_ip; //0 eth0 auto	, 1 eth0 static , 2 WLAN
    CNC_UINT8 network_aut_get_dns; //0 aut	, 1 static
    CNC_CHAR static_ip_address[50];
    CNC_CHAR static_mask_address[50];
    CNC_CHAR static_gateway_address[50];
    CNC_CHAR static_dns_address[50];
    CNC_UINT8 IsSoftAp;
    CNC_UINT8 IsAvailableSoftAp;
	/*
 	 * End Obsolete, need to be removed
	 */

    CNC_UINT8 networkPreference; //0 eth, 1 wlan
    CNC_UINT8 networkIpPreference; //0 dhcp client, 1 dhcp server, 2 static
    
    // For the Analog TV setting
    CNC_UINT8 encoder_rate; //0  25fps  , 1 30fps
    CNC_UINT8 encoder_bitrate; //0 low  , 1 standard, 2 high
    CNC_UINT8 encoder_gpio;//gpio15
    CNC_UINT8 tunertype; //0 palm  , 1 pldc, 2 ntsc

    // Audio Setting
    CNC_BOOL auto_output_mode; //AS output/HD output //add by guangxi TVOUT
    CSAUD_PCM_CHANNEL audio_pcm_channel; //audio pcm channel

    CNC_SINT32 HDMI_Mode;//0:auto 1:Stereo PCM 2:Multi-channel PCM 3:Dolby Digital 4:Dolby Digital Plus
    CNC_SINT32 SPDIF_Mode;//0:SPDIF PCM 1:SPDIF Dolby Digital
    //Media Settings
    cncAVVideoWindowFitType_t Output_fit_mode;
    CNC_SINT32 Close_Caption;// 1:on  0:off

    /* MediaClt Module */
    CNC_SINT32  Photo_Display;      //0:actual size 1:fit to screen. 
    CNC_SINT32  PhotoShow_Delay;    //in seconds
    CNC_SINT32  PhotoShow_Music;    //0:off 1:on
    CNC_SINT32  minBandwidthAtStart;    /** Minimum bandwidth (in bps) of the stream to be selected at start of playback (seek).  0 -> Least, -1 -> Max, Non-zero +ve ->  Minimum Bandwidth */
    CNC_SINT32  minBufferTime;
    CNC_SINT32  minRebufferTime;
    //language setting
    CNC_SINT32 language;  // 0:English 1:french
} cncSettings_t;

typedef struct
{
    CNC_UINT32 speed;      /* I2C_SPEED_100K 1; I2C_SPEED_400K 2 */
    CNC_UINT8   loop;          /* I2C_ADDR_NOLOOP 0, I2C_ADDR_LOOP 1 */
    CNC_UINT32 subaddr_num;
    CNC_UINT32 write_delayus;
} cncI2CConfig_t;

typedef struct {
    CNC_SINT32 width;
    CNC_SINT32 height;
    CSVID_FRAME_RATE framerate;
    CSTVOUT_HDMI_ASPECT_RATIO aspect_ratio;
    CNC_SINT32 interlaced;
    CNC_SINT32 is_3D;
} cncResolutionDetails_t;

typedef enum {
    LED_STATE_OFF = 0,
    LED_STATE_ON,
    LED_STATE_DELAY_START,
    LED_STATE_STOPPED
} cncLedBlinkState_t;

typedef struct {
    CNC_SINT32 gpio;
    CNC_UINT32 delayStart_ms;
    CNC_UINT32 timeOn_ms;
    CNC_UINT32 timeOff_ms;
    cncLedBlinkState_t current_state;
} cncLedBlinkPattern_t;


typedef CNC_SINT32 (*cncSmartCardStatusNotify_t)( cncSmartCardStatus_t status, cncSmartCardStandard_t standard);

typedef CNC_SINT32 CNC_DEV_EDIDHandle_t;
#define INVALID_EDID_HANDLE (-1)


/**
 * Initialize the cncsystem module
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_System_Init(void);

/**
 * Retrieve the serial number for the device
 * @return 13 character device serial number
 */
char *CNC_System_GetSerialNumber(void);

/**
 * Retrieve a unique device id for the system
 * @return 10 character device id
 */
char *CNC_System_GetDeviceId(void);

/**
 * Retrieve the mac address for the system
 * @return 20 character mac address
 */
char *CNC_System_GetMacAddress(void);

/**
 * Power off the system
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
void CNC_System_PowerOff(void);

/**
 * Power down a component in the system
 * @param powerMask bitmask flag for selecting components to power down or up
 * @param mode either CNC_POWER_SLEEP or CNC_POWER_WAKE
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_System_PowerSetMode(CNC_SINT32 powerMask, cncPowerMode_t mode);

/**
 * Reset the default power mask to be used for CNC_System_StandbyOn and CNC_System_StandbyOff
 * @param powerMask bitmask flag for selecting components to power down or up
 */
void CNC_System_SetActivePowerMask(int powerMask);

/**
 * Default power down for standby mode using the following mask:
    int powerMask = CNC_POWER_CRYPTO +
                    CNC_POWER_CVBS +
                    CNC_POWER_YUV +
                    CNC_POWER_AUDDAC +
                    CNC_POWER_VIB +
                    CNC_POWER_VIDEO +
                    CNC_POWER_AUDIODACIF +
                    CNC_POWER_AUDIOPLL;
 */
void CNC_System_StandbyOn(void);

/**
 * Default power wakeup from standby mode
 */
void CNC_System_StandbyOff(void);

/**
 * Get the current time in the system; based on CPU time ticks
 * @return current system  timestamp
 */
CNC_UINT64 CNC_System_GetTickCount(void);

/**
 * Synchronize the system time using a network ntp server.
 * @param timeserver URL of a network ntp server
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TimeSetup_SyncFromNetwork(char *timeserver);

/**
 * Get the current system time in cncDateTime_t format.
 * @param p_time Current system time
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TimeSetup_GetSysTime(cncDateTime_t *p_time);

/**
 * Set the current system time in cncDateTime_t format.
 * @param p_time New system time
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TimeSetup_SetSysTime(cncDateTime_t *p_time);

/**
 * Get an ASCII representation of the current system time.
 * @return string for current time
 */
char *CNC_TimeSetup_GetStrings(void);

/**
 * Reset the link to timezones to the given path
 * @param path system path name to timezone files
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TimeSetup_SetTimeZone(char *path);

/**
 * Get the current date and time using Offset
 * @param OffSet offset for date and time
 * @param DateTime date/time structure to modify
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TimeSetup_GetCurLocalDataTime(CNC_SINT16 OffSet, cncDateTime_t *DateTime);

/**
 * Default graphics configuration (can be used in CNC_DEV_SetGraphicsConfig).  720P, ARGB888, 32BIT
 */
extern CSOSD_Config defaultGraphicsConfig;

/**
 * Initialize the user interface devices in the system including TV outputs, graphics layers, front panel, gpio
 * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
 */
Result_Type_e CNC_DEV_Init();

/**
 * Retrieves the data structure CNC_Device_Params_t that holds handles to tvout, tvmode, graphics config, and graphics handles
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
cncDeviceContainer_t * CNC_DEV_GetDeviceParams(void);

/**
 * Retrieves a handle for the graphics layer
 * @param index 0 for OSD 0, 1 for OSD 1
 * @return handle to the specified graphics layer
 */
CSOSD_HANDLE CNC_DEV_GetGraphicsHandle(int index);

/**
 * Retrieves a handle for the front panel
 * @return handle to the front panel
 */
CSFP_HANDLE CNC_DEV_GetFrontPanelHandle(void);

/**
 * Retrieves a handle for the TV out device; TVOUT 0 is the HDMI/digital output, TVOUT 1 is the analog output
 * @param index 0 for TVOUT 0, 1 for TVOUT 1
 * @return handle to the specified TV output device
 */
CSTVOUT_HANDLE CNC_DEV_GetTVOutHandle(int index);

/**
 * Retrieves the mode for the TV out device; TVOUT 0 is the HDMI/digital output, TVOUT 1 is the analog output
 * @param index 0 for TVOUT 0, 1 for TVOUT 1
 * @return the mode to the specified TV output device
 */
CSTVOUT_MODE CNC_DEV_GetTVOutMode(int index);

/**
 * Enable the front panel controller (including IR receiver).
 * @param KeyScanOn Set whether the front panel continuously scans keys; generally should be CNC_BOOL_FALSE
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_EnableFrontPanel(CNC_BOOL KeyScanOn);

/**
 * If character-base LED is available on front panel, display string; otherwise ignore
 * @param str String to display (typically 4 characters)
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetLEDDisplay(char *str);

/**
 * Change the Kthread priority of HDMI Driver to RealTime/ Normal priority
 * @param Enable : 1 - RTPriority, 0 - Normal Priority
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_ChangeHdmiKthreadPrioToRealTime(CNC_BOOL Enable);

/**
 * Open the HD TV Output; should be used for processes that will control display output; this function also enables the output
 * And also has the option of Opening TVOUT 0 either with CEC Enabled or Disabled.
 * @param cecEnable :cecEnable -1 Supports CEC -0 Doesn't Support CEC while TVOUT0 Open
 * @param mode Initial Display Resolution
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_OpenHDOutput(CSTVOUT_MODE mode, CNC_SINT32 cecEnable);
Result_Type_e CNC_DEV_OpenHDOutputCheckHpd(CSTVOUT_MODE mode, CNC_SINT32 cecEnable);

/**
 * Set the output mode of the HD TV Out display. If CSTVOUT_MODE is set to TVOUT_MODE_USE_EDID it will use EDID
 * @param mode screen resolution of the TV display
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetHDOutputMode(CSTVOUT_MODE mode);

/**
 * This function informs the user of the current display resolution for the given tvout interface.
 * Will return the same thing as CNC_DEV_GetTVOutMode except for the case of using EDID, in which
 * CNC_DEV_GetTVOutMode will return TVOUT_MODE_USE_EDID and this will return the actual display
 * resolution.
 * @param index this is the tvout interface
 * @return the display resolution
 */
CSTVOUT_MODE CNC_DEV_GetDisplayResolution(int index);

/**
 * Open the SD TV Output (typically CVBS); should be used for processes that will control the analog TV output; this function does NOT enable the output
 * @param mode Initial Display Resolution
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_OpenSDOutput(CSTVOUT_MODE mode);

/**
 * Enable conversion of HD content into the SD output
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_EnableSDOutput(void);

/** Adjust CVBS Overscan Setting with the co-ordinates given as xstart ,xend ,ystart,yend 
 *@param xstart: Co-Ordinate Position Left
 *@param xend  : Co-Ordinate Position Right
 *@param ystart:  Co-Ordinate Position Top
 *@param yend  : Co-Ordinate Position Bottom
 *@param enableFlag, 0 ->disable CVBS overscan adjustment, all coordinate parameters will be ignored  , 1-> Do CVBS Overscan adjustment
 * @return  RESULT_OK on success, RESULT_FAIL on error
*/
Result_Type_e CNC_DEV_SetSDOverScan(int xstart, int xend, int ystart, int yend, CNC_BOOL enableFlag);

/**
 * Disable conversion of HD content into the SD output
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_DisableSDOutput(void);

/**
 * Enable/Disable CVBS Output for the TV out device; TVOUT 1 is the analog output
 * @param  enableFlag - 0 0r 1 to disable/enable respectively
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetCVBSOutput(CNC_BOOL enableFlag);

/**
 * Enable CGMS  (Copy Control for analog output)
 * @param  e_cgms       CGMS options for PAL.
 * @param  e_wss        CGMS options for NTSC.
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_EnableCGMS(cncDeviceCGMS_t e_cgms, cncDeviceWSS_t  e_wss);

/**
 * Disable CGMS  (Copy Control for analog output)
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_DisableCGMS(void);

/**
 * Sets the SD Output Mode (display resolution); should only be called in process that has opened SD output
 * @param mode Display Resolution
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetSDOutputMode(CSTVOUT_MODE mode);

/**
 * Register CEC Callback 
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_RegisterCECNotification();

/**
 * Enable/Disable HDMI Audio Output for the TV out device;
 * @param  b_setHDMIAudioOutput - 0 0r 1 to disable/enable respectively
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetHDMIAudioOutput(CNC_BOOL b_setHDMIAudioOutput);

/**
 * Given a width, height and framerate returns the appropriate TV out mode
 * @param Width Width of the target display
 * @param Height Height of the target display
 * @param Framerate Framerate of the target display
 * @return TV out mode (e.g., 1080P60)
 */
CSTVOUT_MODE CNC_DEV_GetVideoTVMode(int Width, int Height, int Framerate);

/**
 * Retrieves the mode (display rate) and width and height for the output video on CSTVOUT 0 (digital out)
 * @param Width width of the video window
 * @param Height height of the video window
 * @return mode (display rate) of the current output
 */
CSTVOUT_MODE CNC_DEV_GetScreenVideoSize(int *Width, int *Height);

/**
 * Retrieves the width, height and aspect ratio for the HD output video on current TV Mode setting
 * @param mode TVOUT mode
 * @param width [out] width of the video window
 * @param height [out] height of the video window
 * @param aspectRatio [out] aspect ratio
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetScreenSizeFromMode(CSTVOUT_MODE mode, CNC_SINT32 *width, CNC_SINT32 *height, CNC_FLOAT32 *aspectRatio);

/** 
 * Retrieves framerate value for CSVID_FRAMERATE enumeration tag
 * @param fr [in] FRAMERATE mode
 * @param value [out] real framerate value
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetFrameRateFromMode( CSVID_FRAME_RATE fr, CNC_FLOAT32 *value);

/** 
 * Retrieves framerate tag from FRAMERATE value
 * @param value [in] FRAMERATE value
 * @param fr [out] framerate enumeration tag
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetFrameRateFromValue( CNC_FLOAT32 value, CSVID_FRAME_RATE *fr);

/** 
 * Retrieves sample rate value for CSAUD_SAMPLE_RATE enumeration tag
 * @param fr [in] SAMPLERATE mode
 * @param value [out] real samplerate value
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetSampleRateFromMode( CSAUD_SAMPLE_RATE fr, CNC_UINT32 *value);

/** 
 * Retrieves samplerate tag from SAMPLERATE value
 * @param value [in] SAMPLERATE value
 * @param fr [out] samplerate enumeration tag
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetSampleRateFromValue( CNC_UINT32 value, CSAUD_SAMPLE_RATE *fr);

/**
 * Retrieves the width and height of the OSD output
 * @param Width integer pointer where the Width will be written
 * @param Height integer pointer where the Height will be written
 * @return none
 */
void CNC_DEV_GetScreenOSDSize(int *Width, int *Height);

/**
 * Set the current virtual (/dev/fb2) OSD size.
 * @param width [out] width
 * @param height [out] height
 */
void CNC_DEV_SetOsdVirtualResolution(CNC_SINT32 width, CNC_SINT32 height);

/**
 * Get the current virtual (/dev/fb2) OSD size.
 * @param width [out] width
 * @param height [out] height
 */
void CNC_DEV_GetOsdResolution(CNC_SINT32 *width, CNC_SINT32 *height);

/**
 * Get the Resolution number from a null terminated string.
 * @param resolution_name [in] resolution name 
 * @return  resolution number or -1 if not found
 */
CSTVOUT_MODE CNC_DEV_GetResolutionByName(const char *resolution_name);

/**
 * Get the Resolution name from its tvout mode
 * @param mode [in] resolution tvout mode 
 * @return  null terminated string with the name of the resolution or NULL if not found.
 */
char* CNC_DEV_GetResolutionName(CSTVOUT_MODE mode);

/**
 * Get the Resolution characteristics (width, height, framerate, interlaced and if it is 3D or not) from the TVOUT_MODE.
 * @param mode [in] resolution
 * @param resolution [out] resolution details structure.
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetResolutionDetails(CSTVOUT_MODE mode, cncResolutionDetails_t* resolution);

/**
 * Get the Resolution mode from its characteristics (width, height, framerate, interlaced and if it is 3D or not)
 * @param in_resolution [in] resolution characteristics
 * @param out_mode [out] corresponding tvout_mode
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_GetTvOutMode(cncResolutionDetails_t* in_resolution, CSTVOUT_MODE* out_mode);

/**
 * Reset the devices
 * @return none
 */
void CNC_DEV_Reset(void);

/**
 * Specialty routine for cases where the TV Out should be reset;  for 1080P30, will flip from 1080P25 to 1080P30
 * @return none
 */
void CNC_DEV_ResetTVOut(void);

/**
 * Register Callback events for the different possible HDMI events
 * @return none
 */
void CNC_DEV_RegisterCallback(CNC_DEV_CallbackFunction_t func);

/**
 * Unregister Callback events for the different possible HDMI events
 * @return none
 */
void CNC_DEV_UnregisterCallback();

/**
 * This function returns the filtered EDID. This is the EDID without the resolutions we can not support
 * @param edid_len the length of the EDID buffer returned will be given here
 * @return poiter to the filtered EDID.
 */
const unsigned char* CNC_DEV_GetFilteredHDMIEdid(CNC_SINT32* edid_len);

/**
 * This function returns the EDID.
 * @param edid_len the length of the EDID buffer returned will be given here
 * @return poiter to the EDID.
 */
const unsigned char* CNC_DEV_GetHDMIEdid(CNC_SINT32* edid_len);

/**
 * This function informs the user if the given resolution is supported by the TV or not, based on the EDID info
 * @param mode resolution that we want to check for support
 * @return 1 or 0 if the resolution is supported by the TV or not
 */
CNC_SINT32 CNC_DEV_IsResSupported(CSTVOUT_MODE mode);

/**
 * This function informs the user if the given audio mode is supported by the TV or not, based on the EDID info
 * @param mode audio mode that we want to check for support
 * @return 1 or 0 if the audio mode is supported by the TV or not
 */
CNC_SINT32 CNC_DEV_IsAudioModeSupported(CSAUD_HDMI_MODE mode);

/**
 * This function returns the plug status: 0 unplugged, 1 plugged.
 * @return 0 unplugged, 1 plugged.
 */
CNC_SINT32 CNC_DEV_GetHDMIPlugStatus(void);

/**
 * This function triggers an HDCP 1.x authentication.
 * @return void.
 */
void CNC_DEV_TriggerHDCP1XAuth(void);

/**
 * This function returns the HDCP 1.x downstream info
 * @return pointer to an area of memory with the downstream info
 */
const void* CNC_DEV_GetHDCP1xDownstreamInfo(void);

/**
 * This function allocates howmany EDID structures for use with merging of EDIDs
 * @param howmany How many EDID structures to allocate
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_InitEDIDs(CNC_UINT32 howmany);


/**
 * This function adds an EDID to the merge list and returns a handle
 * @param handle Updated handle for use in future references to this EDID
 * @param edid raw EDID bytes to be added. Application should free() this pointer when done.
 * @param len length of EDID 
 * @param local Set to CNC_BOOL_FALSE for remote sinks, and CNC_BOOL_TRUE for a local sink (splitter)
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_AddEDID(CNC_DEV_EDIDHandle_t *handle, CNC_UINT8 *edid, CNC_UINT32 len, CNC_BOOL local);

/**
 * This function removes an EDID from the merge list
 * @param handle EDID reference to be removed
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_RemoveEDID(CNC_DEV_EDIDHandle_t handle);

/**
 * This function returns whether the EDID supports 3D
 * @param handle EDID reference 
 * @param supports3D updated bool indicating whether EDID supports 3D or not 
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_EDIDSupports3D(CNC_DEV_EDIDHandle_t handle, CNC_BOOL *supports3D);

/**
 * This function merges the EDID list created by CNC_DEV_AddEDID(),
 * by updating it to the least common denominator of video timings and audio formats.
 * It allows 3D if a local EDID has 3D, or if no local EDID is present and one or more
 * other EDIDs have 3D.
 * @param len length of EDID. It is zero if there is an error.
 * @return buffer with resulting EDID on success, NULL on error. The resulting EDID should be free()'d by the caller.
 */
CNC_UINT8 *CNC_DEV_MergeEDIDs(CNC_UINT32 *edid_len);

/**
 * Set the graphics configuration
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @param config structure including mode, color depth and color format
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetGraphicsConfig(CSOSD_LAYER layer, CSOSD_Config config);

/**
 * update the graphics configuration. Need call this after change the tv out mode.
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_ResetGraphicsConfig(CSOSD_LAYER layer);

/**
 * Enable (display) the graphics layer
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_EnableGraphics(CSOSD_LAYER layer);

/**
 * Disable (don't display) the graphics layer
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_DisableGraphics(CSOSD_LAYER layer);

/**
 * Open the second layer of graphics; kernel must be configured for two graphics layers
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_OpenSecondGraphicsLayer(void);

/**
 * Set the key color mask to be used in the graphics layer
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @param red   Red value
 * @param green Green valuke
 * @param blue  Blue value
 * @param range Window around values; mask RGB +/- this value
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetGraphicsKeyColor(CSOSD_LAYER layer, char red, char green, char blue, int range);

/**
 * Sets the alpha transparency of the graphics layer
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @param alpha alpha transparency value: can be between 0 and 0xFF
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_SetGraphicsAlpha(CSOSD_LAYER layer, CNC_SINT32 alpha);

/**
 * Enable the Graphics Key Color Masking
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_EnableGraphicsKeyColor(CSOSD_LAYER layer);

/**
 * Disable the Graphics Key Color Masking
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_DisableGraphicsKeyColor(CSOSD_LAYER layer);

/**
 * Dynamically enable graphics layer
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @return
 */
Result_Type_e CNC_DEV_EnableGraphics(CSOSD_LAYER layer);

/**
 * Dynamically disable graphics layer
 * @param layer The graphics layer to enable; should be either CSOSD_LAYER_0 or CSOSD_LAYER_1
 * @return
 */
Result_Type_e CNC_DEV_DisableGraphics(CSOSD_LAYER layer);

/**
 * Configure if cnc_dev_cec will identify as the active source
 * @param active
 * @return
 */
Result_Type_e CNC_DEV_CEC_ReportActive(CNC_BOOL active);

/**
 * Set the cec name of the device
 * @param osdName If the string size if greater than 14 will be truncated
 * @return
 */
Result_Type_e CNC_DEV_CEC_OSDName(const CNC_CHAR *osdName);

/**
 * Get the current system settings
 * @param SettingData the return structure with the current system settings
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Setting_GetSetting(cncSettings_t *SettingData);

/**
 * Save the current system settings
 * @param SettingData The structure containing the values for the system settings to save
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Setting_SaveSetting(cncSettings_t *SettingData);

/**
 * Initialize the settings module.
 *
 * This function should be called before initializing the settings.
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Setting_Init(void);

/**
 * Load the system settings from storage or default if first load.
 *
 * This function should be called during initialization of the system.  It reads the stored
 * system settings; if there are no stored settings, it will load the default settings and
 * store them.
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Setting_InitSetting(void);

/**
 * Load the default system settings
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Setting_LoadDefaultSettingData(void);

Result_Type_e CNC_Setting_ReadSettingFromE2PROM(void);
Result_Type_e CNC_Setting_WriteSettingToE2PROM(void);


/**
 * Set the EEPROM for settings to a virtual filename; for systems that don't have hardware EEPROM
 * @param filename full path name of the filename for storing virtual EEPROM data
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_EEPROM_SetVirtualFile(char *filename);

/**
 * Get the size of the EEPROM
 * @param size integer pointer in which the EEPROM size is written
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_EEPROM_GetInfo(CNC_SINT32 *size);

/**
 * Read data from the EEPROM
 * @param offset_addr read byte offset from the base address of the EEPROM
 * @param buf_addr pointer to the data in which the read data will be returned
 * @param len length in bytes of the data to read
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
CNC_SINT32 CNC_EEPROM_Read(CNC_SINT32 offset_addr, CNC_UINT8 *buf_addr, CNC_SINT32 len);

/**
 * Write data to the EEPROM
 * @param offset_addr read byte offset from the base address of the EEPROM
 * @param buf_addr pointer to the data to write to the EEPROM
 * @param len length in bytes of the data to write
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
CNC_SINT32 CNC_EEPROM_Write(CNC_SINT32 offset_addr, const CNC_UINT8 *buf_addr, CNC_SINT32 len);

/**
 * Initialize the GPIO module
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_GPIO_Init(void);

/**
 * Terminate the GPIO module
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_GPIO_Final(void);

/**
 * Read a GPIO port
 * @param IO_Port number for the GPIO port
 * @param bit pointer to the data read from the port, will be 0 or 1
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_GPIO_Read(CNC_UINT8 IO_Port, CNC_UINT8 *bit);

/**
 * Write a GPIO port
 * @param IO_Port number for the GPIO port
 * @param bit data to write to the port, should be 0 or 1
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_GPIO_Write(CNC_UINT8 IO_Port, CNC_UINT8 bit);

/**
 * Set a GPIO port to 1
 * @param IO_Port number for the GPIO port
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_GPIO_SetBit(CNC_UINT8 IO_Port);

/**
 * Set a GPIO port to 0
 * @param IO_Port number for the GPIO port
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_GPIO_ClearBit(CNC_UINT8 IO_Port);

/**
 * Get size of the flash
 * @param flash_size integer pointer in which to write the size
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Flash_GetInfo(CNC_SINT32 *flash_size);

/**
 * Read data from the Flash
 * @param flash_offset read byte offset from the base address of the Flash
 * @param buff_addr pointer to the data in which the read data will be returned
 * @param len length in bytes of the data to read
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
CNC_SINT32 CNC_Flash_Read(CNC_SINT32 flash_offset, CNC_UINT8 *buff_addr,  CNC_SINT32 len);

/**
 * Write data to the Flash (blocking)
 * @param flash_addr read byte offset from the base address of the EEPROM
 * @param buf_addr pointer to the data to write to the EEPROM
 * @param len length in bytes of the data to write
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
CNC_SINT32 CNC_Flash_Write(CNC_SINT32 flash_addr, const CNC_UINT8 *buf_addr, CNC_SINT32 len);

/**
 * Erase flash section (usually at least 64K)
 * @param flash_offset byte offset to start erasing
 * @param len length in bytes to erase
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Flash_Erase(CNC_SINT32 flash_offset, CNC_SINT32 len);

/**
 * Open the I2C module at the give address
 * @param chipaddress address for the I2C
 * @return handle to the I2C location
 */
CNC_HANDLE CNC_I2C_Open(CNC_WORD chipaddress);

/**
 * Close the I2C module at the give address
 * @param handle handle for the I2C
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_I2C_Close(CNC_HANDLE handle);

/**
 * Read data from the I2C
 * @param handle handle for the I2C
 * @param subaddr address at which to read the data
 * @param buffer pointer to the memory into which to write the byte data
 * @param num amount of data to read
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_I2C_Read(CNC_HANDLE handle, CNC_WORD subaddr, CNC_SINT8 *buffer, CNC_UINT32 num);

/**
 * Write data to the I2C
 * @param handle handle for the I2C
 * @param subaddr address at which to write the data
 * @param buffer pointer to the data to write
 * @param num amount of data to write
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_I2C_Write(CNC_HANDLE handle, CNC_WORD subaddr, CNC_SINT8 *buffer, CNC_UINT32 num);

/**
 * Get the configuration of the I2C including speed, loop type, address, and write delay
 * @param handle handle for the I2C
 * @param i2c_config pointer to the CNC_I2C_Config_t structure to return the data
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_I2C_GetConfig(CNC_HANDLE handle, cncI2CConfig_t * i2c_config);

/**
 * Set the configuration of the I2C including speed, loop type, address, and write delay
 * @param handle handle for the I2C
 * @param i2c_config pointer to the CNC_I2C_Config_t structure with the configuration information
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_I2C_SetConfig(CNC_HANDLE handle, cncI2CConfig_t * i2c_config);


/**
 * Initialize the Smart Card interface and provide a callback for status changes
 * @param func callback function for status changes
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_SCard_Init(cncSmartCardStatusNotify_t func);

/**
 * Reset the Smart Card interface and provide a callback for status changes
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_SCard_Reset(char* pbyATR, char* pbyLen);

/**
 * Transfer data to the SmartCard.
 * @param message byte message
 * @param message_size size of message
 * @param response_data [out] byte response
 * @param response_size [out] size of response
 * @return
 */
Result_Type_e CNC_SCard_TransferData( CNC_UINT8*  message,  CNC_UINT8  message_size, CNC_UINT8*  response_data,  CNC_UINT8*  response_size);

/**
 * Get the current status of the smart card.
 * @param scardstatus [out] card status
 * @return
 */
Result_Type_e CNC_SCard_GetStatus(cncSmartCardStatus_t* scardstatus) ;

/**
 * Get the ATR value from the smart card.
 * @param ATR [out] ATR value
 * @return
 */
Result_Type_e CNC_SCard_GetATR(CNC_UINT8 ATR[20]) ;

/**
 * Get current resolution mode .
 * @return string with value of resolution
 */
CNC_CHAR* CNC_DEV_GetResolution(void);

/**
 * Get support tv outpute mode IDs .
 * @return numbers of support TV modes
 */
int CNC_DEV_GetTVoutModeID(int outmode[TVOUT_MODE_MAX]);

/**
 * Start a LED blinking pattern.
 * @param led_pattern [in] Led blinking pattern
 * @param leds [in] Number of leds to handle
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_StartLEDPattern(cncLedBlinkPattern_t* led_pattern, int leds);

/**
 * Stop a LED blinking pattern. LEDs will stop in whatever state they are at.
 * If you want them to stop with a certain value, just use CNC_DEV_StartLEDPattern
 * with ontime or offtime set to 0
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_DEV_StopLEDPattern(void);

#ifdef __cplusplus
}
#endif

#endif //__CNC_SYSTEM_H__
