/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
/**
 * @file cnc_util.h
 * @ingroup core
 * The CNCUTIL module provides a set of utilities for OS and Time operations
 */

#ifndef __CNC_UTIL_H__
#define __CNC_UTIL_H__

#include <errno.h>
#include "cnc_common_typedef.h"
#include "cnc_os.h"
#include "popen_noshell.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Get the CRC of a buffer of data
 * @param buf pointer to the data buffer
 * @param len length of the data buffer (in bytes)
 * @return computed CRC value
 */
CNC_UINT32 CNC_UTIL_CRC(CNC_UINT8* buf, CNC_UINT32 len);

/**
 * Compare to universal time code (UTC) values
 * @param time1 the first UTC value
 * @param time2 the second UTC value
 * @return 1 if matches, 0 if doesn't match, -1 if error
 */
CNC_SINT32  CNC_UTIL_CmpUTC(cncUTC_t *time1, cncUTC_t *time2);

/**
 * Add time to a UTC time valkue
 * @param time1 the original UTC time value
 * @param hh the number of hours to add
 * @param mm the number of minutes to add
 * @param ss the number of seconds to add
 * @param time2 the new UTC time value
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e  CNC_UTIL_AddTimeToUTC(cncUTC_t *time1, CNC_UINT8 hh, CNC_UINT8 mm, CNC_UINT8 ss, cncUTC_t *time2);

/**
 * Convert an MJD time value to year/month/day/weekday
 * @param mjd the MJD time value
 * @param year the year in the MJD time value
 * @param month the month in the MJD time value
 * @param day the day of the month in the MJD time value
 * @param weekday the day of the week from the MJD time value
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e  CNC_UTIL_CnvMjdToDate(CNC_UINT16 mjd, CNC_UINT16 *year, CNC_UINT8 *month, CNC_UINT8 *day, CNC_UINT8 *weekday);

/**
 * Convert a year/month/day/weekday to an MJD value
 * @param year the year in the date
 * @param month the month in the date
 * @param day the day of the month in the date
 * @return calculated MJD value
 */
CNC_UINT16 CNC_UTIL_CnvDateToMjd(CNC_UINT16 year, CNC_UINT8 month, CNC_UINT8 day);

/**
 * Convert a BCD encoded value to not value
 * @param bcd BCD encoded value
 * @return normal value
 */
CNC_UINT8 CNC_UTIL_CnvBcdToVal(CNC_UINT8 bcd);

/**
 * Convert a normal value to a BCD encoded value
 * @param val normal value
 * @return BCD encoded value
 */
CNC_UINT8 CNC_UTIL_CnvValToBcd(CNC_UINT8 val);

/**
 * Calculate the local time into UTC time
 * @param SummerTime daylight savings value
 * @param Gmt GMT time value
 * @param Offset time offset
 * @param Local [out] UTC time
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_UTIL_CalcLocalUTCTime(CNC_UINT8 SummerTime,  cncUTC_t const * const Gmt, CNC_SINT16  Offset, cncUTC_t *Local);

/**
 * Get the local date time in cncDateTime_t format
 * @param Time UTC time
 * @param OffSet time offset
 * @param DateTime [out] date/time
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_UTIL_GetLocalDataTime(cncUTC_t *Time, CNC_SINT16 OffSet, cncDateTime_t *DateTime);

/**
 * Special purpose string to value conversion; not for general use
 * @param str string value
 * @return signed integer value
 */
CNC_SINT32 CNC_UTIL_CnvStrToVal(CNC_UINT8 *str);

/**
 * Special purpose value to string conversion; not for general use
 * @param val signed integer value
 * @param str string value
 * @return  RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_UTIL_CnvValToStr(CNC_SINT32 val, CNC_UINT8 *str);

/**
 * Get the current system tick count
 * @return current system tick count
 */
CNC_UINT32 CNC_UTIL_GetTickCount(void);

/**
 * Calculate daylight savings value
 * @param time current time
 * @return new hour based on daylight savings
 */
CNC_UINT8 CNC_UTIL_CalcSummerTime(cncDateTime_t *time);

/**
 * Special purpose string function.
 * @param str input string
 * @param c char
 * @return last index of character in string
 */
CNC_SINT32 CNC_UTIL_strcchr(char* str, char c);

void CNC_UTIL_Print(const char *format, ...) 
    __attribute__ ((format (printf, 1, 2)));

int CNC_UTIL_ShellCommand( char *format, ... )
    __attribute__ ((format (printf, 1, 2)));

/* returns pointer to buf, allocated if buf was NULL when called */
#define LARGE_CMD_BUF_SIZE 12000
#define CNC_UTIL_MAX_CMD 4000
char* CNC_UTIL_ShellCommandOutput( size_t bufSize, char *buf, char *format, ... ) __attribute__ ((format (printf, 3, 4)));

int CNC_UTIL_system(const char *command);
int CNC_UTIL_pclose(FILE *fp);
FILE *CNC_UTIL_popen(const char *command, const char *type);

#ifdef __cplusplus
}
#endif

/**
 *  Get a line from buffer 'pp_inBuf' to 'p_outLineBuf'
 *  @param [in,out]  pp_inBuf
 *                     Input buffer reference. This will be moved with amount of bytes used.
 *  @param [in,out]  p_inBufLen
 *                     Input buffer length. This will be updated with the amount of bytes used.
 *  @param [out]     pp_outLineBuf
 *                     "Output buffer where one line of data is stored with out '\r' and '\n' character."
 *  @param [out]     p_lineBufLen
 *                     This will be updated with size of the line including '\0' character.
 *  @return          > 0   --> Number of bytes written into output buffer (p_outLineBuf)
 *                   == 0  --> End of Input Buffer (No data available in input buffer for reading when this API called)
 *                   < 0   --> Error (Ex: Invalid parameters)
 */
CNC_SINT32 CNC_UTIL_bfindline(CNC_CHAR    **pp_inBuf, CNC_UINT32   *p_inBufLen, CNC_CHAR    **pp_outLineBuf, CNC_UINT32   *p_lineBufLen);


char * CNC_UTIL_StrCaseStr(const char *s, const char *find);

char* CNC_UTIL_StrNCpy(char *dest, const char *src, size_t n);

void CNC_UTIL_Free(void **pptr);

CNC_SINT32 CNC_UTIL_GetCurTimeSinceEpoch(CNC_SINT64  *pCurTimeInMicroSec);
#endif //__CNC_UTIL_H__
