/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csaud.h
 * @ingroup csapi
 * Low Level Audio API
 *
 * The CSAUD API provides programming interfaces for the audio decoding functions of Cavium SoC chipsets.
 * The CSAUD API supports decoding of multiple audio encoding standards and
 * provides necessary calling interfaces for the applications to control the input/output interfaces and
 * the audio decoder status.
 */
#ifndef  __CSAPI_AUD_H__
#define  __CSAPI_AUD_H__

#include "global.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void *CSAUD_HANDLE;

/**
*Specify an audio device ID.
* Support AUD_DEV_0 \n
* @see CSAUD_Open()
*/
typedef enum
{
	AUD_DEV_0 = 0,///<The device ID is 0.
	AUD_DEV_1,///<The device ID is 1.
	AUD_DEV_2,
	AUD_DEV_MAX///<Unused.
} CSAUD_DEV;

/**
*Contains decoding info.
* @see CSAUD_DecodingInfo()
*
*/
typedef struct tagAUD_DecodingInfo
{
	unsigned int skipped_samples;      /// PCM Skipped samples
	unsigned int repeated_samples;     /// PCM Repeated samples
	unsigned int decoding_errors;      /// Decoding Errors, not available for LPCM
    unsigned int silence_frames;
} CSAUD_DecodingInfo;

/**
*Set the volume of each channel.
* @see CSAUD_SetVolume()
*
*/
typedef struct tagAUD_Volume
{
	unsigned char front_left;///<Volume of the front left channel.
	unsigned char front_right;///<Volume of the front right channel.
	unsigned char rear_left;///<Volume of the rear left channel.
	unsigned char rear_right;///<Volume of the rear right channel.
	unsigned char center;///<Volume of the center channel.
	unsigned char lfe;///<Volume of the low frequency affection channel.
	unsigned char surround_left;///<Volume of the surround left channel.
	unsigned char surround_right;///<Volume of the surround right channel.
} CSAUD_Volume;

/**
*Define the supported stream types.
* Encoder only supports G729/726/723/711 and MP2ENC.Decoder can support all stream type
* @see CSAUD_SetCodecType function
*
*/
typedef enum
{
    AUD_STREAM_TYPE_UNKNOWN = -1,
	AUD_STREAM_TYPE_MPA = 0,///<Stream type for MPEG1.2,2.5 audio.
	AUD_STREAM_TYPE_AC3 = 1,///<Stream type for Dolby AC-3 audio.
	AUD_STREAM_TYPE_DTS = 2,///<Stream type for DTS.
	AUD_STREAM_TYPE_LPCM = 3,///<Stream type for LPCM.
	AUD_STREAM_TYPE_AAC = 4,///<Stream type for MPEG2, 4 AAC audio.
	AUD_STREAM_TYPE_AAC_LATM = 5,///<Stream type for HEAAC
	AUD_STREAM_TYPE_AIB = 6,///<Stream type for Audio input block.
	AUD_STREAM_TYPE_G729 = 7,///<Stream type for G729 audio.
	AUD_STREAM_TYPE_G723 = 8,///<Stream type for G723 audio.
	AUD_STREAM_TYPE_G726 = 9,///<Stream type for G726
	AUD_STREAM_TYPE_G711 = 10,///<Stream type for G711
	AUD_STREAM_TYPE_MP2ENC = 11,///<Stream type for MP2ENC
	AUD_STREAM_TYPE_FLAC = 12,///<Stream type for FLAC
	AUD_STREAM_TYPE_DTSWAV = 13,///<Stream type for DTSWAV
	AUD_STREAM_TYPE_REAL_COOK = 14,///<Stream type for real cook
	AUD_STREAM_TYPE_WMA = 15,///<Stream type for WMA
	AUD_STREAM_TYPE_OGG_VORBIS = 16, ///<Stream type for Ogg Vorbis
	AUD_STREAM_TYPE_APE = 17,///<Stream type for APE
	AUD_STREAM_TYPE_REAL_ATRC = 18,///<Stream type for real atrc
	AUD_STREAM_TYPE_AV3_AASF = 19,///<Stream type for AV3 AASF
	AUD_STREAM_TYPE_AV3_AATF = 20,///<Stream type for AV3 AATF
	AUD_STREAM_TYPE_MS_WMAPRO = 21,///<Stream type for MS WMAPRO
	AUD_STREAM_TYPE_PCMENC = 22,///<Stream type for PCMENC
	AUD_STREAM_TYPE_REAL_SIPR = 23,///<Stream type for MS WMAPRO
	AUD_STREAM_TYPE_AMR=24,///<Stream type for AMR
	AUD_STREAM_TYPE_DOLBY = 25,
	AUD_STREAM_TYPE_LPCM_DVD=26,///<Stream type for LPCM_DVD
	AUD_STREAM_TYPE_PCM_MULAW = 27,///<Stream type for PCM_MULAW
	AUD_STREAM_TYPE_PCM_ALAW = 28,///<Stream type for PCM_ALAW
	AUD_STREAM_TYPE_ADPCM_G726 = 29,///<Stream type for ADPCM_G726
	AUD_STREAM_TYPE_TRUEHD = 30,///<Stream type for Dolby TrueHD
	AUD_STREAM_TYPE_DTS_HD = 31,///<Stream type for DTS HD MA
	AUD_STREAM_TYPE_AAC_ENC = 32,///<Stream type for AAC Encoder
	AUD_STREAM_TYPE_DOLBY_COMPRESSED = 33, ///<Stream type for compressed Dolby Stream>
	AUD_STREAM_TYPE_MAX_NUM,
} CSAUD_STREAM_TYPE;

/**
*Define the sampling rate of the input stream.
* MP2ENC only supports 48kHZ,44kHZ and 32kHZ.
* @see CSAUD_SetSampleRate
*/
typedef enum
{
	AUD_SAMPLE_RATE_192KHZ = 0,
	AUD_SAMPLE_RATE_176KHZ,
	AUD_SAMPLE_RATE_128KHZ, ///<The sampling rate is 128 kHz.
	AUD_SAMPLE_RATE_96KHZ, ///<The sampling rate is 96 kHz
	AUD_SAMPLE_RATE_88_200KHZ, ///<The sampling rate is 88.2 kHz.
	AUD_SAMPLE_RATE_64KHZ,///<The sampling rate is 64 kHz.
	AUD_SAMPLE_RATE_48KHZ,///<The sampling rate is 48 kHz.
	AUD_SAMPLE_RATE_44_100KHZ,///< The sampling rate is 44.1 kHz.
	AUD_SAMPLE_RATE_32KHZ, ///<The sampling rate is 32 kHz.
	AUD_SAMPLE_RATE_24KHZ,///<The sampling rate is 24 kHz.
	AUD_SAMPLE_RATE_22_050KHZ, ///<The sampling rate is 22.05 kHz.
	AUD_SAMPLE_RATE_16KHZ,///<The sampling rate is 16 kHz.
	AUD_SAMPLE_RATE_12KHZ, ///<The sampling rate is 12 kHz.
	AUD_SAMPLE_RATE_11_025KHZ,///<The sampling rate is 11.025 kHz.
	AUD_SAMPLE_RATE_8KHZ///<The sampling rate is 8 kHz.
} CSAUD_SAMPLE_RATE;


/**
*Define the configuration of the bit rate for encoder.
* @see CSAUD_BitRate()
*/
typedef enum
{
    AUD_BIT_RATE_0k = 0,///<The bit rate of input stream, firmware needs this parameter to set bit rate of encoder and decoder.
    AUD_BIT_RATE_32k , ///<only mono
    AUD_BIT_RATE_48k , ///<only mono
    AUD_BIT_RATE_56k , ///<only mono
    AUD_BIT_RATE_64k , ///<mono and stereo
    AUD_BIT_RATE_80k ,///<only mono
    AUD_BIT_RATE_96k , ///<mono and stereo
    AUD_BIT_RATE_112k ,///< mono and stereo
    AUD_BIT_RATE_128k , ///<mono and stereo
    AUD_BIT_RATE_160k ,///< mono and stereo
    AUD_BIT_RATE_192k ,///< mono and stereo
    AUD_BIT_RATE_224k ,///< only stereo
    AUD_BIT_RATE_256k ,///<only stereo
    AUD_BIT_RATE_320k ,///<only stereo
    AUD_BIT_RATE_384k ,///< only stereo
}AUD_BIT_RATE;

/**
*Define the configuration of the input mode.
* @see CSAUD_SetInputMode()
*/
typedef enum
{
	AUD_INPUT_NOBLOCK = 0,
	AUD_INPUT_BLOCK
} CSAUD_INPUT_MODE;

/**
* Error code.
* @see CSAUD_GetErrCode()
*/
typedef enum
{
	AUD_NO_ERROR = 0,
	AUD_ERROR_OPEN_FAILED,///<open filed
	AUD_ERROR_IOCTL_FAILED,	///<ioctl filed
	AUD_ERROR_INVALID_PARAMETERS,///<Bad parameter passed
	AUD_ERROR_UNKNOWN_DEVICE,	///<Unknown device name
	AUD_ERROR_DEVICE_BUSY,///<Device is currently busy
	AUD_ERROR_INVALID_HANDLE,	///<Handle is not valid
	AUD_ERROR_ALREADY_INITIALIZED,///< Device already initialized
	AUD_ERROR_NOT_INITIALIZED,	///< Device not initialized
	AUD_ERROR_NO_MEMORY,	///<Device not enough memory
	AUD_ERROR_MEMORY_MAP    ///< Device memory map error
} CSAUD_ErrCode;

/**
* Define audio buffer status
* @see CSAUD_GetBufferStatus
*/
typedef enum
{
	AUD_BUFFER_EMPTY = 0,///<Audio buffer is empty.
	AUD_BUFFER_FULL = 1,///<Audio buffer is full.
	AUD_BUFFER_NORMAL = 2///<Audio buffer is normal.
} CSAUD_BUFFER_STATUS;

/**
* Define audio input source mode when audio codec was AIB.
* @see CSAUD_SetInputSource
*/
typedef enum{
    AUD_INPUT_SOURCE_ADC = 0,	///<that is AIB
    AUD_INPUT_SOURCE_DDR = 1,	///< that is from DDR.
}CSAUD_INPUT_SOURCE;

/**
* Define audio Equalizer type.
* @see CSAUD_SetEQtype

*/
typedef enum _DRIVER_EUQALIZER_TYPE
{
    NO_EQ           = 0,///<Disable EQ
    EQ_POP          = 1,///<Pop EQ
    EQ_HOME_THETRER = 2,///<Home thetrer EQ
    EQ_ROCK         = 3,///<Rock EQ
    EQ_CLASSIC      = 4,///<Classic EQ
    EQ_RESERVE1     = 5,///<Reserve
    EQ_RESERVE2     = 6,///<Reserve
    EQ_RESERVE3     = 7,///<Reserve
}DRIVER_EQUALIZER_TYPE;

/**
* Define struct of audio encoder stuff.
* @see CSAUD_SetEQtype
*/

typedef struct tagAUD_Encoder_Init_Params
{
	unsigned int decoder_number;///< Specify how many decoder user needed.
	CSAUD_DEV encoder_dev;///< Specify encoder device ID.
	CSAUD_INPUT_SOURCE input_mode;///< Specify input data source.
	CSAUD_SAMPLE_RATE sample_rate;///< Specify inout data sample rate.
}CSAUD_Encoder_Init_Params;

/**
* Define audio firmware version.
*/
typedef struct tagAUD_FIRMWARE_VERSION
{
	CSAUD_STREAM_TYPE type; ///<Invalid.
	unsigned int year;///< year.
	unsigned int month;///<month.
	unsigned int day;///<day.
	unsigned int time;///<Invalid.
}CSAUD_Firmware_Version;

/**
* Define dolby decoding parameters.
*/
typedef struct tagDOLBY_DECODING_PARA
{
	CSAUD_HANDLE handle; ///<audio handle.
	unsigned int channel_number;///< output channel number.
}DOLBY_DECODING_PARA;

/* common interface */

/**
 *Initialize the audio decoder.
 *
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note    Initialize the audio decoder or encoder. set sample rate at 48KHZ.
 */
CSAPI_RESULT CSAUD_Init(void);

/**
 *Stop and reset audio decoder
 */
void CSAUD_Term(void);

/**
 *Open an appointed audio decoding channel.
 *
 * @param	dev  Device ID of the opened audio decoder. In/Out : In
 * @return  CSAUD_HANDLE Pointer to an opened audio decoding device.
 * @see   CSAUD_Close()
 */
CSAUD_HANDLE CSAUD_Open(CSAUD_DEV dev);

/**
 *Open an appointed audio decoding channel.
 *
 * @param	handle  An opened audio handle (from the CSAUD_Open() call). In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @see	CSAUD_Open()
 */
CSAPI_RESULT CSAUD_Close(CSAUD_HANDLE handle);

/**
 *Start decoding.
 *
 * @param	handle  An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @see	CSAUD_Stop()
 */
CSAPI_RESULT CSAUD_Play(CSAUD_HANDLE handle);

/**
 *Stop decoding.
 *
 * @param	handle  An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @see	CSAUD_Play()
 */
CSAPI_RESULT CSAUD_Stop(CSAUD_HANDLE handle);

/**
 *Pause decoding.
 *
 * @param	handle  An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	Pause decoding.
 * @see	CSAUD_Resume()
 */
CSAPI_RESULT CSAUD_Pause(CSAUD_HANDLE handle);

/**
 *Resume decoding.
 *
 * @param	handle  An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	Resume decoding from pause status.
 * @see	    CSAUD_Pause()
 */
CSAPI_RESULT CSAUD_Resume(CSAUD_HANDLE handle);

/**
 *Set the type of input mode.
 *
 * @param	handle  An opened audio handle . In/Out : In
 * @param	input_mod  The mode of input, AUD_INPUT_NOBLOCK, AUD_INPUT_BLOCK. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	The user must set the input mode before audio decoding. The audio driver needs a correct input mode to inject data.
 * @see	CSAUD_INPUT_MODE
 */
CSAPI_RESULT CSAUD_SetInputMode(CSAUD_HANDLE handle, CSAUD_INPUT_MODE input_mod);

 /**
 *Set the different volumes of different channels
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	audio_bitrate Audio stream bitrate. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	MP2ENC encoder stream only support 64, 96, 112, 128, 160, 192, 224, 256, 320, 384kbps bit rate.
 */

CSAPI_RESULT CSAUD_SetBitRate(CSAUD_HANDLE handle, AUD_BIT_RATE audio_bitrate);

/**
 *Set the different volumes of different channels.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	vol Pointer to the configured parameter. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	Volume is frome 0 to 70.
 * @see	 CSAUD_Volume
 */
CSAPI_RESULT CSAUD_SetVolume(CSAUD_HANDLE handle, CSAUD_Volume * vol);

/**
 *Set the audio decoder to specified codec.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	stream_type Specify audio codec. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	For encoder only G729/726/723/711 and MP2ENC can be set.
 * @see	 CSAUD_STREAM_TYPE
 */
CSAPI_RESULT CSAUD_SetCodecType(CSAUD_HANDLE handle, CSAUD_STREAM_TYPE stream_type);

/**/
/**
 *Set the input source of audio data.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	input_src Specify input source of audio data. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	This function is only for AIB mode. It is used to chose AIB source. if the source is from DDR, it must be 48K PCM data..
 * @see	 CSAUD_STREAM_TYPE
 */
CSAPI_RESULT CSAUD_SetInputSource(CSAUD_HANDLE handle, CSAUD_INPUT_SOURCE input_src);


/**
 *Enable the audio PTS synchronization.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EnablePTSSync(CSAUD_HANDLE handle);


/**
 *Disable the audio PTS synchronization.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_DisablePTSSync(CSAUD_HANDLE handle);

/**
 *Mute output device.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EnableMute(CSAUD_HANDLE handle);

/**
 *Unmute output device.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_DisableMute(CSAUD_HANDLE handle);

/**
 *Enable audio downmix function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EnableDownMix(CSAUD_HANDLE handle);

/**
 * Disable audio downmix function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_DisableDownMix(CSAUD_HANDLE handle);

/**
 *Get the error status code of last call of any function.
 *
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAUD_ErrCode CSAUD_GetErrCode(void);

/**
 *Get the error string of audio driver.
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
char *CSAUD_GetErrString(void);

/**
 *Prepare for PFM function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	Set driver to access decoder input memory buffer directly for write rather than TS demux. Initialize buffer management variables.
 */
CSAPI_RESULT CSAUD_PFMOpen(CSAUD_HANDLE handle);

/**
 *Close PFM function
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	Return decoder buffer write access to TS demux and clears PFM state.
 */

CSAPI_RESULT CSAUD_PFMClose(CSAUD_HANDLE handle);

/**
 *Reset PFM function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note     Reset all variables related to PFM memory management.
 */


CSAPI_RESULT CSAUD_PFMReset(CSAUD_HANDLE handle);

/**
 *Get current decoder buffer size available.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   bufsize Pointer to the returned value In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_GetPFMBufferSize(CSAUD_HANDLE handle, unsigned int * bufsize);

/**
 *Get current i2s buffer status (in bytes).
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   bufsize Pointer to the returned value In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_i2sBufferStatus(CSAUD_HANDLE handle, int *i2s_status);

/**
 *Write audio ES data to decoder buffer.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   src Audio ES data buffer start address In/Out : In
 * @param   size Length of input data In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_WritePFMData(CSAUD_HANDLE handle, unsigned char *src, int size);


/**
 *Notify upper layers upon empty_threshold. Callback function invoked.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   empty_threshold used to invoke callback function event. If remaining data in decoder input buffer is less than or equal to it, callback function event will be invoked. In/Out : In
 * @param   call_back_function Pointer to the callback function In/Out : In
 * @param   event_enable It controls event on/off. 0:disable 1:enable   In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetNotifyPFMDataEmpty(CSAUD_HANDLE handle, unsigned int *empty_threshold, void (* call_back_function)(CSAUD_HANDLE), unsigned char event_enable);


/**
 *Notify upper layers upon empty_threshold. Callback function invoked.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   full_threshold used to invoke callback function event. If remaining data in decoder input buffer is less than or equal to it, callback function event will be invoked. In/Out : In
 * @param   call_back_function Pointer to the callback function In/Out : In
 * @param   event_enable It controls event on/off. 0:disable 1:enable   In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetNotifyPFMDataFull(CSAUD_HANDLE handle, unsigned int *full_threshold,void (*call_back_function) (CSAUD_HANDLE), unsigned char event_enable);

/**
 *Open audio encoder function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   src Audio ES data buffer start address. In/Out : In
 * @param   size Length of input data. In/Out : In
 * @param   pts Current pts.  In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_WritePFMDataWithPTS(CSAUD_HANDLE handle, unsigned char *src, int size, unsigned long long pts);

/**
 *Open audio encoder function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   src Audio ES data buffer start address. In/Out : In
 * @param   size Length of input data. In/Out : In
 * @param   frame_length Frame length of src data
 * @param   pts Current pts.  In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_WritePFMDataWithPTS_For_NoHeader(CSAUD_HANDLE handle, unsigned char *src, int size, int frame_length, unsigned long long pts);

/**
 *Enable automatic video synchronization with local system clock.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   buffer_status The status of the video buffer. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_GetBufferStatus(CSAUD_HANDLE handle, CSAUD_BUFFER_STATUS *buffer_status);

/**
 *Set the I2S shift count.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   format_cnt Indicate which bits will be putted into FIFO. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note In I2S, each channel contains 32 bit, only 16 bit will be putted into FIFO, format_cnt will indicate which bits will be putted into FIFO.
 */
CSAPI_RESULT CSAUD_SetI2SFormat(CSAUD_HANDLE handle, unsigned short format_cnt);


/**
 *Get pts value of the audio stream.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   aud_pts Pts value of the audio stream. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_GetPTS(CSAUD_HANDLE handle, unsigned int *aud_pts);

/**
 *Get pts and stc value of the audio stream.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   aud_pts Pts value of the audio stream. In/Out : Out
 * @param   stc_value Stc value of the audio stream. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note stc_value is 34 bits. If the audio stream has no pts or the pts is 0, then the aud_pts is 0xffffffff.
 */
CSAPI_RESULT CSAUD_GetPTSAndSTC(CSAUD_HANDLE handle, unsigned int *aud_pts, unsigned long long *stc_value);

/**
 *Set audio equalizer type.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   type PAudio equalizer type. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @see	DRIVER_EQUALIZER_TYPE.
 */
CSAPI_RESULT CSAUD_SetEQtype(CSAUD_HANDLE handle, DRIVER_EQUALIZER_TYPE type);


/**
 *Get current audio firmware version.
 *
 * @param   audio_firmware_version struct point of CSAUD_Firmware_Version. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_GetFirmware_Version(CSAUD_Firmware_Version *audio_firmware_version);


/**
 *Enable audio DAC.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   type Specify DAC type In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note Default setting is disabled.
 */

CSAPI_RESULT CSAUD_EnableDAC(CSAUD_HANDLE handle);

/**
 *Disable audio DAC.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param   type Specify DAC type In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note Default setting is disabled.
 */

CSAPI_RESULT CSAUD_DisableDAC(CSAUD_HANDLE handle);

typedef enum
{
	CSAUD_HDMI_PCM_DOWN_MIX = 0,
	CSAUD_HDMI_PCM_MULTI_CHANNEL,
	CSAUD_HDMI_DOLBY_DIGITAL,
	CSAUD_HDMI_DOLBY_DIGITAL_PLUS,
    CSAUD_HDMI_HBR,
	CSAUD_HDMI_MAX
}CSAUD_HDMI_MODE;

typedef enum
{
	AUD_ENC_IN_I2S = 0,
	AUD_ENC_IN_SPDIF,
	AUD_ENC_NO_IN,
}CSAUD_ENCINPUT_TYPE;

/**
 *Define enum of SPFID type.
 *
 */

typedef enum
{
	AUD_SPDIF_TYPE_PCM = 0,///<SPDIF will transfer PCM data which our decoded.
	AUD_SPDIF_TYPE_COMPRESS = 1///<SPDIF will transfer source data which bypass by our decoder.
}CSAUD_SPDIF_TYPE;

/**
 *Set audio HDMI output type.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	type Specify output data type of CSAUD_HDMI_MODE. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_Set_HDMI_Mode(CSAUD_HANDLE handle, CSAUD_HDMI_MODE type);
/**
 *Enable audio SPDIF output.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	type Specify output data type of SPDIF. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EnableSPDIF(CSAUD_HANDLE handle, CSAUD_SPDIF_TYPE type);

/**
 *Disable audio SPDIF output.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_DisableSPDIF(CSAUD_HANDLE handle);
/**
 *Choose input type.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	type i2s/spdif. 
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_I2SorSPDIF(CSAUD_HANDLE handle, CSAUD_ENCINPUT_TYPE type);
/**
 *Open audio mixer function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	src_sr One of audio sample rate. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note  Before CSAUD_MixerOpen and play, one audio stream must be played. And src_sr is no use now. The mix pcm data sample rate must be same as current audio output sample rate.
 */
CSAPI_RESULT CSAUD_MixerOpen(CSAUD_HANDLE handle, CSAUD_SAMPLE_RATE src_sr);

/**
 *Close mixer function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	Close one audio mixer.
 */
CSAPI_RESULT CSAUD_MixerClose(CSAUD_HANDLE handle);

/**
 *Write mix data to audio buffer.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	src Pointer of mix data . In/Out : In
 * @param	size Size of mix data . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_WriteMixerBuffer(CSAUD_HANDLE handle, char *src, int size);

/**
 *Adjust audio mix data volume.
 *
 * @param	vol Specify mixer volume. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	The volue is from 0 to 70.
 */
CSAPI_RESULT CSAUD_SetMixerVolume(int vol);

/**
 *Pause audio mixer.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_PauseMixer(CSAUD_HANDLE handle);

/**
 *Resume audio mixer function.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_ResumeMixer(CSAUD_HANDLE handle);

/**
 *Define channel index of encoder audio.
 */
typedef enum
{
	AUD_ENCODER_INPUT = 0,///<Encoder input channel
	AUD_ENCODER_OUTPUT,///<Encoder output channel
	AUD_ENCODER_DECODE_1,///< encoder decoder channel 1
	AUD_ENCODER_DECODE_2,///< encoder decoder channel 2
	AUD_ENCODER_DECODE_3,///< encoder decoder channel 3
	AUD_ENCODER_DECODE_4,///< encoder decoder channel 4
	AUD_ENCODER_DECODE_5,///< encoder decoder channel 5
	AUD_ENCODER_DECODE_6,///< encoder decoder channel 6
	AUD_ENCODER_DECODE_7,///< encoder decoder channel 7
	AUD_ENCODER_DECODE_8,///< encoder decoder channel 8
}AUD_ENCODER_CHANNEL_INDEX_e;

/**
 *Open audio encoder function.
 *
 * @param	params configure parameters of audio encoder . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EncoderOpen(CSAUD_Encoder_Init_Params params);

/**
 *Close audio encoder function.
 *
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EncoderClose(void);

/**
 *Get audio encoder channel handle.
 *
 * @param	index Get one channel to encode audio In/Out : In
 * @return	Specified audio channel handle
 */
CSAUD_HANDLE CSAUD_Encoder_Get_Handle(AUD_ENCODER_CHANNEL_INDEX_e index);

/**
 *Check current encoded data size which can be read.
 *
 * @param	size Get buffer size could be used In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_Encoder_Check_Datasize(unsigned int *size);


/**
 *Read data from encoder buffer.
 *
 * @param	src Pointer of buffer which contain encoded data. In/Out : In
 * @param	size Specify how many data use want to read. In/Out : In
 * @param	pts Current pts refer to encoded data. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_Encoder_Fetch_Data(unsigned char *src, unsigned int size,unsigned long long *pts);

/**
 *Read data from encoder buffer, one frame per time.
 *
 * @param	src Pointer of buffer which contain encoded data. In/Out : In
 * @param	pts Pointer of current pts related to this frame data. In/Out : Out
 * @param	frame_size Pointer of current read frame size. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_Encoder_Fetch_Data_OneFrame(unsigned char *src,unsigned long long *pts,unsigned int *frame_size);

/**
 *Read data from encoder buffer, one frame per time, remove header data.
 *
 * @param	src Pointer of buffer which contain encoded data. In/Out : In
 * @param	pts Pointer of current pts related to this frame data. In/Out : Out
 * @param	frame_size Pointer of current read frame size. In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note  This function auto remove header data. Since other App cannot use G729 encoded data with header.
 */

CSAPI_RESULT CSAUD_Encoder_Fetch_Data_OneFrame_With_NoHeader(unsigned char *src,unsigned int *pts,unsigned int *frame_size);

/**
 *Define enum of audio output mode for two channels.
 */
typedef enum
{
	AUD_PCM_LEFT_MONO = 1, ///<Copy left channel to right.
	AUD_PCM_RIGHT_MONO = 2, ///<Cpoy right channel to left.
	AUD_PCM_STEREO = 0, ///<Stereo.
	AUD_PCM_LEFT_ONLY = 3,///<Disable right channel output.
	AUD_PCM_RIGHT_ONLY = 4,///<Disable left channel output.
	AUD_PCM_INVALID
} CSAUD_PCM_CHANNEL;

/**
 *Setting output channel mode.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	channel Specify output channel mode. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetOutputChannel(CSAUD_HANDLE handle, CSAUD_PCM_CHANNEL channel);

/**
 *Define enum of input PCM data channel numnber.
 */
typedef enum
{
	AUD_PCM_CHANNEL_1 = 1, 	///< 1_C CODEC_CHANNEL_CONFIG_TRIGGER_BIT
	AUD_PCM_CHANNEL_2 = 2,	///< 2_M.  Default is this one
	AUD_PCM_2F_1 = 3,	///< 2_M + 1_LFE
	AUD_PCM_2F_2 = 4,	///< 1_C + 2_M + 1_S
	AUD_PCM_4F_1 = 5,	///< 1_C + 2_M + 2_S
	AUD_PCM_5F_1 = 6,	///< 1_C + 2_M + 2_S + 1_LFE
	AUD_PCM_7F_1 = 7,	///< 1_C + 2_M + 2_0 + 2_S + 1_LFE
	AUD_PCM_CHANNEL_INVALID ///<Invalid
} CSAUD_PCM_CHANNEL_NUMBER_e;

/**
 *Define enum of input PCM data bit depth.
 */
typedef enum
{
	AUD_PCM_8_BITS = 0, ///<8 bits per sample
	AUD_PCM_16_BITS = 1, ///<16 bits per sample
	AUD_PCM_20_BITS = 2,///<20 bits per sample
	AUD_PCM_24_BITS = 3,///<24 bits per sample
	AUD_PCM_BITS_INVALID///<Invaild.
} CSAUD_PCM_BIT_DEPTH_e;

/**
 *Define enum of input PCM data endian.
 */
typedef enum
{
	AUD_ENDIAN_LITTLE = 0, ///< Litter endian.  Default is this one
	AUD_ENDIAN_BIG =1,///< Big endian.
	AUD_ENDIAN_INVALID ///<Invaild.
}CSAUD_ENDIAN_e;

/**
 *Define enum of input PCM data sign.
 */
typedef enum
{
	AUD_PCM_SIGN = 0, ///< Sign. Default is this one
	AUD_PCM_UNSIGN =1,///<Unsign.
	AUD_PCM_SIGN_INVALID///<Invaild.
}CSAUD_PCM_SIGN_e;

/**
 *Define enum of output PCM data bit depth.
 */
typedef enum
{
	AUD_PCM_OUTPUT_BIT_DEPTH_16 = 0, ///< 16 bits per sample.. Default is this one
	AUD_PCM_OUTPUT_BIT_DEPTH_24 =1,///< 24 bits per sample.
	AUD_PCM_OUTPUT_BTT_DEPTH_INVALID///< Invalid.
}CSAUD_PCM_OUTPUT_BIT_DEPTH_e;

/**
 *Define struct of PCM data related sutff.
 */

typedef struct tagAUD_PCM_CAPABILITY
{
	CSAUD_PCM_BIT_DEPTH_e bit_depth;///< Input pcm data bit depth.
	CSAUD_PCM_CHANNEL_NUMBER_e chl_num;///<Input pcm data channel number.
	CSAUD_SAMPLE_RATE sample_rate;///< Input pcm data sample rate.
	CSAUD_ENDIAN_e endian;///< Input pcm data endian.
	CSAUD_PCM_SIGN_e sign;///< Input pcm data sign. Only use when bit_depth is AUD_PCM_8_BITS
	CSAUD_PCM_OUTPUT_BIT_DEPTH_e output_bit_depth;///< Output pcm data bit depth.
} CSAUD_PCM_Capability_s;

/**
 *Setting Input pcm data sample rate.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	sample_rate Specify sample rate. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetPCMDataSampleRate(CSAUD_HANDLE handle, CSAUD_SAMPLE_RATE sample_rate);

/**
 *Setting Input pcm data channel number.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	channel_num Specify channel number. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetPCMDataChannelNumber(CSAUD_HANDLE handle, CSAUD_PCM_CHANNEL_NUMBER_e channel_num);

/**
 *Setting Input pcm data bit depth.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	bit_depth Specify bit depth. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetPCMDataBitDepth(CSAUD_HANDLE handle, CSAUD_PCM_BIT_DEPTH_e bit_depth);

/**
 *Setting Input pcm data sign.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	sign Specify sign. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	This function is only available when PCM Data bit depth is AUD_PCM_8_BITS.
 */
CSAPI_RESULT CSAUD_SetPCMDataSign(CSAUD_HANDLE handle, CSAUD_PCM_SIGN_e sign);

/**
 *Setting Input pcm data endian.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	endian Specify endian. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note	This function is only available when PCM Data bit depth is AUD_PCM_8_BITS.
 */
CSAPI_RESULT CSAUD_SetPCMDataEndian(CSAUD_HANDLE handle, CSAUD_ENDIAN_e endian);

/**
 *Setting ontput pcm data bitdepth.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	bit_depth Specify bit depth. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetPCMOutputDataBitDepth(CSAUD_HANDLE handle, CSAUD_PCM_OUTPUT_BIT_DEPTH_e bit_depth);

/**
 *Setting whole pcm data capability.
 *
 * @param   handle An opened audio handle . In/Out : In
 * @param	pcm_capability Specify pcm capability. In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetPCMCapability(CSAUD_HANDLE handle, CSAUD_PCM_Capability_s pcm_capability);

/**
 *Swap output of audio decoder channel 0 and 1.
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_Swap_Output(void);

/**
 *Enable echo cancellation.
 * @param	handle An opened audio handle . In/Out : In
 * @param	enable_nlp    Specify NLP status. In/Out : In
 * @return	CSAPI_SUCCEED	Success, CSAPI_FAILED Failure
 * @note  enable_nlp: 0 disbale NLP. 1 enable NLP.(NLP is audio post-process algorithm. It can enhances echo cancellation effect. But it also can make sound distortion.
 */
CSAPI_RESULT CSAUD_EnableAEC(CSAUD_HANDLE handle, int enable_nlp);

/**
 *Disable echo cancellation.
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED   Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_DisableAEC(CSAUD_HANDLE handle);

/**
 *Set the audio decode delay time.
 * @param	handle An opened audio handle . In/Out : In
 * @param	delay_ms pecify delay time. Valid 9 bits. Unit is millisecond.. In/Out : In
 * @return  CSAPI_SUCCEED   Success, CSAPI_FAILED Failure
 * @note  This action will take effect. Take into account:
 *          1) If audio started, this value will apply only to keep latency, if re-buffering needed you must use CSAUD_ForceStartDelay after.
 *          2) If audio not started, this value will apply to keep latency and to start delay (buffering, as if CSAUD_ForceStartDelay is called).
 *        And needing two conditions as below:
 *			1) CSAUD_EnablePTSSync() need be called.
 *			2) audio stream must have pts value.
 */
CSAPI_RESULT CSAUD_SetStartDelay(CSAUD_HANDLE handle, unsigned int delay_ms);

/**
 *Force Start Delay (using audio decode delay time setting by CSAUD_SetStartDelay)
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED   Success, CSAPI_FAILED Failure
 * @note  This action will take effect at anytime. And needing two conditions as below:
 *			1) CSAUD_EnablePTSSync() need be called.
 *			2) audio stream must have pts value.
 */
CSAPI_RESULT CSAUD_ForceStartDelay(CSAUD_HANDLE handle);

/*
Reload audio firmware.
handle: no useful. Can be NULL.
This function will reset audio decoder and reload audio firmware. when calling this function, all the setting is gone. You should set again.
I suggest that call this function before CSAUD_Open() or CSAUD_SetCodecType().
See sample code below:
-----------------------------------------------------------------------
//option 1:
	if(CSAPI_FAILED == CSAUD_Reload_Firmware(a_handle)){
		printf("%s:%s in line %d\n",__FILE__,__FUNCTION__,__LINE__);
		return -1;
	}
//end
	a_handle = CSAUD_Open(dev_num);
	if(a_handle == NULL){
		printf("error : CSAUD_Open %d\n",dev_num);
		return -1;
	}

//option 2:
	if(CSAPI_FAILED == CSAUD_Reload_Firmware(a_handle)){
		printf("%s:%s in line %d\n",__FILE__,__FUNCTION__,__LINE__);
		return -1;
	}
//end
	if(handle->codec_type == AUD_STREAM_TYPE_AIB){
		CSAUD_SetPCMDataSampleRate(a_handle, AUD_SAMPLE_RATE_44_100KHZ);
		CSAUD_SetInputSource(a_handle, AUD_INPUT_SOURCE_DDR);
		......
	}
	if(CSAPI_FAILED == CSAUD_SetCodecType(a_handle, codec_type)){
//option 3:
		if(CSAPI_FAILED == CSAUD_Reload_Firmware(a_handle)){
			printf("%s:%s in line %d\n",__FILE__,__FUNCTION__,__LINE__);
			return -1;
		}
		if(handle->codec_type == AUD_STREAM_TYPE_AIB){
			CSAUD_SetPCMDataSampleRate(a_handle, AUD_SAMPLE_RATE_44_100KHZ);
			CSAUD_SetInputSource(a_handle, AUD_INPUT_SOURCE_DDR);
			......
		}
		CSAUD_SetCodecType(a_handle, codec_type);
	}
//end
	}

	CSAUD_SetInputMode(a_handle, AUD_INPUT_BLOCK);
	CSAUD_SetVolume(a_handle, &vol);
	CSAUD_DisableDownMix(a_handle);
	CSAUD_EnableSPDIF(a_handle, AUD_SPDIF_TYPE_PCM);
	CSAUD_SetOutputChannel(a_handle, AUD_PCM_STEREO);

	CSAUD_PFMOpen(a_handle);
	CSAUD_SetI2SFormat(a_handle, 1);
	CSAUD_Play(a_handle);
-------------------------------------------------------------------------
*/

/**
 *Reload audio firmware.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED   Success, CSAPI_FAILED Failure
 * @note  This function will reset audio decoder and reload audio firmware. when calling this function, all the setting is gone. You should set again.
 *				I suggest that call this function before CSAUD_Open() or CSAUD_SetCodecType().
 */
CSAPI_RESULT CSAUD_Reload_Firmware(CSAUD_HANDLE handle);

/*
The HDMI get pcm samples from I2S, and when sampling-rate is lower than 32kHz, there may be no audio because some of the TV HDMI receiver
cannot support such low sampling-rate. So now when the sampling-rate is 8kHz~24kHz, it will up-sampling the output pcm as default setting.
when sampling-rate is  8kHz~12kHz, the up-sampling factor is 4;
when sampling-rate is  16kHz~24kHz, the up-sampling factor is 2;

Also this processing can be enable/disabled by below APIs.
We suggest that app don't call any below APIs. Since have already do SRC as default setting to ensure all TVs can output any audio with HDMI.
Note: You must call CSAUD_EnableSPDIF(a_handle, AUD_SPDIF_TYPE_PCM) to enable the SPDIF output mode first.
*/

/**
 *Enable audio SPDIF SRC function.
 *      The HDMI get pcm samples from I2S, and when sampling-rate is lower than 32kHz, there may be no audio because some of the TV HDMI receiver
 *      cannot support such low sampling-rate. So now when the sampling-rate is 8kHz~24kHz, it will up-sampling the output pcm as default setting.
 *      when sampling-rate is  8kHz~12kHz, the up-sampling factor is 4;
 *      when sampling-rate is  16kHz~24kHz, the up-sampling factor is 2;
 *
 *      Also this processing can be enable/disabled by below APIs.
 *      We suggest that app don't call any below APIs. Since have already do SRC as default setting to ensure all TVs can output any audio with HDMI.
 *      You must call CSAUD_EnableSPDIF(a_handle, AUD_SPDIF_TYPE_PCM) to enable the SPDIF output mode first.
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED   Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EnableSPDIF_SRC(CSAUD_HANDLE handle);

/**
 *Disable audio SPDIF SRC function.
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_DisableSPDIF_SRC(CSAUD_HANDLE handle);

/*
Enable/Disable PCM data output function.
Audio firmware will automatic copy the decoded PCM data to second audio buffer.
Note : Need config kernel buffer to support second audio buffer. Normally there is only one audio buffer.
*/
/**
 *Enable  PCM data output function. Audio firmware will automatic copy the decoded PCM data to second audio buffer.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note  Need config kernel buffer to support second audio buffer. Normally there is only one audio buffer.
 */
CSAPI_RESULT CSAUD_EnablePCMOut(CSAUD_HANDLE handle);

/**
 *Disable  PCM data output function.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_DisablePCMOut(CSAUD_HANDLE handle);

/*
Get whole PCM output data buffer size.
Need enable PCM data output function.
*/

/**
 *Get whole PCM output data buffer size.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	size Pointer of size . In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note Need enable PCM data output function.
 */
CSAPI_RESULT CSAUD_GetPCMOutBufferSize(CSAUD_HANDLE handle, unsigned int *size);

/*
Get current PCM data size.
Need enable PCM data output function.
*/

/**
 *Get current PCM data size which can be read.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	size Pointer of size . In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note Need enable PCM data output function.
 */
CSAPI_RESULT CSAUD_GetPCMDateSize(CSAUD_HANDLE handle, unsigned int *size);
/*
Get PCM data.
Parameter src : pointer of the buffer where the PCM data should copy.
Parameter size : input : how many you want to get.
                         output : how many you actually get.
Note : if the you set the size to 0, you will get all current PCM data. The size value will change to current size.
          if you set the size is larger than current size, you will get current size data. The size value will change to current size.
          if you set the size is samller than current size, you will get size data. the size vaule will not change.
I suggest you check the size value.
*/

/**
 *Get PCM data.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	src    Pointer of buffer which contain pcm data . In/Out : In
 * @param	size   Pointer of size . In/Out : Out
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note Need enable PCM data output function.
 */
CSAPI_RESULT CSAUD_GetPCMDate(CSAUD_HANDLE handle, char *src, unsigned int *size);

/**
 *Enables/Disables silence detection.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	int 1 Enables 0 Disables Detection
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note    Silence audio frames counter will be found in "silence_frames" DecodingInfo field.
 * If want to use a callback need to use CSAPI_RESULT CSAUD_SetNotifySilenceDetection as usual.
 */
CSAPI_RESULT CSAUD_SilenceDetection(CSAUD_HANDLE handle, int enable);

/**
 *Set callback function for Silence Detection.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	call_back_function Pointer of callback function . In/Out : In
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note If the call_back_function = NULL, then the silence detection function will be disabled.
 */
CSAPI_RESULT CSAUD_SetNotifySilenceDetection(CSAUD_HANDLE handle, void (* call_back_function)(CSAUD_HANDLE));

/**
 *Enables/Disables Latency recovering on silence.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	int 1 Enables 0 Disables Tool
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note    Uses 3rd bit of ena_silence_detection, so tool can be enabled/disabled independtly of rest of bits
 */
CSAPI_RESULT CSAUD_RecoverLatencyOnSilence(CSAUD_HANDLE handle, int enable);

/**
 *Set MAX pts threshold to audio firmware.
 *
 * @param	handle An opened audio handle . In/Out : In
 * @param	MaxPtsThreshold Value of MAX pts threshold. In/Out : In
 * @return	CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 * @note This function must be called after CSAUD_Init(). Otherwise the default value is 100ms. The unit of MaxPtsThreshold is 10ms. And the range MaxPtsThreshold is 0 ~ 127.
 */
CSAPI_RESULT CSAUD_SetMAXPTSThreshold(CSAUD_HANDLE handle, unsigned int MaxPtsThreshold);

/* Below two function are temporary used for Dolby decoder. Maybe be changed in the future.*/
/**
 * Set up Dolby thread process
 * @param handle Audio Handle
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetDolbyThread(CSAUD_HANDLE handle);

/**
 * Set the HDMI Audio Output Mode
 * @param handle Audio Handle
 * @param type HDMI Audio Mode Type
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetHDMIAudioOutputMode(CSAUD_HANDLE handle, CSAUD_HDMI_MODE type);

/**
 * Set the Audio Sample Rate
 * @param handle Audio Handle
 * @param sample_rate Sample Rate
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetSampleRate(CSAUD_HANDLE handle, CSAUD_SAMPLE_RATE sample_rate);

/**
 * Set Audio Endianess
 * @param handle Audio Handle
 * @param flag Endianess Flag
 * @return  CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetAUDEndian(CSAUD_HANDLE handle, unsigned int flag);

/** 
 * Enable HDMI
 * 
 * @param handle An opened audio handle . In/Out : In
 * @param type The hdmi mode used 
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_EnableHDMI(CSAUD_HANDLE handle, CSAUD_HDMI_MODE type);

/** 
 * Enable Enable data discard
 * 
 * @param handle: An opened audio handle . In/Out : In
 * @param pts: The pts of first frame to play. discard the data before this value.
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_SetStartPts(CSAUD_HANDLE handle, unsigned int pts);

/** 
 * get the number of audio frames that failed to decode
 * 
 * @return the decoding info. See CSAUD_DecodingInfo.
 */
void CSAUD_GetDecodingInfo(CSAUD_DecodingInfo* info);

/** 
 * Set AudioSTCDiff update threshold to firmware. If the difference between 
 * audio PTS and STC changes beyond the threshold, Audio firmware should update
 * new diff value to video firmware for sync. Default value is 300ms.
 * 
 * @param handle: An opened audio handle . In/Out : In
 * @param threshold: AudioSTCDiff update threshold (0 - 0xFFFF)
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */
CSAPI_RESULT CSAUD_SetSourcePTSSTCDriftTolerance(CSAUD_HANDLE handle, unsigned int threshold);

/** 
 * Set delay to start latency recovering mechanism, Just in case source recovers 
 * audio latency by itself. Default value is 4000ms.
 * 
 * @param handle: An opened audio handle . In/Out : In
 * @param delay: delay in msecs (0 - 0xFFFF)
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED Failure
 */

CSAPI_RESULT CSAUD_SetLatencyRecoveringDelay(CSAUD_HANDLE handle, unsigned int delay);

CSAPI_RESULT CSAUD_EnableTransCode(CSAUD_HANDLE handle);
CSAPI_RESULT CSAUD_DisableTransCode(CSAUD_HANDLE handle);

#ifdef __cplusplus
}
#endif

#endif
