/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csdemux.h
 * Low Level Demux API
 * @ingroup csapi
 *
 * CSDEMUX is a subsystem of Celestial multi-media SoC chipsets. This module, which includes a special MCU, is responsible for demultiplexing and preprocessing
 * of the input data.
 * CSDEMUX supports various input interfaces and data formats. It is capable of processing up to 2 external SPI/SSI tuner inputs or up to 4 internal DMA playback channels.
 * CSDEMUX API can support a variety of data output formats based on the capability of the firmware. It supports up to 79 demultiplexing data output channels
 * which can be used for A/V decoding and host recoding etc.
 * @image html csdemux_detail.jpg
 * @image rtf  csdemux_detail.jpg
 * The above diagram presents a CSDEMUX module which is composed of four stream processing channels. Each channel supports tuner or DMA data input.
 * CSDEMUX has two groups 64 hardware PID filters shared by the channels0/1 and channels2/3, and 8 descrambling channels shared by first group 64 PID filters.
 * The descrambling channels of CSDEMUX can support DVB-CSA/ ATSC TDES/AES128 standards. On the right side of the diagram, the MCU processor parses all input
 * streams from channel0/1 and channel2/3 and then sends data to various output channels set by the host.
 * Features include:
 * @li Support H.264 over TS
 * @li Support MPEG2 over TS
 * @li Support AVS over TS
 * @li Support 2 Scatter/Gather DMA input Interfaces one for each input channel
 * @li Support up two groups 64 PID filter shared by the four input channels
 * @li Support 12 bytes filter and 12 bytes mask section filter (depend on the firmware)
 * @li Support one STC/PWM synchronized to a single PCR source
 * @li Support pushing mode input with the clock recovery
 * @li Support pulling mode input with no clock recovery
 * @li Support up to DVB-CSA/ ATSC TDES/AES128 descramble channels shared by the 64 PID filter for real-time TS descramble
 *
 */
#ifndef __CSAPI_DEMUX_H__
#define __CSAPI_DEMUX_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"

#define CSDEMUX_DES_NUM            8

#ifdef ARCH_CNC1800H
#define CSDEMUX_PIDFT_NUM          128
#else
#define CSDEMUX_PIDFT_NUM          64
#endif

#define CSDEMUX_CRC_NUM            16

#ifdef ARCH_CNC1800H
#define CSDEMUX_CHL_NUM            4
#else
#define CSDEMUX_CHL_NUM            2
#endif

#define CSDEMUX_VIDOUT_NUM         8
#define CSDEMUX_AUDOUT_NUM         8

#define CSDEMUX_PCR_NUM            1

#define CSDEMUX_FILTER_NUM         64
#define CSDEMUX_FILTER_EVENT_NUM   2

#define CSDEMUX_UNVALID_HANDLE     0

typedef void *CSDEMUX_HANDLE;

/* Constant ID define */

/**
* The structure defines all DES ID number.
*/
typedef enum
{
	DEMUX_DES_UNDEF = -1,///<Indicating the DES id number is undefined.
	DEMUX_DES_ID0 = 0,///<Indicating the DES id number is 0.
	DEMUX_DES_ID1,///<Indicating the DES id number is 1.
	DEMUX_DES_ID2,///<Indicating the DES id number is 2.
	DEMUX_DES_ID3,///<Indicating the DES id number is 3.
	DEMUX_DES_ID4,///<Indicating the DES id number is 4.
	DEMUX_DES_ID5,///<Indicating the DES id number is 5.
	DEMUX_DES_ID6,///<Indicating the DES id number is 6.
	DEMUX_DES_ID7///<Indicating the DES id number is 7.
} CSDEMUX_DES_ID;

/**
* The structure defines all hardware PID filter ids in the csdemux. Each ID represents a programmable PID filter. A total of 64 PID filters are supported and each one can be bound to either of the two input channels.
*/
typedef enum
{
	DEMUX_PIDFT_UNDEF = -1,
	DEMUX_PIDFT_ID0 = 0,
	DEMUX_PIDFT_ID1,
	DEMUX_PIDFT_ID2,
	DEMUX_PIDFT_ID3,
	DEMUX_PIDFT_ID4,
	DEMUX_PIDFT_ID5,
	DEMUX_PIDFT_ID6,
	DEMUX_PIDFT_ID7,
	DEMUX_PIDFT_ID8,
	DEMUX_PIDFT_ID9,
	DEMUX_PIDFT_ID10,
	DEMUX_PIDFT_ID11,
	DEMUX_PIDFT_ID12,
	DEMUX_PIDFT_ID13,
	DEMUX_PIDFT_ID14,
	DEMUX_PIDFT_ID15,
	DEMUX_PIDFT_ID16,
	DEMUX_PIDFT_ID17,
	DEMUX_PIDFT_ID18,
	DEMUX_PIDFT_ID19,
	DEMUX_PIDFT_ID20,
	DEMUX_PIDFT_ID21,
	DEMUX_PIDFT_ID22,
	DEMUX_PIDFT_ID23,
	DEMUX_PIDFT_ID24,
	DEMUX_PIDFT_ID25,
	DEMUX_PIDFT_ID26,
	DEMUX_PIDFT_ID27,
	DEMUX_PIDFT_ID28,
	DEMUX_PIDFT_ID29,
	DEMUX_PIDFT_ID30,
	DEMUX_PIDFT_ID31,
	DEMUX_PIDFT_ID32,
	DEMUX_PIDFT_ID33,
	DEMUX_PIDFT_ID34,
	DEMUX_PIDFT_ID35,
	DEMUX_PIDFT_ID36,
	DEMUX_PIDFT_ID37,
	DEMUX_PIDFT_ID38,
	DEMUX_PIDFT_ID39,
	DEMUX_PIDFT_ID40,
	DEMUX_PIDFT_ID41,
	DEMUX_PIDFT_ID42,
	DEMUX_PIDFT_ID43,
	DEMUX_PIDFT_ID44,
	DEMUX_PIDFT_ID45,
	DEMUX_PIDFT_ID46,
	DEMUX_PIDFT_ID47,
	DEMUX_PIDFT_ID48,
	DEMUX_PIDFT_ID49,
	DEMUX_PIDFT_ID50,
	DEMUX_PIDFT_ID51,
	DEMUX_PIDFT_ID52,
	DEMUX_PIDFT_ID53,
	DEMUX_PIDFT_ID54,
	DEMUX_PIDFT_ID55,
	DEMUX_PIDFT_ID56,
	DEMUX_PIDFT_ID57,
	DEMUX_PIDFT_ID58,
	DEMUX_PIDFT_ID59,
	DEMUX_PIDFT_ID60,
	DEMUX_PIDFT_ID61,
	DEMUX_PIDFT_ID62,
	DEMUX_PIDFT_ID63,

#ifdef ARCH_CNC1800H
	DEMUX_PIDFT_ID64,
	DEMUX_PIDFT_ID65,
	DEMUX_PIDFT_ID66,
	DEMUX_PIDFT_ID67,
	DEMUX_PIDFT_ID68,
	DEMUX_PIDFT_ID69,
	DEMUX_PIDFT_ID70,
	DEMUX_PIDFT_ID71,
	DEMUX_PIDFT_ID72,
	DEMUX_PIDFT_ID73,
	DEMUX_PIDFT_ID74,
	DEMUX_PIDFT_ID75,
	DEMUX_PIDFT_ID76,
	DEMUX_PIDFT_ID77,
	DEMUX_PIDFT_ID78,
	DEMUX_PIDFT_ID79,
	DEMUX_PIDFT_ID80,
	DEMUX_PIDFT_ID81,
	DEMUX_PIDFT_ID82,
	DEMUX_PIDFT_ID83,
	DEMUX_PIDFT_ID84,
	DEMUX_PIDFT_ID85,
	DEMUX_PIDFT_ID86,
	DEMUX_PIDFT_ID87,
	DEMUX_PIDFT_ID88,
	DEMUX_PIDFT_ID89,
	DEMUX_PIDFT_ID90,
	DEMUX_PIDFT_ID91,
	DEMUX_PIDFT_ID92,
	DEMUX_PIDFT_ID93,
	DEMUX_PIDFT_ID94,
	DEMUX_PIDFT_ID95,
	DEMUX_PIDFT_ID96,
	DEMUX_PIDFT_ID97,
	DEMUX_PIDFT_ID98,
	DEMUX_PIDFT_ID99,
	DEMUX_PIDFT_ID100,
	DEMUX_PIDFT_ID101,
	DEMUX_PIDFT_ID102,
	DEMUX_PIDFT_ID103,
	DEMUX_PIDFT_ID104,
	DEMUX_PIDFT_ID105,
	DEMUX_PIDFT_ID106,
	DEMUX_PIDFT_ID107,
	DEMUX_PIDFT_ID108,
	DEMUX_PIDFT_ID109,
	DEMUX_PIDFT_ID110,
	DEMUX_PIDFT_ID111,
	DEMUX_PIDFT_ID112,
	DEMUX_PIDFT_ID113,
	DEMUX_PIDFT_ID114,
	DEMUX_PIDFT_ID115,
	DEMUX_PIDFT_ID116,
	DEMUX_PIDFT_ID117,
	DEMUX_PIDFT_ID118,
	DEMUX_PIDFT_ID119,
	DEMUX_PIDFT_ID120,
	DEMUX_PIDFT_ID121,
	DEMUX_PIDFT_ID122,
	DEMUX_PIDFT_ID123,
	DEMUX_PIDFT_ID124,
	DEMUX_PIDFT_ID125,
	DEMUX_PIDFT_ID126,
	DEMUX_PIDFT_ID127
#endif
} CSDEMUX_PIDFT_ID;


/**
* The structure defines all CRC ID.
*/
typedef enum
{
	DEMUX_CRC_UNDEF = -1,///<Indicating the CRC id is not defined.
	DEMUX_CRC_ID0 = 0,///<Indicating CRC id number is 0.
	DEMUX_CRC_ID1,///<Indicating CRC id number is 1.
	DEMUX_CRC_ID2,///<Indicating CRC id number is 2.
	DEMUX_CRC_ID3,///<Indicating CRC id number is 3.
	DEMUX_CRC_ID4,///<Indicating CRC id number is 4.
	DEMUX_CRC_ID5,///<Indicating CRC id number is 5.
	DEMUX_CRC_ID6,///<Indicating CRC id number is 6.
	DEMUX_CRC_ID7,///<Indicating CRC id number is 7.
	DEMUX_CRC_ID8,///<Indicating CRC id number is 8.
	DEMUX_CRC_ID9,///<Indicating CRC id number is 9.
	DEMUX_CRC_ID10,///<Indicating CRC id number is 10.
	DEMUX_CRC_ID11,///<Indicating CRC id number is 11.
	DEMUX_CRC_ID12,///<Indicating CRC id number is 12.
	DEMUX_CRC_ID13,///<Indicating CRC id number is 13.
	DEMUX_CRC_ID14,///<Indicating CRC id number is 14.
	DEMUX_CRC_ID15///<Indicating CRC id number is 15.
} CSDEMUX_CRC_ID;

#ifdef ARCH_CNC1800H

/**
* The structure defines all streaming input channel ids in the csdemux. Each ID represents a TS-stream source channel, each of which can be a tuner-input or a DMA feed. A total of 4 input sources are supported.

* @see \n

*/

typedef enum
{
	DEMUX_CHL_UNDEF = -1,///<Indicating the invalid streaming input channel.
	DEMUX_CHL_ID0 = 0,///<Indicating streaming input channel 0.
	DEMUX_CHL_ID1 = 1,///<Indicating streaming input channel 1.
	DEMUX_CHL_ID2 = 2,///<
	DEMUX_CHL_ID3 = 3///<
} CSDEMUX_CHL_ID;
#else
typedef enum
{
	DEMUX_CHL_UNDEF = -1,
	DEMUX_CHL_ID0 = 0,
	DEMUX_CHL_ID1 = 1
} CSDEMUX_CHL_ID;
#endif

typedef enum
{
    DEMUX_TSOUT_DISABLE = 0,
    DEMUX_TSOUT_ENABLE
}CSDEMUX_TSOUT_ENA;

typedef enum
{
    DEMUX_TSOUT_PARALLEL = 0,
    DEMUX_TSOUT_SERIAL
}CSDEMUX_TSOUT_MODE;

typedef enum
{
    DEMUX_TSOUT_CLK_INV_DISABLE = 0,
    DEMUX_TSOUT_CLK_INV_ENABLE
}CSDEMUX_TSOUT_INVERSE_CLK;

typedef enum
{
    CSDEMUX_TSOUT_CLOCK_54M = 4,
    CSDEMUX_TSOUT_CLOCK_27M = 8,
    CSDEMUX_TSOUT_CLOCK_13_5M = 16,
    CSDEMUX_TSOUT_CLOCK_6_75M = 32,
    CSDEMUX_TSOUT_CLOCK_3_375M = 64
}CSDEMUX_TSOUT_CLOCK_DIV;

typedef enum
{
    DEMUX_TSOUT_DMA0 = 0,
    DEMUX_TSOUT_DMA1
}CSDEMUX_TSOUT_DMA_SOURCE;

typedef struct
{
    CSDEMUX_TSOUT_ENA ena : 1;
    CSDEMUX_TSOUT_MODE mode : 1;
    CSDEMUX_TSOUT_INVERSE_CLK inv_clk : 1;
    CSDEMUX_TSOUT_CLOCK_DIV clk_div : 7;
    CSDEMUX_TSOUT_DMA_SOURCE dma_src : 1;
    unsigned int tsout_gap : 8;
    unsigned int reserve : 13;
}CSDEMUX_TSOUT_PARAM;

/**
* The structure defines all video output channels id in the csdemux. Each ID represents a unique compressed video output channel. A total of 2 video output channels are supported.
*/
typedef enum
{
	DEMUX_VIDOUT_UNDEF = -1,///<Indicating the invalid video output channel.
	DEMUX_VIDOUT_ID0 = 0,///<Indicating video output channel 0.
	DEMUX_VIDOUT_ID1///<Indicating video output channel 1.
} CSDEMUX_VIDOUT_ID;

/**
* The structure defines all audio output channels�� id in the csdemux. Each ID represents a unique compressed audio output channel. A total of 1 audio output channel I supported.
*/
typedef enum
{
	DEMUX_AUDOUT_UNDEF = -1,///<Indicating the invalid audio output channel.
	DEMUX_AUDOUT_ID0 = 0,///<Indicating audio output channel 0.
} CSDEMUX_AUDOUT_ID;///<Indicating audio output channel 1.

/**
* The structure defines PCR ID in the csdemux. There is only one PCR channel used for clock recovery. This PCR output channel can be associated with a PID filter and the corresponding input channel.
*/
typedef enum
{
	DEMUX_PCRDEV_UNDEF = -1,///<Indicating the invalid PCR ID.
	DEMUX_PCRDEV_ID0 = 0,///<Indicating PCR ID 0.
} CSDEMUX_PCRDEV_ID;

/**
* The structure defines all TS/PES/section filters�� id in the csdemux. Each ID represents a data output channel which could be of many types which could be Section, TS, PES, or ES. Each filter ID can be associated with one PID filter. There are a total of 32 data filters supported.
*/

typedef enum
{
	DEMUX_FILTER_UNDEF = -1,///<Indicating the invalid filter ID.
	DEMUX_FILTER_ID0 = 0,///<Indicating filter 0��31.
	DEMUX_FILTER_ID1,
	DEMUX_FILTER_ID2,
	DEMUX_FILTER_ID3,
	DEMUX_FILTER_ID4,
	DEMUX_FILTER_ID5,
	DEMUX_FILTER_ID6,
	DEMUX_FILTER_ID7,
	DEMUX_FILTER_ID8,
	DEMUX_FILTER_ID9,
	DEMUX_FILTER_ID10,
	DEMUX_FILTER_ID11,
	DEMUX_FILTER_ID12,
	DEMUX_FILTER_ID13,
	DEMUX_FILTER_ID14,
	DEMUX_FILTER_ID15,
	DEMUX_FILTER_ID16,
	DEMUX_FILTER_ID17,
	DEMUX_FILTER_ID18,
	DEMUX_FILTER_ID19,
	DEMUX_FILTER_ID20,
	DEMUX_FILTER_ID21,
	DEMUX_FILTER_ID22,
	DEMUX_FILTER_ID23,
	DEMUX_FILTER_ID24,
	DEMUX_FILTER_ID25,
	DEMUX_FILTER_ID26,
	DEMUX_FILTER_ID27,
	DEMUX_FILTER_ID28,
	DEMUX_FILTER_ID29,
	DEMUX_FILTER_ID30,
	DEMUX_FILTER_ID31,
	DEMUX_FILTER_ID32,
	DEMUX_FILTER_ID33,
	DEMUX_FILTER_ID34,
	DEMUX_FILTER_ID35,
	DEMUX_FILTER_ID36,
	DEMUX_FILTER_ID37,
	DEMUX_FILTER_ID38,
	DEMUX_FILTER_ID39,
	DEMUX_FILTER_ID40,
	DEMUX_FILTER_ID41,
	DEMUX_FILTER_ID42,
	DEMUX_FILTER_ID43,
	DEMUX_FILTER_ID44,
	DEMUX_FILTER_ID45,
	DEMUX_FILTER_ID46,
	DEMUX_FILTER_ID47,
	DEMUX_FILTER_ID48,
	DEMUX_FILTER_ID49,
	DEMUX_FILTER_ID50,
	DEMUX_FILTER_ID51,
	DEMUX_FILTER_ID52,
	DEMUX_FILTER_ID53,
	DEMUX_FILTER_ID54,
	DEMUX_FILTER_ID55,
	DEMUX_FILTER_ID56,
	DEMUX_FILTER_ID57,
	DEMUX_FILTER_ID58,
	DEMUX_FILTER_ID59,
	DEMUX_FILTER_ID60,
	DEMUX_FILTER_ID61,
	DEMUX_FILTER_ID62,
	DEMUX_FILTER_ID63
} CSDEMUX_FILTER_ID;

/* DEMUX TYPE define */

/**
* The structure defines filter types.
*/

typedef enum
{
	DEMUX_FILTER_TYPE_UNDEF = -1,///<Indicating the invalid filter type.
	DEMUX_FILTER_TYPE_SEC = 0,///<Indicating the section filter.
	DEMUX_FILTER_TYPE_TS,///<Indicating the invalid filter type.
	DEMUX_FILTER_TYPE_PES,///<Indicating the TS filter.
	DEMUX_FILTER_TYPE_ES///<Indicating the PES packets filter.
} CSDEMUX_FILTER_TYPE;///<Indicating the element stream data filter.

/**
* The structure defines descramble channel type.
*/

typedef enum
{
	DEMUX_DES_TYPE_DVB = 0,///<Indicating the DVB standard descramble.
	DEMUX_DES_TYPE_ATSC///<Indicating the ATSC standard descramble.
} CSDEMUX_DES_TYPE;

/**
* The structure defines the input type of the input channel.
*/

typedef enum
{
	DEMUX_INPUT_MOD_TUNER = 0,///<Indicating the tuner input.
	DEMUX_INPUT_MOD_DMA,///<Indicating data input by the DMA mode.
	DEMUX_INPUT_MOD_DIRECT///<Indicating data input by the direct DMA mode.
} CSDEMUX_INPUT_MOD;

/**
* The structure defines all input stream type.
*/

typedef enum
{
	DEMUX_INPUT_TS_TYPE = 0,///<Indicating the input stream type is TS.
	DEMUX_INPUT_M2TS_TYPE,///<Indicating the input stream type is M2TS.
	DEMUX_INPUT_TS204_TYPE,///<Indicating the input stream type is TS204.
	DEMUX_INPUT_OTHER_TYPE///<Indicating the input stream type is OTHERs.
} CSDEMUX_STREAM_TYPE;


/**
* The structure defines all tuner mode.
*/

typedef enum
{
	DEMUX_TUNER_MOD_PARALLEL = 0,///<Indicating the tuner mode is parallel.
	DEMUX_TUNER_MOD_SERIAL,///<Indicating the tuner mode is parallel.
} CSDEMUX_TUNER_MOD;

/**
* The structure defines all demux serial mode.
*/
typedef enum
{
	DEMUX_SERIAL_MOD_MSB = 0,///<Indicating demux serial mode is MSB.
	DEMUX_SERIAL_MOD_LSB,///<Indicating demux serial mode is LSB.
} CSDEMUX_SERIAL_MODE;

/**
* The structure defines all tuner ID.
*/
typedef enum
{
	DEMUX_TUNER_0 = 0,///<Indicating demux tuner ID 0.
	DEMUX_TUNER_1///<Indicating demux tuner ID 1.
}CSDEMUX_TUNER_ID;


/**
* The structure defines demux switch mode.
*/
typedef enum
{
	DEMUX_SWITCH_MOD_CHL0 = 0,///<Indicating the switching channel 0 mode
	DEMUX_SWITCH_MOD_CHL1,///<Indicating the switching channel 1 mode
	DEMUX_SWITCH_MOD_CHL2,///<Indicating the switching channel 2 mode
	DEMUX_SWITCH_MOD_CHL3,///<Indicating the switching channel 3 mode
	DEMUX_SWITCH_MOD_HOST///<Indicating the switching host mode
} CSDEMUX_SWITCH_MOD;

/**
* The structure defines MCU output data mode.This attribute is applicable to the output channels, namely video, audio, and the data outputs.
*		If an output channel is set to BLOCK mode, an output buffer fullcondition of this output channel blocks the demuxing process.
*/

typedef enum
{
	DEMUX_OUTPUT_MOD_BLOCK = 0,///<Indicating MCU output data by the blocking mode.
	DEMUX_OUTPUT_MOD_NONBLOCK///<Indicating MCU output data by the no blocking mode.
} CSDEMUX_OUTPUT_MOD;


/**
* The structure defines the section data output events. Callback functions can be registered for these events. The events are section data available or section data CRC error.
*/
typedef enum
{
	DEMUX_CRC_ERROR = 0,///<CRC error occurred in the section output channel.
	DEMUX_SECTION_AVAIL///<New Section data available in the section output channel.
} CSDEMUX_SECEVENT;

/**
* The structure defines all error type.
*/
typedef enum
{
	DEMUX_NO_ERROR = 0,
	DEMUX_ERROR_OPEN_FAILED, ///<open filed
	DEMUX_ERROR_IOCTL_FAILED,	///< ioctl filed
	DEMUX_ERROR_INVALID_PARAMETERS,///<Bad parameter passed
	DEMUX_ERROR_UNKNOWN_DEVICE,	///<Unknown device name
	DEMUX_ERROR_DEVICE_BUSY,	///<Device is currently busy
	DEMUX_ERROR_INVALID_HANDLE,	///<Handle is not valid
	DEMUX_ERROR_ALREADY_INITIALIZED,///<Device already initialized
	DEMUX_ERROR_NOT_INITIALIZED,///< Device not initialized
	DEMUX_ERROR_INVALID_STATUS	///<invalid status
} CSDEMUX_ErrCode;

typedef unsigned long long CSDEMUX_PCRVALUE;

/* function define */

/**
* Initializes the demux hardware, firmware, driver, and API variables. \n
*			Additionally downloads firmware to the demux MCU.
*			This API must be called first before any other APIs called in the application or higher level abstract layer.
*@return CSAPI_FAILED	Initialization failure
*				CSAPI_SUCCEED	Initialization succeeded
*@see CSDEMUX_Terminate()
*/
CSAPI_RESULT CSDEMUX_Init(void);

/**
* Shuts down the CSDEMUX_API interface, \n
*			 releases and closes all the object handles and closes the device.\n
*			 Last to be called in the CSDEMUX API group before the application shuts down or resets the demux.
*/
void CSDEMUX_Terminate(void);

/*pid filter interface*/

/**
* Opens a PID filter. One of the 64 PID filters is opened. Further actions are needed to associate this PID filter to other reconfigurable hardware like the input channel and the descrambler. The output channels are associated to the PID-filter through the value assigned to the PID-filter, namely the PID value. The firmware links the output channel to the data from PID-filter via common PID-value association.
*
*@param pidfilter_id  The PID Filter ID.  In/Out: Out
*@return CSDMUX_HANDLE PID filter handle
*@note Open a PID filter and get a PID Filter handle to operate the PID Filter. There are 64 PID Filters can be opened.
*The PID filter has responsibility for filtrating input data which are form tuner or DMA according the PID filter��s configuration.
*@see CSDEMUX_PIDFT_SetChannel() CSDEMUX_PIDFT_SetPID() CSDEMUX_PIDFT_MallocDES()
*/
CSDEMUX_HANDLE CSDEMUX_PIDFT_Open(CSDEMUX_PIDFT_ID pidfilter_id);

/**
* Closes the PID filter.
*
*@param handle	The PID filter handle. In/Out: In
*@return CSAPI_FAILED	Close the PID filter failed.
*				CSAPI_SUCCEED	Close the PID filters succeed.
*@note Close the PID filter. And then the PID filter will be free and waiting for Open.
*/
CSAPI_RESULT CSDEMUX_PIDFT_Close(CSDEMUX_HANDLE handle);


/**
* Choose a channel to get data. Associate input channel to PID filter identified by handle.
*
*@param handle	The PID filter handle. In/Out: In
*@param chl_id	The channel ID.	In/Out: In
*@return CSAPI_FAILED	Set channel failed.\n
*			    CSAPI_SUCCEED	Set channel succeed.
*@note Binds an input channel to specified PID filter. And then the PID filter can deal with input data to this input channel .There are two channels which are input channel0 and input channel1.
*/
CSAPI_RESULT CSDEMUX_PIDFT_SetChannel(CSDEMUX_HANDLE handle, CSDEMUX_CHL_ID chl_id);

/**
* Set the PID-filter��s PID. Actual TS stream PID is set into the PID-filter identified by handle.
*
*@param handle	The PID filter handle. In/Out: In
*@param pid	PID input by user. In/Out: In
*@return CSAPI_FAILED	Set PID failed.\n
*				CSAPI_SUCCEED	Set PID succeed.
*@note Set the PID-filter��s PID. And then the PID filter can filter corresponding data which include the same PID.
*/
CSAPI_RESULT CSDEMUX_PIDFT_SetPID(CSDEMUX_HANDLE handle, unsigned short pid);

/**
* To get the descramble channel��s number which are free. Total number of available descrambler engines is returned. A total of 8 descrambler engines are available.
*
*@param handle	The PID filter handle. In/Out: In
*@param des_number Descrambler number	The value of free descrambler number.
*@return CSAPI_FAILED   Set PID failed.\n
*               CSAPI_SUCCEED   Set PID succeed.
*@note To get the descramble number which are free. If there is no free descrambler, the function will return 0.
*/
CSAPI_RESULT CSDEMUX_PIDFT_GetFreeDESNumber(CSDEMUX_HANDLE handle, unsigned int * des_number);

/**
* Allocate a descrambler to specified PID-filter. If at least one descrambler is available, it is allocated and bound to PID-filter identified by given handle.
*
*@param handle	The PID filter handle. In/Out: In
*@return CSAPI_FAILED	Get descrambler failed.\n
*				CSAPI_SUCCEED	Get descrambler succeed.
*@note Allocate a descrambler to the specified PID-filter. And then the PID-filter can decrypt TS data which are encrypted.
*/
CSAPI_RESULT CSDEMUX_PIDFT_MallocDES(CSDEMUX_HANDLE handle);

/**
* Free the descrambler which is bound to specified PID-filter. The descrambler associated with PID-filter becomes free for re-allocation and is no more associated with PID-filter identified by given handle.
*
*@param handle	The PID filter handle. In/Out: In
*@return CSAPI_FAILED	Free descramble failed.
*				CSAPI_SUCCEED	Free descramble succeed.
*@note Free the descrambler which is bound to specified PID-filter.If operation succeed, the descrambler will be free, and waiting for allocated. And then the PID-filter can��t support decrypting function.
*/
CSAPI_RESULT CSDEMUX_PIDFT_FreeDES(CSDEMUX_HANDLE handle);

/**
* Set odd key value to descrambler device associated with a PID-filter identified by given handle.
*
*@param handle	The PID filter handle. In/Out: In
*@param	odd_key	The odd key value
*@return CSAPI_FAILED	Free descramble failed.\n
*				CSAPI_SUCCEED	Free descramble succeed.
*@note Set odd key value to descrambler device associated with a PID-filter identified by given handle.
*/
CSAPI_RESULT CSDEMUX_PIDFT_SetDESOddKey(CSDEMUX_HANDLE handle, const unsigned char *const odd_key);

/**
* Set even key value to descrambler device associated with a PID-filter identified by given handle.
*
*@param handle	The PID filter handle.	In/Out:In
*@param	even_key	The odd key value
*@return CSAPI_FAILED	Free descramble failed.\n
*				CSAPI_SUCCEED	Free descramble succeed.
*@note Set even key value to descrambler device associated with a PID-filter identified by given handle.
*/
CSAPI_RESULT CSDEMUX_PIDFT_SetDESEvenKey(CSDEMUX_HANDLE handle, const unsigned char *const even_key);

/**
* Enable the descrambler which is bound to specified PID-filter.
*
*@param handle	The PID filter handle. In/Out: In
*@return CSAPI_FAILED	Enable descrambler failed.\n
*				CSAPI_SUCCEED	Enable descrambler succeed.
*@note Enable the descrambler which is bound to specified PID-filter. Also, the specified PID-filter must have been allocated a descrambler. Or else the operation will fail.
*/
CSAPI_RESULT CSDEMUX_PIDFT_EnableDES(CSDEMUX_HANDLE handle);

/**
* Disable  the descrambler which is bound to specified PID-filter.
*
*@param handle	The PID filter handle.	In/Out: In
*@return CSAPI_FAILED	Enable descrambler failed.\n
*				CSAPI_SUCCEED	Enable descrambler succeed.
*@note Disable  the descrambler which is bound to specified PID-filter. Also, the specified PID-filter must have been allocated a descrambler. Or else the operation will fail.
*/
CSAPI_RESULT CSDEMUX_PIDFT_DisableDES(CSDEMUX_HANDLE handle);

/**
* Get filter DES key index.
*
*@param handle	The channel handle.	In/Out: In
*@param des_idx	The DES key index getted.In/Out: Out
*@return CSAPI_FAILED	DES key index get failed.\n
*				CSAPI_SUCCEED	DES key index get succeed.
*/
CSAPI_RESULT CSDEMUX_PIDFT_GetDESIdx(CSDEMUX_HANDLE handle, CSDEMUX_DES_ID * des_idx);

/**
* Enable the PID-filter device. This API starts the TS-stream filtering process based on the PID value in the PID-filter. TS-stream(include streams need descrambling) from the associated input channel will be passed for PID-match -checking by the PID filter device and will be subsequently send to MCU firmware for parsing into ES or SECTION.
*
*@param handle 	A PID filter handle. In/Out: In
*@return CSAPI_FAILED	Enable the PID-filter failed.\n
*				CSAPI_SUCCEED	Enable the PID-filter succeed.
*@note Enable a specified PID-filter. And then the PID-filter will filtrate the corresponding data and input to channel.
*/
CSAPI_RESULT CSDEMUX_PIDFT_Enable(CSDEMUX_HANDLE handle);

/**
* Disable a specified PID-filter. This stops the filtering based on the PID-value in the PID-filter.
*
*@param handle	The PID filter handle. In/Out: In
*@return CSAPI_FAILED	Disable the PID-filter failed.\n
*				CSAPI_SUCCEED	Disable the PID-filter succeed.
*@note Disable a specified PID-filter.
*/
CSAPI_RESULT CSDEMUX_PIDFT_Disable(CSDEMUX_HANDLE handle);

/* channel interface */

/**
* Allocate input channel devices by seriel operations of creating device object and initializing,and return void * to the CSDEMUX_HANDLE which contain parameters of input channel object. anyone who operates with the void * need to convert it into (CSDEMUX_CHL *).
*
*@param chl_id	The channel ID.	In/Out: In
*@return CSDMUX_HANDLE	Channel handle
*@note Use this function to allocate a channel device.
*/
CSDEMUX_HANDLE CSDEMUX_CHL_Open(CSDEMUX_CHL_ID chl_id);

/**
* Free the input channel device.
*
*@param 	handle	The channel handle.	In/Out: In
*@return CSAPI_FAILED	Close the channel failed.\n
*				CSAPI_SUCCEED	Close the channels succeed.
*@note Use this function to free a channel device.
*/
CSAPI_RESULT CSDEMUX_CHL_Close(CSDEMUX_HANDLE handle);

/**
* Set the input mode of specified channel. Input channel identified by handle is set to consume tuner-input or DMA input.
*
*@param handle	The channel handle.	In/Out: In
*@param input_mod	Input mode, tuner or DMA.	In/Out: In
*@return CSAPI_FAILED	Set channel input mode failed.\n
*				CSAPI_SUCCEED	Set channel input mode succeed.
*@note You can choose two input mode, tuner or DMA for specified channel device. This function implements channel��s data source choosing.There are four data sources (tuner0, tuner1, DMA0 and DMA1) for channel. But each channel one to one for tuner and DMA. If the channel is channel0, the data source must be tuner0 or dma0. Also, if the channel is channel1, the data source must be tuner1 or DMA1.
*/
CSAPI_RESULT CSDEMUX_CHL_SetInputMode(CSDEMUX_HANDLE handle, CSDEMUX_INPUT_MOD input_mod);

/**
* Set tuner input mode.
*
*@param handle	The channel handle.	In/Out: In
*@param input_mod	The input mode parameter.	In/Out: In
*@return CSAPI_FAILED	Set tuner input mode failed.\n
*				CSAPI_SUCCEED	Set tuner input mode succeeds.
*/
CSAPI_RESULT CSDEMUX_CHL_SetTunerInputMode(CSDEMUX_HANDLE handle, CSDEMUX_TUNER_MOD input_mod);

/**
* Set demux serial mode.
*
*@param handle	The channel handle.	In/Out: In
*@param serial_mode	The serial mode parameter.	In/Out: In
*@return CSAPI_FAILED	Set demux serial mode failed.\n
*				CSAPI_SUCCEED	Set demux serial mode succeeds.
*/
CSAPI_RESULT CSDEMUX_CHL_SetSerialMode(CSDEMUX_HANDLE handle, CSDEMUX_SERIAL_MODE serial_mode);

/**
* Enable tuner input
*
*@param handle	The channel handle.	In/Out: In
*@param t_id	Tuner ID.	In/Out: In
*@return CSAPI_FAILED	Enable tuner failed.\n
*				CSAPI_SUCCEED	Enable tuner succeeds.
*/
CSAPI_RESULT CSDEMUX_TUNER_Enable(CSDEMUX_HANDLE handle,CSDEMUX_TUNER_ID t_id);

/**
* Disable tuner input
*
*@param handle	The channel handle.	In/Out: In
*@param t_id	Tuner ID.	In/Out: In
*@return CSAPI_FAILED	Disable tuner failed.\n
*				CSAPI_SUCCEED	Disable tuner succeeds.
*/
CSAPI_RESULT CSDEMUX_TUNER_Disable(CSDEMUX_HANDLE handle,CSDEMUX_TUNER_ID t_id);

/**
* Set demux des type.
*
*@param handle	The channel handle.	In/Out: In
*@param des_type	The des type parameter. In/Out: In
*@return CSAPI_FAILED	Set demux des type failed.\n
*				CSAPI_SUCCEED	Set demux des type succeeds.
*/
CSAPI_RESULT CSDEMUX_CHL_SetDesType(CSDEMUX_HANDLE handle, CSDEMUX_DES_TYPE des_type);

/**
* Set demux stream type.
*
*@param handle	The channel handle.	In/Out: In
*@param stream_type	The stream type parameter.	In/Out: In
*@return CSAPI_FAILED	Set demux stream type failed.\n
*				CSAPI_SUCCEED	Set demux stream type succeeds.
*/
CSAPI_RESULT CSDEMUX_CHL_SetStreamType(CSDEMUX_HANDLE handle, CSDEMUX_STREAM_TYPE stream_type);

/**
* Reset a specified input channel device. The input channel identified by handle is reset.
*
*@param handle	The channel handle.	In/Out: In
*@return CSAPI_FAILED	Channel reset failed.\n
*				CSAPI_SUCCEED	Channel reset succeeds.
*@note Use this function to reset the specified channel.
*/
CSAPI_RESULT CSDEMUX_CHL_Reset(CSDEMUX_HANDLE handle);

/**
* Enable the input channel device. This enables the input channel identified by handle to pass stream from its source to associated PID-filters.
*
*@param handle	The channel handle.	In/Out: In
*@return CSAPI_FAILED	The channel enable failed.\n
*				CSAPI_SUCCEED	The channel enable succeed.
*@note Use this function to enable the channel device. And then it will receive TS-stream to be filtered by associated PID-filters.
*@see CSDEMUX_CHL_SetInputMode: Define data source.
*@see CSDEMUX_PIDFT_SetChannel: For bind down channel to PID-filter.
*/
CSAPI_RESULT CSDEMUX_CHL_Enable(CSDEMUX_HANDLE handle);

/**
* Disable the input channel device.
*
*@param handle	The channel handle.In/Out: In
*@return CSAPI_FAILED	The channel disable failed.\n
*				CSAPI_SUCCEED	The channel disable succeed.
*@note Use this function to disable the channel device. And then the channel can��t receive data. But the channel��s configurations also valid like buffer base address, unit size and so on.
*/
CSAPI_RESULT CSDEMUX_CHL_Disable(CSDEMUX_HANDLE handle);

/**
* Clear demux buffer.
*
*@param handle	The channel handle.	In/Out: In
*@return CSAPI_FAILED	Clear demux buffer failed.\n
*				CSAPI_SUCCEED	Clear demux buffer succeeds.
*/
CSAPI_RESULT CSDEMUX_CHL_ClearBuffer(CSDEMUX_HANDLE handle);

/**
* Write data to DMA. Data is written to DMA channel associated with input channel identified by handle.
*
*@param handle	The channel handle.	In/Out: In
*@param buf	The data buffer base address. In/Out: In
*@param len	The data length.	In/Out: In
*@return CSAPI_FAILED	DMA write failed.\n
*				CSAPI_SUCCEED	DMA write succeed.
*@note Write data to DMA.
*/
CSAPI_RESULT CSDEMUX_CHL_DMA_Write(CSDEMUX_HANDLE handle, const unsigned char *const buf, unsigned int len);

/**
* Reset DMA. DMA function associated with input channel identified by handle is reset.
*
*@param handle	The channel handle. In/Out: In
*@param dma_id DMA Identifier
*@return CSAPI_FAILED	The channel DMA reset failed.\n
*				CSAPI_SUCCEED	The channel DMA reset succeeds.
*@note Use this function to reset DMA.
*/
CSAPI_RESULT CSDEMUX_CHL_DMA_Reset(CSDEMUX_HANDLE handle,unsigned int dma_id);

/*
 * the below 1 methods for optimizing CHL DMA transfer too.
 * you can take any one of them.
 */

 /**
* Write data support DMA mode.
*
*@param handle	The channel handle.	In/Out: In
*@param func	Callback function to write the data to Xport buffer.	In/Out: In
*@param len	How many data to write, the second parameter of callback function.	In/Out: In
*@param context	The third parameter of callback function, the context of read data. For example, can pass the filename by this parameter. In/Out: In
*@return CSAPI_FAILED	Failed .\n
*				CSAPI_SUCCEED	Succeeds.
*@note This function will map XPORT channel buffer to user space and write data to this buffer directly, this will reduce one time memory copy.
*/
CSAPI_RESULT CSDEMUX_CHL_DMA_GenericWrite(CSDEMUX_HANDLE handle, int (*func) (void *buff, int sz, void *context),
					  unsigned int len, void *context);
/* video output interface */

/**
* Allocate a video output channel device. Open video output channel and allocate handle. The CSDEMUX_HANDLE is of type (void *) and therefore can be cast to many handle types including PID-filter, input channel and other output channels. A total of 2 video output channels are supported.
*
*@param vid_id	The video ID.	In/Out: In
*@return CSDMUX_HANDLE	The video channel handle
*@note Use this function to allocate a video channel device. This device is used to get video ES and deal with it.
*/
CSDEMUX_HANDLE CSDEMUX_VID_Open(CSDEMUX_VIDOUT_ID vid_id);

/**
* Free the video output channel device. Release and close video output channel.
*
*@param handle	The video channel ID.	In/Out: In
*@return CSAPI_FAILED	Close video channel failed.\n
*				CSAPI_SUCCEED	Close video channel succeed.
*@note Use this function to free a video channel device.
*/
CSAPI_RESULT CSDEMUX_VID_Close(CSDEMUX_HANDLE handle);

 /**
*  Set low-latency mode for video output. This mode aims to output video frame
*   immediately and add proper stuffing pattern for following video decoding.
*   This mode is built on assumption that Tx side packet one video into one PES packet!
*   The behavior is described as below:
*     If PES packet has length information, it will output video frame and add stuffing
*     pattern immediately once one PES packet is arrived.
*     If PES packet has not length information, it will output video frame and add
*     stuffing pattern after next PES packet is coming.
*
*@param handle	The video channel ID.	In/Out: In
*@return CSAPI_FAILED	Set video low latancy failed.\n
*				CSAPI_SUCCEED	Set video low latancy succeed.
*/
CSAPI_RESULT CSDEMUX_VID_SetLowLatencyMode(CSDEMUX_HANDLE handle);

/**
* Set the PID for video channel device��s PID-filter. Set the value of PID to be used by the MCU firmware to demux filtered TS-stream from the PID-filters. Explicit binding of the video output channel to associated PID-filter is not necessary. The MCU firmware looks at input TS-stream packet header for matching PID-value and output��s payload to this channel in whichever format the output channel is configured.
*
*@param handle	The video channel handle.	In/Out: In
*@param pid	The video data PID value.	In/Out: In
*@return CSAPI_FAILED	Set video channel PID failed.\n
*				CSAPI_SUCCEED	Set video channel PID succeed.
*@note If you want to set PID for the PID-filter of video channel device, you should use this function.
*/
CSAPI_RESULT CSDEMUX_VID_SetPID(CSDEMUX_HANDLE handle, unsigned short pid);

/**
* Set the video channel device��s output mode. See definition of DEMUX_OUTPUT_MOD data type for explanation.
*
*@param handle	The video channel handle.	In/Out: In
*@param block_mod	The video channel output mode.	In/Out: In
*@return CSAPI_FAILED	Set video channel output mode failed.\n
*				CSAPI_SUCCEED	Set audio channel output mode succeeds.
*@note Set the video channel output mode, and there are two modes block and unblock.
*/
CSAPI_RESULT CSDEMUX_VID_SetOutputMode(CSDEMUX_HANDLE handle, CSDEMUX_OUTPUT_MOD block_mod);

/**
* Set the demux input channel for video
*
*@param handle	The video channel handle.	In/Out: In
*@param sw_mod	The input demux channel.	In/Out: In
*@return CSAPI_FAILED	Set demux input channel for video failed.\n
*				CSAPI_SUCCEED	Set demux input channel for video succeeds.
*/
CSAPI_RESULT CSDEMUX_VID_SetSwitchMode(CSDEMUX_HANDLE handle, CSDEMUX_SWITCH_MOD sw_mod);

/**
* Reset a video output channel device. Video output channel identified by handle is reset.
*
*@param handle	The video channel handle.	In/Out: In
*@return CSAPI_FAILED	Reset video channel device failed.\n
*				CSAPI_SUCCEED	Reset video channel device succeed.
*@note Reset a video channel device.
*/
CSAPI_RESULT CSDEMUX_VID_Reset(CSDEMUX_HANDLE handle);

/**
* Enable the video output channel device. Enables the video output channel to make it accept by demuxed video.
*
*@param handle	The video channel handle.	In/Out: In
*@return CSAPI_FAILED	The video channel enable failed.\n
				CSAPI_SUCCEED	The video channel enable succeed.
*@note You should use this function to enable video channel device.
*@see CSDEMUX_VID_Disable
*/
CSAPI_RESULT CSDEMUX_VID_Enable(CSDEMUX_HANDLE handle);

/**
* Disable the video output channel device. This function stops the video output channel from accepting video being demuxed.
*
*@param handle	The video channel handle.	In/Out: In
*@return CSAPI_FAILED	The video channel disable failed.\n
				CSAPI_SUCCEED	The video channel disable succeed.
*@note If you want to hold the device but do not need video data, you should use this function.
*@see CSDEMUX_VID_Enable
*/
CSAPI_RESULT CSDEMUX_VID_Disable(CSDEMUX_HANDLE handle);

/* audio output interface */

/**
* Allocate an audio output channel device.
*
*@param aud_id	The audio ID. In/Out: In
*@return CSDMUX_HANDLE	The audio channel handle.
*@note Use this function to allocate an audio channel device.
*/
CSDEMUX_HANDLE CSDEMUX_AUD_Open(CSDEMUX_AUDOUT_ID aud_id);

/**
* Free the audio output channel device.
*
*@param handle	The audio channel ID.	In/Out: In
*@return CSAPI_FAILED	Close audio channel failed.\n
*				CSAPI_SUCCEED	Close audio channel succeed.
*@note Use this function to free an audio channel device.
*/
CSAPI_RESULT CSDEMUX_AUD_Close(CSDEMUX_HANDLE handle);

/**
* Set PID-value for the audio output channel. Definition is similar to CSDEMUX_VID_SetPID.
*
*@param handle	The audio channel handle. In/Out: In
*@param pid	The audio data PID value.	In/Out: In
*@return CSAPI_FAILED	Set audio channel PID failed.\n
*				CSAPI_SUCCEED	Set audio channel PID succeed.
*@note If you want to set PID for the audio channel device��s PID-filter, you should use this function.
*/
CSAPI_RESULT CSDEMUX_AUD_SetPID(CSDEMUX_HANDLE handle, unsigned short pid);

/**
* Set output mode for an audio output channel device.
*
*@param handle	The audio channel handle.	In/Out: In
*@param block_mod	The audio channel output mode.	In/Out: In
*@return CSAPI_FAILED	Set audio channel output mode failed.\n
*				CSAPI_SUCCEED	Set audio channel output mode succeeds.
*@note Set output mode for an audio channel device, and there are two modes, block and unblock.
*/
CSAPI_RESULT CSDEMUX_AUD_SetOutputMode(CSDEMUX_HANDLE handle, CSDEMUX_OUTPUT_MOD block_mod);

/**
* Set demux switch mode.
*
*@param handle	The channel handle.	In/Out: In
*@param sw_mod	The demux switch mode parameter.	In/Out: In
*@return CSAPI_FAILED	Demux switch mode set failed.\n
*				CSAPI_SUCCEED	Demux switch mode set succeeds.
*/
CSAPI_RESULT CSDEMUX_AUD_SetSwitchMode(CSDEMUX_HANDLE handle, CSDEMUX_SWITCH_MOD sw_mod);

/**
* Reset an audio output channel device.
*
*@param handle	The audio channel handle.	In/Out: In
*@return CSAPI_FAILED	Reset audio channel device failed.\n
*				CSAPI_SUCCEED	Reset audio channel device succeed.

*@note Reset an audio channel device.
*/
CSAPI_RESULT CSDEMUX_AUD_Reset(CSDEMUX_HANDLE handle);

/**
* Enable the audio output channel device.
*
*@param handle	The audio channel handle. In/Out: In
*@return CSAPI_FAILED	The audio channel enable failed.\n
*				CSAPI_SUCCEED	The audio channel enable succeed.
*@note You should use this function to enable audio channel device.
*/
CSAPI_RESULT CSDEMUX_AUD_Enable(CSDEMUX_HANDLE handle);

/**
* Disable the audio output channel device.
*
*@param handle	The audio channel handle.	In/Out: In
*@return CSAPI_FAILED	The audio channel disable failed.\n
*				CSAPI_SUCCEED	The audio channel disable succeed.
*@note If you don��t need audio data, but want to hold the device, you should use this function.
*/
CSAPI_RESULT CSDEMUX_AUD_Disable(CSDEMUX_HANDLE handle);

/* PCR interface */

/**
* Allocate a PCR output channel device. Only one PCR output device is supported.
*
*@param pcr_id	The PCR channel ID.	In/Out: In
*@return CSDMUX_HANDLE	The PCR channel device handle.
*@note Use this function to allocate a PCR channel device, and then you would choose a channel for it.
*@see CSDEMUX_PCR_SetSwitchMode: Bind a channel device.
*/
CSDEMUX_HANDLE CSDEMUX_PCR_Open(CSDEMUX_PCRDEV_ID pcr_id);

/**
* Free the PCR output channel device.
*
*@param handle	The PCR channel ID.	In/Out: In
*@return CSAPI_FAILED	Close The PCR channel failed.\n
*				CSAPI_SUCCEED	Close The PCR channel succeed.
*@note You can use this function when you don��t nee the PCR channel device.
*/
CSAPI_RESULT CSDEMUX_PCR_Close(CSDEMUX_HANDLE handle);

/**
* Set pcr pid.
*
*@param handle	The channel handle.	In/Out: In
*@param pid	Pid value to be set.	In
*@return CSAPI_FAILED	Pid set failed.\n
*				CSAPI_SUCCEED	Pid set succeed.
*/
CSAPI_RESULT CSDEMUX_PCR_SetPID(CSDEMUX_HANDLE handle, unsigned short pid);

/**
* Set input demux channel for PCR
*
*@param handle	The channel handle.	In/Out: In
*@param sw_mod	Demux input channel ID.	In/Out: In
*@return CSAPI_FAILED	Set input demux channel for PCR failed.\n
*				CSAPI_SUCCEED	Set input demux channel for PCR succeed.
*/
CSAPI_RESULT CSDEMUX_PCR_SetSwitchMode(CSDEMUX_HANDLE handle, CSDEMUX_SWITCH_MOD sw_mod);

/**
* Enable the PCR output channel device.
*
*@param handle	The PCR channel ID.	In/Out: In
*@return CSAPI_FAILED	The PCR channel enable failed.\n
*				CSAPI_SUCCEED	The PCR channel enable succeed.
*@note Use this function to enable the PCR channel device, and then you will get the filtrated data.
*/
CSAPI_RESULT CSDEMUX_PCR_Enable(CSDEMUX_HANDLE handle);

/**
* Disable the PCR output channel device.
*
*@param handle	The PCR channel ID.	In/Out: In
*@return CSAPI_FAILED	The PCR channel disable failed.\n
*				CSAPI_SUCCEED	The PCR channel disable succeed.
*@note If you don��t need PCR data, but want to hold the device, you should use this function.
*/
CSAPI_RESULT CSDEMUX_PCR_Disable(CSDEMUX_HANDLE handle);

/**
* Get PCR value
*
*@param handle	The PCR channel ID.	In/Out: In
*@param value	The PCR value.	Out
*@return CSAPI_FAILED	Get PCR value failed.\n
*				CSAPI_SUCCEED	Get PCR value succeed.
*/
CSAPI_RESULT CSDEMUX_PCR_GetPCR(CSDEMUX_HANDLE handle, CSDEMUX_PCRVALUE *value);

/* Filter interface */

/**
* Allocate a data filter output device. 32 data filter output devices are supported.
*
*@param filter_id	The filter ID.	In/Out: In
*@return CSDMUX_HANDLE	The filter device handle.
*@note Allocate a filter device by using this function. There are 32 filter devices to be allocated.
*@see DEMUX_FILTER_ID
*/
CSDEMUX_HANDLE CSDEMUX_Filter_Open(CSDEMUX_FILTER_ID filter_id);

/**
* Close the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Close The filter failed.\n
*				CSAPI_SUCCEED	Close The filter succeeds.
*@note If you don��t need the function of filter, you should use this function.
*/
CSAPI_RESULT CSDEMUX_Filter_Close(CSDEMUX_HANDLE handle);

/**
* Set filter work mode to the required type. According to the output devices filter, the filter can be set into SECTION/TS/PES/ES of CSDEMUX_FILTER_TYPE. Among the types SECTION type was the most frequently set as work mode.
*
*@param handle	The filter handle. In/Out: In
*@param type	The filter type.	In
*@return CSAPI_FAILED	Set filter type failed.\n
*				CSAPI_SUCCEED	Set filter type succeed.
*@note Set a function for the new allocated filter device and use it.
*@see CSDEMUX_FILTER_Open\n
*			DEMUX_FILTER_TYPE
*/
CSAPI_RESULT CSDEMUX_Filter_SetFilterType(CSDEMUX_HANDLE handle, CSDEMUX_FILTER_TYPE type);

/**
* Set filters of output channel to match one of the input channel 0-3 , Filter would be acquiescently attached to input channel0 if this api wasn't called
*
*@param handle	The filter handle.	In/Out: In
*@param sw_mode	The channel id that filter attached to. 	In
*@return CSAPI_SUCCEED	Set input channel successful
*/
CSAPI_RESULT CSDEMUX_Filter_SetSwitchMode(CSDEMUX_HANDLE handle, CSDEMUX_SWITCH_MOD sw_mode);

/**
* Set filter parameters for the data filter output device. 12 byte match and 12 byte mask supported.
*
*@param handle	The filter handle.	In/Out: In
*@param filter_ptr	Match buffer pointer.	In/Out: In
*@param mask_ptr	Mask buffer pointer.	In/Out: In
*@return CSAPI_FAILED	Set filter parameter failed.\n
*				CSAPI_SUCCEED	Set filter parameter succeed.
*@note There are 24 byte data parameters you should set, including 12 byte match and 12 byte mask.
*/
CSAPI_RESULT CSDEMUX_Filter_SetFilter(CSDEMUX_HANDLE handle, const unsigned char *const filter_ptr,
				      const unsigned char *const mask_ptr);

/**
* Get the number of the free CRC devices. A total of 16 CRC engines are supported.
*
*@param handle	The filter handle.	In/Out: In
*@param crc_number	Free CRC number.	In/Out: In
*@return CSAPI_FAILED	Get free CRC number failed.\n
*				CSAPI_SUCCEED	Get free CRC number succeeds.
*@note Using this function, you will get the number of free CRC device.
*/
CSAPI_RESULT CSDEMUX_Filter_GetFreeCRCNumber(CSDEMUX_HANDLE handle, unsigned int *crc_number);

/**
* Allocate a CRC device for data filter output device. A free CRC engine is allocated to the data filter output device identified by handle.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Allocate CRC device failed.\n
*				CSAPI_SUCCEED	Allocate CRC device succeeds.
*@note Allocate a CRC device for the filter device by using this function.
*/
CSAPI_RESULT CSDEMUX_Filter_MallocCRC(CSDEMUX_HANDLE handle);

/**
* Free CRC device for the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Free CRC device failed.\n
*				CSAPI_SUCCEED	Free CRC device succeed.
*@note If you want to release the CRC device which bind to the filter, this function should be adopt.
*/
CSAPI_RESULT CSDEMUX_Filter_FreeCRC(CSDEMUX_HANDLE handle);

/**
* Enable the CRC device for the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Enable CRC device failed.\n
*				CSAPI_SUCCEED	Enable CRC device succeeds.

*@note If you bind down a CRC device for the filter device and want to enable it, you should use this function.
*/
CSAPI_RESULT CSDEMUX_Filter_EnableCRC(CSDEMUX_HANDLE handle);

/**
* Disable a CRC device for the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Disable CRC device failed.\n
*				CSAPI_SUCCEED	Disable CRC device succeeds.
*@note If the filter device is not need CRC device, you should use this function to disable it.
*/
CSAPI_RESULT CSDEMUX_Filter_DisableCRC(CSDEMUX_HANDLE handle);

/**
* Register callback function for notification of section data filter output events. Events are defined in data type CSDEMUX_SECEVENT.
*
*@param handle	The filter handle.	In/Out: In
*@param call_back_function	Name of callback function registered for notification. Parameters consists of CSDEMUX_HANDLE handle and pointer to return CSDEMUX_SECEVENT *event.
*@param demux_event	Event registration of type CSDEMUX_SECEVENT for callback trigger.
*@param event_enable	Selectively enable (event_enable =1) or disable (event_enable=0) the callback notification.
*@return CSAPI_FAILED	Notification registration failed\n
*				CSAPI_SUCCEED	Notification registration succeeded
*/
CSAPI_RESULT CSDEMUX_FILTER_SetSectionNotify(CSDEMUX_HANDLE handle,
					     void (*call_back_function) (CSDEMUX_HANDLE, CSDEMUX_SECEVENT *),
					     CSDEMUX_SECEVENT demux_event,
					     int event_enable);

/**
* Enable or disable CRC error when section data output from section output filter device. Default is the data with CRC error is dropped.
*
*@param handle	The filter handle.	In/Out: In
*@param enable		If enable=1, enable saving of section data upon CRC error.	If enable=0, disable saving of section data upon CRC error.
*@return CSAPI_FAILED	Disable the filter failed.\n
*				CSAPI_SUCCEED	Disable the filter succeed.
*/
CSAPI_RESULT CSDEMUX_FILTER_SetSaveErrSectionDataFlag(CSDEMUX_HANDLE handle, int enable);

/**
* Set PID value for the data filter output device. Definition is similar to CSDEMUX_VID_SetPID.
*
*@param handle The filter handle. In/Out: In
*@param pid	Pid value. In
*@return CSAPI_FAILED	The filter adds PID failed.\n
*				CSAPI_SUCCEED	The filter adds PID succeeds.
*@note Use this function to set PID value for the filter device.
*/
CSAPI_RESULT CSDEMUX_Filter_AddPID(CSDEMUX_HANDLE handle, unsigned short pid);

/**
* Clear PID value for the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	The filter adds PID failed.\n
*				CSAPI_SUCCEED	The filter adds PID succeeds.
*@note Use this function to clear PID value for the filter device.
*/
CSAPI_RESULT CSDEMUX_Filter_ResetPID(CSDEMUX_HANDLE handle);

/**
* Monitor if data is available in data filter output device until timeout. Blocking is not recommended.
*
*@param handle	The filter handle.	In/Out: In
*@param ms_timeout	The time read data timeout.	In/Out: In
*@return CSAPI_FAILED	Read timeout.\n
*				CSAPI_SUCCEED	The filter device has gotten data.
*@note If user want monitor the data be or not be filtrated in some time, this function should be adopt
*/
CSAPI_RESULT CSDEMUX_Filter_ReadWait(CSDEMUX_HANDLE handle, unsigned int ms_timeout);

/**
* Check the data size available in data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@param size	The data size.	In
*@return CSAPI_FAILED	Operate failed.\n
*				CSAPI_SUCCEED	Operate succeed.
*@note If you want to check the data size the filter filtrated, you should use this function.
*/
CSAPI_RESULT CSDEMUX_Filter_CheckDataSize(CSDEMUX_HANDLE handle, unsigned int *size);

/**
* Read data which is available in the data filter output device.
*
*@param handle	The PID filter handle.	In/Out: In
*@param buf	The buffer base address.	In/Out: In
*@param size	The buffer size.	In/Out: In
*@return CSAPI_FAILED	Read buffer data failed.\n
*				CSAPI_SUCCEED	Read buffer data succeed.
*@note If the data is filtrated by the filter but not by the user, use this function to read.
*/
CSAPI_RESULT CSDEMUX_Filter_ReadData(CSDEMUX_HANDLE handle, unsigned char *buf, unsigned int *size);

/**
* Checks the data of the next section to help filtering the output for the given data
*
*@param handle	The PID filter handle.	In/Out: In
*@param num	The section number in buffer.	In/Out: In
*@return CSAPI_FAILED	Check section number failed.\n
*				CSAPI_SUCCEED	Check section number succeed.
*@note Please use this function when you know the number of section can not be reached by user.
*/
CSAPI_RESULT CSDEMUX_Filter_CheckSectionNum(CSDEMUX_HANDLE handle, unsigned int *num);

/**
* Read section data which is available in data output filter.
*
*@param handle	The filter handle. In/Out: In
*@param buf	The buffer base address.	In/Out: Out
*@param size	The buffer size.	In/Out: Out
*@return CSAPI_FAILED	Read section data failed.\n
*				CSAPI_SUCCEED	Read section data succeed.
*@note If there are one or more section data, and is not gotten by user, use this function to read section data which filtrated by the filter.
*/
CSAPI_RESULT CSDEMUX_Filter_ReadSectionData(CSDEMUX_HANDLE handle, unsigned char *buf, unsigned int *size);

/*
* Enable the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Enable the filter failed.\n
*				CSAPI_SUCCEED	Enable the filter succeed.
*@note After setting the corresponding parameters of the filter, you should use this function to enable the filter device to get data.
*@see None
*/
CSAPI_RESULT CSDEMUX_Filter_Enable(CSDEMUX_HANDLE handle);

/*
* Disable the data filter output device.
*
*@param handle	The filter handle.	In/Out: In
*@return CSAPI_FAILED	Disable the filter failed.\n
*				CSAPI_SUCCEED	Disable the filter succeed.
*@note When the filter is enabled, you want to stop filtrating data, you should use the function.
*@see None
*/
CSAPI_RESULT CSDEMUX_Filter_Disable(CSDEMUX_HANDLE handle);

/**
* Read available data in the section filter of output channel from memory mapped to user space.
*
*@param handle	The output filter handle. In/Out: In
*@param func	The buffer operation func pointor to read filter data. In/Out: In
*@param len	The data length to read. In/Out:In
*@param context	The output buffer. In/Out: Output
*@return CSAPI_FAILED	Read buffer data failed.\n
*				CSAPI_SUCCEED	Read buffer data succeed.
*@note This api will map kernel space to user space and read data from user space, furthermore it will update write & read pointer in user space.
*/
CSAPI_RESULT CSDEMUX_FILTER_GenericRead(CSDEMUX_HANDLE handle, int (*func) (void *buff, int sz, void *context),
	                                          unsigned int len, void* context);

typedef enum {
    CSDEMUX_AV_NOT_SCRAMBLED = 0,
    CSDEMUX_AV_SCRAMBLED = 1,
}CSDEMUX_AV_SCRAMBLE_STATUS;

typedef void (*demux_av_scramble_cb)(CSDEMUX_HANDLE, CSDEMUX_AV_SCRAMBLE_STATUS );
CSAPI_RESULT CSDEMUX_VID_SetScrambleStatusNotify(CSDEMUX_HANDLE handle, demux_av_scramble_cb cb, int enable);
CSAPI_RESULT CSDEMUX_AUD_SetScrambleStatusNotify(CSDEMUX_HANDLE handle, demux_av_scramble_cb cb, int enable);

#ifdef __cplusplus
}
#endif

#endif
