/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csgpio.h
 * @ingroup csapi
 * Low Level GPIO API
 */
#ifndef __CSAPI_GPIO_H__
#define __CSAPI_GPIO_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"

typedef void *CSGPIO_HANDLE;

/*!
*@brief Definition error code for CSGPIO API interface.
* @see
*/
typedef enum {
    GPIO_SUCCESS = 0,///<Operation is success.
    GPIO_OPEN_ERROR,///<Open operation is failed.
	GPIO_CLOSE_ERROR,///<Close operation is failed.
	GPIO_READ_ERROR,///<Read operation is failed.
	GPIO_WRITE_ERROR,///<Write operation is failed.
	GPIO_BAD_PARAMATER,///<Input parameter is invalid.
	GPIO_SETDIR_ERROR,///<Set attribute is failed.
	GPIO_GETDIR_ERROR,///<Get attribute is failed.
	GPIO_ENABLE_IRQ_ERROR,///<Enable interrupt is failed.
	GPIO_DISABLE_IRQ_ERROR///<Disable interrupt is failed.
} CSGPIO_ErrCode;

/*!
*@brief Specifies the read or write direction of the GPIO.
* @see
*/
typedef enum {
	GPIO_DIRECTION_READ = 0,///<Specifies a port with only reading.
	GPIO_DIRECTION_WRITE ///<Specifies a port with only writing.
} CSGPIO_DIRECTION;

/*!
*@brief IRQ type of level sensitivity GPIO irq.
* @see
*/
typedef enum {
    GPIO_IRQ_TYPE_LEVEL_LOW = 0,
    GPIO_IRQ_TYPE_LEVEL_HIGH
}CSGPIO_IRQ_TYPE;
/*!
*@brief Event type of GPIO button.
* @see
*/
typedef enum {
    GPIO_IRQ_EVENT_PRESS,///<Specifies a gpio button callback with pressing.
    GPIO_IRQ_EVENT_RELEASE///<Specifies a gpio button callback with releasing.
}CSGPIO_IRQ_EVENT_TYPE;

/*!
*@brief Type of GPIO button callback function.
* @see
*/
typedef void (*GPIO_IRQ_CallBack)(CSGPIO_HANDLE handle, CSGPIO_IRQ_TYPE irq_type);

typedef int (*pf_PBCDeviceEventCallBack)(int state, unsigned int gpio);

/*!
 *@brief Open a particular GPIO.
 *
 *     @param	gpio     The index of the GPIO opened. In/Out : In
 *     @return  A valid handle on open success
 *     @note    None
 *     @see     None
 */
CSGPIO_HANDLE CSGPIO_Open(unsigned int gpio);

/*!
 *@brief Close an opened GPIO.
 *
 *     @param	handle    An opened GPIO handle (from the CSGPIO_Open() call). In/Out : In
 *     @return 	CSAPI_FAILED  close gpio failed.
 *     @return  CSAPI_SUCCEED close gpio succeed.
 *	 @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_Close(CSGPIO_HANDLE handle);

/*!
 *@brief Read the GPIO bits associated with the appointed handle.
 *
 *     @param	handle   An opened GPIO handle.          In/Out : In
 *     @param   bit      Pointer to the read data buffer In/Out : Out
 *     @return	CSAPI_SUCCEED  Read GPIO succeed
 *     @return	CSAPI_FAILED   Read GPIO failed
 *	 @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_Read(CSGPIO_HANDLE handle, unsigned char *bit);

/*!
 *@brief Write the GPIO bits associated with the appointed handle.
 *
 *     @param	handle    An opened GPIO handle.                    In/Out : In
 *     @param	bit       Write bit values, only 0 and 1 are valid. In/Out : In
 *     @return	CSAPI_SUCCEED Write GPIO succeed
 *     @return	CSAPI_FAILED  Write GPIO failed
 *	 @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_Write(CSGPIO_HANDLE handle, unsigned char bit);	/* only 0 and 1 are valid */

/*!
 *@brief Specifies the read or write attribute of the opened general purpose input output ports (GPIO).
 *
 *     @param	handle          An opened GPIO handle.                                In/Out : In
 *     @param	gpio_direction  Specifies a read/write attribute for the opened GPIO. In/Out : In
 *     @return	CSAPI_SUCCEED Specifies the read or write attribute succeed
 *     @return	CSAPI_FAILED  Specifies the read or write attribute failed
 *	 @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_SetDirection(CSGPIO_HANDLE handle, CSGPIO_DIRECTION gpio_direction);

/*!
 *@brief Get the read or write attribute of the opened general parallel input output ports (GPIO).
 *
 *     @param	handle          An opened GPIO handle.                                In/Out : In
 *     @param	gpio_direction  return the read/write attribute for the opened GPIO.  In/Out : Out
 *     @return	CSAPI_SUCCEED Get the read or write attribute succeed
 *     @return	CSAPI_FAILED  Get the read or write attribute failed
 *     @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_GetDirection(CSGPIO_HANDLE handle, CSGPIO_DIRECTION * gpio_direction);

/*!
 *@brief Get the error code of the current operation result.
 *
 *     @param	handle  An opened GPIO handle. In/Out : In
 *     @return	The error code of the current operation.
 *	 @note    None
 *     @see     None
 */
CSGPIO_ErrCode CSGPIO_GetErrCode(CSGPIO_HANDLE handle);

/*!
 *@brief Get the error string pointer of the current operation result.
 *
 *     @param	handle   An opened GPIO handle. In/Out : In
 *     @return	The error string pointer of the current operation result.
 *	 @note    None
 *     @see     None
 */
char *CSGPIO_GetErrString(CSGPIO_HANDLE handle);

/*!
 *@brief Register a callback function for GPIO interrupt
 *
 *     @param	handle       An opened GPIO handle.                            In/Out : In
 *     @param	first_trigger_type The event type for the first call back.     In/Out : In
 *     @param	callback     Be invoked when receive a GPIO irq.               In/Out : In
 *     @return	CSAPI_SUCCEED Register callback function succeed
 *     @return	CSAPI_FAILED  Register callback function failed
 *	   @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_RegisterIRQNotify(CSGPIO_HANDLE handle, CSGPIO_IRQ_TYPE first_trigger_type, GPIO_IRQ_CallBack callback);

/*!
 *@brief Unregister the callback function for GPIO interrupt
 *
 *     @param	handle        An opened GPIO handle.                             In/Out : In
 *     @return	CSAPI_SUCCEED Unregister callback function succeed
 *     @return	CSAPI_FAILED  Unregister callback function failed
 *	   @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_UnregisterIRQNotify(CSGPIO_HANDLE handle);

/*!
 *@brief Register the callback function for PBC
 *
 *     @param	callback     Callback function to be invoked based on the event received at the given GPIO. In/Out: In
 *     @param	gpio         The GPIO number to be checked for events. In/Out : In
 *     @return	CSAPI_SUCCEED Unregister callback function succeed
 *     @return	CSAPI_FAILED  Unregister callback function failed
 *	 @note    None
 *     @see     None
 */
CSAPI_RESULT CSGPIO_registerpbcCallback(pf_PBCDeviceEventCallBack callback, unsigned int gpio);

#ifdef __cplusplus
}
#endif

#endif				/* __CSAPI_GPIO_H */
