/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csi2c.h
 * Low Level I2C API
 * @ingroup csapi
 */
#ifndef __CSAPI_I2C_H__
#define __CSAPI_I2C_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"

#define I2C_SPEED_100K 1
#define I2C_SPEED_400K 2

#define I2C_ADDR_NOLOOP 0
#define I2C_ADDR_LOOP 1

typedef void *CSI2C_HANDLE;

/**
*Error code of the current I2C device.
*/
typedef enum {
	I2C_SUCCESS = 0,///<Operation successful.
	I2C_OPEN_ERROR,///<Open I2C slave device failed
	I2C_CLOSE_ERROR,///<Close I2C slave device failed.
	I2C_READ_ERROR,///<Read I2C slave device failed.
	I2C_WRITE_ERROR,///<Write I2C slave device failed.
	I2C_BAD_PARAMATER,///<Parameter error.
	I2C_SETATTR_ERROR,///<Set attribute of I2C slave device error.
	I2C_GETATTR_ERROR,///<Get attribute of I2C slave device error.
	I2C_ATTR_LOOPPARAMATER_ERROR,///<Invalid parameter of loop state.
	I2C_ATTR_SPEEDPARAMATER_ERROR,///<Invalid parameter of device speed.
	I2C_ATTR_SUBADDRPARAMATER_ERROR///<Invalid parameter of device sub address.
} CSI2C_ErrCode;

/**
*Define I2C device attribute.
* The operating parameters above can be changed by using CSI2C_SetAttr(). The current value can be obtained using CSI2C_GetAttr().
Some devices have multi-sub address (e.g. EEPROM), so that user can configure the number of the sub address (Using the subaddr_num parameter).
* @see CSI2C_HANDLE_t
*/
typedef struct {
	int speed;	///< I2C_SPEED_100K 1; I2C_SPEED_400K 2
	unsigned char loop;	///< I2C_ADDR_NOLOOP 0, I2C_ADDR_LOOP 1
	unsigned int subaddr_num;///<The number of sub address.Default is 1.
	unsigned int write_delayus;///<The time delayed when writing.
} CSI2C_Attr;

#ifdef MULTI_I2C_SUPPORT

/**
*The enum defines all I2C device ID.
*@note Need configurate kernel. Don't support I2C device 1 with kernel default setting.
*/
typedef enum _I2C_ID_
{
	i2c_0      =0,///<I2C device 0.
	i2c_1      =1,///<I2C device 1.
}I2C_ID;

/**
 *Open an I2C slave device.
 *
 *     @param	Slave device address  In/Out : In
 *     @param	Number of i2c (only kernel support the second i2c)In/Out : In
 *     @return	Slave device handle
 *
 *     @note    Open an I2C slave device and return a handle of the device. The handle can be used to operate this device. The definition of our slave address is different with the definition of standard slave address, our slave address is obtain by right shifting a bit from standard slave address.
 */
CSI2C_HANDLE CSI2C_Open(unsigned int chipaddress,I2C_ID I2C);
#else
CSI2C_HANDLE CSI2C_Open(unsigned int chipaddress);
#endif

/**
 *Close an I2C slave device.
 *
 *     @param	handle I2C slave device handle  In/Out : In
 *     @return	CSAPI_FAILED  close i2c slave device failed.
 *              CSAPI_SUCCEED close i2c slave device succeed.
 * @see CSI2C_Open()
 */
CSAPI_RESULT CSI2C_Close(CSI2C_HANDLE handle);

/**
 *Read data from device associated with a handle.
 *
 *     @param handle   I2C slave device handle                 In/Out : In
 *     @param subaddr  Start address of source data in device  In/Out : In
 *     @param buffer   Pointer to destination buffer           In/Out : Out
 *     @param num      Number of bytes to be read              In/Out : In
 *     @return	CSAPI_FAILED  read i2c data failed.
 *              CSAPI_SUCCEED read i2c data succeed.
 */
CSAPI_RESULT CSI2C_Read(CSI2C_HANDLE handle, unsigned int subaddr, char *buffer, unsigned int num);

/**
 *Write data to device associated with a handle.
 *
 *     @param handle   I2C slave device handle                 In/Out : In
 *     @param subaddr  Start address of source data in device  In/Out : In
 *     @param buffer   Pointer to the source buffer            In/Out : In
 *     @param num      Number of bytes to be written           In/Out : In
 *     @return	CSAPI_FAILED  I2C slave device write failed.
 *              CSAPI_SUCCEED I2C slave device write succeed.
 */
CSAPI_RESULT CSI2C_Write(CSI2C_HANDLE handle, unsigned int subaddr, char *buffer, unsigned int num);

/**
 *Return the current operation parameters associated with a handle.
 *
 *     @param handle   I2C slave device handle                 In/Out : In
 *     @param i2c_attr Pointer to the CSI2C_Attr structure     In/Out : Out
 *     @return	CSAPI_FAILED  Get parameters failed.
 *              CSAPI_SUCCEED Get parameters succeed.
 */
CSAPI_RESULT CSI2C_GetAttr(CSI2C_HANDLE handle, CSI2C_Attr * i2c_attr);

/**
 *Set the current operation parameters associated with a handle.
 *
 *     @param handle   I2C slave device handle                  In/Out : In
 *     @param i2c_attr Pointer to the attribute data structure  In/Out : In
 *     @return	CSAPI_FAILED  Set parameters failed.
 *              CSAPI_SUCCEED Set parameters succeed.
 */
CSAPI_RESULT CSI2C_SetAttr(CSI2C_HANDLE handle, CSI2C_Attr * i2c_attr);

/**
 *Return the error code of I2C driver.
 *
 *     @param  handle   I2C slave device handle                 In/Out : In
 *     @return The error code of I2C driver.
 */
CSI2C_ErrCode CSI2C_GetErrCode(CSI2C_HANDLE handle);

/**
 *Return the error string of I2C driver.
 *     @param  handle   I2C slave device handle                 In/Out : In
 *     @return The error string of I2C driver.
 */
char *CSI2C_GetErrString(CSI2C_HANDLE handle);

/**
 *Print the error string associated with error code
 *     @param  errcode   Error code   In/Out : In
 */
void CSI2C_PrintErrorStr(CSI2C_ErrCode errcode);

#ifdef __cplusplus
}
#endif

#endif				/* __CSAPI_I2C_H */
