/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Monta-Vista.
 Monta-Vista reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Monta-Vista

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Monta-Vista
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/** ============================= File Header ================================
 *  \brief     : Implements trace functionality.
 *               Defines macros that can be used to generate trace messages.
 *               Trace messages can be logged and may provide useful information
 *               to assist in debugging.
 * ======================================================================== */

#ifndef _CNC_TRACE_H_
#define _CNC_TRACE_H_

/* ============================ System Includes =========================== */
#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>
#include <time.h>

#ifdef PLATFORM_X86
#include <stdio.h>
#include <string.h>
#endif

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_traceModules.h"   // Include the list of module IDs

#ifdef __cplusplus
extern "C"
{
#endif

/* =========================== Macro definitions ========================== */
/* \def    : TRACE_GLOBAL_LEVEL
 * \brief  : Global trace level flag. This can be overridden by individual modules
 *           Values between 0 (OFF) to 4 (Verbose) can be specified.
 */
#ifdef SET_TRACE_GLOBAL_LEVEL
#define TRACE_GLOBAL_LEVEL              SET_TRACE_GLOBAL_LEVEL
#else
#define TRACE_GLOBAL_LEVEL              0x0F
#endif

/* \def    : TRACE_DYNAMIC_CONFIG
 * \brief  : Dynamic trace configuration. Allows trace filters specified at run-time
 */
#define TRACE_DYNAMIC_CONFIG            1

/* \def    : TRACE_FLAG_NONE, TRACE_FLAG_ERROR, TRACE_FLAG_WARNING
 *           TRACE_FLAG_DEBUG, TRACE_FLAG_TEST, TRACE_FLAG_ALL
 * \brief  : Flags which indicate the type of a trace message
 */
#define TRACE_FLAG_NONE                 0x0000    //! No tracing
#define TRACE_FLAG_ERROR                0x0001    //! Error requiring user attention, user visible by default
#define TRACE_FLAG_WARNING              0x0002    //! Warning requiring user attention, user visible by default
#define TRACE_FLAG_STATUS               0x0004    //! Key status event, user visible by default
#define TRACE_FLAG_INFO                 0x0008    //! Informational event
#define TRACE_FLAG_DETAIL               0x0010    //! Details for informational events
#define TRACE_FLAG_IO                   0x0020    //! Debug for I/O events
#define TRACE_FLAG_TEST                 0x0040    //! Debug output for self tests
#define TRACE_FLAG_DEBUG                0x0080    //! Debug
#define TRACE_FLAG_DDEBUG               0x0100    //! Detailed debug
#define TRACE_FLAG_ALL                  0xFFFF    //! All debug

#define TRACE_FLAG_DEFAULT              0x0007    //! Default tracing mask

/* Set the trace level as defined globally or as overridden by a particular
 * module. Note that each module includes this header file to acheive the
 * desired effect.
 */
#ifdef TRACE_MODULE_LEVEL
#define CNC_TRACE_LEVEL        TRACE_MODULE_LEVEL
#else
#define CNC_TRACE_LEVEL        TRACE_GLOBAL_LEVEL
#endif

/* Select the trace functions to be enabled, based on the trace level */
#define TRACE_ERROR_ENABLE             ((CNC_TRACE_LEVEL & TRACE_FLAG_ERROR) | (CNC_TRACE_LEVEL & TRACE_FLAG_WARNING))
#define TRACE_WARNING_ENABLE           (CNC_TRACE_LEVEL & TRACE_FLAG_WARNING)
#define TRACE_INFO_ENABLE              (CNC_TRACE_LEVEL & TRACE_FLAG_INFO)
#define TRACE_STATUS_ENABLE            (CNC_TRACE_LEVEL & TRACE_FLAG_STATUS)
#define TRACE_DETAIL_ENABLE            (CNC_TRACE_LEVEL & TRACE_FLAG_DETAIL)
#define TRACE_IO_ENABLE                (CNC_TRACE_LEVEL & TRACE_FLAG_IO)
#define TRACE_TEST_ENABLE              (CNC_TRACE_LEVEL & TRACE_FLAG_TEST)
#define TRACE_DEBUG_ENABLE             (CNC_TRACE_LEVEL & TRACE_FLAG_DEBUG)
#define TRACE_DDEBUG_ENABLE            (CNC_TRACE_LEVEL & TRACE_FLAG_DDEBUG)

/*! \TODO: Uses variadic macros and may not be compatible with non-GNU compilers */
/* Definitions for trace macros. Trace macros are used by applications to
 * display trace messages.
 *------------------------------------------------------------------------------
 * IMPORTANT:
 * The following functions *MUST NOT* call any of the trace macros, since it
 * would result in a recursion.
 * CNC_TRACE_Printf, cnc_devicePrintf, cnc_deviceVprintf
 *------------------------------------------------------------------------------
 */

/* ============================================= TRACE_ERROR_ENABLE ============================================= */
#if (TRACE_ERROR_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_ERROR(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_ERROR, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_ERROR(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_ERROR, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_ERROR(stringId, ...)
#else
#define CNC_TRACE_ERROR(stringId, fmt, ...)
#endif
#endif  /* TRACE_ERROR_ENABLE */
/* ============================================================================================================== */
/* ============================================ TRACE_WARNING_ENABLE ============================================ */
#if (TRACE_WARNING_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_WARNING(stringId, ...)       CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_WARNING, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_WARNING(stringId, fmt, ...)  CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_WARNING, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_WARNING(stringId, ...)
#else
#define CNC_TRACE_WARNING(stringId, fmt, ...)
#endif
#endif  /* TRACE_WARNING_ENABLE */
/* ============================================================================================================== */
/* ============================================= TRACE_INFO_ENABLE ============================================== */
#if (TRACE_INFO_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_INFO(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_INFO, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_INFO(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_INFO, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_INFO(stringId, ...)
#else
#define CNC_TRACE_INFO(stringId, fmt, ...)
#endif
#endif  /* TRACE_INFO_ENABLE */
/* ============================================================================================================== */
/* ============================================ TRACE_STATUS_ENABLE ============================================= */
#if (TRACE_STATUS_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_STATUS(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_STATUS, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_STATUS(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_STATUS, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_STATUS(stringId, ...)
#else
#define CNC_TRACE_STATUS(stringId, fmt, ...)
#endif
#endif  /* TRACE_STATUS_ENABLE */
/* ============================================================================================================== */
/* ============================================ TRACE_DETAIL_ENABLE ============================================= */
#if (TRACE_DETAIL_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_DETAIL(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_DETAIL, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_DETAIL(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_DETAIL, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_DETAIL(stringId, ...)
#else
#define CNC_TRACE_DETAIL(stringId, fmt, ...)
#endif
#endif  /* TRACE_DETAIL_ENABLE */
/* ============================================================================================================== */
/* ============================================== TRACE_IO_ENABLE =============================================== */
#if (TRACE_IO_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_IO(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_IO, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_IO(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_IO, \
                                                             __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                             (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_IO(stringId, ...)
#else
#define CNC_TRACE_IO(stringId, fmt, ...)
#endif
#endif  /* TRACE_IO_ENABLE */
/* ============================================================================================================= */
/* ============================================= TRACE_TEST_ENABLE ============================================= */
#if (TRACE_TEST_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_TEST(stringId, ...)          CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_TEST, \
                                                                 __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                                 (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_TEST(stringId, fmt, ...)     CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_TEST, \
                                                                 __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                                 (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_TEST(stringId, ...)
#else
#define CNC_TRACE_TEST(stringId, fmt, ...)
#endif
#endif  /* TRACE_TEST_ENABLE */
/* ============================================================================================================== */
/* ============================================= TRACE_DEBUG_ENABLE ============================================= */
#if (TRACE_DEBUG_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_DEBUG(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_DEBUG, \
                                                                 __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                                 (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_DEBUG(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_DEBUG, \
                                                                 __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                                 (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_DEBUG(stringId, ...)
#else
#define CNC_TRACE_DEBUG(stringId, fmt, ...)
#endif
#endif  /* TRACE_DEBUG_ENABLE */
/* ============================================================================================================== */
/* ============================================= TRACE_DDEBUG_ENABLE ============================================= */
#if (TRACE_DDEBUG_ENABLE)
#ifdef __STRICT_ANSI__
#define CNC_TRACE_DDEBUG(stringId, ...)         CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_DDEBUG, \
                                                                 __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                                 (CNC_CHAR *) __VA_ARGS__)
#else   /* __STRICT_ANSI__ */
#define CNC_TRACE_DDEBUG(stringId, fmt, ...)    CNC_TRACE_Printf(TRACE_MODULE_ID, TRACE_FLAG_DDEBUG, \
                                                                 __FUNCTION__, __FILE__, __LINE__, stringId, \
                                                                 (CNC_CHAR *)fmt, ##__VA_ARGS__)
#endif  /* __STRICT_ANSI__ */
#else
#ifdef __STRICT_ANSI__
#define CNC_TRACE_DDEBUG(stringId, ...)
#else
#define CNC_TRACE_DDEBUG(stringId, fmt, ...)
#endif
#endif  /* TRACE_DDEBUG_ENABLE */
/* ============================================================================================================== */


/* Special Macros to assist logging of Function Entry/Exit and Stub implementations */

#ifdef TRACE_FUNCTION_ENTRY_EXIT_ENABLE

#ifndef CNC_TRACE_FUNCTION_ENTRY
#define CNC_TRACE_FUNCTION_ENTRY()     CNC_TRACE_DDEBUG(E_STRING_DB_FUNCTION_ENTRY, "%s [Entry]\n", __FUNCTION__)
#endif

#ifndef CNC_TRACE_FUNCTION_EXIT
#define CNC_TRACE_FUNCTION_EXIT()      CNC_TRACE_DDEBUG(E_STRING_DB_FUNCTION_EXIT, "%s [Exit]\n", __FUNCTION__)
#endif

#ifndef CNC_TRACE_FUNCTION_STUB
#define CNC_TRACE_FUNCTION_STUB()      CNC_TRACE_DDEBUG(E_STRING_DB_FUNCTION_STUB, "%s is a stub function\n", __FUNCTION__)
#endif

#else

// Empty Macro in case detail logs of Function Entry-Exit is NOT required
#ifndef CNC_TRACE_FUNCTION_ENTRY
#define CNC_TRACE_FUNCTION_ENTRY()
#endif

#ifndef CNC_TRACE_FUNCTION_EXIT
#define CNC_TRACE_FUNCTION_EXIT()
#endif

#ifndef CNC_TRACE_FUNCTION_STUB
#define CNC_TRACE_FUNCTION_STUB()
#endif

#endif


/* [TRACE_MACROS_END] */

/* =========================== Type definitions =========================== */
/* \enum   : traceModuleId_t
 * \brief  : Data type used for specifying module IDs
 */
typedef CNC_UINT32  traceModuleId_t;

/* \enum   : traceModuleFlag_t
 * \brief  : Data type used for specifying trace type
 */
typedef CNC_UINT32  traceModuleFlag_t;

typedef enum _cncTraceErrCode_t
{
    CNC_TRACE_SUCCESS = 0,
    CNC_TRACE_ERR = -1
} cncTraceErrCode_t;


/* ===================== Global variable declarations ===================== */
/* Trace filter. Each element contains flags for a specific module.
 * eTraceNModules = total number of modules. Defined in trace_modules.h.
 */
#if (TRACE_DYNAMIC_CONFIG)
extern traceModuleFlag_t osalTraceFilter[];
#endif


/* ======================== Function declarations ========================= */
/* Platform independent functions */

/**
 *  \brief     : Sets the trace filter for the given module.
 *               Available only if TRACE_DYNAMIC_CONFIG is defined.
 *  \param     : -> moduleId
 *                    ID of the module whose trace filter is to be configured.
 *               -> flags
 *                    Bit-flags which specify which type of trace messages are
 *                    on and which are off.
 *  \return    : None
 */
void CNC_TRACE_SetConfig(traceModuleId_t moduleId, traceModuleFlag_t flags);

/**
 *  \brief     : Returns the trace filter for the given module.
 *               Available only if TRACE_DYNAMIC_CONFIG is defined.
 *  \param     : -> moduleId
 *                    ID of the module whose trace filter should be returned.
 *  \return    : None
 */
traceModuleFlag_t CNC_TRACE_GetConfig(traceModuleId_t moduleId);

/**
 *  \brief     : Returns whether a debug level is enabled
 *               This is useful if a block of supplementary code should be avoided
 *               if a particular debug level is not on.
 *  \param     : -> moduleId
 *                    ID of the module generating this trace message.
 *               -> logType
 *                    Specify the type of the trace message.
 *  \return    : Whether the logType is enabled or not
 */
CNC_BOOL CNC_TRACE_Enabled(traceModuleId_t moduleId, CNC_SINT32 logType);
#if (TRACE_DYNAMIC_CONFIG)
#define CNC_TRACE_ENABLED(level) CNC_TRACE_Enabled(TRACE_MODULE_ID, TRACE_FLAG_##level)
#else
#define CNC_TRACE_ENABLED(level) (1)
#endif

#ifndef PLATFORM_X86
 /**
 *  \brief     : Initializes the trace module. Should be called once at startup
 *
 *  This function reads a number of variables from the shell environment.
 *  CNC_TRACE_ALL: Sets the trace level for all modules
 *  CNC_TRACE_<module_name>: Sets the trace level for a given module OR'ed with
 *      the CNC_TRACE_ALL value.  Module names are defined in cnc_traceModules.h
 * 
 *  By default, tracing is set to only output according to TRACE_FLAG_DEFAULT for
 *  every module.
 *
 *  Please be judicious in your use of debug statements, and adhere to the definitions above.
 * 
 *  \param     : None
 *  \return    : None
 */
void CNC_TRACE_Initialize(void);

/**
 *  \brief     : Deinitializes the trace module. Should be called once at shutdown
 *  \param     : None
 *  \return    : None
 */
void CNC_TRACE_Deinitialize(void);


/**
 *  \brief     : Implements printing of trace messages.
 *               This should invoked through one of the macros defined in trace.h.
 *               This implementation prints function, filename and line number along
 *               with the rest of the trace message for easy location of the origin
 *               of the message.
 *  \param     : -> moduleId
 *                    ID of the module generating this trace message.
 *               -> logType
 *                    Specify the type of the trace message.
 *               -> funcName
 *                    Function from where this trace message originated.
 *               -> fileName
 *                    Filename from where this trace message originated.
 *               -> lineNum
 *                    Line number from where this trace message originated.
 *               -> logID
 *                    String ID for the Log Message.
 *               -> fmt
 *                    Format string, similar to that of printf.
 *  \return    : Number of characters printed, similar to that of printf.
 */
CNC_SINT32 CNC_TRACE_Printf(traceModuleId_t    moduleId,
                             CNC_SINT32        logType,
                             const CNC_CHAR   *funcName,
                             const CNC_CHAR   *fileName,
                             const CNC_SINT32  lineNum,
                             const CNC_UINT32  stringID,
                             const CNC_CHAR   *fmt,
                             ...);
#else 
/* Following structure copied from cnc_trace.c for module name print for x86 */

#if (TRACE_DYNAMIC_CONFIG)
/*! This controls which modules print what traces */
static traceModuleFlag_t     g_cncTraceFilter[CNC_TRACE_NMODULES] =
{
    0x0000,
    0x0000,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF,
    0xFFFF                /*< For CNC_RUI Module */
};
#endif

static CNC_CHAR *gp_cncModuleNames[CNC_TRACE_NMODULES] =
{
    NULL,
    "CNC_TRACE",
    "CNC_CORE_AV",
    "CNC_CORE_SYSTEM",
    "CNC_CORE_DEMUX",
    "CNC_CORE_FILEMGR",
    "CNC_CORE_UTILS",
    "CNC_MEDIA_CTL",
    "CNC_MEDIA_STREAM",
    "CNC_MEDIA_DRM",
    "CNC_MEDIA_IMAGE",
    "CNC_MEDIA_PLAYER",
    "CNC_MEDIA_SUBTITLE",
    "CNC_MEDIA_DECODER",
    "CNC_MEDIA_DEMUXER",
    "CNC_MEDIA_MUXER",
    "CNC_MEDIA_RECORDER",
    "CNC_HTTP",
    "CNC_WD",
    "CNC_OSD",
    "CNC_BTV",
    "CNC_CRYPTO",
    "CNC_DLNA_DMR",
    "CNC_DLNA_DMP",
    "CNC_DLNA_DMCP",
    "CNC_DLNA_DMS",
    "CNC_NETWORK",
    "CNC_NETWORK_PLUGIN",
    "CNC_UPDATE",
    "CNC_DFB",
    "CNC_WEB_ESPIAL",
    "CNC_SUBTITLE_RENDER",
    "CNC_IPREMOTE",
    "CNC_DIAL",
    "CNC_AIRPLAY",
    "CNC_IR",
    "CNC_APP_FRAMEWORK",
    "CNC_APP",
    "CNC",
    "CNC_RUI"
};

static CNC_SINT32 CNC_TRACE_Printf(traceModuleId_t moduleId, 
                            CNC_SINT32 logType,
                            const CNC_CHAR *funcName, 
                            const CNC_CHAR *fileName,
                            const CNC_SINT32 lineNum, 
                            const CNC_UINT32 logID,
                            const CNC_CHAR *fmt, ...) 
{
    va_list args;
    va_start(args, fmt);

    CNC_SINT32 len;
    CNC_CHAR *logText = CNC_NULL;
    struct timeval curTimeSinceEpochInSec;
    struct tm timeStamp;
    struct tm *p_timeStamp = CNC_NULL;
    CNC_CHAR buffer[32];
    CNC_CHAR *temp = CNC_NULL;

#if (TRACE_DYNAMIC_CONFIG)
    if (!(g_cncTraceFilter[moduleId] & logType))
    {
        return CNC_TRACE_ERR;
    }
#endif

    switch(logType) {
    case TRACE_FLAG_NONE:
        logText = "NONE";
        break;
    case TRACE_FLAG_ERROR:
        logText = "ERR ";
        break;
    case TRACE_FLAG_WARNING:
        logText = "WARN";
        break;
    case TRACE_FLAG_INFO:
        logText = "INFO";
        break;
    case TRACE_FLAG_STATUS:
        logText = "STAT";
        break;
    case TRACE_FLAG_DETAIL:
        logText = "DET ";
        break;
    case TRACE_FLAG_IO:
        logText = "I/O ";
        break;
    case TRACE_FLAG_TEST:
        logText = "TEST";
        break;
    case TRACE_FLAG_DEBUG:
        logText = "DBG ";
        break;
    case TRACE_FLAG_DDEBUG:
        logText = "DDBG";
        break;
    case TRACE_FLAG_ALL:
        logText = "ALL ";
        break;
    default:
        logText = "----";
    }
    /* Get basename of the File */
    temp = strrchr(fileName, '/');
    if(temp != CNC_NULL)
    {
        fileName = temp + 1;
    }

    gettimeofday(&curTimeSinceEpochInSec, CNC_NULL);

    /* Get the "broken down local time" from "time since Epoch" */
    p_timeStamp = localtime_r(&curTimeSinceEpochInSec.tv_sec, &timeStamp);

    /* Format the local time to the format "YYYYmmDDHHMMSS" */
    strftime(buffer, sizeof(buffer), "%F %H:%M:%S", p_timeStamp);

    // Print filenames, line numbers and function names only when using detail/debug logs.
    len = fprintf(stdout, "%s.%03ld %s %s [%s +%d %s()]: ",buffer, curTimeSinceEpochInSec.tv_usec/1000, gp_cncModuleNames[moduleId], logText, fileName, lineNum, funcName);

    len += (CNC_SINT32)vfprintf(stdout, fmt, args);


    va_end(args);

    return 0;
}

/*  \function  : cnc_traceInitialize
 *  \brief     : Initializes the trace module. Should be called once at startup
 *  \return    : None
 */
inline static void CNC_TRACE_Initialize(void)
{
#if (TRACE_DYNAMIC_CONFIG)
    /* Setup the default trace filter */
    CNC_SINT32      moduleIndex = 0;
    CNC_SINT32      retValue = 0;
    CNC_CHAR        *p_envTraceLevel = NULL;
    CNC_UINT32      moduleTraceLevel = 0x0;
    CNC_UINT32      traceLevelAll = 0x0;

    /* Check if ALL-module trace level is set */
    p_envTraceLevel = getenv("CNC_TRACE_ALL");
    if (p_envTraceLevel != NULL)
    {
        /* Find ALL-module trace level */
        retValue = sscanf(p_envTraceLevel, "%x", &traceLevelAll);
        if(retValue <= 0)
        {
            traceLevelAll = 0x0;
        }
    }

    /* Apply the filter level for each module with ALL-module trace level */
    for(moduleIndex = 0; moduleIndex < CNC_TRACE_NMODULES; moduleIndex++)
    {
        if(NULL != gp_cncModuleNames[moduleIndex])
        {
            /* Check if an env variable is set for this module trace level config. If yes, use it */
            /* Read the env variable of this module */
            p_envTraceLevel = getenv(gp_cncModuleNames[moduleIndex]);
            if (p_envTraceLevel != NULL)
            {
                retValue = sscanf(p_envTraceLevel, "%x", &moduleTraceLevel);
                if(retValue > 0)
                {
                    g_cncTraceFilter[moduleIndex] = (traceLevelAll | moduleTraceLevel);
                }
                else
                {
                    g_cncTraceFilter[moduleIndex] = (traceLevelAll | TRACE_FLAG_DEFAULT);
                }
            }
            else
            {
                g_cncTraceFilter[moduleIndex] = (traceLevelAll | TRACE_FLAG_DEFAULT);
            }
        }
        else
        {
            g_cncTraceFilter[moduleIndex] = (traceLevelAll | TRACE_FLAG_DEFAULT);
        }
    }
#endif

    return;
}

inline static void CNC_TRACE_Deinitialize(void)
{
    return;
}

#endif //End of PLATFORM_X86

#ifdef __cplusplus
}
#endif

#endif /* _CNC_TRACE_H_ */
