/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
*/
#ifndef __CNC_AIRPLAY_H_
#define __CNC_AIRPLAY_H_

#ifdef __cplusplus
extern "C" {
#endif

#define CNC_AIRPLAY_EVENT_NONE     -1
#define CNC_AIRPLAY_EVENT_PLAYING   0
#define CNC_AIRPLAY_EVENT_PAUSED    1
#define CNC_AIRPLAY_EVENT_LOADING   2
#define CNC_AIRPLAY_EVENT_STOPPED   3

/**
 * Struct with callbacks that will be call when there is some event
 */
typedef struct {
    /** received a play command, location is the uri to play, position
     *   is the offset to start, in seconds
     */
    void (*mediaPlay)(const char* location, float position);
    /** audio stream started to play */
    void (*audioPlay)(int bits, int channels, int samplerate);
    /** audio stream stopped to play */
    void (*audioStop)();
    /** audio stream requested a flush of the buffer */
    void (*audioFlush)();    
    /** audio stream data to process received */
    void (*audioProcess)(void *session, const void *buffer, int buflen);    
    /** audio stream metadata received */
    void (*audioMetadata)(char* artist, char* album, char* title);    
    /** audio stream coverart received and copied to "/tmp/airplay_coverart.jpg" */
    void (*audioCoverart)();    
    /** received a pause command */
    void (*mediaPause)();
    /** received a stop command */
    void (*mediaStop)();
    /** should return the active volume [0-255]*/
    int (*getVolume)();
    /** set volume [0-255]*/
    void (*setVolume)(int volume);
    /** returns 1 if it is playing, if not 0*/
    int (*isPlaying)();
    /** returns 1 if it is paused, if not 0*/
    int (*isPaused)();
    /** returns 1 if it is stoped, if not 0*/
    int (*isStopped)();
    /** returns 1 if it is caching, if not 0*/
    int (*isCaching)();
    /** returns the playing time in secs*/
    float (*getTime)();
    /** returns the length of the video in secs*/
    float (*getTotalTime)();
    /** received a command to seek to position in secs*/
    void (*seekTime)(float position);
    /** received a command to show a picture, filename path to the image */
    void (*pictureShow)(const char *filename); //TODO change to buffer + size
    /** the client has disconnected*/
    void (*clientDisconnected)();
} cncAirplayCallbacks_t;


/** 
 * Initialize airplay. Will also update/create the avahi service files
 * and restart avahi.
 * 
 * @param callbacks the callbacks that will be used for event
 *                  notifying. There couldn't be any NULL callback,
 *                  function will return error.
 * @param name Name that will be shown when searching airplay server.
 * @param mac String with the mac that will be used to announce the
 *            service. Itunes server will only work in the interface
 *            with this mac. The mac should be in a format like:
 *            "01:23:45:67:89:AB", uppercase an using ':' as separator
 * 
 * @return 0 if correctly initialized, any other value is 
 */
int CNC_AIRPLAY_Init(cncAirplayCallbacks_t *callbacks, const char * name,
                     const char *mac);

/** 
 * Stop airplay. Free resources and set values according 
 * 
 * @return   
 */
void CNC_AIRPLAY_Stop();

/** 
 * Notify a change in the video stream state
 * 
 * @param newState New state to notify, it can be CNC_AIRPLAY_EVENT_PLAYING, CNC_AIRPLAY_EVENT_PAUSED, CNC_AIRPLAY_EVENT_LOADING or CNC_AIRPLAY_EVENT_STOPPED
 */
void CNC_AIRPLAY_NotifyStreamState(int newState);


/** 
 * Close all active connections. cnc_airplay is still listening to new
 * connections
 * 
 */
void CNC_AIRPLAY_CloseConnections();

#ifdef __cplusplus
}
#endif


#endif
