/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_dfb.h
 * @ingroup appframework
 * This module implements Virtual OSD Layers using DirectFB's Window Manager.
 *     - It manages the compositing of multiple virtual OSD layers into the corresponding Physical OSD Layers.
 * Physical OSD Layers are configured outside this module (See core/cncsystem/cnc_device.c).
 *     - The external module configures the dimensions and positions each Physical OSD Layer.
 *     - This includes both actual and visual dimensions, which implies the scaling factor of each Physical OSD Layer.
 * Virtual Layers are stacked according to their order of creation. A Virtual Layer created earlier will be displayed below a Virtual Layer created later.
 * The stacking order of the Physical Layers is beyond the scope of this module and it should be handled externally.
 */

#ifndef __CNC_DFB_H__
#define __CNC_DFB_H__

#define HAVSIGNAL_H 1

/* ============================ System Includes =========================== */
#include "directfb.h"

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_common_error.h"
#include "csosd.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* =========================== Macro definitions ========================== */
// NONE

/* =========================== Type definitions =========================== */

typedef enum _cncDfbModuleID_t
{
    CNC_DFB_MODULE_ID_INVALID = 0,                                            /**< Invalid module ID. This value should not be used and is for internal validation purpose only. */
    CNC_DFB_MODULE_ID_SUBTITLE,                                               /**< Module ID for CNC-SubTitle */
    CNC_DFB_MODULE_ID_OSD,                                                    /**< Module ID for CNC-OSD */
    CNC_DFB_MODULE_ID_WEB,                                                    /**< Module ID for CNC-WEB */
    CNC_DFB_MODULE_ID_SOFTWARE_KEYBOARD,                                      /**< Module ID for software keyboard implemented in CNC-WEB */
    CNC_DFB_MODULE_ID_APPLICATION_OFFSET                                      /**< Module ID start offset for application modules */
} cncDfbModuleID_t;

/**
 *  @brief      Specifies the flags for the Physical Layer Information structure.
 */
typedef enum _cncDfbPhysicalLayerFlags_t
{
    CNC_DFB_PHYSICAL_LAYER_FLAG_NONE                      = 0x00,             /**< No flags are set */
    CNC_DFB_PHYSICAL_LAYER_FLAG_SET_COOPERATIVE_LEVEL     = 0x01,             /**< The layer should be initialized with the specified cooperative level */
    CNC_DFB_PHYSICAL_LAYER_FLAG_SET_CONFIGURATION         = 0x02              /**< The layer should be configured with the specified configuration */
} cncDfbPhysicalLayerFlags_t;

/**
 *  Physical Layer Information structure.
 *  An external module should provide this information structure for each Physical Layer through the CNC_DFB_Init() function.
 */
typedef struct _cncDfbPhysicalLayerInfo_t
{
    cncDfbPhysicalLayerFlags_t              flags;                            /**< Flags specifying additional options or actions to be taken. */

    /* Actual dimensions of the layer, in pixels. An external module should handle the rescaling and compositing of Physical Layers, so the visual (rescaled) dimensions are beyond the scope of this module. */
    CNC_SINT32                              width;                            /**< Width of the Physical Layer in pixels. This should be the actual width and not the visual (rescaled) width. */
    CNC_SINT32                              height;                           /**< Height of the Physical Layer in pixels. This should be the actual height and not the visual (rescaled) height. */

    /* CNC OSD specific members */
    CSOSD_LAYER                             cncLayerId;                       /**< CNC OSD Layer ID. This is the Physical Layer ID used by the lower layer. */
    CSOSD_MODE                              cncMode;                          /**< CNC OSD Mode. This specifies the OSD's resolution according to HDMI/NTSC/PAL signal naming conventions. */
    CSOSD_COLOR_DEPTH                       cncColorDepth;                    /**< CNC OSD Color Depth. This is an enumeration that evaluates to the color depth in bits per pixel. */
    CSOSD_COLOR_FORMAT                      cncColorFormat;                   /**< CNC OSD Color Format. Specifies the pixel color format (color components such as R, G, B, A involved and their ordering). */

    /* DirectFB specific members */
    DFBDisplayLayerID                       dfbLayerId;                       /**< DirectFB's Layer ID. This is equivalent to the CNC OSD Layer ID, but specified according to DirectFB's enumerations. */
    DFBSurfacePixelFormat                   dfbPixelFormat;                   /**< DirectFB's Pixel Color Format. This is equivalent to the CNC OSD Color Format, but specified according to DirectFB's enumerations. */
    DFBDisplayLayerCooperativeLevel         dfbCooperativeLevel;              /**< DirectFB's Layer Cooperative Level. This is used only when the e_flags member has the CNC_DFB_PHYSICAL_LAYER_FLAG_SET_COOPERATIVLEVEL flag enabled. */
    DFBDisplayLayerConfig                   dfbConfig;                        /**< DirectFB's Layer Configuration. This is used only when the e_flags member has the CNC_DFB_PHYSICAL_LAYER_FLAG_SET_CONFIGURATION flag enabled. */
} cncDfbPhysicalLayerInfo_t;

/**
 *  @brief      Specifies the flags for the Virtual Layer Information structure.
 */
typedef enum _cncDfbVirtualLayerFlags_t
{
    CNC_DFB_VIRTUAL_LAYER_FLAG_NONE                       = 0x00,             /**< No flags are set */
    CNC_DFB_VIRTUAL_LAYER_FLAG_SET_SURFACE_CAPABILITIES   = 0x01,             /**< The Virtual Layer's surface should be initialized with the specified capabilities */
    CNC_DFB_VIRTUAL_LAYER_FLAG_CREATE_ABOVE               = 0x02,             /**< The Virtual Layer should be created above an existing Virtual Layer (applicable only to Window Surfaces) */
    CNC_DFB_VIRTUAL_LAYER_FLAG_CREATE_BELOW               = 0x04,             /**< The Virtual Layer should be created below an existing Virtual Layer (applicable only to Window Surfaces) */
    CNC_DFB_VIRTUAL_LAYER_FLAG_ENABLE_ALPHA_BLENDING      = 0x08,             /**< The Virtual Layer should have alpha blending enabled (applicable only to Window Surfaces) */
    CNC_DFB_VIRTUAL_LAYER_FLAG_ENABLE_DOUBLE_BUFFERING    = 0x10,             /**< The Virtual Layer should have double buffering enabled (applicable only to Window Surfaces) */
    CNC_DFB_VIRTUAL_LAYER_FLAG_ENABLE_OPAQUE_REGION       = 0x20,             /**< Allows the definition of an opaque area within the actual area of the Virtual Layer, for which alpha blending will not be performed (faster performance) */
    CNC_DFB_VIRTUAL_LAYER_FLAG_ENABLE_COLOR_KEYING        = 0x40              /**< The Virtual Layer should have color keying enabled (applicable only to Window Surfaces) */
} cncDfbVirtualLayerFlags_t;

/**
 * Specifies the Virtual Layer type. This selection impacts the performance and memory footprint of a surface.
 */
typedef enum _cncDfbVirtualLayerType_t
{
    CNC_DFB_VIRTUAL_LAYER_TYPE_NONE = 0,                                      /**< Invalid type. This value should not be used and is for internal validation purpose only. */
    CNC_DFB_VIRTUAL_LAYER_TYPE_GET_PRIMARY_SURFACE,                           /**< [Not fully supported: do not use] The surface allocated should be the primary surface of the specified Physical Layer. */
    CNC_DFB_VIRTUAL_LAYER_TYPE_CREATE_SECONDARY_SURFACE,                      /**< [Not fully supported: do not use] The surface allocated should be a secondary (non-primary) surface of the specified Physical Layer. */
    CNC_DFB_VIRTUAL_LAYER_TYPE_CREATE_WINDOW_SURFACE,                         /**< The surface allocated should be a on a Window associated with the specified Physical Layer. */
    CNC_DFB_VIRTUAL_LAYER_TYPE_CREATE_SUB_SURFACE                             /**< [Not fully supported: do not use] The surface allocated should be created as a sub-surface on an existing surface. The existing surface is specified through its Virtual Layer Index. */
} cncDfbVirtualLayerType_t;

/**
 * Position and dimensions of an area, in pixels.
 */
typedef struct _cncDfbArea_t
{
    CNC_SINT32                          x;
    CNC_SINT32                          y;
    CNC_SINT32                          width;
    CNC_SINT32                          height;
} cncDfbArea_t;

/**
 *  Virtual Layer Information structure.
 *  An external module should provide this information structure for each Virtual Layer through the CNC_DFB_Init() function.
 */
typedef struct _cncDfbVirtualLayerInfo_t
{
    cncDfbVirtualLayerFlags_t               flags;                          /**< Flags specifying additional options or actions to be taken. */
    cncDfbVirtualLayerType_t                type;                           /**< Specifies the type of surface that is associated with this Virtual Layer */
    cncDfbArea_t                            actualArea;                     /**< Position and dimensions of the actual area, in pixels. The position fields are valid only if the surface type is a sub-surface. Dimension fields are valid in all cases. */
    cncDfbArea_t                            opaqueArea;                     /**< Position and dimensions of an opaque area inside the actual area, in pixels. The opaque area is valid only if the surface */
    cncDfbArea_t                            visualArea;                     /**< Position and dimensions of the visual area, in pixels. This represents the rescaled dimensions with which this Virtual Layer is composited onto the Physical Layer. */
    CNC_SINT32                              parentPhysicalLayerIndex;       /**< Index of the Physical Layer with which this Virtual Layer is associated. This will be the Physical Layer on which this Virtual Layer is composited. */
    CNC_SINT32                              parentVirtualLayerID;           /**< Index of the Virtual Layer with which this Virtual Layer is associated. This is only applicable when the surface type is a sub-surface, or the Virtual Layer is being created with the CNC_DFB_VIRTUAL_LAYER_FLAG_CREATABOVE or CNC_DFB_VIRTUAL_LAYER_FLAG_CREATBELOW flags enabled. */
    CNC_SINT32                              zIndex;                         /**< This is a read-only member, indicating the Virtual Layer's z-order on the Physical Layer with which it is associated. This is ignored when passed to CNC_DFB_CreateLayer. If A < B, the layer with value A is below the layer with value B. */
    CNC_UINT8                               colorKeyR;                      /**< Color key R component of the Virtual Layer. */
    CNC_UINT8                               colorKeyG;                      /**< Color key G component of the Virtual Layer. */
    CNC_UINT8                               colorKeyB;                      /**< Color key B component of the Virtual Layer. */
    CNC_UINT8                               alpha;                          /**< Virtual Layer's opacity. */

    /* DirectFB specific members */
    DFBSurfacePixelFormat                   dfbPixelFormat;                 /**< DirectFB's Pixel Color Format. This specifies the pixel format of the surface associated with this Virtual Layer */
    DFBSurfaceCapabilities                  dfbSurfaceCapabilities;         /**< DirectFB's Surface Capabilities. This is used only when the e_flags member has the CNC_DFB_VIRTUAL_LAYER_FLAG_SET_SURFACCAPABILITIES flag enabled. */
} cncDfbVirtualLayerInfo_t;

/**
 *  Uniquely identifies fb0 and fb2.
 */
typedef enum _cncDfbFrameBufferID_t
{
    CNC_DFB_FRAME_BUFFER_ID_INVALID = 0,
    CNC_DFB_FRAME_BUFFER_ID_FB0,
    CNC_DFB_FRAME_BUFFER_ID_FB2
} cncDfbFrameBufferID_t;

/* ======================== Function declarations ========================= */

/**
 * @brief           Intializes this module. This should be called during the following cases:
 *                      1. First time during application startup.
 *                      2. Subsequent times after invoking CNC_DFB_Deinit(), whenever the OSD resolution (Physical Layers) changes.
 * @param [in]      physicalLayerCount
 *                      Specifies number of Physical OSD Layers available in the system.
 * @param [in]      physicalLayers
 *                      Array of information structures corresponding to the Physical Layers. This array can be destroyed immediately after this call returns.
 *                      The index into this array is used as the Physical Layer Index.
 *                      The order in this array corresponds to the Z-order of the Physical Layers. The layer with the lowest index (0) is the bottom most layer.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_Init(CNC_SINT32 physicalLayerCount, cncDfbPhysicalLayerInfo_t *physicalLayers);

/**
 *  @brief          Deinitializes this module. This should be called during the following cases:
 *                      1. Each time before invoking CNC_DFB_Init(), whenever the OSD resolution (Physical Layers) changes.
 *                      2. Last time during application shutdown.
 *  @return         CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
#ifdef __cplusplus
/* C++ only accepts "void" as keyword, but not as a typedef when providing empty arguments list. This is expected to be resolved (can use typedefs of void) in future versions of C++ specification.
 * See: http://www.open-std.org/jtc1/sc22/wg21/docs/cwg_defects.html#577
 */
E_cncErrCode_t CNC_DFB_Deinit(void);
#else
E_cncErrCode_t CNC_DFB_Deinit(CNC_VOID);
#endif

/**
 * @brief           Changes the visual area (rescaled dimensions) of a virtual layer. The caller must invoke this for each Virtual Layer before invoking CNC_DFB_ChangeResolution().
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      visualArea
 *                      Structure containing the new visual area.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_ChangeLayerVisual(CNC_SINT32 virtualLayerID, cncDfbArea_t *visualArea);

/**
 * @brief           Changes the OSD Resolution without restarting CNC-DFB. The caller must invoke CNC_DFB_ChangeLayerVisual() for each virtual layer before invoking this.
 * @param [in]      physicalLayerCount
 *                      Specifies number of Physical OSD Layers available in the system.
 * @param [in]      physicalLayers
 *                      Array of information structures corresponding to the Physical Layers. This array can be destroyed immediately after this call returns.
 *                      The index into this array is used as the Physical Layer Index.
 *                      The order in this array corresponds to the Z-order of the Physical Layers. The layer with the lowest index (0) is the bottom most layer.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_ChangeResolution(CNC_SINT32 physicalLayerCount, cncDfbPhysicalLayerInfo_t *physicalLayers);

/**
 * @brief           Returns a reference to the DirectFB Super Interface.
 * @param [out]     interface
 *                      Returns a reference to the DirectFB Super Interface (IDirectFB).
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_GetInterface(IDirectFB **interface);

/**
 * @brief           Creates a Virtual Layer based on the specified attributes.
 * @param [out]     virtualLayerID
 *                      Returns the ID of the Virtual Layer which was created.
 * @param [in]      virtualLayer
 *                      Pointer to the structure containing the configuration of the Virtual Layer to be created. This structure can be destroyed immediately after this call returns.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_CreateLayer(CNC_SINT32 *virtualLayerID, cncDfbVirtualLayerInfo_t *virtualLayer);

/**
 * @brief           Destroys the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer to be destroyed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_DestroyLayer(CNC_SINT32 virtualLayerID);

/**
 * @brief           Returns the surface associated with the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [out]     surface
 *                      Reference to the DirectFB surface corresponding to the specified Virtual Layer. This surface can be used to invoke the IDirectFBSurface methods (except Flip).
 *                      The caller should not destroy this surface by directly invoking the pp_s_surface->Release() method. CNC_DFB_DestroyLayer() must be invoked instead.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_GetSurface(CNC_SINT32 virtualLayerID, IDirectFBSurface **surface);

/**
 * @brief           Flips the front and back buffers in case of double-buffered surfaces.
 *                  IDirectFBSurface::Flip() should not be invoked for the surfaces allocated through CNC_DFB_AllocateSurface() and this function must be used instead.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      region
 *                      Specifies the region to be flipped. Passing NULL here will cause the entire surface to be flipped.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_Flip(CNC_SINT32 virtualLayerID, const DFBRegion *region);

/**
 * @brief           Gets the configuration of the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [out]     virtualLayer
 *                      The configuration of the specified Virtual Layer is returned through this caller allocated structure. The caller can destroy this immediately after the call returns.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_GetLayerConfig(CNC_SINT32 virtualLayerID, cncDfbVirtualLayerInfo_t *virtualLayer);

/**
 * @brief           Allows an application to register a use-case specific layer management mechanism.
 * @param [in]      AllocateLayerCallback
 *                      This callback function is invoked when any module requests a layer to be allocated.
 * @param [in]      ReleaseLayerCallback
 *                      This callback function is invoked when any module requests a layer to be released.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_RegisterLayerManager(E_cncErrCode_t (*AllocateLayerCallback)(CNC_SINT32 *virtualLayerID, CNC_SINT32 moduleID), E_cncErrCode_t (*ReleaseLayerCallback)(CNC_SINT32 virtualLayerID, CNC_SINT32 moduleID));

/**
 * @brief           Allocates a Virtual Layer to the specified module.
 * @details         This works closely with the application's use-case specific allocation mechanism.
 * @param [out]     virtualLayerID
 *                      Returns the ID of the Virtual Layer which was allocated.
 * @param [in]      moduleID
 *                      Unique ID of the module that wants to allocate a layer.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_AllocateLayer(CNC_SINT32 *virtualLayerID, CNC_SINT32 moduleID);

/**
 * @brief           Releases a Virtual Layer allocated to the specified module.
 * @details         This works closely with the application's use-case specific allocation mechanism.
 * @param [in]      virtualLayerID
 *                      ID of the Virtual Layer which should be released.
 * @param [in]      moduleID
 *                      Unique ID of the module that wants to release the layer.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_ReleaseLayer(CNC_SINT32 virtualLayerID, CNC_SINT32 moduleID);

/**
 * @brief           Checks if any other module has currently allocated the specified layer.
 * @param [in]      virtualLayerID
 *                      ID of the Virtual Layer which should be checked.
 * @param [out]     moduleID
 *                      On return, this will be updated with the ID of the module that has allocated the specified layer.
 *                      This will be assigned CNC_DFB_MODULE_ID_INVALID if the specified layer is not allocated.
 *
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_CheckLayer(CNC_SINT32 virtualLayerID, CNC_SINT32 *moduleID);

/**
 * @brief           Gets the ID of the framebuffer being used.
 * @param [out]     framebufferID
 *                      On return, this will contain the ID of the framebuffer being used.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_GetFrameBufferID(cncDfbFrameBufferID_t *framebufferID);

/* The following functions are only supported for Virtual Layers created as independent DirectFB Windows
 * For other cases, calling these functions will have no effect.
 * =========================================================================================================================
 */

/**
 * @brief           Makes the specified Virtual Layer visible.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_Show(CNC_SINT32 virtualLayerID);

/**
 * @brief           Makes the specified Virtual Layer invisible.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_Hide(CNC_SINT32 virtualLayerID);

/**
 * @brief           Sets the opacity of the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      alpha
 *                      Specifies the opacity value. 0xFF = Completely opaque. 0x00 = Completely transparent.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_SetAlpha(CNC_SINT32 virtualLayerID, CNC_UINT8 alpha);

/**
 * @brief           Disables blending for the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_DisableBlending(CNC_SINT32 virtualLayerID);

/**
 * @brief           Enables blending for the specified Virtual Layer.
 *                  Blending is enabled by default unless:
 *                      - The Virtual Layer was created without specifying the CNC_DFB_VIRTUAL_LAYER_FLAG_ENABLALPHA_BLENDING flag.
 *                      - CNC_DFB_DisableBlending() was called previously for the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_EnableBlending(CNC_SINT32 virtualLayerID);

/**
 * @param [in]      virtualLayerID:
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      x:
 *                      X co-ordinate.
 * @param [in]      y:
 *                      Y co-ordinate.
 * @param [in]      w:
 *                      Width.
 * @param [in]      h:
 *                      Height.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_ClearArea(CNC_SINT32 virtualLayerID, CNC_SINT32 x, CNC_SINT32 y, CNC_SINT32 w, CNC_SINT32 h);

/**
 * @brief           This is a wrapper over IDirectFBSurface::Dump().
 *                  It dumps the contents of the specified virtual layer into two separate image files:
 *                      .ppm - for RGB channels
 *                      .pgm - for Alpha channel
 *                  These image files will be stored as directory/prefix_####.ppm and directory/prefix_####.pgm
 *                  #### is a automatically generated sequence number: this routine does not overwrite any existing files.
 *                  DirectFB may choose to compress (tar.gz) the generated image files.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      directory
 *                      Directory in which the captured images are stored.
 * @param [in]      prefix
 *                      Prefix used to name the image files.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_DumpSurface(CNC_SINT32 virtualLayerID, const CNC_CHAR *directory, const CNC_CHAR *prefix);

/**
 * @brief           Dumps the contents of the specified DirectFB surface into a monochrome BMP file stored in a memory buffer.
 *                  This function should operate more efficiently than CNC_DFB_DumpSurface() because of the following:
 *                      - It does not compress (tar.gz) the output.
 *                      - It allows only a part of the surface to be captured (smaller sub-areas for faster capture).
 *                      - It allows scaling on the area being captured (scale down for faster capture).
 *                        (Scaling here sacrifices correctness for speed, so it skips entire rows and/or columns of pixels).
 *                      - It captures the image in monochrome to reduce memory usage.
 *                      - It captures the image into a buffer, so writing to the file system can be deferred.
 *                  Because of the above factors, this function can only be used for some special cases, where colors and resolution are unimportant.
 *                      For example, when capturing animation frames to check performance.
 * @param [in]      surface
 *                      Reference to the DirectFB surface which must be captured.
 *                      This function only supports surfaces having the pixel format DSPF_ARGB.
 * @param [in]      x, y, width, height
 *                      Sub-area that must be captured.
 *                      To capture the entire surface, set all these values to 0.
 * @param [in]      xScale, yScale
 *                      Horizontal and vertical scaling factors.
 *                      Set both values to 1.0 to capture without scaling.
 * @param [out]     bmpBuffer
 *                      Used to return a pointer to the buffer allocated by this function to store the BMP image output.
 * @param [out]     bmpWidth, bmpHeight
 *                      Dimensions of the captured image in pixels.
 * @param [out]     bmpBufferSize
 *                      Total length in bytes of the buffer allocated by this function to store the BMP image output.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_QuickDumpSurface(IDirectFBSurface *surface, CNC_SINT32 x, CNC_SINT32 y, CNC_SINT32 width, CNC_SINT32 height,
        CNC_FLOAT32 xScale, CNC_FLOAT32 yScale,
        CNC_UINT8 **bmpBuffer, CNC_SINT32 *bmpWidth, CNC_SINT32 *bmpHeight, CNC_SINT32 *bmpBufferSize);

/**
 * @brief           Disables color-keying for the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_DisableColorKey(CNC_SINT32 virtualLayerID);

/**
 * @brief           Enables Color-keying for the specified Virtual Layer.
 *                  Color-keying is disabled by default.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_EnableColorKey(CNC_SINT32 virtualLayerID);

/**
 * @brief           Sets the color-key of the specified Virtual Layer.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      R
 *                      Red component of the key color.
 * @param [in]      G
 *                      Green component of the key color.
 * @param [in]      B
 *                      Blue component of the key color.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_SetColorKey(CNC_SINT32 virtualLayerID, CNC_UINT8 R, CNC_UINT8 G, CNC_UINT8 B);

/**
 * @brief           Checks if the specified Virtual Layer is having the same resolution as its associated Physical Layer (that is, no scaling will be used).
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @param [in]      nativeResolution
 *                      CNC_BOOL_TRUE = Virtual Layer is having the same resolution as its associated Physical Layer (that is, no scaling will be used).
 *                      CNC_BOOL_FALSE = Virtual Layer is having a different resolution from its associated Physical Layer (that is, scaling will be used).
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_IsNativeResolution(CNC_SINT32 virtualLayerID, CNC_BOOL *nativeResolution);

/**
 * @brief           Raises the specified Virtual Layer, such that it now appears above the Virtual Layer that was directly above it.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_Raise(CNC_SINT32 virtualLayerID);

/**
 * @brief           Lowers the specified Virtual Layer, such that it now appears below the Virtual Layer that was directly below it.
 * @param [in]      virtualLayerID
 *                      Specifies the Virtual Layer on which this operation must be performed.
 * @return          CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_DFB_Lower(CNC_SINT32 virtualLayerID);

#ifdef __cplusplus
}
#endif

#endif /* __CNC_DFB_H__ */
