/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_osd.h
 * @ingroup appframework
 * The CNCOSD library is a small, efficient drawing library for developing on-screen displays
 */

#ifndef  _CNC_OSD_H_
#define  _CNC_OSD_H_

#include "csosd.h"
#include "cnc_common_typedef.h"
#include "iconv.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void *          TTFFont_Handle;


//#define OSD_USE_ARGB8888 	1
#define OSD_USE_BGRA8888 	1
#define OSD_SCREEN_WIDTH 	1280
#define OSD_SCREEN_HEIGHT 	720
#define OSD_SCREEN_DEPTH	4

#define CNC_OSD_SURFACE_OPAQUE    0x3FF /* TODO: Must be under PLATFORM Switch */
#define CNC_OSD_SURFACE_TRANS     0x00

#define PALETTE_SIZE 256

/**
 * Points are used for locations on a graphic device
 */
typedef struct _cncOSDPoint_t
{
    int x;
    int y;
} cncOSDPoint_t;

/**
 * Rectangles are defined by their top-left point, width and height.
 * For simplicity all fields are signed, but negative widths and
 * heights are unlikely to be correct.
 */
typedef struct _cncOSDRect_t
{
    int x;
    int y;
    int w;
    int h;
} cncOSDRect_t;

/**
 * Rectangles are defined by their top-left point, width and height.
 * For simplicity all fields are signed, but negative widths and
 * heights are unlikely to be correct.
 */
typedef struct _cncOSDColor_t
{
    unsigned char red;   /**< Red component */
    unsigned char green; /**< Green component */
    unsigned char blue;  /**< Blue component */
    unsigned char trans; /**< Transparency component */
} cncOSDColor_t;

/**
 * Mask type. Masks are typically used for text rendering. A depth of
 * 8, for example, could be used for anti-aliased text.
 */
typedef struct _cncOSDMask_t
{
    int width;   /**< Pixels in each row */
    int height;  /**< Rows in the mask */
    int pitch;   /**< Bytes in each row (each new row starts at a
                           byte boundary). */
    int depth;   /**< Depth in bits of each multiplier in the mask data (e.g. 1
                            or 8) */
    unsigned char * mask;  /**< Mask data (alpha multipliers). */
} cncOSDMask_t;

/**
 * A Color Palette
 */
typedef struct _cncOSDPalette_t
{
    int entry_size;
    cncOSDColor_t entry[PALETTE_SIZE]; /**< An array of colours */
} cncOSDPalette_t;

/**
 * Error codes returned from the CNCOSD module.
 */
typedef enum _cncOSDErrorCode_t
{
    CNC_OSD_OK = 0,        /**< The call was successful */
    CNC_OSD_FAIL,          /**< Generic failure */
    CNC_OSD_NOT_SUPPORTED, /**< The requested call is not supported by this implementation */
    CNC_OSD_NOT_READY,     /**< The action can not be completed due to a resource being tied up */
    CNC_OSD_NOT_FOUND,     /**< The resource could not be located */
    CNC_OSD_NO_RESOURCE,   /**< The target does not have sufficient resources to complete the action */
    CNC_OSD_TIMEOUT        /**< The operation has not completed within a defined timeout */
} cncOSDErrorCode_t;

/**
 * The picture format supported
 */
typedef enum _cncOSDPictureFormat_t
{
    CNC_PICTURE_PNG = 4, 	/**< Portable Network Graphics */
    CNC_PICTURE_JPEG,    		/**< Joint Photographic Experts Group */
    CNC_PICTURE_BMP,
    CNC_PICTURE_GIF
} cncOSDPictureFormat_t;

/**
 * Color format for the pixels
 */
typedef enum _cncOSDPixelFormat_t
{
     /**
      * 8bpp paletted. Each pixel is represented as an 8-bit index into the
      * colour palette.
      */
    CNC_FB_LUT8 = 0,

    /**
     * 16bpp, rgba. Each pixel is represented by a byte-pair. The byte at
     * the lower memory address contains the blue value in bits 4-7 and the
     * alpha value in bits 0-3. The byte at the higher address contains the
     * red value in bits 4-7 and the green value in bits 0-3.
     */
    CNC_FB_RGBA4444,

    /**
     * 16bpp, argb. Each pixel is represented by a byte-pair. The byte at
     * the lower memory address contains the green value in bits 4-7 and the
     * blue value in bits 0-3. The byte at the higher address contains the
     * alpha value in bits 4-7 and the red value in bits 0-3.
     */
    CNC_FB_ARGB4444,

    /**
     * 32bpp, rgba. Each pixel is represented by 4 bytes. The byte at
     * the lowest memory address contains an 8-bit alpha value; consecutive
     * bytes hold the blue, green and red values in that order.
     */
    CNC_FB_RGBA8888,

    /**
     * 32bpp, argba. Each pixel is represented by 4 bytes. The byte at
     * the lowest memory address contains an 8-bit blue value; consecutive
     * bytes hold the green, red and alpha values in that order.
     */
    CNC_FB_ARGB8888
}cncOSDPixelFormat_t;

/**
 * Font alignment
 */
typedef enum _cncOSDFontFormat_t
{
      /*Aligns text to the left. This is default mode*/
      FF_LEFT = 0,

      /*Centers text horizontally in the rectangle.*/
      FF_CENTER,

      /*Aligns text to the right.*/
      FF_RIGHT,

      /*Top-justifies text (single line only).*/
      //FF_TOP,

      /*Centers text vertically (single line only).*/
      //FF_VCENTER,

      /*Justify the text to the bottom of the rectangle. This value must be combined with FF_SINGLELINE.*/
      //FF_BOTTOM,

      /*Break words. Lines are automatically broken beteween words if a word would
      **extend past the edge of the rectangle specified by the the pRect parameter. A
      **carriage return or linefeed also breaks the line.
      */
      //FF_WORDBREAK,

      /*Break characters. Lines are automatically broken beteween characters if a
      **character would extend past the edge of the rectangle specified by the the
      **pRect parameter. A carriage return or linefeed also breaks the line.
      **FF_CHARBREAK will override FF_WORDBREAK.
      */
      //FF_CHARBREAK,

      /*Display text ont the single line only. Carriage returns and linefeeds do not break the line.*/
      //FF_SINGLELINE,

      /*Expands tab characters. The default number of character per tab is eight.*/
      //FF_EXPANDTABS,

      /*Set tab stops. Bits 15-8 (hight-order byte of the low-order word) of the
      **uForma parameter specify the number of characters for each tab. The default
      **number of characters per tab is eight.
      */
      //FF_TABSTOP,

      /*Draws without clipping. DrawText is somewhat faster when FF_NOCLIP is used.*/
      //FF_NOCLIP,

      /*Determines the width and the height of the rectangle. If there are multiple
      **lines of text, DrawText uses the width of the rectangle pointed to by the
      **lpRect parameter and extends the base of the rectangle to bound the last line
      **of text. If there is only one line of text, DrawText modifies the right side
      **of the rectangle so that it bounds the last character in the line. In either
      **case, DrawText returns the height of the formatted text but does not draw the
      **text.
      */
      //FF_CALCRECT,
}cncOSDFontFormat_t;

/**
 * A Bitmap structure defining a picture and its memory
 */
typedef struct _cncOSDBitmap_t
{
    int width;   /**< Pixels in each row */
    int height;  /**< Rows in the bitmap */
    int pitch;   /**< Bytes in each row */
    cncOSDPixelFormat_t format; /**< Format of pixels in the bitmap */
    int pixel_resolution_w; /**< The horizontal pixel resolution */
    int pixel_resolution_h; /**< The vertical pixel resolution */
    unsigned char * bitmap;  /**< Bitmap pixel data.
                                  Access this memory only after invoking CNC_OSD_LockBitmap().
                                  Do not invoke any CNC_OSD functions on this after invoking CNC_OSD_LockBitmap().
                                  This is invalidated after invoking CNC_OSD_UnlockBitmap(). */
    void * private_data;     /**< Pointer to private data used internally by CNCOSD and should not be accessed or modified by the caller */
} cncOSDBitmap_t;

/**
 * Type for fonts.
 */
typedef enum
{
	FNT = 0, /**< Normal font */
	TTF      /**< True Type Font */
}cncOSDFontType_t;

/**
 * Font standard types
 */
typedef enum
{
      GBK= 0,                          // defoult
      GB2312,

      ISO_8859_1,
      ISO_8859_2,
      ISO_8859_3,
      ISO_8859_4,
      ISO_8859_5,
      ISO_8859_6,
      ISO_8859_7,
      ISO_8859_8,
      ISO_8859_9,
      ISO_8859_10,
      ISO_8859_11,
      ISO_8859_12,
      ISO_8859_13,
      ISO_8859_14,
      ISO_8859_15,
      ISO_8859_16,

      UNICODE,

      UTF_8,
      UTF_16,

      BIG5,

      UNKNOW
}cncOSDFontStandard_t;

/*
 * Parameters for a font
 */
typedef struct
{
	int width;
	int heigth;
	int x;
	int y;
	int distx;
	int disty;
	unsigned char *buf;
}cncOSDFontParam_t;


typedef enum  _cncOSDDisplayState_t
{
    CNC_OSD_VISIBLE,
    CNC_OSD_HIDDEN
} cncOSDDisplayState_t;

/**
Set Simple UI Draw Position Function,
It will become effective when next draw , Set w=h=0 it will set to default config
*/
void CNC_OSD_SetPosition(int x, int y, int w, int h);

/**
 * Initialize the CNCOSD Module
 * @param osd_handle Handle to the low level graphics layer
 * @return 0 if failure, 1 if success
 */
int CNC_OSD_Init(CSOSD_HANDLE osd_handle);

/**
 * Initialize a font for the module
 * @param font_standard Standard type of the font
 * @param ui_config_path pathname to the font
 * @return 0 if failure, 1 if success
 */
int CNC_OSD_InitFont(cncOSDFontStandard_t font_standard, char * ui_config_path);

/**
 * For some graphics base address will change in some conditions,
 * Never call this function when a draw action not done !
 * @param baseAddr new base address for the OSD layer
 */
void CNC_OSD_ResetBaseAddr(const char * baseAddr);

/**
 * Start a drawing operation on a clean graphics area
 * @param x the x location
 * @param y the y location
 * @param w the width of the area
 * @param h the height of the area
 * @see CNC_OSD_EndDraw
 */
int CNC_OSD_StartDraw(int x, int y, int w,int h);

/**
 * Complete the drawing operation and render the graphics to the screen (scaled to display size)
 * @see CNC_OSD_StartDraw
 */
int CNC_OSD_EndDraw(void);

/**
 * Simple UI Load Bitmap Function for loading images from filenames
 * Now olny support: png jpeg bmp gif
 * Auto identify by suffixes
 * @param file_name pathname of the image file
 * @return a bitmap structure including pointer to the image memory
*/
cncOSDBitmap_t *CNC_OSD_LoadBitmap(char * file_name);

/**
 * Simple UI Load Bitmap Function
 * Now only support: png jpeg bmp gif
 * Auto identify by suffixes
 * @param pData pointer to the image data in memory
 * @param size  size of the image data
 * @param fmt   format type of the image in memory
 * @return a bitmap structure including pointer to the image memory
 */
cncOSDBitmap_t *CNC_OSD_LoadBitmapFromMemory(char * pData,int size,cncOSDPictureFormat_t fmt);


/**
 * Simple UI Draw Function to fill a space with the given bitmap;
 * Auto scale by CPU,
 * @param bmp the bitmap structure
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width to fill
 * @param h the height to fill
 */
int CNC_OSD_FillBitmap(cncOSDBitmap_t const * bmp, int x, int y, int w,int h);

/**
 * Simple UI Draw Bitmap Function: Tiled mode
 * @param bmp the bitmap structure
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width to fill
 * @param h the height to fill
 */
int CNC_OSD_FillBitmapTiled(cncOSDBitmap_t const * bmp, int x, int y, int w,int h);

/**
 * Simple UI Draw Bitmap Function: mix with graphic layer, r g b a <=5 will be miss
 * @param bmp the bitmap structure
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width to fill
 * @param h the height to fill
*/
int CNC_OSD_FillBitmapMix(cncOSDBitmap_t const * bmp, int x, int y, int w,int h);

/**
 * Simple UI Draw Bitmap Function: Blending with graphic layer, based on standard
 * @param bmp the bitmap structure
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width to fill
 * @param h the height to fill
 */
int CNC_OSD_FillBitmapBlend(cncOSDBitmap_t const * bmp, int x, int y, int w,int h);

/**
 * Free the bitmap memory
 * @param bmp the bitmap to free
 */
int CNC_OSD_FreeBitmap(cncOSDBitmap_t * bmp);

/**
 * Load a raw (predecoded) image from a file
 * @param file_name the file name of the raw image
 * @return bitmap structure holding the image info and memory
 */
cncOSDBitmap_t *CNC_OSD_LoadBitmap_Raw(char * file_name);

/**
 * Get a bitmap from the current graphics plane
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width of the graphics area
 * @param h the height of the graphics area
 * @return bitmap structure holding the image info and memory
 */
cncOSDBitmap_t * CNC_OSD_GetBitmapFromGFX(int x, int y, int w,int h);

/**
 * Fill an area with a color defined by the input pen
 * @param pen the color to fill the area
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width to fill
 * @param h the height to fill
 */
int CNC_OSD_FillArea(cncOSDColor_t const * pen, int x, int y, int w,int h);

/**
 * Clear the graphics in the defined area
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width to clear
 * @param h the height to clear
 */
int CNC_OSD_ClearArea(int x, int y, int w,int h);


/**
 * Set a background color for the font
 * @param red red component
 * @param green green component
 * @param blue blue component
 * @param trans transparency component
 */
int CNC_OSD_SetFontBk(unsigned char red, unsigned char green, unsigned char blue,unsigned char trans);

/**
 * Set a background color for the font
 * @param red red component
 * @param green green component
 * @param blue blue component
 * @param trans transparency component
 */
int CNC_OSD_SetFontBackground(unsigned char red, unsigned char green, unsigned char blue, unsigned char trans);

/**
 * Set the foreground color for the font
 * @param red red component
 * @param green green component
 * @param blue blue component
 * @param trans transparency component
 */
int CNC_OSD_SetFontForeground(unsigned char red, unsigned char green, unsigned char blue,unsigned char trans);

/**
 * Set the font size for rendering
 * @param size font size
 */
int CNC_OSD_SetFontSize(unsigned int size);

/**
 * Get the current font size for rendering
 * @return font size
 */
int CNC_OSD_GetFontSize(void);


/**
 * This is a truetype font function, that loads the truetype lib to get font and draw into right position.
 * @param font the text to draw
 * @param x the x coordinate
 * @param y the y coordinate
 * @return the width of character string
*/
int CNC_OSD_DrawFont(char * font, int x, int y);

/**
 * This is a truetype font function, that loads the truetype lib to get font and draw into right position with extra alignment information
 * @param font the text to draw
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width of the text
 * @param fmt the alignment of the text
 * @return the width of character string
*/
int CNC_OSD_DrawFontEx(char * font , int x, int y,int w,cncOSDFontFormat_t fmt);

/**
 * Get the width of the text in the current font
 * @param font the text
 * @return a string width value.
*/
int CNC_OSD_GetFontWidth(char * font);

/**
 * Load text using the TTF font
 * @param font the text to load
 * @return a handle to the created object
 */
TTFFont_Handle CNC_OSD_LoadTTFFont(char * font);

/**
 * This is a truetype font function, that loads the truetype lib to get font and draw into right position.
 * @param ttf_handle TTF Font String handle
 * @param x the x coordinate
 * @param y the y coordinate
 * @return the width of character string
*/
int CNC_OSD_DrawTTFFont(TTFFont_Handle ttf_handle , int x, int y);

/**
 * This is a truetype font function, that loads the truetype lib to get font and draw into right position.
 * @param ttf_handle TTF Font String handle
 * @param x the x coordinate
 * @param y the y coordinate
 * @param w the width of the TTF font string
 * @param fmt the format of the string
 * @return the width of character string
*/
int CNC_OSD_DrawTTFFontEx(TTFFont_Handle ttf_handle ,int x, int y,int w,cncOSDFontFormat_t fmt);

/**
 * Free the TTF Font String
 * @param ttf_handle TTF Font String handle
 */
int CNC_OSD_FreeTTFFont(TTFFont_Handle ttf_handle);

/**
 * Get the width of the TTF font String
 * @param ttf_handle TTF Font String handle
 * @returns width of the string (in pixels)
 */
int CNC_OSD_Get_TTFFont_Width(TTFFont_Handle ttf_handle);

/**
 * Allows the caller to create a bitmap by specifying all the structure's public members.
 * This function only prepares the bitmap for further operations by initialising any private data.
 * It does not allocate any bitmap data and this must be taken care by the caller.
 * @param bmp the bitmap structure
 */
int CNC_OSD_PrepareBitmap(cncOSDBitmap_t *bmp);

/**
 * Lock a bitmap for direct access (when directly reading or writing the bitmap data)
 * @param bmp the bitmap structure
 * @param wr  if 1, specifies the bitmap is being locked for a write operation, otherwise read.
 */
int CNC_OSD_LockBitmap(cncOSDBitmap_t * bmp, int wr);

/**
 * Unlock a bitmap that was previously locked for direct access
 * @param bmp the bitmap structure
 */
int CNC_OSD_UnlockBitmap(cncOSDBitmap_t * bmp);

/**
 * Set the current state of OSD
 * @param[in]   state
 *                  CNC_OSD_VISIBLE, if Visible \n
 *                  CNC_OSD_HIDDEN, if Not Visible
 */
void CNC_OSD_SetDisplayState(cncOSDDisplayState_t state);

/**
 * Convert unicode string to G-based string
 * @param inbuf the input string
 * @param inlen the length of the input string
 * @param outbuf the resulting string
 * @param outlen  the length of the resulting string
 */
int u2g(char *inbuf,int inlen,char *outbuf,int outlen);

/**
 * Convert g-based string to Unicode string
 * @param inbuf the input string
 * @param inlen the length of the input string
 * @param outbuf the resulting string
 * @param outlen  the length of the resulting string
 */
int g2u(char *inbuf,int inlen,char *outbuf,int outlen);

/**
 * Convert from ISI 8859-1 to UTF8 String
 * @param inbuf the input string
 * @param inlen the length of the input string
 * @param outbuf the resulting string
 * @param outlen  the length of the resulting string
 */
int ISO88591_UTF8(char *inbuf,int inlen,char *outbuf,int outlen);

#ifdef __cplusplus
}
#endif

#endif  // _CNC_OSD_H_

