/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_pvr.h
 * @ingroup btv
 * The CNCPVR library is used to implement personal video recording (PVR) capabilities
 */

#ifndef  __CNC_PVR_H__
#define  __CNC_PVR_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"
#include "cnc_btv.h"
#include "cnc_av.h"

#define TIMESHIFT_FILENAME 	"timeshift"
#define PVR_CMP_FLAG 	      "celestial"


typedef enum
{
      CNC_PVR_STATUS_IDLE 		= 0x00,
      CNC_PVR_STATUS_REC		= 0x01,
      CNC_PVR_STATUS_PLAYER 	= 0x02,
      CNC_PVR_STATUS_TIMESHIFT 	= 0x03,
      CNC_PVR_STATUS_UNKNOW
} cncPVRStatus_t;

typedef enum
{
      CNC_RECORD_TYPE_NONE		= 0x00,
      CNC_RECORD_TYPE_PVR		= 0x01,
      CNC_RECORD_TYPE_TIMESHIFT	= 0x02,
      CNC_RECORD_TYPE_ANLOGCHANNEL  = 0x03,
      CNC_RECORD_TYPE_UNKNOWN
} cncPVRRecordType_t;

typedef struct _cncPVRRecordData_t
{
      char checkflag[10];
      int duration;           //sec
      int vpid;
      int vtype;
      int apid;
      int atype;
} cncPVRRecordData_t;

typedef enum 
{
        CNC_PVR_DISKOK,
        CNC_PVR_DISKERR,
        CNC_PVR_DISKFULL,
        CNC_PVR_DISKFILEMAX
} cncPVR_DiskStatus_t;

/// Callback function type for receiving DISK state
typedef void (*cncPVR_DiskCallback_t)(cncPVR_DiskStatus_t diskstate);

/**
 * Write recording data for the digital PVR
 * @param filename file name for the recorded file
 * @param RecDate channel recording parameters
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_WriteFile(char* filename, cncPVRRecordData_t RecDate);

/**
 * Read the recording data from the previously recorded content
 * @param filename file name for the recorded file
 * @param p_RecData recording data
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_ReadFile(char* filename, cncPVRRecordData_t *p_RecData);

/**
 * Start the digital PVR recording
 * @param filename filename for the recording
 * @param serviceInfo recorded channel info
 * @param rec_type Recording Type
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_Start(char * filename, cncBTVServiceInfo_t serviceInfo, cncPVRRecordType_t rec_type);
Result_Type_e CNC_PVRTS_Start(char * filename, cncBTVServiceInfo_t serviceInfo, cncPVRRecordType_t rec_type);

/**
 * Stop the recording process
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_Stop(void);

/**
 * Pause the recording process
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_Pause(void);

/**
 * Resume a previously paused recording process
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_Resume(void);

/**
 * Terminate the PVR module
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_Term(void);

/**
 * Get the recording status
 * @param nStatus [out] status
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_GetStatus( CNC_UINT32 *nStatus );

/**
 * Get the active recording file name
 * @param filename File Name
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_GetFileName( char* filename );

/**
 * Get the total time that has been recorded
 * @param nSecs Time (in seconds)
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_PVR_GetTime(CNC_UINT32 *nSecs);

/**
 * Get the active recording type
 * @return Recording Type
 */
cncPVRRecordType_t CNC_PVR_GetRecordType(void);

/**
 * Initialize the analog recording
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
CNC_BOOL CNC_APVR_RecInit(void);

/**
 * Re-initialize the analog PVR to start recording again
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
CNC_BOOL CNC_APVR_ReInit(void);

/**
 * Start analog recording into the input filename
 * @param filename File Name
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
CNC_BOOL CNC_APVR_RecStart(char* filename);

/**
 * Stop the analog PVR recording
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
CNC_BOOL CNC_APVR_RecStop(void);

/**
 * Terminate the analog PVR module
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
CNC_BOOL CNC_APVR_RecTerm(void);

/**
 * Set the parameters for analog PVR recording
 * @param ratio Ratio
 * @param frame Frame
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
CNC_BOOL CNC_APVR_RecSetParameters( unsigned int ratio, unsigned int frame );

/**
 * Install a callback function when disks are inserted or removed
 * @return none
 */
void CNC_PVR_InstallCallback(cncPVR_DiskCallback_t callback);

/**
 * Uninstall the disk callback function
 * @return none
 */
void CNC_PVR_UninstallCallback(void);


#ifdef __cplusplus
} 
#endif 

#endif

