/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_tuner.h
 * @ingroup btv
 * The CNCTUNER module is part of the CNCBTV library and is used for managing broadcast tuner inputs
 */
#ifndef  __CNC_TUNER_H__
#define  __CNC_TUNER_H__

#include "cnc_common_typedef.h"
#include "cstuner.h"

#ifdef __cplusplus
extern "C" {
#endif

#define CNC_TUNER_DEBUG printf
#define CHECK_TUNER_PARAMS_VALID(p) do { \
	if (NULL == p) { \
		return RESULT_BAD_PARAMETER; \
	} \
} while(0)

typedef unsigned char cncTunerHandle_t; 

typedef enum
{
      CNC_TUNER_ID_INVALID = 0,
      CNC_TUNER_ID_0 = 1,
      CNC_TUNER_ID_1,
      CNC_TUNER_ID_2,
      CNC_TUNER_ID_3,
      CNC_TUNER_ID_MAX
} cncTunerID_t;

typedef enum
{
    CNC_TUNER_INVALID = -1,
    CNC_TUNER_DVBC = 0,
    CNC_TUNER_ISDB = 1,
    CNC_TUNER_DVBT = 2,
    CNC_TUNER_DVBS2 = 3,
    CNC_TUNER_ATV = 4,
    CNC_TUNER_ALL,
    CNC_TUNER_TYPE_NUM,
} cncTunerModulationType_t;

typedef enum
{
    CNC_TUNER_INPUT_PARALLEL = 0,         ///tuner  normal input mode 
    CNC_TUNER_INPUT_SERIAL = 1,             ///for HW2.0 version, three tuner input mode.
} cncTunerInputMode_t;

#define cncTunerQAMMode_t int
#define CNC_TUNER_QAM_4    0
#define CNC_TUNER_QAM_16   1
#define CNC_TUNER_QAM_32   2
#define CNC_TUNER_QAM_64   3
#define CNC_TUNER_QAM_128  4
#define CNC_TUNER_QAM_256  5
#define CNC_TUNER_QAM_AUTO 6

typedef struct{
		CNC_UINT32 m_uiFrequency_kHz;		///< The satellite carrier frequency, in a unit of kHz. This member is used ONLY for carrying back information from blind scan.
		CNC_UINT32 m_uiSymbolRate_Hz;		///< The symbol rate in a unit of Hz. 
		CNC_UINT32 m_Flags;					///< Other informations like the signal standard and IQ spectrum \sa CI_FLAG_DVBS, CI_FLAG_DVBS2, CI_FLAG_IQ_SWAPPED ...
} cncTunerBlindChannel_t;

typedef enum
{
    CNC_TUNER_UNLOCKED 	= 0,
    CNC_TUNER_LOCKED 	= 1,
    CNC_TUNER_LOCKING 	= 2,
    CNC_TUNER_RESET       = 3
} cncTunerLockStatus_t;

typedef void (*cncTunerCallBackFunction_t)(cncTunerHandle_t hand, cncTunerLockStatus_t value);

typedef enum
{
    NOTIFY_AlWAYS,
    NOTIFY_CHANGE,
    NOTIFY_INVALID
} cncTunerInstallNotifyType_t;

typedef struct
{
    CNC_UINT32 frequency; //MHZ
    CNC_UINT32 symbol_rate;
    cncTunerQAMMode_t modulation;
   // תλṹ /*ĿǰAuto*/
    unsigned int	 bandwidth;
} cncTunerQAMParams_t;

typedef struct
{        
      CNC_UINT32 frequency; //MHZ
      CNC_UINT32    symbol_rate;   /* symbol rate in Symbols per second */
      CNC_SINT16   fec_inner;     /* forward error correction (see above) */
      CNC_SINT16   dvbsors2;      /*DVB-S OR DVB-S2*/
      CNC_SINT16   qbskor8bsk;    /*qbsk or 8psk*/
      CNC_SINT16   sec_voltage;   /*18ѹ SEC_VOLTAGE_13ʾ13SEC_VOLTAGE_18ʾ18*/
      CNC_SINT16   sec_22k_onoff; /*22KHZο*/
      CNC_UINT8    polarization;
} cncTunerDVBS2_t;

typedef struct
{
    CNC_UINT32 frequency; //MHZ
    CNC_UINT32 symbol_rate;
    CNC_UINT8 modulation;
    CNC_UINT8 bandwidth;
    CNC_UINT32 msdelay;
} cncTunerISDB_t;

typedef struct
{
    CNC_UINT32 frequency; //MHZ
    CNC_UINT32 symbol_rate;
    CNC_UINT8 modulation;
    CNC_UINT8 bandwidth;
} cncTunerDVBT_t;

typedef struct
{
    CNC_UINT32 frequency; //MHZ
    CNC_UINT32 symbol_rate;
    CNC_UINT8 modulation;
} cncTunerATV_t;

typedef struct
{
    cncTunerModulationType_t type;
    cncTunerInputMode_t  input_module;
    union
    {
        cncTunerQAMParams_t 	dvbc;
        cncTunerISDB_t 	isdb;
		cncTunerDVBT_t 	dvbt;
        cncTunerDVBS2_t   dvbs2;
        cncTunerATV_t       atv;
    } mode;
} cncTuner_t;

/**
 * Initialize the tuner module
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Init(void);

/**
 * Terminate the use of the tuner module
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_DeInit(void);

/**
 * Open the tuner
 * @param id Tuner Id
 * @param tunergpio Tuner GPIO control number
 * @return Tuner Handle
 */
cncTunerHandle_t CNC_Tuner_Open(cncTunerID_t id,char tunergpio);

/**
 * Close the tuner
 * @param handle  Tuner Handle
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Close(cncTunerHandle_t handle);

/**
 * Set tuner parameters
 * @param handle Tuner Handle
 * @param params Tuner Parameter Configuration
 * @param block blocking/non-blocking
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_SetParams(cncTunerHandle_t handle, cncTuner_t  *params ,CNC_BOOL block);

/**
 * Read the tuner parameters
 * @param handle Tuner Handle
 * @param params [out] Tuner Parameter Structure
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_GetParams(cncTunerHandle_t handle, cncTuner_t *params);

/**
 * Reset the tuner
 */
void CNC_Tuner_Reset(void);

/**
 * Get the tuner lock status
 * @param handle Tuner Handle
 * @return status (unlocked, locked, locking, reset)
 */
cncTunerLockStatus_t CNC_Tuner_GetLockStatus(cncTunerHandle_t handle);

/**
 * Install a notify function for tuner status change events
 * @param handle Tuner Handle
 * @param func Callback function
 * @param type Notification type (always, changed, invalid)
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_InstallNotify(cncTunerHandle_t handle, cncTunerCallBackFunction_t func,cncTunerInstallNotifyType_t type);

/**
 * Uninstall notify function
 * @param handle Tuner Handle
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_UninstallNotify(cncTunerHandle_t handle);

/**
 * Read the current signal quality on the tuner
 * @param handle Tuner Handle
 * @return tuner quality level
 */
CNC_SINT32 CNC_Tuner_GetSignalQuality(cncTunerHandle_t handle);

/**
 * Read the signal strength from the tuner
 * @param handle Tuner Handle
 * @param parastatus
 * @return signal strength
 */
CNC_SINT32 CNC_Tuner_GetSignalStrength(cncTunerHandle_t handle, TUNER_STATUS_BA *parastatus);

/**
 * Return signal strength from the tuner (doesn't require tuner handle)
 * @return signal strength
 */
CNC_SINT32 CNC_Tuner_GetCurSignalStrength(void);

/**
 * Return the bit error rate (BER) from the tuner
 * @param handle Tuner Handle
 * @return bit error rate
 */
CNC_UINT32 CNC_Tuner_GetSignalBer(cncTunerHandle_t handle);

/**
 * Set the low-noise block for satellite tuner
 * @param handle Tuner Handle
 * @param lnb low noice block value
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_SetSatLnb(cncTunerHandle_t handle,CNC_UINT32 lnb);

/**
 * Get the tuner handle from the tuner id
 * @param id Tuner Id
 * @return Tuner Handle
 */
cncTunerHandle_t CNC_Tuner_GetHandle(cncTunerID_t id);

/**
 * Set the monitor flag for the active tuner
 * @param flag on or off
 */
void CNC_Tuner_SetMonitorFlag(CNC_BOOL flag);

/**
 * Get the current lock state of the analog TV tuner (if exists)
 * @return unlocked, locked
 */
cncTunerLockStatus_t CNC_Tuner_GetAtvTunerLockState(void);

/**
 * Set the lock state of an analog tuner
 * @param lockflag on or off
 */
void CNC_Tuner_SetAtvTunerLockState(cncTunerLockStatus_t lockflag);

/**
 * Get the current lock state of the digital TV tuner (if exists)
 * @return unlocked, locked
 */
cncTunerLockStatus_t CNC_Tuner_GetDtvTunerLockState(void);

/**
 * Set the lock state of a digital tuner
 * @param lockflag on or off
 */
void CNC_Tuner_SetDtvTunerLockState(cncTunerLockStatus_t lockflag);

/**
 * Set the low noise black parameter for the tuner
 * @param handle Tuner handle
 * @param lnb low noice block value
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Setlnb(cncTunerHandle_t handle, unsigned char lnb);

/**
 * Run blind scan on the tuner
 * @param handle Tuner Handle
 * @param channels [out] blind channels
 * @param numberchannel [out] number of blind channels
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_BlindScan(cncTunerHandle_t handle, cncTunerBlindChannel_t *channels,unsigned short *numberchannel);

/**
 * Read modulation from satellite control (DVB-S)
 * @param handle Tuner Handle
 * @param ucData modulation data
 * @param numb size of modulation data
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Diseqc_ReadModulationData(cncTunerHandle_t handle,unsigned char *ucData, unsigned char *numb);

/**
 * Send Modulation Data to the satellite controller
 * @param handle Tuner Handle
 * @param ucData modulation data
 * @param numb size of modulation data
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Diseqc_SendModulationData(cncTunerHandle_t handle,unsigned char *ucData,unsigned char numb);

/**
 * Start continuous mode in satellite controller
 * @param handle Tuner Handle
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Diseqc_StartContinuous(cncTunerHandle_t handle);

/**
 * Stop continuous mode in satellite controller
 * @param handle Tuner Handle
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_Tuner_Diseqc_StopContinuous(cncTunerHandle_t handle);

#ifdef __cplusplus
} 
#endif 

#endif
