#ifndef _CNCDIAL_H
#define _CNCDIAL_H

/**
 * The maximum DIAL payload accepted per the DIAL 1.6.1 specification.
 */
#define DIAL_MAX_PAYLOAD 4096

#define DIAL_URL_LEN 128

/**
 * The maximun size of an dial application
 */
#define DIAL_APP_NAME_SIZE 64


/**
 * Define the state of a dial application
 */
typedef enum {
    CNC_DIAL_APP_STATE_STOPPED = 0,
    CNC_DIAL_APP_STATE_RUNNING,
} cncDialAppState_t;

/** 
 * Define the type of the application. For chromecast applications
 * there is a websockets emulation layer.
 */
typedef enum {
    CNC_DIAL_APP_TYPE_NORMAL,
    CNC_DIAL_APP_TYPE_CHROMECAST,
} cncDialAppType_t;

/**
 * Struct containing the description and state of a dial application
 */
typedef struct _cncDialApp_t {
    /**
     * Callback function that will be called when cncdial wants to
     * start the application. If it returns CNC_DIAL_APP_STATE_STOPPED
     * cncdial will report that the application couldn't start.
     */
    cncDialAppState_t (*startCallback)(const struct _cncDialApp_t*);
    /**
     * Callback function that will be called when cncdial wants to
     * stop the application
     */
    void (*stopCallback)(const struct _cncDialApp_t*);
    /**
     * Callback function that will be called when cncdial wants to
     * know the state of an application.
     * It should return the actual state.
     * In *canStop the callback should write a 0 if the application
     * can't be stopped from the dial protocol or 1 if it can be"
     */
    cncDialAppState_t (*stateCallback)(const struct _cncDialApp_t*,
                                       int *canStop);
    char name[DIAL_APP_NAME_SIZE]; /**< Name of the dial applcation */
    char url[DIAL_URL_LEN];      /**< Url to be launched when the application starts */
    char args[DIAL_MAX_PAYLOAD]; /**< Payload received from the client */
    cncDialAppType_t type;    /**< Type of the application */
    cncDialAppState_t state;    /**< State of the application */
} cncDialApp_t;

/**
 * Configuration struct for cncdial
 */
typedef struct {
    char ipAddr[32];            /**< Ip address to be used  */
    char friendlyName[32];      /**< Name that will be announced */
    char modelName[32];         /**< Model name announced */
    char manufacturer[32];      /**< Manufacturer announced */
    char uuid[64];              /**< UUID announced, it uses the format "deadbeef-dead-beef-dead-beefdeadbeef" */
} cncDialConf_t;

/** 
 * Function that initialize cncdial. It initialize its own thread and returns.
 * 
 * @param conf Configuration used to start cncdial. It does not
 * depends on conf being available once CNC_DIAL_Init ends.
 */
void CNC_DIAL_Init(cncDialConf_t *conf);

/** 
 * Adds a new dial application to cncdial
 * 
 * @param app Application to be added, it copy app information and
 * doesn't depends on app being available after it ends.
 */
void CNC_DIAL_RegisterApp(const cncDialApp_t *app);

/** 
 * Removes an application from Dial
 * 
 * @param app Application to be removed.
 */
void CNC_DIAL_UnregisterApp(const char *app_name);

/** 
 * Stop cncdial. It also closes the cncdial thread.
 */
void CNC_DIAL_Close();

/** 
 * Notify cncdial that the running application has been stopped.
 */
void CNC_DIAL_StopCurrentApp();

#endif // _CNCDIAL_H
