/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc., reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.,

 ALL RIGHTS RESERVED.

 Copyright 2013-2014, Cavium Inc.,
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_dmcp.h
 * @brief CRISP DLNA Digital Media Control Point Module API
 *        DMCP can behave either as a Digital Media Player (DMP) or Digital Media Controller (DMC)
 *        This enables creations of applications which can allow user to select a Server (DMS)
 *        and browse A/V/I contents using folder or AVI views and allows user to render a selected content
 *        either locally (using CRISP media player) or remotely by selecting one of the Digital Media Renderers
 *        - DMRs available on the network, thereby behaving as either a DMP or DMC seamlessly.
 *        Application though can choose to configure (while calling DMCP_Create()) this module to work as DMP 
 *        only (local playback only) OR as DMCP (local or remote playback)
 *      
 * @ingroup dlna
 */

#ifndef __CNC_DMCP_H__
#define __CNC_DMCP_H__

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_dlna.h"
#include "cnc_upnp.h"
#include "cnc_mediainfo.h"

/*========================= (Internal) Macro definitions=================== */
#define CNC_DMCP_MAX_FORMAT_SIZE             32  /**< Maximum Format String length */

/* ================= (Internal) Type definitions =========================== */

/**
 * @enum  cncDmcpConfigType_t
 * @brief Used to configure Control point Mode to act like either DMP or DMC.
 */
typedef enum _cncDmcpConfigType_t
{
    CNC_DMCP_NOT_CONFIGURED       = 0,
    CNC_DMCP_DIGITAL_MEDIA_PLAYER,                                      /**< Used to configure  DMP mode */
    CNC_DMCP_DIGITAL_MEDIA_CONTROLLER,                                  /**< Used to configure  DMC mode */
}cncDmcpConfigType_t;

/**
 * @struct  cncDmcpServerAviContentItem_t
 * @brief   Contains Media details of All contents of specific type [either image, audio or video]
 * for each DMS device while retrieving Content in AVI view
*/
typedef struct _cncDmcpServerAviContentItem_t
{
    cncMediaCtlInfo_t                       *p_mediaCtlInfo;            /**< Media details of Content*/
    struct _cncDmcpServerAviContentItem_t   *p_nextContent;             /**< Next content details of same content type*/
} cncDmcpServerAviContentItem_t;


/**
 * @struct cncDmcpServerAVIviewList_t
 * @brief  Contains All different content Types information present in a DMS device
 *          while Retrieving content in AVI view
 */
typedef struct _cncDmcpServerAVIviewList_t
{
    CNC_SINT32                     numOfAudios;                         /**< Number of audio items present in DMs device*/
    CNC_SINT32                     numOfVideos;                         /**< Number of Video items present in DMs device*/
    CNC_SINT32                     numOfImages;                         /**< Number of Image items present in DMs device*/
    cncDmcpServerAviContentItem_t  *p_audioList;                        /**< Contains each Audio content details Available in DMS */
    cncDmcpServerAviContentItem_t  *p_videoList;                        /**< Contains each Video content details Available in DMS*/
    cncDmcpServerAviContentItem_t  *p_imageList;                        /**< Contains each Image content details Available in DMS*/

    void                           *p_v_private;                        /**< Private Member used by internal Core APIs */
} cncDmcpServerAVIviewList_t;


/**
 * @brief  The enumeration which Contains Type of View to retrieve contents from DMS device
 */
typedef enum _cncDmcpServerContentViewType_t
{
    CNC_DMCP_MEDIA_SERVER_CONTENT_AVI_LIST_VIEW,                         /**< Use to retrieve contents in AVI view from DMS */
    CNC_DMCP_MEDIA_SERVER_CONTENT_FILE_SYSTEM_VIEW                       /**< Use to retrieve contents in File system view from DMS */

}cncDmcpServerContentViewType_t;


/**
 * @brief  The enumeration which Contains Media Content Item Type
 */
typedef enum _cncDmcpServerContentItemType_t
{
    CNC_DMCP_AUDIO_CONTENT_ITEM,                                         /**< Content is Audio */
    CNC_DMCP_VIDEO_CONTENT_ITEM,                                         /**< Content is video */
    CNC_DMCP_IMAGE_CONTENT_ITEM                                          /**< Content is Image */

}cncDmcpServerContentItemType_t;


/**
 * @brief  The enumeration which Contains File System Item type
 */
typedef enum _cncDmcpFlieSysItemType_t
{
    CNC_DMCP_FS_ITEM_DIRECTORY,                                          /**< Item is a directory */
    CNC_DMCP_FS_ITEM_MEDIA_CONTENT                                       /**< Item is a media content*/

}cncDmcpFlieSysItemType_t;


/**
 * @struct cncDmcpFileSysItem_t
 * @brief  A structure which Contains File System Item node
 */
typedef struct _cncDmcpFileSysItem_t
{
    cncDmcpFlieSysItemType_t        itemType;                           /**< Enumeration of cncDmcpFlieSysItemType_t type*/
    void                            *p_v_item;                          /**< Points to cncDms_FS_DIR_Item_t if item is directory, cncDms_FS_Content_Item_t if item is Content*/
    struct _cncDmcpFileSysItem_t    *p_next;                            /**< Holds next item of cncDms_FS_Item_t type*/

}cncDmcpFileSysItem_t;


/**
 * @struct cncDmcpFileSysItemList_t
 * @brief  Contains File System Item List
 */
typedef struct _cncDmcpFileSysItemList_t
{
    cncDmcpFileSysItem_t     *p_fsItem;                                  /**< ItemList of type cncDmcpFileSysItem_t*/

}cncDmcpFileSysItemList_t;


/**
 * @struct cncDmcpFileSysDirItem_t
 * @brief  A structure which Contains File System Directory Item
 */
typedef struct _cncDmcpFileSysDirItem_t
{
    CNC_CHAR                    *p_Id;                                  /**< Directory Id */
    CNC_CHAR                    *p_title;                               /**< Name of Directory*/
    CNC_SINT32                  childCount;                             /**< Total Number of items Present in Directory includes both folders, contents*/
    CNC_SINT32                  numReturn;                              /**< Number of items retrieved from Directory*/

    cncDmcpFileSysItemList_t    *p_itemList;                            /**< ItemList present in the Directory*/

}cncDmcpFileSysDirItem_t;


/**
 * @struct cncDmcpFileSysContentItem_t
 * @brief  A structure which Contains File System Content Item
 */
typedef struct _cncDmcpFileSysContentItem_t
{
    cncMediaCtlInfo_t           *p_mediaCtlInfo;                        /**< Contains Media details of Content*/
    CNC_VOID                    *p_metadata;                            /**< Can be used only for renderer playback */

}cncDmcpFileSysContentItem_t;


/**
 * @struct cncDmcpFileSysView_t
 * @brief  A structure which Contains  File System View
 */
typedef struct _cncDmcpFileSysView_t
{
    cncDmcpFileSysDirItem_t     rootDir;                                 /**< Root folder of DMS device*/

}cncDmcpFileSysView_t;


/**
 * @struct  cncDmcpAVIview_t
 * @brief   A structure which Contains Grouped Content List in AVI View
 */
typedef struct _cncDmcpAVIview_t
{
    cncDmcpServerAVIviewList_t     *p_contentList; /**< Content List */

}cncDmcpAVIview_t;


/**
 * @struct  cncDmcpServerViewResult_t
 * @brief   A structure which Contains Content View Result
 */
typedef struct _cncDmcpServerViewResult_t
{
    cncDmcpServerContentViewType_t  viewType;                           /**< Content Retrieving type either AVI view or FS view*/

    //TODO union
    cncDmcpAVIview_t                *p_aviView;                         /**< Use while retrieving Content in AVI view*/
    cncDmcpFileSysView_t            *p_fsView;                          /**< Use while retrieving content in FS view */

}cncDmcpServerViewResult_t;


/**
 * @struct cncDmcpConfigParam_t
 * @brief  Contains All configurable Control Point parameters
 */
typedef struct _cncDmcpConfigParam_t
{
    cncDmcpConfigType_t     dmcpConfigType;                             /**< To configure Control point mode */
    CNC_SINT32              minAsyncTimeOut;                            /**< Minimum time Asynchronous content Retriever API can wait before return.*/
    CNC_SINT32              maxAsyncTimeOut;                            /**< Maximum time Asynchronous content Retriever wait before return API.*/
    CNC_SINT32              sliceCount;                                 /**< Request number of Contents required in GetContentListFromFolderAsync API response at a time.*/
    CNC_SINT32              soapClientBufSize;                          /**< Soap-Client Buffer Size */
    cncUpnpConfigParam_t    upnpConfigParam;                            /**< Contains UPnP Configurable parameters.*/
}cncDmcpConfigParam_t;


/*
 * @brief DMCP  Instance Handle.
 */
typedef void*  cncDmcpHandle_t;

/* ======================== Function declarations ========================= */


/**
 * @brief   : Upnp intialization
 * @param   : None
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t  CNC_DMCP_Init(void);


/**
 * @brief   Creating DMCP  instance.
 * @param   [OUT]*pp_DMCHandle     : DMCP  Handle
 * @param   [IN]*p_dmcpConfigParam : DMCP  Configured parameters Details by Applcation.
 * @param   [IN]p_f_eventNotifier  : Notifier
 *
 * @see cncDmcpConfigParam_t
 * @see F_UpnpEventNotification_t
 *
 * @return  : --> E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t  CNC_DMCP_Create(cncDmcpHandle_t           *pp_dmcpHandle,
                                  cncDmcpConfigParam_t      *p_dmcpConfigParam,
                                  F_UpnpEventNotification_t p_f_eventNotifier);


/**
 * @brief   Start DMCP  instance.
 * @usage   To detect DMS, DMR Devices which are Already present in the network
 * @param   [IN]p_dmcpHandle  : DMCP  Handle
 * @return  E_CNC_DLNA_ERR_CODE_NONE if success, On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t  CNC_DMCP_Start(cncDmcpHandle_t p_dmcpHandle);


/**
 * @brief   Stop DMCP  instance.
 * @param   [IN]p_dmcpHandle  : DMCP  Handle
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t  CNC_DMCP_Stop(cncDmcpHandle_t p_dmcpHandle);


/**
 * @brief   Start searching DMS, DMR devices in network.
 *    This Api will be used for Refreshing. It will delete existing device List And again will search for DMS, DMR devices present in the network
 * @param   [IN]p_dmcpHandle  : DMCP_Handle
 *
 * @return  : --> E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_SearchDevice(cncDmcpHandle_t   p_dmcpHandle);


/**
 * @brief   Get/update available DMS Devices list in the network.
 * @param   [IN]p_dmcpHandle      : DMCP_Handle
 * @param   [OUT]**pp_deviceList  : DLNA Client Device Information List.
 * @param   [OUT]*p_numberOfDevices  : Number of Devices Found
 *
 * @see cncUpnpDeviceList_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_GetServerDeviceList(cncDmcpHandle_t        p_dmcpHandle,
                                              cncUpnpDeviceList_t    **pp_deviceList,
                                              CNC_SINT32             *p_numOfDevices);

/**
 * @brief   Get/update available Device list in the network.
 *    Device List contains All DMR devices
 * @param   p_dmcpHandle [IN] : DMCP_Handle
 * @param   **pp_dlnaClientDeviceList [OUT] : DLNA Client Device Information List.
 * @param   *p_numberOfDevices [OUT] : Number of Devices Found
 *
 * @see cncUpnpDeviceList_t
 *
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_GetRendererDeviceList(cncDmcpHandle_t        p_dmcpHandle,
                                                cncUpnpDeviceList_t    **pp_deviceList,
                                                CNC_SINT32             *p_numOfDevices);


/**
 * @brief   Get/update available content list for particular DMS device.
 *    This API can be used while retrieving content in AVI[audio, video, image] view.
 *          At the end User will have Separate ContentLists for audio, video, image.
 * @param   [IN]p_dmcpHandle : DMCP_Handle
 * @param   [IN]*p_deviceId  : DMS Device UUID
 * @param   [OUT]**pp_contentList  : Device's content details List
 *
 * @see cncDmcpServerAVIviewList_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_GetContentList(cncDmcpHandle_t            p_dmcpHandle,
                                         CNC_CHAR                   *p_deviceId,
                                         cncDmcpServerAVIviewList_t **pp_contentList);


/**
 * @brief   Get contentList in File System View.
 *    Usage : This API Will give Content in File system view which exactly replica of
 *           how content present in DMS Device.
 * @param   [IN]p_dmcpHandle    : DMCP_Handle
 * @param   [IN]viewRequest     : Type View
 * @param   [IN]*p_deviceId     : DMS Device UUID
 * @param   [OUT]pp_viewResult  : Content details List
 *
 * @see cncDmcpServerContentViewType_t
 * @see cncDmcpServerViewResult_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_GetContentListByView(cncDmcpHandle_t                  p_dmcpHandle,
                                               cncDmcpServerContentViewType_t   viewRequest,
                                               CNC_CHAR                         *p_deviceId,
                                               cncDmcpServerViewResult_t        **pp_viewResult);

/**
 * @brief   Get ContentList of particular Folder.
 *    Usage : This API can be used where user wants to know information and
 *          contents of a specific folder
 * @param   [IN]p_dmcpHandle   : DMCP_Handle
 * @param   [IN]*p_deviceId    : DMS Device UUID
 * @param   [IN]*p_folderId    : FolderID
 * @param   [OUT]**pp_dirItem  : directoryItem
 *
 * @see cncDmcpFileSysDirItem_t
 *
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_GetContentListFromFolder(cncDmcpHandle_t          p_dmcpHandle,
                                                   CNC_CHAR                 *p_deviceId,
                                                   CNC_CHAR                 *p_folderId,
                                                   cncDmcpFileSysDirItem_t  **pp_dirItem);


/**
 * @brief   Get ContentList of particular Folder in Async .
 *    This API will retrieve "n" number of items Requested by User means
 * User will have option to get n number of items From total items present in DMS Device.
 * Useful While retrieving contents from a folder which having large number of records.
 *
 * @param   [IN]p_dmcpHandle : DMCP _Handle
 * @param   [IN]*p_deviceId  : DMS Device UUID
 * @param   [IN]*p_folderId  : FolderID
 * @param   [IN]TimeOut      : Return after "timeOut" seconds and move remaining process into background.
 * @param   [IN]TotalItems   : Total Item present in Folder
 * @param   [OUT]pp_dirItem  : directoryItem
 *
 * @see cncDmcpFileSysDirItem_t
 *
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_GetContentListFromFolderAsync(cncDmcpHandle_t          p_dmcpHandle,
                                                        CNC_CHAR                 *p_deviceId,
                                                        CNC_CHAR                 *p_folderId,
                                                        CNC_SINT32               timeOut,
                                                        CNC_SINT32               totalItems,
                                                        cncDmcpFileSysDirItem_t  **pp_dirItem);


/**
 * @brief   Free the ContentList received from Async.
 *    After calling CNC_DMCP_GetContentListFromFolderAsync() API,  APP must call
 *          this API to free the resources Allocated by CNC_DMCP_GetContentListFromFolderAsync()
 *
 * @param   [IN]p_dmcpHandle  : DMCP_Handle
 * @param   [IN]*p_dirItem  : directoryItem
 *
 * @see cncDmcpFileSysDirItem_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_FreeContentListAsync(cncDmcpHandle_t          p_dmcpHandle,
                                               cncDmcpFileSysDirItem_t  *p_dirItem);


/**
 * @brief   Free the list Of FileSystemView.
 *    This API should call to free the resources which are allocated by
 *          CNC_DMCP_GetContentListByView() API
 *
 * @param   [IN]*p_viewResult  : viewResult variable
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_FreeViewResult(cncDmcpServerViewResult_t    *p_viewResult);


/**
 * @brief   Free the list Of Specific Folder.
 *    This API should call to free the resources which are allocated by
 *          CNC_DMCP_GetContentListFromFolder() API
 *
 * @param   [IN]*p_dirItem  : directoryItem
 *
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t  CNC_DMCP_FreeFolderItems(cncDmcpFileSysDirItem_t *p_dirItem);


/**
 * @brief    Free the DMS Device Content list For AVI view.
 *    This API should call to free the resources which are allocated by
 *           CNC_DMCP_GetContentList() API
 *
 * @param   [IN]*p_contentList  : Device's content details List
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_FreeContentList(cncDmcpServerAVIviewList_t    *p_contentList);


/**
 * @brief   Free the  DMS Device list.
 *          This API should call to free the resources which are allocated by
 *          CNC_DMCP_GetDMSList() API.
 *
 * @param   [IN]*p_deviceList  : Device details List
 *
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_FreeDeviceList(cncUpnpDeviceList_t  *p_deviceList);


/**
 * @brief   Destroy DMCP  instance.
 *   This API will release All resources allocated by internal Core APIs like Destroying
 * internal DMS/DMR device List, releasing Upnp Handle.
 *
 * @param   [IN]p_dmcpHandle  : DMCP_Handle to destroy
 *
 * @return  :E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t  CNC_DMCP_Destroy(cncDmcpHandle_t    p_dmcpHandle);


/**
 * @brief   Upnp Finalize
 * @param   : Void
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_DeInit(void);

/******************** CNC-DMCP Control Channel APIs ************************************/


/**
 * @brief DMCP Renderer Control Handle.
 */
typedef void* cncDmcpRenderCtrlHandle_t;


/**
 * @struct cncDmcpRenderCtrlInfo_t
 * @brief  Renderer Control Information
 * @see    cncDmcpFileSysContentItem_t
 */
typedef struct _cncDmcpRenderCtlInfo_t
{
    cncMediaCtlInfo_t       *p_mediaCtrlInfo;                           /**< Media Control Information, which can be populated this information for any stream playback */
    CNC_VOID                *p_metadata;                                /**< Should take metadata from  cncDmcpFileSysContentItem_t */
    //TODO if required*/

}cncDmcpRenderCtrlInfo_t;


/**
 * @struct cncDmcpCustomActionArgs_t
 * @brief  Contains Required Custom Action Argument details.
 */
typedef struct _cncDmcpCustomActionArgs_t
{
    CNC_CHAR     *p_argName;                                            /**< Argument Name */
    CNC_CHAR     *p_argValue;                                           /**< Argument Value */
}cncDmcpCustomActionArgs_t;


/**
 * @enum    cncDmcpCustomActionResult_t
 * @brief   To know Requested DMR's action response whether successful or falied
 */
typedef enum _cncDmcpCustomActionResult_t
{
    CNC_DMCP_CUSTOM_ACTION_INVOKE_SUCCESS,                              /**< Custom Action is successful */
    CNC_DMCP_CUSTOM_ACTION_INVOKE_FAILURE                               /**< Custom Action is failed */
}cncDmcpCustomActionResult_t;


/**
 * @struct cncDmcpCustomActionReq_t
 * @brief  Contains Required information to invoke Custom Action to DMR.
 */
typedef struct _cncDmcpCustomActionReq_t
{
    CNC_SINT32                  numOfArgs;                               /* Number of Input arguments for that custom Action */
    CNC_CHAR                    *p_serviceType;                          /* Service Type */
    CNC_CHAR                    *p_serviceUrl;                           /* Service Control URL */
    CNC_CHAR                    *p_actionName;                           /* Requested Action Name */
    cncDmcpCustomActionArgs_t   **pp_actionArgs;                         /* Input arguments of Requested Action contains argument name, value*/
}cncDmcpCustomActionReq_t;


/**
 * @struct cncDmcpCustomActionResp_t
 * @brief  Contains DMR Response for requested Custom Action by DMCP.
 */
typedef struct _cncDmcpCustomActionResp_t
{
    CNC_SINT32                  numOfArgs;                              /* Number of result arguments for that custom Action */
    cncDmcpCustomActionResult_t actionResponse;                         /* Success Or Failure. cncDmcpCustomActnResType_t enum is available in ctrlChannel Header file*/
    cncDmcpCustomActionArgs_t   **pp_actionRespArgs;                    /* Action response output arguments contains argument name, value */
}cncDmcpCustomActionResp_t;

/**
 * @brief   Will open Renderer control Channel between DMCP and DMR Devices.
 * @param   [IN]p_dmcpHandle                  : DMCP Handle.
 * @param   [IN]*p_dmrId                      : DMR Device UUID.
 * @param   [OUT]*pp_ctrlHandle               : Renderer Control Handle which will updated by this API.
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlOpen(cncDmcpHandle_t              p_dmcpHandle,
                                           CNC_CHAR                     *p_dmrId,
                                           cncDmcpRenderCtrlHandle_t    *pp_ctrlHandle);


/**
 * @brief   Will destoryes Renderer control channel between DMCP and DMR which is created by
 * CNC_DMCP_RendererCtrlOpen() API.
 *
 * @param   [IN]p_ctrlHandle    : Renderer Control Handle
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlDestroy(cncDmcpRenderCtrlHandle_t p_ctrlHandle);


/**
 * @brief   Starts playing requested content on DMR.
 * This asynchronous API will return soon after getting response from DMR, Application should ensure DMR Status
 * trough Event notifications in F_UpnpEventNotification_t callback
 *
 * @param   [IN]p_ctrlHandle    : Renderer Control Handle
 * @param   [IN]*p_rendererInfo : Renderer Control Info, Contains infromation of content which need to be played on DMR.
 *
 * @see     cncDmcpRenderCtrlInfo_t
 * @see     F_UpnpEventNotification_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlPlayStart(cncDmcpRenderCtrlHandle_t   p_ctrlHandle,
                                                cncDmcpRenderCtrlInfo_t     *p_rendererInfo);

/**
 * @brief   Stops the playing content on DMR.
 * This asynchronous API will return soon after getting response from DMR, Application should ensure DMR Status
 * trough Event notifications in F_UpnpEventNotification_t callback
 *
 * @param   [IN]p_ctrlHandle    : Renderer Control Handle
 *
 * @see     F_UpnpEventNotification_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlPlayStop(cncDmcpRenderCtrlHandle_t    p_ctrlHandle);


/**
 * @brief   Performs Trick Play actions like pause, seek etc on DMR.
 * This asynchronous API will return soon after getting response from DMR, Application should ensure DMR Status
 * trough Event notifications in F_UpnpEventNotification_t callback
 *
 * @param   [IN]p_ctrlHandle    : Renderer Control Handle
 * @param   [IN]*p_trickInfo    : Trick Info
 *
 * @see     cncTrickInfo_t
 * @see     F_UpnpEventNotification_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlTrickPlay(cncDmcpRenderCtrlHandle_t   p_ctrlHandle,
                                                cncTrickInfo_t              *p_trickInfo);

/**
 * @brief   Changes requested Display settings like Volume, Mute etc of DMR device according to cncRendererConfigFlag_t flag Value.
 * This asynchronous API will return soon after getting response from DMR, Application should ensure DMR Status
 * trough Event notifications in F_UpnpEventNotification_t callback.
 *
 * Application should set "cncRendererConfigFlag_t" with following flags according to its requierment.
 *
 * CNC_RCF_VOLUME_LEVEL : To set DMR device Volume. Requested volume should updated in "volumeLevel" parameter of cncRendererConfigInfo_t.
 *
 * CNC_RCF_MUTE_STATE   : To set DMR device Mute status. Requested Mute status should updated in "muteState" parameter of cncRendererConfigInfo_t.
 *
 * @param   [IN]p_ctrlHandle    : Renderer Control Handle
 * @param   [IN]*p_settings     : Renderer Config Info. Application Need to provide new values in this parameter
 *                                To change DMR settings.
 *
 * @see     cncRendererConfigInfo_t
 * @see     F_UpnpEventNotification_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlSetDisplaySettings(cncDmcpRenderCtrlHandle_t p_ctrlHandle,
                                                         cncRendererConfigInfo_t   *p_settings);

/**
 * @brief   gets DMR status according to cncMediaInfoFlag_t value.
 * Application should set "cncMediaInfoFlag_t" with below flags according to its requierment.
 *
 * CNC_MEDIACTL_PLAYBACK_STATUS : Updates DMR's playback status in cncPlaybackStatus_t.
 *
 * CNC_MEDIACTL_RENDERER_CONFIG : Updates DMR's rendering configurations in cncRendererConfigInfo_t.
 *
 * CNC_MEDIACTL_MEDIA_FORMAT    : Updates DMR's currently playing content format info in cncMediaFormatInfo_t.
 *
 * CNC_MEDIACTL_MEDIA_SOURCE    : Updates DMR's currently playing source info in cncMediaSourceInfo_t.
 *
 * @param   [IN]p_ctrlHandle        : Renderer Control Handle
 * @param   [IN/OUT]p_rendererInfo  : Contains DMR Status information
 *
 * @see     cncDmcpRenderCtrlInfo_t
 * @see     cncMediaInfoFlag_t
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlGetStatus(cncDmcpRenderCtrlHandle_t   p_ctrlHandle,
                                                cncDmcpRenderCtrlInfo_t     *p_rendererInfo);

/**
 * @brief   Used to invoke any specific [vender specific or mandatory] actions of DMS/DMR
 *
 * @param   [IN]p_ctrlHandle          : Renderer Control Handle
 * @param   [IN]*p_actionRequest      : Custom Action Request
 * @param   [IN]**pp_actionResponse   : Custom Action Response, Application need to call CNC_DMCP_RendererCtrlFreeCustomActionResponse() API to free the response.
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlInvokeCustomAction(cncDmcpRenderCtrlHandle_t    p_ctrlHandle,
                                                         cncDmcpCustomActionReq_t     *p_actionRequest,
                                                         cncDmcpCustomActionResp_t    **pp_actionResponse);
/**
 * @brief   Used to free Custom action response.
 *
 * @param   [IN]*p_actionResponse   : Custom Action Response which need to be freed.
 *
 * @return  : E_CNC_DLNA_ERR_CODE_NONE if success,
 *              On Error errCode of type cncDlnaErrCode_t
 */
cncDlnaErrCode_t CNC_DMCP_RendererCtrlFreeCustomActionResponse(cncDmcpCustomActionResp_t    *p_actionResponse);


#endif /* __CNC_DMCP_H_ */
