/*
 * # COPYRIGHT NOTICE
 * #
 * #***************************************************************************
 *
 * The following software code is the property of Cavium Networks.
 * Cavium Networks reserves all rights in this software, including but
 * not limited to copyright and shall be vested with all rights incidental
 * thereto.
 *
 * Rights granted to you by or under any license shall be limited to those
 * explicitly granted. You may not record, store, reproduce, reverse engineer,
 * decompile, inter-operate and/or transmit this software in any medium,
 * electronic or otherwise, without the express written consent of Cavium Networks.
 *
 * ALL RIGHTS RESERVED.
 *
 * Copyright 2013-2014 Cavium Networks.
 * #***************************************************************************
 * #
 * # END OF COPYRIGHT NOTICE
 * #
 * */

/**
 * @file cnc_dms.h
 * @brief CRISP DLNA - Digital Media Server Module header file.
 * \n Contains all required information about CNC_DMS Core APIs, Structure and Enum members to Develop cnc_dms Application
 * \n !!!! DISCLAIMER !!!!!
 * \n Interfaces specified below are subject to change in future versions since development of this module 
 * \n is still in progress.
 */
#ifndef _CNC_DMS_H_
#define _CNC_DMS_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE


/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_dlna.h"
#ifdef CNC_HEADLESS_GATEWAY_PROTOTYPE
#include "cnc_btv.h"
#endif

/* =========================== Macro definitions ========================== */



#define CNC_DMS_MAX_TITLE_LENTH                  (256)
#define CNC_DMS_MAX_FILE_PATH_LENTH              (256)

/* Maximum size of shared path. */
#define CNC_DMS_MAX_SHARED_PATH_LENTH            (1024)

/* DMS default device description values*/
#define CNC_DMS_DEFAULT_FRIENDLY_NAME            "CNC_DLNA_DMS"
#define CNC_DMS_DEFAULT_DEV_MANUFACTURER         "Cavium Inc"
#define CNC_DMS_DEFAULT_DEV_MANUFACTURER_URL     "www.cavium.com"
#define CNC_DMS_DEFAULT_DEV_MODEL_NAME           "Alpha"
#define CNC_DMS_DEFAULT_DEV_MODEL_DESC           "DMS"
#define CNC_DMS_DEFAULT_DEV_MODEL_URL            "www.cavium.com"
#define CNC_DMS_DEFAULT_DEV_MODEL_NUM            "v1.0"
#define CNC_DMS_DEFAULT_DEV_SERIAL_NUM           "123"
#define CNC_DMS_DEFAULT_X_HARDWARE_ID            "VEN_0B21&amp;DEV_0011&amp;REV_01"
#define CNC_DMS_DEFAULT_X_MODEL_ID               "0F2a8B63-C4d7-Ac43-Eb8347501af9"
#define CNC_DMS_DEFAULT_X_COMPATIBLE_ID          "MS_DigitalMediaDeviceClass_DMS_V001"

/* =========================== Type definitions =========================== */

 /*
  *  DMS Device Handle.
  */
typedef void* cncDmsHandle_t;

/**
 * @struct cncDmsSharedPathInfo_t
 * @brief  Contains DMS Content Shared Path Details.
 */
typedef struct _cncDmsSharedPathInfo_t
{
    CNC_CHAR               a_sharedPath[CNC_DMS_MAX_SHARED_PATH_LENTH];    /**< File system path to be shared */
#ifdef CNC_HEADLESS_GATEWAY_PROTOTYPE
    CNC_SINT32             numOfBtvChannels;
    cncBTVServiceInfo_t   *p_btvChannelList;
#endif
    CNC_SINT32             maxNumOfContents;                               /**< Maximum number of contents can be added in to DMS */
}cncDmsSharedPathInfo_t;

/* ===================== Global variable declarations ===================== */
// NONE


/* ======================== Function declarations ========================= */

/**
 * @brief   This API Initializes DMS and allocates global memory
 *           And should be called first by Application
 * @param   None
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Init(void);


/**
 * @brief   Creates an instance of DMS and gives a handle to it of further API calls.
 *          Need to Call this API after initialisation.
 * @param   **pp_dmsHandle [OUT] : Address where the DMS Instance handle should be updated
 * @param   *p_dlnaDmsConfig [IN]: DLNA/UPnP Configuration information
 * @param   *p_deviceConfig [IN] : Device Configuration information
 * @param   *p_sharedPath [IN]   : DMS Content Shared path
 * @see     cncDlnaConfig_t
 * @see     cncDlnaDeviceConfig_t
 * @see     cncDmsSharedPathInfo_t
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Create(cncDmsHandle_t          **pp_dmsHandle,
                                cncDlnaConfig_t         *p_dlnaDmsConfig,
                                cncDlnaDeviceConfig_t   *p_deviceConfig,
                                cncDmsSharedPathInfo_t  *p_sharedPath);


/**
 * @brief   Starts broad casting initial Alive notifications in to network.
 * @param   *p_dmsHandle [IN] : Device Handle
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Start(cncDmsHandle_t *p_dmsHandle);


/**
 * @brief   Stops the UPnP component
 * @param   *p_dmsHandle [IN] : Device Handle
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Stop(cncDmsHandle_t *p_dmsHandle);


/**
 * @brief   Stop and Starts DMS
 * @param   *p_dmsHandle [IN] : Device Handle
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Restart(cncDmsHandle_t *p_dmsHandle);

/**
 * @brief   Utitlity API used to modify the device friendly name
 * @param   *p_dmsHandle [IN]     : Device Handle
 * @param   *p_deviceFriendlyName [IN] : New name of device.
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_UpdateFriendlyName(cncDmsHandle_t *p_dmsHandle, CNC_CHAR *p_deviceFriendlyName);


/**
 * @brief   Destroys the DMS instance
 * @param   *p_dmsHandle [IN] : Device Handle which need to be destoryed
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Destroy(cncDmsHandle_t *p_dmsHandle);


/**
 * @brief   DeiInitializes DMS and used for freeing of global memory
 * @param   None
 * @return  E_CNC_DLNA_ERR_CODE_NONE on success.
 */
cncDlnaErrCode_t CNC_DMS_Deinit(void);


#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_DMS_H_ */

