/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @ingroup media
 * @file cnc_drm.h
 * Data types for digital rights management.
 */

#ifndef _CNC_COMMON_DRM_H_
#define _CNC_COMMON_DRM_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================ User Includes   =========================== */
#include "cnc_common_typedef.h"

/* ============================ Macro Definitions ========================= */
#define CNC_MAX_LAURL_LENGTH  (512)            /**< License URL MAX size  */ 
#define CNC_MAX_COOKIE_LENGTH (512)            /**< Cookie Header Value MAX size  */ 
#define DATAPOOLSIZE          (512)            /**< MAX Drm Custom data pool size */
#define MAXPOOLENTRIES        (10)             /**< Number of Drm Custom data entried in Custom Data pool */
#define CNC_DRM_MAX_DATE_SIZE (10)             /**< Date format size */

/* ============================ Type Definitions ========================== */
/**
 * @enum     cncAesCipherMode_t
 * @brief    AES Cipher Mode
 */
typedef enum _cncAesCipherMode_t
{
    CNC_AES_MODE_NONE = 0,          /**< None */
    CNC_AES_MODE_BLOCK,             /**< AES Input is BLOCK Cipher */
    CNC_AES_MODE_STREAM,            /**< AES Input is STREAM Cipher */
} cncAesCipherMode_t;

/**
 * @enum     cncAesMode_t 
 * @brief    Specifies AES Encrpytion/Decryption Mode
 */
typedef enum _cncAesMode_t
{
    CNC_DECRYPT_NONE       = 0,        /**< None */
    CNC_DECRYPT_AES128_ECB,            /**< AES Decryption Mode 128 bit ECB */
    CNC_DECRYPT_AES128_CBC,            /**< AES Decryption Mode 128 bit CBC */
    CNC_DECRYPT_AES128_CTR,            /**< AES Decryption Mode 128 bit CTR */
    CNC_ENCRYPT_AES128_ECB,            /**< AES Encrpytion Mode 128 bit ECB */
    CNC_ENCRYPT_AES128_CBC             /**< AES Encrpytion Mode 128 bit CBC */
} cncAesMode_t;

/**
 * @struct   cncAesInfo_t
 * @brief    Specifies AES Information
 */
typedef struct _cncAesInfo_t
{
    cncAesMode_t          mode;           /**< AES Mode CBC/CTR/ECB */
    cncAesCipherMode_t    cipherMode;     /**< AES Cipher Mode */
    CNC_UINT8             key[16];        /**< AES Key */
    CNC_UINT8             iv[16];         /**< AES IV */
} cncAesInfo_t;

/**
 * @enum     cncMarlinProtectionType_t
 * @brief    Marlin Broad Band Protection Type
 */
typedef enum _cncMarlinProtectionType_t
{
    CNC_MARLIN_PROTECTION_TYPE_NONE = 0,          /**< None */
    CNC_MARLIN_PROTECTION_TYPE_BBTS,              /**< Marlin Broadband Transport Stream */
    CNC_MARLIN_PROTECTION_TYPE_PIFF,              /**< Marlin protected interoperable file format */
    CNC_MARLIN_PROTECTION_TYPE_PDCF               /**< Marlin PDCF */
} cncMarlinProtectionType_t;

/**
 * @enum     cncMarlinIptvesContentType_t
 * @brief    Marlin Content Type
 */
typedef enum _cncMarlinIptvesContentTypeTag
{
    E_CNC_MARLIN_IPTVES_CONTENT_TYPE_NONE = 0,    /**< None */
    E_CNC_MARLIN_IPTVES_CONTENT_TYPE_TS,          /**< Content in TS format - 188 Bytes */
    E_CNC_MARLIN_IPTVES_CONTENT_TYPE_TTS          /**< Content in TTS format - 192 Bytes */
} cncMarlinIptvesContentType_t;

/**
 * @struct   cncMarlinInfo_t
 * @brief    Marlin Broad Band Information
 * @see      cncMarlinProtectionType_t
 */
typedef struct _cncMarlinInfo_t
{
    cncMarlinProtectionType_t     protectionType; /**< Marlin Broad Band protection type */
    CNC_SINT32                      noContents;     /**< No of contents */
    CNC_CHAR                      **contentID;    /**< Contains array of content ID's */
    CNC_CHAR                      **contentKey;   /**< Contains array of content keys corresponding to each content ID */
} cncMarlinInfo_t;

/**
 * @struct   cncMarlinIptvEsInfo_t
 * @brief    Marlin IPTV ES information
 * @see      cncMarlinIptvesContentType_t
 */
typedef struct _cncMarlinIptvEsInfo_t
{
    CNC_CHAR                        *p_ch_marlinServerURL;      /**< URL of the server to be connected to */
    CNC_CHAR                        *p_ch_crlDPURL;             /**< CRL distribution point URL */
    CNC_UINT32                       u32_serverUrlLength;       /**< Length of server URL */
    CNC_CHAR                        *p_ch_serverPkiPath;        /**< PkiPath of the server certificates in Hex */
    CNC_UINT32                       u32_serverPkiPathLength;   /**< Length of PkiPath data for server certificates */
    CNC_UINT8                       *p_u8_signature;            /**< 56 bytes signature of the server in Hex*/
    CNC_UINT32                       u32_sigLength;             /**< signature length */
    CNC_UINT8                       *p_u8_URR;                  /**< Array of UsageRuleReference(URR) */
    cncMarlinIptvesContentType_t     marlinIptvesContentType;   /**< Format of the content type */
} cncMarlinIptvEsInfo_t;


/**
 * @enum     cncDtcpAkeProtoType_t
 * @brief    Authentication Key Exchange Protocol Type
 */
typedef enum  _cncDtcpAkeProtoType_T
{
    CNC_DTCP_AKE_PROTO_TYPE_MAIN,           /**< Input drm DTCP - MAIN */
    CNC_DTCP_AKE_PROTO_TYPE_MOVE,           /**< Input drm DTCP - MOVE */
    CNC_DTCP_AKE_PROTO_TYPE_MAIN_RTT_AKE
}cncDtcpAkeProtoType_t;

/**
 * @struct   cncDtcpInfo_t
 * @brief    DTCP Information
 * @see      cncDtcpAkeProtoType_t
 */
typedef struct _cncDtcpInfo_t
{
    cncDtcpAkeProtoType_t     akeProtoType;           /**< Authentication Key Exchange Protocol Type */
    CNC_CHAR                  akeIP[32];              /**< Server IP address for Authentication Key Exchange */
    CNC_SINT32                akePort;                /**< Server Port for Authentication Key Exchange */
    CNC_UINT8                 kxmLabel;               /**< BLKMove.dctp.com: <kxmLabel> */
} cncDtcpInfo_t;

/**
 * @enum     cncDrmFeaturesFlag_t
 * @brief    DRM Features Flag  
 */
typedef enum _cncDrmFeaturesFlag_t
{
    CNC_DRM_FEATURE_NONE = 0,                           /**< None */

    /**< Generic */
    CNC_DRM_FEATURE_SECURE_CLOCK          = (1 << 0),   /**< Supports Secure Clock */
    CNC_DRM_FEATURE_VIEWRIGHTS            = (1 << 1),   /**< Supports View Rights */
    CNC_DRM_FEATURE_METERING              = (1 << 2),   /**< Supports Metering */
    CNC_DRM_FEATURE_DLA                   = (1 << 3),   /**< Supports License Acquisition */

    /**< PlayReady */
    CNC_DRM_FEATURE_ANTIROLLBACK_CLOCK    = (1 << 9),   /**< Supports Anti Rollback Clock */
    CNC_DRM_FEATURE_DOMAIN                = (1 << 10),  /**< Supports Domain */
    CNC_DRM_FEATURE_HDS                   = (1 << 11),  /**< Supports Data Store */
    CNC_DRM_FEATURE_LICENSESYNC           = (1 << 12),  /**< Supports License Sync */
    CNC_DRM_FEATURE_EST                   = (1 << 13),  /**< Supports Embedded License Store */
    CNC_DRM_FEATURE_DEVICEREVOCATION      = (1 << 14),  /**< Supports Device Revocation */
    CNC_DRM_FEATURE_APPREVOCATION         = (1 << 15),  /**< Supports Application Revocation */
    CNC_DRM_FEATURE_WMDRM                 = (1 << 16)   /**< Supports WMDRM */

    /**< Access */
    /**< Marlin */
} cncDrmFeaturesFlag_t;

/**
 * @struct   cncDrmFeatures_t
 * @brief    DRM Features Supported
 */
typedef struct _cncDrmFeatures_t
{
   cncDrmFeaturesFlag_t playready;     /**< Microsoft PlayReady Supported Features */
   cncDrmFeaturesFlag_t access;        /**< Adobe Access Supported Features */
} cncDrmFeatures_t;

/**
 * @enum     cncDrmType_t
 * @brief    Specifies DRM Type
 */
typedef enum _cncDrmType_t
{
    CNC_DRM_TYPE_NONE        = 0,                       /**< Drm none */
    CNC_DRM_TYPE_PLAYREADY,                             /**< input drm Microsoft PlayReady */
    CNC_DRM_TYPE_DTCP,                                  /**< input drm DTCP */
    CNC_DRM_TYPE_MARLIN,                                /**< input drm Marlin BroadBand */
    CNC_DRM_TYPE_MARLIN_IPTVES,                         /**< input drm Marlin IPTV */
    CNC_DRM_TYPE_AES                                    /**< input drm AES */
} cncDrmType_t;

/**
 * @union    cncDrmData_t
 * @brief    Specifies Input Drm Data (AES/DTCP/Marlin)
 * @see      cncAesInfo_t 
 * @see      cncDtcpInfo_t
 * @see      cncMarlinInfo_t
 * @see      cncMarlinIptvEsInfo_t
 */
typedef union _cncDrmData_t
{
    cncAesInfo_t              aesInfo;                     /**< AES info Key, IV, */
    cncDtcpInfo_t             dtcpInfo;                    /**< DTCP info */
    cncMarlinInfo_t           marlinInfo;                  /**< Marlin BroadBand info */
    cncMarlinIptvEsInfo_t     marlinIptvesInfo;            /**< Marlin IPTV info */
} cncDrmData_t;


/**
 * @enum     cncDrmLicenseState_t
 * @brief    Specifies License state information
 */
typedef enum _cncDrmLicenseState_t
{
    CNC_DRM_LICENSE_STATE_NONE = 0,                        /**< None */
    CNC_DRM_LICENSE_STATE_NORIGHT,                         /**< Playback Not Permitted */
    CNC_DRM_LICENSE_STATE_UNLIMITED,                       /**< License Unlimited */
    CNC_DRM_LICENSE_STATE_COUNT,                           /**< License valid for number of counts */
    CNC_DRM_LICENSE_STATE_FROM,                            /**< License valid from DD/MM/YYYY */
    CNC_DRM_LICENSE_STATE_UNTIL,                           /**< License valid until DD/MM/YYYY */
    CNC_DRM_LICENSE_STATE_FROM_UNTIL,                      /**< License valid from DD/MM/YYYY until DD/MM/YYYY */
    CNC_DRM_LICENSE_STATE_COUNT_FROM,                      /**< License valid from DD/MM/YYYY with number of counts */
    CNC_DRM_LICENSE_STATE_COUNT_UNTIL,                     /**< License valid until DD/MM/YYYY with number of counts */
    CNC_DRM_LICENSE_STATE_COUNT_FROM_UNTIL,                /**< License valid from DD/MM/YYYY until DD/MM/YYYY with number of counts */
    CNC_DRM_LICENSE_STATE_EXPIRE_AFTER_FIRSTUSE,           /**< License expires after first use */
} cncDrmLicenseState_t;

/**
 * @enum     cncDrmRightsInfo_t
 * @brief    Specifies DRM Rights information
 */
typedef struct _cncDrmRightsInfo_t
{
    cncDrmLicenseState_t    licState;
    CNC_SINT32              count;                               /**< Playback Valid count */
    CNC_UINT64              duration;                            /**< Valid Duration in secs */
    CNC_CHAR                startDate[CNC_DRM_MAX_DATE_SIZE];    /**< in DD/MM/YYYY */
    CNC_CHAR                endDate[CNC_DRM_MAX_DATE_SIZE];      /**< in DD/MM/YYYY */
} cncDrmRightsInfo_t;

/**
 * @enum     cncLicenseType
 * @brief    Specifies License renew information
 */
typedef enum _cncLicenseType
{
    CNC_DRM_LICENSE_NONE = 0,                                 /**< None */
    CNC_DRM_LICENSE_NOT_RENEWABLE,                            /**< Drm license cannot be renewed */ 
    CNC_DRM_LICENSE_RENEWABLE,                                /**< Drm license can be renewed */
} cncLicenseType;

/**
 * @enum     cncDrmInfoFlag_t
 * @brief    Specifies DRM Flag to fetch Specific drm info from cncdrm module, like, license state.. etc
 */
typedef enum _cncDrmInfoFlag_t
{
    CNC_DRM_INFO_NONE = 0,                                 /**< None */
    CNC_DRM_INFO_RIGHTS,                                   /**< Flag to fetch DRM rights */
} cncDrmInfoFlag_t;

/**
 * @struct   cncDrmInfo_t
 * @brief    Specifies DRM Information
 * @see      cncDrmInfoFlag_t
 * @see      cncDrmType_t
 * @see      cncDrmData_t 
 * @see      cncDrmRightsInfo_t
 * @see      cncLicenseType
 */
typedef struct _cncDrmInfo_t
{
    CNC_BOOL                  disableDrm;                          /**< If 'TRUE' DRM protected streams wont be played/recorded for given request*/
    cncDrmInfoFlag_t          flag;                                /**< Drm Status */
    cncDrmType_t              drmType;                             /**< Drm Type PLAYREADY/MARLIN/DTCP/AES */
    cncDrmData_t              drmData;                             /**< Drm input data for PLAYREADY/MARLIN/DTCP/AES */
    CNC_CHAR                  licAcqUrl[CNC_MAX_LAURL_LENGTH];     /**< License Acquisition URL, which will override URL passed during Init and Content License Acquisition URL */
    CNC_CHAR                  cookie[CNC_MAX_COOKIE_LENGTH];       /**< HTTP Header Cookie Value which need to be set during License Acquisition */
    CNC_UINT8                 customDataPool[DATAPOOLSIZE];        /**< Drm memory pool used for passing Custom Data */
    CNC_UINT8                *customDataEntry[MAXPOOLENTRIES];     /**< Custom Data Index Entry reference Pointers in customDataPool */
    CNC_UINT8                 customDataEntrySize[MAXPOOLENTRIES]; /**< Custom Data Entry size in bytes (along with '\0' if its a string) */
    cncDrmRightsInfo_t        rightsInfo;                          /**< Drm Rights info */
    cncLicenseType            renewLicenseType;                    /**< Drm license renewal type */
    CNC_BOOL                  avoidLicenseStore;                   /**< Drm Override License behaviour TRUE: force persistance license not to be stored, FALSE: store/not store as per value set by server */
    CNC_SINT32                renewLicenseDuration;                /**< Drm License renew duration in seconds, valid if license is CNC_DRM_LICENSE_RENEWABLE */
    CNC_SINT32                gracePeriod;                         /**< Grace Period Time in seconds used for Playback incase of license expiry */
} cncDrmInfo_t;

/* =========================== Function Prototypes ======================== */
// NONE

#ifdef __cplusplus
}
#endif

#endif /* END:  _CNC_COMMON_DRM_H_ */
