/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_fupgrade.h
 * @ingroup fw
 * The CNCFWUPGRADE library is used to upgrade firmware in any board supported by CRISP
 */


#ifndef __CNC_FWUPGRADE_H__
#define __CNC_FWUPGRADE_H__


#include "cnc_common_typedef.h"
//----------------------------------------------------------------------------------------------

#define CNC_FW_UPGRADE_LIB_VERSION ( "0.1.00.b" )

//----------------------------------------------------------------------------------------------

/** Enumeration with the possible values returned by api file 
 */
typedef enum _cncFwUpgradeReturn_t{
    CNC_FWU_FAILURE = -1,
    CNC_FWU_SUCCESS = 0
} cncFWUReturn_t;

/** Enumeration with all the possible error values which can be returned 
 */
typedef enum e_cncFwUpgradeErrorCode_t_
{
    CNC_FW_UPGRADE_OK      		= 0 	,
    CNC_FW_UPGRADE_ERROR   		= -1	,
    CNC_FW_UPGRADE_FILE_ERROR 	= -1000	,
    CNC_FW_UPGRADE_ERASE_ERROR 			,
    CNC_FW_UPGRADE_WRITE_ERROR 			,
    CNC_FW_UPGRADE_VERIFY_ERROR 		,
}e_cncFWUErrorCode;

/** Enumeration with all the states that can be returned by callback 
 */
typedef enum e_cncFwUpgradeStage_t_
{
	CNC_FW_UPGRADE_STAGE_ERASE=0		,
	CNC_FW_UPGRADE_STAGE_WRITING		,
	CNC_FW_UPGRADE_STAGE_VERIFYING		,
	CNC_FW_UPGRADE_PROGRAMMING			,
	CNC_FW_UPGRADE_DONE					,
}e_cncFWUStage;

/** Enumeration with the events used to register callbacks 
 */
typedef enum _cncFwUpgradeEvent_t {
    CNC_FW_UPGRADE_EVENT_ALL_EVENTS = -1,             ///< This can be used in the CNC_FW_UPGRADE_registerCallback to assign all events to the same callback
    CNC_FW_UPGRADE_EVENT_PROGRESS_CHANGE,
    CNC_FW_UPGRADE_EVENT_UPGRADE_COMPLETION,
    CNC_FW_UPGRADE_EVENT_NUMBER,
} cncFWUEvent_t;


//----------------------------------------------------------------------------------------------

/** Library handle
 */
typedef CNC_VOID* cncFWUHandle_t;

/** typedef to define callbacks used in main app
 */
typedef CNC_VOID (*cncFWUCallback_t) (cncFWUHandle_t, cncFWUEvent_t, void* data);

typedef struct cncFWUEx_t 
{
    CNC_CHAR                                   mFileName[512]                           ;//Device logical address
	CNC_CHAR                                   mDeviceName[24]                          ;//Device Name
}cncFwUpgrade_t;

//----------------------------------------------------------------------------------------------

/** FW upgrade create function. Memory is allocated here
 *
 * @param 
 * @return returns allocated handle
 */
cncFWUHandle_t CNC_FWU_Create(CNC_VOID);

/** FW upgrade destroy function. Memory is deallocated here
 *
 * @param handle allocated by CNC_FWU_Create. 
 * @return returns CNC_FWU_FAILURE or CNC_FWU_SUCCESS depending if deallocation was succesful or not
 */
e_cncFWUErrorCode CNC_FWU_Destroy(cncFWUHandle_t handle);

/** FW upgrade initialization function. Does initialization tasks and parses image here.
 *
 * @param handle: handle allocated by CNC_FWU_Create.
 * @param filename: file name to flash into the dongle
 * @param boundary: boundary data from the cgi-bin
 * @param verbose: set verbose mode on or off
 * @param NoUpgrade: used to write env variables and avoid to upgrade at the same time      
 * @return returns CNC_FWU_FAILURE or CNC_FWU_SUCCESS depending if subtasks fail or not
 */
e_cncFWUErrorCode CNC_FWU_Init( cncFWUHandle_t handle, const CNC_CHAR filename[512], const CNC_CHAR* boundary, CNC_SINT32 verbose, CNC_SINT32 NoUpgrade, CNC_SINT32 factoryreset);

/** FW upgrade open function. Checks for partition to flash the image
 *
 * @param handle allocated by CNC_FWU_Create. 
 * @return returns CNC_FWU_FAILURE or CNC_FWU_SUCCESS depending if subtasks fail or not
 */
e_cncFWUErrorCode CNC_FWU_Open( cncFWUHandle_t handle );

/** FW upgrade start function. Thread is launched here. This thread is in charge of read file, flash it and update callbacks
 *
 * @param handle allocated by CNC_FWU_Create. 
 * @return returns CNC_FWU_FAILURE or CNC_FWU_SUCCESS depending if thread has been launched or not
 */
e_cncFWUErrorCode CNC_FWU_Start( cncFWUHandle_t handle );

/** FW upgrade stop function. Everything is stopped here, thread, global variables initializated and pointers used reset.
 *
 * @param handle allocated by CNC_FWU_Create. 
 * @return returns CNC_FWU_SUCCESS
 */
e_cncFWUErrorCode CNC_FWU_Stop( cncFWUHandle_t handle );

/** FW upgrade register callback function. This function is used to register all callbacks, it has to be called from main app. 
 *
 * @param handle allocated by CNC_FWU_Create. 
 * @return returns CNC_FWU_SUCCESS
 */
e_cncFWUErrorCode CNC_FWU_registerCallback(cncFWUHandle_t fwu_hnd, cncFWUEvent_t fwu_event, cncFWUCallback_t func);
//----------------------------------------------------------------------------------------------
#endif //__FW_UPGRADE_H__
//----------------------------------------------------------------------------------------------

