/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Defines the APIs for media
 * @file cnc_media.h
 * @ingroup media
 * @note  This file should be used only for end applications of the media pipeline
 */

#ifndef _CNC_MEDIA_API_H_
#define _CNC_MEDIA_API_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_pvr_db.h"
#include "cnc_common_typedef.h"
#include "cnc_common_error.h"
#include "cnc_mediainfo.h"

/* ================================ Macros =================================== */
#define CNC_MEDIA_CTL_VOLUME_LEVEL_MIN             (0)              /**< Minimum volume level */
#define CNC_MEDIA_CTL_VOLUME_LEVEL_MAX             (255)            /**< Maximum volume level */
#define CNC_MEDIA_MAX_FILE_NAME                    (4096)           /**< For support url, need 4096 bytes. */
#define CNC_MEDIA_FILE_PATH_SIZE                   (256)            /**< To support storage path */
#define CNC_MEDIA_VERSION_SIZE                     (64)             /**< Media version size */
#define CNC_MEDIA_MIN_DISK_AVAIL_SPACE             (40*1024*1024)   /**< Min Disk Avail */
#define CNC_MEDIA_MAX_PRIVDATA_SIZE                (512)            /**< The maximum size of private data */

/* =============================== typedef Info ============================== */
/**
 * @enum          cncMediaCtlType_t
 * @brief         Specifies Media Control Instance Type which will be used during cnc media open
 */
typedef enum _cncMediaCtlType_t
{
    CNC_MEDIACTL_TYPE_GENERIC = 0,          /**< media control instance which could support AV Playback, Record AV, Record Image, Thumbnail, Tuning */
    CNC_MEDIACTL_TYPE_WATCH_AV,             /**< media control instance which support only AV playback */
    CNC_MEDIACTL_TYPE_WATCH_IMAGE,          /**< media control instance which support only Image */
    CNC_MEDIACTL_TYPE_RECORD_AV,            /**< media control instance which support only Record AV*/
    CNC_MEDIACTL_TYPE_RECORD_IMAGE,         /**< media control instance which support only downloading of image */
    CNC_MEDIACTL_TYPE_THUMBNAIL_CREATION,   /**< media control instance which support only thumbnail creation */
    CNC_MEDIACTL_TYPE_TUNING                /**< media control instance which support only tuning operation */
} cncMediaCtlType_t;

/**
 * @enum          cncMediaCtlEventType_t
 * @brief         Specifies Media Control Event Notification Type
 */
typedef enum _cncMediaCtlEventType_t
{
    CNC_MEDIACTL_EVENT_TYPE_NONE  = 0,               /**< None */
    CNC_MEDIACTL_EVENT_TYPE_ERROR_NOTIFICATION,      /**< Error Notification */     
    CNC_MEDIACTL_EVENT_TYPE_STATUS_NOTIFICATION,     /**< Status Notification */
    CNC_MEDIACTL_EVENT_TYPE_DATA_NOTIFICATION        /**< Data Notification */  
} cncMediaCtlEventType_t;

/**
 * @enum          cncMediaRecMode_t 
 * @brief         Specifies Recording mode
 */
typedef enum _cncMediaRecMode_t
{
    CNC_MEDIA_REC_MODE_NONE = 0,      /**< Non program based recording */
    CNC_MEDIA_REC_MODE_PROGRAM        /**< Program based recording. Each program will be stored as separate content */
} cncMediaRecMode_t;

/**
 * @struct        cncMediaCtlConfigParams_t
 * @brief         Specifies Media Control Configuration parameters
 */
typedef struct _cncMediaCtlConfigParams_t
{
    CNC_SINT32  maxMediaCtlInstances;              /**< Maximum number of Media Control Instances. <= 0: Infinite */
    CNC_SINT32  maxPlayerInstances;                /**< Maximum number of Player Instances. <= 0: Infinite */
    CNC_SINT32  maxRecordingInstances;             /**< Maximum number of Recording Instances. <= 0: Infinite */
    CNC_SINT32  maxFileSrcInstances;               /**< Maximum number of File Stream Instances. <= 0: Infinite */
    CNC_SINT32  maxHlsSrcInstances;                /**< Maximum number of HLS Stream Instances. <= 0: Infinite */
    CNC_SINT32  maxHttpSrcInstances;               /**< Maximum number of HTTP Stream Instances. <= 0: Infinite */
    CNC_SINT32  maxSmoothStreamingSrcInstances;    /**< Maximum number of SS Stream Instances. <= 0: Infinite */
    CNC_SINT32  maxPrimeTimeSrcInstances;          /**< Maximum number of PT Stream Instances. <= 0: Infinite */
    CNC_SINT32  maxPlayReadyDrmInstances;          /**< Maximum number of Play Ready DRM Instances. <= 0: Infinite */
    CNC_SINT32  maxMarlinDrmInstances;             /**< Maximum number of Marling DRM Instances. <= 0: Infinite */
    CNC_SINT32  maxDashSrcInstances;               /**< Maximum number of DASH Instances. <= 0: Infinite */
    CNC_CHAR    drmPlayReadyStorePath[CNC_MEDIA_FILE_PATH_SIZE];  /**< DRM Store Path */
    CNC_CHAR    licAcqUrl[CNC_MAX_LAURL_LENGTH];   /**< License Acquisition URL, which will override Content License Acquisition URL */
    CNC_SINT32  maxDrmLicenses;                    /**< Maximum number of Licenses stored in datastore per DRM, if zero no limit on licenses stored  in datastore */
    CNC_SINT64  minStorageMemoryInBytes;           /**< Minimum storage device space needed for recording in bytes */
    CNC_SINT8   minStorageMemoryInPercent;         /**< Minimum storage device space needed for recording in percentage of total memory */
} cncMediaCtlConfigParams_t;

/**
 * @struct        cncMediaVersionInfo_t
 * @brief         cncmedia Supported Versions
 */
typedef struct _cncMediaVersionInfo_t
{
    CNC_CHAR mediaPipeline[CNC_MEDIA_VERSION_SIZE];                     /**< cncmedia PipeLine Version */
    /**< Adaptive Streaming */    
    CNC_CHAR hls[CNC_MEDIA_VERSION_SIZE];                               /**< HTTP Live Streaming Standard Version */
    CNC_CHAR hss[CNC_MEDIA_VERSION_SIZE];                               /**< Microsoft HTTP Smooth Streaming Version */
    CNC_CHAR dash[CNC_MEDIA_VERSION_SIZE];                              /**< DASH Standard Version */
    CNC_CHAR primeTime[CNC_MEDIA_VERSION_SIZE];                         /**< Adobe PrimeTime Version */
    /**< DRM */    
    CNC_CHAR playReady[CNC_MEDIA_VERSION_SIZE];                         /**< Microsoft PlayReady DRM Version */  
    CNC_CHAR marlinIPTVES[CNC_MEDIA_VERSION_SIZE];                      /**< Marlin IPTV ES Version */
    CNC_CHAR marlinBroadband[CNC_MEDIA_VERSION_SIZE];                   /**< Marlin BroadBand Version */
    CNC_CHAR dtcp[CNC_MEDIA_VERSION_SIZE];                              /**< DTCP Standard Version */
    CNC_CHAR access[CNC_MEDIA_VERSION_SIZE];                            /**< Adobe Access Version */
    /**< FFMPEG */    
    CNC_CHAR ffmpeg[CNC_MEDIA_VERSION_SIZE];                            /**< FFMPEG Version */  
} cncMediaVersionInfo_t;

/**
 * @enum      cncMediaCtlPrivDataType_t
 * @brief     Specifies Media Control Event Data Type
 */
typedef enum _cncMediaCtlPrivDataType_t
{
    CNC_MEDIACTL_PRIV_DATA_TYPE_UNKNOWN    = 0,    /**< Media Control Event data type unknown */
    CNC_MEDIACTL_PRIV_DATA_TYPE_CAT5_ADVT          /**< Media Control Event data Ex: CAT5, ADVT */
} cncMediaCtlPrivDataType_t;

/**
 * @struct        cncMediaCtlPrivData_t
 * @brief         Contains Media Control Event Data Information
 * @see           cncMediaCtlPrivDataType_t
 */
typedef struct _cncMediaCtlPrivData_t
{
    cncMediaCtlPrivDataType_t   type;                                  /**< media control event private data type */
    CNC_UINT8                   dataBuf[CNC_MEDIA_MAX_PRIVDATA_SIZE];  /**< event data buffer */
    CNC_SINT32                  dataBufSize;                           /**< event data size */
} cncMediaCtlPrivData_t;

/**
 * @struct        cncMediaCtlEventInfo_t    
 * @brief         Contains Media Control Event Information
 * @see           cncMediaCtlEventType_t
 * @see           cncMediaCtlInfo_t
 * @see           cncMediaCtlPrivData_t
 * @see           E_cncErrCode_t
 */
typedef struct _cncMediaCtlEventInfo_t
{
    cncMediaCtlEventType_t   eventType;           /**< Media Control Event Type (Error/Status/Data) */
    union _cncMediaCtlEventData_t
    {
        cncMediaCtlInfo_t      mediaCtlInfo;     /**< Media Control Information, if eventType is STATUS */ 
        cncMediaCtlPrivData_t  privData;         /**< Data if eventType is DATA */
        E_cncErrCode_t         e_errCode;        /**< Error code if eventType is ERROR */
    } eventData;
} cncMediaCtlEventInfo_t;

/**
 * @brief         Function pointer for Event Notifications from Media Control
 * @param [in]    mediaCtlHandle
 *                   Media Control Handle (Instance)
 * @param [in]    eventInfo
 *                   Media Control Even Info
 */
typedef CNC_VOID (*cncMediaCtlEventCallback_t)(CNC_VOID *mediaCtlHandle, cncMediaCtlEventInfo_t *eventInfo);

/**
 * @brief         Function pointer definition for Subtitle rendering
 * @param [in]    subtitleData  Subtitles Instance
 * @param [in]    clear
 *                 1: clear the subtitle
 *                 0: render the subtitle
 */
typedef CNC_VOID (*cncSubtitleCbf_t)(void *subtitleData, CNC_BOOL clear);

/**
 * @enum          cncMediaTimeShiftType_t
 * @brief         Contains the information about recording types of window.
 */
typedef enum _cncMediaTimeShiftLimitType_t_
{
    CNC_MEDIA_TIME_SHIFT_LIMIT_TYPE_CONTENT_SIZE,
    CNC_MEDIA_TIME_SHIFT_LIMIT_TYPE_CONTENT_DURATION
} cncMediaTimeShiftLimitType_t;

/**
 * @struct        cncMediaTimeShiftInfo_t
 * @brief         Contains details about time-shift information, used for LIVE
 * @see           cncMediaTimeShiftLimitType_t  
 */
typedef struct _cncMediaTimeShiftInfo_t
 {
    CNC_CHAR                         folderPath[CNC_MEDIA_FILE_PATH_SIZE];         /**< Path where timeshift content needs to be stored */
    CNC_BOOL                         useEncryption;                                /**< If true Encrypt timeshift content */
    cncMediaTimeShiftLimitType_t     timeshiftLimitType;                           /**< TimeShift limit type duration or size based */
    CNC_SINT64                       timeShiftLimit;                               /**< Bytes or Seconds */
} cncMediaTimeShiftInfo_t;

/**
 * @struct        cncMediaCacheInfo_t
 * @brief         Contains details about Cache information, used for VOD
 */
typedef struct _cncMediaCacheInfo_t
 {
    CNC_CHAR                         folderPath[CNC_MEDIA_FILE_PATH_SIZE];      /**< Path where media caching needs to be stored */
    CNC_BOOL                         useEncryption;                             /**< If true Encrypt timeshift content */
    cncMediaTimeShiftLimitType_t     cacheLimitType;                            /**< TimeShift limit type duration or size based */
    CNC_SINT64                       cacheLimit;                                /**< Bytes or Seconds */
} cncMediaCacheInfo_t;

/**
 * @struct        cncMediaProgramInfo_t
 * @brief         Program Info provided by EPG
 */
typedef struct _cncMediaProgramInfo_t
{
    CNC_CHAR     *p_channelName;               /**< Channel Name */
    CNC_CHAR     *p_programName;               /**< Program Name */
    CNC_CHAR     *p_programSummary;            /**< Program description */
    CNC_CHAR     *p_genre;                     /**< Genre */ 
    CNC_CHAR      programStartTime[14];        /**< Program Start time in YYYYmmDDHHMMSS */
    CNC_CHAR      programDuration[6];          /**< Program duration in HHMMSS */
    CNC_SINT32    parentalRating;              /**< Parental rating for program */
} cncMediaProgramInfo_t;

/**
 * @struct        cncMediaProgramList_t
 * @brief         Contains Program List Information
 */
typedef struct _cncMediaProgramList_t
{
    cncMediaProgramInfo_t            programInfo;        /**< Information about a particular program */   
    struct _cncMediaProgramList_t   *p_next;             /**< Used for traversing to next program entry in the program list */
} cncMediaProgramList_t;

/**
 * @struct        cncMediaRecInfo_t
 * @brief         Contains information for recording a live channel
 * @see           cncMediaRecMode_t
 * @see           cncFileMgrThumbnailInfo_t
 * @see           cncMediaProgramList_t
 */
typedef struct _cncMediaRecInfo_t 
{
    cncMediaRecMode_t           recMode;                                /**< Mode of recording */
    CNC_CHAR                    folderPath[CNC_MEDIA_FILE_PATH_SIZE];   /**< Path where recording content needs to be stored */
    CNC_BOOL                    createThumbnail;                        /**< If true Thumbnail is created during recording stop */
    cncFileMgrThumbnailInfo_t   thumbnailInfo;                          /**< Valid only if createThumbnail is TRUE */
    CNC_BOOL                    useEncryption;                          /**< If true Encrypt recorded content */
    cncMediaProgramList_t      *p_programList;                          /**< Used for monitoring programs and updating content data base. Set 'NULL' if unknown */
} cncMediaRecInfo_t;

/**
 * @struct        cncMediaCtlParamInfo_t
 * @brief         Specifies configuaration parameters while creating new media instance
 * @see           cncAVDevice_t
 * @see           cncSubtitleCbf_t
 * @see           cncMediaCtlEventCallback_t
 * @see           cncMediaCtlType_t
 * @see           cncMediaCacheInfo_t
 * @see           cncMediaTimeShiftInfo_t
 **/
typedef struct _cncMediaCtlParamInfo_t
{
    cncAVDevice_t                    dev;                      /**< Hardware AV device ID */
    CNC_VOID                        *subTitleGraphicsContext;  /**< Subtitle graphic context, which specifies on which context, subtitle should use/render */
    CNC_VOID                        *imageGraphicsContext;     /**< Image graphic context, which specifies on which context, Image should use/render */ 
    cncSubtitleCbf_t                 subTitleCbf;              /**< Upper layer Subtitle Cbf, cncsubtitle will call this function, when there is subtitle */
    cncMediaCtlEventCallback_t       mediaCtlEventCbf;         /**< Notifies media Events to upper layer */
    cncMediaCtlType_t                mediaCtlType;             /**< cncmedia(Play/Record/Image) type */
    cncMediaCacheInfo_t              cacheInfo;                /**< Configure options used for caching. Valid only for VOD */
    cncMediaTimeShiftInfo_t          timeShiftInfo;            /**< Configure options for timeshift playback. Valid only for LIVE */
} cncMediaCtlParamInfo_t;

/* ======================== Function declarations ========================= */
/**
 * @brief         Initializes Media for both Playback Mode and Recording Mode
 * @param [in]    configParams Configuration input for Media Control
 * @see           cncMediaCtlConfigParams_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_Initialize(cncMediaCtlConfigParams_t  *configParams);

/**
 * @brief         DeiInitialize Media for both Playback and Recording Mode
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_DeInitialize(void);

/**
 * @brief         Get the Adapative Streaming/DRM/FFMPEG versions supported by cncmedia, 
 *                Need to Call after CNC_Media_Initialize
 * @param [in]    mediaVersionInfo  updated with all the versions supported by cncmedia 
 * @see           cncMediaVersionInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_GetVersion(cncMediaVersionInfo_t *mediaVersionInfo);

/**
 * @brief         Creates instance of Media Control which can be used for Playback/Recording/Tuning/Thumbnail
 * @param [out]   mediaCtlHandle Handle for the Media Control instance
 * @param         mediaCtlParamInfo Configuration Parameters for the Media Control Instance
 * @see           cncMediaCtlParamInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_Open(CNC_VOID **mediaCtlHandle, cncMediaCtlParamInfo_t *mediaCtlParamInfo);

/**
 * @brief         Close the Media Control Instance
 * @param [in]    mediaCtlHandle Media Control Handle
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_Close(CNC_VOID *mediaCtlHandle);

/**
 * @brief         Start Audio/Video/Image playback from given Source (Ex: File/HTTP/RTSP/HLS/SS etc)
 * @param [in]    mediaCtlHandle Media Control instance to be used for Playback
 * @param [in]    mediaCtlInfo   Media Info for starting Playback
 * @see           cncMediaCtlInfo_t
 * @return        E_CNC_ERR_NONE On Success 
 */
E_cncErrCode_t CNC_Media_PlayStart(CNC_VOID *mediaCtlHandle, cncMediaCtlInfo_t *mediaCtlInfo);

/**
 * @brief         Stop Playback of given Media Control instance
 * @param [in]    mediaCtlHandle Media Control Instance
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_PlayStop(CNC_VOID *mediaCtlHandle);

/**
 * @brief         Start Recording from given Source (Ex: File/HTTP/HLS/SS/RTSP etc)
 * @param [in]    mediaCtlHandle     Media Control instance to be used for Recording
 * @param [in]    mediaCtlInfo       Media Info for starting Recording
 * @param [in]    recInfo            Information on how/where to record
 * @see           cncMediaCtlInfo_t
 * @see           cncMediaRecInfo_t
 * @return        E_CNC_ERR_NONE On Success 
 */
E_cncErrCode_t CNC_Media_RecStart(CNC_VOID *mediaCtlHandle, cncMediaCtlInfo_t *mediaCtlInfo, cncMediaRecInfo_t *recInfo);

/**
 * @brief         Stop Recording of given Media Control instance
 * @param [in]    mediaCtlHandle  Media Control Instance
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_RecStop(CNC_VOID *mediaCtlHandle);

/**
 * @brief         Performs requested trickplay operation
 * @param [in]    mediaCtlHandle Media Control Instance 
 * @param [in]    trickInfo      Trickplay information which needs to be performed
 * @see           cncTrickInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_TrickPlay(CNC_VOID *mediaCtlHandle, cncTrickInfo_t *trickInfo);

/**
 * @brief         Get current status of Media Control(Playback/Recording/TimeShift) Instance
 * @param [in]    mediaCtlHandle Media Control Instance
 * @param [out]   mediaCtlInfo   Media Control Info where current status is to be populated
 * @see           cncMediaCtlInfo_t
 * @return        E_CNC_ERR_NONE On Success 
 */
E_cncErrCode_t CNC_Media_GetCurStatus(CNC_VOID *mediaCtlHandle, cncMediaCtlInfo_t *mediaCtlInfo);

/**
 * @brief         Performs requested renderer operation
 * @param [in]    mediaCtlHandle  Media Control Instance
 * @param [in]    rendererConfig  Renderer Config Info, which consists of Setting Volume Info, Video Scale Info etc
 * @see           cncRendererConfigInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_SetRendererInfo(CNC_VOID *mediaCtlHandle, cncRendererConfigInfo_t  *rendererConfig);

/**
 * @brief         Get the number of audio/video/subtitle tracks 
 * @param [in]    mediaCtlHandle    Media Control Instance
 * @param [in]    type              Track Type audio/video/subtitle
 * @param [out]   tracklist         array to hold all relevant Track List
 * @param [out]   numoftracks       Number of tracks present in stream
 * @see           cncTrackType_t
 * @see           cncTrackInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_GetTrackList(CNC_VOID *mediaCtlHandle, cncTrackType_t type, cncTrackInfo_t **tracklist, CNC_SINT32 *numoftracks);

/**
 * @brief         Set the audio/video/subtitle track to be played
 * @param [in]    mediaCtlHandle  Media Control Instance
 * @param [in]    type            Track Type audio/video/subtitle
 * @param [in]    track           track number to be played
 * @see           cncTrackType_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_SetTrack(CNC_VOID *mediaCtlHandle, cncTrackType_t type, CNC_SINT32 track);

/**
 * @brief         Performs to validate requested trickplay operation is allowed based on mediaplayer status
 * @param [in]    mediaCtlHandle   Media Control Instance
 * @param [in]    trickInfo
 *                    Trickplay information which needs to be cross check
 * @param [out]   isReqTrickPlaySupported
 *                    Returns TRUE if requested trickplay is supported by media player
 * @see           cncTrickInfo_t  
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_CheckTrickPlaySupport(CNC_VOID *mediaCtlHandle, cncTrickInfo_t *trickInfo, CNC_BOOL *isReqTrickPlaySupported);

/**
 * @brief         Performs preparation for next playback, in order to start delay of next stream
 * @param [in]    mediaCtlHandle Media Control Instance
 * @param [in]    mediaCtlInfo
 *                    Media Info for starting Playback
 * @see           cncMediaCtlInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_PrepareNext(CNC_VOID *mediaCtlHandle, cncMediaCtlInfo_t *mediaCtlInfo);

/**
 * @brief         Performs play next playback
 * @param [in]    mediaCtlHandle  Media Control Instance
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_PlayNext(CNC_VOID *mediaCtlHandle);

/**
 * @brief         Creates thumbnail for given video path
 * @param [in]    mediaCtlHandle  Media Control Instance 
 * @param [in]    path            path of video file
 * @param [in]    thumbnailInfo   Contains information required for generating thumbnail
 * @see           cncFileMgrThumbnailInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_CreateThumbnail(CNC_VOID *mediaCtlHandle, CNC_CHAR *path, cncFileMgrThumbnailInfo_t *thumbnailInfo);

/**
 * @brief         Enables TimeShift for Live Channels
 * @param [in]    mediaCtlHandle   Media Control Instance
 * @param [in]    timeShiftInfo    Contains information about path and size for performing timeshift on Live Channels
 * @see           cncMediaTimeShiftInfo_t
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_EnableTimeShift(CNC_VOID *mediaCtlHandle, cncMediaTimeShiftInfo_t *timeShiftInfo);

/**
 * @brief         Disables TimeShift for Live Channels
 * @param [in]    mediaCtlHandle
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_DisableTimeShift(CNC_VOID *mediaCtlHandle);


/**
 * @brief         Get DRM Supported Features
 * @param [in]    mediaCtlHandle
 * @return        E_CNC_ERR_NONE On Success
 */
E_cncErrCode_t CNC_Media_GetDrmFeatures(CNC_VOID *mediaCtlHandle, cncDrmType_t type, cncDrmFeatures_t *features);

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_MEDIA_API_H_ */
