/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_mediainfo.h
 * @ingroup common
 * Data structures for information and configuration of media pipeline control
 *
 * Defines the following:
 * @li Data structures for output (renderer) config info
 * @li Data structures for mediactl info which comprises of all the other data structures
 */

#ifndef _CNC_MEDIAINFO_H_
#define _CNC_MEDIAINFO_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"
#include "cnc_common_error.h"
#include "cnc_mediaformat.h"
#include "cnc_mediasource.h"
#include "cnc_subtitle.h"
#include "cnc_drm.h"
#include "cnc_av.h"

/* ================================ Macros =================================== */
#define  CNC_RENDERER_CONFIG_DEFAULT_BRIGHTNESS             (-1)       /**< Default brightness value. If -1, no configuration is done */
#define  CNC_RENDERER_CONFIG_DEFAULT_CONTRAST               (-1)       /**< Default contrast value. If -1, no configuration is done */
#define  CNC_RENDERER_CONFIG_DEFAULT_SHARPNESS              (-1)       /**< Default sharpness value. If -1, no configuration is done */
#define  CNC_RENDERER_CONFIG_DEFAULT_MUTE_STATE             (0)        /**< Mute is OFF by default */
#define  CNC_RENDERER_CONFIG_DEFAULT_VOLUME_LEVEL           (180)      /**< Default volume level. Actual range is 0-255 */

/* =============================== typedef Info ================================ */
/**
 * @enum    cncTrackType_t
 * @brief   Track Type Specifier Like, Audio/Video/Subtitle
 */
typedef enum _cncTrackType_t
{
    CNC_MEDIA_TRACK_NONE = 0,    /** None */
    CNC_MEDIA_TRACK_AUDIO,       /** Audio Track Type */
    CNC_MEDIA_TRACK_VIDEO,       /** Video Track Type */
    CNC_MEDIA_TRACK_SUBTITLE,    /** Subtitle Track Type */
} cncTrackType_t;

/**
 * @struct   cncTrackInfo_t
 * @brief    Contains Track Information, Like Track ID, Language, Track name
 */
typedef struct _cncTrackInfo_t
{
   CNC_UINT32                       id;                  /* Track ID */
   CNC_BOOL                         isCurrentlySelected; /* Currently Selected. */
   CNC_BOOL                         isClosedCaption;     /*< Valid for Subtitle Track. TRUE: Subtitle is closed caption (hard of hearding subtitle). FALSE: Normal subtitle. */
   CNC_CHAR                         lang[CNC_MEDIA_MAX_LANG_CODE_SIZE];         /* Language */
   CNC_CHAR                         name[16];                                   /* Track Name */
} cncTrackInfo_t;

/**
 * @enum     cncRendererPictureMode_t
 * @brief    Renderer picture mode, Lile Standard, Vivid, Cinema..etc
 * @see 
 */
typedef enum _cncRendererPictureMode_t
{
    CNC_RENDERER_PICTURE_MODE_NONE       = 0,            /**< Invalid Picture Mode */
    CNC_RENDERER_PICTURE_MODE_STANDARD,                  /**< Standard picture mode */
    CNC_RENDERER_PICTURE_MODE_VIVID,                     /**< Vivid picture mode */
    CNC_RENDERER_PICTURE_MODE_CINEMA,                    /**< Cinema picture mode */
    CNC_RENDERER_PICTURE_MODE_IMAGE,                     /**< Image playback picture mode */

    /* Note: Add other modes above this comment */
    CNC_RENDERER_PICTURE_MODE_DEFAULT = CNC_RENDERER_PICTURE_MODE_STANDARD
} cncRendererPictureMode_t;

/**
 * @enum     cncRendererAspectRatio_t
 * @brief    Renderer aspect ratio
 * @see 
 */
typedef enum _cncRendererAspectRatio_t
{
    CNC_RENDERER_ASPECT_RATIO_NONE = 0,         /**< Invalid Aspect Ratio */
    CNC_RENDERER_ASPECT_RATIO_4_3,              /**< 4:3 Aspect Ratio */
    CNC_RENDERER_ASPECT_RATIO_16_9,             /**< 16:9 Aspect Ration */

    /* Note: Add other aspect ratio above this comment */
    CNC_RENDERER_ASPECT_RATIO_DEFAULT = CNC_RENDERER_ASPECT_RATIO_16_9  /**< */
} cncRendererAspectRatio_t;


/**
 * @enum     cncRendererResolution_t
 * @brief    Renderer resolution
 */
typedef enum _cncRendererResolution_t
{
    CNC_RENDERER_RESOLUTION_NONE = 0,         /**< Default Resolution */
    CNC_RENDERER_RESOLUTION_480P,             /**< 640×480 Resolution - Progressive Scan */
    CNC_RENDERER_RESOLUTION_480I,             /**< 640×480 Resolution - Interlaced Scan */
    CNC_RENDERER_RESOLUTION_720P,             /**< 1280*720 Resolution - Progressive Scan */
    CNC_RENDERER_RESOLUTION_1080I,            /**< 1920*1080 Resolution - Interlaced Scan */

    /* Note: Add other resolution above this comment */
    CNC_RENDERER_RESOLUTION_DEFAULT = CNC_RENDERER_RESOLUTION_720P
} cncRendererResolution_t;

/**
 * @enum          cncMediaAVSyncStatus_t 
 * @brief         Specifies AV Sync Status.
 */
typedef enum _cncMediaAVSyncStatus_t
{
    CNC_MEDIA_AVSYNC_STATUS_NONE,
    CNC_MEDIA_AVSYNC_STATUS_PIC_READY,
    CNC_MEDIA_AVSYNC_STATUS_SYNC_SUCCEED,
    CNC_MEDIA_AVSYNC_STATUS_TIME_OUT,
} cncMediaAVSyncStatus_t;

/**
 * @enum   cncRendererConfigFlag_t
 * @brief  Renderer configuration flags
 */
typedef enum _cncRendererConfigFlag_t
{
    CNC_RCF_NONE                         = 0,                /**< Invalid Renderer Config Type */
    CNC_RCF_BRIGHTNESS                   = (1 << 0),         /**< Brightness Configuration */
    CNC_RCF_CONTRAST                     = (1 << 1),         /**< Contrast Configuration */
    CNC_RCF_SHARPNESS                    = (1 << 2),         /**< Sharpness Configuration */
    CNC_RCF_MUTE_STATE                   = (1 << 3),         /**< Audio Mute Configuration */
    CNC_RCF_VOLUME_LEVEL                 = (1 << 4),         /**< Volume Level Configuration */
    CNC_RCF_ASPECT_RATIO                 = (1 << 5),         /**< Aspect Ratio Configuration */
    CNC_RCF_PICTURE_MODE                 = (1 << 6),         /**< Picture Mode Configuration */
    CNC_RCF_RESOLUTION                   = (1 << 7),         /**< Resolution Configuration */
    CNC_RCF_V_POS                        = (1 << 8),         /**< Vertical Position Configuration */
    CNC_RCF_H_POS                        = (1 << 9),         /**< Horizantal Position Configuration */
    CNC_RCF_PHASE                        = (1 << 10),        /**< PHASE Configuration */
    CNC_RCF_CLOCK                        = (1 << 11),        /**< Clock Configuration */
    CNC_RCF_VIDEO_SCALE                  = (1 << 12),        /**< Video Scaling Configuration For PIP, Browser Video Window ..etc */
    CNC_RCF_OSD_SCALE                    = (1 << 13),        /**< OSD Configuration - For Browser Window, Graphics ..etc */
    CNC_RCF_AUDIO_ANALOG_OUTPUT          = (1 << 14),        /**< Audio Analog Output Configuration */
    CNC_RCF_CVBS_OUTPUT                  = (1 << 15),        /**< CVBS Video Output Configuration */
    CNC_RCF_COMPONENT_OUTPUT             = (1 << 16),        /**< Component Video Output Configuration */
    CNC_RCF_HDMI_AUDIO_OUTPUT            = (1 << 17),        /**< HDMI Audio Configuration */
    CNC_RCF_SUBTITLE_STATUS              = (1 << 18),        /**< Subtitle Status Configuration */
    CNC_RCF_SUBTITLE_FULL_SCREEN         = (1 << 19),        /**< Enable Subtitle Full Screen Configuration */
    CNC_RCF_SUBTITLE_SUB_WINDOW          = (1 << 20)         /**< Modify Subtitle Display Area Configuration */
} cncRendererConfigFlag_t;

/**
 * @struct cncSubtitleWindowResolution_t
 * @brief  Subtitle Window Resolution
 */
typedef struct _cncSubtitleWindowResolution_t
{
    CNC_SINT32 xPos;           /**< X Position */
    CNC_SINT32 yPos;           /**< Y Position */
    CNC_SINT32 width;          /**< Width */
    CNC_SINT32 height;         /**< Height */
} cncSubtitleWindowResolution_t;

/**
 * @enum   cncSubtitleEnableDisable_t
 * @brief  Enable/Disable Subtitle Display
 */
typedef enum _cncSubtitleEnableDisable_t
{
    CNC_DISABLE_SUBTITLE = 0,   /**< Disable Subtitle Display */
    CNC_ENABLE_SUBTITLE         /**< Enable Subtitle Display */
} cncSubtitleEnableDisable_t;

/**
 * @enum    cncOutputMacrovisionMode_t
 * @brief   Macrovision mode
 */
typedef enum _cncOutputMacrovisionMode_t
{
    CNC_OUTPUT_MACROVISION_LEVEL_0 = 0,          /**< (Macrovision mode 00) - Level 0 (Off) */
    CNC_OUTPUT_MACROVISION_LEVEL_1,              /**< (Macrovision mode 01) - Level 1 */
    CNC_OUTPUT_MACROVISION_LEVEL_2,              /**< (Macrovision mode 10) - Level 2 */
    CNC_OUTPUT_MACROVISION_LEVEL_3,              /**< (Macrovision mode 11) - Level 3 */
    /*! Attention! : The following values could be set only in case of file playback. Since in case of dtv watch
     * the video_agc and video_aps are the same and only in case of file playback they could differ
     *! Assumption : video_aps and video_ag can take only 0 or 1 or 2 or 3 as valid values and they both are same with
     *               exception being that video_agc may sometimes OFF even if video_aps is specified
     * This logic needs review.
     */
    CNC_OUTPUT_MACROVISION_LEVEL_4  = 0x4,       /**< video_aps - Level 1 video_agc - Level 0 */
    CNC_OUTPUT_MACROVISION_LEVEL_8  = 0x8,       /**< video_aps - Level 2 video_agc - Level 0 */
    CNC_OUTPUT_MACROVISION_LEVEL_12 = 0xC        /**< video_aps - Level 3 video_agc - Level 0 */
} cncOutputMacrovisionMode_t;

/**
 * @enum    cncOutputCgmsMode_t
 * @brief   Copy Generation Management System Mode
 */
typedef enum _cncOutputCgmsMode_t
{
    CNC_OUTPUT_CGMS_COPY_NONE = 0,
    CNC_OUTPUT_CGMS_COPY_FREELY,              /**< CopyFreely (CGMS mode 00) */
    CNC_OUTPUT_CGMS_COPY_NO_MORE,             /**< CopyNoMore (CGMS mode 01) */
    CNC_OUTPUT_CGMS_COPY_ONCE,                /**< CopyOnce   (CGMS mode 10) */
    CNC_OUTPUT_CGMS_COPY_NEVER                /**< CopyNever  (CGMS mode 11) */
} cncOutputCgmsMode_t;

/**
 * @enum    cncOutputSpdifProtectionMode_t
 * @brief   SPDIF protection mode
 */
typedef enum _cncOutputSpdifProtectionMode_t
{
    CNC_OUTPUT_SPDIF_COPY_FREELY = 0,         /**< CopyFreely (Cp bit = 1, L bit = 0) */
    CNC_OUTPUT_SPDIF_COPY_NO_MORE,            /**< CopyNoMore (Cp bit = 0, L bit = 1) */
    CNC_OUTPUT_SPDIF_COPY_ONCE,               /**< CopyOnce   (Cp bit = 0, L bit = 0) */
    CNC_OUTPUT_SPDIF_COPY_NEVER               /**< CopyNever  (Cp bit = 0, L bit = 1) */
} cncOutputSpdifProtectionMode_t;

/**
 * @enum    cncOutputHdcpMode_t
 * @brief   High-bandwidth Digital Content Protection mode
 */
typedef enum _cncOutputHdcpMode_t
{
    CNC_OUTPUT_HDCP_NONE  = 0,
    CNC_OUTPUT_HDCP_DISABLE,                  /**< HDCP is disable */
    CNC_OUTPUT_HDCP_ENABLE                    /**< HDCP is enable */
} cncOutputHdcpMode_t;

/**
 * @struct   cncCopyControlInfo_t
 * @brief    Copy control information.
 * @see      cncOutputMacrovisionMode_t
 * @see      cncOutputCgmsMode_t
 * @see      cncOutputSpdifProtectionMode_t
 * @see      cncOutputHdcpMode_t 
 */
typedef struct _cncCopyControlInfo_t
{
    cncOutputMacrovisionMode_t       macrovisionMode;         /**< Macro Vision Mode */
    cncOutputCgmsMode_t              cgmsMode;                /**< Copy Generation Management System Mode */
    cncOutputSpdifProtectionMode_t   spdifProtectionMode;     /**< SPDIF Protection Mode, Like Copy Once, Never, No more, freely..*/
    cncOutputHdcpMode_t              hdcpMode;                /**< HDCP Control Flag */
} cncCopyControlInfo_t;

/**
 * @struct cncScalingParamInfo_t
 * @brief  Scalling Parameters Like X Position, Y Position, Width and Height
 */
typedef struct _cncScalingParamInfo_t
{
    CNC_SINT32    posX;            /**< X Position for Scaling */
    CNC_SINT32    posY;            /**< Y Postion for Scaling */
    CNC_SINT32    width;           /**< Width */
    CNC_SINT32    height;          /**< Height */
}cncScalingParamInfo_t;

/**
 * @enum    cncSetAnalogAudioOutput_t 
 * @brief   Enable/Disable Analog Audio Output
 */
typedef enum _cncSetAnalogAudioOutputInfo_t
{
    CNC_ANALOG_AUDIO_OUTPUT_NONE = 0,  /**< Nop */
    CNC_ANALOG_AUDIO_OUTPUT_DISABLE,   /**< Disable Analog Audio Output */
    CNC_ANALOG_AUDIO_OUTPUT_ENABLE     /**< Enable Analog Audio Output */
} cncSetAnalogAudioOutput_t;

/**
 * @enum   cncSetSPDIFOutput_t
 * @brief  Enable/Disable SPDIF Audio Output
 */
typedef enum _cncSetSPDIFOutputInfo_t
{
    CNC_SPDIF_OUTPUT_NONE = 0,        /**< Nop */
    CNC_SPDIF_OUTPUT_DISABLE,         /**< Disable SPDIF Audio Output */
    CNC_SPDIF_OUTPUT_ENABLE           /**< Enable SPDIF Audio Output*/
} cncSetSPDIFOutput_t;

/**
 * @enum     cncSetCVBSOutput_t
 * @brief    Enable/Disable CVBS/Composite Video Output.
 */
typedef enum _cncSetCVBSOutputInfo_t
{
    CNC_CVBS_OUTPUT_NONE = 0,         /**< Nop */
    CNC_CVBS_OUTPUT_DISABLE,          /**< Disable CVBS Video Output */
    CNC_CVBS_OUTPUT_ENABLE            /**< Enable CVBS Video Output */
} cncSetCVBSOutput_t;

/**
 * @enum     cncSetComponentOutput_t
 * @brief    Enable/Disable Component Video Output
 */
typedef enum _cncSetComponentOutputInfo_t
{
    CNC_COMPONENT_OUTPUT_NONE = 0,    /**< Nop */
    CNC_COMPONENT_OUTPUT_DISABLE,     /**< Disable Component Video Output */
    CNC_COMPONENT_OUTPUT_ENABLE       /**< Enable Component Video Output */
} cncSetComponentOutput_t;

/**
 * @enum     cncSetHDMIAudioOutput_t
 * @brief    Enable/Disable HDMI Audio Output
 */
typedef enum _cncSetHDMIAudioOutputInfo_t
{
    CNC_HDMI_AUDIO_OUTPUT_NONE = 0,   /**< Nop */
    CNC_HDMI_AUDIO_OUTPUT_DISABLE,    /**< Disable HDMI Audio Output */
    CNC_HDMI_AUDIO_OUTPUT_ENABLE      /**< Enable HDMI Audio Output */
} cncSetHDMIAudioOutput_t;

/**
 * @enum   cncSetImageDisplayScalInfo_t
 * @brief  Image Display Mode
 */
typedef enum _cncSetImageDisplayScalInfo_t
{
    CNC_IMAGE_DISPLAY_SCALE_MODE_NONE, /**< Nop */
    CNC_IMAGE_DISPLAY_SCALE_MODE_FIT   /**< Fit Image rendering to active OSD window */
}cncSetImageDisplayScalInfo_t;

/**
 * @struct  cncRendererConfigInfo_t
 * @brief   Renderer configuration information, Like, Control Flag, BrightNess, Contrast, Volume Level, HDMI Audio enable/disable..etc
 * @see 
 */
typedef struct _cncRendererConfigInfo_t
{
    cncRendererConfigFlag_t              renderFlag;                 /**< Renderer Control Flag */
    CNC_SINT32                           brightness;                 /**< Brightness of the renderer */
    CNC_SINT32                           contrast;                   /**< Contrast of the renderer */
    CNC_SINT32                           sharpness;                  /**< Sharpness of the renderer */
    CNC_BOOL                             muteState;                  /**< Mute state of the renderer */
    CNC_SINT32                           volumeLevel;                /**< Volume level of the renderer */
    CNC_BOOL                             vidAlphaAppCtlFlag;         /**< App handle the vid alpha flag*/
    cncMediaAVSyncStatus_t               avSyncStatus;
    CNC_SINT32                           vidAlphaLevel;              /**< vid alpha will be setup this value if vidAlphaAppCtlFlag == 1 */
    cncRendererAspectRatio_t             aspectRatio;                /**< Aspect ratio of the renderer */
    cncRendererPictureMode_t             pictureMode;                /**< Picture mode of the renderer */
    cncRendererResolution_t              resolution;                 /**< Resolution of the renderer */
    CNC_SINT32                           vPosition;                  /**< Vertical position of the renderer */
    CNC_SINT32                           hPosition;                  /**< Horizontal position of the renderer */
    CNC_SINT32                           phase;                      /**< Phase of the renderer */
    CNC_SINT32                           clock;                      /**< Clock of the renderer */
    cncAVVideoWindowFitType_t            videoWindowFitType;         /**< */
    cncScalingParamInfo_t                scaleInfo;                  /**< Setting Video Scale */
    cncSetAnalogAudioOutput_t            setAnalogAudioOutput;       /**< Enable/Disable Analog Audio Output */
    cncSetSPDIFOutput_t                  setSPDIFOutput;             /**< Enable/Disable SPDIF Audio Output */
    cncSetCVBSOutput_t                   setCVBSOutput;              /**< Enable/Disable CVBS Output */
    cncSetComponentOutput_t              setComponentOutput;         /**< Enable/Disable Component Video YUV/RGB Video Output */
    cncSetHDMIAudioOutput_t              setHDMIAudioOutput;         /**< Enable/Disable HDMI Audio Output */
    cncSetImageDisplayScalInfo_t         setImageDisplayFitType;     /**< Set image scale type */
    cncSubtitleWindowResolution_t        subtitleWindowResolution;   /**< Set subtitle window resolution */
    cncSubtitleEnableDisable_t           subtitleStatus;             /**< Set subtitle status. Enable/Disable */
} cncRendererConfigInfo_t;


//TODO this structure must need to move for different file
/**
 * @struct cncNetIfChangeInfo_t
 * @brief  Network Interface Info Data structure
 * @see 
 */
typedef struct _cncNetIfChangeInfo_t
{
    CNC_CHAR netIFName[32];     /**< eth0 */
    CNC_CHAR ipAddress[16];     /**< 192.168.123.123 */
}cncNetIfChangeInfo_t;

/**
 * @enum    cncRecordingState_t
 * @brief   Defines status of Media Recording
 */
typedef enum _cncRecordingState_t
{
    CNC_RECORDING_STATUS_IDLE = 0,            /**< Recording is stopped or not started yet */
    CNC_RECORDING_STATUS_STARTING,            /**< Media Recorder accepted the start request and will start processing request. Or in simple terms LOADING */
    CNC_RECORDING_STATUS_RECORDING,           /**< Started Recording Process of given URI */
    CNC_RECORDING_STATUS_SUSPENDED,           /**< Recording suspended and buffers are flushed */
    CNC_RECORDING_STATUS_STOPPING             /**< STOP requested accepted. Starting to process STOP request. */
} cncRecordingState_t;

/**
 * @struct  cncRecordingStatus_t
 * @brief   Recording Status, Contains, Current Recording Duration, 
 * @see     cncRecordingState_t
 * @see     E_cncErrCode_t
 */
typedef struct _cncRecordingStatus_t
{
    cncRecordingState_t             status;                      /**< Recording Status */
    CNC_SINT32                      recordingDurationInMilliSec; /**< Recording Duration */
    E_cncErrCode_t                  errorStatus;                 /**< Error Status */
    CNC_BOOL                        isTimeShiftEnabled;          /**< Indicates whether timeshift is enabled in cncmedia for current playback for given instance */
} cncRecordingStatus_t;

/**
 * @enum    cncMediaInfoFlag_t
 * @brief   Media Control Flag to fetch Specific media info from cncmedia module, like, Media format info, media source info, media playback status, recording status..etc
 */
typedef enum _cncMediaInfoFlag_t
{
    CNC_MEDIACTL_NONE                   = 0,                /**< Nop */
    CNC_MEDIACTL_MEDIA_FORMAT           = (1 << 0),         /**< Flag to Fetch Media Format Only */
    CNC_MEDIACTL_MEDIA_SOURCE           = (1 << 1),         /**< Flag to Fetch Media Source Only */
    CNC_MEDIACTL_PLAYBACK_STATUS        = (1 << 2),         /**< Flag to Fetch Playback Status Only */
    CNC_MEDIACTL_RECORDING_STATUS       = (1 << 3),         /**< Flag to Fetch Recording Status Only */
    CNC_MEDIACTL_RENDERER_CONFIG        = (1 << 4),         /**< Flag to Fetch Renderer Config Only */
    CNC_MEDIACTL_RENDERER_INSTANCE      = (1 << 5)          /**< Reserved for instance */
} cncMediaInfoFlag_t;

/**
 * @struct cncMediaCtlInfo_t
 * @brief Media Control Information, which can be populated by or provided to upper/application. 
 *        User should populate this information for any stream playback, for Ex: for http playback, mediaSourceInfo should be populated
 * 
 * @see cncMediaInfoFlag_t
 * @see cncMediaFormatInfo_t
 * @see cncMediaSourceInfo_t
 * @see cncPlaybackStatus_t
 * @see cncRecordingStatus_t
 * @see cncRendererConfigInfo_t
 * @see cncDrmInfo_t
 * @see cncSubtitleInfoAndData_t
 */
typedef struct _cncMediaCtlInfo_t
{
    cncMediaInfoFlag_t              flag;                   /**< Flag to specify, which specific data need to fetch like Media Format Info, Playback Status Info..etc */
    cncMediaFormatInfo_t            mediaFormatInfo;        /**< Contains Media Format Information */
    cncMediaSourceInfo_t            mediaSourceInfo;        /**< Hold Media Source Information */
    cncPlaybackStatus_t             playbackStatus;         /**< Contains Playback Status Information */
    cncRecordingStatus_t            recordingStatus;        /**< Contains Recording Status Information */
    cncRendererConfigInfo_t         rendererConfig;         /**< Contains Renderer Config Information */
    cncDrmInfo_t                    drmInfo;                /**< Contains DRM Information */
    cncSubtitleInfoAndData_t        subtitleInfo;           /**< Contains Subtitle Information */
    CNC_VOID                       *p_v_decoderHandle;      /**< Decoder Resouce Handle */
} cncMediaCtlInfo_t;

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_MEDIAINFO_H_ */
