/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any license shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012-2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */



#ifndef _CNC_NET_H_
#define _CNC_NET_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stddef.h>

#include "cnc_nw_common.h"
#include "cnc_nw_device.h"
#include "cnc_nw_config.h"
#include "cnc_nw_cmds.h"
#include "cnc_nw_events.h"
#include "cnc_nw_policy.h"

/**
 * Instance of cnc_network to be used for controlling network device.
 */
typedef struct _cncNetInstance_t
{
    cncNetDeviceInfo_t devInfo;      /**< device information for this instance */

}cncNetInstance_t;

/**
 * \brief Definition of network event processing callback.
 *
 * This function should be implemented by caller and should do minimal processing.
 * Ideally it should copy the event information and retur as soon as possible.
 * Caller should do event processing in its own context.
 *
 * @param p_eventInfo : [IN] pointer to @cncNetEvent_t
 */
typedef cncNetError_t ( *pfn_CNC_Net_ProcessEvent )( CNC_VOID* p_context, cncNetEvent_t* p_event );

/**
 * \brief Create a new instance of CNC_NETWORK
 *
 * This instance can be used to control one network device.
 *
 * @param p_deviceInfo : [IN,OUT] Pointer to device information
 * @param pp_instance : [IN,OUT] Pointer to new initialized instance which supports device described in p_deviceInfo
 */
cncNetError_t CNC_Net_Create( cncNetDeviceInfo_t* p_deviceInfo, cncNetInstance_t **pp_instance );

/**
 * \brief Get default configuration
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t.
 * @param p_config : [IN/OUT] pointer to @cncNetConfig_t.
 *
 * @return : CNC_NET_ERROR_NONE on success.
 */
cncNetError_t CNC_Net_GetDefaultConfig( cncNetInstance_t* p_instance, cncNetConfig_t* p_config );

/**
 * \brief Get default policy
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t.
 * @param p_config : [IN/OUT] pointer to @cncNetPolicy_t.
 *
 * @return : CNC_NET_ERROR_NONE on success.
 */
cncNetError_t CNC_Net_GetDefaultPolicy( cncNetInstance_t* p_instance, cncNetPolicy_t* p_policy );

/**
 * \brief Register event listener for cnc_nw events. Multiple listeners can be registered.
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 * @param p_registerEvents : [IN] Pointer to @cncNetSubscribeEvents_t which provides bitmap of events to be registered.
 * @param pfn_eventCallback : [IN] pointer to callback function to be called when cnc_nw event occurs.
 * @param p_callbackParam : [IN/OUT] Callback parameter to be passed on by cnc_network.
 * @param p_id : [OUT] id returned should be used for unregistering the callback
 */
cncNetError_t CNC_Net_SubscribeEvents( cncNetInstance_t* p_instance, cncNetEventsSubscribe_t* p_events, pfn_CNC_Net_ProcessEvent pfn_eventCallback, CNC_VOID* p_callbackParam, CNC_SINT32* p_id );

/**
 * \brief Register event listener for cnc_nw events. Multiple listeners can be registered.
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 * @param id : [IN] id of the callback to be unregistered
 * @return : CNC_NET_RESULT_OK on success, CNC_NET_RESULT_FAIL on error
 */
cncNetError_t CNC_Net_UnSubscribeEvents( cncNetInstance_t* p_instance, CNC_SINT32 id );


/**
 * \brief Configure @cncNetInstance_t for desired modes.
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 * @param p_config : [IN] pointer to @cncNetConfig_t
 * @param p_policy : [IN] pointer to @cncNetPolicy_t
 */
cncNetError_t CNC_Net_Configure( cncNetInstance_t* p_instance, cncNetConfig_t* p_config, cncNetPolicy_t* p_policy );

/**
 * \brief Get current configuration
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 * @param p_params : [OUT] pointer to @cncNetConfig_t
 */
cncNetError_t CNC_Net_GetConfig( cncNetInstance_t* p_instance, cncNetConfig_t* p_config );

/**
 * \brief Get current policy
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 * @param p_params : [OUT] pointer to @cncNetPolicy_t
 */
cncNetError_t CNC_Net_GetPolicy( cncNetInstance_t* p_instance, cncNetPolicy_t* p_policy );

/**
 * \brief Start cnc_network after configuring desired modes.
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 */
cncNetError_t CNC_Net_Start( cncNetInstance_t* p_instance );

/**
 * \brief Execute commands on network instance
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 * @param p_cmd : [IN] pointer to cncNetCmd_t
 */
cncNetError_t CNC_Net_Cmd( cncNetInstance_t* p_instance, cncNetCmd_t* p_cmd );

/**
 * \brief Stop cnc_network
 *
 * @param p_instance : [IN] pointer to @cncNetInstance_t
 */
cncNetError_t CNC_Net_Stop( cncNetInstance_t* p_instance );

/**
 * \brief Destroy instance of CNC_NETWORK
 *
 */
cncNetError_t CNC_Net_Destroy( cncNetInstance_t* p_instance );

#ifdef __cplusplus
}
#endif

#endif /* END:  _CNC_NET_H_ */

