/*
# COPYRIGHT NOTICE
#
#***************************************************************************
The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but not limited
to copyright and shall be vested with all rights incidental thereto.
Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.
ALL RIGHTS RESERVED.
Copyright 2012-2013, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/

#ifndef __CNC_NET_PLUGIN_H__
#define __CNC_NET_PLUGIN_H__

#include "cnc_nw_exports.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum _cncNetPluginError_t
{
    CNC_NET_PLUGIN_ERROR_NONE,
    CNC_NET_PLUGIN_ERROR_NOT_FOUND,                      /**< Requested data was not found */
    CNC_NET_PLUGIN_ERROR_SYS_API_FAIL,                   /**< Library call failed */
    CNC_NET_PLUGIN_ERROR_SYS_CALL_FAIL,                  /**< System call failed */
    CNC_NET_PLUGIN_ERROR_ABORT,                          /**< Flow aborted */
    CNC_NET_PLUGIN_ERROR_FEATURE_NOT_SUPPORTED,          /**< Feature not supported */
    CNC_NET_PLUGIN_ERROR_INVALID_HANDLE,                 /**< Provided handle was invalid */
    CNC_NET_PLUGIN_ERROR_BAD_PARAMETER,                  /**< Invalid parameter */
    CNC_NET_PLUGIN_ERROR_TIMEOUT,                        /**< Call timedout */
    CNC_NET_PLUGIN_ERROR_NO_MEMORY,                      /**< Memory allocation failure */
} cncNetPluginError_t;

/**
 * \brief plugin version
 *
 */
typedef enum _cncNetPluginVersion_t
{
    CNC_NET_PLUGIN_VERSION_1_0 = 1//!< CNC_NET_PLUGIN_VERSION_1_0
}cncNetPluginVersion_t;

/**
 * \brief Plugin Information
 *
 */
typedef struct _cncNetWiFiPluginInfo
{
    cncNetDevice_t supportedDevices;                    /**< Bitmap of devices supported, defined in cnc_nw_device.h */

    CNC_UINT8 a_supportedDeviceWeights[32];             /**< Weight of support on a scale of 1 to 10, Higher value indicates more preferred support for chip */

    cncNetWifiSubType_t a_supportedModesBitmap[32];     /**< Bitmap of wifi modes supported, defined in cnc_nw_wifi.h */

    cncNetL2Proto_t supportedL2Modes;                   /**< Supported L2 modes defined in cnc_nw_common.h */

    cncNetInterfaceInfo_t staInfo;                      /**< information about station interface */

    cncNetInterfaceInfo_t apInfo;                       /**< information about ap interface */

    cncNetInterfaceInfo_t p2pInfo;                      /**< information about p2p interface */

    cncNetInterfaceInfo_t p2pAutoGoInfo;				/**< information about p2p auto GO interface */

} cncNetWiFiPluginInfo_t;

/**
 * \brief Plugin Information
 *
 */
typedef struct _cncNetEthPluginInfo
{
    cncNetDevice_t supportedDevices;                    /**< Bitmap of devices supported, defined in cnc_nw_device.h */

    cncNetInterfaceInfo_t ethInfo;                      /**< information about interface */

} cncNetEthPluginInfo_t;

/**
 * \brief Plugin will expose all relevant information through this structure
 *
 * Plugin loader mechanism will be able to make a decision to load the
 * plugin based on information in this structure.
 */
typedef struct _cncNetPluginInfo_t
{
    cncNetPluginVersion_t version;                  /**< plugin version */

    cncNetType_t type;                              /**< plugin type eth/wifi etc. */

    union
    {
        cncNetWiFiPluginInfo_t wifi;                /**< valid if type==wifi */

        cncNetEthPluginInfo_t eth;                  /**< valid if type==eth */

    }dev;

} cncNetPluginInfo_t;

/**
 * \brief Plugin will expose structure filled with function pointers cncNetWiFiPluginOps_t
 *
 * Caller should use these function pointers to control the plugin
 */
typedef struct _cncNetPluginInstance_t
{
    cncNetPluginInfo_t info;        /**< information about this plugin instance */

    CNC_VOID* p_callbackContext;    /**< context to be used by this instance when calling back to user
                                         this will be set by the user after creating instance */

    CNC_HANDLE p_ops;               /**< cncNetWiFiPluginOps_t* defined in cnc_nw_wifi_plugin.h */

} cncNetPluginInstance_t;

/**
 * \brief function pointers to functions defined below.
 *
 */
typedef cncNetPluginError_t ( *pfn_CNC_Net_Plugin_GetInfo)( cncNetPluginInfo_t* p_info );
typedef cncNetPluginError_t ( *pfn_CNC_Net_Plugin_Create)( cncNetPluginInstance_t **pp_instance );
typedef cncNetPluginError_t ( *pfn_CNC_Net_Plugin_Destroy)( cncNetPluginInstance_t* p_instance );


/**
 * \brief Get information from the plugin
 *
 * @param p_info : [OUT] Pointer to plugininfo which will befilled by plugin
 */
LIB_CNC_NET_EXPORT cncNetPluginError_t CNC_Net_Plugin_GetInfo( cncNetPluginInfo_t* p_info );

/**
 * \brief Create a new instance of plugin
 *
 * @param pp_instance : [IN,OUT] Pointer to newly allocated plugin instance
 */
LIB_CNC_NET_EXPORT cncNetPluginError_t CNC_Net_Plugin_Create( cncNetPluginInstance_t **pp_instance );

/**
 * \brief Destroy instance of plugin
 *
 * @param p_instance : [IN] Pointer to valid plugin instance
 */
LIB_CNC_NET_EXPORT cncNetPluginError_t CNC_Net_Plugin_Destroy( cncNetPluginInstance_t* p_instance );

#ifdef __cplusplus
}
#endif


#endif //__CNC_NW_WIFI_PLUGIN_H__
