/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2013-2014, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_upnp.h
 * @brief CRISP Universal Plug and Play header file.
 * \n Contains Structures which having required information about UPnP Devices
 */

#ifndef _CNC_UPNP_H_
#define _CNC_UPNP_H_

/* ======================== System Includes =============================== */
/* NONE*/
/*========================= User includes ================================= */
#include "cnc_common_typedef.h"

/*========================= (Internal) Macro definitions=================== */
#define CNC_UPNP_ICON_URL_SIZE                         (128)   /**< Icon URL size */
#define CNC_UPNP_MIME_TYPE_SIZE                        (32)    /**< MimeType size */
#define CNC_UPNP_MAX_NAME_SIZE                         (256)   /**< Maximum Name String Size */
#define CNC_UPNP_USER_AGENT_LENGTH                     (64)    /**<Maximum XML Doc User Agent Length*/

/* ================= (Internal) Type definitions =========================== */

/**
 * @enum  cncUpnpDeviceType_t
 * @brief Used to classify the UPnP Device type.
 */
typedef enum _cncUpnpDeviceType_t
{
    CNC_UPNP_DEV_NONE          = 0,
    CNC_UPNP_DEV_MEDIA_SERVER,
    CNC_UPNP_DEV_MEDIA_RENDERER,
    /* New device profiles can be added here*/
    CNC_UPNP_DEV_UNKNOWN
}cncUpnpDeviceType_t;


/**
 * @struct  cncUpnpDevIconList_t
 * @brief   Contains UPnP Devices [either Media Server or Media Renderer] IconList information
 */
typedef struct _cncUpnpDevIconList_t
{
    CNC_SINT16                      height;                                 /**<  Height of Icon */
    CNC_SINT16                      width;                                  /**<  Width of Icon */
    CNC_SINT16                      depth;                                  /**<  Depth of Icon*/
    CNC_CHAR                        a_mimeType[CNC_UPNP_MIME_TYPE_SIZE];    /**<  MimeType of Icon */
    CNC_CHAR                        a_iconURL[CNC_UPNP_ICON_URL_SIZE];      /**<  Icon URL */
    struct _cncUpnpDevIconList_t    *p_next;                                /**<  Points next icon with different resolution (or) mimeType of same UPnP device*/

}cncUpnpDevIconList_t;


/**
 * @struct  cncUpnpDeviceList_t
 * @brief   Contains List of required UPnP devices present in the Network.
 * @see also cncUpnpDeviceType_t
 * @see also cncUpnpDevIconList_t
 * */
typedef struct _cncUpnpDeviceList_t
{
    CNC_CHAR                    a_name[CNC_UPNP_MAX_NAME_SIZE];              /**<  UPnP Device Friendly Name */
    CNC_CHAR                    a_udn[CNC_UPNP_MAX_NAME_SIZE];               /**<  UUID of UPnP device */
    CNC_CHAR                    a_baseUrl[CNC_UPNP_MAX_NAME_SIZE];           /**<  Base URL of UPnP device*/
    cncUpnpDeviceType_t         deviceType;                                  /**<  Device Type */
    cncUpnpDevIconList_t        *p_iconList;                                 /**<  Contains all Icon Types of Same UPnP Device. */
    struct _cncUpnpDeviceList_t *p_nextDevice;                               /**<  Next UPnP Device in the List */

}cncUpnpDeviceList_t;


/**
 * @struct cncUpnpConfigParam_t
 * @brief  Contains All configurable UPnP parameters
 */
typedef struct _cncUpnpConfigParam_t
{
    CNC_SINT32      networkTimeOut;                                          /**< Connect timeout for getting Description XML of UPnP Device*/
    CNC_SINT32      searchTimeOut;                                           /**< Max Time out value for SEARCH action.*/
    CNC_SINT32      maxActionRespSize;                                       /**< Maximum size to store UPnP Browse Action response.*/
    CNC_SINT32      upnpPort;                                                /**< UPnP Port*/
    CNC_SINT32      connectTimeOut;                                          /**< Connection timeout */
    CNC_CHAR        a_UsrAgentHeader[CNC_UPNP_USER_AGENT_LENGTH];            /**< User agent while sending HTTP request.*/
    CNC_CHAR        a_networkIP[CNC_UPNP_MAX_NAME_SIZE];                     /**< Contains Network IP address*/
    CNC_CHAR        a_networkIfName[CNC_UPNP_MAX_NAME_SIZE];                 /**< Contains Network Interface Name */
}cncUpnpConfigParam_t;


/**
 * @enum   cncUpnpEventType_t
 * @brief  Enumeration which Contains Basic Event details on UPnP device.
 */
typedef enum _cncUpnpEventType_t
{
    CNC_UPNP_NEW_DEVICE_FOUND        = 101,                         /**<101 :    New UPnPdevice entered in the network by sending alive packets*/
    CNC_UPNP_DEVICE_SHUT_DOWN,                                      /**<102 :    Already existing UPnP device left from Network by sending BYE_BYE packets*/
    CNC_UPNP_DEVICE_SEARCH_COMPLETED,                               /**<103 :    UPnP device will send Discovery Response to DMP */
    CNC_UPNP_SUBSCRIBED_EVENT_RECEIVED,                             /**<104 :    Subscription event received*/
    CNC_UPNP_DEVICE_CONTENT_MODIFIED,                               /**<105 :    Content modified in UPnP device which is subscribed*/
    CNC_UPNP_FOLDER_ITEM_RETRIEVE_COMPLETED_SUCCESSFULLY,           /**<106 :    Requested number of contents retrieved completely in Async process*/
    CNC_UPNP_FOLDER_ITEM_RETRIEVE_COMPLETED_WITH_ERR,               /**<107 :    Error occurs while retrieving contents from folder in Async process*/
    CNC_UPNP_FOLDER_ITEM_RETRIEVE_COMPLETED_PARTIALLY,              /**<108 :    Requested number of contents retrieved partially in Async process*/
    CNC_UPNP_RENDERER_DISPLAY_NOTIFICATION,                         /**<109 :    Event over DMR Renderer Configuration Changes*/
    CNC_UPNP_RENDERER_PLAYBACK_NOTIFICATION,                        /**<110 :    Event over DMR Playback Changes*/

} cncUpnpEventType_t;

/**
 * @struct cncUpnpNotification_t
 * @brief  This Structure will Contains Information about UPnP Device Event Notification.
 */
typedef struct _cncUpnpNotification_t
{
    cncUpnpEventType_t  eventType;                                  /**< UPnP Event type received to event handler */
    cncUpnpDeviceType_t deviceType;                                 /**< Device Type */
    CNC_CHAR            *p_deviceId;                                /**< Device UUID */
    CNC_VOID            *p_deviceEvent;                            /**< Device Specific Subscription Event */

}cncUpnpNotification_t;


/**
 * \brief Event notification Callback Function which trigger backs to application if any event will occurs.
 */
typedef void (*F_UpnpEventNotification_t)(cncUpnpNotification_t eventNotification);


#endif /* CNC_UPNP_H_ */
