/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_ldenc.h
 * Low Delay AV processing for wireless display applications.
 * @ingroup wd
 */

#ifndef CNC_LDENC_INTERFACE_H
#define CNC_LDENC_INTERFACE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <pthread.h>
#include "csapi.h"
#include "cnc_common_typedef.h"
#include "cnc_system.h"

#include "cnc_network/cnc_nw.h"
#include "cnc_network/cnc_nw_device.h"
#include "cnc_network/cnc_nw_policy.h"

#define MAX_MBS_PER_SEC 0xFFFFFFFF

typedef enum _cncLDENCReturnValue_t {
    CNC_LDENC_FAILURE = -1,
    CNC_LDENC_SUCCESS = 0
} cncLDENCReturnValue_t;

/** This enumeration has all the possible events that could be triggered
 * from the ldenc library.
 */
typedef enum _cncLDENCEvent_t {
    CNC_LDENC_EVENT_ALL_EVENTS = -1,                      ///< This can be used in the CNC_LDENC_registerCallback to assign all events to the same callback
    CNC_LDENC_EVENT_RESOLUTION_DETECTED,
    CNC_LDENC_EVENT_HPD,
    CNC_LDENC_EVENT_INFOFRAME_CHANGE,
    CNC_LDENC_EVENT_TV_PLUGGED_IN_TO_HDMI_SPLITTER,
    CNC_LDENC_EVENT_TV_UNPLUGGED_FROM_HDMI_SPLITTER,
    CNC_LDENC_EVENT_HDMI_IN_CABLE_EVENT,
    CNC_LDENC_EVENT_NUMBER
} cncLDENCEvent_t;

/** Enumeration with the different possible audio types supported.
 * Some of this are no longer used and are there for consitency with old wivu code.
 */ 
typedef enum _cncLDENCAudioTypes_t {
     CNC_LDENC_AUD_TYPE_NETHD=0,                          ///< Audio in RTP mode for old NETHD sources
     CNC_LDENC_AUD_TYPE_RTP_16BIT_LPCM = 2,               ///< RTP 16 bit LPCM audio format
     CNC_LDENC_AUD_TYPE_TS_RFC2250_NETHD = 3,             ///< RTP+MPEGTS+for old NETHD sources
     CNC_LDENC_AUD_TYPE_TS_RFC2250_16BIT_LPCM = 4,        ///< RTP+MPEGTS+16 bit LPCM audio format
     CNC_LDENC_AUD_TYPE_AAC = 6,                          ///< RTP+MPEGTS+AAC 
} cncLDENCAudioTypes_t;

/** Enumeration with the different possible video types supported.
 */ 
typedef enum _cncLDENCVideoTypes_t {
    CNC_LDENC_VID_TYPE_3984 = 0,                          ///< Video in RTP format, according to 3984
    CNC_LDENC_VID_TYPE_WFD = 2,                           ///< Video in WFD format (RTP+MPEG2TS RFC2250)
} cncLDENCVideoTypes_t;

/** This enumeration describe the different actions regarding saving of incoming streams 
 */
typedef enum _cncLDENCSaveStream_t {
    CNC_LDENC_SAVE_STREAM_STOP = 0,                       ///< Stop stream saving prcess
    CNC_LDENC_SAVE_STREAM_ES = 1,                         ///< Save elementary streams
    CNC_LDENC_SAVE_STREAM_TS = 2,                         ///< Save transport streams
} cncLDENCSaveStream_t;

/** This enumeration describe the different actions regarding saving of incoming streams 
 */
typedef enum _cncLDENCRateCtrlMode_t {
    CNC_LDENC_RATE_CONTROL_CONSTANT_QP = 0,               ///< constant Qp
    CNC_LDENC_RATE_CONTROL_CBR = 1,                       ///< CBR
    CNC_LDENC_RATE_CONTROL_ABR = 2,                       ///< ABR
} cncLDENCRateCtrlMode_t;

/** This enumeration describe the different slice modes
*/
typedef enum _cncLDENCSliceMode_t {
    CNC_LDENC_SLICE_MODE_FULLFRAME = 0,                   ///< Encode at single slice per frame
    CNC_LDENC_SLICE_MODE_MULTISLICE_IN_MB = 1,            ///< Encode at multi slice per frame. Slice partition will be per MB division
    CNC_LDENC_SLICE_MODE_MULTISLICE_IN_BITS = 2,          ///< Encode at multi slice per frame. Slice partition will be per bytes division
} cncLDENCSliceMode_t;

/** This enumeration describe the different partial frame modes available
 */
typedef enum _cncLDENCPartialFrameMode_t {
    CNC_LDENC_PFM_DISABLED = 0,                           ///< Partial Frame Mode Disabled
    CNC_LDENC_PFM_EARLY_ENC = 1,                          ///< Early Encoding Partial Frame Mode
    CNC_LDENC_PFM_EARLY_ENC_AND_PROC = 2,                 ///< Early Encoding and Processing Partial Frame Mode
} cncLDENCPartialFrameMode_t;

/** This enumeration describes the different builtin splitter options available
 */
typedef enum _cncLDENCBuiltinSplitter_t {
    CNC_LDENC_BUILTIN_SPLITTER_DISABLED = 0,              ///< Builtin splitter Disabled
    CNC_LDENC_BUILTIN_SPLITTER_BYPASS = 1,                ///< Builtin splitter uses a/v bypass path (out=in)
    CNC_LDENC_BUILTIN_SPLITTER_CAPTURE = 2,               ///< Builtin splitter uses a/v capture path (scale, crop, window, OSD possible)
} cncLDENCBuiltinSplitter_t;

/** Structure with the parameters used to start a network sink for A/V encoding
 * This is passed to CNC_LDENC_Start() when a sink is discovered
 *
 */
typedef struct _cncLDENCNetSinkConfig_t {
    CNC_SINT32                  sink;                       ///< sink ID to be configured
    CNC_SINT32                  port;                       ///< Destination port
    CNC_CHAR                    IP[16];                     ///< Destination IP address
    unsigned int                maxMBsPerSec;               ///< Maximum MBs per second encoder must process (Default 0xFFFFFFFF, no limit)
} cncLDENCNetSinkConfig_t;
       
/** Structure with the parameters that can be configured in the ldenc module.
 * Calling a CNC_LDENC_GetConfig will fill up the structure with the current parameters
 * After init, the current parameters are those chosen as defaults
 */ 
typedef struct _cncLDENCConfig_t {
    CNC_CHAR                    dstIpAddress[16];           ///< Destination IP address
    CNC_SINT32                  dstPort;                    ///< Destination port
    CNC_SINT32                  sink;                       ///< sink ID to be configured
       
    unsigned int                bitrate_kbps;               ///< video encoder bitrate when rate control is enabled
    unsigned int                enc_framerate;              ///< video encoding frame rate, it could be a divisor
    unsigned int                enc_framerate_eff;          ///< effective video encoding frame rate just in case divisor is applied
    unsigned int                enc_framerate_corr;         ///< Frame Rate correction
    unsigned int                maxMBsPerSec;               ///< Maximum MBs per second encoder must process (Default 0xFFFFFFFF, no limit)
    unsigned int                max_bitrate_kbps;           ///< maximum video encoder bitrate when sqos is enabled
    unsigned int                qp;                         ///< video qp when rate control is disabled
    unsigned int                gop_size;                   ///< gop size
    unsigned int                num_proc_blocks;            ///< Maximum number of blocks a frame is divided to be processed: encryption and sending (for partialFrameMode = 2)

    cncLDENCRateCtrlMode_t      rate_control_mode;          ///< rate control mode
    cncLDENCPartialFrameMode_t  partialFrameMode;           ///< Partial Frame Mode: 0 Disabled, 1: Early Encoding, 2: Early Processing
    cncLDENCSliceMode_t         sliceMode;                  ///< Slice mode: Full frame or multi slice
    int                         slicePartition;             ///< Multi slice partition in bits or MB depending on sliceMode
    CSAUD_SAMPLE_RATE           aud_sample_rate;            ///< audio samplig rate
    CSAUD_STREAM_TYPE           aud_codec_type;             ///< audio codec type: LPCM or AAC
    
    unsigned short              mp2ts_video_pid;            ///< video PID
    unsigned short              mp2ts_audio_pid;            ///< audio PID
    CNC_SINT32                  enableSWCrypto;             ///< Set to 1 to enable SW crypto and 0 to enable HW crypto
    CNC_SINT32                  hdcpFacsimileKeyEnable;     ///< Set this parameter to 1 to use facsimile keys
    cncLDENCBuiltinSplitter_t   builtinSplitter;            ///< Supports Disabled (0), Bypass (1), or Capture (2) modes
    cncNetInstance_t*           nwInstance;                 ///< WIFI Network instance handle
    unsigned int                sqosMode;                   ///< SQOS Mode: 0 = Disabled; 1 = Enabled
    
} cncLDENCConfig_t;

/** Structure showing the status of the ldEnc library. Requested with CNC_LDENC_GetStats.
 * typically used after an event like resolution change or HDCP change.
 */
typedef struct _cncLDENCStats_t {
    CNC_UINT32 total_video_bytes;       ///< total amount of encoded video bitstream data (does not include TS or RTP)
    CNC_UINT32 total_audio_bytes;       ///< total amount of encoded audio bitstream data (does not include TS or RTP)
    CNC_UINT32 total_bytes;             ///< total amount of encoded audio and video bitstream data including TS and RTP.
    cncResolutionDetails_t resolution;  ///< structure with the resolution details (width, height, framerate...)
    int plug_status;                    ///< Set to 1 when plugged and 0 when unplugged (this indicates hpd status, not necessary cable status)    
    int cable_status;                   ///< Set to 1 when cable is plugged and 0 when cable is unplugged    
} cncLDENCStats_t;


/** Library handle
 */
typedef CNC_VOID* cncLDENCHnd_t;

typedef CNC_VOID (*cncLDENCCallback_t)(cncLDENCHnd_t, cncLDENCEvent_t);

/** LD Enc creator function. Allocation of the ldEnc handle happens here.
 * 
 * @return returns a newly allocated ldenc handle
 */
cncLDENCHnd_t CNC_LDENC_Create( CNC_VOID );


/** LD Enc destroy function. deallocation of the ldEnc handle happens here.
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_Destroy(cncLDENCHnd_t handle);


/** Configure the ldenc library
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param config previously allocated structure with the configuration parameters.
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_SetConfig(cncLDENCHnd_t handle, cncLDENCConfig_t* config);

/** Get the ldenc library configuration structure
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param config previously allocated structure were the current configuration will be written
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_GetConfig(cncLDENCHnd_t handle, cncLDENCConfig_t* config);

/** Get the ldenc library status
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param stats previously allocated structure were the current ld Enc status will be written
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_GetStats(cncLDENCHnd_t handle, cncLDENCStats_t* stats);

/** Register a callback for a particular event
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param ldenc_event event for which to register the callback
 * @param func function to use for the callback
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_registerCallback(cncLDENCHnd_t handle, cncLDENCEvent_t ldenc_event, cncLDENCCallback_t func);

/** Register a callback for determining if Power Management state is active
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param func_ptr pointer to function that returns 1 if Power Management state is active; returns 0 otherwise
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_registerPmStateCallback(cncLDENCHnd_t handle, int (*func_ptr)(void));

/** Initialize the audio encoder and HDMI Rx. Certain config parameters are only processed once when the thread is 
 * started (HDCP, for example), so make sure it is properly set before calling CNC_LDENC_Init. The audio and video
 * threads are started upon detection of a valid video input signal.
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_Init(cncLDENCHnd_t handle);


/** Start encoding video and audio to a network sink
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param netSink information regarding the sink to send encoded A/V data
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_Start(cncLDENCHnd_t handle, cncLDENCNetSinkConfig_t *netSink);


/** Stop the encoding process for a netSink
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param sink sink ID to be stopped
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_Stop(cncLDENCHnd_t handle, CNC_SINT32 sink);


/** Request next frame to be an IDR
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_RequestIDR(cncLDENCHnd_t handle);


/** Skip the next frameToSkip frames
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param frameToSkip Number of frames to skip
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_SkipFrames(cncLDENCHnd_t handle, int framesToSkip);

/** set Bitrate
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_SetBitrate(cncLDENCHnd_t handle, CNC_SINT32 bitrate);

/** set source input select
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_SetSourceInputSelect(cncLDENCHnd_t handle, CNC_SINT32 sourceInputSelect);

/** Set the EDID for the given sink.
 * @brief This will result in a new EDID being presented to the upstream HDMI device (toggles HPD)
 *        A zero length will remove the EDID information for a given sink.
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param sink internal ID of the sink with that EDID
 * @param edid pointer to buffer containing the edid
 * @param len length of the edid buffer
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_SetEDID(cncLDENCHnd_t handle, CNC_SINT32 sink, CNC_UINT8* edid, CNC_UINT32 len);

/** Set whether HDCP is supported on a remote sink or not
 * @brief If not supported, this will result protected content to stop flowing
 *        to that sink, and any authentication attempts will cease
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param sink internal ID of the sink with that EDID
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_SetSinkHDCPSupported(cncLDENCHnd_t handle, CNC_SINT32 sink, CNC_BOOL hdcpSupported);

/** Get the Infoframes from the source
 * @brief This retrieves the latest Infoframes from the source. Called usually after a
 *        CNC_LDENC_EVENT_INFOFRAME_CHANGE event.
 *
 * @param handle ld Enc handle created with CNC_LDENC_Create
 * @param data storage for the infoframe, 1024 bytes minimum
 * @param len length of the infoframe in 32-bit words
 * @return returns CNC_LDENC_FAILURE or CNC_LDENC_SUCCESS
 */
CNC_SINT32 CNC_LDENC_GetInfoFrames(cncLDENCHnd_t handle, CNC_UINT32 *data, CNC_UINT32 *len);

#ifdef __cplusplus
}
#endif

#endif // CNC_LDENC_INTERFACE_H
