/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_ldmedia.h
 * Low Delay AV processing for wireless display applications.
 * @ingroup wd
 */

#ifndef CNC_LDMEDIA_INTERFACE_H
#define CNC_LDMEDIA_INTERFACE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <pthread.h>
#include "cnc_common_typedef.h"

typedef enum _cncLDMEDIAReturnValue_t {
    CNC_LDMEDIA_FAILURE = -1,
    CNC_LDMEDIA_SUCCESS = 0
} cncLDMEDIAReturnValue_t;

/** This enumeration has all the possible events that could be triggered
 * from the ldmedia library.
 */
typedef enum _cncLDMEDIAEvent_t {
    CNC_LDMEDIA_EVENT_ALL_EVENTS = -1,                      ///< This can be used in the CNC_LDMEDIA_registerCallback to assign all events to the same callback
    CNC_LDMEDIA_EVENT_PACKETLOSS = 0,                       ///< This callback will be called upon detecting a packet loss
    CNC_LDMEDIA_EVENT_RESOLUTION_CHANGE,                    ///< This callback gets called when there is a resolution change in the incoming video stream and also sent when there is an overscan change. cncLDMEDIAStats_t will have the actual information on the resolution
    CNC_LDMEDIA_EVENT_NUMAUDIOCHANNELS_CHANGE,              ///< This callback is called when the number of audio channels has changed in the incoming stream (i.e. going from stereo to 5.1 or viceversa). cncLDMEDIAStats_t will have the actual information on the number of channels
    CNC_LDMEDIA_EVENT_MISSING_IDR,                          ///< This callback gets called if the decoder needs an IDR to sync
    CNC_LDMEDIA_EVENT_PWRMGMT_ACTIVITY,                     ///< This callback indicates powermgmt module that there is activity in the datapath
    CNC_LDMEDIA_EVENT_PWRMGMT_INACTIVITY,                   ///< This callback indicates powermgmt module that there is no activity in the datapath
    CNC_LDMEDIA_EVENT_PWRMGMT_PAUSED,                       ///< This callback indicates powermgmt module that the video is currently paused
    CNC_LDMEDIA_EVENT_HDCP_STATUS_CHANGE,                   ///< This callback indicates an HDCP status change.
    CNC_LDMEDIA_EVENT_HDCP_PROTECTED_CONTENT_CHANGE,        ///< This callback is used to indicate that the incoming video has changed from HDCP protected. This is used to start/stop the cvbs output. protectedContent in cncLDMEDIAStats_t will have the status information
    CNC_LDMEDIA_EVENT_SEND_BUFFER_STATUS,                   ///< This callback is called once per second in TCP mode and is used to communicate the RTSP application the buffer status and last decoded PTS. This information is required by certain handsets.
    CNC_LDMEDIA_EVENT_AV_STATUS,                            ///> This callback is sent when av resources are ready to be used and when they have been stopped
	CNC_LDMEDIA_EVENT_KEY_EXTRACTION_FAILURE,				///> This callback is invoked when the HDCP server cannot extract the encryption keys.
    CNC_LDMEDIA_EVENT_HDCP_LIB_MISSING,	        			///> This callback is invoked when the HDCP library cannot be loaded
    CNC_LDMEDIA_EVENT_RESOLUTION_MISMATCH,                  ///> This callback is called if the decoded resolution is bigger than the display resolution, indicating that scaledown is happening. This callback will continue to happen every second until the issue gets resolved.
    CNC_LDMEDIA_EVENT_CRITICAL_ERROR,                       ///> This callback is called if critical error avoids decoder continue decoding in any case (totally closing and opening again ldmedia is recommended once this event is received).
    CNC_LDMEDIA_EVENT_NUMBER
} cncLDMEDIAEvent_t;

/** Enumeration with the different possible audio types supported.
 * Some of this are no longer used and are there for consitency with old wivu code.
 */ 
typedef enum _cncLDMEDIAAudioTypes_t {
     CNC_LDMEDIA_AUD_TYPE_NETHD=0,                          ///< Audio in RTP mode for old NETHD sources
     CNC_LDMEDIA_AUD_TYPE_RTP_16BIT_LPCM = 2,               ///< RTP 16 bit LPCM audio format
     CNC_LDMEDIA_AUD_TYPE_TS_RFC2250_NETHD = 3,             ///< RTP+MPEGTS+for old NETHD sources
     CNC_LDMEDIA_AUD_TYPE_TS_RFC2250_16BIT_LPCM = 4,        ///< RTP+MPEGTS+16 bit LPCM audio format
     CNC_LDMEDIA_AUD_TYPE_AAC = 6,                          ///< RTP+MPEGTS+AAC 
} cncLDMEDIAAudioTypes_t;

/** Enumeration with the different possible video types supported.
 */ 
typedef enum _cncLDMEDIAVideoTypes_t {
    CNC_LDMEDIA_VID_TYPE_3984 = 0,                          ///< Video in RTP format, according to 3984
    CNC_LDMEDIA_VID_TYPE_WFD = 2,                           ///< Video in WFD format (RTP+MPEG2TS RFC2250)
} cncLDMEDIAVideoTypes_t;

/** This enumeration describe the different possible types of control messages that
 * could be used within the WD applications 
 */
typedef enum _cncLDMEDIAControlMsgs_t {
    CNC_LDMEDIA_NO_CONTROL_MSGS = 0,                        ///< No control messages at all
    CNC_LDMEDIA_WIVU_CONTROL_MSGS = 1,                      ///< WiVu control messages, sent through RTP port + 101
    CNC_LDMEDIA_WFD_CONTROL_MSGS = 2,                       ///< WFD control messages are the RTSP IDR requests. Are actually not sent from ldMedia but RTSP
    CNC_LDMEDIA_RTCP_CONTROL_MSGS = 4,                      ///< ldMedia supports sending RTCP messages reporting packet loss
} cncLDMEDIAControlMsgs_t;

/** This enumeration describe the different actions regarding saving of incoming streams 
 */
typedef enum _cncLDMEDIASaveStream_t {
    CNC_LDMEDIA_SAVE_STREAM_STOP = 0,                       ///< Stop stream saving prcess
    CNC_LDMEDIA_SAVE_STREAM_ES = 1,                         ///< Save elementary streams
    CNC_LDMEDIA_SAVE_STREAM_TS = 2,                         ///< Save transport streams
} cncLDMEDIASaveStream_t;

/** This enumeration describes the different control playback commands that can be sent while in TCP mode 
 */
typedef enum _cncLDMEDIAControlPlaybackCmd_t {
    CNC_LDMEDIA_CP_NONE = 0,                                ///< No command
    CNC_LDMEDIA_CP_FLUSH,                                   ///< Flushes the audio and video buffer
    CNC_LDMEDIA_CP_PLAY,                                    ///< Starts playing
    CNC_LDMEDIA_CP_PAUSE,                                   ///< Pauses audio and video
    CNC_LDMEDIA_CP_INDICATOR,                               ///< This is used to show a buffering indicator. Currently not used
    CNC_LDMEDIA_CP_FLUSH_PLAY,                              ///< Flush buffers and start playback again
    CNC_LDMEDIA_CP_FLUSH_PAUSE,                             ///< Flush buffers and leave audio and video paused
    CNC_LDMEDIA_CP_INCREASE_VOLUME,                         ///< Increase volume
    CNC_LDMEDIA_CP_DECREASE_VOLUME,                         ///< Decrease volume
    CNC_LDMEDIA_CP_SET_VOLUME                               ///< Set volume (0-15)
} cncLDMEDIAControlPlaybackCmd_t;


/** This enumeration describe the different status the datapath can be at
 */
typedef enum _cncLDMEDIAStatus_t {
    CNC_LDMEDIA_STATUS_STOPPED = 0,                       ///< Stop stream saving prcess
    CNC_LDMEDIA_STATUS_READY = 1,                         ///< Save elementary streams
} cncLDMEDIAStatus_t;

/** This enumeration describe the different partial frame modes available
 */
typedef enum _cncLDMEDIAPartialFrameMode_t {
    CNC_LDMEDIA_PFM_NOT_SET = -1,                           ///< Partial Frame Mode Disabled
    CNC_LDMEDIA_PFM_DISABLED = 0,                           ///< Partial Frame Mode Disabled
    CNC_LDMEDIA_PFM_EARLY_PROC = 1,                         ///< Early Decoding Partial Frame Mode
    CNC_LDMEDIA_PFM_EARLY_PROC_AND_DEC = 2,                 ///< Early Decoding and Processing Partial Frame Mode
} cncLDMEDIAPartialFrameMode_t;

#define  CNC_LDMEDIA_HDCP_2_2              0x22
#define  CNC_LDMEDIA_HDCP_2_1              0x21
#define  CNC_LDMEDIA_HDCP_2_0              0x20
#define  CNC_LDMEDIA_HDCP_NONE             0x00

#define  CNC_LDMEDIA_VIDEO_SURFACE_OPAQUE  0x3FF
#define  CNC_LDMEDIA_VIDEO_SURFACE_TRANS   0x00

/** Structure with the parameters that can be configured in the ldmedia module.
 * Calling a CNC_LDMEDIA_GetConfig will fill up the structure with the current parameters
 * After init, the current parameters are those chosen as defaults
 */ 
typedef struct _cncLDMEDIAConfig_t {
    CNC_SINT32 doNotConfigTvOut;                            ///< if this is set to 0, the ldmedia library will take ownership of cstvout and reconfigure it at will
    CNC_SINT32 hdcpRepeaterSupport;                         ///< This parameter used to enable/disable HDCP repeater support
    CNC_SINT32 hdcpFacsimileKeyEnable;                      ///< Set this parameter to 1 to use facsimile keys
    CNC_SINT32 closeHdcpAKESocket;                          ///< if set to 1, the HDCP socket is closed after a successful connection
    CNC_SINT32 enableSWCrypto;                              ///< Set to 1 to enable SW crypto and 0 to enable HW crypto
    CNC_SINT32 recvBufSize;                                 ///< This parameter can be used to change the receive socket buffer size. It translate into a setsockopt call
    CNC_SINT32 overscanX;                                   ///< amount of overscan compensation in the horizontal direction (in percentage)
    CNC_SINT32 overscanY;                                   ///< amount of overscan compensation in the vertical direction (in percentage)
    CNC_SINT32 errSkipMode;                                 ///< Error skip mode: 0 for showing as much as possible from corrupted frames and 3 for skipping corrupted frames. 4 to 7 values includes treat frames gaps as errors 
    CNC_SINT32 skipSocket;                                  ///< This is a debug tool, which causes the RX socket not to be read while it is set to 1. reading from the socket resumes once set to 0.
    CNC_SINT32 jitterBuffer;                                ///< this sets the jitterBuffer of the decoder. This is mainly used in TCP mode.
    CNC_SINT32 avSync;                                      ///< avSync type: 0- No avSync (should not be used), 1- avSync, 2- fast mode (av might get out of sync, but latency is normally lower).
    CNC_SINT32 avSync_error;                                ///< avSync error: When avSync = 1, error accepted (video ahead audio) to improve latency.
    CNC_SINT32 rtpTCP;                                      ///< if set to 0 it works in UDP mode if set to 1 it works in TCP mode
    CNC_SINT32 controlMsgs;                                 ///< Enable control messages. Different types of control messages can be enabled by OR'ing elements of cncLDMEDIAControlMsgs_t
    CNC_SINT32 RTCPRRPeriod;                                ///< When CNC_LDMEDIA_RTCP_CONTROL_MSGS are being used, this specifies the time between consecutive RTCP RR messages (in seconds)
    CNC_SINT32 IDRReqPeriod;                                ///< When CNC_LDMEDIA_WFD_CONTROL_MSGS are being used, this specifies the minimum time between consecutive IDR requests.
    CNC_SINT32 videoOffset;                                 ///< Offset added to video PTS before sending it to the AV decoding firmware.
    CNC_SINT32 audioOffset;                                 ///< Offset added to audio PTS before sending it to the AV decoding firmware.
    CNC_SINT32 tcpVideoPtsOffset;                           ///< Same thing as videoOffset but only applies for the TCP mode.
    CNC_UINT64 flushTiming;                                 ///< Specifies a PTS up to which all the data received should be dropped.
    CNC_SINT32 volume;                                      ///< This allows configuration of the volume of the audio range is 0 to 70 while on UDP and 0 to 15 when on TCP (as per customer requirement)
    cncLDMEDIASaveStream_t saveStream;                      ///< Possible actions are those in the structure cncLDMEDIASaveStream_t
    CNC_SINT32 debugMask;                                   ///< Enable different debugging possibilities
    CNC_SINT32 udpTimeoutMs;                                ///< Time out for udp reorder and retransmission
    CNC_SINT32 udpRetransmissionPortOffset;                 ///< offset of the port used for UDP retransmission
    CNC_SINT32 videoRes;                                    ///< Resolution to use for the decoded video. Look at CSTVOUT_MODE for possibilities
    CNC_SINT32 frameRate;                                   ///< Framerate to use on the video decoding, only used for case avSync 0. Look at CSVID_FRAME_RATE for possibilities
    CNC_SINT32 frameType;                                   ///<  0-auto detect; 1-progressive; 2-interlace; 3-use coda detected vaule.
    CNC_SINT32 decoderLatency;                              ///< Specify the AV decoder latency. Setting this too low might result in video artifacts, setting it too high might result in too much latency.
    CNC_SINT32 udpAudLatencyMarginHighEn;                   ///< Enable/disables mechanism to relax latency restrictions for avSync = 2, (AV sync will suffer, but there will be less audio drops if this is increased).
    CNC_SINT32 udpAudLatencyMarginHigh;                     ///< When udpAudLatencyMarginHighEn = 1, High margin for latency value. This value will act as a threshold to return to udpAudLatencyMargin.
    CNC_SINT32 udpAudLatencyMargin;                         ///< When udpAudLatencyMarginHighEn = 1, final latency value after udpAudLatencyMarginHigh threshold is achieved.
    CNC_SINT32 sqosEnable;                                  ///< SQOS Enable. SQOS main enable/disable. 0 = OFF; 1 = ON;
    CNC_SINT32 sqosMode;                                    ///< SQOS Mode. SQOS is a mechanism to automatically adapt decoder latency. 0 = disabled; 1 = Fixed Step; 2 = Smart Step. 
    CNC_SINT32 initialMute;                                 ///< Initial mute time after audio STOP/PLAY
    CNC_SINT32 dropInitialAudio;                            ///< Drops all audio until the first PCR is received
    CNC_UINT64 startingPTS;                                 ///< apply starting PTS
    CNC_SINT32 errorRecovery;                               ///< chooses how we recover against errors. Set to 1.
    CNC_SINT32 drop;                                        ///< For debug purposes. Drops AV packet.
    CNC_SINT32 sendPlayBeforeSKE;                           ///< Some devices seemed to erroneously expect us to send the play before SKE, this allows for this condition.
    CNC_CHAR   srcIpAddress[17];                            ///< IP address from the source device
    CNC_SINT32 audioPort;                                   ///< port in which audio will be received
    CNC_SINT32 videoPort;                                   ///< port in which video will be received
    CNC_SINT32 controlMessagePort;                          ///< port used for WiVu control messages
    CNC_SINT32 rtcpPort;                                    ///< port used for RTCP control messages
    cncLDMEDIAVideoTypes_t videoType;                       ///< Video type. a member of enumeration cncLDMEDIAVideoTypes_t
    cncLDMEDIAAudioTypes_t audioType;                       ///< Audio type. a member of enumeration cncLDMEDIAAudioTypes_t
    CNC_SINT32 audioDac;                                    ///< Enable or disable audio DAC: 1 for enable, 0 for disable
    CNC_SINT32 hdcp;                                        ///< Enable or disable HDCP. Should only be set before doing the run    
    CNC_SINT32 hdcpRxVersion;                               ///< Set Sink device HDCP protocol version. Should only be set before doing the run    
    CNC_SINT32 hdcpPort;                                    ///< Port to use for HDCP AKE
    CNC_UINT32 num_proc_blocks;                             ///< Maximum number of blocks a frame is divided to be processed: decryption and decoding (valid for partialFrameMode > 0)
    cncLDMEDIAControlPlaybackCmd_t controlPlaybackCmd;      ///< Control playback command
    CNC_UINT32 alpha;                                       ///< alpha of the video, from 0x3FF (opaque) to 0 (tranparent). Set to -1 to follow input
    cncLDMEDIAPartialFrameMode_t partialFrameMode;          ///< Partial Frame Mode: See cncLDMEDIAPartialFrameMode_t for details
    CNC_SINT32 datapath_inactive_timeout;                    ///< AV datapath inactive timeout in seconds.

} cncLDMEDIAConfig_t;

/** Structure showing the status of the ldMedia library. Requested with CNC_LDMEDIA_GetStats.
 * typically used after an event like resolution change or HDCP change.
 */
typedef struct _cncLDMEDIAStats_t {
    CNC_SINT32 numAudioChannels;                            ///< Number of audio channels that the decoder has encountered in the stream
    CNC_SINT32 overscanTop;                                 ///< Currently used vertical overscan compensation
    CNC_SINT32 overscanLeft;                                ///< Currently used horizontal overscan compensation
    CNC_SINT32 width;                                       ///< decoded video width 
    CNC_SINT32 height;                                      ///< decoded video height
    CNC_SINT32 frameRate;                                   ///< decoded video Framerate
    CNC_SINT32 hdcp2xsuccess;                               ///< HDCP status: success (1) or failed (0)
    CNC_SINT32 hdcpVersion;                                 ///< HDCP version: Either CNC_LDMEDIA_HDCP_2_1 or CNC_LDMEDIA_HDCP_2_0
    CNC_SINT32 protectedContent;                            ///< is content protected (1) or on the clear (0).
    CNC_SINT32 audioBufSize;                                ///< Status of the audio buffer
    CNC_UINT32 audioLastPts;                                ///< Last decoded audio PTS
    CNC_UINT32 videoLastPts;                                ///< Last decoded video PTS
    cncLDMEDIAStatus_t status;                              ///< AvDataPath status
    CNC_UINT32 packetsDropped;                              ///< Amount of packet drops (based on RTP sequence gaps)
    CNC_UINT32 sequenceErrors;                              ///< Amount of sequence errors (based on RTP sequence gaps)
} cncLDMEDIAStats_t;


/** Library handle
 */
typedef CNC_VOID* cncLDMEDIAHnd_t;

typedef CNC_VOID (*cncLDMEDIACallback_t)(cncLDMEDIAHnd_t, cncLDMEDIAEvent_t);

/** LD media creator function. Allocation of the ldMedia handle happens here.
 * 
 * @return returns a newly allocated ldmedia handle
 */
cncLDMEDIAHnd_t CNC_LDMEDIA_Create( CNC_VOID );


/** LD media destroy function. deallocation of the ldMedia handle happens here.
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_Destroy(cncLDMEDIAHnd_t handle);


/** Configure the ldmedia library
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @param config previously allocated structure with the configuration parameters.
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_SetConfig(cncLDMEDIAHnd_t handle, cncLDMEDIAConfig_t* config);

/** Get the ldmedia library configuration structure
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @param config previously allocated structure were the current configuration will be written
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_GetConfig(cncLDMEDIAHnd_t handle, cncLDMEDIAConfig_t* config);

/** Get the ldmedia library status
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @param stats previously allocated structure were the current ld media status will be written
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_GetStats(cncLDMEDIAHnd_t handle, cncLDMEDIAStats_t* stats);

/** Register a callback for a particular event
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @param ldmedia_event event for which to register the callback
 * @param func function to use for the callback
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_registerCallback(cncLDMEDIAHnd_t handle, cncLDMEDIAEvent_t ldmedia_event, cncLDMEDIACallback_t func);


/** Start the ld Media thread. This does not start the decoding process. Certain config parameter
 * are only processed once when the thread is started (HDCP, for example), so make sure it is properly
 * set before calling CNC_LDMEDIA_Run
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_Run(cncLDMEDIAHnd_t handle);


/** Start the decoding process
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_Start(cncLDMEDIAHnd_t handle);


/** Stop the decoding process
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_Stop(cncLDMEDIAHnd_t handle);


/** Power management enabling of the AV datapath
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_PwrmgmtEnableAV(cncLDMEDIAHnd_t handle);


/** Power management disabling of the AV datapath
 *
 * @param handle ld media handle created with CNC_LDMEDIA_Create
 * @return returns CNC_LDMEDIA_FAILURE or CNC_LDMEDIA_SUCCESS
 */
CNC_SINT32 CNC_LDMEDIA_PwrmgmtDisableAV(cncLDMEDIAHnd_t handle);


// TODO Add control playback command


#ifdef __cplusplus
}
#endif

#endif // CNC_LDMEDIA_INTERFACE_H
