/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
/**
 * @file cnc_web.h
 * @ingroup web
 * The CNCWEB library is a wrapper around an integrated browser and web engine
 */

#ifndef CNC_WEB_H_
#define CNC_WEB_H_

#include "cnc_web/cnc_browserFeatures.h"
#include "cnc_web/cnc_browserVersion.h"
#include "cnc_web/cnc_browserTypes.h"
#include "cnc_web/cnc_browserCommon.h"
#include "cnc_web/cnc_browserCustomJS.h"
#include "cnc_web/cnc_browserCallback.h"
#include "cnc_web/cnc_browserEvents.h"
#include "cnc_web/cnc_browserHistory.h"
#include "cnc_web/cnc_browserGfx.h"
#include "cnc_web/cnc_browserControls.h"
#include "cnc_web/cnc_browserNavigate.h"
#include "cnc_web/cnc_browserBookmarks.h"
#include "cnc_web/cnc_browserFontEngine.h"
#include "cnc_web/cnc_browserHTML5MediaPlayerI.h"
#include "cnc_web/cnc_browserPlugins.h"
#include "cnc_web/cnc_browserDownload.h"
//#include "cnc_web/cnc_browserGfx3D.h"
#include "cnc_web/cnc_browserMediaPlayerI.h"
#include "cnc_web/cnc_browserPrinter.h"
#include "cnc_web/cnc_browserSettings.h"
#include "cnc_web/cnc_browserWebSocket.h"
#include "cnc_web/cnc_browserPluginEvents.h"
#include "cnc_web/cnc_browserTextCodec.h"

#include "cnc_common_typedef.h"
#include "cnc_input_keys.h"
#include "cnc_media.h"

#ifdef __cplusplus
extern "C" {
#endif


#define CNC_WEB_MEDIA_USER_AGENT_SIZE                                           (256)

typedef enum {
	CNC_WEB_VISIBLE,
	CNC_WEB_HIDDEN
} CNC_Web_DisplayState;

typedef enum {
    CNC_WEB_INPUT_KEY_UNKNOWN = 0,
    CNC_WEB_INPUT_KEY_DOWN    = 1,
    CNC_WEB_INPUT_KEY_UP      = 2
} cncWebInputKeyboardEvent_t;

typedef struct _cncWebEventKeyboard_t
{
    cncWebInputKeyboardEvent_t keyEvent;    /*! Type of event */
    CNC_SINT32                 keycode;     /*! Ascii Character value */
} cncWebEventKeyboard_t;

typedef struct _cncWebEventMouse_t
{
    CNC_SINT32  x;                          /*! X Coordinate  */
    CNC_SINT32  y;                          /*! Y Coordinate  */
} cncWebEventMouse_t;

typedef enum {
    CNC_WEB_INPUT_TYPE_IP_KEYBOARD = 0x01,     /*! Input type = Keyboard */
    CNC_WEB_INPUT_TYPE_IP_MOUSE    = 0x02,     /*! Input type = mouse */
    // TODO Add new input type
} cncWebEventTypes_t;

typedef struct _keyEventStructure_t
{
    cncWebEventTypes_t  eventType;          /*! Event Type */
    union
    {
        cncWebEventKeyboard_t keyboard;     /*! Keyboard event */
        cncWebEventMouse_t    mouse;        /*! Mouse event */
    };
}CNC_Web_keyEventStructure_t;

typedef struct _cncWebMediaConfigInfo_t
{
    CNC_SINT32  maxNetworkBandWidth;    /*!< max network bandwidth to be use for media playback */
    CNC_SINT32  loopFlag;               /*!< Enable/Disable loop play after playback END */
    CNC_SINT32  preBufferingTime;       /*!< Buffering time during playback in secs. */
    CNC_SINT32  preBufferingTimeAtStart;/*!< Buffering time before playback starts in secs. (First time buffering time) */
    CNC_SINT32  preBufferingInBytes;    /*!< Buffering time in terms of Size during playback. */
    CNC_SINT32  minBandWidthAtStart;    /*!< */
    CNC_SINT32  minVideoResAtStart;     /*!< */
    CNC_SINT32  bandWidthOverhead;      /*!< */
    CNC_SINT32  manifestDownloadTimeout;
    CNC_SINT32  adaptType;
    CNC_CHAR    userAgent[CNC_WEB_MEDIA_USER_AGENT_SIZE];

}cncWebMediaConfigInfo_t;

/**
 * To initialize all the required thing for browser.
 * @details     This should be called before CNC_Web_Init().
 * @param       launchpath
 *                  Browser's application dir which contains resources, fonts, etc.. related to browser.
 *                  eg, "/home/applications"
 */
void CNC_Web_PreInit(char *launchpath);

/**
 * This sets the use of overscan safety in the display
 * @details     This should be called before CNC_Web_PreInit().
 * @param       setting
 *                  CNC_TRUE causes the browser output to be restricted within the overscan safe region.\n
 *                  CNC_FALSE causes the browser output to occupy the entire screen.
 */
void CNC_Web_SetEnableOverscanSafety(CNC_BOOL setting);

/**
 * This will start the main loop of browser.
 *                You should do all the registration of callback functions before calling this.
 * @param[in]   launchpath:
 *                  Not used, Can be NULL.
 */
void CNC_Web_Init(char *launchpath);

/**
 * Start browser with specified URL.
 *                This should be called before CNC_Web_Start().
 * @param[in]   url
 *                  URL of the page.
 * @param[in]   w Width
 * @param[in]   h Height
 */
void CNC_Web_Open(const char *url, int w, int h);

/**
 * Start browser with specified URL.
 *                This should be called after CNC_Web_Open().
 * @param[in]   url
 *                  URL of the page.
 * @param[in]   x X Position.
 * @param[in]   y Y position
 * @param[in]   w Width
 * @param[in]   h Height
 */
void CNC_Web_Start(const char *url, int x, int y, int w, int h);

/**
 * Flush the browser's history and go to the specified URL.
 * @details     The caller must first register a callback which returns an appropriate value, with CNC_Web_RegisterExitRequestCallback.
 * @param[in]   url
 *                  URL of the page.
 */
void CNC_Web_GotoHomeUrl(char *url);

/**
 * @brief       : Flush Browser History and Session History.
 *                Once this function is called Browser will clear all the URL it had navigated earlier and After that Back Key has no effect.
 * @param       : None
 * @return      : None
 */
void CNC_Web_BrowserClearHistory(void);

/**
 * @brief       : Used to load New Page.
 * @param[in]   : url : Url to Load
 * @return      : None
 */
void CNC_Web_GotoUrl(const char* url);

/**
 * Exits the browser without terminating it.
 *                This should be called before CNC_Web_Stop().
 */
void CNC_Web_Stop(void);

/**
 * Set the current state of browser
 * @param[in]   state
 *                  CNC_WEB_VISIBLE, if Visible \n
 *                  CNC_WEB_HIDDEN, if Not Visible
 */
void CNC_Web_SetDisplayState(CNC_Web_DisplayState state);

/**
 * Get the current state of browser.
 * @return      The current state of browser.
 *                  CNC_WEB_VISIBLE, if Visible \n
 *                  CNC_WEB_HIDDEN, if Not Visible
 */
CNC_Web_DisplayState CNC_Web_GetDisplayState(void);

/**
 * Send IR/Keyboard/Mouse event to browser.
 * @param[in]   keyval
 *                  Integer key value defined above.
 * @return      Return the result,
 *                  CNC_BOOL_FALSE, if Fails to send. \n
 *                  CNC_BOOL_TRUE, if success.
 */
CNC_BOOL CNC_Web_SendKeyEvent(int keyval);

/**
 * @brief       Send IR/IP-Keyboard/IP-Mouse event to browser.
 *              Currently it supports only IP-Keyboard.
 * @param[in]   event
 *                  Event structure.
 * @return      Return the result,
 *                  CNC_BOOL_FALSE, if Fails to send. \n
 *                  CNC_BOOL_TRUE, if success.
 */
CNC_BOOL CNC_Web_SendKeyEventStructure(CNC_Web_keyEventStructure_t event);

/**
 * Exits the browser without terminating it.
 *                This should be called before CNC_Web_Exit() and after CNC_Web_Stop().
 */
void CNC_Web_Close(void);

/**
 * Exits the browser without terminating it.
 *                This should be called before after CNC_Web_Close().
 * @details     After this is invoked, it is still possible to relaunch the browser using CNC_Web_Open() again.
 */
void CNC_Web_Exit(void);

/**
 * Registers a callback which is invoked by the browser to determine how to handle a "Back" navigation on the first page.
 * @param[in]   exitRequestCallback
 *                  A callback function defined by the caller. This should be implemented as follows, for handling "Back" navigation on the first page:
 *                      - Remain in the first page without exiting.
 *                      - Remain in the first page but exit: invoke CNC_Web_Stop(), CNC_Web_Close(), CNC_Web_Exit().
 *                          Will return to the page when relaunched. If the URL specified at relaunch is not the same as the first page, it will then jump to the new URL.
 *                      - Exit and prepare to load a new page: invoke CNC_Web_Stop(), CNC_Web_Close(), CNC_Web_Exit().
 *                          Will not return to the first page when relaunched, but directly goes to the new URL.
 *                  Return(int): Return value is unused(dummy)(For Espial6.1 Release package, For Espial6.0 release package, return value is valid).
 */
void CNC_Web_RegisterExitRequestCallback(int (*exitRequestCallback)(void));

/**
 * Clear the browser output. This will allow the layers below the browser to become visible.
 */
void CNC_Web_ClearSurface(void);

/**
 * Release memory allocated in CNC_Web_PreInit().
 *                This terminates the browser and must be called after CNC_Web_Exit().
 * @details     After this is invoked, browser cannot be relaunched unless CNC_Web_PreInit() and other steps are performed again.
 */
void CNC_Web_Release(void);

void CNC_Web_SetOnScreenKeyBoardDir(const char *dir);

/**
 * @brief       Returns the layer ID currently being used by the browser.
 * @return      OSD virtual layer ID on which the browser's output is being displayed.
 *              A value lesser than zero indicates the browser is not being displayed.
 */
CNC_SINT32 CNC_Web_LayerID(void);


/**
 * @brief       : Registers a callback which is invoked by the html5 plugin to set media setting before video/audio playback.
 * @param[in]   : p_f_mediaSettingCallback :
 *                  Callback function to register.
 *                  This Cbf function will be called to get the media setting value when any Video/audio will be played from browser using html5 video/audio.
 */
void CNC_Web_RegisterMediaSettingsCallback(CNC_SINT32 (*p_f_mediaSettingCallback)(cncWebMediaConfigInfo_t *p_configInfo));

/**
 * @brief       : Callback function to handle window close, if browser will not able to handle this and got error while processing window.close().
 * @param[in]   : p_f_windowCloseCallback:
 *                  Function Which will get called if any error occurred during window.close().
 *                  Input param : None
 *                  Returns 1 if handled, 0 if not handled.
 * @return      : Returns 0 on Success \n
 *                non-zero value
 */
CNC_SINT32 CNC_Web_RegisterWindowCloseCallback(CNC_SINT32 (*p_f_windowCloseCallback)());

/**
 * @brief       : Registers a callback which is invoked by the html5 plugin when the plugin's cncmedia handle changes.
 * @param[in]   : callback :
 *                  Callback function to register.
 *                  This Cbf function will be called when the cncmedia handle changes
 * @param[in]   : context :
 *                  context for the callback registered.
 */
void CNC_Web_RegisterMediaHandleCallback(void (*p_f_callback)(void *handle, void *context), void* context);

/**
 * @brief       : Registers a callback which is invoked when there is a new cncmedia event on the cncmedia handle.
 * @param[in]   : callback :
 *                  Callback function to register.
 *                  This Cbf function will be called when there is a new cncmedia event
 */
void CNC_Web_RegisterMediaEventCallback(cncMediaCtlEventCallback_t p_f_callback);

/**
 * @brief       : Set if Adobe PrimeTime will be used with for following streams. It is reset to not use primetime when releasing the video tag.
 * @param[in]   :  adobePrimeTime
 *                      CNC_BOOL_TRUE Adobe primetime will be used
 *                      CNC_BOOL_FALSE Adobe primetime will not be used
 */
void CNC_Web_UseAdobePrimetime(CNC_BOOL adobePrimeTime);

/** 
 * @brief Set the customdata field for playready. If it is not null
 * the html5 plugin will use smooth streaming with playready
 * 
 * @param customData 
 */
void CNC_Web_SetCustomData(const char *customData);

/** 
 * @brief Use this cookie with drm http requests
 * 
 * @param cookie 
 */
void CNC_Web_SetCookie(const char *cookie);

/** 
 * @brief Set the license server url, it takes precedence to the one
 * in the manifest
 * 
 * @param licenseServerUrl 
 */
void CNC_Web_SetLicenseServerUrl(const char *licenseServerUrl);

/** 
 * @brief Tell the DRM agent to store the license, overriding the server decision
 * 
 */
void CNC_Web_StoreDrmLicense();

/** 
 * @brief Set the subtitles url that will be used for the next video load
 * 
 * @param url 
 */
void CNC_Web_SetSubtitleUrl(const char *url);

/** 
 * @brief Notify cnc_web that the volume has been changed. The new
 * volume will be read from cnc_settings
 * 
 */
void CNC_Web_NotifyVolumeChange(void);

#ifdef __cplusplus
}
#endif


#endif /* CNC_WEB_H_ */
