/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser callback operations
 * @file cnc_browserCallback.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_CALLBACK_H__
#define __CNC_BROWSER_CALLBACK_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum
{
    CNC_Browser_MEM_NORMAL,        /**< No OOM happend */
    CNC_Browser_MEM_HIT_THRESHOLD, /**< It hits the OOM Threashold */
    CNC_Browser_MEM_HIT_CAP,       /**< It hits the memory cap */

    /**
     * out-of-memory happened during content loading/rendering
     *
     * the return value is ignored.
     */
    CNC_Browser_OOM_Detected,
} CNC_Browser_OOMState_e;


typedef enum
{
    CNC_Browser_OOMAction_StopLoading			= 0x00000001, /*stop all loadings*/
    CNC_Browser_OOMAction_StopScript  		= 0x00000002, /*stop executing scripts*/
    CNC_Browser_OOMAction_StopImageDecoding 	= 0x00000004, /*stop decoding image data*/
    CNC_Browser_OOMAction_StopPlugins 		= 0x00000008, /*stop loading plugins*/

    CNC_Browser_OOMAction_ClearCache		 	= 0x00000100, /*clear cached data*/
    CNC_Browser_OOMAction_ClearDecodedImage 	= 0x00000200, /*clear image decoded data*/

    CNC_Browser_OOMAction_LoadBlankPage		= 0x00001000, /*load black page*/
} CNC_Browser_OOMAction_e;

/*
 * List of call back functions that to be called by the browser
 * This should be registered right after the browser is initialized
 */
typedef struct
{

	/**
	 * @brief called when making a new connection,
	 *
	 * @param protocol, the connection protocol, for example, "http" or "https"
	 * @param host,		the server, for example, "google.com"
	 * @param file,		[Not used] the resource, for example, "/doc/libc/libc_600.html"
	 * @param value, 	value to fill in
	 * @param VeriSign
	 * @return
	 * 		1, if new user agent is set in value
	 * 		0, no user agent is set, use the default one
	 */
	int (*userAgent)(const char* protocol, const char* host, const char* file, CNC_BrowserCString_t* value, int VeriSign);

    /**
     * Called when the CNC browser detects OOM occurs
     * @param state The level of Out-of-memory
     * @return CNC_Browser_OOMAction_e masks
    */
	int (*memoryCapHandler)(CNC_Browser_OOMState_e state);

    /**
     * @brief Called when a request to open a new URL happens.
     *
     * @param gotourl 	The requested URL
	 * @param newurl 	A different URL to load.
	 * 		If the release pointer is set, then the memory is freed by calling the
	 * 		release function after the browser finished using the newurl.
     * @return
     * 		0,	stop loading, the browser does not load 'gotourl', also, it does not
     * 			check the 'newurl', it stays on the current content.
     * 		otherwise,
     * 			if 'char* chars' in newurl is set, load the 'newurl';
     * 			if 'char* chars' in newurl is null, load the 'gotourl';
     */
    int (*urlPreload)(const char *gotourl, CNC_BrowserString_t *newurl);

    /**
     * @brief Called when ENTER key is hit on an HTML input element.
     * @param input
     * @param inputScreenRect, input bounds relative to screen
     */
    void (*textInput)(const CNC_BrowserInput_t *input, const CNC_BrowserRect_t* inputScreenRect);

    /**
     * @brief Called when an HTML input element lost focus.
     */
    void (*inputFocusLost)(void);

	/**
	 * Called when the CNC_ browser's visibility changes
	 * @param visible - 1 if browser is visible and 0 if not visible
	 */
	void (*visibilityChanged) (int visible);

	/**
	 * @brief Called when an error has occurred.
	 * @return int Returns 1 if handled, 0 if not handled.
	 */
	int (*errorOccurred)(CNC_BrowserError_t* error);

	/**
	 * handle viewport change request by content
	 *
	 * "https://developer.apple.com/library/safari/#documentation/AppleApplications/Reference/SafariHTMLRef/Articles/MetaTags.html
	 * <meta name="viewport" content="width=1920, height=1080, initial-scale=1.0, user-scalable=no">"
	 *
	 * @param width,        -1 for default
	 * @param height,       -1 for default
	 * @param initialScale, -1 for default
	 * @param userScalable,  0 if user can not scale the content, otherwise 1
	 * @param url,          the content URL
	 */
	void (*handleViewport)(int width, int height, float initialScale, int userScalable, const char* url);

	/**
	 * @brief the current time zone abbreviation name
	 * @param buf, must be filled with null terminated string
	 * @param buf_len, at least 16 bytes
	 * @return the length of the name string
	 */
	int (*currentTimeZoneName)(char* buf, int buf_len);

	/**
	 * @brief Provides the type of cursor to change to
	 * @param cursor Cursor structure that contains the x, y and the type of cursor
	 */
	void (*setCursor)(CNC_BrowserCursor_t *cursor);

	/**
	 * @brief return the current local system time
	 * @param out_time - to fill
	 * @return 1 if success, 0 on failure, out_time is not set.
	  		For example, (12 February, 2012, 21:51:21.046, EST)
	  			out_time->year = 2012;
	  			out_time->month = 1; //Feb
	 			out_time->day = 12;
	  			out_time->hour = 21;
	  			out_time->minute = 51;
	  			out_time->second = 21;
	  			out_time->ms = 46;
	 */
	int (*currentSystemTime)(CNC_BrowserDateTime_t *out_time);

	/**
	 * @brief Called when the content viewport has reached its edge via scrolling
	 * @param edge The edge that was hit.  When using the CNC_Browser_ScrollBy function, when an edge has been reached, passing 0 for the direction will prevent the callback from being notified again.  E.g. if the right edge has been reached and the callback notified, the next scroll by call should be CNC_Browser_ScrollBy(0, int).
	 */
	void (*contentEdgeReached)(CNC_BrowserDirection_e direction);

	/**
	 * @brief the scrolling called by CNC_Browser_ScrollBy is finished.
	 */
	void (*scrollDone)(void);

	/**
	 * @brief Called when a change has been made to bookmarks
	 */
	void (*bookmarksChanged)(void);


	/*
	 * Get the difference in seconds between this time zone and UTC (GMT)
	 * NOT including DST.
	 * 		for example, (-5 * 60 * 60) //EST, UTC/GMT -5 hours, Eastern Standard Time
	 *                   ( 9 * 60 * 60) //JST, UTC/GMT +9 hours, Japan Standard Time
	 */
	int (*utcOffsetSeconds)(void);

	/*
	 *  Get the DST (day light saving time) offset for the time (in seconds) passed in.
	 */
	int (*dstOffsetSeconds)(double t_seconds);

	/*
	 * @brief Notify the status of the toolbar
	 * @param
	 */
	void (*toolbarStatus)(int visible);
} CNC_BrowserCallbacks_t;

/**
 * @brief Register the list of SDK call back functions
 * @param cb Callback structure
 */
void CNC_Browser_RegisterCallbacks(CNC_BrowserCallbacks_t *cb);


typedef struct CNC_HttpAuthResponse_s
{
	char* name; /* null terminated */
	char* pwd;  /* null terminated */

	/* function pointer to release the memory for the object */
	void (*release) (struct CNC_HttpAuthResponse_s *ref);
} CNC_HttpAuthResponse_t;

typedef struct
{
	/**
	 * @brief Called when a HTTP authentification is nedded
	 *
	 * @param url,   the URL (only include path) requesting authentication, for example, "http://www.chama.ne.jp/htaccess_sample/"
	 * @param realm, the protection realm, for example "chama"
	 *
	 * @return 1 for success, response is filled; 0 on failure
	 */
	CNC_BrowserI_ReturnCode_e (*challenge)(const char* url, const char* realm, CNC_HttpAuthResponse_t* response);

	/**
	 * @brief read saved basic authentication.
	 *
	 * @param url,   the URL (only include path) requesting authentication, for example, "http://www.chama.ne.jp/htaccess_sample/"
	 * @param realm, the protection realm, for example "chama"
	 *
	 * @return 1 for success, response is filled; 0 on failure
	 */
	int (*read)(const char *url, const char* realm, CNC_HttpAuthResponse_t* response);

	/**
	 * @brief save the basic authentication.
	 *
	 * @param url,    the URL (only include path) for authentication, for example, "http://www.chama.ne.jp/htaccess_sample/"
	 * @param realm,  the protection realm, for example "chama"
	 * @param userID, null terminated
	 * @param pwd,    null terminated
	 *
	 * @return 1 for success; 0 on failure
	 */
	int (*write)(const char *url, const char* realm, const char* userID, const char* pwd);

	/**
	 * @brief clear the saved authentication.
	 *
	 * @param url, the URL (only include path) for authentication, for example, "http://www.chama.ne.jp/htaccess_sample/"
	 * @param realm, the protection realm, for example "chama"
	 *
	 * @return 1 for success; 0 on failure
	 */
	int (*remove)(const char *url, const char* realm);

} CNC_BrowserHTTPAuthenticationCallbacks_t;

/**
 * @brief Register HTTP authentication call back functions
 * @param cb Callback structure
 */
void CNC_Browser_RegisterHTTPAuthenticationCallbacks(CNC_BrowserHTTPAuthenticationCallbacks_t *cb);


typedef struct
{
	/*
	 * Callback to provide password for client private key
	 *  used in SSL client authentication.
	 *
	 * @param buf - buffer to write password into
	 * @param size - length of buffer in bytes
	 *
	 * @return length of returned password, in bytes
	 */
	int (*passwd_cb)(char* buf, int size);

} CNC_BrowserCredentialCallbacks_t;


/**
 * @brief Register the list of Plugin call back functions
 * @param cb Callback structure
 */
void CNC_Browser_RegisterCredentialCallbacks(CNC_BrowserCredentialCallbacks_t *cb);

/**
 * \brief   Register the callback used in CNC_Browser_RegisterCallbacks,
 *          CNC_Browser_RegisterHTTPAuthenticationCallbacks and CNC_Browser_RegisterCredentialCallbacks
 *
 *          This function should be call once, before browser main loop.
 */

#ifdef __cplusplus
}//end extern "C"
#endif

#endif /*__ESPAIL_BROWSER_CALLBACK_H__*/

