/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser controls operations
 * @file cnc_browserControls.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_CONTROLS_H__
#define __CNC_BROWSER_CONTROLS_H__

/* ============================= User Includes ============================ */
#include "cnc_browserCommon.h"
#include "cnc_browserGfx.h"
#include "cnc_browserEvents.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Iinitialize the browser.
 *
 * CNC_Browser_Init or CNC_Browser_Init2 must be the first call.
 *
 * @param configFileName
 */
void CNC_Browser_Init(const char* configFileName);

/**
 * Iinitialize the browser.
 *
 * CNC_Browser_Init or CNC_Browser_Init2 must be the first call.
 *
 * @param initConfigFileName initial/default configure file name
 * @param saveConfigFileName
 * 			the file to save the brower configurations,
 * 			if NULL, save to initConfigFileName
 */
void CNC_Browser_Init2(const char* initConfigFileName, const char* saveConfigFileName);


/**
 * set the browser safe area, must be called before MainLoop start.
 *
 * @param x
 * @param y
 * @param w
 * @param h
 */
void CNC_Browser_SetSafeRect(int x, int y, int w, int h);

/**
 * move the browser window and change zoom factor
 *
 * @param x
 * @param y
 * @param w
 * @param h
 * @param contentScale  zoom factor for content
 * @param uiScale  zoom factor for Html UI
 * @param doReload  0 do not reload content, otherwise, do reload
 * 			acTVila content is centered by script, When changing zoom factor, without re-executing script, the content is not centered
 * 			Google Chrome and Firefox do not reload and center content
 */
void CNC_Browser_SetWindow(int x, int y, int w, int h, float contentScale, float uiScale, int doReload);


/**
 * get the browser safe area
 */
void CNC_Browser_GetSafeRect(int* x, int* y, int* w, int* h);


/**
 * set the browser UI dialog window rectangle
 *
 * @param x
 * @param y
 * @param w
 * @param h
 */
void CNC_Browser_SetDialogBounds(int x, int y, int w, int h);


/**
 * get the browser safe area
 */
void CNC_Browser_GetDialogRect(int* x, int* y, int* w, int* h);
/**
 * set the default font size
 */
void CNC_Browser_SetDefaultFontSize(int size);

/**
 * Run the CNC TV Browser main loop.  Will not return until CNCBrowser_Exit is called.
 * @param surface
 */
void CNC_Browser_MainLoop(CNC_BrowserSurface_t surface);


/**
 * Returns the minimum stack size (in bytes) needed by the browser thread.
 */
void CNC_Browser_GetStackSize(int * out_stack_size);


/**
 * Quit the CNC TV Browser
 */
void CNC_Browser_Exit(void);


/**
 * Set the proxy host IP for HTTP connections
 */
void CNC_Browser_SetProxyHost(const char* host);


/**
 * Set the proxy port number for HTTP connections
 */
void CNC_Browser_SetProxyPort(int port);


/**
 * Set the proxy host IP for HTTPS connections, if it is not set, it uses HTTP proxy for HTTPS connections
 */
void CNC_Browser_SetHttpsProxyHost(const char* host);


/**
 * Set the proxy port number for HTTPS connections, if it is not set, it uses HTTP proxy for HTTPS connections
 */
void CNC_Browser_SetHttpsProxyPort(int port);


/**
 * Enable/disable proxy for HTTP connections
 */
void CNC_Browser_SetHttpProxyEnabled(int enabled);


/**
 * Set the comma separated list of hostnames defining no-proxy zones
 */
void CNC_Browser_AddProxyExclude(const char* domains);


/**
 * Set the comma separated list of hostnames defining no-proxy zones
 */
void CNC_Browser_SetProxyExclude(const char* domains);

/**
 * Http proxy authentication
 */
void CNC_Browser_clearProxyAuthentication(void);


/**
 * Http authentications
 */
void CNC_Browser_clearHttpAuthentications(void);

/**
 * Enabled or Disable incremental drawing capability
 * @param enabled on or off
 */
void CNC_Browser_SetIncrementalDrawEnabled(int enabled);

/**
 * Enable/disable images
 * @param enabled on or off
 */
void CNC_Browser_SetImageEnabled(int enabled);


/**
 * Enable/disable Javascript
 * @param enabled on or off
 */
void CNC_Browser_SetScriptEnabled(int enabled);


/**
 * Enable/disable SSL support
 * @param enabled on or off
 */
void CNC_Browser_SetSSLEnabled(int enabled);


/**
 * Enable/disable security popup when switching to HTTPS pages
 * @param enabled on or off
 */
void CNC_Browser_ViewSecureEnabled(int enabled);


/**
 * Enable/disable security popup when switching to non security pages
 */
void CNC_Browser_LeaveSecureForInsecureEnabled(int enabled);


/**
 * set SSL ciphers, NULL to default
 * @param ciphers SSL Ciphers
 */
void CNC_Browser_SetSSLCiphers(const char* ciphers);


/**
 * Enable or Disable scrollbar visibility
 * @param visible on or off
 */
void CNC_Browser_SetScrollbarsVisible(int visible);


/**
 * Load the desired url
 * @param url URL
 */
void CNC_Browser_GotoURL(const char* url);

/**
 * Load the startup url
 */
void CNC_Browser_loadStartUpURL(void);


/**
 * Return the CNC Browser library version
 */
const char *CNC_Browser_GetBrowserVersion(void);


/**
 * Set the user agent string
 * @param agent user agent string
 */
void CNC_Browser_SetUserAgent(const char *agent);

/**
 * Set the BML User Agent
 * @param agent user agent string
 */
void CNC_Browser_SetBMLUserAgent(const char *agent);


/**
 * Enabled or Disable 2D Navigation
 * @param enabled on or off
 */
void CNC_Browser_SetNavigationEnabled(int enabled);


/**
 * Set the default encoding
 * @param encoding encoding string
 */
void CNC_Browser_SetDefaultEncoding(const char* encoding);


/**
 * Set the size of the cache.
 * @param sizeKB Size of the cache in kilobytes
 */
void CNC_Browser_SetCacheSize(int sizeKB);


/**
 * Set the repaint delay in milliseconds.
 * @param delayMS Delay in milliseconds
 */
void CNC_Browser_SetRepaintDelay(int delayMS);

/**
 * Performs a fullscreen repaint
 */
void CNC_Browser_Repaint(void);


/**
 * If visible, close the toolbar, otherwise show it.
 */
void CNC_Browser_ToggleToolbar(void);


/**
 * Show the toolbar.
 */
void CNC_Browser_ShowToolbar(void);


/**
 * show/hide the toolbar, the toolbar is not disposed.
 */
void CNC_Browser_ToggleToolbarVisible(void);


/**
 * Close the toolbar.
 */
void CNC_Browser_CloseToolbar(void);


/**
 * Close the dialog.
 */
void CNC_Browser_CloseDialog(void);


/**
 * Loads the home page.
 */
void CNC_Browser_GoHome(void);


/**
 * Moves forward in the browser history.
 */
void CNC_Browser_GoForward(void);


/**
 * Moves backward in the browser history.
 */
void CNC_Browser_GoBack(void);

/**
 * the web content handles the key event first, if the event is not handled by the content,
 * or the default action is not prevented, then
 * 		if the focus is inside IFrame or plug-in, move the focus out;
 * 		otherwise, go back
 * @param keyEvt if null, the content handling is skipped.
 */
void CNC_Browser_NavigateOut_Or_GoBack(const CNC_BrowserKeyboardEvent_t* keyEvt);

/**
 * the web content handles the key event first, if the event is not handled by the content,
 * or the default action is not prevented, then
 * 		if the focus is inside IFrame, move the focus out;
 * 		otherwise, go back
 * @param keyEvt if null, the content handling is skipped.
 */
void CNC_Browser_NavigateOutIFrame_Or_GoBack(const CNC_BrowserKeyboardEvent_t* keyEvt);

/**
 * Reloads the current page.
 */
#ifndef USE_ESPIAL_V61
void CNC_Browser_Refresh(void);
#else
void CNC_Browser_Refresh (int endToEndReload);
#endif

/**
 * Stops the current page loading.
 */
void CNC_Browser_StopLoading(void);


/**
 * Clears the cache.
 */
void CNC_Browser_ClearCache(void);


/**
 * @brief Enable/Disable cookies.
 *
 * This sets the user's setting, regardless of whether the setting is
 * being overridden.
 *
 * @param enabled
 */
void CNC_Browser_SetCookiesEnabled(int enabled);

/**
 * @brief Enable/disable override cookie on pages.
 *
 * If set, the user's setting is ignored, and cookies are enabled.
 */
void CNC_Browser_SetOverrideCookiesEnabled(int enabled);


/**
 * @brief Enable/disable CSS.
 * @param enabled
 */
void CNC_Browser_SetCSSEnabled(int enabled);


/**
 * @brief Enable/disable popup windows (window.open)
 * @param enabled
 */
void CNC_Browser_SetPopupEnabled(int enabled);


/**
 * @brief Enable/disable warning on opening target window
 * @param enabled
 */
void CNC_Browser_SetTargetWindowWarningEnabled(int enabled);

/**
 * @brief Clears the cookies.
 */
void CNC_Browser_ClearCookies(void);

/**
 * @brief Save the cookies.
 */
void CNC_Browser_SaveCookies();

/**
 * @brief Set one cookie.
 * @param domain The domain this cookie belongs to, if no domain was given in the line, set the default
 * @param path the path within the domain
 * @param line The cookie value pair
 * @param fixed  If true, the cookie will not expire and can not be deleted
 */
void CNC_Browser_SetCookie( const char* domain, const char* path, const char* line, int fixed);


/**
 * @brief Delete one cookie.
 * @param domain The domain this cookie belongs to
 * @param path the path within the domain
 * @param name The cookie name
 */
void CNC_Browser_DeleteCookie( const char* domain, const char* path, const char* name);


/**
 * Set the browser mode.
 * @param mode Web or Kiosk mode
 */
void CNC_Browser_SetMode(CNC_BrowserMode_t mode);


/**
 * Set the zoom factor, this applis to both content view and UI views.
 * @param scale scale factor, a value of 1.0 is default
 */
void CNC_Browser_SetZoomFactor(float scale);


/**
 * Returns the currently set zoom factor
 */
float CNC_Browser_GetZoomFactor(void);


/**
 * Set the zoom factor for all webviews except for content one
 * @param scale scale factor, a value of 1.0 is default
 */
void CNC_Browser_SetUIZoomFactor(float scale);


/**
 * Returns the currently set zoom factor for all webviews except for content one
 */
float CNC_Browser_GetUIZoomFactor(void);


/**
 * Set the zoom factor for content webview.
 * @param scale scale factor, a value of 1.0 is default
 */
void CNC_Browser_SetContentZoomFactor(float scale);


/**
 * Returns the currently set zoom factor for content webview
 */
float CNC_Browser_GetContentZoomFactor(void);


/**
 * Clears the browser surface and stops painting.  A call to the blit callback function is made in order to process the clear.
 */
void CNC_Browser_Hide(void);


/**
 * Resumes painting.  A repaint is also called to ensure the entire screen is painted correctly.
 */
void CNC_Browser_Show(void);


/**
 * Sets the text of the currently focused input element.
 */
void CNC_Browser_SetInputText(CNC_BrowserString_t *string);

/**
 * notify the native soft keyboard (invoked by textInput callback function) is closed
 */
void CNC_Browser_CancelInputText(void);

/**
 *       : Sets the home page
 * @attention   : Currently Home page is set to Empty html file, Change this only if you really need.
 */
void CNC_Browser_SetHomeURL(const char* url);

/**
 * Sets the HTML UI base directory
 *
 * must be set before the main loop start
 */
void CNC_Browser_SetUIBaseDir(const char *base_dir);

/**
 * Sets the name of the directory to append to the HtmlUI directory.
 */
void CNC_Browser_SetUILanguageDir(const char *dir);

/**
 * Sets the HTML UI language
 *
 * Language Codes (2 Letters), http://www.abbreviations.com/acronyms/LANGUAGES2L
 * for example, 'EN' english, 'FR' french, 'ES' Spanish
 */
void CNC_Browser_SetUILanguage(const char *language);

/**
 * Set the User country
 */
void CNC_Browser_SetUserCountry(const char *country);

typedef enum
{
    CNC_Browser_HTTP_VERSION_NONE, /* setting this means we don't care, and that we'd
                             			like the library to choose the best possible
                             			for us! */
    CNC_Browser_HTTP_VERSION_1_1,  /* please use HTTP 1.1 in the request */
    CNC_Browser_HTTP_VERSION_1_0,  /* please use HTTP 1.0 in the request */
    CNC_Browser_HTTP_VERSION_LAST /* *ILLEGAL* http version */
} CNCBrowser_HTTPVersion_e;;

/**
 * Set the HTTP version to be used for future connections
 * @param version int The HTTP version to be set
 *        0 The HTTP engine will try HTTP 1.1 and fall back to 1.0 depending on the connections
 *        1 The HTTP engine uses HTTP 1.1 in the request
 *        2 The HTTP engine uses HTTP 1.0 in the request
 */
void CNC_Browser_SetHttpVersion(CNCBrowser_HTTPVersion_e version);


/**
 * Get the HTTP version currently being used
 * @return The previously set HTTP version
 */
CNCBrowser_HTTPVersion_e CNC_Browser_GetHttpVersion(void);


typedef enum
{
    CNC_Browser_SSLVERSION_DEFAULT,
    CNC_Browser_SSLVERSION_TLSv1,
    CNC_Browser_SSLVERSION_SSLv2,
    CNC_Browser_SSLVERSION_SSLv3,
    CNC_Browser_SSLVERSION_LAST /* never use, keep last */
} CNC_Browser_SSLVersion_e;

/**
 * Set the SSL version
 * @param version The SSL version to be used
 *        0 default, the SSL handshaking determines which of the three versions will be used
 *        1 TLS V1, the other versions are disabled
 *        2 SSL V2, the other versions are disabled
 *        3 SSL V3, the other versions are disabled
 */
void CNC_Browser_SetSSLVersion(CNC_Browser_SSLVersion_e version);


/**
 * Get the SSL version that is previously set
 * @return The previously set SSL version or 0 if not set
 */
CNC_Browser_SSLVersion_e CNC_Browser_GetSSLVersion(void);

typedef enum
{
    CNC_Browser_DNS_Resolver_Unspecified, /* try IPv6 if enabled, then fall back to IPv4 */
    CNC_Browser_DNS_Resolver_IPv4,
    CNC_Browser_DNS_Resolver_IPv6,
} CNC_Browser_DNS_Resolver_e;

void CNC_Browser_SetDNSResolver(CNC_Browser_DNS_Resolver_e reslover);

/**
 * Force the browser to use a fresh connection instead of re-using an existing one for new
 *        requests
 * @param enabled 1 to force fresh connections, 0 to allow reuse of existing connections
 */
void CNC_Browser_SetForceFreshConnections(int enabled);

CNC_Browser_DNS_Resolver_e CNC_Browser_GetDNSResolver(void);

/**
 * Get the list of the browser's currently available root CA certificates
 * @return An instance of CNC_BrowserCertificateList_t
 */
CNC_BrowserCertificateList_t* CNC_Browser_GetRootCertificates(void);


/**
 * Get the list of the browser's currently available server certificates
 * @return An instance of CNC_CertificateList_t
 */
CNC_BrowserCertificateList_t* CNC_Browser_GetServerCertificates(void);

/**
 * Get the list of the browser's currently available CA certificates
 * @return An instance of CNCCertificateList_t
 */
CNC_BrowserCertificateList_t* CNC_Browser_GetCACertificates(void);


/**
 * Enable or disable individual root CA certificate, this setting should be saved into a file, otherwise it takes no effect
 * @param issuer The issuer of the desired certificate
 * @param serial The serial number of this certificate issued by the issuer
 * @param enabled 1 enable 0 disable
 */
void CNC_Browser_EnableRootCertificate(const  char* issuer, const char* serial, int enabled);


/**
 * Enable or disable individual server certificate, this setting should be saved into a file, otherwise it takes no effect. NOT IMPLEMENTED YET
 * @param issuer The issuer of the desired certificate
 * @param serial The serial number of this certificate issued by the issuer
 * @param enabled 1 enable 0 disable
 */
void CNC_Browser_EnableCACertificate(const  char* issuer, const char* serial, int enabled);

/**
 * save the disabled certificates to file
 */
void CNC_Browser_SaveDisabledCertificates(void);


/**
 * Set the URL to go to at startup
 *
 * The startup URL, if set, will override the home URL from the settings file at runtime.  The startup URL must be set before calling CNC_Browser_MainLoop.
 * @param url Default URL
 */
void CNC_Browser_SetStartupURL(const char* url);

/**
 * set launching the browser, then show the bookmarks UI
 * 		Home URL or startup URL is not loaded, the content is blank
 */
void CNC_Browser_SetLaunchWithBookmarksUI(void);

/**
 * set launching the browser with the last history URL
 */
void CNC_Browser_SetLaunchWithLastHistoryURL(void);

/**
 * set launching the browser with the last visited URL
 */
void CNC_Browser_SetLaunchWithLastVisitedURL(void);

/**
 * Enable or disable Japanese hyphenation feature, NOT IMPLEMENTED YET
 * @param enabled 1 enable 0 disable
 */
void CNC_Browser_SetJpHyphenationEnabled(int enabled);

/**
 * Enable or disable wordwrap feature
 * @param enabled 1 enable 0 disable
 */
void CNC_Browser_SetWordWrapEnabled(int enabled);


void CNC_Browser_SetJpWordWrapEnabled(int enabled);

/**
 * Enable or disable mouse mode
 * @param enabled - 1 enable 0 disable
 */
void CNC_Browser_SetMouseModeEnabled(int enabled);

/**
 * toggle the browser mode.
 */
void CNC_Browser_ToggleMouseMode(void);

/**
 * scroll the content , this is an absolute scroll
 * @param ax absolute position on x
 * @param ay absolute position on y
 */
void CNC_Browser_ScrollTo(int ax, int ay);

/**
 * scroll the content
 * @param dx positive, scroll right; negative, scroll left
 * @param dy positive, scroll down; negative, scroll up
 */
void CNC_Browser_ScrollBy(int dx, int dy);


/**
 * Set the custom HTTP headers by the browsers on certain URL
 * @param headers A list of custom HTTP headers to be sent in the HTTP request
 * @param size size of the list
 * @param domain Domain Name
 */
void CNC_Browser_SetExtendHttpHeaders(const char** headers, int size, const char* domain);


/**
 * Search the given text in current page
 * @param text The given search text
 * @param matchCase Case sensitive or not
 * @return Number of found matches
 */
int CNC_Browser_StartTextSearch(CNC_BrowserString_t* text, int matchCase);


/**
 * Search the given text in forward direction
 * @param text The given search text
 * @param matchCase Case sensitive or not
 * @param wrap Whether to continue looking at the beginning after reaching the end
 * @return Ordinal number of matched item. -1 no more matched item found
 */
int CNC_Browser_FindNext(CNC_BrowserString_t* text, int matchCase, int wrap);


/**
 * Search the given text in backward direction
 * @param text The given search text
 * @param matchCase Case sensitive or not
 * @param wrap Whether to continue looking at the beginning after reaching the end
 * @return Ordinal number of matched item. -1 no more matched item found
 */
int CNC_Browser_FindPrevious(CNC_BrowserString_t* text, int matchCase, int wrap);


/**
 * Get the information of current page
 * @return An instance of CNC_Browser_PageInfo_t to be released by the release member function
 */
CNC_BrowserPageInfo_t* CNC_Browser_GetCurrentPageInfo(void);


/**
 * Get the current page's encoding
 * @return CNC_BrowserCString_t, please call release to free the allocated memory
 */
CNC_BrowserCString_t CNC_Browser_GetCurrentPageEncoding(void);


/**
 * override the current page's encoding, the content is reloaded with the specified encoding.
 * @param encoding
 */
void CNC_Browser_SetCurrentPageEncoding(const char* encoding);

/**
 * open a new window
 * @param url
 * 			null, load the home URL; otherwise, load the specified url
 */
void CNC_Browser_WindowOpen(const char* url);

/**
 * close a window
 * @param index the window number [0-]
 */
void CNC_Browser_WindowClose(int index);

/**
 * raise a window
 * @param index the window number [0-]
 */
void CNC_Browser_WindowRaise(int index);

/**
 * enable/disable Restrict Network Service
 * @param state
 * 		0 to disable (user can load open-web contents)
 * 		otherwise to enable (user can not load open-web contents)
 */
void CNC_Browser_SetRestrictNetworkService(int state);


#ifdef __cplusplus
}//end extern "C"
#endif

#endif /*__CNC_BROWSER_CONTROLS_H__*/
