/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser custom JavaScript operations
 * @file cnc_browserCustomJS.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_CUSTOM_JS_H__
#define __CNC_BROWSER_CUSTOM_JS_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"
#include "cnc_browserCommon.h"

#ifdef __cplusplus
extern "C"
{
#endif

struct CNC_BrowserJSVariant;


/**
 * CNC Browser Custom JS array
 */
typedef struct CNC_BrowserJSArray_s
{
    unsigned int length;
    struct CNC_BrowserJSVariant* array;

    /**< Pointer to a memory release function, can be null */
    void (*release)(struct CNC_BrowserJSArray_s *);
} CNC_BrowserJSArray_t;


/**
 * CNC Browser Custom JS object
 */
struct CNC_BrowserJSObjectFunctions_s;

typedef struct
{
    void* nativeInstance; /* can be null */
    struct CNC_BrowserJSObjectFunctions_s* functions; /* must be valid until the browser exit */
} CNC_BrowserJSObject_t;

/**
 * CNC Browser Custom JS variant
 */
typedef struct CNC_BrowserJSVariant
{
    CNC_BrowserType_t type;

    union
    {
        int intValue;
        double doubleValue;
        CNC_BrowserString_t strValue;
        CNC_BrowserJSArray_t arrayValue;
        CNC_BrowserUnicodeString_t unicodeValue;
        CNC_BrowserJSObject_t objValue;
    };
} CNC_BrowserJSVariant_t;

/**
 * CNC Browser Custom JS method.
 *
 * The method is called on browser main thread, the browser is blocked until this method returns.
 * It may affect browser performance.
 *
 * @param args the argument passed from JavaScript
 *      Do not change the passing argument,
 *      Do not call release function, the memory is allocated by the browser, and released by the browser.
 * @param instance if the JavaScript object is created with new, the custom object
 *          returned by the registered contructor
 * @param url the absolute url of the document that is calling this method.
 *
 * @param certVerifyOk 1 if certificate is verify ok, 0 otherwise (even when user selects OK)
 *
 * @return
 *      value to the calling scripts
 *      After the browser finished using the returned value, it calls the release function
 *      attached to the return value. If the returned value is an array, the browser only calls
 *      the release function of the array, please recursively release all array elements.
 */
typedef CNC_BrowserJSVariant_t (*CNC_BrowserJSMethod)(CNC_BrowserJSVariant_t args, void* instance, const char *url, int certVerifyOk);

/**
 * CNC Browser Custom JS Constructor.
 *
 * @param args the argument passed from JavaScript
 *      Do not change the passing argument,
 *      Do not call release function, the memory is allocated by the browser, and released by the browser.
 * @return
 *      a native instance, can be null, the browser does not intrepret it, it is directly passed back when
 *      calling registered custom method.
 */
typedef void* (*CNC_BrowserJSConstructor)(CNC_BrowserJSVariant_t args, const char *url, int certVerifyOk);

/**
 * CNC Browser Custom JS get property.
 *
 * The method is called on browser main thread, the browser is blocked until this method returns.
 * It may affect browser performance.
 *
 * @param name the property name
 * @param instance if the JavaScript object is created with new, the custom object returned by the registered contructor
 * @param url the absolute url of the document that is calling this method.
 *
 * @return
 *      value to the calling scripts
 *      After the browser finished using the returned value, it calls the release function
 *      attached to the return value. If the returned value is an array, the browser only calls
 *      the release function of the array, please recursively release all array elements.
 */
typedef CNC_BrowserJSVariant_t (*CNC_BrowserJSGetProperty)(const char* name, void* instance, const char *url, int certificateVerifyOk);


/**
 * CNC Browser Custom JS set property.
 *
 * The method is called on browser main thread, the browser is blocked until this method returns.
 * It may affect browser performance.
 *
 * @param name the property name
 * @param value the property value
 *      Do not change the passing arguments,
 *      Do not call release function, the memory is allocated by the browser, and released by the browser.
 * @param instance if the JavaScript object is created with new, the custom object returned by the registered contructor
 * @param url the absolute url of the document that is calling this method.
 *
 * @return 1 for success, 0 on failure
 */
typedef int (*CNC_BrowserJSSetProperty)(const char* name, CNC_BrowserJSVariant_t value, void* instance, const char *url, int certificateVerifyOk);

/**
 * CNC Browser Custom JS destructor.
 *
 * @param instance the native instance created by the registered contructor
 *
 * @return
 */
typedef void (*CNC_BrowserJSDestructor)(void* instance);

/**
 * @brief register custom JS object
 * @param name The name of the custom JS object, script references it by this name
 *      for example,
 *          var a = new Cavium()    if constructor is registered.
 *          Cavium.method()         if constructor is  null.
 * @param methodCount The number of cutom JS object methods
 * @param methodNames Function names to be used on javascript side
 * @param methods A pointer to custom JS object methods
 * @param constructor call be null,
 *      create native instance associated with a new JavaScript object, called when
 *          var a = new Cavium();
 * @param destructor call be null
 *      destroy the native object, called when the associated JavaScript object is destroyed.
 */
void CNC_Browser_RegisterJSObject(
        const char* name,
        unsigned int methodCount,
        const char* methodNames[],
        CNC_BrowserJSMethod methods[],
        CNC_BrowserJSConstructor constructor,
        CNC_BrowserJSDestructor destructor
        );


/**
 * @brief register custom JS object with properties
 * @param name The name of the custom JS object, script references it by this name
 * @param methodCount The number of cutom JS object methods
 * @param methodNames Function names to be used on javascript side
 * @param methods A pointer to custom JS object methods
 * @param constructor can be null
 * @param destructor can be null
 * @param getter can be null
 * @param setter can be null
 */
void CNC_Browser_RegisterJSObjectWithProperty(
        const char* name,
        unsigned int methodCount,
        const char* methodNames[],
        CNC_BrowserJSMethod methods[],
        CNC_BrowserJSConstructor constructor,
        CNC_BrowserJSDestructor destructor,
        CNC_BrowserJSGetProperty getter,
        CNC_BrowserJSSetProperty setter
        );


/**
 * @brief custom JS object functions
 * @param methodCount The number of cutom JS object methods
 * @param methodNames Function names to be used on javascript side
 * @param methods A pointer to custom JS object methods
 * @param destructor can be null
 * @param getter can be null
 * @param setter can be null
 */
struct CNC_BrowserJSObjectFunctions_s
{
    unsigned int methodCount;
    const char** methodNames;
    CNC_BrowserJSMethod* methods;
    CNC_BrowserJSDestructor destructor;
    CNC_BrowserJSGetProperty getter;
    CNC_BrowserJSSetProperty setter;
};

/**
 * @brief register Input Method custom JS object
 *
 * @param name - The name of the custom JS object, script references it by this name
 *      for example,
 *          var a = new Cavium()    if constructor is registered.
 *          Cavium.method()         if constructor is  null.
 * @param methodCount The number of cutom JS object methods
 * @param methodNames Function names to be used on javascript side
 * @param methods A pointer to custom JS object methods
 * @param constructor call be null,
 *      create native instance associated with a new JavaScript object, called when
 *          var a = new Cavium();
 * @param destructor call be null
 *      destroy the native object, called when the associated JavaScript object is destroyed.
 */
void CNC_Browser_RegisterInputMethodJSObject(
        const char* name,
        unsigned int methodCount,
        const char* methodNames[],
        CNC_BrowserJSMethod methods[],
        CNC_BrowserJSConstructor constructor,
        CNC_BrowserJSDestructor destructor
        );

/**
 * @brief deregister custom JS object
 * @param name The name of the custom JS object, script references it by this name
 */
void CNC_Browser_DeregisterJSObject(const char* name);


/**
 * @brief register custom JS for plugin
 * @param mime
 * @param methodCount The number of cutom JS object methods
 * @param methodNames Function names to be used on javascript side
 * @param methods A pointer to custom JS object methods
 * @param constructor can be null
 * @param destructor can be null
 * @param getter can be null
 * @param setter can be null
 */
void CNC_Browser_RegisterPluginJSObject(
        const char* mime,
        unsigned int methodCount,
        const char* methodNames[],
        CNC_BrowserJSMethod methods[],
        CNC_BrowserJSConstructor constructor,
        CNC_BrowserJSDestructor destructor,
        CNC_BrowserJSGetProperty getter,
        CNC_BrowserJSSetProperty setter
        );

/**
 * @brief deregister custom JS for none-visual plugin
 * @param mime
 */
void CNC_Browser_DeregisterPluginJSObject(const char* mime);

/**
* @brief dispatch a custom JS event to registered event listeners
*   the custom event listener can be registered with
*
function handleCustomEvent(eventName, args)
{
    Cavium.log("handleCustomEvent(" + eventName +")");

    if(args){
        Cavium.log(args);
    }
}

Cavium.onCustomEvent = handleCustomEvent;
*
* @param eventName the event name, for example "powerOff"
* @param args the arguments passed to the event listeners (memory is freed by the attached release function)
* @param toUI 0, not fired on UI; otherwise, send to UI
* @param toContent 0, not fired on content; otherwise, send to content
*
*/
void CNC_Browser_dispatchCustomJSEvent(const char* eventName, CNC_BrowserJSVariant_t args, int toUI, int toContent);

#ifdef __cplusplus
}//end extern "C"
#endif

#endif /*__CNC_BROWSER_CUSTOM_JS_H__*/
