/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser event operations
 * @file cnc_browserEvents.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_EVENTS_H__
#define __CNC_BROWSER_EVENTS_H__

#ifdef __cplusplus
extern "C"
{
#endif

/*! General keyboard/mouse state definitions */
enum
{
    CNC_Browser_RELEASED, CNC_Browser_PRESSED
};

/*! Mouse button index */
enum
{
    CNC_Browser_BUTTON_LEFT = 1,
    CNC_Browser_BUTTON_MIDDLE,
    CNC_Browser_BUTTON_RIGHT
};

/*! Event enumerations */
enum
{
    CNC_Browser_NOEVENT = 0,    /**< Unused (do not remove) */
    CNC_Browser_KEYDOWN,        /**< Keys pressed */
    CNC_Browser_KEYUP,          /**< Keys released */
    CNC_Browser_MOUSEMOTION,    /**< Mouse moved */
    CNC_Browser_MOUSEBUTTONDOWN,/**< Mouse button pressed */
    CNC_Browser_MOUSEBUTTONUP,  /**< Mouse button released */
    CNC_Browser_MOUSEWHEEL,     /**< Mouse wheel motion */
    CNC_Browser_QUIT,            /**< User-requested quit */
    CNC_Browser_EXEC_FUNCTION,
/**< execute the function on browser thread */
};

/* Keyboard event source */
enum
{
    CNC_BrowserKeyboardEvent_USB, CNC_BrowserKeyboardEvent_IR,
};

/*! Keyboard event structure */
typedef struct
{
    unsigned int type;      /**< CNC_Browser_KEYDOWN or CNC_Browser_KEYUP */
    unsigned int scancode;  /**< Hardware key code */
    unsigned int sym;       /**< Virtual key code. See CNC_BrowserKey_t for details */
    unsigned int mod;       /**< current key modifiers */
    unsigned int unicode;
    unsigned int source;    /**< event source */
} CNC_BrowserKeyboardEvent_t;

/*
 * Mouse motion event structure
 * if x and y are 0, use xrel and yrel
 */
typedef struct
{
    unsigned int type;      /**< CNC_Browser_MOUSEMOTION */
    unsigned int state;     /**< The current button state */
    int x;                  /**< X coordinate, relative to window */
    int y;                  /**< Y coordinate, relative to window */
    int cursor;             /**< The cursor being used in the event */
    int xrel;               /**< The relative motion in the X direction */
    int yrel;               /**< The relative motion in the Y direction */
} CNC_BrowserMouseMotionEvent_t;

/*
 * Mouse button event stucture
 * if x and y are 0, use the browser current cursor position
 */
typedef struct
{
    unsigned int type;      /**< CNC_Browser_MOUSEBUTTONDOWN or CNC_Browser_MOUSEBUTTONUP */
    unsigned int button;    /**< The mouse button index */
    unsigned int state;     /**< CNC_Browser_PRESSED or CNC_Browser_RELEASED */
    int x;                  /**< X coordinate, relative to window */
    int y;                  /**< Y coordinate, relative to window */
} CNC_BrowserMouseButtonEvent_t;

/*
 * Mouse wheel event stucture
 * if x and y are 0, use the browser current cursor position
 */
typedef struct
{
    unsigned int type;  /**< CNC_Browser_MOUSEWHEEL */
    int xticks;         /**< Horizontal wheel ticks */
    int yticks;         /**< Vertical wheel ticks */
    int x;              /**< X coordinate, relative to window */
    int y;              /**< Y coordinate, relative to window */
} CNC_BrowserMouseWheelEvent_t;

/*! Browser execute function event.  Happens on the browser thread. */
typedef struct
{
    unsigned int type;
    void (*func) (void* param);
    void *param;
} CNC_BrowserExecFunctionEvent_t;

/*! General event structure */
typedef union
{
    unsigned int type;                      /**< Event type, shared with all events */
    CNC_BrowserKeyboardEvent_t key;         /**< Keyboard event data */
    CNC_BrowserMouseMotionEvent_t motion;   /**< Mouse motion event data */
    CNC_BrowserMouseButtonEvent_t button;   /**< Mouse button event data */
    CNC_BrowserMouseWheelEvent_t wheel;     /**< Mouse wheel event data */
    CNC_BrowserExecFunctionEvent_t exec;    /**< execute function event data */
} CNC_BrowserEvent_t;

/**
 * @brief  Add an event to the event queue.
 * @param event
 * @return 1 if successful, 0 if failed
 */
int CNC_Browser_PushEvent (CNC_BrowserEvent_t * event);

/* What we really want is a mapping of every raw key on the keyboard.
 To support international keyboards, we use the range 0xA1 - 0xFF
 as international virtual keycodes.  We'll follow in the footsteps of X11...
 The names of the keys
 */
/*! Virtual key mapping */
typedef enum
{
    /* The keyboard syms have been cleverly chosen to map to ASCII */
    CNC_Browser_KEY_UNKNOWN     = 0,
    CNC_Browser_KEY_FIRST       = 0,
    CNC_Browser_KEY_BACKSPACE   = 8,
    CNC_Browser_KEY_TAB         = 9,
    CNC_Browser_KEY_CLEAR       = 12,
    CNC_Browser_KEY_RETURN      = 13,
    CNC_Browser_KEY_ENTER       = 13,
    CNC_Browser_KEY_PAUSE       = 19,
    CNC_Browser_KEY_ESCAPE      = 27,
    CNC_Browser_KEY_SPACE       = 32,
    CNC_Browser_KEY_EXCLAIM     = 33,
    CNC_Browser_KEY_QUOTEDBL    = 34,
    CNC_Browser_KEY_HASH        = 35,
    CNC_Browser_KEY_DOLLAR      = 36,
    CNC_Browser_KEY_PERCENT     = 37,
    CNC_Browser_KEY_AMPERSAND   = 38,
    CNC_Browser_KEY_QUOTE       = 39,
    CNC_Browser_KEY_LEFTPAREN   = 40,
    CNC_Browser_KEY_RIGHTPAREN  = 41,
    CNC_Browser_KEY_ASTERISK    = 42,
    CNC_Browser_KEY_PLUS        = 43,
    CNC_Browser_KEY_COMMA       = 44,
    CNC_Browser_KEY_MINUS       = 45,
    CNC_Browser_KEY_PERIOD      = 46,
    CNC_Browser_KEY_SLASH       = 47,
    CNC_Browser_KEY_0           = 48,
    CNC_Browser_KEY_1           = 49,
    CNC_Browser_KEY_2           = 50,
    CNC_Browser_KEY_3           = 51,
    CNC_Browser_KEY_4           = 52,
    CNC_Browser_KEY_5           = 53,
    CNC_Browser_KEY_6           = 54,
    CNC_Browser_KEY_7           = 55,
    CNC_Browser_KEY_8           = 56,
    CNC_Browser_KEY_9           = 57,
    CNC_Browser_KEY_COLON       = 58,
    CNC_Browser_KEY_SEMICOLON   = 59,
    CNC_Browser_KEY_LESS        = 60,
    CNC_Browser_KEY_EQUALS      = 61,
    CNC_Browser_KEY_GREATER     = 62,
    CNC_Browser_KEY_QUESTION    = 63,
    CNC_Browser_KEY_AT          = 64,

    CNC_Browser_KEY_A = 65,
    CNC_Browser_KEY_B = 66,
    CNC_Browser_KEY_C = 67,
    CNC_Browser_KEY_D = 68,
    CNC_Browser_KEY_E = 69,
    CNC_Browser_KEY_F = 70,
    CNC_Browser_KEY_G = 71,
    CNC_Browser_KEY_H = 72,
    CNC_Browser_KEY_I = 73,
    CNC_Browser_KEY_J = 74,
    CNC_Browser_KEY_K = 75,
    CNC_Browser_KEY_L = 76,
    CNC_Browser_KEY_M = 77,
    CNC_Browser_KEY_N = 78,
    CNC_Browser_KEY_O = 79,
    CNC_Browser_KEY_P = 80,
    CNC_Browser_KEY_Q = 81,
    CNC_Browser_KEY_R = 82,
    CNC_Browser_KEY_S = 83,
    CNC_Browser_KEY_T = 84,
    CNC_Browser_KEY_U = 85,
    CNC_Browser_KEY_V = 86,
    CNC_Browser_KEY_W = 87,
    CNC_Browser_KEY_X = 88,
    CNC_Browser_KEY_Y = 89,
    CNC_Browser_KEY_Z = 90,

    CNC_Browser_KEY_LEFTBRACKET = 91,
    CNC_Browser_KEY_BACKSLASH   = 92,
    CNC_Browser_KEY_RIGHTBRACKET= 93,
    CNC_Browser_KEY_CARET       = 94,
    CNC_Browser_KEY_UNDERSCORE  = 95,
    CNC_Browser_KEY_BACKQUOTE   = 96,
    CNC_Browser_KEY_a           = 97,
    CNC_Browser_KEY_b           = 98,
    CNC_Browser_KEY_c           = 99,
    CNC_Browser_KEY_d           = 100,
    CNC_Browser_KEY_e           = 101,
    CNC_Browser_KEY_f           = 102,
    CNC_Browser_KEY_g           = 103,
    CNC_Browser_KEY_h           = 104,
    CNC_Browser_KEY_i           = 105,
    CNC_Browser_KEY_j           = 106,
    CNC_Browser_KEY_k           = 107,
    CNC_Browser_KEY_l           = 108,
    CNC_Browser_KEY_m           = 109,
    CNC_Browser_KEY_n           = 110,
    CNC_Browser_KEY_o           = 111,
    CNC_Browser_KEY_p           = 112,
    CNC_Browser_KEY_q           = 113,
    CNC_Browser_KEY_r           = 114,
    CNC_Browser_KEY_s           = 115,
    CNC_Browser_KEY_t           = 116,
    CNC_Browser_KEY_u           = 117,
    CNC_Browser_KEY_v           = 118,
    CNC_Browser_KEY_w           = 119,
    CNC_Browser_KEY_x           = 120,
    CNC_Browser_KEY_y           = 121,
    CNC_Browser_KEY_z               = 122,
    CNC_Browser_KEY_LCURLYBRACKET   = 123,
    CNC_Browser_KEY_VBAR            = 124,
    CNC_Browser_KEY_RCURLYBRACKET   = 125,
    CNC_Browser_KEY_TILDE           = 126,
    CNC_Browser_KEY_DELETE          = 127,
    /* End of ASCII mapped keysyms */

    /* International keyboard syms */
    CNC_Browser_KEY_WORLD_0  = 160, /* 0xA0 */
    CNC_Browser_KEY_WORLD_1  = 161,
    CNC_Browser_KEY_WORLD_2  = 162,
    CNC_Browser_KEY_WORLD_3  = 163,
    CNC_Browser_KEY_WORLD_4  = 164,
    CNC_Browser_KEY_WORLD_5  = 165,
    CNC_Browser_KEY_WORLD_6  = 166,
    CNC_Browser_KEY_WORLD_7  = 167,
    CNC_Browser_KEY_WORLD_8  = 168,
    CNC_Browser_KEY_WORLD_9  = 169,
    CNC_Browser_KEY_WORLD_10 = 170,
    CNC_Browser_KEY_WORLD_11 = 171,
    CNC_Browser_KEY_WORLD_12 = 172,
    CNC_Browser_KEY_WORLD_13 = 173,
    CNC_Browser_KEY_WORLD_14 = 174,
    CNC_Browser_KEY_WORLD_15 = 175,
    CNC_Browser_KEY_WORLD_16 = 176,
    CNC_Browser_KEY_WORLD_17 = 177,
    CNC_Browser_KEY_WORLD_18 = 178,
    CNC_Browser_KEY_WORLD_19 = 179,
    CNC_Browser_KEY_WORLD_20 = 180,
    CNC_Browser_KEY_WORLD_21 = 181,
    CNC_Browser_KEY_WORLD_22 = 182,
    CNC_Browser_KEY_WORLD_23 = 183,
    CNC_Browser_KEY_WORLD_24 = 184,
    CNC_Browser_KEY_WORLD_25 = 185,
    CNC_Browser_KEY_WORLD_26 = 186,
    CNC_Browser_KEY_WORLD_27 = 187,
    CNC_Browser_KEY_WORLD_28 = 188,
    CNC_Browser_KEY_WORLD_29 = 189,
    CNC_Browser_KEY_WORLD_30 = 190,
    CNC_Browser_KEY_WORLD_31 = 191,
    CNC_Browser_KEY_WORLD_32 = 192,
    CNC_Browser_KEY_WORLD_33 = 193,
    CNC_Browser_KEY_WORLD_34 = 194,
    CNC_Browser_KEY_WORLD_35 = 195,
    CNC_Browser_KEY_WORLD_36 = 196,
    CNC_Browser_KEY_WORLD_37 = 197,
    CNC_Browser_KEY_WORLD_38 = 198,
    CNC_Browser_KEY_WORLD_39 = 199,
    CNC_Browser_KEY_WORLD_40 = 200,
    CNC_Browser_KEY_WORLD_41 = 201,
    CNC_Browser_KEY_WORLD_42 = 202,
    CNC_Browser_KEY_WORLD_43 = 203,
    CNC_Browser_KEY_WORLD_44 = 204,
    CNC_Browser_KEY_WORLD_45 = 205,
    CNC_Browser_KEY_WORLD_46 = 206,
    CNC_Browser_KEY_WORLD_47 = 207,
    CNC_Browser_KEY_WORLD_48 = 208,
    CNC_Browser_KEY_WORLD_49 = 209,
    CNC_Browser_KEY_WORLD_50 = 210,
    CNC_Browser_KEY_WORLD_51 = 211,
    CNC_Browser_KEY_WORLD_52 = 212,
    CNC_Browser_KEY_WORLD_53 = 213,
    CNC_Browser_KEY_WORLD_54 = 214,
    CNC_Browser_KEY_WORLD_55 = 215,
    CNC_Browser_KEY_WORLD_56 = 216,
    CNC_Browser_KEY_WORLD_57 = 217,
    CNC_Browser_KEY_WORLD_58 = 218,
    CNC_Browser_KEY_WORLD_59 = 219,
    CNC_Browser_KEY_WORLD_60 = 220,
    CNC_Browser_KEY_WORLD_61 = 221,
    CNC_Browser_KEY_WORLD_62 = 222,
    CNC_Browser_KEY_WORLD_63 = 223,
    CNC_Browser_KEY_WORLD_64 = 224,
    CNC_Browser_KEY_WORLD_65 = 225,
    CNC_Browser_KEY_WORLD_66 = 226,
    CNC_Browser_KEY_WORLD_67 = 227,
    CNC_Browser_KEY_WORLD_68 = 228,
    CNC_Browser_KEY_WORLD_69 = 229,
    CNC_Browser_KEY_WORLD_70 = 230,
    CNC_Browser_KEY_WORLD_71 = 231,
    CNC_Browser_KEY_WORLD_72 = 232,
    CNC_Browser_KEY_WORLD_73 = 233,
    CNC_Browser_KEY_WORLD_74 = 234,
    CNC_Browser_KEY_WORLD_75 = 235,
    CNC_Browser_KEY_WORLD_76 = 236,
    CNC_Browser_KEY_WORLD_77 = 237,
    CNC_Browser_KEY_WORLD_78 = 238,
    CNC_Browser_KEY_WORLD_79 = 239,
    CNC_Browser_KEY_WORLD_80 = 240,
    CNC_Browser_KEY_WORLD_81 = 241,
    CNC_Browser_KEY_WORLD_82 = 242,
    CNC_Browser_KEY_WORLD_83 = 243,
    CNC_Browser_KEY_WORLD_84 = 244,
    CNC_Browser_KEY_WORLD_85 = 245,
    CNC_Browser_KEY_WORLD_86 = 246,
    CNC_Browser_KEY_WORLD_87 = 247,
    CNC_Browser_KEY_WORLD_88 = 248,
    CNC_Browser_KEY_WORLD_89 = 249,
    CNC_Browser_KEY_WORLD_90 = 250,
    CNC_Browser_KEY_WORLD_91 = 251,
    CNC_Browser_KEY_WORLD_92 = 252,
    CNC_Browser_KEY_WORLD_93 = 253,
    CNC_Browser_KEY_WORLD_94 = 254,
    CNC_Browser_KEY_WORLD_95 = 255, /* 0xFF */

    /* Numeric keypad */
    CNC_Browser_KEY_KP0         = 256,
    CNC_Browser_KEY_KP1         = 257,
    CNC_Browser_KEY_KP2         = 258,
    CNC_Browser_KEY_KP3         = 259,
    CNC_Browser_KEY_KP4         = 260,
    CNC_Browser_KEY_KP5         = 261,
    CNC_Browser_KEY_KP6         = 262,
    CNC_Browser_KEY_KP7         = 263,
    CNC_Browser_KEY_KP8         = 264,
    CNC_Browser_KEY_KP9         = 265,
    CNC_Browser_KEY_KP_PERIOD   = 266,
    CNC_Browser_KEY_KP_DIVIDE   = 267,
    CNC_Browser_KEY_KP_MULTIPLY = 268,
    CNC_Browser_KEY_KP_MINUS    = 269,
    CNC_Browser_KEY_KP_PLUS     = 270,
    CNC_Browser_KEY_KP_ENTER    = 271,
    CNC_Browser_KEY_KP_EQUALS   = 272,

    /* Arrows + Home/End pad */
    CNC_Browser_KEY_UP          = 273,
    CNC_Browser_KEY_DOWN        = 274,
    CNC_Browser_KEY_RIGHT       = 275,
    CNC_Browser_KEY_LEFT        = 276,
    CNC_Browser_KEY_INSERT      = 277,
    CNC_Browser_KEY_HOME        = 278,
    CNC_Browser_KEY_END         = 279,
    CNC_Browser_KEY_PAGEUP      = 280,
    CNC_Browser_KEY_PAGEDOWN    = 281,

    /* Function keys */
    CNC_Browser_KEY_F1  = 282,
    CNC_Browser_KEY_F2  = 283,
    CNC_Browser_KEY_F3  = 284,
    CNC_Browser_KEY_F4  = 285,
    CNC_Browser_KEY_F5  = 286,
    CNC_Browser_KEY_F6  = 287,
    CNC_Browser_KEY_F7  = 288,
    CNC_Browser_KEY_F8  = 289,
    CNC_Browser_KEY_F9  = 290,
    CNC_Browser_KEY_F10 = 291,
    CNC_Browser_KEY_F11 = 292,
    CNC_Browser_KEY_F12 = 293,
    CNC_Browser_KEY_F13 = 294,
    CNC_Browser_KEY_F14 = 295,
    CNC_Browser_KEY_F15 = 296,

    /* Key state modifier keys */
    CNC_Browser_KEY_NUMLOCK     = 300,
    CNC_Browser_KEY_CAPSLOCK    = 301,
    CNC_Browser_KEY_SCROLLOCK   = 302,
    CNC_Browser_KEY_RSHIFT      = 303,
    CNC_Browser_KEY_LSHIFT      = 304,
    CNC_Browser_KEY_RCTRL       = 305,
    CNC_Browser_KEY_LCTRL       = 306,
    CNC_Browser_KEY_RALT        = 307,
    CNC_Browser_KEY_LALT        = 308,
    CNC_Browser_KEY_RMETA       = 309,
    CNC_Browser_KEY_LMETA       = 310,
    CNC_Browser_KEY_LSUPER      = 311,   /* Left "Windows" key */
    CNC_Browser_KEY_RSUPER      = 312,   /* Right "Windows" key */
    CNC_Browser_KEY_MODE        = 313,     /* "Alt Gr" key */
    CNC_Browser_KEY_COMPOSE     = 314,  /* Multi-key compose key */

    /* Miscellaneous function keys */
    CNC_Browser_KEY_HELP    = 315,
    CNC_Browser_KEY_PRINT   = 316,
    CNC_Browser_KEY_SYSREQ  = 317,
    CNC_Browser_KEY_BREAK   = 318,
    CNC_Browser_KEY_MENU    = 319,
    CNC_Browser_KEY_POWER   = 320,    /* Power Macintosh power key */
    CNC_Browser_KEY_EURO    = 321,     /* Some european keyboards */
    CNC_Browser_KEY_UNDO    = 322,     /* Atari keyboard has Undo */

    /*color buttons*/
    CNC_Browser_KEY_BLUE    = 323,
    CNC_Browser_KEY_RED     = 324,
    CNC_Browser_KEY_GREEN   = 325,
    CNC_Browser_KEY_YELLOW  = 326,

    /*color buttons*/
    CNC_Browser_KEY_CHANNELUP   = 330,
    CNC_Browser_KEY_CHANNELDOWN = 331,

    /* Remote Control buttons */
    CNC_Browser_KEY_INPUT       = 401,    /* IR period or line-break, or special */
    CNC_Browser_KEY_10          = 402,       /* IR key 10 */
    CNC_Browser_KEY_11          = 403,       /* IR key 11 */
    CNC_Browser_KEY_12          = 404,       /* IR key 12 */
    CNC_Browser_KEY_PAGELEFT    = 405, /* IR key page left */
    CNC_Browser_KEY_PAGERIGHT   = 406,/* IR key page right */
    CNC_Browser_KEY_SKIPLEFT    = 407, /* IR key skip left */
    CNC_Browser_KEY_SKIPRIGHT   = 408,/* IR key skip right */

    CNC_Browser_KEY_SEARCH      = 411,   /* key search */
    CNC_Browser_KEY_RECORD      = 412,   /* key record */
    CNC_Browser_KEY_STOP        = 413,     /* key stop */
    CNC_Browser_KEY_PLAY        = 414,     /* key play */
    CNC_Browser_KEY_FORWARD     = 415,  /* key forward */
    CNC_Browser_KEY_REWIND      = 416,   /* key rewind/backward */

    CNC_Browser_KEY_EXIT        = 420, /* key Exit */
    CNC_Browser_KEY_TOOLS       = 421, /* key Tools */
    CNC_Browser_KEY_BOOKMARKS   = 422, /* key Bookmarks */

    CNC_Browser_KEY_LAST        = 999,
} CNC_BrowserKey_t;

/*! Enumeration of valid key mods (possibly OR'd together) */
typedef enum
{
    CNC_Browser_KMOD_NONE   = 0x0000,
    CNC_Browser_KMOD_LSHIFT = 0x0001,
    CNC_Browser_KMOD_RSHIFT = 0x0002,
    CNC_Browser_KMOD_LCTRL  = 0x0040,
    CNC_Browser_KMOD_RCTRL  = 0x0080,
    CNC_Browser_KMOD_LALT   = 0x0100,
    CNC_Browser_KMOD_RALT   = 0x0200,
    CNC_Browser_KMOD_LMETA  = 0x0400,
    CNC_Browser_KMOD_RMETA  = 0x0800,
    CNC_Browser_KMOD_NUM    = 0x1000,
    CNC_Browser_KMOD_CAPS   = 0x2000,
    CNC_Browser_KMOD_MODE   = 0x4000,
} CNC_BrowserMod_t;

#define CNC_Browser_KMOD_CTRL   (CNC_Browser_KMOD_LCTRL|CNC_Browser_KMOD_RCTRL)
#define CNC_Browser_KMOD_SHIFT  (CNC_Browser_KMOD_LSHIFT|CNC_Browser_KMOD_RSHIFT)
#define CNC_Browser_KMOD_ALT    (CNC_Browser_KMOD_LALT|CNC_Browser_KMOD_RALT)
#define CNC_Browser_KMOD_META   (CNC_Browser_KMOD_LMETA|CNC_Browser_KMOD_RMETA)

#ifdef __cplusplus
}
#endif

#endif /* __CNC_BROWSER_EVENTS_H__ */

