/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser font engine operations
 * @file cnc_browserFontEngine.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_FONT_ENGINE_H__
#define __CNC_BROWSER_FONT_ENGINE_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

/** Font metrics struct */
typedef struct {
   /** font height */
   int height;
   /** font ascent */
   int ascent;
   /** font descent */
   int descent;
   /** font leading */
   int leading;
   /** the height of the x character */
   int xHeight;
   /** width of a space for this font */
   int spaceWidth;
} CNC_FontMetrics_t;

/** glyph definition */
typedef unsigned short CNC_Glyph_t;

/** platform font definition */
typedef void CNC_platform_font_t;

/** Font weight */
typedef enum {
    CNC_WEIGHT_EXTRALIGHT,
    CNC_WEIGHT_LIGHT,
    CNC_WEIGHT_BOOK,
    CNC_WEIGHT_NORMAL,
    CNC_WEIGHT_MEDIUM,
    CNC_WEIGHT_SEMIBOLD,
    CNC_WEIGHT_BOLD,
    CNC_WEIGHT_EXTRABOLD,
    CNC_WEIGHT_BLACK
} CNC_FontWeight_e;

/** Font style */
typedef enum  {
    CNC_FontItalic,
    CNC_FontNormal
} CNC_FontStyle_e;

typedef struct {

  /**
   * Create a Font
   * @param name - font name
   * @param style - font style
   * @param size - font size
   * @return the font
   */
   CNC_platform_font_t* (*createFont)(CNC_BrowserString_t* name, CNC_FontStyle_e style, CNC_FontWeight_e weight, int size);

  /**
   * Release a Font
   * @param font - font to release
   */
   void (*releaseFont)(CNC_platform_font_t* font);

  /**
   * Add the DRCS data
   * @param data - DRCS data
   * @param data_size - size of the DRCS data
   */
   void (*addDRCS)(const char* data, int data_size);

  /**
   * Clear the DRCS data
   */
   void (*clearAllDRCS)(void);

   /**
    * set the drawing clip, relative to the screen
    *
   * @param x - clip left posistion, relative to the screen
   * @param y - clip top position , relative to the screen
   * @param width - clip width
   * @param height - clip height
   *
    */
   void (*setClip)(int x, int y, int width, int height);

   /**
      * Draw text at the specified x,y position.
      * @param font - the font to draw the text
      * @param glyphs - array of glyps
      * @param x - array of x posistion of each character, relative to the screen
      * @param y - y poistion of text, relative to the screen
      * @param count - number of characters/glyphs
      * @param color - font color ARGB (8888)
      */
      void (*drawText)(CNC_platform_font_t* font, CNC_Glyph_t glyphs[], int x[], int y, int count, unsigned color);

  /**
   * Get the font metrics for the specified font
   * @param font - the font to get the font metrics for
   * @param out_font_metrics - the font metrics of specified font
   */
   void (*getFontMetrics)(CNC_platform_font_t*  font, CNC_FontMetrics_t *out_font_metrics);

   /**
    * Get the character�s glyph index
    * @param font - the platform font
    * @param ch - the unicode character
    * @return The glyph for the unicode character of the specified font
    */
    CNC_Glyph_t (*getUnicodeCharIndex)(CNC_platform_font_t*  font, unsigned short unicode);


  /**
   * return 1 if the Font contains the character
   * @param font - the platform font
   * @param ch - the unicode character
   * @return
   *		1: Font contains the character
   *	    0: Font does not contain the character
   */
    int (*containsUnicode)(CNC_platform_font_t*  font, unsigned short unicode);


 /**
  * return the glyph width
  * @param font - the platform font
  * @param glyph - the glyph to get the width of
  * @return The width of the glyph
  */
   int (*glyphWidth)(CNC_platform_font_t* font, CNC_Glyph_t glyph);

} CNC_FontEngine_t;

/**
 * Register the Font engine with the browser.
 * @param engine - the font engine
 */
void CNC_Browser_registerFontEngine (CNC_FontEngine_t *engine);

/**
 * \brief   Register the All font engine used in CNC_Browser_registerFontEngine,
 *          This function should be call once, before browser main loop.
 */

#ifdef __cplusplus
}
#endif

#endif //__CNC_BROWSER_FONT_ENGINE_H__
