/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser HTML5 Media Player.
 * @file cnc_browserHTML5MediaPlayerI.h
 * @ingroup web
 * An API that allows third party Media Players to be integrated into the CNC TV Browser to provide
 * HTML5 Media capabilities.\n
 * \n
 * @subpage overview\n
 * @subpage lifecycle\n
 * @subpage mapping\n
 * @subpage capabilities\n
 * @subpage methods\n
 *
 * @section overview Overview of the CNC_Browser HTML5 Media Player Porting API
 *
 * The CNC_Browser HTML5 Media Player Porting API can be broken in to two main sections: the media player
 * engine, which is responsible for registering the functions provided by the implemntor to provide the
 * functionality of the engine and player, providing information about which MIME types are supported by
 * the implementation for use in the HTML5 resource selection algorithm and for creating media player instances
 * and the media player instances, which are responsible for loading and playing the media resources.  Functions
 * that are part of the media player engine may be called before any media player instance has been created.
 *
 * The following methods can be considered to be part of the media player engine:
 *  - CNC_Browser_registerHTML5MediaPlayer: The method that registers the implementation of the CNC Browser
 *      HTML5 Media Player Porting API by filling out the CNC_HTML5MediaPlayerFunctions structure.
 *  - setCallback: The method that receives and stores the function pointer to the callback method that the
 *      media player instances will use to notify the browser of state changes and other events.
 *  - canPlayType: The method that indicates whether a given MIME type is supported by the implementation.
 *  - getSupportedTypes: The method that provides a list of all MIME types supported by the implementation.
 *  - create: The method that creates a media player instance and returns an integer identifier that will be
 *      used to refer to the media player instance in subsequent API operations.
 *  - release: The method that destroys a media player instance (identified by the identifer returned by the
 *      create method when it was initially created) and frees all resources (memory, sockets, etc.) associated
 *      with that instance.
 *
 * All of the other API methods can be considered to be part of the media player instances.
 *
 * @section lifecycle HTML5 Media Player Instance Lifecycle
 *
 * The lifecycle of a media player instance begins with a call to the create API method.  This method should return
 * an integer identifier that can be used to uniquely identify the newly created media player instance in
 * subsequent API calls.  This identifier will also be available as the "mediaPlayerID" property of the associated
 * Media (Video or Audio) element in the DOM.  The implementation of the create method should allocate any
 * resources that are not specific to the media type (which is not yet known) and may start a separate thread of
 * execution for the newly created media player (although this can also be deferred to the load API call
 * implementation.)
 *
 * The next call will be to the load method and will include the identifier for the specific media player
 * instance being addresses, the URL of the media resource to be loaded, its MIME type, an integer flag
 * to indicate whether loading should pause after loading the metadata and initial frame of the media or
 * may continue to load media data beyond the initial frame and any cookies that may be required when making
 * HTTP requests to load media resource data.  Loading should occur on a separate thread of execution, with the
 * load API method returning as soon as this process has been initiated.
 *
 * Once the media player instance has loaded the metadata for the media resource, it should set up its capability
 * flags and determine the media properties such as duration, expected number of bytes of data, native size, etc.
 * and then set its state to CNC_HTML5MediaState_HaveMetadata and call the callback method with an
 * CNC_HTML5_MEDIA_STATE_CHANGED event and the identifier of the media player instance.  If the autoLoad flag provided
 * to the load method is 1, the player may now continue to load the media resource data, otherwise, if the autoLoad
 * flag value is 0, the player should only load enough data to display the media at the current playback position.
 *
 * As soon as the player has enough data buffered to display the media at the current playback position, it should
 * set its state to CNC_HTML5MediaState_Buffering and call the callback method with an CNC_HTML5_MEDIA_STATE_CHANGED
 * event and the identifier of the media player instance to inform the browser that it has media data.
 *
 * Once the media player instance is loading the media resource data, if it encounters a situation where playback
 * gets ahead of buffering (i.e. a situation where only enough data to display the media at the current playback
 * position is buffered), it should set its state to CNC_HTML5MediaState_Stalled and call the callback method with an
 * CNC_HTML5_MEDIA_STATE_CHANGED event and the identifier of the media player instance.  When enough data to move the
 * current playback position in the direction of play is once again buffered, the media player instance should set
 * its state back to CNC_HTML5MediaState_Buffering and call the callback method with an CNC_HTML5_MEDIA_STATE_CHANGED event
 * and the identifier of the media player instance.
 *
 * If the media player instance encounters an error loading, buffering or decoding the media resource data, it
 * should set its state to the appropriate error value (CNC_HTML5MediaState_FormatError, CNC_HTML5MediaState_NetworkError,
 * or CNC_HTML5MediaState_DecodeError) and call the callback method with an CNC_HTML5_MEDIA_STATE_CHANGED event and the
 * identifier of the media player instance.
 *
 * Once enough of the media resource data is buffered to allow playback to the end of the resource without the
 * playback position overtaking the buffering, the media player instance should set its state to
 * CNC_HTML5MediaState_Loaded and call the callback method with an CNC_HTML5_MEDIA_STATE_CHANGED event and the identifier
 * of the media player instance.
 *
 * Once playback has reached the end of the media resource data, the media player instance should set its state
 * to CNC_HTML5MediaState_Ended and call the callback method with an CNC_HTML5_MEDIA_STATE_CHANGED event and the identifier
 * of the media player instance.
 *
 * If the media player instance receives a call on the seek API method, it should set the current play position to
 * the requested value on its separate thread of execution - the seek method implementation itself should
 * return as soon as this process is initiated.  Once the new play position is reached, the media player instance
 * should call the callback method with an CNC_HTML5_PLAY_POSITION_CHANGED event and the identifier of the media
 * player instance.  The CNC_HTML5_PLAY_POSITION_CHANGED event should also be signaled to the browser if the media
 * player instance encounters discontinuities in the timeline of the media resource but it is not necessary to
 * signal the CNC_HTML5_PLAY_POSITION_CHANGED event during regular playback, when the playback position is advancing
 * normally.
 *
 * It is very important that the media player implementation manage it's state and raise the appropriate events
 * to the browser at the correct junctures in order for the DOM to operate to the HTML 5 specifications.
 *
 * @section mapping Mapping CNC_HTML5MediaState_e values to HTML5 Network and Ready States
 * When the browser processes an CNC_HTML5_MEDIA_STATE_CHANGED event and retrieves the current Media
 * State value from the media player implementation, it will set the HTML 5 Ready and Media State
 * based on the retrieved Media State using the following mapping:
 * - CNC_HTML5MediaState_NoMedia:
 *  - Ready State: HaveNothing (numeric value: 0)
 *  - Network State: Empty (numeric value: 0)
 * - CNC_HTML5MediaState_HaveMetadata:
 *  - Ready State: HaveMetadata (numeric value: 1)
 *  - Network State: Loading (numeric value: 2)
 * - CNC_HTML5MediaState_Stalled:
 *  - Ready State: HaveCurrentData (numeric value: 2)
 *  - Network State: Loading (numeric value: 2)
 * - CNC_HTML5MediaState_Buffering:
 *  - Ready State: HaveFutureData (numeric value: 3)
 *  - Network State: Loading (numeric value: 2)
 * - CNC_HTML5MediaState_Loaded:
 *  - Ready State: HaveEnoughData (numeric value: 4)
 *  - Network State: Idle (numeric value: 1)
 * - CNC_HTML5MediaState_Ended:
 *  - Ready State: HaveEnoughData (numeric value: 4)
 *  - Network State: Idle (numeric value: 1)
 *  - A transistion to this state will also send an internal event that causes the DOM element to
 *    check for the end-of-stream condition.
 * - CNC_HTML5MediaState_FormatError:
 *  - Ready State: No Change
 *  - Network State: FormatError (browser internal value, numeric value: 4)
 * - CNC_HTML5MediaState_NetworkError:
 *  - Ready State: No Change
 *  - Network State: NetworkError (browser internal value, numeric value: 5)
 * - CNC_HTML5MediaState_DecodeError:
 *  - Ready State: No Change
 *  - Network State: DecodeError (browser internal value, numeric value: 6)
 *
 * The three Network State Error values (FormatError, NetworkError and DecodeError) that are indicated as being
 * browser internal values are not specified by the HTML 5 specification, but are used internally by the browser.
 * These states will cause error events to be fired and an appropriate error object to appear in the error attribute,
 * but their values will not be reflected in the networkState property.
 *
 * Note that the Idle Network State from the HTML5 specification (numeric value: 1) may also appear
 * in the element's networkState property if the user cancels a load operation, but this is handled internally by
 * the browser and does not affect this API.
 *
 * The DOM will only be notified of Ready State and/or Network State changes only if the Media
 * State change results in a true change of value - for example, moving from HTMLMediaState_Loaded
 * to CNC_HTML5MediaState_Ended does not change either the Ready State or the Network State, so the
 * DOM will only receive the internal event that triggers the end-of-stream check; moving between
 * the CNC_HTML5MediaState_HaveMetadata, CNC_HTML5MediaState_Stalled, and CNC_HTML5MediaState_Buffering states
 * changes only the Ready State, so the DOM will only be notified of the Ready State change.
 *
 * @section capabilities HTML5 Media Player and Media Resource Capabilities
 *
 * The CNC_Browser HTML5 Media Player Porting API uses a set of flags, defined in the
 * CNC_HTML5PlayerCapabilities_e and CNC_HTML5MediaCapabilities_e enumerations, to determine what the media player
 * implementation is capable of in general (the CNC_HTML5PlayerCapabilities_e enumeration) and when playing the
 * current media resource (the CNC_HTML5MediaCapabilities_e enumeration).  The media player implementation should
 * return the ORed combination of the flags from the appropriate enumeration to indicate its capabilities in the
 * getPlayerCapabilities and getMediaCapabilities API methods.  Since the Media capabilities may depend
 * on the type of media being processed, these capabilities will not be queried until the media player implementation
 * has indicated that it has loaded the metadata for a media resource.
 *
 * The supported Player Capabilities are:
 * - CNC_HTML5_PLAYER_CAP_SUPPORTS_MUTE: The media player instance supports muting of audio  in the current media resource.
 * - CNC_HTML5_PLAYER_CAP_SUPPORTS_FULLSCREEN: The media player instance can display the current media resource in full screen
 *     mode.
 * - CNC_HTML5_PLAYER_CAP_SUPPORTS_SAVE: The media player instance can save the current media resource to local storage.
 * - CNC_HTML5_PLAYER_CAP_SUPPORTS_POSTER: The media player instance can display a poster image from a URL.
 *
 * The supported Media Capabilities are:
 * - CNC_HTML5_MEDIA_CAP_HAS_VIDEO: A video stream is present in the current media resource and can be processed by this
 *    media player instance.
 * - CNC_HTML5_MEDIA_CAP_HAS_AUDIO: An audio stream is present in the current media resource and can be processed by this
 *    media player instance.
 * - HTML5_MEDIA_CAP_HAS_CLOSED_CAPTIONS: Closed Captions are present in the current media resource and can be displayed
 *     by this media player instance.
 * - CNC_HTML5_MEDIA_CAP_IS_SEEKABLE: The media player instance supports seeking in the current media resource.
 *
 * @section methods Required and Optional HTML5 Media API methods
 *
 * To simplify implementation for more limited media player implementation while still supporting more
 * sophisticated capabilities, the CNC Browser HTML5 Media Player Porting API methods that apply to the
 * media player instances are divided into Required methods, which must be implemented, and Optional methods
 * where the browser provides default behaviour if the method is not provided by the implementor.  All of the
 * methods that apply to the media player engine (rather then to individual media player instances created by
 * call to the create API method) are Required.
 *
 * The Required API methods that apply to the media player instances are:
 *  - load: The method that initiates the loading of a given media resource of a given type.
 *  - play: The method that starts or resumes playback of the loaded media resource.
 *  - pause: The method that pauses playback of the loaded media resource.
 *  - getPlayerCapabilities: The method that returns the ORed flag values from the CNC_HTML5PlayerCapabilities_e
 *      enumeration that indicate the media player's basic capabilities, regardless of the media type being
 *      processed.
 *  - getMediaCapabilities: The method that returns the ORed flag values from the CNC_HTML5MediaCapabilities_e
 *      enumeration that indicate the media player's capabilities when processing the loaded media resource.
 *  - getCurrentTime: The method that returns the current play position of the media player instance in the
 *      loaded media resource.
 *  - getDuration: The method that returns the duration of the loaded media resource.
 *  - getMediaState: The method that returns the current state of the media player implementation, using values
 *      from the CNC_HTML5MediaState_e enumeration.
 *  - setSize: The method that sets the size of the media player, in pixels.
 *
 * The Optional API methods that apply to the media player instances are:
 *  - seek: The method that initiates seeking to a given play position within the media resource.  If this method
 *      is not implemented, seek functionality will be disabled, regardless of the value of the
 *      CNC_HTML5_MEDIA_CAP_IS_SEEKABLE capability flag.  If the CNC_HTML5_MEDIA_CAP_IS_SEEKABLE is not set, this method will not
 *      be called.
 *  - getBufferedTimeRanges: The method that returns the time ranges that the media player instance has buffered.
 *      if this method is not implemented, the browser will generate an empty range (i.e from 0.0 to 0.0)
 *  - getBytesLoaded: The method that returns the number of bytes of media resource data that the media player
 *      instance has loaded.  If this method is not implemented, the browser will use a value of 0.0.
 *  - getTotalBytes: The method that returns the total number of bytes of media resource data expected by the
 *      media player instance.  If this method is not implemented, the browser will use a value of 0.0.
 *  - getNativeSize: The method that returns the preferred size of the media player instance while displaying
 *      the currently loaded media resource.  If this method is not implemented, the browser will use a value
 *      of 300 x 150 (w x h).
 *  - getMaxTimeSeekable: The method that returns the latest time value that the media player instance can seek
 *      to in the current media resource.  If this method is not implemented (and the CNC_HTML5_MEDIA_CAP_IS_SEEKABLE
 *      capability is set) the browser will use the value returned by the getDuration API call.  If the
 *      CNC_HTML5_MEDIA_CAP_IS_SEEKABLE capability is not set, this method will not be called.
 *  - getStartTime: The method that returns the start time of the current media resource.  If this method is not
 *      implemented, the browser will use a default value of 0.0.
 *  - getInitialTime: The method that returns the initial time of the current media resource.  If this method is not
 *      implemented, the browser will use a default value of 0.0.
 *  - setRate: The method that sets the playback rate for the media player instance.  If this method is not
 *      implemented, fast forward and rewind functionality will be disabled.
 *  - getRate: The method that returns the current playback rate of the media player instance.
 *  - setMuted: The method that sets the muted status of audio in the media player instance.  If this method is
 *      not implemented, mute functionality will be disabled.
 *  - getMuted: The method that returns the muted status of the media player instance.
 *  - setVolume: The method that sets the audio volume of the media player instance.  If this method is not
 *      implemented, volume adjustment functionality will be disabled.
 *  - getVolume: The method that returns the current audio volume of the media player implementation, scaled to between
 *      0.0 (silence) and 1.0 (full volume).
 *  - setClosedCaptionsVisible: The method that sets the visibility of Closed Captions in the media player instance.
 *      If this method is not implemented, Closed Caption functionality will be disabled.  If the
 *      CNC_HTML5_MEDIA_CAP_HAS_CLOSED_CAPTIONS capability is not set, this method will not be called.
 *  - setPoster: The method that sets the URL for the poster image in the media player instance.  If the
 *      CNC_HTML5_PLAYER_CAP_SUPPORTS_POSTER capability is not set, this method will not be called.
 *
 * The members in the CNC_HTML5MediaPlayerFunctions structure for Optional API methods that the implementor elects not
 * to implement must be set to NULL.
 */

#ifndef __CNC_MIDDLEWARE_HTML5_H__
#define __CNC_MIDDLEWARE_HTML5_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"
#include "cnc_browserFeatures.h"

#ifdef __cplusplus
extern "C"{
#endif

#ifdef CNC_BROWSER_ENABLE_HTML5_VIDEO

/** \enum CNC_HTML5MediaState_e
 * HTML5 Media and player state
 */
typedef enum {
	CNC_HTML5MediaState_NoMedia,	    /**< Initial State, networkState = Empty, readyState = HaveNothing */
	CNC_HTML5MediaState_HaveMetadata,	/**< Metadata loaded and Capabilities set, networkState = Loading, readyState = HaveMetadata */
	CNC_HTML5MediaState_Buffering,	    /**< Loading Media Resource, networkState = Loading, readyState = HaveFutureData */
	CNC_HTML5MediaState_Stalled,	    /**< Loading of Media Resource Stalled, networkState = Loading, readyState = HaveCurrentData */
	CNC_HTML5MediaState_Loaded,		    /**< Enough Media Resource data loaded to play to end, networkState = Idle, readyState = HaveEnoughData */
	CNC_HTML5MediaState_Ended,		    /**< Media Resource played to end of stream, states same as Loaded */
	CNC_HTML5MediaState_FormatError,	/**< Error resolving Media Resource URL or unsupported Media format */
	CNC_HTML5MediaState_NetworkError,	/**< Network Error downloading Media Resource after it has been determined to be usable */
	CNC_HTML5MediaState_DecodeError,	/**< Error decoding Media Resource data */

	CNC_HTML5MediaState_HaveEnoughData = 21, /**< networkState = NETWORK_LOADING, readyState = HAVE_ENOUGH_DATA */
} CNC_HTML5MediaState_e;

/** \enum CNC_HTML5MediaPlayerEvent_e
 * Types of HTML5 media player events.
 */
typedef enum {
	CNC_CNC_HTML5_MEDIA_STATE_CHANGED,	/**< Event indicating the state of the player or media has changed */
	CNC_HTML5_PLAY_POSITION_CHANGED,    /**< Event indicating that the play position has moved discontinuously */
	CNC_HTML5_DURATION_CHANGED,         /**< Event indicating that the media resource's duration has changed */
	CNC_HTML5_VOLUME_CHANGED,     	    /**< Event indicating that the player's volume level has changed */
	CNC_HTML5_MUTED_CHANGED,     	    /**< Event indicating that the player's mute status has changed */
	CNC_HTML5_RATE_CHANGED,     	    /**< Event indicating that the player's playback rate has changed */
} CNC_HTML5MediaPlayerEvent_e;

/** \enum CNC_HTML5MediaPlayerError_e
 * HTML5 Media player error codes.
 */
 typedef enum {
 	CNC_HTML5_MEDIAPLAYER_NOERROR, 		        /**< No Error */
 	CNC_HTML5_MEDIAPLAYER_COMMUNICATION_ERROR,	/**< Communication error */
 	CNC_HTML5_MEDIAPLAYER_INVALID_ARGS, 	    /**< Arguments are invalid */
 	CNC_HTML5_MEDIAPLAYER_BUSY, 		        /**< Player is busy and cannot process request */
 	CNC_HTML5_MEDIAPLAYER_UNKNOWN_ERROR 	    /**< Unknown error */
 } CNC_HTML5MediaPlayerError_e;

/** \enum CNC_HTML5PlayerCapabilities_e
 * HTML5 Media Player capabilities flags describing the capabilities of the Media Player
 */
typedef enum {
	CNC_HTML5_PLAYER_CAP_UNKNOWN = 0x00000000,  	        /**< Flag indicating that capabilities are not yet set */
	CNC_HTML5_PLAYER_CAP_SUPPORTS_MUTE = 0x00000001,  	    /**< Flag indicating that the player supports muting */
	CNC_HTML5_PLAYER_CAP_SUPPORTS_FULLSCREEN = 0x00000002,  /**< Flag indicating that the player supports full screen display */
	CNC_HTML5_PLAYER_CAP_SUPPORTS_SAVE = 0x00000004,  	    /**< Flag indicating that the player supports saving media */
	CNC_HTML5_PLAYER_CAP_SUPPORTS_POSTER = 0x00000008,      /**< Flag indicating that the player supports displaying a poster image */
} CNC_HTML5PlayerCapabilities_e;

/** \enum CNC_HTML5MediaCapabilities_e
 * HTML5 Media capabilities flags describing the capabilities of the current Media Resource in the Media Player
 */
typedef enum {
    CNC_HTML5_MEDIA_CAP_UNKNOWN = 0x00000000,  		   /**< Flag indicating that capabilities are not yet set */
    CNC_HTML5_MEDIA_CAP_HAS_VIDEO = 0x00000001,  	   /**< Flag indicating the presence of a video stream in the media resource */
    CNC_HTML5_MEDIA_CAP_HAS_AUDIO = 0x00000002,  	   /**< Flag indicating the presence of an audio stream in the media resource */
    CNC_HTML5_MEDIA_CAP_HAS_CLOSED_CAPTIONS = 0x00000004,  /**< Flag indicating the presence of closed captions in the media resource */
    CNC_HTML5_MEDIA_CAP_IS_SEEKABLE = 0x00000008,  	   /**< Flag indicating that the media resource and player support seeking */
} CNC_HTML5MediaCapabilities_e;

/** \struct CNC_HTML5TimeRange_t
 * Time Ranges for HTML5 Media
 */
typedef struct {
    float start;	/**< Start time of range, in seconds relative to the start of the Media Resource */
    float end;		/**< End time of the range, in seconds relative to the start of the Media Resource */
} CNC_HTML5TimeRange_t;

/** \struct CNC_HTML5TimeRangeList_t
 * A list of Time Ranges for HTML5 Media
 */
typedef struct CNC_HTML5TimeRangeList {
    int size;					          /**< Number of items in the list */
    void (*releaseItems)(struct CNC_HTML5TimeRangeList list); /**< The release method to be used by the browser to free memory used by the items member */
    CNC_HTML5TimeRange_t** items;			          /**< The list of time ranges */
} CNC_HTML5TimeRangeList_t;

/** \struct CNC_HTML5MimeType_t
 * MIME types for HTML5 Media
 */
typedef struct {
    char* type;		/**< The MIME type string */
} CNC_HTML5MimeType_t;

/** \struct CNC_HTML5SupportedMimeTypeList_t
 * A list of supported MIME types for HTML5 Media
 */
typedef struct CNC_HTML5SupportedMimeTypeList {
    int size;						          /**< Number of items in the list */
    void (*releaseItems)(struct CNC_HTML5SupportedMimeTypeList list); /**< The release method to be used by the browser to free memory used by the items member */
    CNC_HTML5MimeType_t** items;				          /**< The list of supported MIME types */
} CNC_HTML5SupportedMimeTypeList_t;

/**
 * Function prototype for HTML5 media player callback.
 * @param playerID - the id of the player generating the event
 * @param event - the event.
 */
typedef void (*CNC_HTML5MediaPlayerCallback_f)(int playerID, CNC_HTML5MediaPlayerEvent_e event);

/** \struct CNC_HTML5MediaPlayerFunctions
 * HTML5 Media Player API
 */
typedef struct {
	/**
	 * Set the media callback function
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should save the pointer to the callback function
         * and use it to notify the browser of Media state changes and events.
         * Note that this method will be called before the "create" method is called to
         * create a media player instance.
	 *
	 * @param[in] callback - the callback function.
         *
         * @see CNC_HTML5MediaPlayerCallback_f
         * @see CNC_HTML5MediaPlayerEvent_e
	 */
	void (*setCallback)(CNC_HTML5MediaPlayerCallback_f callback);

	/**
	 * Indicate whether content of a given MIME type can be played
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should return 1 if the player can process media of the given
         * MIME type or 0 if this MIME type is not supported.
         * Note that this method will be called before the "create" method is called to
         * create a media player instance.
         *
	 * @param[in] type - the MIME type to be checked
	 *
	 * @return 1 if the given MIME type is supported by the player
	 *         0 if the MIME type is not supported
	 */
	int (*canPlayType)(const char* type);

        /**
         *  Return a list of MIME types for media that the Media Player implementation
         *  can process.
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should construct and return an CNC_HTML5SupportedMimeTypeList_t
         * structure containing a list of MIME types that the player supports.  The
         * CNC_HTML5SupportedMimeTypeList_t structure includes a pointer to a method that the
         * browser will call to release the memory associated with the list items once they
         * have been processed.
         * Note that this method will be called before the "create" method is called to
         * create a media player instance.
         *
         * @return an CNC_HTML5SupportedMimeTypeList_t structure containing a list of MIME types that
         *          the player can support.
         *
         * @see CNC_HTML5SupportedMimeTypeList_t
         * @see CNC_HTML5MimeType_t
         */
        CNC_HTML5SupportedMimeTypeList_t (*getSupportedTypes)(void);

	/**
	 * Create a Media Player instance
         *
         * This is a required method that the implementor MUST implement.
	 * The ID returned by this method will be available as the value of the "mediaPlayerID"*
         * property of the associated Media (Video or Audio) element in the DOM and will
         * be used in other methods in this API to identify the specific media player instance
         * that the method is intended for.
         * Note that the browser will invoke the "release" method for any existing media player
         * instance and then invoke the "create" method to create a new instance when a load
         * operation is requested.  If the mediaPlayerID value for the released instance is re-used
         * immediately for the newly created player instance, any events still pending for the
         * original instance may be executed against the new instance.  For this reason, implementors
         * should refrain from re-using mediaPlayerID values immediately, although it is permissable to
         * reuse a value after enough time has passed to allow events associated with the previous
         * user of that value to have expired.
         * The implemenatation should perform non-media-type-specific set up in this method.
         *
	 * @return
	 *	Returns the media player id on success.
	 *	-1 on failure.
	 */
	int (*create)(void);

	/**
	 * Release a Media Player instance.
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should release all memory and other resources associated with
         * the media player instance being released.
         * This method will be called if the user cancels a load operation as well as when
         * the Media (Video or Audio) element is destroyed.  It will also be called for any existing
         * media player instances when a new load operation is requested at the DOM element.
	 *
	 * @param[in] playerID - player id to release
	 */
	void (*release)(int playerID);

	/**
	 * Load the specified media resource.
         *
         * This is a required method that the implementor MUST implement.
	 * The implemenation should start loading the metadata for the specified
         * Media Resource.  When the metadata has been loaded and the player
         * has access to duration and capability information, etc., the
         * implementation should set up its capability flags, etc., set its state
         * to "CNC_HTML5MediaState_HaveMetadata" and call the callback with the
         * "CNC_HTML5_MEDIA_STATE_CHANGED" event to notify the browser that it has
         * metadata.  If the autoLoad flag is 0, the player should only load the
         * enough media data to display the media at the current playback position.
         * If the autoLoad flag is 1, the player may load and buffer as much media
         * data as it wishes prior to being asked to play.  As soon as enough media data
         * to display the media at the current playback position is available, the player
         * should set its state to "CNC_HTML5MediaState_Buffering" and call the callback with the
         * "CNC_HTML5_MEDIA_STATE_CHANGED" event.
         * If the player ever reaches a state where it only has enough media data
         * buffered to display the media frame at the current playback position, it should
         * set it's state to "CNC_HTML5MediaState_Stalled" and call the callback with the
         * "CNC_HTML5_MEDIA_STATE_CHANGED" event.  As soon as enough data is available to advance
         * the playback position again, the player should set it's state back to "CNC_HTML5MediaState_Buffering"
         * and call the callback with the "CNC_HTML5_MEDIA_STATE_CHANGED" event to notify the browser
         * of this.
         * When enough of the media data is buffered that the playback position can be advanced to the end
         * of the media resource without overtaking the buffering, the player should set it's state to
         * "CNC_HTML5MediaState_Loaded" and call the callback with the "CNC_HTML5_MEDIA_STATE_CHANGED" event
         * to notify the browser.
         * Finally, when playback reaches the end of the media stream, the player should set its state
         * to CNC_HTML5MediaState_Ended"  and call the callback with the "CNC_HTML5_MEDIA_STATE_CHANGED" event
         * to notify the browser.
         * At any time the player may set its sstate to one of the error states (CNC_HTML5MediaState_FormatError,
         * CNC_HTML5MediaState_NetworkError or CNC_HTML5MediaState_DecodeError) and call the callback with the
         * "CNC_HTML5_MEDIA_STATE_CHANGED" event to notify the browser that an error has occurred.
         * The CNC_HTML5MediaState_FormatError state indicates that the URL for the media resource could not
         * be resolved or that the media resource itself is in an unsupported format (either due to an
         * unsupported codec or an incorrect MIME type), CNC_HTML5MediaState_NetworkError indicates a problem
         * with the network while loading the media resource data and CNC_HTML5MediaState_DecodeError indicates
         * an error in decoding and processing the media resource data.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] url - the URL of the media resource to load
	 * @param[in] type - the MIME-type of the media resource to load
	 * @param[in] autoLoad - 1 if the player should start buffering media data automatically,
         *                       0 if the player should only buffer the initial frame of media
         * @param[in] cookie - cookies for the URL connection
         *                     for example "GMAIL_RTT=294; NID=54=W2_lmYWn9Uo7YPJrmadWx; PREF=ID=6b"
	 * @return
	 *	CNC_HTML5MediaPlayerError_e
         *
         * @see CNC_HTML5MediaPlayerCallback_f
         * @see CNC_HTML5MediaState_e
         * @see CNC_HTML5MediaPlayerEvent_e
	 */
	CNC_HTML5MediaPlayerError_e (*load)(int playerID, const char* url, const char *type, int autoLoad, const char* cookie);

	/**
	 * Start or resume playback.
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should instruct the player thread to begin playback of the current Media Resource
         * at the current play position and return.  The player thread should report any errors and state
         * changes by setting it's media state appropriately and calling the callback with the "CNC_HTML5_MEDIA_STATE_CHANGED"
         * event to notify the browser.
         *
	 * @param[in] playerID - the media player id
         *
         * @see CNC_HTML5MediaPlayerCallback_f
         * @see CNC_HTML5MediaPlayerEvent_e
	 */
	void (*play)(int playerID);

	/**
	 * Pause the playback.
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should instruct the player thread to pause playback of the current Media Resource
         * and return.
         *
	 * @param[in] playerID - the media player id
	 */
	void (*pause)(int playerID);

	/**
	 * Seek to a new playback position.
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports setting the playback positon (i.e. seeking.)  If this method is not
         * implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure should be
         * set to NULL.
         * If this method is implemented, it should instruct the player thread to set the play position to the requested
         * value and return.  Once the player has reached the new playback position and is ready to enter or continue playback,
         * the player thread should call the callback with the "CNC_HTML5_PLAY_POSITION_CHANGED" event to notify the browser that
         * the seek operation is complete.
         * If a second call is made to this method while a seek is still in progress, the implementation should cancel the
         * original seek operation (or allow it to complete without sending the "CNC_HTML5_PLAY_POSITION_CHANGED" event to
         * the browser) and then seek to the location requested by the most recent call to this method.  The
         * "CNC_HTML5_PLAY_POSITION_CHANGED" event should only be sent when the seek to this new location is complete.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] pos - the new playback position (in seconds) relative to the start of the media
         *
         * @see CNC_HTML5MediaPlayerCallback_f
         * @see CNC_HTML5MediaPlayerEvent_e
	 */
	void (*seek)(int playerID, float pos);

	/**
	 * Return the capabilities of the Media Player
	 *
         * This is a required method that the implementor MUST implement.
         * The implementation should return the combination of flags from the
         * CNC_HTML5PlayerCapabilities_e enumeration, ORed together, to indicate
         * the capabilities of the media player.  This method will be invoked
         * after the "create" method has been called to create a player instance.
         *
	 * @param[in] playerID - the media player id
	 * @return The capabilities of the Media Player using flags from the
         *           CNC_HTML5PlayerCapabilities_e enumeration ORed together.
         *
         * @see CNC_HTML5PlayerCapabilities_e
	 */
	unsigned int (*getPlayerCapabilities)(int playerID);

	/**
	 * Return the capabilities of the current Media Resource in the Media Player
	 *
         * This is a required method that the implementor MUST implement.
         * The implementation should return the combination of flags from the
         * CNC_HTML5MediaCapabilities_e enumeration, ORed together, to indicate
         * the capabilities of the player with respect to the current Media Resource.  This value is
         * not expected to be valid until the player reaches the "CNC_HTML5MediaState_HaveMetadata"
         * state - until this point, the browser will use default values for items controlled
         * by the capabilities rather than invoking this method.
         *
	 * @param[in] playerID - the media player id
         *
	 * @return
	 *	The current capabilities of the Media Player and current Media Resource
	 *       using flags from the CNC_HTML5MediaCapabilities_e enumeration ORed together.
         *
         * @see CNC_HTML5MediaCapabilities_e
         * @see CNC_HTML5MediaState_e
	 */
	unsigned int (*getMediaCapabilities)(int playerID);

	/**
	 * Return the playback position in seconds relative to the start of the media
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should return the player's current playback position.
         * This method will be called frequently to calculate playback progress.
         *
	 * @param[in] playerID - the media player id
         *
	 * @return
	 *	The playback position in seconds
	 */
	float (*getCurrentTime)(int playerID);

	/**
	 * Return the duration of the current media resource.
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should return the duration of the current Media Resource.
         * If the media resource is unbounded (i.e. if its duration is not known), the
         * implementation should return -1.  The browser will generate a value of
         * positive infinity internally to satisfy the requirements of the HTML 5
         * specification.  THis method will not be called until the player signals
         * that it has reached at least the "CNC_HTML5MediaState_HaveMetadata" state -
         * prior to that, the browser will use a default value of NaN (Not a Number),
         * as per the HTML 5 specification.
         * This method will be called frequently to calculate playback progress.
         *
	 * @param[in] playerID - the media player id
	 * @return
	 *	The duration of the current media in seconds
	 *	-1 if the iduration media stream is unknown (i.e. if the stream is unbounded.)
	 */
	float (*getDuration)(int playerID);

	/**
	 * Return the current state of the player.
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should return the current State of the player and Media Resource.
         *
	 * @param[in] playerID - the media player id
	 * @return
         *    The current media state value from the CNC_HTML5MediaState_e enumeration
         * @see CNC_HTML5MediaState_e
	 */
	CNC_HTML5MediaState_e (*getMediaState)(int playerID);

	/**
	 * Return the time range(s) of data buffered by the player
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting of its currently buffered data.  If this method is not
         * implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure should be
         * set to NULL.
         * If this method is implemented, it should return a list of CNC_HTML5TimeRange_t structures
         * specifying the start and end times of the sections of media data it has buffered.  For
         * most implementations, the list wil contain a single time range, but some players may
         * buffer discontinuous sections of media data (due to seeking, etc.), so this method supports
         * reporting of such situations.
         * The CNC_HTML5TimeRangeList_t structure includes a pointer to a method that the
         * browser will call to release the memory associated with the list items once they
         * have been processed.
         *
	 * @param[in] playerID - the media player id
	 *
	 * @return an HTML5RangeList_t structure containing a list of buffered time ranges
	 * @see CNC_HTML5TimeRangeList_t
         * @see CNC_HTML5TimeRange_t
	 */
	CNC_HTML5TimeRangeList_t (*getBufferedTimeRanges)(int playerID);

	/**
	 * Return the count of bytes loaded by the player
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting the amount of data it has loaded.  If this method is not
         * implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure should be
         * set to NULL.
         * If this method is implemented, it should return the number of bytes of media data
         * it has loaded so far.
         *
	 * @param[in] playerID - the media player id
	 *
	 * @return the number of bytes loaded or 0 if not known
	 */
	unsigned int (*getBytesLoaded)(int playerID);

	/**
	 * Return the total number of bytes expected by the player for the current media resource
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting the amount of data it expects to load for the current
         * Media Resource.  If this method is not implemented, its slot in the CNC_HTML5MediaPlayerFunctions
         * structure should be set to NULL.
         * If this method is implemented, it should return the number of bytes of media data
         * expected for the current Media Resource.
         *
	 * @param[in] playerID - the media player id
	 *
	 * @return the number of bytes expected or 0 if not known
	 */
	unsigned int (*getTotalBytes)(int playerID);

	/**
	 * Return the native size for the player, in pixels
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting its preferred size (and/or the preferred size
         * for the current Media Resource.)  If this method is not implemented, its slot in the
         * CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should return the preferred width and height
         * for the player and current Media Resource in pixels.
         * If this method is not implemented or returns 0, the default size of 300 x 150 (w x h)
         * will be used.
         *
	 * @param[in] playerID - the media player id
	 * @param[out] width - pointer to storage for the width
	 * @param[out] height - pointer to storage for the height
	 *
	 * @return 1 if the native size is known
	 *         0 if the native size is unknown
	 */
	int (*getNativeSize)(int playerID, unsigned int* width, unsigned int* height);

	/**
	 * Return the latest time that can be seeked to in the current media resource.
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting the latest time (relative to the timeline for
         * the current Media Resource) that it is able to seek to.  If this method is not
         * implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should return the latest time (relative to the timeline for
         * the current Media Resource) that the player can seek to.
         * If this method is not implemented, but the Media Capabilities include CNC_HTML5_MEDIA_CAP_IS_SEEKABLE
         * the browser will use the value returned by getDuration instead.
         *
	 * @param[in] playerID - the media player id
	 *
	 * @return the latest time that the current media resource can be seeked to,
         *          in seconds relative to the start of the media resource
         *
         * @see seek
         * @see getDuration
         * @see CNC_HTML5MediaCapabilities_e
	 */
        float (*getMaxTimeSeekable)(int playerID);

	/**
	 * Return the start time of the current media resource.
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting the start time of the current Media Resource.
         * If this method is not implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure
         * should be set to NULL.
         * If this method is implemented, it should return the start time of the current Media Resource.
         * If this method is not implemented, the browser will use a default value of 0.0.
         *
	 * @param[in] playerID - the media player id
	 *
	 * @return the start time of the current media resource
	 */
        float (*getStartTime)(int playerID);

	/**
	 * Return the initial time of the current media resource.
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting the initial time of the current Media Resource.
         * If this method is not implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure
         * should be set to NULL.
         * If this method is implemented, it should return the initial time of the current Media Resource.
         * This values will be used as the value of the "initialTime" property of the HTML Media (Video
         * or Audio) Element associated with this player.  In addition, if the value returned by this method
         * is greater than 0.0, the browser will ask the player to seek to this position when the player's
         * media state changes to CNC_HTML5MediaState_HaveMetadata from HTMLMediaState_HaveNothing.
         * If this method is not implemented, the browser will use a default value of 0.0.
         *
	 * @param[in] playerID - the media player id
	 *
	 * @return the initial time of the current media resource
	 */
        double (*getInitialTime)(int playerID);

	/**
	 * Set the size of the media player on screen
         *
         * This is a required method that the implementor MUST implement.
         * The implementation should adjust its size to the requested dimensions.
	 *
	 * @param[in] playerID - the media player id
	 * @param[in] x - X axis origin of the player, in pixels
	 * @param[in] y - Y axis origin of the player, in pixels
	 * @param[in] w - width of player in pixels
	 * @param[in] h - height of player in pixels
	 *
	 * @return
	 *	CNC_HTML5MediaPlayerError_e
	 */
	CNC_HTML5MediaPlayerError_e (*setScreenPosition)(int playerID, int x, int y, int w, int h);

	/**
	 * Set the playback rate.
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports Fast-Forward or Rewind modes.  If this method is not
         * implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure should be
         * set to NULL.
         * If this method is implemented, it should instruct the player thread to set its
         * playback rate to the requested value and return.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] rate - the new playback rate, positive values indicate forward play,
                                negative values indicate reverse play
	 *			(i.e 1=1x forward, -1=1x reverse, 2=2x forward, -2=2x reverse, etc).
	 *			(i.e 0.5=1/2 speed forward, -0.5=1/2 speed reverse, etc.)
	 */
	void (*setRate)(int playerID, float rate);

	/**
	 * Set the muted state of the player
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports muting of its audio output.  If this method is not implemented, its slot
         * in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should set the muted state of the player's audio output to the
         * requested value: 1 = muted, 0 = unmuted.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] muted - the new muted state for the player: 1 = muted, 0 = unmuted
	 */
        void (*setMuted)(int playerID, int muted);

	/**
	 * Set the volume of the player
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports setting the volume of its audio output.  If this method is not implemented,
         * its slot in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should set the volume of the player's audio output to the
         * requested value.  The volume value will be between 0.0 and 1.0 - it is up to the
         * implementation to scale this value into the appropriate range for their player.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] volume - the new volume value for the player
	 */
        void (*setVolume)(int playerID, float volume);

	/**
	 * Show or hide Closed Captions
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports displaying closed captions.  If this method is not implemented,
         * its slot in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should set the display of Closed Captions to
         * the requested value: 1 = shown, 0 = hidden.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] show - the new state for Closed Captions: 1 = shown, 0 = hidden
	 */
        void (*setClosedCaptionsVisible)(int playerID, int show);

	/**
	 * Get the volume of the player
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports reporting the volume of its audio output.  If this method is not implemented,
         * its slot in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * This method should return the volume of the player's audio output
         * as a floating point value scaled to between 0.0 and 1.0 where 0.0 is silent and
         * 1.0 is full volume.
         *
	 * @param[in] playerID - the media player id
         *
	 * @return the current volume of the media player, scaled to between 0.0 and 1.0
	 */
        float (*getVolume)(int playerID);

	/**
	 * Get the mute state of the player
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports rate changing.  If this method is not implemented,
         * its slot in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should return 1 if the media player is muted
         * and 0 if it is not.
         *
	 * @param[in] playerID - the media player id
         *
	 * @return the muted status of the player: 1 = muted, 0 = not muted
	 */
        int (*getMuted)(int playerID);

	/**
	 * Get the playback rate.
         *
         * This is an optional method that the implementor is only required to implement
         * if the player supports Fast-Forward or Rewind modes.  If this method is not
         * implemented, its slot in the CNC_HTML5MediaPlayerFunctions structure should be
         * set to NULL.
         * If this method is implemented, it should return the player's current playback
         * rate as a float, where positive values indicate forward play, negative values
         * indicate reverse play
	 *	(i.e 1=1x forward, -1=1x reverse, 2=2x forward, -2=2x reverse, etc).
	 *	(i.e 0.5=1/2 speed forward, -0.5=1/2 speed reverse, etc.)
	 *
	 * @param[in] playerID - the media player id
	 * @return the current playback rate of the player
	 */
        float (*getRate)(int playerID);

	/**
	 * Set the poster URL of the player
         *
         * This is an optional method.  If this method is not implemented,
         * its slot in the CNC_HTML5MediaPlayerFunctions structure should be set to NULL.
         * If this method is implemented, it should store the provided URL for the
         * poster image file so that the player can display it as appropriate.
         *
	 * @param[in] playerID - the media player id
	 * @param[in] url - the URL of the poster image
	 */
	void (*setPoster)(int playerID, const char *url);

} CNC_HTML5MediaPlayerFunctions;

/**
 * Register the supported MediaPlayer functions.
 * @param[out] functions - CNC_HTML5MediaPlayerFunctions struct containing the functions pointers supported.
 */
void CNC_Browser_registerHTML5MediaPlayer(CNC_HTML5MediaPlayerFunctions* functions);


#endif //#ifdef __CNC_MIDDLEWARE_HTML5_H__

#ifdef __cplusplus
}
#endif
#endif
