/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser Media Player.
 * @file cnc_browserMediaPlayerI.h
 * @ingroup web
 */

#ifndef __CNC_MIDDLEWARE_MEDIA_H__
#define __CNC_MIDDLEWARE_MEDIA_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"
#include "cnc_browserFeatures.h"
#include "cnc_browserEvents.h"

#ifdef __cplusplus
extern "C"{
#endif


/**
  *the player state.
  */
typedef enum {
	CNC_MediaPlayer_State_Unknown = 0,
	CNC_MediaPlayer_State_Unused = 1,
	CNC_MediaPlayer_State_Stopped = 2, //player is stopped
	CNC_MediaPlayer_State_Busy = 3, //player is currently busy
	CNC_MediaPlayer_State_Paused = 4, //player is paused
	CNC_MediaPlayer_State_Playing = 5, //player is currently playing
	CNC_MediaPlayer_State_Idle = 6, //player is idle state
	CNC_MediaPlayer_State_FFwd = 7, //player is in fast forward mode
	CNC_MediaPlayer_State_Rewind = 8, //player is in rewind mode
	CNC_MediaPlayer_State_EndOfStream = 9, //The player has reach the end of the stream.
	CNC_MediaPlayer_State_BeginOfStream = 10, //The player has reach the begining of the stream.
	CNC_MediaPlayer_State_Error = -2, //The player is in an error state.
} CNC_MediaPlayerState_e;

/**
 * Types of media player events.
 */
typedef enum {
	CNC_MEDIA_PLAY_STATE_CHANGED,   /*Event indicating the state of the player has changed */
	CNC_MEDIA_NETWORK_STATE_CHANGED,/*Event indicating the state of the network has changed */
	CNC_MEDIA_READY_STATE_CHANGED,  /*Event indicating the state of the buffering data has changed */
	CNC_MEDIA_ELAPSED_TIME_CHANGED, /**/
	CNC_MEDIA_DURATION_CHANGED,     /**/
} CNC_MediaPlayerEvent_e;

/**
 * Media player error codes.
 */
 typedef enum {
	/**
	 * No Error.
	 */
 	CNC_MEDIAPLAYER_NOERROR,
 	/**
 	 * Communication error.
 	 */
 	CNC_MEDIAPLAYER_COMMUNICATION_ERROR,
 	/**
 	 * Arguments are invalid.
 	 */
 	CNC_MEDIAPLAYER_INVALID_ARGS,
 	/**
 	 * player is busy and cannot process request.
 	 */
 	CNC_MEDIAPLAYER_BUSY,
 	/**
 	 * Unknown error.
 	 */
 	CNC_MEDIAPLAYER_UNKNOWN_ERROR
 } CNC_MediaPlayerError_e;

/**
 * Media player type
 */
typedef enum {
	/**
	 * broadcast video player (i.e. broadcast tv)
	 */
    CNC_MEDIAPLAYER_VIDEO_BROADCAST,
	/**
	 * IP embedded window video player (i.e. VOD player)
	 */
    CNC_MEDIAPLAYER_VIDEO_IP_WINDOW,
	/**
	 * IP fullscreen video player (external app)
	 */
    CNC_MEDIAPLAYER_VIDEO_IP_FULLSCREEN,
	/**
	 * Audio player
	 */
    CNC_MEDIAPLAYER_AUDIO
} CNC_MediaPlayerType_e;


/**
 * Function prototype for media player callback.
 * @param playerID - the id of the player generating the event
 * @param event - the event.
 */
typedef void (*CNC_MediaPlayerCallback_f) (int playerID, CNC_MediaPlayerEvent_e event);

typedef struct {

	/**
	 * Set the media callback function
	 * @param callback - the callback function.
	 # @return
	 *	1 on success
	 *	0 on failure
	 */
	int (*setCallback) (CNC_MediaPlayerCallback_f callback);

	/**
	 * Get a Media Player instance
	 * @param type - the player type
	 *
	 * @return
	 *	Returns the media player id on success
	 *	-1 on failure.
	 */
	int (*create) (CNC_MediaPlayerType_e type);

	/**
	 * release a Media Player instance.
	 * @param playerID - player id to release
	 * @return
	 *	1 on success
	 *	0 on failure
	 */
	int (*release) (int playerID);

	/**
	 * Set the display window of the media player
	 * @param playerID - the media player id
	 * @param x - x position
	 * @param y - y position
	 * @param w - width of video plane
	 * @param h - height of video plane
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*setVideoPlane) (int playerID, int x, int y, int w, int h);

	/**
	 * Play the specified uri.
	 * @param playerID - the media player id
	 * @param args - array of arguments
	 *		For NetTV, args will be:
	 *			args[0] = name=errorCode, value.integer=xxx
	 *			args[1] = name=status, value.integer=xxx
	 *			args[2] = name=metaData, value.str=xxx
	 *			args[3] = name=metaDataSize, value.integer=xxx
	 *		    args[4] = name=licenceId, value.str=xxx
	 *		    args[5] = name=scaling, value.str=xxx
	 *			args[6] = name=playpos, value.floatv=xxx
	 *			args[7] = name=returnUrl, value.str=xxx (for CNC_MEDIAPLAYER_VIDEO_IP_FULLSCREEN)
	 *		For HTML/BML, args will be:
	 *			args[0] = name=uri, value.str=xxx
	 *			args[1] = name=playpos, value.floatv=xxx
	 *			args[2] = name=autoplay, value.integer= 0 or 1 (optional)
	 *			args[3] = name=mimetype, value.str=xxx (optional mime type of content)
	 * @param arg_count - size of array.
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*play) (int playerID, CNC_BrowserParameter_t* args[], int arg_count);

	/**
	 * Pause the playback.
	 * @param playerID - the media player id
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*pause) (int playerID);

	/**
	 * Resume a paused playback.
	 * @param playerID - the media player id
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*resume) (int playerID);

	/**
	 * Fast forward the playback
	 * @param playerID - the media player id
	 * @param speed - the speed of the fastforward, positive values indicate fastforward level
	 *			(i.e 1=1x, 2=2x, ..., etc). Fraction values indicate slow forward level
	 *			(i.e 0.5=1/2 speed)
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*fastForward) (int playerID, float speed);

	/**
	 * Rewind the playback
	 * @param playerID - the media player id
	 * @param speed - the speed of the rewind, positive values indicate rewind level
	 *			(i.e 1=1x, 2=2x, ..., etc). Fraction values indicate slow rewind level
	 *			(i.e 0.5=1/2 speed)
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*rewind) (int playerID, float speed);

	/**
	 * Set the playback position
	 * @param playerID - the media player id
	 * @param pos - the playback position (in seconds) relative to the start of the media
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*setPosition) (int playerID, float pos);

	/**
	 * Get the playback position in seconds relative to the start of the media
	 * @param playerID - the media player id
	 * @return
	 *	The playback position in seconds
	 *	-1 if error
	 */
	int (*getPosition) (int playerID);

	/**
	 * Get the duration of the current media.
	 * @param playerID - the media player id
	 * @return
	 *	The duration of the current media in seconds
	 *	-1 if error.
	 */
	int (*getDuration) (int playerID);

	/**
	 * Stop the playback.
	 * @param playerID - the media player id
	 * @return
	 *	CNC_MediaPlayerError_e
	 */
	CNC_MediaPlayerError_e (*stop) (int playerID);

	/**
	 * Get the play state.
	 * @param playerID - the media player id
	 * @param out_error, if not null
	 * 		if state == CNC_MediaPlayer_State_Error, then out_error contains the error code.
	 * @return
	 */
    CNC_MediaPlayerState_e (*getPlayState) (int playerID, int *out_error);

	/**
	 * Handle key event
	 * @param playerID - the media player id
	 * @param keyEvt
	 * @return 1 if handled, 0 for not
	 */
	int (*handleKeyEvent)(int playerID, CNC_BrowserKeyboardEvent_t* keyEvt);

	/**
	 * Get the player buffer state.
	 * @param playerID - the media player id
	 * @return
	 */
	int (*not_used2) (int playerID);

} CNC_MediaPlayerFunctions;

/**
 * Register the supported MediaPlayer functions.
 * @param functions - CNC_MediaPlayerFunctions struct containing the functions pointers supported.
 */
void CNC_Browser_registerMediaPlayer (CNC_MediaPlayerFunctions* functions);


typedef struct {
	/**
	 * This function plays sound of an event built in the receiver that is specified
	 * with soundID based on the conventions on the namespace
	 * @param soundId - the id of the ROM sound
	 * @return
	 */
	CNC_BrowserI_ReturnCode_e (*playRomSound) (const char* soundId);
} CNC_AudioFunctions;

/**
 * Register the supported Audio functions. Functions that are not supported should have a NULL function pointer.
 * @param functions - CNC_AudioFunctions struct containing the functions pointers supported.
 */
void CNC_Browser_RegisterAudioFunctions(CNC_AudioFunctions* functions);

#ifdef __cplusplus
}
#endif

#endif  //__CNC_MIDDLEWARE_MEDIA_H__
