/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser Printer .
 *           The CNC Browser print functions provide support for rendering
 *           printable image data from a webpage.
 *           To use this, implement the callback functions in CNC_Printer,
 *           then pass an CNC_Printer to CNC_Browser_Print().
 * @file cnc_browserPrinter.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_PRINTER_H__
#define __CNC_BROWSER_PRINTER_H__

/* ============================ System Includes =========================== */
#include <stdio.h>
#include <stdbool.h>

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * CNC_Printer is an interface for receiving print callbacks. Printing
 * modifies the document layout in-place, so the print must happen synchronously
 * on the WebKit main thread.
 */
typedef struct CNC_Printer {

    /**
     * A context to use for callback functions.
     */
    void* context;

    /**
     * Indicates that the browser will begin the print job. Printing is done synchronously
     * on the WebKit thread, so the browser will be paused during printing; a notification
     * screen should be shown to the user to indicate that they will not be able to
     * interact with the browser until printing is complete.
     *
     * \param context    The print context.
     * \param pageCount  The total number of pages in the print job.
     *
     * \return True to continue with printing; false to abort the print job.
     */
    bool (*startPrintJob)(void* context, int pageCount);

    /**
     * Indicates that the browser will begin printing a page. If true is returned, a series
     * of imageData() calls will follow containing the rasterized page. If false is returned,
     * the page will be skipped. A 180 degree rotation can optionally be applied to the page
     * before printing by setting the rotate parameter to true.
     *
     * \param [in]  context    The print context.
     * \param [in]  pageNumber The page number to begin printing (from 0 to totalPages-1).
     * \param [out] rotate     True if the page should be rotated 180 degrees before printing.
     *
     * \return True to print this page; false to skip it.
     */
    bool (*startPage)(void* context, int pageNumber, bool* rotate);

    /**
     * Passes a portion of the current page rasterized as 32-bit XRGB pixel data in native-endian
     * order. The data for a pixel is given by data[x + y * stride]. Pages are always
     * painted from top to bottom.
     *
     * \param context   The print context.
     * \param data      The image data in XRGB native-endian order.
     * \param stride    The number of 32-bit elements between the start of successive rows of image data.
     * \param row       The page row at which the image data starts.
     * \param width     The width of the image data.
     * \param height    The height of the image data.
     * \param done      True if this is the last chunk of image data for this page; false if there is more data to come.
     */
    void (*imageData)(void* context, const uint32_t* data, int stride, int row, int width, int height, bool done);

    /**
     * Indicates that printing is complete. This will always be called, even if the
     * print job is aborted. It is safe to free the context during this call; the
     * CNC_Printer is discarded and the browser resumes normal operation after this
     * call returns.
     *
     * \param context The print context.
     */
    void (*done)(void* context);

} CNC_Printer;

/**
 * Schedules a print job for the current webpage into the given CNC_Printer interface.
 * This call returns immediately; callback functions will be called later from WebKit's
 * main thread. A copy of the given CNC_Printer is made (but not of the included
 * context), so the caller does not need to keep the CNC_Printer in memory.
 *
 * The given page dimensions should not include room for printer margins. The webpage
 * layout will consume the full width and height given.
 *
 * \param printer       The callback interface to use to print the webpage.
 * \param pageWidth     The width of a page in pixels.
 * \param pageHeight    The height of a page in pixels.
 * \param landscape     True to print in landscape mode (rotating the webpage 90 degrees clockwise); false for portrait.
 * \param prntbg        true to print with backgroud, false to print without background
 */
#ifndef USE_ESPIAL_V61
void CNC_Browser_Print(const CNC_Printer* printer, int pageWidth, int pageHeight, bool landscape);
#else
void CNC_Browser_Print (const CNC_Printer* printer, int pageWidth, int pageHeight, bool landscape, bool prntbg);
#endif

#ifdef __cplusplus
}
#endif

#endif  //__CNC_BROWSER_PRINTER_H__

