/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser Types
 * @file cnc_browserTypes.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_TYPES_H__
#define __CNC_BROWSER_TYPES_H__

/* ============================ System Includes =========================== */
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * CNC Browser color
 *
 * the format is aarrggbb.
 */
typedef unsigned int CNC_BrowserARGB8888_t;

typedef enum {
    CNC_PARAMETER_INTEGER, /**< Parameter is an integer */
    CNC_PARAMETER_FLOAT,   /**< Parameter is a float */
    CNC_PARAMETER_STRING,  /**< Parameter is a string */
    CNC_PARAMETER_COLOR,   /**< Parameter is a CNC_BrowserARGB8888_t */
} CNC_ParameterType_e;

/**
 * CNC Browser Input types.
 */
typedef enum
{
    CNC_Browser_InputType_TextField = 0,
    CNC_Browser_InputType_TextArea,
    CNC_Browser_InputType_Plugin,
    CNC_Browser_InputType_Other = 9,
} CNC_BrowserInputType_e;

/**
 * CNC Browser error types.
 */
typedef enum
{
    CNC_Browser_ErrorType_NETWORK = 0,
    CNC_Browser_ErrorType_HTML,
    CNC_Browser_ErrorType_PLUGIN,
    CNC_Browser_ErrorType_BML,
    CNC_Browser_ErrorType_DSMCC,
    CNC_Browser_ErrorType_GENERIC,
} CNC_BrowserErrorType_e;

/**
 * Media player arguement
 */
typedef struct {
    const char* name;     /**< Name of parameter */
    CNC_ParameterType_e type; /**< Type of parameter */

    union {
        int integer;      /**< Value is an integer */
        float floatv;     /**< Value is a float */
        const char* str;  /**<  Value is a string */
    } value;
} CNC_BrowserParameter_t;


typedef struct CNC_BrowserCString_s
{
    char* data;         /*null terminated C-style string*/
    void (*release) (struct CNC_BrowserCString_s *ref);
} CNC_BrowserCString_t;


/**
 * A structure to represent a unicode string
 */
typedef struct CNC_BrowserUnicodeString_s
{
    unsigned int len;         /**< Length of the unicode string */
    unsigned short* uchars; /**< Pointer to data */
    void (*release) (struct CNC_BrowserUnicodeString_s *ref);
} CNC_BrowserUnicodeString_t;


typedef enum
{
    CNC_Browser_Encoding_UTF8 = 0,
} CNC_BrowserEncoding_t;


/**
 * A structure to represent a string
 */
typedef struct CNC_BrowserString_s
{
    CNC_BrowserEncoding_t encoding;
    unsigned int len;   /**< Length of the unicode string */
    char* chars;    /**< Pointer to data */
    void (*release)(struct CNC_BrowserString_s *); /**< Pointer to a memory release function */
} CNC_BrowserString_t;


typedef enum
{
    CNC_Browser_InputCharacterType_All      = 0,
    CNC_Browser_InputCharacterType_Number   = 1,
    CNC_Browser_InputCharacterType_Alphabet = 2,
    CNC_Browser_InputCharacterType_Hankaku  = 3,
    CNC_Browser_InputCharacterType_Zenkaku  = 4,
    CNC_Browser_InputCharacterType_Katakana = 5,
    CNC_Browser_InputCharacterType_Hiragana = 6,
    CNC_Browser_InputCharacterType_URL      = 20,
    CNC_Browser_InputCharacterType_Blank    = 100,
    CNC_Browser_InputCharacterType_Other    = 101,
} CNC_BrowserInputCharacterType_e;

/**
 * @brief A structure to represent a generic HTML input element
 */
typedef struct
{
    CNC_BrowserInputType_e type;
    unsigned int readOnly;
    union
    {
        struct
        {
            int16_t password;
            uint16_t maxLength;
        } textfield;

        struct
        {
            int16_t rows;
            int16_t cols;
            uint16_t maxLength;
        } textarea;
    } u;

    CNC_BrowserString_t string;
    CNC_BrowserInputCharacterType_e charactertype;
    const char* charactertypeUTF8Str; /* null-terminated utf-8 string */
} CNC_BrowserInput_t;


/**
 * @brief A structure to represent an error
 */
typedef struct
{
    CNC_BrowserErrorType_e type;
    int16_t code;
} CNC_BrowserError_t;


/**
 * @brief A structure to represent a rectangle
 */
typedef struct
{
    int32_t x; /**< The x coordinate of the CNC_BrowserRect */
    int32_t y; /**< The y coordinate of the CNC_BrowserRect */
    int32_t w; /**< The width of the CNC_BrowserRect */
    int32_t h; /**< The height of the CNC_BrowserRect */
} CNC_BrowserRect_t;


/**
 * CNC Browser mode enumeration
 */
typedef enum {
    CNC_Browser_Mode_Web = 0,          /**< Web mode */
    CNC_Browser_Mode_Kiosk = 1,        /**< Kiosk mode */
    CNC_Browser_Mode_BML = 2,          /**< BML mode */
    CNC_Browser_Mode_IPlayer = 3,      /**< BBC iPlayer mode */
} CNC_BrowserMode_t;


/**
 * CNC Browser Custom JS variant types
 */
typedef enum
{
    CNC_Browser_Type_void,
    CNC_Browser_Type_int,
    CNC_Browser_Type_double,
    CNC_Browser_Type_string,
    CNC_Browser_Type_array,
    CNC_Browser_Type_NaN,
    CNC_Browser_Type_PosInfinity,
    CNC_Browser_Type_NegInfinity,
    CNC_Browser_Type_null,
    CNC_Browser_Type_boolean, /* set intValue to 0 for false, other values for true */
    CNC_Browser_Type_UnicodeString,
    CNC_Browser_Type_Object,
} CNC_BrowserType_t;


/**
 * @brief A structure to represent a X509 certificate extension
 */
typedef struct
{
    char* object;                          /**< The type of the extension  */
    int   critical;                        /**< 1: critical extension;  0: non-critical extension */
    char* value;                           /**< object type specific string value */
} CNC_X509_Extension_t;


/**
 * @brief A structure to represent a X509 certificate
 *
 * issuer
 * subject
 *  UTF8 encoded
 *  Contains name=value pairs separated by '/', if the value has '/' or '\', then it is escape with '\'
 *  for example,
 *    /C=US/O=VeriSign, Inc./OU=VeriSign Trust Network/OU=Terms of use at https:\/\/www.verisign.com\/rpa (c)06/CN=VeriSign Class 3 Extended Validation SSL SGC CA
 *    /C=US/ST=California/L=Mountain View/O=Google Inc/CN=www.google.com
 */
typedef struct
{
    int enabled;                          /**< 1: this certificate is currently enabled  0: disabled   */
    int version;                          /**< X509 version */
    char* serialNumber;                   /**< Unique for each certificate issued by a specific CA */
    char* subjectPublicKeyAlgorithm;      /**< The encrypt algorithm used to create the signature */
    char* subjectPublicKeyValue;          /**< The signature signed by the CA's private key */
    char* issuer;                         /**< The entity that verified the information and issued the certificate - UTF8 encoded */
    char* validityNotBefore;
    char* validityNotAfter;
    char* subject;                        /**< The person, or entity identified - UTF8 encoded */
    char* issuerUID;                      /**< Issuer Unique Identifier */
    char* subjectUID;                     /**< Subject Unique Identifier */
    char* signatureAlgorithm;
    char* signatureValue;
    int numberOfExtensions;
    CNC_X509_Extension_t **extensions;        /**< Array of X509 extensions */
} CNC_X509_Certificate_t;


/**
 * @brief A structure to represent a list of X509 certificates
 */
typedef struct  CNC_BrowserCertificateList
{
    void (*release) (struct CNC_BrowserCertificateList *list);  /**< The release method to be used by the application */
    int numberOfCertificates;             /**< How many certificates included */
    CNC_X509_Certificate_t** certificates;    /**< List of certificates */
} CNC_BrowserCertificateList_t;

typedef enum
{
    CNC_BrowserI_Failure = 0,
    CNC_BrowserI_Success = 1,
} CNC_BrowserI_ReturnCode_e;

typedef enum
{
    CNC_Browser_CursorType_POINTER = 0,
    CNC_Browser_CursorType_CROSS,
    CNC_Browser_CursorType_HAND,
    CNC_Browser_CursorType_IBEAM,
    CNC_Browser_CursorType_WAIT,
    CNC_Browser_CursorType_HELP,
    CNC_Browser_CursorType_NONE,
    CNC_Browser_CursorType_MOVE,
    CNC_Browser_CursorType_CUSTOM,
} CNC_BrowserCursorType_e;

/**
 * A structure to represent a cursor and its location
 */
typedef struct
{
    int x;                          /**< x position of the cursor */
    int y;                          /**< y position of the cursor */
    CNC_BrowserCursorType_e type;   /**< cursor type */
} CNC_BrowserCursor_t;


/**
 * A structure to represent the page's information that browser could be interested in
 */
typedef struct CNC_BrowserPageInfo
{
    void (*release) (struct CNC_BrowserPageInfo* info);
    char* url;                             /**< The URL of this page */
    int pageSize;                          /**< The number of byes of this page */
    CNC_BrowserString_t title;             /**< The title of this page */
    CNC_BrowserString_t mimeType;          /**< The MIME type of this page */
    CNC_BrowserString_t loadTime;          /**< Theloaded time of this page */
    CNC_X509_Certificate_t* cert;              /**< The certificate pertains to current page if any */
    int certType;                          /**< 0: trusted certificate; 1: untrusted certificate */
} CNC_BrowserPageInfo_t;

/**
 * Direction
 */
typedef enum
{
    CNC_Browser_Direction_TOP = 0,
    CNC_Browser_Direction_LEFT,
    CNC_Browser_Direction_BOTTOM,
    CNC_Browser_Direction_RIGHT,
} CNC_BrowserDirection_e;


/**
 * Common structs
 */
typedef struct CNC_DateTime_s {
   /**
    * Year specified as an number, i.e. 2010
    */
   int year;
   /**
    * Month specified as a number (0-11), i.e. 5
    */
   int month;
   /**
    * Day of the month specified as a number (1-31), i.e. 18
    */
   int day;
   /**
    * Hour specified as a number (0-23), i.e. 13
    */
   int hour;
   /**
    * Minute specified as a number (0-59), i.e. 34
    */
   int minute;
   /**
    * Second specified as a number (0-59), i.e. 23
    */
   int second;
   /**
    * Milli-Second specified as a number (0-999), i.e. 240
    */
   int ms;


   void (*unused) (struct CNC_DateTime_s* ref);
} CNC_BrowserDateTime_t;


/**
 * CNC Browser secure warning types.
 */
typedef enum
{
    CNC_SecurityWarningType_VIEW_SECURE = 0,
    CNC_SecurityWarningType_LEAVE_SECURE_FOR_INSECURE,
    CNC_SecurityWarningType_SUBMIT_INSECURE, /* NOT IMPLEMENTED */
    CNC_SecurityWarningType_VIEW_SECURE_WITH_INSECURE_INFORMATION, /* NOT IMPLEMENTED */
} CNC_BrowserSecurityWarningType_e;

#ifdef __cplusplus
}
#endif

#endif /* __CNC_BROWSER_TYPES_H__ */
