/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Web Socket.
 * @file cnc_browserWebSocket.h
 * @ingroup web
 */

#include <stdbool.h>
/* ============================= User Includes ============================ */
#ifndef __CNC_BROWSERWEBSOCKET_H__
#define __CNC_BROWSERWEBSOCKET_H__


#ifdef __cplusplus
extern "C"
{
#endif

typedef unsigned short CNC_Browser_WebSocketClose_t;

enum {

    /**
     * @brief 1000 indicates a normal closure, meaning that the purpose for
     * which the connection was established has been fulfilled.
     */
    CNC_Browser_WS_CloseEventCodeNormalClosure = 1000,
    /**
     * @brief 1001 indicates that an endpoint is "going away", such as a server
     * going down or a browser having navigated away from a page.
     */
    CNC_Browser_WS_CloseEventCodeGoingAway = 1001,
    /**
     * @brief 1002 indicates that an endpoint is terminating the connection due
     * to a protocol error.
     */
    CNC_Browser_WS_CloseEventCodeProtocolError = 1002,
    /**
     * @brief 1003 indicates that an endpoint is terminating the connection
     * because it has received a type of data it cannot accept (e.g., an
     * endpoint that understands only text data MAY send this if it
     * receives a binary message).
     */
    CNC_Browser_WS_CloseEventCodeUnsupportedData = 1003,
    /**
     * @brief Reserved.  The specific meaning might be defined in the future.
     */
    CNC_Browser_WS_CloseEventCodeFrameTooLarge = 1004,
    /**
     * @brief 1005 is a reserved value and MUST NOT be set as a status code in a
     * Close control frame by an endpoint.  It is designated for use in
     * applications expecting a status code to indicate that no status
     * code was actually present.
     */
    CNC_Browser_WS_CloseEventCodeNoStatusRcvd = 1005,
    /**
     * @brief 1006 is a reserved value and MUST NOT be set as a status code in a
     * Close control frame by an endpoint.  It is designated for use in
     * applications expecting a status code to indicate that the
     * connection was closed abnormally, e.g., without sending or
     * receiving a Close control frame.
     */
    CNC_Browser_WS_CloseEventCodeAbnormalClosure = 1006,
    /**
     * @brief 1007 indicates that an endpoint is terminating the connection
     * because it has received data within a message that was not
     * consistent with the type of the message (e.g., non-UTF-8 [RFC3629]
     * data within a text message).
     */
    CNC_Browser_WS_CloseEventCodeInvalidUTF8 = 1007,
    /**
     * @brief Status codes in the range 3000-3999 are reserved for use by
     * libraries, frameworks, and applications.  These status codes are
     * registered directly with IANA.  The interpretation of these codes
     * is undefined by this protocol.
     */
    CNC_Browser_WS_CloseEventCodeMinimumUserDefined = 3000,
    /**
     * @brief Status codes in the range 4000-4999 are reserved for private use
     * and thus can't be registered.  Such codes can be used by prior
     * agreements between WebSocket applications.  The interpretation of
     * these codes is undefined by this protocol.
     */
    CNC_Browser_WS_CloseEventCodeMaximumUserDefined = 4999
};

typedef enum {
    /**
     * @brief No error
     */
    CNC_Browser_WS_NOError,
    /**
     * @brief Out of Memory
     */
    CNC_Browser_WS_OUTOFMEMORY

}CNC_Browser_WebSocketErrorCode_t;


typedef struct {
    /**
     * @brief Called whenever a websocket was created.
     * @param context Used by application
     * @param handle WebSocket handle, if no error, handle >=0
     * @param error 0 means no error, 1 means out of memory
     */
    void (*didCreateCB)(void* context, int handle, CNC_Browser_WebSocketErrorCode_t error);
    /**
     * @brief Called whenever a WebSocket was destroyed.
     * @param context Used by application
     * @param handle WebSocket handle
     */
    void (*didDestroyCB)(void* context, int handle);
    /**
     * @brief Called whenever a connection was successful.
     * @param context Used by application
     * @param handle WebSocket handle
     * @param subProt Subprotocol returned from the server, utf8 encoded
     * @param sz1 Size of the subProt
     * @param exten Extension returned from the server, , utf8 encoded
     * @param sz2 Size of exten
     */
    void (*didConnCB)(void* context, int handle, const char* subProt, int sz1, const char* exten, int sz2);
    /**
     * @brief Called when a WebSocket connection was closed, you can reconnect.
     * @param context Used by application
     * @param handle WebSocket handle
     * @param isClosingHndshakComplete Indicates whether the closing hanshake has completed.
     * @param code Close Event Code see enum CNC_Browser_WS_CloseEventCodeXXXXXXXX or rfc-6455 paper
     */
    void (*didCloseCB)(void* context, int handle, bool isClosingHndshakComplete, CNC_Browser_WebSocketClose_t code);
    /**
     * @brief Called whenever a text message was received
     * @param context Used by application
     * @param handle WebSocket handle
     * @param pMsg Is the message, utf8 encoded
     * @param sz Size of pMsg
     */
    void (*didRecvMsgCB)(void* context, int handle, const char* pMsg, int sz);
    /**
     * @brief [Optional, could be NULL] Called whenever a trunk of binary data is received.
     * @param context Used by application
     * @param handle WebSocket handle
     * @param data Data
     * @param sz Size of the binary data
     */
    void (*didRecvBinCB)(void* context, int handle, const char* data, int sz);
    /**
     * @brief [Optional, could be NULL] Called Whenever a Ping is Received. WebSocket automatically response a Pong
     * With the same data as in Ping. You don't need to do it twice. This callback is mainly
     * for notification.
     * @param context Used by application
     * @param handle WebSocket handle
     */
    void (*didRecvPingCB)(void* context, int handle, const char* payload, int sz);
    /**
     * @brief [Optional, could be NULL] Called whenever a Pong was received.
     * @param context Used by application
     * @param handle WebSocket handle
     * @param payload Data to be sent
     * @param sz Size of payload
     */
    void (*didRecvPongCB)(void* context, int handle, const char* payload, int sz);
    /**
     * @brief [Optional, could be NULL] Called whenever a Closing handshake is started.
     * @param context Used by application
     * @param handle WebSocket handle
     */
    void (*didStCloseHndShakeCB)(void* context, int handle);
}CNC_BrowserWebSocketCallBacks_t;



/**
 * @brief Register WebSocket callback functions
 * @param cbs CNC_BrowserWebSocketCallBacks_t
 */
void CNC_Browser_RegisterWebSocketCallBacks(CNC_BrowserWebSocketCallBacks_t* cbs);
/**
 * @brief Create a new WebSocket. If success CNC_BrowserWebSocketCallBacks_t::didCreateCB
 * will be called with handle >=0. If failed, CNC_BrowserWebSocketCallBacks_t::didCreateCB
 * called with handle = -1, with error != CNC_Browser_WebSocketErrorCode_t::CNC_Browser_WS_NOError
 * @param context Used by application
 * @param closeTimeoutSecs Specify a closing handshake timeout in seconds to overwrite the 4 mins default setting. closeTimeoutSecs = "0" equals 4 mins default.
 */
void CNC_Browser_WebSocketCreate(void* context, int closeTimeoutSecs);
/**
 * @brief Destroy a websocket. If the handle has already closed. This only frees the handle resource.
 * If not closed, it will call close the underlying socket first and then frees the resource (therefore CNC_BrowserWebSocketCallBacks_t::didDestroyCB will be called).
 * In this case, the CNC_BrowserWebSocketCallBacks_t::didCloseCB will not be called ony didDestroyCB will be called.
 * @param handle WebSocket handle
 */
void CNC_Browser_WebSocketDestroy(int handle);

/**
 * @brief WebSocket Connect
 * If any errors happened, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called with corresponding code (rfc-6455)
 * @param handle WebSocket handle
 * @param url ASCII coded null terminated url
 */
void CNC_Browser_WebSocketConnect(int handle, const char* url);

/**
 * @brief WebSocket Connect, Reserved for future
 * If any errors happened, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called with corresponding code (rfc-6455)
 * @param handle WebSocket handle
 * @param url ASCII coded null terminated url
 * @param protocol ONE ASCII coded null terminated subprotocol (rfc-6455)
 */
void CNC_Browser_WebSocketConnect2(int handle, const char* url, const char* protocol);

/**
 * @brief WebSocket Connect, Reserved for future
 * If any errors happened, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called with corresponding code (rfc-6455)
 * @param handle WebSocket handle
 * @param url ASCII coded null terminated url
 * @param protocols a couple of ASCII coded null terminated subprotocols (rfc-6455)
 * @param sz number of subprotocols
 */
void CNC_Browser_WebSocketConnect3(int handle, const char* url, const char* protocols[], int sz);

/**
 * @brief WebSocket send text message
 * If any errors happened, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called with corresponding code (rfc-6455)
 * @param handle WebSocket handle
 * @param message Message, utf8 coded
 * @param sz Size of message
 */
void CNC_Browser_WebSocketSendMessage(int handle, const char* message, int sz);

/**
 * @brief WebSocket send Binary data, Reserved for future
 * If any errors happened, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called with corresponding code (rfc-6455)
 * @param handle WebSocket handle
 * @param buf Binary data
 * @param sz Size of buf
 */
void CNC_Browser_WebSocketSendBinary(int handle, const char* buf, int sz);

/**
 * @brief WebSocket send Ping frame
 * If any errors happened, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called with corresponding code (rfc-6455)
 * @param handle WebSocket handle
 * @param payload data sent in Ping frame
 * @param sz Size of payload
 */
void CNC_Browser_WebSocketSendPing(int handle, const char* payload, int sz);

/**
 * @brief WebSocket Cose. Once done, CNC_BrowserWebSocketCallBacks_t::didCloseCB will be called.
 * @param handle WebSocket handle
 */
void CNC_Browser_WebSocketClose(int handle);






#ifdef __cplusplus
}
#endif

#endif // __CNC_BROWSERWEBSOCKET_H__

