/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file        cnc_webapp.h
 * @ingroup     appframework
 * @brief       Public interface of the application framework.
 */

#ifndef __CNC_WEBAPP_H__
#define __CNC_WEBAPP_H__

/* ============================ System Includes =========================== */

/* ============================= User Includes ============================ */
#include "cnc_webapptypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* =========================== Type definitions =========================== */

/**
 *  @brief          Callback function type invoked whenever there is a change in download progress.
 *  @param [in]     count           Number of elements in the array.
 *  @param [in]     list            Array of pointers to download list entires.
 *                                  The called function must not modify or release the contents of the entries.
 *                                  The called function must not release this array of pointers either.
 *                                  The called function must copy the required information in the entires for further use.
 *  @param [in]     userData        Pointer to user data, provided by the caller at the time of registering for notifications.
 *  @return         CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
typedef E_cncErrCode_t (*cncWebAppDownloadProgressCallback_t)(CNC_SINT32 count, cncWebAppMgrDownloadListEntry_t *list[], void *userData);


/* ======================== Function declarations ========================= */

/**
 *  @brief          Initializes the application framework and its internal sub-modules.
 *  @return         CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_Init(void);

/**
 *  @brief          Deinitializes the application framework and its internal sub-modules.
 *  @return         CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_Deinit(void);

/* ================== Interfaces for handling downloading items =========== */

/**
 *  @brief              Retrieves the list of download requests along with their current status.
 *  @param [in]         start       Specifies the (zero-based) index from where the entries should be retrieved.
 *  @param [in, out]    count       On entry it specifies the number of entries to be retrieved.
 *                                  On return it will be updated with the actual number of entries retrieved.
 *  @param [out]        list        The caller must allocate an array of entry structures according to the count specified in the invocation.
 *                                  On return, this will contain the requested entries from the list of downloads.
 *                                  If this is specified as NULL, only count will be updated with the number of entries.
 *  @return             CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_GetDownloadList(CNC_UINT32 start, CNC_UINT32 *count, cncWebAppMgrDownloadListEntry_t *list);

/**
 *  @brief              Registers a callback for download progress indications.
 *  @param [in]         start       Specifies the (zero-based) index from where the entries should be considered.
 *  @param [in, out]    count       On entry it specifies the number of entries to be considered.
 *                                  On return it will be updated with the actual number of entries considered.
 *  @param [in]         callback    Function which is invoked whenever there is any change in the download progress.
 *  @param [in]         userData    Pointer to user data, which should be provided in the callback notification.
 *  @return             CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_DownloadProgressRegister(CNC_UINT32 start, CNC_UINT32 *count, cncWebAppDownloadProgressCallback_t callback, void *userData);

/**
 *  @brief              Changes the state of a download.
 *  @param [in]         entry       An entry from the list of download requests whose state has to be changed.
 *  @param [in]         newState    Specifies the new state of the download. Note that it is not valid to specify the state as "completed" in this case.
 *  @return             CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_ChangeDownloadState(cncWebAppMgrDownloadListEntry_t *entry, cncWebAppMgrDownloadStatus_t newState);


/* ================== Interfaces for handling downloaded media ============ */

/**
 *  @brief              Retrieves the list of downloaded media.
 *  @param [in]         start       Specifies the (zero-based) index from where the entries should be retrieved.
 *  @param [in, out]    count       On entry it specifies the number of entries to be retrieved.
 *                                  On return it will be updated with the actual number of entries retrieved.
 *  @param [out]        list        The caller must allocate an array of entry structures according to the count specified in the invocation.
 *                                  On return, this will contain the requested entries from the list of downloaded media.
 *                                  If this is specified as NULL, only count will be updated with the number of entries.
 *  @return             CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_GetDownloadedMedia(CNC_UINT32 start, CNC_UINT32 *count, cncWebAppMgrDownloadedMediaListEntry_t *list);

/**
 *  @brief          Removes a downloaded media. This will remove the entry from the downloaded media list, as well as the media from the file-system.
 *  @details        This is applicable only to media that has completed downloading.
 *  @param [in]     entry   Pointer to an entry in the downloaded media list (as returned by a query).
 *  @return         CNC_ERR_NONE if successful or a specific error code otherwise (see E_cncErrCode_t).
 */
E_cncErrCode_t CNC_WebApp_RemoveDownloadedMedia(cncWebAppMgrDownloadedMediaListEntry_t *entry);

#ifdef __cplusplus
}
#endif

#endif  // __CNC_WEBAPP_H__
