/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file        cnc_webapptypes.h
 * @ingroup     appframework
 * @brief       Definitions of common object types used throughout the application framework.
 */

#ifndef __CNC_WEBAPPTYPES_H__
#define __CNC_WEBAPPTYPES_H__

/* ============================ System Includes =========================== */

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_common_error.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* =========================== Macro definitions ========================== */

/**
 *  @brief  Length of a schema version string.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_SCHEMA_VERSION_SIZE               (24)

/**
 *  @brief  Length of a RTE version string.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_RTE_VERSION_SIZE                  (24)

/**
 *  @brief  Length of a application version string.
 *          This specifies the number of characters (including the terminating '\0').
 *          The range of versions is 0.0.0.0 to 65535.65535.65535.65535
 *  @see    https://developer.chrome.com/extensions/manifest/version.html
 */
#define CNC_WEBAPPMGR_APPLICATION_VERSION_SIZE          (24)

/**
 *  @brief  Length of an application's name.
 *          This specifies the number of characters (including the terminating '\0').
 *  @see    https://developer.chrome.com/extensions/manifest/name.html
 */
#define CNC_WEBAPPMGR_APPLICATION_NAME_SIZE             (64)

/**
 *  @brief  Length of a application's brief description.
 *          This specifies the number of characters (including the terminating '\0').
 *  @see    https://developer.chrome.com/extensions/manifest/description.html
 */
#define CNC_WEBAPPMGR_APPLICATION_DESCRIPTION_SIZE      (256)

/**
 *  @brief  Length of paths to resources on the local file-system, used throughout the application framework.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_PATH_SIZE                         (4096)

/**
 *  @brief  Length of URLs to resources either on the local file-system or remote, used throughout the application framework.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_URL_SIZE                          (4096)

/**
 *  @brief  Length of Internet Media Types (MIME types). See RFC 4288.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_IMT_TYPE_SIZE                     (256)

/**
 *  @brief  Length of simple meta-text fields associated with an application.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_APPLICATION_META_TEXT_SIZE        (64)

/**
 *  @brief  Length of a downloaded media's name.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_MEDIA_NAME_SIZE                   (128)

/**
 *  @brief  Length of a downloaded media's description.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_MEDIA_DESCRIPTION_SIZE            (256)

/**
 *  @brief  Length of simple meta-text fields associated with a downloaded media.
 *          This specifies the number of characters (including the terminating '\0').
 */
#define CNC_WEBAPPMGR_MEDIA_META_TEXT_SIZE              (64)

/* =========================== Type definitions =========================== */

/**
 *  @brief      Application capabilities defined in bit-fields.
 *  @details    When both UPGRADEABLE or REMOVABLE flags are not set, it indicates a FIXED application that cannot be removed or upgraded.
 *              When WIDGET flag is not set, the application will execute as a typical full-screen application.
 *              When both NATIVE and WEB flags are set, the application will execute using the both the native (backend functionality) and web RTEs (UI through web-pages).
 */
typedef enum _cncWebAppMgrCapability_t
{
    CNC_WEBAPPMGR_CAPABILITY_NONE                       = 0,
    CNC_WEBAPPMGR_CAPABILITY_UPGRADEABLE                = 0x1,                                          /**< User/System can upgrade this application */
    CNC_WEBAPPMGR_CAPABILITY_REMOVABLE                  = 0x2,                                          /**< User can remove this application */
    CNC_WEBAPPMGR_CAPABILITY_WIDGET                     = 0x4,                                          /**< Application will execute within another container application */
    CNC_WEBAPPMGR_CAPABILITY_BACKGROUND                 = 0x8,                                          /**< Application will execute using the native RTE, but as a background task and not have any UI */
    CNC_WEBAPPMGR_CAPABILITY_OFFLINE                    = 0x10,                                         /**< Application is capable of operating offline. (For example an application using web-pages as its UI, but not requiring access to any resources over the network.) */
    CNC_WEBAPPMGR_CAPABILITY_NATIVE                     = 0x20,                                         /**< Application will execute using the native RTE and provides its own UI (native) */
    CNC_WEBAPPMGR_CAPABILITY_WEB                        = 0x40                                          /**< Application will execute using the web RTE and provides web-pages as its UI */
} cncWebAppMgrCapability_t;

/**
 *  @brief      Classification for applications based on their genre.
 */
typedef enum _cncWebAppMgrClassificationGenre_t
{
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_NONE = 0,                                                        /**< The classification of an application is not specified */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_GENERIC,                                                         /**< The application cannot be classified under any of the current genres */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_HOME_SCREEN,                                                     /**< Home / idle / main screen type of application */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_MAIN_MENU,                                                       /**< Main-menu type of application */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_SUB_MENU,                                                        /**< Sub-menu type of application */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_MEDIA,                                                           /**< Provides multimedia functionality */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_SOCIAL_NETWORKING,                                               /**< Provides social-networking functionality */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_SETTINGS,                                                        /**< Provides settings functionality */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_UTILITY,                                                         /**< Applications providing utility features */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_EDUCATION,                                                       /**< Applications providing educational features */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_ENTERTAINMENT,                                                   /**< Applications providing entertainment features */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_GAME,                                                            /**< A game application */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_NEWS,                                                            /**< A news application */

    /* Add new application types above this line */
    CNC_WEBAPPMGR_CLASSIFICATION_GENRE_MAX
} cncWebAppMgrClassificationGenre_t;

/**
 *  @brief      Classification tags for applications.
 *              These are bit-fields and support the combination of more than one tag.
 */
typedef enum _cncWebAppMgrClassificationTags_t
{
    CNC_WEBAPPMGR_CLASSIFICATION_TAG_NONE                           = 0,

    /* Add new classification tags above this line */
    CNC_WEBAPPMGR_CLASSIFICATION_TAG_MAX
} cncWebAppMgrClassificationTags_t;

/**
 *  @brief      Requirements to be satisfied for an application to run on the specified RTE.
 *  @see        https://developer.chrome.com/extensions/manifest/requirements.html
 */
typedef enum _cncWebAppMgrRequirementTags_t
{
    CNC_WEBAPPMGR_REQUIREMENT_TAG_NONE                              = 0,
    CNC_WEBAPPMGR_REQUIREMENT_TAG_3D                                = 0x1,
    CNC_WEBAPPMGR_REQUIREMENT_TAG_PLUGIN                            = 0x2,

    /* Add new requirement tags above this line */
    CNC_WEBAPPMGR_REQUIREMENT_TAG_MAX
} cncWebAppMgrRequirementTags_t;

/**
 *  @brief      Suitability of an application or media for viewing by different audiences (for example, parental control).
 *  @todo       We need to select a standard rating system here.
 */
typedef enum _cncWebAppMgrContentRating_t
{
    CNC_WEBAPPMGR_CONTENT_RATING_INVALID = 0,

    /** @todo   This is yet to be completed. */
} cncWebAppMgrContentRating_t;

/**
 *  @brief      Brief meta-data of an application.
 *              When the list of applications is retrieved from the 'market', each entry would contain this information.
 *              This information may not be directly available in case of the 'Chrome Web Store'.
 */
typedef struct _cncWebAppMgrApplicationBriefMetaData_t
{
    CNC_CHAR                            name[CNC_WEBAPPMGR_APPLICATION_NAME_SIZE];                      /**< Name of the application */
    CNC_CHAR                            url[CNC_WEBAPPMGR_URL_SIZE];                                    /**< URL from where the application is or was downloaded */
    CNC_CHAR                            version[CNC_WEBAPPMGR_APPLICATION_VERSION_SIZE];                /**< Version of the application */
    CNC_CHAR                            description[CNC_WEBAPPMGR_APPLICATION_DESCRIPTION_SIZE];        /**< Description of the application */
    CNC_CHAR                            iconUrl[CNC_WEBAPPMGR_URL_SIZE];                                /**< Location of the icon representing the application.
                                                                                                                When retrieving the list of applications from the 'market', this will be a remote URL.
                                                                                                                When retrieving the list of applications currently installed, this will be a local URL. */
    cncWebAppMgrContentRating_t         contentRating;                                                  /**< Content rating is used to filter applications */
    cncWebAppMgrClassificationTags_t    classificationTags;                                             /**< Classification(s) of this application (a list of tags) */
    cncWebAppMgrClassificationGenre_t   classificationGenre;                                            /**< Predefined genre of this application */
} cncWebAppMgrApplicationBriefMetaData_t;

/**
 *  @brief      Detailed meta-data of an application.
 *              When the list of applications is retrieved from the 'market', an entry may not contain this information and it may need to be explicitly retrieved.
 *              This information may not be directly available in case of the 'Chrome Web Store'.
 */
typedef struct _cncWebAppMgrApplicationDetailedMetaData_t
{
    CNC_CHAR                            RTEversion[CNC_WEBAPPMGR_RTE_VERSION_SIZE];                     /**< Minimum firmware Run Time Engine version required to run this application */
    cncWebAppMgrRequirementTags_t       requirementTags;                                                /**< Requirement(s) to run this application (a list of tags) */
    cncWebAppMgrCapability_t            capabilities;                                                   /**< Capabilities of the application such as installation and run-time behaviour */
    CNC_CHAR                            author[CNC_WEBAPPMGR_APPLICATION_META_TEXT_SIZE];               /**< Name of the author (maybe empty if represented by a company) */
    CNC_CHAR                            company[CNC_WEBAPPMGR_APPLICATION_META_TEXT_SIZE];              /**< Name of the company (maybe empty if represented by an independent author) */
    CNC_CHAR                            publisher[CNC_WEBAPPMGR_APPLICATION_META_TEXT_SIZE];            /**< Name of the publisher (maybe empty if represented by an independent author) */
    CNC_UINT64                          downloadSize;                                                   /**< Total size of the package to be downloaded - used in download decision */
    CNC_UINT64                          installSize;                                                    /**< Total size of the application after installation - used in download decision */

    /** @todo   Need to add 'user rating' and 'reviews'. */
} cncWebAppMgrApplicationDetailedMetaData_t;

/**
 *  @brief      A combined structure containing both the brief and detailed meta-data of an application.
 */
typedef struct _cncWebAppMgrApplicationMetaData_t
{
    cncWebAppMgrApplicationBriefMetaData_t      briefMetaData;                                          /**< Structure specifying the brief meta-data associated with an application */
    cncWebAppMgrApplicationDetailedMetaData_t   detailedMetaData;                                       /**< Structure specifying the detailed meta-data associated with an application */
} cncWebAppMgrApplicationMetaData_t;

/**
 *  @brief      Structure of an installed application.
 *              This information is used by the following:
 *                  - package manager to remove or upgrade an application.
 *                  - application manager to list the installed applications in a menu.
 *                  - application manager to launch the installed application when selected in the menu.
 *              The package manager maintains two separate databases { system applications, user applications } with objects of this type.
 *                  - system applications = { non-removable applications, non-removable but upgradable applications }
 *                  - user applications = { removable or upgradable applications }
 */
typedef struct _cncWebAppMgrApplication_t
{
    cncWebAppMgrApplicationMetaData_t   metaData;                                                       /**< Structure containing the meta-data associated with an application */
    CNC_CHAR                            startUrl[CNC_WEBAPPMGR_PATH_SIZE];                              /**< Entry point of an application. This is a local URL and locates
                                                                                                                a HTML file in case of a web-application
                                                                                                                or a shared object in case of a native application. */
    CNC_CHAR                            packageDefinitionUrl[CNC_WEBAPPMGR_PATH_SIZE];                  /**< Location of the package definition file.
                                                                                                                This file contains information on the files installed on behalf of this application.
                                                                                                                This member maybe an empty string in case of { non-removable + non-upgradable applications } */
} cncWebAppMgrApplication_t;

/**
 *  @brief      High-level classification for the media requested for download.
 */
typedef enum _cncWebAppMgrMediaType_t
{
    CNC_WEBAPPMGR_MEDIA_TYPE_INVALID = 0,
    CNC_WEBAPPMGR_MEDIA_TYPE_AUDIO,                                                                     /**< High-level classification for audio types */
    CNC_WEBAPPMGR_MEDIA_TYPE_VIDEO                                                                      /**< High-level classification for video types */
} cncWebAppMgrMediaType_t;

/**
 *  @brief      Structure of meta data associated with media requested for download.
 */
typedef struct _cncWebAppMgrMediaMetaData_t
{
    CNC_CHAR                            name[CNC_WEBAPPMGR_MEDIA_NAME_SIZE];                            /**< Name of the media */
    CNC_CHAR                            url[CNC_WEBAPPMGR_URL_SIZE];                                    /**< URL from where it is or was downloaded */
    CNC_CHAR                            internetMediaType[CNC_WEBAPPMGR_IMT_TYPE_SIZE];                 /**< MIME type of the media */
    CNC_CHAR                            description[CNC_WEBAPPMGR_MEDIA_DESCRIPTION_SIZE];              /**< Description of the media */
    CNC_CHAR                            author[CNC_WEBAPPMGR_MEDIA_META_TEXT_SIZE];                     /**< Name of the author (maybe empty if represented by a company) */
    CNC_CHAR                            company[CNC_WEBAPPMGR_MEDIA_META_TEXT_SIZE];                    /**< Name of the company (maybe empty if represented by an independent author) */
    CNC_CHAR                            publisher[CNC_WEBAPPMGR_MEDIA_META_TEXT_SIZE];                  /**< Name of the publisher (maybe empty if represented by an independent author) */
    CNC_CHAR                            iconUrl[CNC_WEBAPPMGR_URL_SIZE];                                /**< Location of the icon representing the media, if available on the server.
                                                                                                                Before downloading the media from the web, this will be a remote URL.
                                                                                                                After downloading the media, this will be a local URL. */
    CNC_UINT64                          totalSize;                                                      /**< Total length in bytes */
    CNC_UINT64                          totalDuration;                                                  /**< Total duration (in seconds) */
    cncWebAppMgrMediaType_t             mediaType;                                                      /**< High-level classification of the media */
    cncWebAppMgrContentRating_t         contentRating;                                                  /**< Content rating is used to filter the media */
} cncWebAppMgrMediaMetaData_t;

/**
 *  @brief      Status of download.
 */
typedef enum _cncWebAppMgrDownloadStatus_t
{
    CNC_WEBAPPMGR_DOWNLOAD_STATUS_INVALID = 0,
    CNC_WEBAPPMGR_DOWNLOAD_STATUS_IN_PROGRESS,
    CNC_WEBAPPMGR_DOWNLOAD_STATUS_PAUSED,
    CNC_WEBAPPMGR_DOWNLOAD_STATUS_FAILED,
    CNC_WEBAPPMGR_DOWNLOAD_STATUS_CANCELED,
    CNC_WEBAPPMGR_DOWNLOAD_STATUS_COMPLETED
} cncWebAppMgrDownloadStatus_t;

/**
 *  @brief      Type of resource being downloaded.
 */
typedef enum _cncWebAppMgrDownloadType_t
{
    CNC_WEBAPPMGR_DOWNLOAD_TYPE_INVALID = 0,
    CNC_WEBAPPMGR_DOWNLOAD_TYPE_APPLICATION,
    CNC_WEBAPPMGR_DOWNLOAD_TYPE_MEDIA
} cncWebAppMgrDownloadType_t;

/**
 *  @brief      An opaque type for download requester ID.
 *  @details    A download requester ID is unique across different requesters.
 *              It is returned by the Download Manager when a new requester is registered and is used to determine the entity to which a download notification is to be delivered.
 */
typedef CNC_UINT32  cncWebAppMgrDownloadRequesterID_t;

/**
 *  @brief      Structure of the notification provided when a download completes.
 */
typedef struct _cncWebAppMgrDownloadNotification_t
{
    cncWebAppMgrDownloadRequesterID_t           ID;                                                     /**< ID of the entity that requested the download */
    cncWebAppMgrDownloadType_t                  downloadType;                                           /**< The metadata below is accessed based on this */
    union
    {
        cncWebAppMgrMediaMetaData_t             media;
        cncWebAppMgrApplicationMetaData_t       application;
    }                                           metaData;                                               /**< Metadata associated with the resource to be downloaded. This includes the URL from where the resource was downloaded as well as the total length downloaded */

    cncWebAppMgrDownloadStatus_t                status;                                                 /**< In this notification, this can be either failed, canceled or completed and not any of the other states. */
    CNC_CHAR                                    storagePath[CNC_WEBAPPMGR_PATH_SIZE];                   /**< Path where the downloaded resource is stored */
} cncWebAppMgrDownloadNotification_t;

/**
 *  @brief      Structure of a download request.
 */
typedef struct _cncWebAppMgrDownloadRequest_t
{
    cncWebAppMgrDownloadRequesterID_t           ID;                                                     /**< ID of the entity that is requesting the download */
    cncWebAppMgrDownloadType_t                  downloadType;                                           /**< The metadata below is accessed based on this */
    union
    {
        cncWebAppMgrMediaMetaData_t             media;
        cncWebAppMgrApplicationMetaData_t       application;
    }                                           metaData;                                               /**< Metadata associated with the resource to be downloaded. This includes the URL from where the resource should be downloaded as well as the length of the download. This maybe 0 if the length is unknown beforehand. */

    CNC_CHAR                                    storagePath[CNC_WEBAPPMGR_PATH_SIZE];                   /**< Path where the resource should be placed after download completes */
} cncWebAppMgrDownloadRequest_t;

/**
 *  @brief      Structure of an entry in the download list.
 *              This reflects the contents of the database maintained by the download manager.
 */
typedef struct _cncWebAppMgrDownloadListEntry_t
{
    cncWebAppMgrDownloadRequest_t               request;                                                /**< Download request information including the URL and total length of the download */
    CNC_UINT64                                  downloadedSize;                                         /**< Length downloaded so far */
    cncWebAppMgrDownloadStatus_t                status;                                                 /**< Current status of the download */
} cncWebAppMgrDownloadListEntry_t;

/**
 *  @brief      Journal entry for rolling back or completing interrupted installations.
 */
typedef struct _cncWebAppMgrApplicationInstallJournal_t
{
    /** @todo   Structure is yet to be defined. */
    CNC_UINT8                                   dummy;
} cncWebAppMgrApplicationInstallJournal_t;

/**
 *  @brief      Structure of an entry in the install list.
 *              This reflects the contents of the database maintained by the package manager.
 */
typedef struct _cncWebAppMgrApplicationInstallListEntry_t
{
    cncWebAppMgrApplicationMetaData_t           metaData;                                               /**< Structure containing the meta-data associated with an application */
    CNC_CHAR                                    storagePath[CNC_WEBAPPMGR_PATH_SIZE];                   /**< Path where the package file was placed after download completed */
    cncWebAppMgrApplicationInstallJournal_t     journal;
} cncWebAppMgrApplicationInstallListEntry_t;

/**
 *  @brief      Structure of an entry in the downloaded media list.
 *              This reflects the contents of the database for listing all the downloaded media.
 */
typedef struct _cncWebAppMgrDownloadedMediaListEntry_t
{
    cncWebAppMgrMediaMetaData_t                 metaData;                                               /**< Structure containing the meta-data associated with media */
    CNC_CHAR                                    storagePath[CNC_WEBAPPMGR_PATH_SIZE];                   /**< Path where the media file was placed after download completed */
} cncWebAppMgrDownloadedMediaListEntry_t;

#ifdef __cplusplus
}
#endif

#endif  // __CNC_WEBAPPTYPES_H__
