/* Define TRACE level override */
#ifdef TRACE_MODULE_APP_LEVEL
#if (TRACE_MODULE_APP_LEVEL >= 0)
#define TRACE_MODULE_LEVEL   TRACE_MODULE_APP_LEVEL
#endif
#endif

#include <stdint.h>
#include <pthread.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/timerfd.h>
#include <sys/select.h>
#include <directfb.h>
#include <cnc_app.h>
#include <cnc_system.h>
#include <cnc_ir.h>
#include <cnc_network/cnc_nw.h>
#include <cnc_network/cnc_nw_policy.h>

#include "cnc_system.h"
#include "cnc_media_wd_app_ui.h"
#include "cnc_wd/cnc_wivu.h"

#include <cnc_trace.h>

// Define TRACE module ID
#define TRACE_MODULE_ID    CNC_TRACE_MODULE_APP 

#define red   "\033[0;31m"        /* 0 -> normal ;  31 -> red */
#define none   "\033[0m"        /* to flush the previous property */

static CNC_SINT32 appOsdVirtualLayerId;
#define OSD_SCREEN_MODE         OSD_MODE_720P60
#define APP_SCREEN_WIDTH        1280
#define APP_SCREEN_HEIGHT       720
#define APP_FONT_PATH "/mnt/datafs/fonts/Vera.ttf"
#define APP_BG_COLOR {0, 0, 0, 0}

void osdClear();
void osdHide();
void osdText(char *);
void osdBanner(char *);
void osdSetAlpha(CNC_SINT32 alpha);

// Helper to set the alpha and enable/disable graphics to save DDR bandwidth as needed.
void osdSetAlpha(CNC_SINT32 alpha)
{
    static int graphicsEnabled = 0;

    if (alpha == 0) // completely transparent (invisible) -- disable graphics if enabled
    {
        CNC_DEV_SetGraphicsAlpha(OSD_LAYER_0, alpha);
        if (graphicsEnabled)
        {
            CNC_DEV_DisableGraphics(OSD_LAYER_0); // disable graphics to save DDR bandwidth
            graphicsEnabled = 0;
        }
    }
    else // visable graphics -- enable graphics if disabled
    {
        if (graphicsEnabled == 0)
        {
            CNC_DEV_EnableGraphics(OSD_LAYER_0); // enable graphics so we can see the OSD
            graphicsEnabled = 1;
        }
        CNC_DEV_SetGraphicsAlpha(OSD_LAYER_0, alpha);
    }
}

// Clear the screen
void osdClear()
{
    const cncOSDColor_t color = APP_BG_COLOR;

    printf("%s\n", __FUNCTION__);

    osdSetAlpha(255);
    CNC_OSD_StartDraw(0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_FillArea(&color, 0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_EndDraw();
}

// Make screen transparent
void osdHide()
{
    printf("%s\n", __FUNCTION__);

    const cncOSDColor_t color = APP_BG_COLOR;

    osdSetAlpha(0);
    
    CNC_OSD_StartDraw(0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_FillArea(&color, 0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_EndDraw();
}

// Display text on the center of the screen
void osdText( char *text )
{
    printf("%s%s%s\n", red, text, none);
    const cncOSDColor_t color = APP_BG_COLOR;

    osdSetAlpha(255);
    CNC_OSD_StartDraw(0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_FillArea(&color, 0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_SetFontForeground(255, 255, 255, 0);
    CNC_OSD_SetFontSize(30);
    CNC_OSD_DrawFontEx(text, APP_SCREEN_WIDTH/2, APP_SCREEN_HEIGHT/2, 100, FF_CENTER);
    CNC_OSD_EndDraw();

}


// Display text on the center of the screen
void osdBanner( char *text )
{
    printf("%s%s%s\n", red, text, none);
    const cncOSDColor_t color = {10, 10, 10, 50};

    CNC_OSD_StartDraw(0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_ClearArea(0, 0, APP_SCREEN_WIDTH, APP_SCREEN_HEIGHT);
    CNC_OSD_FillArea(&color, 100, 50, APP_SCREEN_WIDTH-200, 50);
    CNC_OSD_SetFontForeground(255, 255, 255, 0);
    CNC_OSD_SetFontSize(30);
    CNC_OSD_DrawFontEx(text, 150, 60, 100, FF_LEFT);
    CNC_OSD_EndDraw();
    osdSetAlpha(255);

}

static void appDirectFbInitialize(void)
{
    E_cncErrCode_t                e_cncErrorCode;
    cncDfbVirtualLayerInfo_t      s_virtualLayerInfo;
    cncDfbPhysicalLayerInfo_t     s_physicalLayerInfo;

    s_physicalLayerInfo.flags = CNC_DFB_PHYSICAL_LAYER_FLAG_SET_COOPERATIVE_LEVEL;
    s_physicalLayerInfo.dfbCooperativeLevel = DLSCL_ADMINISTRATIVE;
    s_physicalLayerInfo.dfbLayerId = DLID_PRIMARY;
    e_cncErrorCode = CNC_DFB_Init(1, &s_physicalLayerInfo);
    if (e_cncErrorCode != E_CNC_ERR_NONE)
    {
        CNC_TRACE_ERROR(0,"CNC_DFB_Init: Failed to initialize, error code %d\n", e_cncErrorCode);
    }
    else
    {
        /* Create the virtual layer required by the OSD */
        s_virtualLayerInfo.flags = CNC_DFB_VIRTUAL_LAYER_FLAG_NONE;
        s_virtualLayerInfo.type = CNC_DFB_VIRTUAL_LAYER_TYPE_CREATE_WINDOW_SURFACE;
        s_virtualLayerInfo.actualArea.x = 0;
        s_virtualLayerInfo.actualArea.y = 0;
        s_virtualLayerInfo.actualArea.width = APP_SCREEN_WIDTH;
        s_virtualLayerInfo.actualArea.height = APP_SCREEN_HEIGHT;
        s_virtualLayerInfo.visualArea.x = 0;
        s_virtualLayerInfo.visualArea.y = 0;
        s_virtualLayerInfo.visualArea.width = s_physicalLayerInfo.width;
        s_virtualLayerInfo.visualArea.height = s_physicalLayerInfo.height;
        s_virtualLayerInfo.opaqueArea.x = 0;
        s_virtualLayerInfo.opaqueArea.y = 0;
        s_virtualLayerInfo.opaqueArea.width = APP_SCREEN_WIDTH;
        s_virtualLayerInfo.opaqueArea.height = APP_SCREEN_HEIGHT;
        s_virtualLayerInfo.parentPhysicalLayerIndex = 0;
        s_virtualLayerInfo.dfbPixelFormat = DSPF_ARGB;
        s_virtualLayerInfo.dfbSurfaceCapabilities = (DFBSurfaceCapabilities)(DSCAPS_NONE);

        e_cncErrorCode = CNC_DFB_CreateLayer(&appOsdVirtualLayerId, &s_virtualLayerInfo);
        if (e_cncErrorCode != E_CNC_ERR_NONE)
        {
            CNC_TRACE_ERROR(0,"Failed to create Virtual Layer for OSD\n");
            appOsdVirtualLayerId = -1;
        }
        else
        {
            CNC_DFB_EnableBlending(appOsdVirtualLayerId);
        }
    }
}

static void appSystemInit(void)
{
    cncDeviceContainer_t *cncDeviceInstance = NULL;

    CNC_System_Init();

    CNC_TRACE_Initialize();
    CNC_TRACE_RegisterCallback(g_CNC_TRACE_Syslog);

    CNC_AV_Init();

    CNC_DEV_Init();
    CNC_DEV_SetGraphicsConfig(OSD_LAYER_0, defaultGraphicsConfig);
    CNC_DEV_OpenHDOutputCheckHpd(TVOUT_MODE_USE_EDID, 1);  // This should be a blocking call
    CNC_DEV_DisableGraphicsKeyColor(OSD_LAYER_0);
    cncDeviceInstance = CNC_DEV_GetDeviceParams();

    appDirectFbInitialize();
    
    CNC_OSD_Init(cncDeviceInstance->graphic0_handle);

    CNC_OSD_InitFont(UTF_8, APP_FONT_PATH);
}


int main(int argc, char* argv[])
{
    appSystemInit();

    osdText("Actiontec MyWirelessTV");
    
    while(1) {
        sleep(10);
    }
    
    CNC_TRACE_Deinitialize();

    return 0;
}
