/*
 * Copyright (c) 2003-2012, Cavium Networks. All rights reserved.
 *
 * This Software is the property of Cavium Networks.  The Software and all
 * accompanying documentation are copyrighted.  The Software made available
 * here constitutes the proprietary information of Cavium Networks.  You
 * agree to take reasonable steps to prevent the disclosure, unauthorized use
 * or unauthorized distribution of the Software.  You shall use this Software
 * solely with Cavium hardware.
 *
 * Except as expressly permitted in a separate Software License Agreement
 * between You and Cavium Networks, you shall not modify, decompile,
 * disassemble, extract, or otherwise reverse engineer this Software.  You
 * shall not make any copy of the Software or its accompanying documentation,
 * except for copying incident to the ordinary and intended use of the
 * Software and the Underlying Program and except for the making of a single
 * archival copy.
 *
 * This Software, including technical data, may be subject to U.S.  export
 * control laws, including the U.S.  Export Administration Act and its
 * associated regulations, and may be subject to export or import regulations
 * in other countries.  You warrant that You will comply strictly in all
 * respects with all such regulations and acknowledge that you have the
 * responsibility to obtain licenses to export, re-export or import the
 * Software.
 *
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 * AND WITH ALL FAULTS AND CAVIUM MAKES NO PROMISES, REPRESENTATIONS OR
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT
 * TO THE SOFTWARE, INCLUDING ITS CONDITION, ITS CONFORMITY TO ANY
 * REPRESENTATION OR DESCRIPTION, OR THE EXISTENCE OF ANY LATENT OR PATENT
 * DEFECTS, AND CAVIUM SPECIFICALLY DISCLAIMS ALL IMPLIED (IF ANY) WARRANTIES
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR
 * PURPOSE, LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET
 * POSSESSION OR CORRESPONDENCE TO DESCRIPTION.  THE ENTIRE RISK ARISING OUT
 * OF USE OR PERFORMANCE OF THE SOFTWARE LIES WITH YOU.
 *
 */

/** \file cavm_ipc.h
    \brief This file contains the protocol syntax and APIs for communicating on the pipe FIFOs
           in a Wireless Display system.
*/

/*! \page IPC Interprocess Communication

   All the processes communicate with netHDcontrol via Unix FIFO's which are created in the rc script
   through mkfifo command.  See \ref cavm_ipc.h for details on the pipes and APIs.

   -#   \c /cavium/avdp_to_nethd_pipe, \c /cavium/nethd_to_avdp_pipe.
   \li    This is used by netHDcontrol to control avDataPath. This can be used to set A/V ports, A/V types and IP address etc.
   -#   \c /cavium/wifi_to_nethd_pipe, \c /cavium/nethd_to_wifi_pipe
   \li    This is used for communication between netHDcontrol and wifi_mgmt process.
   -#   \c /cavium/wfd_to_nethd_pipe, \c /cavium/nethd_to_wfd_pipe
   \li    This is used for communication between netHDcontrol and wfd_rtsp process
   -#   \c /cavium/wivu_to_nethd_pipe, \c /cavium/nethd_to_wivu_pipe
   \li    This is used for communication between netHDcontrol and netHDdecode process
   -#   \c /cavium/widi_to_nethd_pipe, \c /cavium/nethd_to_widi_pipe
   \li    This is used for communication between netHDcontrol and sample_widi process
   -#   \c /cavium/dlna_to_nethd_pipe, \c /cavium/nethd_to_dlna_pipe
   \li    This is used for communication between netHDcontrol and the DLNA process, cavmWirelessDisplayDongle
   -#   \c /cavium/simplegui_pipe
   \li    This is used by scripts to control the GUI. Some of the scripts are invoked by netHDcontrol.
   -#   \c /cavium/ledctrl_pipe
   \li    This is used by scripts to control the LED. Some of the scripts are invoked by netHDcontrol.

 */

#ifndef _CAVM_IPC_H_
#define _CAVM_IPC_H_

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <syslog.h>
#include <limits.h>
#include <time.h>

/* OS-specific include files */
#include <linux/sockios.h>
#include <linux/types.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>


#define PROTOCOL_NAME_DLNA   "DLNA::"
#define PROTOCOL_NAME_WIDI   "WIDI::"
#define PROTOCOL_NAME_WIVU   "WIVU::"
#define PROTOCOL_NAME_WFD    "WFD::"
#define WIFI_MGMT_NAME       "WIFI::"


#define MAX_MSG_SIZE (2*4096) //1024
#define MAX_NPEERS 128
#define PERSIST_DIR "/mnt/jffs2/cavium/crumbs/"
/* This 2 seconds short scan interval is needed by WIDI detection */
#define WIFI_SCAN_INTERVAL 30000
#define WIFI_WIDI_SCAN_INTERVAL 2000
#define WPS_WINDOW_MSECS 120000

//#define BUF_LEN                     (256)
#define MESSAGE_BUFFER_SIZE         (4096)


/* DA -> WiVu commands */

/**@{*/

/** \def CAVM_IPC_CMD_DISCOVERY
  * DA/netHDcontrol to WiVu Discovery command
  *
  * \par
  * DA to WiVu commands:
  * \li  "DISCOVERY VALUE=0 | 1" will set discovery on/off.
  * \li  "DISCOVERY" will query for current discovery.
  * \par
  * Response from WiVu:
  * \li  "STATUS=SUCCEED|FAILED  VALUE= 0 | 1"
*/
#define CAVM_IPC_CMD_DISCOVERY          "DISCOVERY"

/** \def CAVM_IPC_CMD_PLUGGED
  * DA to WiVu Plugged command
  *
  * \par
  * DA to WiVu commands:
  * \li  "PLUGGED VALUE= 0 | 1" will set plugged on/off.
  * \li  "PLUGGED" will query for current plug status.
  * \par
  * Response from WiVu
  * \li  "PLUGGED STATUS=SUCCEED|FAILED  VALUE= 0 | 1"
*/
#define CAVM_IPC_CMD_PLUGGED            "PLUGGED"

/** \def CAVM_IPC_CMD_UUID
  * DA to WiVu UUID commands
  *
  * \par
  * DA to WiVu commands:
  * \li  "UUID VALUE=String of max 36bytes" will set UUID.
  * \li  "UUID" will query for current discovery.
  * \par
  * Response from WiVu:
  * \li  "UUID STATUS=SUCCEED|FAILED  VALUE=String"
*/
#define CAVM_IPC_CMD_UUID            "UUID"

/** \def CAVM_IPC_CMD_INTERFACE
  * netHDcontrol to WiVu interface designation
  *
  * \par
  * netHDcontrol to WiVu commands:
  * \li  "INTERFACE VALUE=eth1" will reconfigure interface.
  * \par
  * Response from WiVu:
  * \li  "INTERFACE STATUS=SUCCEED|FAILED  VALUE=eth1"
*/
#define CAVM_IPC_CMD_INTERFACE          "INTERFACE"

/** \def CAVM_IPC_CMD_SWUPGRADE
  * netHDcontrol to WiVu Software Upgrade command
  *
  * \par
  * netHDcontrol to WiVu commands:
  * \li  "SWUPGRADE VALUE=string" will inform netHDdecode that upgrade is available.
  * There is no response from WiVu.
  * On tablet this will sent by netHDencode to encoder.
*/
#define CAVM_IPC_CMD_SWUPGRADE          "SWUPGRADE"

/** \def CAVM_IPC_CMD_SWVERSION
  * DA to WiVU Software version query
  *
  * \par
  * DA to WiVu command:
  * \li  "SWVERSION" will query netHDencode for current version and any available upgrade version
  * \par
  * Response from WiVu:
  * \li  "SWVERSION STATUS=SUCCEED|FAILED  VALUE=current version, upgrade version".
*/
#define CAVM_IPC_CMD_SWVERSION          "SWVERSION"

/** \def CAVM_IPC_CMD_TVOUTMODE
  * DA to WiVu specification of TV output mode
  *
  * \par
  * DA to WiVu commands:
  * \li  "TVOUTMODE VALUE=10" will  configure TVOUTMODE_XXX as defined in csapi.h.
  * \par
  * Response from WiVu
  * \li  "TVOUTMODE STATUS=SUCCEED|FAILED  VALUE=10"
*/
#define CAVM_IPC_CMD_TVOUTMODE          "TVOUTMODE"

/** \def CAVM_IPC_CMD_DATAFLOW
  * netHD to DMR/avDatapath control of whether network data should be processed
  *
  * \par
  * netHDcontrol to DMR/avDatapath commands:
  * \li  "DATAFLOW VALUE=STOP" tells the application to stop processing data
  *      by tearing down existing connections until a DATAFLOW VALUE=START message is received
  * \li  "DATAFLOW VALUE=START" tells the application to resume processing data
  * \li  "DATAFLOW VALUE=PAUSE" tells the application to pause processing of data
  *      (sent after received DATAFLOW VALUE=INACTIVE)
  * \par
  * Response from DMR/avDatapath
  * \li  "DATAFLOW VALUE=STOPPED" acknowledgment of the STOP command.
  * \li  "DATAFLOW VALUE=INACTIVE" indication the application has not received data in a long period.
  * \li  "DATAFLOW VALUE=ACTIVE" indication that new data has been received and queued after being INACTIVE.
  *      It releases the queued data once the DATAFLOW VALUE=START is received.
  * \li  "DATAFLOW VALUE=PAUSED" acknowledgment of the PAUSE command
*/
#define CAVM_IPC_CMD_DATAFLOW          "DATAFLOW"

/** \def CAVM_IPC_CMD_POWER_MGMT
  * netHD to power management control of doze/wake functionality
  *
  * \par
  * netHDcontrol to PowerMgmt commands:
  * \li  "POWER_MGMT STATUS=DOZE" tells the power mgmt to enter "Doze" mode. It assumes
  *      all applications have been already stopped feeding data into the the A/V pipeline.
  * \li  "POWER_MGMT STATUS=WAKE" tells the power mgmt to wake up.
  * \par
  * Response from Power Mgmt
  * \li  "POWER_MGMT STATUS=AWAKE" acknowledgment of the WAKE command.
*/
#define CAVM_IPC_CMD_POWER_MGMT          "POWER_MGMT"

/** \def CAVM_IPC_CMD_CEC_MSG
  * simpleGUI to netHDcontrol CEC commands.
  *
  * \par
  * The following CEC opcodes are forwarded by simpleGUI to netHDcontrol CEC commands:
  * \li  Currently, the following are communicated to netHDcontrol:
  *
  * \li  "CEC VALUE=STANDBY" tells netHDcontrol that the TV is going to standby mode.
  *      This should cause netHD control to discard packets and enter the doze 2 state.
  *
  * \li  "CEC VALUE=ON" tells netHDcontrol to wake the dongle from doze and enable the
  *      the processing of A/V packets.
  *
*/
#define CAVM_IPC_CMD_CEC_MSG               "CEC"

/**@}*/



/* WiVu -> DA commands */

/**@{*/

/** \def CAVM_IPC_CMD_WIVUSTART
  * WiVu to DA indication that peer has been discovered.
  *
  * \par
  * WiVu to DA commands:
  * \li  "WIVUSTART IP=192.168.162.130 VIDEO= UDP port, 0  AUDIO=UDP port, 0= NETHD 1=LPCM 44.1KHZ, 2=LPCM 48Khz"
  * If plugged status is off DA will need to send the "PLUGGED" command
*/
#define CAVM_IPC_CMD_WIVUSTART        "WIVUSTART"

/** \def CAVM_IPC_CMD_WIDISTART
  * WiDi to DA indication that peer has been discovered. See \ref CAVM_IPC_CMD_WIVUSTART.
*/
#define CAVM_IPC_CMD_WIDISTART        "WIDISTART"

/** \def CAVM_IPC_CMD_WFDSTART
  * WFD to DA indication that peer has been discovered. See \ref CAVM_IPC_CMD_WIVUSTART.
*/
#define CAVM_IPC_CMD_WFDSTART         "WFDSTART"

/** \def CAVM_IPC_CMD_WIVUSTOP
  * WiVu to DA indication that peer wants to stop video.
*/
#define CAVM_IPC_CMD_WIVUSTOP        "WIVUSTOP"

/** \def CAVM_IPC_CMD_WIDISTOP
  * WIDI to DA indication that peer wants to stop video.
*/
#define CAVM_IPC_CMD_WIDISTOP        "WIDISTOP"

/** \def CAVM_IPC_CMD_WFDSTOP
  * WFD to DA indication that peer wants to stop video.
*/
#define CAVM_IPC_CMD_WFDSTOP         "WFDSTOP"

/** \def CAVM_IPC_CMD_DMR_START
  * NethdControl to DA indication to start DMR after WiFi-Mgmt sends notififcation about
  * availability of network interface
*/
#define CAVM_IPC_CMD_DMR_START         "DMRSTART"

/** \def CAVM_IPC_CMD_DMR_STOP
  * NethdControl to DA indication to stop DMR without going through Post-Terminate procedure.
*/
#define CAVM_IPC_CMD_DMR_STOP         "DMRSTOP"

/** \def CAVM_IPC_CMD_DMR_RESTART
  * NethdControl to DA indication to restart DMR without going through Pre-Init procedure.
*/
#define CAVM_IPC_CMD_DMR_RESTART      "DMRRESTART"


/**@}*/


/**@{*/
/** \def CAVM_IPC_CMD_ROTATION
  * DA Rotation command  FIXME
  *
  * \par
  * DA to  commands:
  * \li "ROTATION"
  * \par
  * Response from
  * \li  "ROTATION"
*/
#define CAVM_IPC_CMD_ROTATION      "ROTATION"

/** \def CAVM_IPC_CMD_FACTORY_DEFAULT
  * This command is to get current FIXME
  *
  * \par
  * DA to commands:
  * \li "FACTORYDEFAULT"
  * \par
  * Response from
  * \li  "FACTORYDEFAULT STATUS=SUCCEED/FAILED"
*/
#define CAVM_IPC_CMD_FACTORY_DEFAULT   "FACTORYDEFAULT"


/** \def CAVM_IPC_CMD_CONNECTION_WFD_FAILED
  *
  * \par
  * DA to commands:
  * \li "WFD CONNECTION FAILED"
  * \par
  * Response from
  * \li  "PBC press for wfd pairing"
*/

#define CAVM_IPC_CMD_CONNECTION_WFD_FAILED    "WFD_ATTEMPT_FAILED"


/** \def CAVM_IPC_CMD_UPGRADE
  * DA Upgrade command  FIXME
  *
  * \par
  * DA to  commands:
  * \li "UPGRADE"
  * \par
  * Response from
  * \li  "UPGRADE"
*/
#define CAVM_IPC_CMD_UPGRADE       "UPGRADE"

/**@}*/

/* DA -> WIFI commands */

/**@{*/

/** \def CAVM_IPC_CMD_LIST
  * This command provides list of Access Points / peers to netHDcontrol
  *
  * \par
  * Commands to WIFI
  * \li "LIST"
  * \par
  * Response from WiFi
  * \li
  *  "LIST SSID=<ssid_name> MAC=<bssid>  P2P=<0|1> PERSIST=<0|1> STATUS=<ON|OFF>"
*/
#define CAVM_IPC_CMD_LIST          "LIST"

/** \def CAVM_IPC_CMD_DLNA
  * DA to WiFi DLNA command so that WiFi scan will be turned off if DLNA is running
  *
  * \par
  * DA to WiVu commands:
  * \li "DLNA START"  -- DLNA is running
  * \par
  * DA to WiVu commands:
  * \li "DLNA STOP" -- DLNA  is Not Running
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_DLNA		   "DLNA"

/** \def CAVM_IPC_CMD_PBC
  * netHDcontrol to WiFi PBC command when PBC pressed
  *
  * \par
  * netHDcontrol to Wifi commands:
  * \li "PBC"  -- indicates PBC was pressed
  * \par
  * Responses from WiFi
  * \li  "PBC STATUS=FAILED" -- push button pairing failed
  * \li  "PBC STATUS=SUCCESS WIFI_CONNECT" -- push button pairing succeeded
  * \li  "PBC P2P_WAIT" -- Waiting for p2p
*/
#define CAVM_IPC_CMD_PBC           "PBC"

/** \def CAVM_IPC_CMD_DISPLAY_PIN
  * This commands display current pin configured
  *
  * \par
  *  Commands to WiFi
  * \li "DISPLAYPIN"
  * \par
  * Response from WiFi
  * \li  "DISPLAYPIN MAC=uu:vv:ww:xx:yy:zz PIN=<pinvalue> "
*/
#define CAVM_IPC_CMD_DISPLAY_PIN   "DISPLAYPIN"

/** \def CAVM_IPC_CMD_DISPLAY_PBC
  * WiFi to netHDcontrol PBC command
  *
  * \par
  * WiFi to netHDcontrol command:
  * \li "DISPLAYPBC MAC=uu:vv:ww:xx:yy:zz" -- MAC ADDRESS of DONGLE sent to netHDcontrol"
  * \par
  * No response from netHDcontrol
*/
#define CAVM_IPC_CMD_DISPLAY_PBC   "DISPLAYPBC"

/** \def CAVM_IPC_CMD_SETPIN
  * Command to configure a PIN value to be used for connect to peer or external AP
  *
  * \par
  * Command to WiFi
  * \li  "SETPIN PIN=<value>"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_SETPIN        "SETPIN"

/** \def CAVM_IPC_CMD_WPSCFG
  * Commmand to configure WPS pairing method
  *
  * \par
  * Wifi to netHDcontrol command:
  * \li "WPSCFG VALUE=<PBC|PIN|KEYPAD|UNKNOWN>"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_WPSCFG        "WPSCFG"

/** \def CAVM_IPC_CMD_ADDNETWORK
  * This command adds an external AP to wifi manager database.
  *
  * \par
  * Command to WiFi:
  * \li "ADDNETWORK MAC=<bssid> SSID=<ssidname> PIN=<valueofpeer>"
  * \li "ADDNETWORK SSID=<ssidname> SECTYPE=<OPEN|WEP|WPA2AES|WPA2TKIP|WPAAES|WPATKIP> PIN=<passphrase>"
  * \par
  * Command from WiFi to netHDcontrol:
  * \li "ADDNETWORK "
*/
#define CAVM_IPC_CMD_ADDNETWORK       "ADDNETWORK"


/** \def CAVM_IPC_CMD_CONNECT
  * This command sends connect to a peer/external AP via netHDcontrol. Also used for initiating WFD
  *
  * \par
  * Command to WiFi:
  * \li "CONNECT MAC=<bssid> SSID=<ssidname> PIN=<valueofpeer>"
  * \li "CONNECT SSID=<ssidname> KEY=<key> SECTYPE=<NONE|WEP|WPA-PSK|> INDEX=<index>"
  * \par
  * Command from WiFi to netHDcontrol:
  * \li "CONNECT MAC=<bssid> NIF=<name> WFD_IP=w.x.y.z WFD_PORT=<port> STATUS=SUCCEED"
  * \li "CONNECT MAC=<bssid> NIF=<name> STATUS=SUCCEED"
  * \li "CONNECT MAC=<bssid> STATUS=<SUCCEED|FAILURE>"
*/
#define CAVM_IPC_CMD_CONNECT       "CONNECT"

/** \def CAVM_IPC_CMD_STA_CONNECTED
  * This command sends sta connected event via netHDcontrol.
  * It is sent out by wifi layer when a station connects to us.
  *
  * \par
  * Command from WiFi to netHDcontrol:
  * \li "STA_CONNECTED MAC=<station macaddress> NIF=<name>"
*/
#define CAVM_IPC_CMD_STA_CONNECTED       "STA_CONNECTED"

/** \def CAVM_IPC_CMD_CONNECT
  * This command sends connect to a peer/external AP via netHDcontrol. Also used for initiating WFD
  *
  * \par
  * Command to WiFi:
  * \li "CONNECT MAC=<bssid> SSID=<ssidname> PIN=<valueofpeer>"
  * \li "CONNECT SSID=<ssidname> KEY=<key> SECTYPE=<NONE|WEP|WPA-PSK|> INDEX=<index>"
  * \par
  * Command from WiFi to netHDcontrol:
  * \li "CONNECT MAC=<bssid> NIF=<name> WFD_IP=w.x.y.z WFD_PORT=<port> STATUS=SUCCEED"
  * \li "CONNECT MAC=<bssid> NIF=<name> STATUS=SUCCEED"
  * \li "CONNECT MAC=<bssid> STATUS=<SUCCEED|FAILURE>"
*/
#define CAVM_IPC_CMD_CONNECTING       "CONNECTING"

/** \def CAVM_IPC_CMD_DISCONNECT
  This command send disconnect request of peer/external AP via netHDcontrol
  *
  * \par
  * Commands to Wifi:
  * \li "DISCONNECT MAC=<bssid>"
  * \li "DISCONNECT NIF=<nif>"
  * \par
  * No response from WiFi
  *
*/
#define CAVM_IPC_CMD_DISCONNECT    "DISCONNECT"



/** \def CAVM_IPC_CMD_CONNECTIONT_TO_EXT_AP_FAILED
  This command send to inform connection to external AP failed.
  *
  * No response from WiFi
  *
*/
#define CAVM_IPC_CMD_CONNECTION_TO_EXT_AP_FAILED    "CONNECT_TO_EXT_AP_FAILED"
#define CAVM_IPC_CMD_CONNECTIONT_TO_EXT_AP_FAILED   "CONNECT_TO_EXT_AP_FAILED"




/** \def CAVM_IPC_CMD_ASSOCIATE
  * This commands sends the ASSOCIATE request of particular bssid value to netHDcontrol
  *
  * \par
  * Commands from Wifi to netHDcontrol:
  * \li
  *  "ASSOCIATE MAC=<bssid>"
  * \par
  * No response from netHDcontrol
*/
#define CAVM_IPC_CMD_ASSOCIATE     "ASSOCIATE"

/** \def CAVM_IPC_CMD_DISASSOCIATE
  * This commands sends the DISASSOCIATE request of particular bssid value to netHDcontrol
  *
  * \par
  * Command from WiFi to netHDcontrol:
  * \li "DISASSOCIATE MAC=<bssid>"
  * \par
  * No response from netHDcontrol
*/
#define CAVM_IPC_CMD_DISASSOCIATE  "DISASSOCIATE"

/** \def CAVM_IPC_CMD_FORGET
  * This command is to make particular bssid as non persistent
  *
  * \par
  * WiFi commands:
  * \li "FORGET MAC<bssid>"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_FORGET        "FORGET"

/** \def CAVM_IPC_CMD_NICKNAME
  * This command is to update Nickname of particular bssid if it is persistent mode(ie storing
  * of peer features like ssid,nickname, mac, pin , security mode for later use)
  *
  * \par
  * Command to wifi
  * \li "NICKNAME = <stringname>"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_NICKNAME      "NICKNAME"

/** \def CAVM_IPC_CMD_PAIRING
  * This command is used to send pairing success or failure to netHDcontrol
  *
  * \par
  * WiFi to netHDcontrol
  * \li  "PAIRING STATUS=<SUCCESS|FAILURE> "
  * \par
  * No response from netHDcontrol
*/
#define CAVM_IPC_CMD_PAIRING       "PAIRING"

/** \def CAVM_IPC_CMD_IR
  * This command is input PIN keys via IR
  *
  * \par
  * Commands to WiFi:
  * \li * "LOCALIR KEY=<value>"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_IR            "LOCALIR"

/** \def CAVM_IPC_CMD_PEER_HRTBEAT
  * This command is used to send current hearbeattimer value of mac at time of connect request to netHDcontrol
  *
  * \par
  * Command from  WiFi to netHDcontrol:
  * \li "PEER_HRTBEAT MAC=<bssid> STATUS=<SUCCEED|FAILURE"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_PEER_HRTBEAT  "PEER_HRTBEAT"

/** \def CAVM_IPC_CMD_DHCPD_NOTIFY
  * This command is used for DHCPD notification processing internally by WIFI.
  *
  * \par
  * WiFi commands:
  * \li "DHCPD_NOTIFY"
  * \par
  * No response from WiFi. Initiates CONNECT.
*/
#define CAVM_IPC_CMD_DHCPD_NOTIFY   "DHCPD_NOTIFY"

/** \def CAVM_IPC_CMD_ZCIP_NOTIFY
  * This command is used for Zero config auto IP notification processing internally by WIFI.
  *
  * \par
  * WiFi commands:
  * \li "ZCIP_NOTIFY"
  * \par
  * No response from WiFi. Initiates CONNECT.
*/
#define CAVM_IPC_CMD_ZCIP_NOTIFY   "ZCIP_NOTIFY"

/**@}*/


/**@{*/

/** \def CAVM_IPC_CMD_LEDON
  * netHDcontrol to LED command to turn on an LED
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDON VALUE=2" turns on LED on GPIO 2
*/
#define CAVM_IPC_CMD_LEDON            "LEDON"

/** \def CAVM_IPC_CMD_LEDOFF
  * netHDcontrol to LED command to turn off an LED
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDOFF VALUE=2"  turns off LED on GPIO 2
*/
#define CAVM_IPC_CMD_LEDOFF           "LEDOFF"

/** \def CAVM_IPC_CMD_LEDBLINK
  * netHDcontrol to LED command to blink an LED
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDBLINK VALUE=2" blinks LED on GPIO 2
*/
#define CAVM_IPC_CMD_LEDBLINK         "LEDBLINK"

/** \def CAVM_IPC_CMD_LEDBLINK2
  * netHDcontrol to LED command to blink a two LEDs
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDBLINK2 VALUE=2,3" blinks LEDs on GPIOs 2 and 3
*/
#define CAVM_IPC_CMD_LEDBLINK2        "LEDBLINK2"

/** \def CAVM_IPC_CMD_GLOWDUALLEDS
  * netHDcontrol to LED command to blink a two LEDs simultaneously
  * to produce different color
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDDUALBLINK VALUE=2,3" blinks LEDs on GPIOs 2 and 3
*/
#define CAVM_IPC_CMD_GLOWDUALLEDS     "LEDDUALBLINK"

/** \def CAVM_IPC_CMD_LEDBLINK3
  * netHDcontrol to LED command to blink an LED with a given duty
  * cycle and interval
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDBLINK3 VALUE=2,100,400" turns GPIO 2 ON for 100 ms,
  * OFF for 400 ms, for a duty cycle of 20% and a frequency of
  * 2 Hz.
*/
#define CAVM_IPC_CMD_LEDBLINK3        "LEDBLINK3"

/** \def CAVM_IPC_CMD_LEDBLINKF
  * netHDcontrol to LED command to blink an LED at a different rate
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDBLINKF VALUE=2" fast blinks an LED on GPIO 2
*/
#define CAVM_IPC_CMD_LEDBLINKF        "LEDBLINKF"

/** \def CAVM_IPC_CMD_LEDBLINKF2
  * netHDcontrol to LED command to fast blink a two LEDs
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDBLINKF2 VALUE=2,3" fast blinks LEDs on GPIOs 2 and 3
*/
#define CAVM_IPC_CMD_LEDBLINKF2       "LEDBLINKF2"

/**@}*/

/** \def CAVM_IPC_CMD_LEDBLINKRAPID
  * netHDcontrol to LED command to fast blink a two LEDs
  *
  * \par
  * netHDcontrol to LED command:
  * \li "LEDBLINKRAPIDLY VALUE=2" fast blinks LEDs on GPIO 2
*/
#define CAVM_IPC_CMD_LEDBLINKRAPID       "LEDBLINKRAPID"

/**@}*/

/**@}*/

/** \def CAVM_IPC_CMD_LANGUAGE
  * netHDcontrol to WIDI command to navigate supported languages forwards/backwards
  *
  * \par
  * netHDcontrol to WIDI command:
  * \li "set language en" > /cavium/nethd_to_widi_pipe
*/
#define CAVM_IPC_CMD_LANGUAGE       "language"

/**@}*/

/**@{*/
/** \def FUJI_CMD_GUI_FULLSCREEN
  * Interactive GUI to netHDcontrol/WiFi command to send Enable discovery service when GUI is on
  *
  * \par
  * WiFi commands:
  * \li "GUIFULLSCREEN STATUS=<ON|OFF>"
  * \par
  * No response from WiFi
*/
#define FUJI_CMD_GUI_FULLSCREEN     "GUIFULLSCREEN"

/** \def FUJI_CMD_GUI_INPUTSELECT
  * Interactive GUI to netHDcontrol/WiFi to indicate that a source input select screen is on and it should do background scanning
  *
  * \par
  * GUI to WiFi command:
  * \li "GUIINPUTSELECT STATUS=<ON|OFF>"
  * \par
  * No response from WiFi
*/
#define FUJI_CMD_GUI_INPUTSELECT    "GUIINPUTSELECT"

/** \def FUJI_CMD_PLUG_STATUS
  * netHDcontrol to Interactive GUI command to inform the status of HDMI plug being inserted or unplugged
  *
  * \par
  * netHDcontrol commands:
  * \li "PLUG_STATUS STATUS=<PLUGGED|UNPLUGGED>"
  * \par
  * No response from GUI
*/
#define FUJI_CMD_PLUG_STATUS        "PLUG_STATUS"

/** \def CAVM_IPC_CMD_DGL_MAC
  * WiFi to netHDcontrol command to report MAC address
  *
  * \par
  * WiFi to netHDcontrol command:
  * \li "DGL_MAC MAC=<bssid>"
  * \par
  * No response from netHDcontrol
  * \li  "DGL_MAC"
*/
#define CAVM_IPC_CMD_DGL_MAC 	"DGL_MAC"

/** \def CAVM_IPC_CMD_WIFI_INFO
  * WiFi to netHDcontrol command to indicate WiFi status
  *
  * \par
  * Wifi to netHDcontrol command:
  * \li "WIFI_INFO SSID=<ssid> SIGNAL=<rssi> STATUS=<CFG|UNCFG> MODE=STA/SOFTAP/P2P/WIVUAP/EXTAP/WIDI/DLNA/WFD/WIVU"
  * \par
  * No response from netHDcontrol
*/
#define CAVM_IPC_CMD_WIFI_INFO 	"WIFI_INFO"

/** \def CAVM_IPC_CMD_CFG_APD
  * netHDcontrol to WiFI command to configure WiFi from web interface
  *
  * \par
  * netHDcontrol to Wifi command:
  * \li "CFG_APD SSID=<ssidname> KEY=<keyvalue>"
  * \par
  * No response from WiFi
*/
#define CAVM_IPC_CMD_CFG_APD	"CFG_APD"

/** \def CAVM_IPC_CMD_WIFICH
  * DA to WiFi WiFi Channel command
  *
  * \par
  * DA to WiFi commands:
  * \li "WIFICH VALUE=AUTO/ ch number" - Set channel number
  * \li "WIFICH" - query current WiFi channel number
  * \par
  * Response from WiFi
  * \li  "WIFICH VALUE=<channel> BW=<value>"
*/
#define CAVM_IPC_CMD_WIFICH        "WIFICH"

/** \def CAVM_IPC_CMD_LATENCY
  * GUI to netHDcontrol Latency command
  *
  * \par
  * GUI to netHDcontrol commands:
  * \li "LATENCY VALUE=THEATRE/GAME/usecs"  - set latency to value
  * \li "LATENCY" = query current setting
  * \par
  * No response from netHDcontrol
*/
#define CAVM_IPC_CMD_LATENCY       "LATENCY"

/** \def CAVM_IPC_CMD_WIFISTATUS
  * netHDcontrol to WiFi Status command
  *
  * \par
  * netHDcontrol to WiFi commands:
  * \li  "WIFISTATUS"
  * \par
  * Response from WiFi
  * \li "WIFISTATUS MAC=XX:XX:XX:XX:XX:XX LASTGO=YY:YY:YY:YY:YY:YY RSSI=-30"
*/
#define CAVM_IPC_CMD_WIFISTATUS    "WIFISTATUS"

/** \def CAVM_IPC_CMD_SYSTEM
  * netHDcontrol to/from Interactive GUI system command
  *
  * \par
  * GUI to netHDcontrol commands:
  * \li "SYSTEM VALUE=LOCAL or MAC addresss 7C:4F:B5:60:CD:73"
  * \par
  * Response from netHDcontrol:
  * \li  "SYSTEM  VERSION=svnversion  MAC=7C:4F:B5:60:CD:73 IP=192.168.162.130 RESOLUTION=1080P60"
*/
#define CAVM_IPC_CMD_SYSTEM        "SYSTEM"

/** \def CAVM_IPC_CMD_WIFIQUALITY
  * Interactive GUI to WiFi Quality command
  *
  * \par
  * GUI to WiFi commands:
  * \li "WIFIQUALITY"
  * \par
  * Response from WiFi
  * \li "WIFIQUALITY VALUE=AUTO/OFF WIFILOWDBM=-65 WIFIHIGHDBM=-40 WIFISCALE=10"
*/
#define CAVM_IPC_CMD_WIFIQUALITY   "WIFIQUALITY"

/** \def CAVM_IPC_CMD_WIFI_CONFIG
  * Interactive GUI to WiFi config get/set command
  *
  * \par
  * GUI to WiFi commands:
  * \li "WIFICONFIG MODE=<GET/SET> <TAG1/TAG1=VALUE1>....<TAGn/TAGn=VALUEn>"
  * \par
  * Response from WiFi
  * \li "WIFICONFIG STATUS=<SUCCESS/FAIL> <TAG1/TAG1=VALUE1>....<TAGn/TAGn=VALUEn>"
  * \par
  * eg.
  * \li "WIFICONFIG MODE=GET P2P_DEVICE_NAME P2P_GO_INTENT"
  * \li "WIFICONFIG MODE=GET STATUS=SUCESS P2P_DEVICE_NAME=\"cavm test device\" P2P_GO_INTENT=8"
  * \par
  * \li "WIFICONFIG MODE=SET P2P_DEVICE_NAME=\"new device name\" P2P_GO_INTENT=1"
  * \li "WIFICONFIG MODE=SET STATUS=SUCESS P2P_DEVICE_NAME=\"new device name\" P2P_GO_INTENT=8"
  *
*/
#define CAVM_IPC_CMD_WIFI_CONFIG	"WIFICONFIG"

/** \def CAVM_IPC_CMD_WIFI_STATS
  * WiFi stats get message
  *
  * \par
  * GUI to WiFi commands:
  * \li "WIFISTATS TYPE=<STA/P2P/AP/WIDI_STA>"
  * \par
  * Response from WiFi
  * \li "WIFISTATS TYPE=<STA/P2P/AP/WIDI_STA> STATUS=<SUCCESS/FAIL> <TAG1/TAG1=VALUE1>....<TAGn/TAGn=VALUEn>"
  * \par
  * eg.
  * \li "WIFISTATS TYPE=P2P"
  * \li "WIFISTATS TYPE=P2P STATUS=SUCESS PEER_MAC=XXX SSID=YYY FREQ=2412 RX_BYTES=1000 TX_BYTES=1000
  *                     RX_PACKETS=2000 TX_PACKETS=2000 SIGNAL=-70 TX_RATE=65 TX_MCS_INDEX=7 CHANNEL WIDTH=20
  *
*/
#define CAVM_IPC_CMD_WIFI_STATS    "WIFISTATS"

/**@}*/

#define CAVM_IPC_TAG_NICKNAME      "NICKNAME="
#define CAVM_IPC_TAG_P2P           "P2P="
#define CAVM_IPC_TAG_NAME          "NAME="
#define CAVM_IPC_TAG_EXTAP         "EXTAP="
#define CAVM_IPC_TAG_WIVUAP        "WIVUAP="
#define CAVM_IPC_TAG_SSID          "SSID="
#define CAVM_IPC_TAG_GO            "GO="
#define CAVM_IPC_TAG_BSSID         "BSSID="
#define CAVM_IPC_TAG_MAC           "MAC="
#define CAVM_IPC_TAG_BANDWIDTH     "BW="
#define CAVM_IPC_TAG_DFS           "DFS="
#define CAVM_IPC_TAG_LASTGO        "LASTGO="
#define CAVM_IPC_TAG_RSSI          "RSSI="
#define CAVM_IPC_TAG_WIFILOWDBM    "WIFILOWDBM="
#define CAVM_IPC_TAG_WIFIHIGHDBM   "WIFIHIGHDBM="
#define CAVM_IPC_TAG_WIFISCALE     "WIFISCALE="
#define CAVM_IPC_TAG_WIFIDBMLIST   "WIFIDBMLIST="
#define CAVM_IPC_TAG_PERSIST       "PERSIST="
#define CAVM_IPC_TAG_STATUS        "STATUS="
#define CAVM_IPC_TAG_PIN           "PIN="
#define CAVM_IPC_TAG_SIGNAL 	   "SIGNAL="
#define CAVM_IPC_TAG_SECTYPE 	   "SECTYPE="
#define CAVM_IPC_TAG_KEY           "KEY="
#define CAVM_IPC_TAG_VALUE         "VALUE="
#define CAVM_IPC_TAG_IPADDR        "IP="
#define CAVM_IPC_TAG_IPPORT        "IP_PORT="
#define CAVM_IPC_TAG_WFDIP         "WFD_IP="
#define CAVM_IPC_TAG_WFDPORT       "WFD_PORT="
#define CAVM_IPC_TAG_P2PIP         "P2P_IP="
#define CAVM_IPC_TAG_P2PPORT       "P2P_PORT="
#define CAVM_IPC_TAG_NIF           "NIF="	/* network interface name */
#define CAVM_IPC_TAG_WIFICH        "WIFICH="
#define CAVM_IPC_TAG_VIDEO         "VIDEO="	/* VIDEO=port, video type 0 = 3984 */
#define CAVM_IPC_TAG_AUDIO         "AUDIO="	/* AUDIO=port, audio type 0= NETHD, 1 =LPCM 44.1Khz, 2= LPCM 48kHz */
#define CAVM_IPC_TAG_INDEX		   "INDEX="
#define CAVM_IPC_TAG_TYPE		   "TYPE="
#define CAVM_IPC_TAG_DEVTYPE       "DEVTYPE=" /* P2P Primary Device Type */
#define CAVM_IPC_TAG_PROTOCOL		"PROTOCOL=" //WIDI, WIVU, WFD, and DLNA
#define CAVM_IPC_TAG_UI_PHASE      "PHASE="
#define CAVM_IPC_TAG_UI_PROGRESS   "PROGRESS="
#define CAVM_IPC_TAG_MODE		   "MODE="
#define CAVM_IPC_TAG_REASON        "REASON="
#define CAVM_IPC_TAG_P2P_DEVNAME   "P2P_DEVICE_NAME="

#define CAVM_IPC_STATUS_START	   "START"
#define CAVM_IPC_STATUS_STOP       "STOP"
#define CAVM_IPC_STATUS_STOPPED    "STOPPED"
#define CAVM_IPC_STATUS_PAUSE      "PAUSE"
#define CAVM_IPC_STATUS_PAUSED     "PAUSED"
#define CAVM_IPC_STATUS_ACTIVE     "ACTIVE"
#define CAVM_IPC_STATUS_INACTIVE   "INACTIVE"
#define CAVM_IPC_STATUS_SUCCEED    "SUCCEED"
#define CAVM_IPC_STATUS_FAILED     "FAILED"
#define CAVM_IPC_STATUS_AWAKE      "AWAKE"
#define CAVM_IPC_STATUS_WAKE       "WAKE"
#define CAVM_IPC_STATUS_DOZE       "DOZE"
#define FUJI_STATUS_PLUGGED        "PLUGGED"
#define FUJI_STATUS_UNPLUGGED      "UNPLUGGED"
#define CAVM_IPC_STATUS_ON         "ON"
#define CAVM_IPC_STATUS_OFF        "OFF"
#define CAVM_IPC_STATUS_STANDBY    "STANDBY"
#define CAVM_IPC_STATUS_AUTO       "AUTO"

#define CAVM_IPC_STATUS_STA_MIRROR_ACTIVE "STA_MIRROR_ACTIVE"
#define CAVM_IPC_STATUS_P2P_MIRROR_ACTIVE "P2P_MIRROR_ACTIVE"
#define CAVM_IPC_STATUS_MIRROR_INACTIVE   "MIRROR_INACTIVE"

#define CAVM_IPC_TYPE_STA          "STA"
#define CAVM_IPC_TYPE_SOFTAP       "SOFTAP"
#define CAVM_IPC_TYPE_P2P		   "P2P"
#define CAVM_IPC_TYPE_WIVUAP       "WIVUAP"
#define CAVM_IPC_TYPE_EXTAP        "EXTAP"
#define CAVM_IPC_TYPE_WIDI		   "WIDI"
#define CAVM_IPC_TYPE_DLNA		   "DLNA"
#define CAVM_IPC_TYPE_WFD		   "WFD"
#define CAVM_IPC_TYPE_WIVU		   "WIVU"
#define WIFI_PBC_TYPE_REQ_P2P_PBC  "REQ_P2P_PBC"
#define WIFI_PBC_TYPE_P2P          "P2P_WAIT"
#define WIFI_PBC_TYPE_WIFI         "WIFI_CONNECT"

#define CAVM_IPC_MODE_GET			"GET"
#define CAVM_IPC_MODE_SET		   	"SET"

#define CAVM_IPC_REASON_UNKNOWN                         "UNKNOWN"
#define CAVM_IPC_REASON_REQUESTED                       "REQUESTED"
#define CAVM_IPC_REASON_NO_IP                           "NO_IP"
#define CAVM_IPC_REASON_AUTH_TIMEOUT                    "AUTH_TIMEOUT"
#define CAVM_IPC_REASON_AUTH_FAILED                     "AUTH_FAILED"
#define CAVM_IPC_REASON_AUTH_OVERLAP                    "AUTH_OVERLAP"
#define CAVM_IPC_REASON_P2P_GROUP_FORMATION_TIMEOUT     "P2P_GROUP_FORMATION_TIMEOUT"
#define CAVM_IPC_REASON_P2P_GROUP_FORMATION_FAILED      "P2P_GROUP_FORMATION_FAILED"
#define CAVM_IPC_REASON_PEER_DEVICE_GONE                "PEER_DEVICE_GONE"

#define CAVM_IPC_FILE_LAST_GO      "LASTGO"

#define CAVM_WIFI_CAVIUM_OUI       "00:0F:B7"
#define WFA_STR_OUI 			   "50:6F:9A"




#define DLNA_STATE_READY             "STATE_READY"
#define DLNA_PLAY                    "PLAY"
#define DLNA_AUDIO                   "PLAY_AUDIO"
#define DLNA_VIDEO                   "PLAY_VIDEO"
#define DLNA_PLAYTIME                "PLAY_TIME"
#define DLNA_TO_NETHD_ERROR          "ERROR"
#define DLNA_TO_NETHD_LOADING        "LOADING"
#define DLNA_TO_NETHD_PLAYTIME       "PLAY_TIME"
#define DLNA_TO_NETHD_PAUSE          "PAUSE"
#define DLNA_TO_NETHD_BUFFERING      "BUFFERING"
#define DLNA_TO_NETHD_RESUME_PLAY    "RESUME_PLAY"
#define DLNA_TO_NETHD_IMAGE_METADATA "IMAGE_METADATA"

#define WIFI_SECTYPE_NONE	"NONE"
#define WIFI_SECTYPE_WEP	"WEP"
#define WIFI_SECTYPE_WPAWPA2PSK	"WPAWPA2PSK"
#define DLNA_VOLUME_LEVEL       "VOLUME_LEVEL"
#define DLNA_TO_NETHD_CLR_VOL_BAR   "CLR_VOL_BAR"
#define DLNA_TO_NETHD_OSD_UPDATE    "OSD_UPDATE"
#define DLNA_TO_NETHD_AUDIO_UPDATE  "AUDIO_UPDATE"
#define DLNA_TO_NETHD_HDMI_AUDIO    "HDMI_AUDIO"
#define DLNA_TO_NETHD_AV_UPDATE     "AV_UPDATE"
#define DLNA_TO_NETHD_CVBS_OUTPUT   "CVBS_OUTPUT"
#define DLNA_GET_TVOUT_MODE         "GET_TVOUT_MODE"

#define NETHD_TO_WIFI_GET_STATION_LIST	"STATION_LIST"	/*Get station list */


#define WIDI_L2EVENT_DISCONNECT             "DISCONNECT"         
#define WIDI_L2EVENT_TRIGGERING             "TRIGGERING"
#define WIDI_L2EVENT_SECURITY_CHECK         "SECURITY_CHECK"
#define WIDI_L2EVENT_SECURITY_SUCCESSFUL    "SECURITY_SUCCESSFUL"
#define WIDI_L2EVENT_SMOOTH_HANDOFF         "SMOOTH_HANDOFF"
#define WIDI_L2EVENT_CONNECTED              "CONNECTED"
#define WIDI_L2EVENT_DEAUTHENTICATED        "DEAUTHENTICATED"
#define WIDI_L2EVENT_SCAN_START             "SCAN_START"




#if 0
#define DEBUG_LINE fprintf(stderr, "%s %d\n", __FUNCTION__, __LINE__)
#define DPRINTF(fmt, ...)                                                       \
do {                                                                            \
    struct timeval dprint_tv;                                                   \
    struct tm dprint_tm;                                                        \
    char dprint_buf[64];                                                        \
    gettimeofday(&dprint_tv, NULL);                                             \
    localtime_r(&dprint_tv.tv_sec, &dprint_tm);                                 \
    strftime(dprint_buf, sizeof(dprint_buf), "%F %H:%M:%S", &dprint_tm);        \
    fprintf(stderr, "[%s.%03d][%s] " fmt, dprint_buf,                           \
            (int)(dprint_tv.tv_usec/1000), __FUNCTION__, ##__VA_ARGS__);        \
} while (0)

#else
#define DEBUG_LINE
#define DPRINTF(fmt, args...)
#endif

#define TIME_DIFF_MS(tv1, tv0) ((tv1.tv_sec - tv0.tv_sec) * 1000 + (tv1.tv_usec - tv0.tv_usec) / 1000)
static inline unsigned long TIME_SINCE_MS(struct timeval time) { struct timeval now; gettimeofday(&now, NULL); return TIME_DIFF_MS(now, time);}


typedef int (*fcallback_t)(unsigned long long arg, char* buf, int len);

typedef int (*event_handler_t)(unsigned long data);

extern char *pipeCommandToBuffer(size_t bufSize, char *buf, char *format, ...);

#define SSID_LEN 33
#define NICKNAME_LEN 33
#define MAC_LEN 18
#define MAX_PIN_LEN 64
#define SEC_TYPE_LEN 30
/* PEER retire after 1 day */
#define PEER_RETIRE_TIME_MS 86400000
/* Treat peer as offline after absence for 30 seconds */
#define PEER_OFFLINE_TIME_MS 30000

typedef struct {
	char    ssid[SSID_LEN + 1];
	char    mac[MAC_LEN + 1]; /* MAC in xx:xx:xx:xx:xx:xx format */
	char    nickname[NICKNAME_LEN + 1];
	char    pin[MAX_PIN_LEN + 1];
	char    sec_type[SEC_TYPE_LEN + 1]; /* sercurity modes configured for extAP */
	char    wifich;
	short	devtype;
	short   wps_win;
	short	rssi;
	struct  timeval tv;
	short	wfd_rtsp_port;
	struct {
		unsigned short   is_p2p:1;
		unsigned short   is_widi:1;
		unsigned short	 is_extap:1;
		unsigned short   is_wivuap:1;
		unsigned short	 is_persistent:1;
		unsigned short	 is_fkwfd:1;
		unsigned short   is_persistent_go:1;
		unsigned short   connected:1;

		unsigned short   online:1;
		unsigned short   wps_pbc:1;
		unsigned short   wps_pin:1;
		unsigned short   reserved:5;
	} state;
} peer_info_t;

typedef struct {
    int fd;
    char* name;
    int flags;
    fcallback_t callback;
    int raw;        /* Indicate raw message without message parititoning */
    unsigned long long arg; /* Argument passed to the callback function */
} cavm_ipc_file_t;


extern cavm_ipc_file_t gfiles[];

static inline int cavm_ipc_process_fd_readable(int *pfd, fcallback_t callback, int raw, unsigned long long arg)
{

    static char buf[MAX_MSG_SIZE];
    int rbytes;
    char *start, *end;
    int ret, error = 0;

    if (!callback)
        return 0;

    if (raw == 2) {
        /* Simply notify the callback function without reading for it */
        error = callback(arg, NULL, 0);
        goto DONE;
    }

    /* Note: Xmit side will always transmit complete messages
       Therefore, read() will be guaranteed to complete
       with at least one complete message or more(multithreaded case).
       Fifo write side always block until the peer starts reading.
       With large enough buffer, it's guaranteed no partial messages
       should ever be received.
       */
    rbytes = read(*pfd, buf, MAX_MSG_SIZE - 1);

    if (rbytes <= 0) {
        /* ENETDOWN may be returned by WL Event socket */
        if ((errno != EAGAIN) && (errno != ENETDOWN) &&
				(errno != EINTR) && (errno != ENOENT)) {
            printf("%s: Read fd %d  error %d (%s)\n", __FUNCTION__,
					*pfd, errno, strerror(errno));
            close(*pfd);
            *pfd = -1;
        }
        return -1;
    }

    buf[rbytes] = 0;

    if (raw == 1) {
        error = callback(arg, buf, rbytes);
        goto DONE;
    }

    start = end = buf;
    /* Search for "\n" as the message seperator */
    while (end < buf + rbytes) {
        end = memchr((void*)start, 10, rbytes - (int)(end - buf));
        /* Convert the C string */
        if (end)
            *end = 0;
        else
            end = buf + rbytes;
        if (callback) {
            ret = callback(arg, start, end - start);
            if (ret)
                error = ret;
        }
        start = ++end;
    }

    if (start < buf + rbytes) {
        buf[rbytes] = 0;
        printf("%s: Partial message received %s\n", __FUNCTION__, (char*)start);
    }

DONE:
    if (!raw)
    {
        DPRINTF("%s: fd %d received %s\n", __FUNCTION__, *pfd, buf);
    }
    return error;
}

static inline void cavm_ipc_event_loop(cavm_ipc_file_t* files, int count,
        event_handler_t handler, unsigned long data)
{
    fd_set rfds;
    struct timeval tv;
    int ret, maxfds, i;

    while (1) {
        FD_ZERO(&rfds);
        maxfds = -1;

        for (i = 0; i < count; i++) {
            if ((files[i].fd == -1) || (files[i].flags & O_WRONLY))
                continue;
            FD_SET(files[i].fd, &rfds);
            maxfds = maxfds > files[i].fd ? maxfds : files[i].fd;
        }

        tv.tv_sec = 1;  // poll once per second
        tv.tv_usec = 0;
        ret = select(maxfds + 1, &rfds, NULL, NULL, &tv);
        if (ret == -1) {
            printf("%s: Select error %d (%s)\n", __FUNCTION__, errno, strerror(errno));

        } else if (ret) {
            /* Data available for read */
            for (i = 0; i < count; i++) {
                if ((files[i].fd == -1) || (files[i].flags & O_WRONLY))
                    continue;
                if (FD_ISSET(files[i].fd, &rfds)) {
                    cavm_ipc_process_fd_readable(&files[i].fd, files[i].callback, files[i].raw, files[i].arg);
                }
            }
        } else {
            /* Timeout */
        }

        /* Invoke additional event handler if any */
        if (handler) {
            ret = handler(data);
            if (ret) {
                DPRINTF("cavm_ipc_event_loop out on handler signal\n");
                return;
            }
        }
    }

}

static inline int get_lastgo(char *mac)
{
	char fname[60];
	FILE *file;
	short imac[6];

	mac[0] = 0;

	snprintf(fname, 60, "%s%s", PERSIST_DIR, CAVM_IPC_FILE_LAST_GO);

	if ((file = fopen(fname, "r")) == NULL)
		return 0;

	if (fread(mac, MAC_LEN, 1, file) > 0) {
		if (sscanf(mac, "%hx:%hx:%hx:%hx:%hx:%hx", &imac[0], &imac[1], &imac[2],
					&imac[3], &imac[4], &imac[5]) != 6) {
			mac[0] = 0;
		}
		mac[MAC_LEN - 1] = 0;
	}

	fclose(file);

	return !!mac[0];
}

static inline void set_lastgo(char* mac)
{
	char cmd_buf[60];
	FILE *file;

	if (!mac || !mac[0]) {
		snprintf(cmd_buf, sizeof(cmd_buf), "rm -rf %s%s", PERSIST_DIR, CAVM_IPC_FILE_LAST_GO);
		system(cmd_buf);
		system("sync");
		return;
	}

	snprintf(cmd_buf, sizeof(cmd_buf), "%s%s", PERSIST_DIR, CAVM_IPC_FILE_LAST_GO);
	if ((file = fopen(cmd_buf, "w")) == NULL)
		return;

	fprintf(file, "%s\n", mac);
	fflush(file);
	fclose(file);
}

static inline int validate_wifi_channel(char* if_name, int channel)
{
	char buf[256], strch[10];
	char *output;
	int len;
	char band[10];
	int ret;

	output = pipeCommandToBuffer(sizeof(band), band, "wl -i %s band |grep -v wl", if_name);
	system("wl band auto");

	buf[0] = ' ';
	output = pipeCommandToBuffer(sizeof(buf) - 2, buf + 1, "wl -i %s channels", if_name);
	len = strlen(output) + 1;

	buf[len] = ' ';
	buf[len + 1] = 0;

	snprintf(strch, sizeof(strch), " %d ", channel);
	ret = !!strstr(buf, strch);

	pipeCommandToBuffer(sizeof(buf), buf, "wl -i %s band %s", if_name, band);

	return ret;
}

typedef enum {
	DO_NOT_MODIFY_IF_NOT_FOUND = 0,
	SET_EMPTY_IF_NOT_FOUND = 1
} on_not_found_t;
/* Extract the value of a tagged value with name tagName, into val (with a maximum of valMaxLength bytes) */
/*   from tags, or write "" into val if nullFailures is set.  Note that tagNames contain the '=' character. */
/* Currently extracts to next space (unless it starts with ' then extracts to next '), '\n' or end. */
/* May want to upgrade later to better handle quoted strings, escaped spaces, etc. */
/* nullFailures allows either writing "" to val or not modifying val based on caller's preference if tagName is not found */
/* Returns 1 on successful extration, else returns 0 */
static inline int get_tag_value(char *tagName, char *val, int valMaxLength, char *tags, on_not_found_t nullFailures)
{
	char *p = strstr(tags, tagName);
	int ii = valMaxLength;
	char space_or_singlequote = ' '; /* usually stop on spaces */

	if (!p) { /* could not find tag */
		if (nullFailures) {
			*val = '\0'; /* put "" in val */
		}
		return 0; /* failure */
	}

	p += strlen(tagName); /* skip the name= to get to the value */
	if (*p == '\'') space_or_singlequote = '\''; /* starts with singlequote.  Ignore spaces and go to next single quote */
	while (--ii > 0) { /* loop at most valMaxLength-1 times */
		if ((*p == space_or_singlequote) || (*p == '\n') || (*p == '\0')) {
			break;
		}
		*val++ = *p++; /* copy the normal character */
	}
	*val = '\0'; /* terminate string */
	return 1; /* success */
}

#if 1
/* IPC tag=val IPC string parse output holder*/
struct ipc_tv
{
    char *tag;
    char *val;
    int	tag_len;
    int val_len;
};

/* Parse IPC Strings with \t delimiter eg: "CMD \tTag1=Val1\t\tTag2=Val2\t.."
 * Params:
 * buf: IPC string buffer to be parsed
 * tv: Tag-Value struct array
 * count: No of items in IPC string buffer
 */

__attribute__ ((unused)) static int parse_ipc_str(char *buf, struct ipc_tv *tv, int  count )
{
    char* sep;
    char* ptr;
    int i=0;
    do{
        //Field start
        ptr = strchr( buf, '\t');
        if(ptr)
        {
            ptr = ptr + 1;
            sep = strchr(ptr,'=');
            if ( sep && (sep+1) != 0 ) {
                tv[i].tag = ptr;
                *sep = '\0';//0;
                buf = sep + 1;
                tv[i].val = sep+1;
            } else {
                tv[i].tag = ptr;
                tv[i].val = NULL;
            }
        }else
            break;

        //Field end
        ptr= strchr(buf, '\t');
        if(ptr) {
            *ptr='\0';
            buf = ptr + 1;
        }else
            break;
        //printf("CAVM_IPC TAG: #%s# VAL: #%s#\n", tv[i].tag, tv[i].val );
        i++;
    }while(i<count);

    if(i != count)
        printf("CAVM_IPC Parse Error\n");

    return i;
}
#endif
#endif /*_CAVM_IPC_H_*/
